#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <mtd/mtd-user.h>
#include <fcntl.h>
#include <iwlib.h>
#include <netinet/ether.h>
#include <stdint.h>
#include "id.h"
#include "cipcfg.h"
#include "wlan.h"
#include "syscall.h"
#include "a210-extension.h"
#include "version.h"

/* Define for ath_wlan_status.h */
#include "ath_wlan_status.h"

extern char err_msg[CIPCFG_ERR_MSG_MAXLEN];
const unsigned int magic = MAGIC_A210;
const unsigned int max_rf = MAX_RF;
const char *device_name = DEVICE_NAME;
const char *config_ram = CONFIG_FILE_RAM;
const char *config_rom = CONFIG_FILE_ROM;
const char *restored_cfg_file = RESTORED_CONFIG_FILE;
const char *backup_cfg_file = BACKUP_CONFIG_FILE;
const char *current_config = CONFIG_FILE_CURRENT;

const char *rf_name[MAX_RF] = { RF_1_NAME };

const int gid_manager_userid[] = {
    GID_USER1, GID_END
};

const int gid_rf[MAX_RF] = { GID_RF1 };
const int gid_wds_link[MAX_RF][MAX_WDS_LINK] =
                { {GID_RF1_WDS1, GID_RF1_WDS2, GID_RF1_WDS3, GID_RF1_WDS4} };
const int gid_vap[MAX_RF][MAX_VAP] = 
                { {GID_RF1_VAP1, GID_RF1_VAP2, GID_RF1_VAP3, GID_RF1_VAP4, GID_RF1_VAP5, GID_RF1_VAP6, GID_RF1_VAP7, GID_RF1_VAP8} };
const int gid_vap_sec[MAX_RF][MAX_VAP] = 
            { {GID_RF1_VAP1_SEC, GID_RF1_VAP2_SEC, GID_RF1_VAP3_SEC, GID_RF1_VAP4_SEC, GID_RF1_VAP5_SEC, GID_RF1_VAP6_SEC, GID_RF1_VAP7_SEC, GID_RF1_VAP8_SEC} };
const int gid_vap_acl[MAX_RF][MAX_VAP] = 
            { {GID_RF1_VAP1_ACL, GID_RF1_VAP2_ACL, GID_RF1_VAP3_ACL, GID_RF1_VAP4_ACL, GID_RF1_VAP5_ACL, GID_RF1_VAP6_ACL, GID_RF1_VAP7_ACL, GID_RF1_VAP8_ACL} };
const char *rf1_if = INTERFACE_HW_RF1;

const char *if_vap_2d[MAX_RF][MAX_VAP] = {{ INTERFACE_VAP1, INTERFACE_VAP2, INTERFACE_VAP3, INTERFACE_VAP4, INTERFACE_VAP5, INTERFACE_VAP6, INTERFACE_VAP7, INTERFACE_VAP8}};

const char *if_vap[MAX_VAP] = { INTERFACE_VAP1, INTERFACE_VAP2, INTERFACE_VAP3, INTERFACE_VAP4, INTERFACE_VAP5, INTERFACE_VAP6, INTERFACE_VAP7, INTERFACE_VAP8};

const char *if_wds[MAX_RF][MAX_WDS_LINK] = { {INTERFACE_WDS1, INTERFACE_WDS2, INTERFACE_WDS3, INTERFACE_WDS4} };


const SYS_ITEM_LIST sys_item_list[] =
{
     {"magic_str",       NULL,       		 print_str, 	get_magic_str },
     {"sw_ver",          SOFTWARE_VERSION,       print_arg, 	get_arg },
     {"uptime",          PATH_UPTIME_FILE,       print_uptime, 	get_uptime },
     {"date",            PATH_DATE,              print_date,	get_date },
     {"default_gw",      PATH_NET_ROUTE,         print_ip,	get_gateway },
     {"wan_mask",        "",                     print_ip,	get_wan_mask },
     {"wan_ip",          "",                     print_ip,	get_wan_ip },
     {"wan_mac",         "wan_if",               print_mac,	get_mac },
     {"rf1_mac",         INTERFACE_VAP1 ":" INTERFACE_VAP2 ":" INTERFACE_VAP3 ":" INTERFACE_VAP4 ":" INTERFACE_VAP5 ":" INTERFACE_VAP6 ":" INTERFACE_VAP7 ":" INTERFACE_VAP8 ":" INTERFACE_WDS1 ":" INTERFACE_WDS2 ":" INTERFACE_WDS3 ":" INTERFACE_WDS4,         print_mac,	get_mac },
     {"vap_1_mac",       INTERFACE_VAP1,         print_mac,	get_mac },
     {"vap_2_mac",       INTERFACE_VAP2,         print_mac,	get_mac },
     {"vap_3_mac",       INTERFACE_VAP3,         print_mac,	get_mac },
     {"vap_4_mac",       INTERFACE_VAP4,         print_mac,	get_mac },
     {"vap_5_mac",       INTERFACE_VAP5,         print_mac,	get_mac },
     {"vap_6_mac",       INTERFACE_VAP6,         print_mac,	get_mac },
     {"vap_7_mac",       INTERFACE_VAP7,         print_mac,	get_mac },
     {"vap_8_mac",       INTERFACE_VAP8,         print_mac,	get_mac },
     /* use rf<n>_vap<m>_mac instead of vap_<m>_mac */
     {"rf1_vap1_mac",    INTERFACE_VAP1,         print_mac,	get_mac },
     {"rf1_vap2_mac",    INTERFACE_VAP2,         print_mac,	get_mac },
     {"rf1_vap3_mac",    INTERFACE_VAP3,         print_mac,	get_mac },
     {"rf1_vap4_mac",    INTERFACE_VAP4,         print_mac,	get_mac },
     {"rf1_vap5_mac",    INTERFACE_VAP5,         print_mac,	get_mac },
     {"rf1_vap6_mac",    INTERFACE_VAP6,         print_mac,	get_mac },
     {"rf1_vap7_mac",    INTERFACE_VAP7,         print_mac,	get_mac },
     {"rf1_vap8_mac",    INTERFACE_VAP8,         print_mac,	get_mac },
     {"rf1_ch",          INTERFACE_VAP1 ":" INTERFACE_VAP2 ":" INTERFACE_VAP3 ":" INTERFACE_VAP4 ":" INTERFACE_VAP5 ":" INTERFACE_VAP6 ":" INTERFACE_VAP7 ":" INTERFACE_VAP8 ":" INTERFACE_WDS1 ":" INTERFACE_WDS2 ":" INTERFACE_WDS3 ":" INTERFACE_WDS4,       print_int,	get_wlan_ch },
     {"rf1_band",        INTERFACE_VAP1 ":" INTERFACE_VAP2 ":" INTERFACE_VAP3 ":" INTERFACE_VAP4 ":" INTERFACE_VAP5 ":" INTERFACE_VAP6 ":" INTERFACE_VAP7 ":" INTERFACE_VAP8 ":" INTERFACE_WDS1 ":" INTERFACE_WDS2 ":" INTERFACE_WDS3 ":" INTERFACE_WDS4,       print_band,	get_wlan_band },
     {"rf1_txpwr",       INTERFACE_VAP1 ":" INTERFACE_VAP2 ":" INTERFACE_VAP3 ":" INTERFACE_VAP4 ":" INTERFACE_VAP5 ":" INTERFACE_VAP6 ":" INTERFACE_VAP7 ":" INTERFACE_VAP8 ":" INTERFACE_WDS1 ":" INTERFACE_WDS2 ":" INTERFACE_WDS3 ":" INTERFACE_WDS4,         a210_print_txpwr,	get_wlan_txpwr },
     {"rf1_txpwr_dbm",   INTERFACE_VAP1 ":" INTERFACE_VAP2 ":" INTERFACE_VAP3 ":" INTERFACE_VAP4 ":" INTERFACE_VAP5 ":" INTERFACE_VAP6 ":" INTERFACE_VAP7 ":" INTERFACE_VAP8 ":" INTERFACE_WDS1 ":" INTERFACE_WDS2 ":" INTERFACE_WDS3 ":" INTERFACE_WDS4,         a210_print_txpwr_dbm,	get_wlan_txpwr },
     {"vap_1_sta",       INTERFACE_VAP1,         print_int,	get_sta },
     {"vap_2_sta",       INTERFACE_VAP2,         print_int,	get_sta },
     {"vap_3_sta",       INTERFACE_VAP3,         print_int,	get_sta },
     {"vap_4_sta",       INTERFACE_VAP4,         print_int,	get_sta },
     {"vap_5_sta",       INTERFACE_VAP5,         print_int,	get_sta },
     {"vap_6_sta",       INTERFACE_VAP6,         print_int,	get_sta },
     {"vap_7_sta",       INTERFACE_VAP7,         print_int,	get_sta },
     {"vap_8_sta",       INTERFACE_VAP8,         print_int,	get_sta },
     /* use rf<n>_vap<m>_sta instead of vap_<m>_sta */
     {"rf1_vap1_sta",    INTERFACE_VAP1,         print_int,	get_sta },
     {"rf1_vap2_sta",    INTERFACE_VAP2,         print_int,	get_sta },
     {"rf1_vap3_sta",    INTERFACE_VAP3,         print_int,	get_sta },
     {"rf1_vap4_sta",    INTERFACE_VAP4,         print_int,	get_sta },
     {"rf1_vap5_sta",    INTERFACE_VAP5,         print_int,	get_sta },
     {"rf1_vap6_sta",    INTERFACE_VAP6,         print_int,	get_sta },
     {"rf1_vap7_sta",    INTERFACE_VAP7,         print_int,	get_sta },
     {"rf1_vap8_sta",    INTERFACE_VAP8,         print_int,	get_sta },
     {"rf1_wds1_peer",   INTERFACE_WDS1,         print_mac,	get_wds_peer},
     {"rf1_wds2_peer",   INTERFACE_WDS2,         print_mac,	get_wds_peer},
     {"rf1_wds3_peer",   INTERFACE_WDS3,         print_mac,	get_wds_peer},
     {"rf1_wds4_peer",   INTERFACE_WDS4,         print_mac,	get_wds_peer},
     {"rf1_wds1_txrate", INTERFACE_WDS1,         print_double,	get_wds_txrate},
     {"rf1_wds2_txrate", INTERFACE_WDS2,         print_double,	get_wds_txrate},
     {"rf1_wds3_txrate", INTERFACE_WDS3,         print_double,	get_wds_txrate},
     {"rf1_wds4_txrate", INTERFACE_WDS4,         print_double,	get_wds_txrate},
     {"rf1_wds1_idle",   INTERFACE_WDS1,	 print_int,	get_wds_idle},
     {"rf1_wds2_idle",   INTERFACE_WDS2,	 print_int,	get_wds_idle},
     {"rf1_wds3_idle",   INTERFACE_WDS3,	 print_int,	get_wds_idle},
     {"rf1_wds4_idle",   INTERFACE_WDS4,	 print_int,	get_wds_idle},
     {"rf1_wds1_snr",    INTERFACE_WDS1,	 print_int,	get_wds_snr},
     {"rf1_wds2_snr",    INTERFACE_WDS2,	 print_int,	get_wds_snr},
     {"rf1_wds3_snr",    INTERFACE_WDS3,	 print_int,	get_wds_snr},
     {"rf1_wds4_snr",    INTERFACE_WDS4,	 print_int,	get_wds_snr},
     {"phy_cnt",	 NULL,			 print_int,	get_phy_cnt},
     {"ether_phy_cnt",   NULL,			 print_int, 	get_ether_cnt},
     {"rf_phy_cnt",	 NULL,			 print_int, 	get_rf_cnt},
     {"build_time",      PATH_BUILD_TIME,        print_str,	get_file_content },
     {"build",           PATH_BUILD,             print_str, 	get_file_content },
     {"build_tag",       PATH_BUILD_TAG,         print_str, 	get_file_content },
     {"kernel",          PATH_KERNEL,            print_str, 	get_file_content },
     {"kernel_tag",      PATH_KERNEL_TAG,        print_str, 	get_file_content },
     {"dev_ver",         PATH_BUILD_TAG,         print_str,	get_dev_ver },
     {"sync_cfg",        PATH_SYNC_CFG,          print_int,	chk_cfg_sync },
     {"site",            PRODUCT_INFO_SITE,      print_str,	get_product_info },
     {"company",  	 PRODUCT_INFO_COMPANY,	 print_str,	get_product_info },
     {"model",		 PRODUCT_INFO_MODEL,	 print_str,	get_product_info },
     {"operating_mode",  "",                     print_str,	getCurrentOptMode },
     {"rf_sta_m_ssid",   INTERFACE_STA,          print_str,     getRfStaModeSsid },
     {"rf_sta_m_mac",    INTERFACE_STA,          print_mac,     getRfStaModeMac },
     {"rf_sta_m_channel",INTERFACE_STA,          print_int,     getRfStaModeChannel },
     {"rf_sta_m_rssi",   INTERFACE_STA,          print_int,     getRfStaModeRssi },
     {"rf_sta_m_tx_rate",INTERFACE_STA,          print_int,     getRfStaModeTxRate },
     {"rf_sta_m_tx_count",INTERFACE_STA,         print_int,     getRfStaModeTxCount },
     {"rf_sta_m_tx_err", INTERFACE_STA,          print_int,     getRfStaModeTxErr },
     {"repeater_status",  "",                    print_str,     getRepeaterIfStatus },
     {"repeater_mode",    "",                    print_str,     getRepeaterMode },
     {"repeater_security","",                    print_str,     getRepeaterSecurity },
     {NULL,              NULL,                   NULL,	NULL } // End of table
};

/* Please sync with buildroot/utils/gen_ap_fw/gen_ap_fw.c */
const BRAND_ID_LIST brand_id_list[] = 
{
    { {' ', 'A'},	"",			"A210" },
    { {'4', 'E'},	"4ipnet",		"EAP200" },
    { {'4', 'T'},	"4iptw",		"EAP206" },
    { {'\0', '\0'},	NULL,			NULL} // End of table
};

group_row_t group_table[] = {
  { GID_DEVICE,         "device",          FALSE, "cipconfig.system.device"},
  { GID_SYSTIME,        "systime",         FALSE, "cipconfig.system.systime"},
  { GID_OPMODE,		"opmode",	   FALSE, "cipconfig.system.opmode"},
  { GID_WAN_IP,         "wan_ip",          FALSE, "cipconfig.network.wan_ip"},
  { GID_NET_L2,   	"network_L2",      FALSE, "cipconfig.network.l2"},
  { GID_BFW,            "bfirewall",       FALSE, "cipconfig.network.bfirewall"},
  { GID_BFW_SERVICE,    "bfw_service",     FALSE, "cipconfig.network.bfw_service"},
  { GID_BFW_DAI,        "bfw_dai",         FALSE, "cipconfig.network.bfw_dai"},
  { GID_ACCOUNT,  	"account",         FALSE, "cipconfig.management.accounts"},
  { GID_USER1,  	"admin",           FALSE, "cipconfig.management.accounts.user:0"},
  { GID_SYSLOG,   	"syslog",          FALSE, "cipconfig.management.syslog"},
  { GID_REPORT_AGENT,   "report_agent",    FALSE, "cipconfig.management.report_agent"},
  { GID_REMOTE_PROTOCOL,"remote_protocol", FALSE, "cipconfig.management.rmt_srv"},
  { GID_SNMP,	        "snmp",            FALSE, "cipconfig.management.rmt_srv.snmp"},
  { GID_REMOTE_ACCESS,	"remote_access",   FALSE, "cipconfig.management.rmt_acc"},
  { GID_WES,		"wes",   	   FALSE, "cipconfig.management.wes"},
  { GID_CAPWAP,		"capwap",   	   FALSE, "cipconfig.management.capwap"},
  { GID_RF1,    	"rf1",             FALSE, "cipconfig.wlan.rfcard:0"},
  { GID_RF1_VAP1,       "rf1_vap1",        FALSE, "cipconfig.wlan.rfcard:0.vap:0"},
  { GID_RF1_VAP1_ACL,   "rf1_vap1_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:0.macacl"},
  { GID_RF1_VAP1_SEC,   "rf1_vap1_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:0.security"},
  { GID_RF1_VAP2,       "rf1_vap2",        FALSE, "cipconfig.wlan.rfcard:0.vap:1"},
  { GID_RF1_VAP2_ACL,   "rf1_vap2_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:1.macacl"},
  { GID_RF1_VAP2_SEC,   "rf1_vap2_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:1.security"},
  { GID_RF1_VAP3,       "rf1_vap3",        FALSE, "cipconfig.wlan.rfcard:0.vap:2"},
  { GID_RF1_VAP3_ACL,   "rf1_vap3_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:2.macacl"},
  { GID_RF1_VAP3_SEC,   "rf1_vap3_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:2.security"},
  { GID_RF1_VAP4,       "rf1_vap4",        FALSE, "cipconfig.wlan.rfcard:0.vap:3"},
  { GID_RF1_VAP4_ACL,   "rf1_vap4_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:3.macacl"},
  { GID_RF1_VAP4_SEC,   "rf1_vap4_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:3.security"},
  { GID_RF1_VAP5,       "rf1_vap5",        FALSE, "cipconfig.wlan.rfcard:0.vap:4"},
  { GID_RF1_VAP5_ACL,   "rf1_vap5_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:4.macacl"},
  { GID_RF1_VAP5_SEC,   "rf1_vap5_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:4.security"},
  { GID_RF1_VAP6,       "rf1_vap6",        FALSE, "cipconfig.wlan.rfcard:0.vap:5"},
  { GID_RF1_VAP6_ACL,   "rf1_vap6_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:5.macacl"},
  { GID_RF1_VAP6_SEC,   "rf1_vap6_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:5.security"},
  { GID_RF1_VAP7,       "rf1_vap7",        FALSE, "cipconfig.wlan.rfcard:0.vap:6"},
  { GID_RF1_VAP7_ACL,   "rf1_vap7_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:6.macacl"},
  { GID_RF1_VAP7_SEC,   "rf1_vap7_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:6.security"},
  { GID_RF1_VAP8,       "rf1_vap8",        FALSE, "cipconfig.wlan.rfcard:0.vap:7"},
  { GID_RF1_VAP8_ACL,   "rf1_vap8_acl",    FALSE, "cipconfig.wlan.rfcard:0.vap:7.macacl"},
  { GID_RF1_VAP8_SEC,   "rf1_vap8_sec",    FALSE, "cipconfig.wlan.rfcard:0.vap:7.security"},
  { GID_RF1_WDS1,       "rf1_wds1",        FALSE, "cipconfig.wlan.rfcard:0.wds:0"},
  { GID_RF1_WDS2,       "rf1_wds2",        FALSE, "cipconfig.wlan.rfcard:0.wds:1"},
  { GID_RF1_WDS3,       "rf1_wds3",        FALSE, "cipconfig.wlan.rfcard:0.wds:2"},
  { GID_RF1_WDS4,       "rf1_wds4",        FALSE, "cipconfig.wlan.rfcard:0.wds:3"},
  { GID_RF1_SCAN,      	"rf1_scan",        FALSE, "cipconfig.wlan.rfcard:0.scan"},
  { GID_RF1_REP1,       "rf1_rep1",        FALSE, "cipconfig.wlan.rfcard:0.rep:0"},
  { GID_RF1_REP1_SEC,   "rf1_rep1_sec",    FALSE, "cipconfig.wlan.rfcard:0.rep:0.security"},
  { GID_END,        	"",                FALSE, ""}	// indicate end of table
};

const   permission_row_t    oper_perm_table[] =
{
    {OID_DHCP_RENEW,    "DHCP_RENEW",       ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_SCAN,          "SCAN",             ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_RESET_CONFIG,  "RESET_CONFIG",     ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_BACKUP_CONFIG, "BACKUP_CONFIG",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_READ|PERM_WRITE|PERM_EXEC)) )},
    {OID_RESTORE_CONFIG,"RESTORE_CONFIG",   ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_SYS_UPGRADE,   "SYS_UPGRADE",      ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_REBOOT,        "REBOOT",           ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_READ|PERM_WRITE|PERM_EXEC)) )},
    {OID_CHANGE_ALL_PWD,"CHANGE_ALL_PWD",   ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {OID_WPS_OPERATION, "WPS",              ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE|PERM_EXEC))|
                                              PERM(USER_GROUP_2,(PERM_NONE)) )},
    {GID_END,            "",                0}
};

const   user_group_row_t    user_table[] =
{
    {USER_GROUP_1,     "admin"},
    {USER_GROUP_END,   ""}
};

const   permission_row_t    conf_perm_table[] =
{
    {GID_DEVICE,         "device",          ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_SYSTIME,        "systime",         ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_OPMODE,         "opmode",          ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_NETWORK,        "network",         ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_WAN_IP,         "wan_ip",          ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},	   
    {GID_NET_L2,         "network_L2",      ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_ACCOUNT,        "account",         ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_USER1,          "admin",           ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_SYSLOG,         "syslog",          ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_REPORT_AGENT,   "report_agent",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_REMOTE_PROTOCOL,"remote_protocol", ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_SNMP,	         "snmp",            ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_CAPWAP,         "capwap",          ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_REMOTE_ACCESS,	 "remote_access",   ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1,    	 "rf1",             ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP1,       "rf1_vap1",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP1_ACL,   "rf1_vap1_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP1_SEC,   "rf1_vap1_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP2,       "rf1_vap2",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP2_ACL,   "rf1_vap2_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP2_SEC,   "rf1_vap2_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP3,       "rf1_vap3",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP3_ACL,   "rf1_vap3_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP3_SEC,   "rf1_vap3_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP4,       "rf1_vap4",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP4_ACL,   "rf1_vap4_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP4_SEC,   "rf1_vap4_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP5,       "rf1_vap5",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP5_ACL,   "rf1_vap5_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP5_SEC,   "rf1_vap5_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP6,       "rf1_vap6",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP6_ACL,   "rf1_vap6_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP6_SEC,   "rf1_vap6_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP7,       "rf1_vap7",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP7_ACL,   "rf1_vap7_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP7_SEC,   "rf1_vap7_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP8,       "rf1_vap8",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP8_ACL,   "rf1_vap8_acl",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_VAP8_SEC,   "rf1_vap8_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_WDS1,       "rf1_wds1",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_WDS2,       "rf1_wds2",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_WDS3,       "rf1_wds3",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_WDS4,       "rf1_wds4",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_REP1,       "rf1_rep1",        ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_RF1_REP1_SEC,   "rf1_rep1_sec",    ( PERM(USER_GROUP_1,(PERM_READ|PERM_WRITE)) )},
    {GID_END,            "",                0}
};

int dr_check_mask(ITEM_LIST *list, char *group_name);
int dr_check_snmp_read_write(ITEM_LIST *list, char *group_name);
int dr_check_vap_sec(ITEM_LIST *list, char *group_name);
int dr_check_wds(ITEM_LIST *list, char *group_name);
int dr_check_vap_acl(ITEM_LIST *list, char *group_name);
int dr_check_rf(ITEM_LIST *list, char *group_name);

const struct dep_fun_map_list dr_function_list[]= 
{ 
    { "checksubnet", dr_check_mask }, 
    { "checknotthesame", dr_check_snmp_read_write },
    { "checkvapsec", dr_check_vap_sec },
    { "checkwds", dr_check_wds },
    { "checkvapacl", dr_check_vap_acl },
    { "checkrf", dr_check_rf },
    { NULL, NULL}  
} ;
const struct dep_arr dependency_chk_list[] = {
/* WAN IP */
{{{"wan_ip",NULL,NULL,"@checksubnet"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},

/* RF mode and channel */
{{{"rf1",NULL,NULL,"@checkrf"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},

/* VAP Security */
{{{"rf1_vap1_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap2_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap3_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap4_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap5_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap6_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap7_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap8_sec",NULL,NULL,"@checkvapsec"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},

/* WDS */
{{{"rf1_wds1",NULL,NULL,"@checkwds"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_wds2",NULL,NULL,"@checkwds"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_wds3",NULL,NULL,"@checkwds"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_wds4",NULL,NULL,"@checkwds"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},

/* Syslog and SNMP */
{{{"syslog","state","enable","&"},{NULL,NULL,NULL,NULL}},{"syslog","server_ip",""}},
{{{"syslog","state","enable","&"},{NULL,NULL,NULL,NULL}},{"syslog","server_port","1~65535"}},
{{{"snmp","state","enable","&"},{NULL,NULL,NULL,NULL}},{"snmp","read","^[^][\"'<> \\]{1,32}$"}},

/* read != write */
{{{"snmp",NULL,NULL,"@checknotthesame"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"snmp","trap","enable","&"},{NULL,NULL,NULL,NULL}},{"snmp","trap_server",""}},

/* TODO - Different groups */
{{{"rf1_vap1_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap2_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap3_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap4_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap5_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap6_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap7_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},
{{{"rf1_vap8_acl",NULL,NULL,"@checkvapacl"},{NULL,NULL,NULL,NULL}},{NULL,NULL,NULL}},

{{{NULL, NULL, NULL, NULL}}, {NULL, NULL, NULL}}
};

// Check format and convert to binary (6-byte MAC address)
int mac2bin(char *mac_from, char *mac_to)
{
    int i, j, n ;
    char buf[3] ;

    /* Case 1: 12 succesive hex-decimal digits */
    if( strlen(mac_from) == 12 ) {
        for(i=j=0 ; i<12 ; i+=2) {
            if( !isxdigit( mac_from[i] ) || !isxdigit( mac_from[i+1]) )
                return -1 ;

            buf[0] = mac_from[i] ;
            buf[1] = mac_from[i+1] ;
            buf[2] = 0 ;
            sscanf(buf, "%x", &n) ;

            mac_to[j++] = n ;
        }
        return 0 ;
    }

    /* Case 2: 12 hex-decimal digits with 5 symbols  */
    if( strlen(mac_from) == 17) {
        for(i=j=0; i<17; i+=3) {
            char c = mac_from[i+2] ;
            char valid_delim[] = ":.,-" ; // valid delimitation characters

            if(!isxdigit( mac_from[i] ) || !isxdigit( mac_from[i+1]))
                return -1 ;
            if(strchr(valid_delim, c) == NULL && c != '\0')
                return -1 ;

            buf[0] = mac_from[i] ;
            buf[1] = mac_from[i+1] ;
            buf[2] = 0 ;
            sscanf(buf, "%x", &n) ;

            mac_to[j++] = n ;
        }

        return 0 ;
    }

    /* Otherwise: invalid format */
    return -1 ;
}

/* Convert IP to uint32
 * @param		ip		ip address
 * @return 		ret	 	uint32	
 */
uint32_t ipv4_to_int(char *ip)
{
    int i=0;
    int idx=0;
    char *ptr;
    uint32_t retval = 0;

    retval |= (atoi(ip) << (OCTET * (NUM_IP_OCTET-idx-1)));
    idx++;

    for(i=0;i<strlen(ip);i++)
    {
        if(ip[i] == '.')
        {
            ptr = ip+i+1;
            retval |= (atoi(ptr) << (OCTET * (NUM_IP_OCTET-idx-1)));
            idx++;
        }
    }
    return retval;
}

int dr_get_item_value(CIPCFG *cipcfg, ITEM_LIST *list, char *item_name, int gid, char *item_value) {

    ITEM_LIST *cur_item = NULL;

    while( list != NULL ) {

        if( strcmp(list->item_name, item_name) == 0 && list->gid == gid) {
            cur_item = list;
            break;
        }
        list = list->next;
    }

    if(!cur_item) {
        get_cfg_item(cipcfg, gid, item_name, item_value);
    } else {
        strcpy(item_value, cur_item->item_value);
    }
    return 0;
}

/* TODO: Add 11n */
int dr_check_rf(ITEM_LIST *list, char *group_name) {

    int ret = CIPCFG_SUCCESS;
    char mode[MAX_VALUE_LEN], channel[MAX_VALUE_LEN], rate[MAX_VALUE_LEN];
    char item_name[32];

    CIPCFG *cipcfg = init_cipcfg();
    group_row_t *cur_group_row;
    cur_group_row = group_table_row_str(group_name);

    dr_get_item_value(cipcfg, list, "mode", cur_group_row->gid, mode);
    dr_get_item_value(cipcfg, list, "channel", cur_group_row->gid, channel);
    dr_get_item_value(cipcfg, list, "rate", cur_group_row->gid, rate);

    if ( !strcmp(mode, "11a") ) {

        if ( strcmp(channel, "auto") ) {
            switch(atoi(channel)) {
                case 34: case 36: case 40:case 42:case 44: case 48:case 50: case 52: case 56:
                case 58: case 60: case 64:case 100:case 104:case 108:case 112:case 116:case 120:
                case 124:case 128:case 132:case 140:case 149:case 152:case 153:case 157:case 160:
                case 161:case 165:
                    ret = CIPCFG_SUCCESS;
                    break;
                default:
                    ret = -1;
                    break;
            }
        }
        if(ret == CIPCFG_SUCCESS)
            ret = cipcfg_item_regex_check("^auto$|^6M$|^9M$|^12M$|^18M$|"
					"^24M$|^36M$|^48M$|^54M$", rate);

    } else if ( !strcmp(mode, "11b") ) {
        if ( strcmp(channel, "auto") ) {
            switch(atoi(channel)) {
                case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
                case 10:case 11:case 12:case 13:case 14:
                    ret = CIPCFG_SUCCESS;
                    break;
                default:
                    ret = -1;
                    break;
            }
        }

        if(ret == CIPCFG_SUCCESS)
            ret = cipcfg_item_regex_check("^auto$|^6M$|^9M$|^12M$|^18M$|"
					"^24M$|^36M$|^48M$|^54M$", rate);
    } else if ( !strcmp(mode, "11g") ) {
        if ( strcmp(channel, "auto") ) {
            switch(atoi(channel)) {
                case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
                case 10:case 11:case 12:case 13:case 14:
                    ret = CIPCFG_SUCCESS;
                    break;
                default:
                    ret = -1;
                    break;
            }
        }
        if(ret == CIPCFG_SUCCESS)
            ret = cipcfg_item_regex_check("^auto$|^6M$|^9M$|^12M$|^18M$|"
					"^24M$|^36M$|^48M$|^54M$", rate);
    } else if ( !strcmp(mode, "11bg") || !strcmp(mode, "auto") ) {
        if ( strcmp(channel, "auto") ) {
            switch(atoi(channel)) {
                case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9:
                case 10:case 11:case 12:case 13:case 14:
                    ret = CIPCFG_SUCCESS;
                    break;
                default:
                    ret = -1;
                    break;
            }
        }

        if(ret == CIPCFG_SUCCESS)
            ret = cipcfg_item_regex_check("^auto$|^1M$|^2M$|^5.5M$|^11M$|^6M$|"
				"^9M$|^12M$|^18M$|^24M$|^36M$|^48M$|^54M$", rate);
    } else {
        ret = -1;
    }

    close_cipcfg(cipcfg);
    return ret;
}

// TODO ACL dependency
int dr_check_vap_acl(ITEM_LIST *list, char *group_name) {
     int ret = CIPCFG_SUCCESS;
     return ret;
}

int dr_check_wds(ITEM_LIST *list, char *group_name) {

    int ret = CIPCFG_SUCCESS;

    char wep_enable[MAX_VALUE_LEN], method[MAX_VALUE_LEN], keylen[MAX_VALUE_LEN], key[MAX_VALUE_LEN];
    char keyidx[MAX_VALUE_LEN], wds_enable[MAX_VALUE_LEN], mac[MAX_VALUE_LEN];
    char item_name[32];
    CIPCFG *cipcfg = init_cipcfg();
    group_row_t *cur_group_row;
    cur_group_row = group_table_row_str(group_name);

    dr_get_item_value(cipcfg, list, "wep.enabled", cur_group_row->gid, wep_enable);
    dr_get_item_value(cipcfg, list, "wep.method", cur_group_row->gid, method);
    dr_get_item_value(cipcfg, list, "wep.txidx", cur_group_row->gid, keyidx);
    memset(item_name, '\0', 32);
    sprintf(item_name, "wep.key%d", (atoi(keyidx)-1));

    dr_get_item_value(cipcfg, list, item_name, cur_group_row->gid, key);
    dr_get_item_value(cipcfg, list, "wep.keylen", cur_group_row->gid, keylen);

    if( !strcmp(wep_enable, "enable") ) {
        if(!strcmp(method, "ascii")) { // Ascii

            switch(atoi(keylen)) {
                case 64:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{5}$", key);
                    break;
                case 128:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{13}$", key);
                    break;
                case 152:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{16}$", key);
                    break;
                default:
                    break;
            }
        } else { // Hex

            switch(atoi(keylen)) {
                case 64:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{10}$", key);
                    break;
                case 128:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{26}$", key);
                    break;
                case 152:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{32}$", key);
                    break;
                default:
                    break;
            }
        }
    }

    /* Make sure MAC is not empty when WDS is enabled */
    dr_get_item_value(cipcfg, list, "state", cur_group_row->gid, wds_enable);
    dr_get_item_value(cipcfg, list, "mac", cur_group_row->gid, mac);

    if(!strcmp(wds_enable, "enable") && strlen(mac) == 0)
        ret = -1;

    close_cipcfg(cipcfg);
    return ret;
}

int dr_check_vap_sec(ITEM_LIST *list, char *group_name) {

    int ret = CIPCFG_SUCCESS;
    ITEM_LIST *cur_item;
    char item_name[32];
    char mode[MAX_VALUE_LEN];
    char key[MAX_VALUE_LEN], keyidx[MAX_VALUE_LEN], keylen[MAX_VALUE_LEN], method[MAX_VALUE_LEN];
    char type[MAX_VALUE_LEN], passphrase[MAX_VALUE_LEN], psk[MAX_VALUE_LEN];
    char radius_8021x_tmp[MAX_VALUE_LEN];

    group_row_t *cur_group_row;
    cur_group_row = group_table_row_str(group_name);

    CIPCFG *cipcfg = init_cipcfg();

    // Security Mode
    dr_get_item_value(cipcfg, list, "mode", cur_group_row->gid, mode);

    if(!strcmp(mode, "shared") || !strcmp(mode, "open") || !strcmp(mode, "auto")) { // WEP

        dr_get_item_value(cipcfg, list, "wep.method", cur_group_row->gid, method);
        dr_get_item_value(cipcfg, list, "wep.txidx", cur_group_row->gid, keyidx);
        memset(item_name, '\0', 32);
        sprintf(item_name, "wep.key%d", (atoi(keyidx)-1));

        dr_get_item_value(cipcfg, list, item_name, cur_group_row->gid, key);
        dr_get_item_value(cipcfg, list, "wep.keylen", cur_group_row->gid, keylen);
        // WEP Method
        if(!strcmp(method, "ascii")) { // Ascii

            switch(atoi(keylen)) {
                case 64:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{5}$", key);
                    break;
                case 128:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{13}$", key);
                    break;
                case 152:
                    ret = cipcfg_item_regex_check("^[\x21-\x7e]{16}$", key);
                    break;
                default:
                    break;
            }
        } else { // Hex

            switch(atoi(keylen)) {
                case 64:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{10}$", key);
                    break;
                case 128:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{26}$", key);
                    break;
                case 152:
                    ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{32}$", key);
                    break;
                default:
                    break;
            }
        }
    } else if( !strcmp(mode, "wpa") ) { // WPA

        dr_get_item_value(cipcfg, list, "wpa.type", cur_group_row->gid, type);
        if( !strcmp(type, "passphrase") ) { // Passphrase
            dr_get_item_value(cipcfg, list, "wpa.passphrase", cur_group_row->gid, passphrase);
            ret = cipcfg_item_regex_check("^[\x21-\x7e]{8,63}$", passphrase);
        } else { // PSK
            dr_get_item_value(cipcfg, list, "wpa.passphrase", cur_group_row->gid, passphrase);
            ret = cipcfg_item_regex_check("^[0-9A-Fa-f]{64}$", passphrase);
        }

        dr_get_item_value(cipcfg, list, "wpa.eap", cur_group_row->gid, psk);
        if( strcmp(psk, "psk") != 0 ) {
            ret = -1;
        }

    } else if( !strcmp(mode, "8021x") ) { // Radius

        dr_get_item_value(cipcfg, list, "radius.server:0.ip", cur_group_row->gid, radius_8021x_tmp);
        if( !strcmp(radius_8021x_tmp, "") ) {
            ret = -1;
        }

        dr_get_item_value(cipcfg, list, "wep.enabled", cur_group_row->gid, radius_8021x_tmp);
        if( !strcmp(radius_8021x_tmp, "enable") ) {
            ret = -1;
        }

        dr_get_item_value(cipcfg, list, "wpa.psk", cur_group_row->gid, radius_8021x_tmp);
        if( strcmp(radius_8021x_tmp, "psk") != 0) {
            ret = -1;
        }
    }
    close_cipcfg(cipcfg);

    return ret;
}

/* Dependency Checking function: SNMP read/write cannot be the same */
int dr_check_snmp_read_write(ITEM_LIST *list, char *group_name)
{
    char *read = malloc(sizeof(char) * MAX_VALUE_LEN);
    char *write = malloc(sizeof(char) * MAX_VALUE_LEN);
    char *state = malloc(sizeof(char) * MAX_VALUE_LEN);
    int ret = CIPCFG_SUCCESS; 

    CIPCFG *cipcfg = init_cipcfg();
    ITEM_LIST *cur_item;
    if(!cipcfg)
    {
        sprintf(err_msg, "ERROR: Initial cipcfg fail.\n");
        ret = -1;
        goto end;      
    }

    cur_item = check_item_exist(list, "state");
    while(cur_item != NULL && cur_item->gid != GID_SNMP)
    {
        cur_item = check_item_exist(cur_item, "state");
    }

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "snmp", "state", state);
    else
        strcpy(read, cur_item->item_value);

    cur_item = check_item_exist(list, "read");

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "snmp", "read", read);
    else
        strcpy(read, cur_item->item_value);

    cur_item = check_item_exist(list, "write");

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "snmp", "write", write);
    else
        strcpy(write, cur_item->item_value);

    /* read and write cannot be the same */
    if(!strcmp(state, "enable") && strlen(read) > 0 && strlen(write) > 0 && !strcmp(read, write))
    {
        ret=CIPCFG_ERR_DEP_SNMP_READ_WRITE;
        sprintf(err_msg, "SNMP read/write cannot be the same\n");
        goto end;
    }

end:
    free(read);
    free(write);
    free(state);
    close_cipcfg(cipcfg);	
    return ret;
}

/* Dependency Checking function: Check netmask */
int dr_check_mask(ITEM_LIST *list, char *group_name)
{
    int ret= CIPCFG_SUCCESS;
    uint32_t int_ip1;
    uint32_t int_ip2;
    uint32_t int_mask;
    char *ip1 = malloc(sizeof(char) * MAX_VALUE_LEN);
    char *ip2 = malloc(sizeof(char) * MAX_VALUE_LEN);
    char *mask = malloc(sizeof(char) * MAX_VALUE_LEN);
    char *type = malloc(sizeof(char) * MAX_VALUE_LEN);
	
    ITEM_LIST *cur_item;
	
    CIPCFG *cipcfg = init_cipcfg();
    if(!cipcfg)
    {
        sprintf(err_msg, "ERROR: Initial cipcfg fail.\n");
        ret = -1;
        goto end;
    }
	
    get_cfg_item_by_str(cipcfg, "wan_ip", "type", type);

    /* Check the LAN is in same subnet only when the type is fixed */	
    if(strcmp(type, "fixed"))
        goto end;

    cur_item = check_item_exist(list, "addr");

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "wan_ip", "addr", ip1);
    else
        strcpy(ip1, cur_item->item_value);
		
    cur_item = check_item_exist(list, "gateway");

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "wan_ip", "gateway", ip2);
    else
        strcpy(ip2, cur_item->item_value);
	
    cur_item = check_item_exist(list, "netmask");

    if(!cur_item)
        get_cfg_item_by_str(cipcfg, "wan_ip", "netmask", mask);
    else
        strcpy(mask, cur_item->item_value);


    int_ip1 = ipv4_to_int(ip1);
    int_ip2 = ipv4_to_int(ip2);
    int_mask = ipv4_to_int(mask);

    if( (int_ip1 & int_mask) != (int_ip2 & int_mask))
    {
        ret = CIPCFG_ERR_DEP_NETMASK;
        sprintf(err_msg, "IP and gateway should be in the same subnet\n");
        goto end;
    }

end:
    free(ip1);
    free(ip2);
    free(mask);
    free(type);
    close_cipcfg(cipcfg);
    return ret;
} 


/** 
 * @brief Get WAN interface name.
 * @param ifname Buffer of interface name.
 * @return Return the ifname pointer.
 */
char *get_wan_ifname(char *ifname)
{
    char buf[MAX_VALUE_LEN];
    int vid = 0;
    CIPCFG *cipcfg = NULL;

    cipcfg = init_cipcfg();

    get_cfg_item_by_str(cipcfg, "wan_ip", "mgmt_vid", buf);
    if (buf[0] == '\0') {
	sprintf(ifname, "%s", INTERFACE_WAN);
    } else {
	vid = atoi(buf);
	if (vid > 0 && vid < 4095)
	    sprintf(ifname, "br%d", vid);
	else 
	    sprintf(ifname, "%s", INTERFACE_WAN);
    }

    close_cipcfg(cipcfg);
    return ifname;
}


/*  TODO: remove it.
 *  Dummy function, the same with get_wan_ifname()
 */
char *get_lan_ifname(char *ifname)
{
    char buf[MAX_VALUE_LEN];
    int vid = 0;
    CIPCFG *cipcfg = NULL;

    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "wan_ip", "mgmt_vid", buf);
    if (buf[0] == '\0') {
	sprintf(ifname, "%s", INTERFACE_WAN);
    } else {
	vid = atoi(buf);
	if (vid > 0 && vid < 4095)
	    sprintf(ifname, "br%d", vid);
	else 
	    sprintf(ifname, "%s", INTERFACE_WAN);
    }

    close_cipcfg(cipcfg);
    return ifname;
}


/**
 * This function will show the uptime, the formate is "DD days, HH:MM:SS"
 */
int get_uptime(char *uptime_file, void *retval)
{
    struct timeval *tv = (struct timeval *)retval;
    FILE* fp;

    /* Read the system uptime and accumulated idle time from /proc/uptime. */
    fp = fopen ( uptime_file , "r");
    if (fp == NULL)
	return CIPCFG_ERR_GETSTAT;
    fscanf (fp, "%lu.%lu\n", &tv->tv_sec, &tv->tv_usec);
    fclose (fp);

    return CIPCFG_SUCCESS;
}


int get_gateway(char *route_file, void *retval)
{
    struct sockaddr_in *gw = (struct sockaddr_in *)retval;
    FILE* fp = NULL;

    if ((fp = fopen( route_file, "r")) != NULL) {
        unsigned int dst, route;
        int ret;
        char devname[64];
        if (fscanf(fp, "%*[^\n]\n") < 0) { //skip 1st line
            fclose(fp);
        } else { 
            while (1) {
                ret = fscanf(fp, "%63s%x%x%*[^\n]\n", devname, &dst, &route);
                if (ret < 3 && feof(fp)) {
		    fclose(fp);
		    return CIPCFG_ERR_GETSTAT;
                }
                if (dst == 0) {
		    fclose(fp);
		    gw->sin_addr.s_addr = route;
		    return CIPCFG_SUCCESS;
                }
            }
        }
    }
    return CIPCFG_ERR_GETSTAT;
}


int get_wan_mask(char *key, void *retval)
{
    struct sockaddr_in *netmask = (struct sockaddr_in *)retval;
    int s=socket(PF_INET, SOCK_STREAM, 0);
    struct ifreq ifr;
    struct sockaddr_in *sin;
    char wan_if[IFNAMSIZ];

    strncpy(ifr.ifr_name, get_wan_ifname(wan_if), IFNAMSIZ-1);
    sin = (struct sockaddr_in*)&ifr.ifr_netmask;
    if (ioctl(s, SIOCGIFNETMASK, &ifr) < 0) {
	close(s);
        return CIPCFG_ERR_GETSTAT;
    } else {
	close(s);
	memcpy(netmask, sin, sizeof(struct sockaddr_in));
	return CIPCFG_SUCCESS;
    }
}

int get_wan_ip(char *key, void *retval)
{
    struct sockaddr_in *ip = (struct sockaddr_in *)retval;
    int s=socket(PF_INET, SOCK_STREAM, 0);
    struct ifreq ifr;
    struct sockaddr_in *sin;
    char wan_if[IFNAMSIZ];

    strncpy(ifr.ifr_name, get_wan_ifname(wan_if), IFNAMSIZ-1);
    sin = (struct sockaddr_in*)&ifr.ifr_addr;
    if (ioctl(s, SIOCGIFADDR, &ifr) < 0) {
	close(s);
	return CIPCFG_ERR_GETSTAT;
    } else {
	close(s);
	memcpy(ip, sin, sizeof(struct sockaddr_in));
	return CIPCFG_SUCCESS;
    }
}

int get_mac(char *intf_name, void *retval)
{
    unsigned char *mac = (char *)retval;
    int s=socket(PF_INET, SOCK_STREAM, 0);
    struct ifreq ifr;
    struct sockaddr_in;
    char *intf = NULL, ifs[MAX_VALUE_LEN], *sp = NULL, d[] = ":";
    char wan_if[IFNAMSIZ];
    
    if ( !strcmp("wan_if", intf_name) ){
        intf = get_wan_ifname(wan_if); 
    }else{
	/* Find the first enabled VAP */
	memset(ifs, 0, sizeof(ifs));
	strncpy(ifs, intf_name, MAX_VALUE_LEN - 1);
	for (intf = strtok_r(ifs, d, &sp); intf; intf = strtok_r(NULL, d, &sp)) {
	    strncpy(ifr.ifr_name, intf, sizeof(ifr.ifr_name));
	    if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
		printf("No such interface : %s\n", ifr.ifr_name);
		continue;
	    }
	    if (ifr.ifr_flags & IFF_UP)
		break;
	}
	if (!intf) {
	    close(s);
	    printf("No available interface.\n");
	    return CIPCFG_ERR_GETSTAT;
	}
    }

    strcpy(ifr.ifr_name, intf);
    if (ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
	close(s);
	return CIPCFG_ERR_GETSTAT;
    } else {
	close(s);
	memcpy(mac, ifr.ifr_hwaddr.sa_data, 6);
	return CIPCFG_SUCCESS;
    }
}


int get_wlan_ch(char *intf, void *retval)
{
    int *ch = (int *)retval;
    char intfs[MAX_VALUE_LEN], buf[MAX_VALUE_LEN], *p, *d=":", *sp = NULL;
    int s=socket(PF_INET, SOCK_STREAM, 0);
    int iw_skfd;
    struct ifreq ifr;
    struct iw_range range;
    struct iwreq wrq;

    memset(intfs, 0, MAX_VALUE_LEN);
    strncpy(intfs, intf, MAX_VALUE_LEN - 1);

    /* Find the fisrt enabled VAP */
    for (p = strtok_r(intfs, d, &sp); p; p = strtok_r(NULL, d, &sp)) {
	strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
	if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
	    printf("No such interface : %s\n", ifr.ifr_name);
	    continue;
	}
	if (ifr.ifr_flags & IFF_UP)
	    break;
    }
    close (s);

    if (!p) {
	printf("No available interface.\n");
	return CIPCFG_ERR_GETSTAT;
    }

    if ((iw_skfd = iw_sockets_open()) < 0) {
	printf("Can't connect to iwlib.\n");
	return CIPCFG_ERR_GETSTAT;
    }

    if (iw_get_range_info(iw_skfd, p, &range) < 0) {
	printf("Can't get frequence range.\n");
	iw_sockets_close(iw_skfd);
	return CIPCFG_ERR_GETSTAT;
    }

    if (iw_get_ext(iw_skfd, p, SIOCGIWFREQ, &wrq) >= 0) {
	double freq = iw_freq2float(&(wrq.u.freq));
	*ch = iw_freq_to_channel(freq, &range);
	if (*ch < 0) {
	    freq /= 1000000;
	    if ((int)(freq / 1000) == 2 && freq >= 2412) {
		*ch = (freq == 2484 ? 14 : (((freq - 2412) / 5) + 1));
	    } else if ((int)(freq / 1000) == 5 && freq >= 5180) {
		*ch = ((freq - 5180) / 5) + 36;
	    } else {
		close(iw_skfd);
		return CIPCFG_ERR_GETSTAT;
	    }
	}
	iw_sockets_close(iw_skfd);
	return CIPCFG_SUCCESS;
    } else {
	iw_sockets_close(iw_skfd);
	return CIPCFG_ERR_GETSTAT;
    }
}


/*
 *  the input value is interfaces connected with ":"
 *  like "ath0ap0:ath0ap1:ath0ap2:ath0ap3"
 */
int get_wlan_band(char *intf, void *retval)
{
    enum band_type *band = (enum band_type *)retval;
    FILE* fp = NULL;
    char value[MAX_VALUE_LEN], value2[MAX_VALUE_LEN];
    char intfs[MAX_VALUE_LEN], buf[MAX_VALUE_LEN], *p, *d=":";
    memset(intfs, 0, MAX_VALUE_LEN);
    strcpy(intfs, intf);
    p = strtok(intfs, d);

    sprintf( buf, "%s %s |grep -q UP", PATH_IFCONFIG, p);
    while(cip_system(buf)){
        if((p = strtok(NULL, d))){
            sprintf( buf, "%s %s |grep -q UP", PATH_IFCONFIG, p);
        } else {
            return CIPCFG_ERR_GETSTAT; // no interface is UP
        }
    }

    sprintf( buf, "%s %s |grep '802.11' > %s", PATH_IWCONFIG, p, PATH_IW_DATA );
    cip_system(buf);
    if ((fp = fopen( PATH_IW_DATA, "r")) != NULL) {
        while (!feof(fp)) {
            fgets(value, MAX_VALUE_LEN, fp);
        }
        fclose(fp);
        if (strstr(value, "11g")) { // 802.11g
            sprintf( buf, "%s %s get_pureg | grep %s > %s", PATH_IWPRIV, p, p, PATH_IWPRIV_DATA );
            cip_system(buf);
            if ((fp = fopen( PATH_IWPRIV_DATA, "r")) != NULL) {
                while (!feof(fp)) {
                    fgets(value, MAX_VALUE_LEN, fp);
                }
                fclose(fp);
                if (strstr(value, "get_pureg:1")) { // pure g
		    *band = IEEE80211_G;
		    return CIPCFG_SUCCESS;
                }else{
		    *band = IEEE80211_BG;
		    return CIPCFG_SUCCESS;
                }
            }
        }else if (strstr(value, "11b")){ //802.11b
	    *band = IEEE80211_B;
	    return CIPCFG_SUCCESS;
        }else if (strstr(value, "11a")) { //802.11a
	    *band = IEEE80211_A;
	    return CIPCFG_SUCCESS;
        }else if (strstr(value, "11n")) { //802.11n
	    sprintf( buf, "%s %s get_puren | grep %s > %s", PATH_IWPRIV, p, p, PATH_IWPRIV_DATA );
	    cip_system(buf);
	    if ((fp = fopen( PATH_IWPRIV_DATA, "r")) != NULL) {
		while (!feof(fp)) {
		    fgets(value2, MAX_VALUE_LEN, fp);
		}
		fclose(fp);
		if (strstr(value2, "get_puren:1")) // pure n
		    *band = IEEE80211_N;
		else if(strstr(value, "11ng"))
		    *band = IEEE80211_GN;
		else if(strstr(value, "11na"))
		    *band = IEEE80211_AN;
		else
		    *band = IEEE80211_N;
	    return CIPCFG_SUCCESS;
	    }
	}else{ // unknown band
	    *band = IEEE80211_UNKNOWN;
	    return CIPCFG_SUCCESS;
        }
    }
    return CIPCFG_ERR_GETSTAT;
}


int get_wlan_txpwr(char *intf, void *retval)
{
    int *txp = (int *)retval;
    char intfs[MAX_VALUE_LEN], buf[MAX_VALUE_LEN], *p, *d=":", *sp = NULL;
    int s=socket(PF_INET, SOCK_STREAM, 0);
    int iw_skfd;
    struct ifreq ifr;
    struct iwreq wrq;

    memset(intfs, 0, MAX_VALUE_LEN);
    strncpy(intfs, intf, MAX_VALUE_LEN - 1);

    /* Find the fisrt enabled VAP */
    for (p = strtok_r(intfs, d, &sp); p; p = strtok_r(NULL, d, &sp)) {
	strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
	if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
	    printf("No such interface : %s\n", ifr.ifr_name);
	    continue;
	}
	if (ifr.ifr_flags & IFF_UP)
	    break;
    }
    close (s);

    if (!p) {
	printf("No available interface.\n");
	return CIPCFG_ERR_GETSTAT;
    }

    if ((iw_skfd = iw_sockets_open()) < 0) {
	printf("Can't connect to iwlib.\n");
	return CIPCFG_ERR_GETSTAT;
    }

    if (iw_get_ext(iw_skfd, p, SIOCGIWTXPOW, &wrq) >= 0) {
	/* Madwifi driver always return power in dBm */
	*txp = (wrq.u.txpower.disabled ? 0 : wrq.u.txpower.value);
	iw_sockets_close(iw_skfd);
	return CIPCFG_SUCCESS;
    } else {
	iw_sockets_close(iw_skfd);
	return CIPCFG_ERR_GETSTAT;
    }
}

void a210_print_txpwr_dbm(char *arg, char *value, int (*get_stat)(char *, void *))
{
    int txp, max_txpwr;

    if ((*get_stat)(arg, &txp) == CIPCFG_SUCCESS) {
	if (txp >= 0)
	    sprintf(value, "%d dBm", txp);
	else
	    sprintf(value, "N/A");
    }
    else
	sprintf(value, "N/A");
}

int get_max_txpower_with_rfcard(char *interface)
{
    char buf[MAX_VALUE_LEN], item[MAX_VALUE_LEN], band[MAX_VALUE_LEN], rf_mode_with_ch[MAX_VALUE_LEN];
    char code[MAX_VALUE_LEN], rf_mode[MAX_VALUE_LEN], ch[MAX_VALUE_LEN], ch_width[MAX_VALUE_LEN];
    char chwidth_ext[MAX_VALUE_LEN];
    FILE *fp = NULL;
    char *p;
    int ret = -1;
    CIPCFG *cipcfg = NULL;

    //1. get conutry code
    cipcfg = init_cipcfg_f(current_config);
    if(!cipcfg){
        cipcfg_print("initial cipcfg error.\n");
        return -1;
    }else if( get_cfg_item(cipcfg, GID_RF1, "country", code) != CIPCFG_SUCCESS){
        cipcfg_print("Can NOT get country code, return value is %d.\n", ret);
        sprintf(code, "NA" ); // if can not get it, give it a default value.
	goto end;
    }

    sprintf(buf, "eval \"`cat %s`\"; echo $%s", COUNTRY_CODE, code);
    if ((fp = popen( buf, "r")) == NULL) {
        cipcfg_print("Can't open the country code file\n");
	goto end;
    }

    if(fgets(buf, sizeof(buf), fp) != NULL) {
        // get the country code
	sprintf(rf_mode, "c%d_", atoi(buf));
    }else
	goto end;
    pclose(fp);

    //2. get band
    sprintf( item, "rf%d_band", 1);
    if(get_sys_item( item, band) != CIPCFG_SUCCESS)
	goto end;
    if(!strcmp(band,"802.11a") ){
	//Not support
    }
    else if(!strcmp(band,"802.11a(S)") ){
	//Not support
    }
    else if(!strcmp(band, "802.11b") ){
	sprintf(rf_mode, "%s11b", rf_mode);
    }
    else if(!strcmp(band, "802.11g") ){
	sprintf(rf_mode, "%s11g", rf_mode);
    }
    else if(!strcmp(band, "802.11b+g") ){
	sprintf(rf_mode, "%s11g", rf_mode);
    }
    else if(!strcmp(band, "802.11n") ){
	//Not support
    }
    else if(!strcmp(band, "802.11a+n") ){
	//Not support
    }
    else if(!strcmp(band, "802.11g+n") || !strcmp(band, "802.11b+g+n")){
	sprintf(rf_mode, "%s11ng", rf_mode);
	if(get_cfg_item_by_str(cipcfg, "rf1", "ch_width", ch_width) == CIPCFG_SUCCESS){
            if(strcmp(ch_width, "20")){
                if(get_cfg_item_by_str(cipcfg, "rf1", "chwidth_ext", chwidth_ext) == CIPCFG_SUCCESS){
                    if(!strcmp(chwidth_ext, "l"))
			sprintf(rf_mode, "%sl", rf_mode);
		    else
			sprintf(rf_mode, "%su", rf_mode);
		}
	    }
	}
    }
    else if(!strcmp(band, "802.11a+b+g+n") ){
	//Not support
    }
    else {
	goto end;
    }

    //3. get channel
    sprintf( item, "rf%d_ch", 1 );
    if(get_sys_item( item, ch) != CIPCFG_SUCCESS)
	goto end;
    sprintf(rf_mode_with_ch, "%s_ch%s", rf_mode,ch);
    //this channel has special power setting?
    sprintf(buf, "eval \"`cat %s`\"; echo $%s", ALLOWED_POWER, rf_mode_with_ch);
    if ((fp = popen( buf, "r")) == NULL) {
        cipcfg_print("Can't open the country code file\n");
	goto end;
    }

    memset(buf, 0, MAX_VALUE_LEN);
    if(fgets(buf, sizeof(buf), fp) != NULL && strlen(buf) > 1) {
	p = strtok(buf, " ");
	ret = atoi(p);
    }else{
	pclose(fp);
	//find the power setting of rf_mode
	sprintf(buf, "eval \"`cat %s`\"; echo $%s", ALLOWED_POWER, rf_mode);
	if ((fp = popen( buf, "r")) == NULL) {
	    cipcfg_print("Can't open the country code file\n");
	    goto end;
	}
	if(fgets(buf, sizeof(buf), fp) != NULL && strlen(buf) > 1) {
	    p = strtok(buf, " ");
	    ret = atoi(p);
	}
    }
    pclose(fp);

end:
    close_cipcfg(cipcfg);
    return ret;
}

void a210_print_txpwr(char *arg, char *value, int (*get_stat)(char *, void *))
{
    int txp, max_txpwr;
    if ((*get_stat)(arg, &txp) == CIPCFG_SUCCESS) {
        max_txpwr = get_max_txpower_with_rfcard(arg);
	if (max_txpwr == -1){
	    sprintf(value, "N/A");
        }else if (txp > max_txpwr -3 ){
            sprintf(value, "Highest");
        }else if (txp <= (max_txpwr - 3) && txp > (max_txpwr - 6)){
            sprintf(value, "High");
        }else if (txp <= (max_txpwr - 6) && txp > (max_txpwr - 9)){
            sprintf(value, "Medium");
        }else if (txp <= (max_txpwr - 9) && txp > (max_txpwr - 12)){
            sprintf(value, "Low");
        }else {
            sprintf(value, "Lowest");
        }
    } else {
        sprintf(value, "N/A");
    }
}

int get_sta(char *intf, void *retval)
{
    int *num = (int *)retval;
    ASSOC_STA_LIST *sta_list = NULL;
    *num = 0;

    sta_list = get_sta_list(intf);
    *num = sta_list_count(sta_list);
    free_sta_list(sta_list);
    return CIPCFG_SUCCESS;
}

int get_wds_peer(char *intf, void *retval)
{
    unsigned char *mac = (char *)retval;
    char del[] = ":", *tok;
    ASSOC_WDS_LIST *wds_node = NULL;
    int i = 0;

    wds_node = get_wds_list(intf);
    if (wds_node == NULL)
	return CIPCFG_ERR_GETSTAT;

    for (i = 0, tok = strtok(wds_node->mac, del); i < 6 && tok; 
	    i++, tok = strtok(NULL, del)) {
	unsigned int tmp;
	sscanf(tok, "%X", &tmp);
	mac[i] = tmp;
    }

    free_wds_list(wds_node);
    return CIPCFG_SUCCESS;
}

int get_wds_txrate(char *intf, void *retval)
{
    double *txrate = (double *)retval;
    ASSOC_WDS_LIST *wds_node = NULL;

    wds_node = get_wds_list(intf);
    if (wds_node == NULL)
	return CIPCFG_ERR_GETSTAT;
    *txrate = wds_node->tx_rate;

    free_wds_list(wds_node);
    return CIPCFG_SUCCESS;
}

int get_wds_idle(char *intf, void *retval)
{
    int *idle = (int *)retval;
    ASSOC_WDS_LIST *wds_node = NULL;

    wds_node = get_wds_list(intf);
    if (wds_node == NULL)
	return CIPCFG_ERR_GETSTAT;
    *idle = wds_node->last_rx;

    free_wds_list(wds_node);
    return CIPCFG_SUCCESS;
}

int get_wds_snr(char *intf, void *retval)
{
    int *snr = (int *)retval;
    ASSOC_WDS_LIST *wds_node = NULL;

    wds_node = get_wds_list(intf);
    if (wds_node == NULL)
	return CIPCFG_ERR_GETSTAT;
    *snr = wds_node->rssi;

    free_wds_list(wds_node);
    return CIPCFG_SUCCESS;
}

int get_phy_cnt(char *file, void *retval)
{
    int *cnt = (int *)retval;
    int ether_cnt, rf_cnt;

    get_ether_cnt(NULL, &ether_cnt);
    get_rf_cnt(NULL, &rf_cnt);

    *cnt = ether_cnt + rf_cnt;
    return CIPCFG_SUCCESS;
}

int get_ether_cnt(char *file, void *retval)
{
    int *cnt = (int *)retval;
    *cnt = 1;
    return CIPCFG_SUCCESS;
}

int get_rf_cnt(char *file, void *retval)
{
    int *cnt = (int *)retval;
    *cnt = 1;
    return CIPCFG_SUCCESS;
}

int write_mtd(char *data, int len, char *mtd_dev)
{
    int fd, ret;
    struct mtd_info_user mtd;
    struct erase_info_user erase;
    int w, i;

    if ((fd = open(mtd_dev, O_SYNC|O_RDWR)) < 0) {
        fprintf(stderr, "Can't open the MTD device form %s\n", mtd_dev);
        return SYS_ERR_WRITE_FLASH;
    }
    
    if(ioctl(fd, MEMGETINFO, &mtd)) {
        fprintf(stderr, "Can't get MTD device info from %s\n", mtd_dev);
        close(fd);
        return SYS_ERR_WRITE_FLASH;
    }

    i = w = 0;
    /* erase 1 block then write 1 block */
    while (len > i) {
        erase.start = i;
        erase.length = mtd.erasesize;

        if (ioctl (fd, MEMERASE, &erase) < 0) {
            fprintf(stderr, "Erase MTD device %s fail\n", mtd_dev);
            close(fd);
            return SYS_ERR_WRITE_FLASH;
        }

        if ((len-i) < mtd.erasesize) {
	    printf("!");
	    fflush(stdout);
            //fprintf(stderr, "Erase MTD device %s, data length(%d) less then erasesize\n", mtd_dev, len);
        }

        if ((len-i) >= mtd.erasesize)
            w = mtd.erasesize;
        else
            w = len - i;

        i += w;
        printf("-");
	fflush(stdout);

        if ((ret = write(fd, data, w)) < w) {
            fprintf(stderr, "Write the firmware image fail, ret = %d\n", ret);
            close(fd);
            return SYS_ERR_WRITE_FLASH;
        }
        sync();
        data += w;
        printf("+");
	fflush(stdout);

    }
    printf("\n");

    if (close(fd) < 0) {
        printf("Can't close\n");
    }

    return SYS_WRITE_SUCCESS;
}

/*
 * Check the brand ID
 */
int check_brand(char *brand_id)
{
    int i;
    char company[MAX_VALUE_LEN];
    char model[MAX_VALUE_LEN];

    if (get_product_info(PRODUCT_INFO_COMPANY, company) != CIPCFG_SUCCESS)
	memset(company, 0, MAX_VALUE_LEN);
    if (get_product_info(PRODUCT_INFO_MODEL, model) != CIPCFG_SUCCESS)
	memset(model, 0, MAX_VALUE_LEN);
    printf("company: \"%s\", model: \"%s\"\n", company, model);

    for (i = 0; brand_id_list[i].id[0] && brand_id_list[i].id[1]; i++) {
	if (memcmp(brand_id, brand_id_list[i].id, 2) == 0 &&
		strcmp(company, brand_id_list[i].company) == 0 &&
		strcmp(model, brand_id_list[i].model) == 0)
	    return 1;
    }

    return -1;
}

/*
 * Upgrade the new firmware
 */
int upload_firmware(unsigned char *upload_img, int len)
{
    unsigned char *fis, *redboot_config, *kernel, *rootfs, magbuf[MAGIC_LEN];
    FILE *fp = NULL;
    char olroot[64];
    char part_name[32];
    char cmd_buf[256];
    FW_HEADER *fw_hdr;
    int ret, root_num;

    if (!upload_img || len < sizeof(FW_HEADER))
	return SYS_ERR_CHECK_SUM;

    fw_hdr = (FW_HEADER *)upload_img;

    if (fw_hdr->fw_len != (fw_hdr->fis_len + fw_hdr->kernel_len + fw_hdr->rootfs_len + fw_hdr->conf_len + fw_hdr->redboot_conf_len) || len != (fw_hdr->fw_len + sizeof(FW_HEADER)))
	return SYS_ERR_CHECK_SUM;

    /* OLROOT path */
    memset(olroot, 0, 64);
    fp = popen("if [ -f /etc/rc.d/rc.env ]; then . /etc/rc.d/rc.env; fi; echo -n $OLROOT/.up", "r");
    fread(olroot, 1, 64, fp);
    pclose(fp);
    printf("\nOLROOT: %s\n", olroot);

#if 0
    memset(part_name, 0, 32);
    cip_system(CHK_ROOT_MTD);
    fp = fopen(ROOT_MTD_FILE, "r");
    fread(part_name, 31, sizeof(char), fp);
    fclose(fp);
    if(strncmp(part_name, MTD_ROOTFS1, sizeof(MTD_ROOTFS1)) == 0) {
        root_num = 0;
    } else if(strncmp(part_name, MTD_ROOTFS2, sizeof(MTD_ROOTFS2)) == 0) {
        root_num = 1;
    } else { printf("Root Partition is Not Found!\n");
        return SYS_ERR_CHECK_SUM;
    }
#endif

    fis    = (unsigned char *)upload_img + sizeof(FW_HEADER);
    kernel = (unsigned char *)fis + fw_hdr->fis_len; //fis size is 60KB Redboot Config
    rootfs = (unsigned char *)kernel + fw_hdr->kernel_len; // kernel size is 512KB
    redboot_config = (unsigned char *)rootfs + fw_hdr->rootfs_len; //fis size is 60KB Redboot Config
#if 0
    if(root_num) { // we use the second redboot config.
        redboot_config += (fw_hdr->redboot_conf_len>>1); // we have two part of redboot_config
    }
#endif

    if ( fw_hdr->fw_sum != checksum(fis, fw_hdr->fw_len)) {
        printf("Checksum mismatch!!!, correct checksum is %d\n", fw_hdr->fw_sum);
        return SYS_ERR_CHECK_SUM;
    }

    if (check_brand(fw_hdr->brand_id) == -1) {
	printf("Can't upgrade firmware with other brand\n");
	return SYS_ERR_BRAND_MISMATCH;
    }

    memset(magbuf, 0, MAGIC_LEN );
    ret=prd_tbl_lookup(magic, magbuf, NULL);
    if( ret < 0 ){
        return ret;
    }
    if (strncmp(fw_hdr->magic, magbuf, MAGIC_LEN) ) {
        return SYS_ERR_MAGIC_NUM;
    } else{
        printf("Correct magic..\n");
    }

    printf("New firmware version %s\n", fw_hdr->fw_ver);

    /* Write the file to flash*/

    /*  Write Kernel  */
    printf("Start write Kernel, len : %d\n", fw_hdr->kernel_len);
    memset(part_name, 0,32);
    if(find_mtd(MTD_KERNEL, part_name) < 0){
        printf("Can not find partition \"%s\".\n", MTD_KERNEL);
        return SYS_ERR_WRITE_FLASH;
    }
    printf("\n%s : %s\n", MTD_KERNEL, part_name);
    if( SYS_WRITE_SUCCESS != write_mtd(kernel, fw_hdr->kernel_len, part_name) ){
        printf("write Kernel error\n");
        return SYS_ERR_WRITE_FLASH;
    }

    /*  Write RootFS  */
    printf("Start write RootFS, len : %d\n", fw_hdr->rootfs_len);
    memset(part_name, 0, 32);
    if(find_mtd(MTD_ROOTFS, part_name) < 0){
	printf("Can not find partition \"%s\".\n", MTD_ROOTFS1);
	return SYS_ERR_WRITE_FLASH;
    }
    printf("\n%s : %s\n", MTD_ROOTFS, part_name);

    if( SYS_WRITE_SUCCESS != write_mtd(rootfs, fw_hdr->rootfs_len, part_name) ) {
        printf("write RootFS error\n");
        return SYS_ERR_WRITE_FLASH;
    }

    /* Write uboot-env */
    printf("Start write uboot-env, len : %d\n", fw_hdr->redboot_conf_len);
    memset(part_name, 0,32);
    if(find_mtd(MTD_UBOOT_CFG, part_name) < 0){
        printf("Can not find partition \"%s\".\n", MTD_UBOOT_CFG);
        return SYS_ERR_WRITE_FLASH;
    }
    printf("\n%s : %s\n", MTD_UBOOT_CFG, part_name);
    if( SYS_WRITE_SUCCESS != write_mtd(redboot_config, 
		fw_hdr->redboot_conf_len, part_name) ){
        printf("write uboot-env error\n");
        return SYS_ERR_WRITE_FLASH;
    }

    if(strcmp("/.up", olroot) != 0) {
        printf("touch %s\n",olroot);
        fp = fopen(olroot, "w+"); /* no truncate */
        fwrite("1", 1, 1, fp);
        fclose(fp) ;
    }

    return SYS_UPLOAD_SUCCESS;
}

/** 
 * @brief Get interface name by RF number and VAP number (both index begins from 0)
 * 
 * @param rf_idx 	RF card index.
 * @param sub_idx 	VAP index.
 * 
 * @return 		the interface name. If the index is incorrect return NULL.
 */
char *get_ifname_by_idx(IF_ROLE_T role, int rf_idx, int sub_idx, char *ifname)
{
    if (rf_idx < 0 || rf_idx >= MAX_RF) 
	return NULL;

    switch (role) {
	case IF_ROLE_MASTER:
	    sprintf(ifname, "wifi%d", rf_idx);
	    return ifname;
	case IF_ROLE_VAP:
	    if (sub_idx >= 0 && sub_idx < MAX_VAP) {
		sprintf(ifname, "ath%dap%d", rf_idx, sub_idx);
		return ifname;
	    } else {
		return NULL;
	    }
	case IF_ROLE_WDS:
	    if (sub_idx >= 0 && sub_idx < MAX_WDS_LINK) {
		sprintf(ifname, "ath%dwds%d", rf_idx, sub_idx);
		return ifname;
	    } else {
		return NULL;
	    }
	case IF_ROLE_STA:
	    if (sub_idx >= 0 && sub_idx < MAX_STA) {
		sprintf(ifname, "ath%dsta%d", rf_idx, sub_idx);
		return ifname;
	    } else {
		return NULL;
	    }
	case IF_ROLE_REP:
	    if (sub_idx >= 0 && sub_idx < MAX_REP) {
                sprintf(ifname, "ath%dsta%d", rf_idx, sub_idx);
                return ifname;
            } else {
                return NULL;
            }
	default:
	    return NULL;
    }
}

/** 
 * @brief Check if the WDS Link is alive.
 * 
 * @param ifname 	Interface name.
 * 
 * @return 		If is alive return 1 otherwise 0.
 */
int isWDSLinkAlive(char *ifname)
{
    ASSOC_STA_LIST * sta = NULL;
    double last_rx;
    int okcnt = 0;
    FILE *fp = NULL;
    char cmd[BUFSIZ], output[BUFSIZ];

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    last_rx = sta->last_rx;
    okcnt = sta->tx_okcnt;

    free_sta_list(sta);

    sprintf(cmd, "/sbin/check_peer %s\n", ifname);
    if ((fp = popen(cmd, "r")) != NULL)
	fgets(output, sizeof(output), fp);
    pclose(fp);

    return (((okcnt > 0 && last_rx < 120) && (memcmp("ACTIVE", output, strlen("ACTIVE")) == 0)) ? 1 : 0);
}

/** 
 * @brief Get WDS Link Tx Rate
 * 
 * @param ifname 	Interface name.
 * @param value 	Return the Tx rate in string.
 * 
 * @return 		Return 1 for sucess otherwise return 0.
 */
int getWDSLinkTxRate(char *ifname, char *value)
{
    ASSOC_STA_LIST * sta = NULL;
    int rate;

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    rate = (int)(sta->tx_rate * 2);
    sprintf(value, "%d%s", rate/2, (rate%2)==1 ? ".5":"");

    free_sta_list(sta);
    return 1;
}

/** 
 * @brief Get WDS Link Quality
 * 
 * @param ifname 	Interface name.
 * @param value 	Return the Quality in string.
 * 
 * @return 		Return 1 for success otherwise reutrn 0.
 */
int getWDSLinkQuality(char *ifname, char *value)
{
    ASSOC_STA_LIST * sta = NULL;

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    sprintf(value, "%d", sta->rssi);

    free_sta_list(sta);
    return 1;
}

/** 
 * @brief Get WDS Link Health
 * 
 * @param ifname 	Interface name.
 * @param value 	Return the health percentage (Tx fail) in string
 * 
 * @return 		1 for success otherwise 0.
 */
int getWDSLinkHealth(char *ifname, char *value)
{
    ASSOC_STA_LIST *sta = NULL;
    int okcnt, errcnt, total;

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    okcnt = sta->tx_okcnt;
    errcnt = sta->tx_errcnt;
    total = okcnt + errcnt;
    free_sta_list(sta);

    if (total != 0) {
	sprintf(value, "%d", errcnt*100/total);
	return 1;
    }
    return 0;
}

/** 
 * @brief Get Tx total frames.
 * 
 * @param ifname 	Interface name.
 * @param value 	Return the frame count in string.
 * 
 * @return 		1 for sucess otherwise 0.
 */
int getWDSLinkTxTotal(char *ifname, char *value)
{
    ASSOC_STA_LIST *sta = NULL;

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    sprintf(value, "%d", sta->tx_okcnt + sta->tx_errcnt);

    free_sta_list(sta);
    return 1;
}

/** 
 * @brief Get Tx fail frames.
 * 
 * @param ifname 	Interface name.
 * @param value 	Return the frame count in string.
 * 
 * @return 		1 for sucess otherwise 0.
 */
int getWDSLinkTxFail(char *ifname, char *value)
{
    ASSOC_STA_LIST *sta = NULL;

    if ((sta = get_sta_list(ifname)) == NULL)
	return 0;

    /* It should be only one associated sta for WDS link */
    sprintf(value, "%d", sta->tx_errcnt);

    free_sta_list(sta);
    return 1;
}


int rfcards(char *func, char *value)
{
    int i, gid[MAX_RF] = { GID_RF1 };
    const char *rf[MAX_RF] = { rf1_if };
    char *p, fun[32], buf[64], attr_value[MAX_VALUE_LEN];
    memset( buf, 0, 64 );

    if( !strncmp("ALL", func, 3) ){
        for ( i=0; i<MAX_RF; i++){
            value[i] = '0';
            sprintf( buf, "cat %s |grep -q %s", PATH_IRQ, rf[i] );
            if( !cip_system(buf) ){ 
                value[i] = '1';
            }
        }
    }else{
        CIPCFG *cipcfg = NULL;
        memset( fun, 0, 32 );
        strncpy(fun, func, strlen(func));

        for(i=0; i<strlen(fun); i++) {
            fun[i]=tolower(fun[i]);
        }

        cipcfg = init_cipcfg();
        for ( i=0; i<MAX_RF; i++){
            value[i] = '0';
            sprintf( buf, "cat %s |grep -q %s", PATH_IRQ, rf[i] );
            if( !cip_system(buf) ){ 
                get_cfg_item(cipcfg, gid[i], "roles", attr_value );
                p = strstr(attr_value, fun);
                if(p){
                    value[i] = '1';
                }
            }
        }

        close_cipcfg(cipcfg);
    }
    return 0;
}

int getWebAccount(int idx, char *n, char *p)
{
    CIPCFG *cipcfg;
    int ret = 0;
    char item_value[MAX_VALUE_LEN], name[64], passwd[64];

    if(idx > 0){
        // Because we just have one account
        return -1;
    }
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    ret=get_cfg_item(cipcfg, GID_ACCOUNT, "user:0.name", item_value);
    if (ret) {
        printf("get_cfg_items(init user name) fail with error code %d.\n", ret);
	goto end;
    }
    else {
        strcpy(name, item_value);
        printf("The value of 'user:0.name' is '%s'.\n", name);
    }

    ret=get_cfg_item(cipcfg, GID_ACCOUNT, "user:0.password", item_value);
    if (ret) {
        printf("get_cfg_items(init user password) fail with error code %d.\n", ret);
	goto end;
    }
    else {
        strcpy(passwd, item_value);
        printf("The value of 'user:0.password' is '%s'.\n", passwd);
    }
    strcpy(n, name);
    strcpy(p, passwd);
end:
    close_cipcfg(cipcfg);
    return ret;
}

int getCurrentOptMode(char *key, void *retval)
{
    char *value = (char *)retval;
    CIPCFG *cipcfg = NULL;
    char    buf[MAX_VALUE_LEN];
    
    memset(buf, 0, MAX_VALUE_LEN);
    cipcfg = init_cipcfg_f(current_config);
    get_cfg_item(cipcfg, GID_OPMODE, "type", buf);
    close_cipcfg(cipcfg);
    strcpy(value, buf);
    return CIPCFG_SUCCESS;
}
#ifdef AUTO_SET_WEP
#if MAX_VAP > 8
#error MAX_VAP cannot be greater than 8 
#endif
ITEM_LIST *set_wep(int grpgid, char *itemname, char *itemvalue)
{   
    int gid[] = {
        GID_RF1_VAP1_SEC, GID_RF1_VAP2_SEC, GID_RF1_VAP3_SEC, GID_RF1_VAP4_SEC,
        GID_RF1_VAP5_SEC, GID_RF1_VAP6_SEC, GID_RF1_VAP7_SEC, GID_RF1_VAP8_SEC
    };
    ITEM_LIST *tmp, *head = NULL;
    int i, j;
    int rfidx=0;

    for(i=0;i< MAX_VAP;i++) 
    {
        tmp = (ITEM_LIST *) malloc(sizeof(ITEM_LIST));
        tmp->gid = gid[i];
        tmp->item_name = strdup(itemname);
        tmp->item_value = strdup(itemvalue);
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
    }
    return head;
}
#endif

int get_magic_str(char *arg, void *retval)
{
    int ret;
    char *value = (char *) retval;

    if (prd_tbl_lookup(magic, value, NULL) < 0)
	return CIPCFG_ERR_GETSTAT;
    value[MAGIC_LEN] = '\0';

    return CIPCFG_SUCCESS;
}

int getWDSLinkStatus(char *ifname, char *value)
{
    int             s = 0, ifUp = -1;
    struct ifreq    ifr;
    s = socket(PF_INET, SOCK_STREAM, 0);
    memset(&ifr, 0, sizeof(struct ifreq));
    strncpy(ifr.ifr_name, ifname, strlen(ifname));
    if (ioctl(s, SIOCGIFFLAGS, &ifr) >= 0){
	if(ifr.ifr_flags & IFF_UP)
	    ifUp = 1;
    }

    strcpy(value, (ifUp == 1)? "Enabled":"Disabled");
    return 1;

}


int getWDSLinkSecurity(char *ifname, char *value)
{
    CIPCFG *cipcfg = NULL;
    char    buf[MAX_VALUE_LEN];
    
    memset(buf, 0, MAX_VALUE_LEN);

    cipcfg = init_cipcfg();
        
    get_cfg_item_by_str(cipcfg, "rf1_wds1", "wep.enabled", buf);
        if(strcmp(buf, "enable") == 0)
            strcpy(value, "WEP");
	else {
	    get_cfg_item_by_str(cipcfg, "rf1_wds1", "wpa.enabled", buf);
	    if(strcmp(buf, "enable") == 0)
		strcpy(value, "WPA-PSK");
	    else 
		strcpy(value, "None");
	}

    close_cipcfg(cipcfg);
    return 1;
}

char *getRepeaterIfName(char *ifname)
{
    CIPCFG *cipcfg = NULL;
    char buf[64], roles[MAX_VALUE_LEN];
    
    cipcfg = init_cipcfg_f(current_config);
    get_cfg_item_by_str(cipcfg, "rf1", "roles", roles);
    close_cipcfg(cipcfg);

    memset(buf, 0, 64);
    if(strstr(roles, "rep") != NULL) {
        get_rep_ifname_by_idx(0, 0, buf);
    } else if(strstr(roles, "wds") != NULL) {
	int i;
	for (i = 0; i < MAX_WDS_LINK; i++) {
	    get_wds_ifname_by_idx(0, i, buf);
	    if(buf[0] != '\0')
		break;
	}
    } else {
	return NULL;
    }
    
    if (buf[0] == '\0') {
	return NULL;
    } else {
	strcpy(ifname, buf);
	return ifname;
    }
}

int getRepeaterIfStatus(char *key, void *retval)
{
    char *value = (char *)retval;
    char            ifname[64], *ret;
    int             s = 0, ifUp = -1;
    struct ifreq    ifr;

    s = socket(PF_INET, SOCK_STREAM, 0);
    memset(ifname, 0, 64);
    memset(&ifr, 0, sizeof(struct ifreq));
    ret = getRepeaterIfName(ifname);
    if (ret) {
	strncpy(ifr.ifr_name, ifname, strlen(ifname));
	if (ioctl(s, SIOCGIFFLAGS, &ifr) >= 0){
	    if(ifr.ifr_flags & IFF_UP)
		ifUp = 1;
	}
    }
    strcpy(value, (ifUp == 1)? "Enabled":"Disabled");
    if(s)
        close(s);
    return CIPCFG_SUCCESS;
}

int getRepeaterMode(char *key, void *retval)
{
    char *value = (char *)retval;
    CIPCFG *cipcfg = NULL;
    char    buf[MAX_VALUE_LEN];
    
    memset(buf, 0, MAX_VALUE_LEN);

    cipcfg = init_cipcfg_f(current_config);
    get_cfg_item_by_str(cipcfg, "rf1", "roles", buf);
    close_cipcfg(cipcfg);
    printf("Roles: %s\n", buf);
    
    if(strstr(buf, "rep") != NULL)
        strcpy(value, "Universal Repeater");
    else if(strstr(buf, "wds") != NULL)
        strcpy(value, "WDS");
    else
        strcpy(value, "Unknown");
    return CIPCFG_SUCCESS;
}

int getRepeaterSecurity(char *key, void *retval)
{
    char *value = (char *)retval;
    CIPCFG *cipcfg = NULL;
    char    buf[MAX_VALUE_LEN];
    
    memset(buf, 0, MAX_VALUE_LEN);

    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "rf1", "roles", buf);
    if(strstr(buf, "rep")){
        get_cfg_item_by_str(cipcfg, "rf1_rep1_sec", "mode", buf);
        if(strcmp(buf, "wpa") == 0)
            strcpy(value, "WPA-PSK");
        else {
            get_cfg_item_by_str(cipcfg, "rf1_rep1_sec", "wep.enabled", buf);
            if(strcmp(buf, "disable") == 0)
                strcpy(value, "None");
            else
                strcpy(value, "WEP");
        }
    }else{
        get_cfg_item_by_str(cipcfg, "rf1_wds1", "wep.enabled", buf);
        if(strcmp(buf, "enable") == 0)
            strcpy(value, "WEP");
	else {
	    get_cfg_item_by_str(cipcfg, "rf1_wds1", "wpa.enabled", buf);
	    if(strcmp(buf, "enable") == 0)
		strcpy(value, "WPA-PSK");
	    else 
		strcpy(value, "None");
	}
    }
    close_cipcfg(cipcfg);
    return CIPCFG_SUCCESS;
}

char *getRfStaInfo(char *ifname, char *buf)
{
    char    cmd[MAX_VALUE_LEN];
    FILE    *fd = NULL;
    
    sprintf(cmd, "%s %s", PATH_BSS_INFO, ifname);
    fd = popen(cmd, "r");
    if(fd){
        fgets(buf, MAX_VALUE_LEN, fd);
        pclose(fd);
    }
    // The result format "Status(10 bytes)SSID(32 bytes)MAC(17 bytes)
    // Channel(2 bytes)RSSI(3 bytes) Refer to 
    // fruitfarm/packages/rtl8186-wireless/bss_info.c
    return buf;
}

char *getRfStaModeIfName(char *ifname)
{
    CIPCFG *cipcfg = NULL;
    char    buf[MAX_VALUE_LEN];
    
    memset(buf, 0, MAX_VALUE_LEN);
    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "opmode", "type", buf);
    close_cipcfg(cipcfg);
    if(strcmp(buf, "CPE") == 0)
        get_sta_ifname_by_idx(0, 0, ifname);
    else if(strcmp(buf, "AP") == 0)
        get_rep_ifname_by_idx(0, 0, ifname);
        
    return ifname;
}

int getRfStaModeSsid(char *key, void *retval)
{
    char *value = (char *)retval;
    struct iwreq wrq;
    int skfd; 

    memset(&wrq, 0, sizeof(wrq));
    *value = '\0';

    if ((skfd = iw_sockets_open()) < 0) {
	return CIPCFG_ERR_GETSTAT; 
    }

    wrq.u.essid.pointer = value;
    wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
    wrq.u.essid.flags = 0;

    iw_get_ext(skfd, key, SIOCGIWESSID, &wrq);
    iw_sockets_close(skfd);

    return CIPCFG_SUCCESS;
}

int getRfStaModeMac(char *key, void *retval)
{
    unsigned char *value = (unsigned char *)retval;
    char buf[MAX_VALUE_LEN], *tmp = NULL;
    int skfd, i = 0;
    struct iwreq wrq;
    unsigned long mac;

    *value = '\0';
    memset(&wrq, 0, sizeof(wrq));
    memset(buf, 0, sizeof(buf));

    if ((skfd = iw_sockets_open()) < 0) {
	return CIPCFG_ERR_GETSTAT;
    }

    if (iw_get_ext(skfd, key, SIOCGIWAP, &wrq) >= 0) {
	iw_sawap_ntop(&wrq.u.ap_addr, buf);
    }
    
    if (skfd >= 0)
	iw_sockets_close(skfd);

    tmp = strchr(buf, '\n');
    if(tmp)
	*tmp = '\0';
    tmp = strtok(buf, ":");
    while(tmp != NULL && i < 6) {
	mac = strtoul(tmp, NULL, 16);
	value[i++] = mac;
	tmp = strtok(NULL, ":");
    }
    return CIPCFG_SUCCESS;
}

int getRfStaModeChannel(char *key, void *retval)
{
    int *ch = (int *)retval;
    FILE* fp = NULL;
    char intfs[MAX_VALUE_LEN], buf[MAX_VALUE_LEN], *p, *d=":";
    int s=socket(PF_INET, SOCK_STREAM, 0);
    int iw_skfd = iw_sockets_open();
    struct ifreq ifr;
    struct iwreq wrq;

    memset(intfs, 0, MAX_VALUE_LEN);
    strcpy(intfs, key);
    p = strtok(intfs, d);

    strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
    if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0){
        // ERROR handle
        printf("No such interface : %s\n", ifr.ifr_name);
        close(s);
        return CIPCFG_ERR_GETSTAT;
    }
    while(!(ifr.ifr_flags & IFF_UP)){
        if((p = strtok(NULL, d))){
            strncpy(ifr.ifr_name, p, sizeof(ifr.ifr_name));
            ioctl(s, SIOCGIFFLAGS, &ifr);
        }else{ // interface DOWN
            printf("Interface %s is Down.\n", ifr.ifr_name );
            close(s);
            return CIPCFG_ERR_GETSTAT;
        }
    }
    close(s);

    if (iw_skfd < 0) {
	close(iw_skfd);
	return CIPCFG_ERR_GETSTAT;
    }

    if (iw_get_ext(iw_skfd, p, SIOCGIWFREQ, &wrq) >= 0) {
	int freq = iw_freq2float(&(wrq.u.freq)) / 1000000;
	if ((freq / 1000) == 2 && freq >= 2412) {
	    *ch = ((freq - 2412) / 5) + 1;
	    close(iw_skfd);
	    return CIPCFG_SUCCESS;
	} else if ((freq / 1000) == 5 && freq >= 5180) {
	    *ch = ((freq - 5180) / 5) + 36;
	    close(iw_skfd);
	    return CIPCFG_SUCCESS;
	} else {
	    close(iw_skfd);
	    return CIPCFG_ERR_GETSTAT;
	}
    } else {
	close(iw_skfd);
	return CIPCFG_ERR_GETSTAT;
    }
}

int getRfStaModeRssi(char *key, void *retval)
{
    int *rssi = (int *)retval;
    int skfd, has_range = 0;
    struct iwreq wrq;
    struct iw_range range;  
    struct iw_statistics stats;
    int ret = CIPCFG_ERR_GETSTAT;

    *rssi = 0;
    if ((skfd = iw_sockets_open()) < 0) {
	return ret;
    }
    
    if (iw_get_range_info(skfd, key, &range) < 0)
	memset(&range, 0, sizeof(range));
    else
	has_range = 1;
   
    if (iw_get_stats(skfd, key, &stats, &range, has_range) >= 0) {
	*rssi = stats.qual.qual;
	ret = CIPCFG_SUCCESS;
    }

    iw_sockets_close(skfd);
    return ret;
}

int getRfStaModeTxErr(char *key, void *retval)
{
    int *tx_err = (int *)retval;
    struct iwreq iwr;
    char mac_str[17];
    struct ieee80211req_sta_stats stats;
    const struct ieee80211_nodestats *ns = &stats.is_stats;
    u_int8_t macaddr[6];
    int s;

    s = socket(AF_INET, SOCK_DGRAM, 0);
    (void) memset(&iwr, 0, sizeof(iwr));
    (void) strncpy(iwr.ifr_name, INTERFACE_STA, sizeof(iwr.ifr_name));

    iwr.u.data.pointer = (void *) &stats;
    iwr.u.data.length = sizeof(stats);

    memset(mac_str, 0, sizeof(mac_str));
    get_sys_item("rf_sta_m_mac", mac_str);

    if (strcmp(mac_str, "00:00:00:00:00:00") == 0) {
	*tx_err = 0;
	return CIPCFG_SUCCESS;
    }

    mac2bin(mac_str, (char *)macaddr);

    memcpy(stats.is_u.macaddr, macaddr, IEEE80211_ADDR_LEN);

    if (ioctl(s, IEEE80211_IOCTL_STA_STATS, &iwr) < 0){
	close(s);
	return CIPCFG_ERR_GETSTAT;
    }
    close(s);
    *tx_err = ns->ns_tx_errcnt;

    return CIPCFG_SUCCESS;
}

int getRfStaModeTxRate(char *key, void *retval)
{
    int *rate = (int *)retval, ret = 0;
    struct iwreq wrq;
    int skfd;

    if ((skfd = iw_sockets_open()) < 0) {
	*rate = 0;
	return CIPCFG_ERR_GETSTAT;
    }

    if(iw_get_ext(skfd, key, SIOCGIWRATE, &wrq) >= 0) {
	*rate = wrq.u.bitrate.value / 1000000;
    }

    iw_sockets_close(skfd);
    return CIPCFG_SUCCESS;
}


int getRfStaModeTxCount(char *key, void *retval)
{
    int *tx_cnt = (int *)retval;
    struct iwreq iwr;
    char mac_str[17];
    struct ether_addr *mac;
    int s;
    struct ieee80211req_sta_stats stats;
    const struct ieee80211_nodestats *ns = &stats.is_stats;
    u_int8_t macaddr[6];

    s = socket(AF_INET, SOCK_DGRAM, 0);
    (void) memset(&iwr, 0, sizeof(iwr));
    (void) strncpy(iwr.ifr_name, INTERFACE_STA, sizeof(iwr.ifr_name));

    iwr.u.data.pointer = (void *) &stats;
    iwr.u.data.length = sizeof(stats);

    memset(mac_str, 0, sizeof(mac_str));
    get_sys_item("rf_sta_m_mac", mac_str);

    if (strcmp(mac_str, "00:00:00:00:00:00") == 0) {
	*tx_cnt = 0;
	return CIPCFG_SUCCESS;
    }

    mac2bin(mac_str, (char *)macaddr);

    memcpy(stats.is_u.macaddr, macaddr, IEEE80211_ADDR_LEN);

    if (ioctl(s, IEEE80211_IOCTL_STA_STATS, &iwr) < 0){
	close(s);
	return CIPCFG_ERR_GETSTAT;
    }
    close(s);
    *tx_cnt = ns->ns_tx_okcnt;

    return CIPCFG_SUCCESS;
}
