#ifndef _A210_EXTENSION_H
#define _A210_EXTENSION_H

#include "syscall.h"

#define DEVICE_NAME             "A210"
#define CONFIG_FILE_ROM         "/conf/cipcfg-ROM.xml"
#define CONFIG_FILE_RAM         "/tmp/cipcfg-RAM.xml"
#define BACKUP_CONFIG_FILE      "/web/html/system/config-backup.conf"
#define RESTORED_CONFIG_FILE    "/conf/restored-config.xml"
#define CONFIG_FILE_CURRENT     "/tmp/cipcfg-current.xml"
#define REDBOOT_CONFIG_FILE     "/tmp/redboot-config"
#define CONFIG_FILE_CURRENT     "/tmp/cipcfg-current.xml"
#define ROOT_MTD_FILE           "/var/root_mtd"
#define CHK_ROOT_MTD            "/sbin/chk_root_mtd.sh file"
#define OCTET 8
#define NUM_IP_OCTET 4
#define PATH_BSS_INFO           "/sbin/bss_info"
#define PATH_RF_STA_MODE_INFO   "/proc/net/madwifi/ath0sta0/associated_sta"

// define partition name
#define MTD_UBOOT_CFG		"u-boot-env"
#define MTD_KERNEL              "uImage"
#define MTD_ROOTFS              "rootfs"
#define MTD_ROOTFS1             "rootfs1"
#define MTD_ROOTFS2             "rootfs2"

#define CHROOT_CMD              "{ dd if=%s of=/dev/ram0; } && { mount -t squashfs /dev/ram0 /mnt; } && { pivot_root /mnt /mnt/rom; } && { mount -o move /rom/proc /proc; mount -o move /rom/var /var; mount -o move /rom/conf /conf; }"

// define max RF number
#ifndef MAX_RF
#define MAX_RF                  1
#endif

//define RF Card Name
#define RF_1_NAME               "RF Card"

//define interface
#define INTERFACE_WAN           "br0"

/*
 * TODO: These wireless interface name definitions are obsolete. 
 * You should used ``get_ifname_by_idx()'' to get interface name of wireless.
 */
#define INTERFACE_VAP1          "ath0ap0"
#define INTERFACE_VAP2          "ath0ap1"
#define INTERFACE_VAP3          "ath0ap2"
#define INTERFACE_VAP4          "ath0ap3"
#define INTERFACE_VAP5          "ath0ap4"
#define INTERFACE_VAP6          "ath0ap5"
#define INTERFACE_VAP7          "ath0ap6"
#define INTERFACE_VAP8          "ath0ap7"
#define INTERFACE_WDS1          "ath0wds0"
#define INTERFACE_WDS2          "ath0wds1"
#define INTERFACE_WDS3          "ath0wds2"
#define INTERFACE_WDS4          "ath0wds3"
#define INTERFACE_STA           "ath0sta0"

#define INTERFACE_HW_RF1        "wifi0"

// get system items
char *get_wan_ifname(char *);
char *get_lan_ifname(char *);

int get_magic_str(char *, void *);
int get_uptime(char *, void *);
int get_gateway(char *, void *);
int get_wlan_ch(char *, void *);
int get_wlan_band(char *, void *);
int get_wlan_txpwr(char *, void *);
int get_wan_ip(char *, void *);
int get_wan_mask(char *, void *);
int get_mac(char *, void *);
int get_sta(char *, void *);
int get_wds_peer(char *, void *);
int get_wds_txrate(char *, void *);
int get_wds_idle(char *, void *);
int get_wds_snr(char *, void *);
int get_phy_cnt(char *, void *);
int get_ether_cnt(char *, void *);
int get_rf_cnt(char *, void *);
int getCurrentOptMode(char *, void *);
void a210_print_txpwr(char *, char *, int (*)(char *, void *));
void a210_print_txpwr_dbm(char *, char *, int (*)(char *, void *));

char *get_ifname_by_idx(IF_ROLE_T, int, int, char *);
int isWDSLinkAlive(char *);
int getWDSLinkTxRate(char *, char *);
int getWDSLinkQuality(char *, char *);
int getWDSLinkHealth(char *, char *);
int getWDSLinkTxTotal(char *, char *);
int getWDSLinkTxFail(char *, char *);
int getWDSLinkStatus(char *, char *);
int getWDSLinkSecurity(char *, char *);
int rfcards(char *, char *);
int getRepeaterIfStatus(char *, void *);
int getRepeaterMode(char*, void*);
int getRepeaterSecurity(char*, void*);

int getRfStaModeStatus(char *, void *);
int getRfStaModeSsid(char *, void *);
int getRfStaModeMac(char *, void *);
int getRfStaModeChannel(char *, void *);
int getRfStaModeRssi(char *, void *);
int getRepeaterIfStatus(char *, void *);
int getRepeaterMode(char*, void*);
int getRfStaModeTxRate(char*, void*);
int getRfStaModeTxCount(char*, void*);
int getRfStaModeTxErr(char*, void*);

const unsigned int max_rf;
const char *device_name;
const char *config_rom;
const char *config_ram;
const char *restored_cfg_file;
const char *backup_cfg_file;
const char *kernel_mtd;
const char *root_mtd;

const char *rf1_if;

const char *rf_name[];
const int gid_rf[];
const int gid_vap[MAX_RF][MAX_VAP];
const int gid_vap_sec[MAX_RF][MAX_VAP];
const int gid_vap_acl[MAX_RF][MAX_VAP];
const int gid_wds_link[MAX_RF][MAX_WDS_LINK];
const char *if_vap[]; 
const STRING_PAIR zone_info[];

const SYS_ITEM_LIST sys_item_list[];
const BRAND_ID_LIST brand_id_list[];
group_row_t group_table[];

#endif
