/*******************************************************************************************
 * Copyright (c) 2006-7 Laboratorio di Sistemi di Elaborazione e Bioingegneria Informatica *
 *                      Universita' Campus BioMedico - Italy                               *
 *                                                                                         *
 * This program is free software; you can redistribute it and/or modify it under the terms *
 * of the GNU General Public License as published by the Free Software Foundation; either  *
 * version 2 of the License, or (at your option) any later version.                        *
 *                                                                                         *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY         *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 	       *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.                *
 *                                                                                         *
 * You should have received a copy of the GNU General Public License along with this       *
 * program; if not, write to the:                                                          *
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    *
 * MA  02111-1307, USA.                                                                    *
 *                                                                                         *
 * --------------------------------------------------------------------------------------- *
 * Project:  Capwap                                                                        *
 *                                                                                         *
 * Author :  Ludovico Rossi (ludo@bluepixysw.com)                                          *  
 *           Del Moro Andrea (andrea_delmoro@libero.it)                                    *
 *           Giovannini Federica (giovannini.federica@gmail.com)                           *
 *           Massimo Vellucci (m.vellucci@unicampus.it)                                    *
 *           Mauro Bisson (mauro.bis@gmail.com)                                            *
 *******************************************************************************************/

 
#include "CWAC.h"
#include "ACProtocol_User.h"

#ifdef DMALLOC
#include "../dmalloc-5.5.0/dmalloc.h"
#endif

__inline__ CWBool CWACSupportIPv6() {
	return (gNetworkPreferredFamily == CW_IPv6);
}

__inline__ char * CWACGetName() {
	if (gACProtocolUser && gACProtocolUser->get_name)
		return gACProtocolUser->get_name();
	return gACName;
}

__inline__ int CWACGetStations() {
	if (gACProtocolUser && gACProtocolUser->get_stations)
		return gACProtocolUser->get_stations();
	return gActiveStations;
}

__inline__ int CWACGetLimit() {
	if (gACProtocolUser && gACProtocolUser->get_limit)
		return gACProtocolUser->get_limit();
	return gLimit;
}

__inline__ int CWACGetActiveWTPs() {
	int tmp;
	if (gACProtocolUser && gACProtocolUser->get_active_WTPs)
		return gACProtocolUser->get_active_WTPs();
	if(!CWErr(CWThreadMutexLock(&gActiveWTPsMutex))) return 0;
		tmp = gActiveWTPs;
	CWThreadMutexUnlock(&gActiveWTPsMutex);
	
	return tmp;
}

__inline__ int CWACGetMaxWTPs() {
	if (gACProtocolUser && gACProtocolUser->get_max_WTPs)
		return gACProtocolUser->get_max_WTPs();
	return gMaxWTPs;
}

__inline__ int CWACGetSecurity() {
	if (gACProtocolUser && gACProtocolUser->get_security)
		return gACProtocolUser->get_security();
	return gACDescriptorSecurity;
}

__inline__ int CWACGetRMACField() {
	if (gACProtocolUser && gACProtocolUser->get_RMAC_field)
		return gACProtocolUser->get_RMAC_field();
	return gRMACField;
}
#if 0
__inline__ int CWACGetWirelessField() {
	if (gACProtocolUser && gACProtocolUser->get_)
		return gACProtocolUser->get_;
	return gWirelessField;
}
#endif

__inline__ int CWACGetDTLSPolicy() {
	if (gACProtocolUser && gACProtocolUser->get_DTLS_policy)
		return gACProtocolUser->get_DTLS_policy();
	return gDTLSPolicy;
}

__inline__ int CWACGetHWVersion() {
	if (gACProtocolUser && gACProtocolUser->get_HW_version)
		return gACProtocolUser->get_HW_version();
	return gACHWVersion;
}

__inline__ int CWACGetSWVersion() {
	if (gACProtocolUser && gACProtocolUser->get_SW_version)
		return gACProtocolUser->get_SW_version();
	return gACSWVersion;
}

__inline__ int CWACGetInterfacesCount() {
	if (gACProtocolUser && gACProtocolUser->get_interface_count)
		return gACProtocolUser->get_interface_count();
	return gInterfacesCount;
}

__inline__ int CWACGetInterfaceIPv4AddressAtIndex(int i) {
	struct sockaddr_in *addrPtr;
	if (gACProtocolUser && gACProtocolUser->get_ipv4_address_at_index)
		return gACProtocolUser->get_ipv4_address_at_index(i);
	
	if(gNetworkPreferredFamily == CW_IPv4) {
		addrPtr = (struct sockaddr_in *) &(gInterfaces[i].addr);
	} else {
		addrPtr = (struct sockaddr_in *) &(gInterfaces[i].addrIPv4);
	}
	
	return ntohl(addrPtr->sin_addr.s_addr);
}

__inline__ char *CWACGetInterfaceIPv6AddressAtIndex(int i) {
	struct sockaddr_in6 *addrPtr;
	if (gACProtocolUser && gACProtocolUser->get_ipv6_address_at_index)
		return gACProtocolUser->get_ipv6_address_at_index(i);
	
	addrPtr = (struct sockaddr_in6 *) &(gInterfaces[i].addr);
	
	return (char*) addrPtr->sin6_addr.s6_addr;
}

__inline__ int CWACGetInterfaceWTPCountAtIndex(int i) {
	if (gACProtocolUser && gACProtocolUser->get_WTP_count_at_index)
		return gACProtocolUser->get_WTP_count_at_index(i);
	return gInterfaces[i].WTPCount;
}

CWBool CWACGetVendorInfos(CWACVendorInfos *valPtr) {
	if (gACProtocolUser && gACProtocolUser->get_vendor_infos)
		return gACProtocolUser->get_vendor_infos(valPtr);
	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	valPtr->vendorInfosCount = 2; 
	CW_CREATE_ARRAY_ERR((valPtr->vendorInfos), valPtr->vendorInfosCount, CWACVendorInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	(valPtr->vendorInfos)[0].vendorIdentifier = 65432;
	(valPtr->vendorInfos)[0].type = CW_AC_HARDWARE_VERSION;
	(valPtr->vendorInfos)[0].length = 4; // just one int
	CW_CREATE_OBJECT_SIZE_ERR((((valPtr->vendorInfos)[0]).valuePtr), 4, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[0]).valuePtr) = CWACGetHWVersion(); // HW version
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	((valPtr->vendorInfos)[1]).vendorIdentifier = 65432;
	((valPtr->vendorInfos)[1]).type = CW_AC_SOFTWARE_VERSION;
	((valPtr->vendorInfos)[1]).length = 4; // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[1] ).valuePtr), 4, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[1] ).valuePtr) = CWACGetSWVersion(); // SW version
	
	return CW_TRUE;
}

__inline__ void CWACDestroyVendorInfos(CWACVendorInfos *valPtr) {
	int i;
	if (gACProtocolUser && gACProtocolUser->get_destory_vendor_infos)
		return gACProtocolUser->get_destory_vendor_infos(valPtr);
	
	if(valPtr == NULL) return;
	
	for(i = 0; i < valPtr->vendorInfosCount; i++) {
		CW_FREE_OBJECT((valPtr->vendorInfos)[i].valuePtr);
	}
	
	CW_FREE_OBJECT(valPtr->vendorInfos);
}

CWBool CWACGetACIPv4List(int **listPtr, int *countPtr) 
{
	struct in_addr addr;
	
	if (gACProtocolUser && gACProtocolUser->get_ac_ipv4_list)
		return gACProtocolUser->get_ac_ipv4_list(listPtr,countPtr);
	// TO-DO this func should return the addresses of eventual other ACs in a cluster. Hey, what? What is the WTP
	// supposed to do with that?
	if(listPtr == NULL || countPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

	*countPtr = 2;
	
	CW_CREATE_ARRAY_ERR((*listPtr), (*countPtr), int, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	inet_pton(AF_INET, "192.168.1.2", &addr); // TO-DO take the addresses from config file?
	(*listPtr)[0] = ntohl(addr.s_addr);
	inet_pton(AF_INET, "192.168.1.66", &addr);
	(*listPtr)[1] = ntohl(addr.s_addr);
	
	return CW_TRUE;
}

CWBool CWACGetACIPv6List(struct in6_addr **listPtr, int *countPtr) 
{
	if (gACProtocolUser && gACProtocolUser->get_ac_ipv6_list)
		return gACProtocolUser->get_ac_ipv6_list(listPtr,countPtr);
	// TO-DO this func should return the addresses of eventual other ACs in a cluster. Hey, what? What is the WTP
	// supposed to do with that?
	if(listPtr == NULL || countPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

	*countPtr = 2;
	
	CW_CREATE_ARRAY_ERR(*listPtr, (*countPtr), struct in6_addr, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	inet_pton(AF_INET6, "5f1b:df00:ce3e:e200:0020:0800:2078:e3e3", &((*listPtr)[0])); // TO-DO take the addresses from config file?
	inet_pton(AF_INET6, "5f1b:df00:ce3e:e200:0020:0800:2078:e3e4", &((*listPtr)[1]));
	
	return CW_TRUE;
}

CWBool CWACGetDiscoveryTimer (int *timer)
{
	if (gACProtocolUser && gACProtocolUser->get_discovery_timer)
		return gACProtocolUser->get_discovery_timer(timer);
	*timer=gDiscoveryTimer;
	return CW_TRUE;
}

CWBool CWACGetEchoRequestTimer (int *timer)
{
	if (gACProtocolUser && gACProtocolUser->get_echo_request_timer)
		return gACProtocolUser->get_echo_request_timer(timer);
	*timer=gEchoRequestTimer;
	return CW_TRUE;
}

CWBool CWACGetIdleTimeout (int *timer)
{
	if (gACProtocolUser && gACProtocolUser->get_idle_timeout)
		return gACProtocolUser->get_idle_timeout(timer);
	*timer=gIdleTimeout;
	return CW_TRUE;
}

/* Il WTP ha la funzione ridefinita */
CWBool CWGetWTPRadiosAdminState(CWRadiosAdminInfo *valPtr) 
{
	int *WTPIndexPtr;	
	if (gACProtocolUser && gACProtocolUser->get_radio_admin_state)
		return gACProtocolUser->get_radio_admin_state(valPtr);

	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if((WTPIndexPtr = ((int*)CWThreadGetSpecific(&gIndexSpecific))) == NULL) {
		return CW_FALSE;
	}

	valPtr->radiosCount = gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radioCount;
	
	CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioAdminInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	int i;
	for(i=0; i<valPtr->radiosCount; i++)
	{
		(valPtr->radios)[i].ID = gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].radioID;
		(valPtr->radios)[i].state = gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].adminState;
		(valPtr->radios)[i].cause = gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].adminCause;
	}
	
	return CW_TRUE;
}

CWBool CWGetWTPRadiosOperationalState(int radioID, CWRadiosOperationalInfo *valPtr)
{
	int i;
	CWBool found = CW_FALSE;
	int *WTPIndexPtr;

	if (gACProtocolUser && gACProtocolUser->get_radio_operational_state)
		return gACProtocolUser->get_radio_operational_state(radioID,valPtr);
	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if((WTPIndexPtr = ((int*)CWThreadGetSpecific(&gIndexSpecific))) == NULL) {
		return CW_FALSE;
	}

	if(radioID<0) {
		valPtr->radiosCount = gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radioCount;
	
		CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioOperationalInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
		for (i=0; i<valPtr->radiosCount; i++)
		{
			(valPtr->radios)[i].ID =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].radioID;
			(valPtr->radios)[i].state =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].operationalState;
			(valPtr->radios)[i].cause =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].operationalCause;
		}
		return CW_TRUE;	
	}	
	else {
		for (i=0; i<valPtr->radiosCount; i++)
		{
			if(gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].radioID == radioID)
			{
				found = CW_TRUE;
				valPtr->radiosCount = 1;
				CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioOperationalInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
				(valPtr->radios)[i].ID =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].radioID;
				(valPtr->radios)[i].state =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].operationalState;
				(valPtr->radios)[i].cause =  gWTPs[*WTPIndexPtr].WTPProtocolManager.radiosInfo.radiosInfo[i].operationalCause;
				break;
			}
		}
		return found;
	}
}

#define APPEND_VSP(count, elems, msg_type, message, msg_size) {	        \
    CW4IPVSPSubElmValues *tmp;			    \
    tmp = realloc((elems), ((count)+1)*sizeof(CW4IPVSPSubElmValues));   \
    if (tmp) {				    \
	tmp[count].type = (msg_type);		        \
	tmp[count].length = (msg_size);		        \
	tmp[count].valuePtr = (int *)((message).msg);	        \
	(count)++;			    \
	(elems) = tmp;			        \
    }				        \
}
CWBool CWACCheckVSP(CW4IPVendorSpecificPayload *valPtr,CWWTPProtocolManager *WTPProtocolManager)
{
    VSP4IPConf vsp4ipConf, *conf;
    CWProtocolMessage msg;

    if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

    conf = &vsp4ipConf;
    CW_PREPARE_VSP4IPCONF_OBJECT(conf, goto out;);

    valPtr->vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    valPtr->subElmId = CW_VSP_SUBELM_CONF;
    valPtr->subElmCount = 0;
    valPtr->subElms = NULL;

    if (gACProtocolUser && gACProtocolUser->get_vsp_gre_conf) {
	if (gACProtocolUser->get_vsp_gre_conf(conf->greConf,WTPProtocolManager)) {
	    unsigned char val = (conf->greConf->greEnabled ? 1 : 0);
	    CW_CREATE_PROTOCOL_MESSAGE(msg, 9, goto out;);
	    CWProtocolStore32(&msg, conf->greConf->remoteIP);
	    CWProtocolStore32(&msg, conf->greConf->key);
	    CWProtocolStore8(&msg, val);
	    APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
		CW_VSP_SUBELM_CONF_GRE, msg, 9);
	}
    }

    if (gACProtocolUser && gACProtocolUser->get_vsp_acct_conf) {
	if (gACProtocolUser->get_vsp_acct_conf(conf->acctConf,WTPProtocolManager)) {
	    int size = 4 + strlen(conf->acctConf->username) + 
		strlen(conf->acctConf->password);
	    CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
	    CWProtocolStore8(&msg, strlen(conf->acctConf->username));
	    CWProtocolStore8(&msg, strlen(conf->acctConf->password));
	    CWProtocolStore16(&msg, 0); // reserved field
	    CWProtocolStoreStr(&msg, conf->acctConf->username);
	    CWProtocolStoreStr(&msg, conf->acctConf->password);
	    APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
		CW_VSP_SUBELM_CONF_ACCT, msg, size);
	}
    }

    if (gACProtocolUser && gACProtocolUser->get_vsp_snmp_conf) {
	if (gACProtocolUser->get_vsp_snmp_conf(conf->snmpConf,WTPProtocolManager)) {
	    unsigned char val = (conf->snmpConf->snmpEnabled ? 1 : 0);
	    int size = 8 + strlen(conf->snmpConf->readComm) + 
		strlen(conf->snmpConf->writeComm);
	    CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
	    CWProtocolStore8(&msg, val);
	    CWProtocolStore8(&msg, strlen(conf->snmpConf->readComm));
	    CWProtocolStore8(&msg, strlen(conf->snmpConf->writeComm));
	    CWProtocolStore8(&msg, 0); // reserved field
	    CWProtocolStore32(&msg, conf->snmpConf->trapIP);
	    CWProtocolStoreStr(&msg, conf->snmpConf->readComm);
	    CWProtocolStoreStr(&msg, conf->snmpConf->writeComm);
	    APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
		CW_VSP_SUBELM_CONF_SNMP, msg, size);
	}
    }

    if (gACProtocolUser && gACProtocolUser->get_vsp_vap_conf) {
	if (gACProtocolUser->get_vsp_vap_conf(conf->vapConf,WTPProtocolManager)) {
	    int i;
	    int size = (4 * conf->vapConf->vapCount);
	    CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
	    for (i = 0; i < conf->vapConf->vapCount; i++) {
		unsigned char val = 0;
		SET_CW_VSP_4IP_VAP_EN(val, (conf->vapConf->vaps)[i].vapEnabled);
		SET_CW_VSP_4IP_VAP_TUN_EN(val, (conf->vapConf->vaps)[i].vapTun);
		CWProtocolStore8(&msg, (conf->vapConf->vaps)[i].vapId);
		CWProtocolStore8(&msg, val);
		CWProtocolStore16(&msg, (conf->vapConf->vaps)[i].vapVlanId);
	    }
	    APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
		CW_VSP_SUBELM_CONF_VAP, msg, size);
	}
    }
out:
    CW_CLEAN_VSP4IPCONF_OBJECT(conf);
    return CW_TRUE;
}
#undef APPEND_VSP

__inline__ void CWACDestroyVSP(CW4IPVendorSpecificPayload *valPtr)
{
    int i;

    if (valPtr == NULL) return;

    for (i = 0; i < valPtr->subElmCount; i++) {
	CW_FREE_OBJECT((valPtr->subElms)[i].valuePtr);
    }

    CW_FREE_OBJECT(valPtr->subElms);
}

int CWOpenDataChannel(CWWTPProtocolManager *WTPProtocolManager)
{
    if (!(WTPProtocolManager->frameTunnelMode & CW_802_DOT_3_BRIDGING))
	return -1;
    if (gACProtocolUser && gACProtocolUser->open_data_channel)
	    return gACProtocolUser->open_data_channel(WTPProtocolManager);
    return 0;
}

int CWCloseDataChannel(CWWTPProtocolManager *WTPProtocolManager)
{
    if (!(WTPProtocolManager->frameTunnelMode & CW_802_DOT_3_BRIDGING))
	return -1;
    if (gACProtocolUser && gACProtocolUser->close_data_channel)
	    return gACProtocolUser->close_data_channel(WTPProtocolManager);
    return 0;
}
