/*******************************************************************************************
 * Copyright (c) 2006-7 Laboratorio di Sistemi di Elaborazione e Bioingegneria Informatica *
 *                      Universita' Campus BioMedico - Italy                               *
 *                                                                                         *
 * This program is free software; you can redistribute it and/or modify it under the terms *
 * of the GNU General Public License as published by the Free Software Foundation; either  *
 * version 2 of the License, or (at your option) any later version.                        *
 *                                                                                         *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY         *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 	       *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.                *
 *                                                                                         *
 * You should have received a copy of the GNU General Public License along with this       *
 * program; if not, write to the:                                                          *
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    *
 * MA  02111-1307, USA.                                                                    *
 *                                                                                         *
 * --------------------------------------------------------------------------------------- *
 * Project:  Capwap                                                                        *
 *                                                                                         *
 * Author :  Ludovico Rossi (ludo@bluepixysw.com)                                          *  
 *           Del Moro Andrea (andrea_delmoro@libero.it)                                    *
 *           Giovannini Federica (giovannini.federica@gmail.com)                           *
 *           Massimo Vellucci (m.vellucci@unicampus.it)                                    *
 *           Mauro Bisson (mauro.bis@gmail.com)                                            *
 *******************************************************************************************/


#include "CWWTP.h"
#include "WTPProtocol_User.h"

#ifdef DMALLOC
#include "../dmalloc-5.5.0/dmalloc.h"
#endif
 
__inline__ int CWWTPGetDiscoveryType() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_discovery_type)
		return gWTPProtocolUser->get_discovery_type();

	return CW_MSG_ELEMENT_DISCOVERY_TYPE_CONFIGURED;
}

__inline__ int CWWTPGetMaxRadios() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_max_radios)
		return gWTPProtocolUser->get_max_radios();

	return 1;
}

__inline__ int CWWTPGetRadiosInUse() 
{
	/*for (i=0; i<gRadiosInfo.radioCount; i++)
	{
		if((gRadiosInfo.radiosInfo[i].operationalState) == ENABLED)
			active++;
	}	
	return active;
	*/
	if (gWTPProtocolUser && gWTPProtocolUser->get_radio_in_use)
		return gWTPProtocolUser->get_radio_in_use();
	
	return gRadiosInfo.radioCount;
}

__inline__ int CWWTPGetEncCapabilities() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_enc_capabilities)
		return gWTPProtocolUser->get_enc_capabilities();

	return 12;
}

#define APPEND_VSP(count, elems, msg_type, message, msg_size) {		    \
	CW4IPVSPSubElmValues *tmp;					    \
	tmp = realloc((elems), ((count)+1)*sizeof(CW4IPVSPSubElmValues));   \
	if (tmp) {							    \
		tmp[count].type = (msg_type);				    \
		tmp[count].length = (msg_size);				    \
		tmp[count].valuePtr = (int *)((message).msg);		    \
		(count)++;						    \
		(elems) = tmp;						    \
	}								    \
}
CWBool CWWTPGetVSP(CW4IPVendorSpecificPayload *valPtr)
{
	VSP4IPConf vsp4ipConf, *conf;
	CWProtocolMessage msg;

	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

	conf = &vsp4ipConf;
	CW_PREPARE_VSP4IPCONF_OBJECT(conf, goto out;);

	valPtr->vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
	valPtr->subElmId = CW_VSP_SUBELM_CONF;
	valPtr->subElmCount = 0;
	valPtr->subElms = NULL;

	if (gWTPProtocolUser && gWTPProtocolUser->get_vsp_gre_conf) {
		if (gWTPProtocolUser->get_vsp_gre_conf(conf->greConf)) {
			unsigned char val = (conf->greConf->greEnabled ? 1 : 0);
			CW_CREATE_PROTOCOL_MESSAGE(msg, 9, goto out;);
			CWProtocolStore32(&msg, conf->greConf->remoteIP);
			CWProtocolStore32(&msg, conf->greConf->key);
			CWProtocolStore8(&msg, val);
			APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
				CW_VSP_SUBELM_CONF_GRE, msg, 9);
		}
	}

	if (gWTPProtocolUser && gWTPProtocolUser->get_vsp_acct_conf) {
		if (gWTPProtocolUser->get_vsp_acct_conf(conf->acctConf)) {
			int size = 4 + strlen(conf->acctConf->username) + 
				strlen(conf->acctConf->password);
			CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
			CWProtocolStore8(&msg, strlen(conf->acctConf->username));
			CWProtocolStore8(&msg, strlen(conf->acctConf->password));
			CWProtocolStore16(&msg, 0); // reserved field
			CWProtocolStoreStr(&msg, conf->acctConf->username);
			CWProtocolStoreStr(&msg, conf->acctConf->password);
			APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
				CW_VSP_SUBELM_CONF_ACCT, msg, size);
		}
	}

	if (gWTPProtocolUser && gWTPProtocolUser->get_vsp_snmp_conf) {
		if (gWTPProtocolUser->get_vsp_snmp_conf(conf->snmpConf)) {
			unsigned char val = (conf->snmpConf->snmpEnabled ? 1 : 0);
			int size = 8 + strlen(conf->snmpConf->readComm) + 
				strlen(conf->snmpConf->writeComm);
			CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
			CWProtocolStore8(&msg, val);
			CWProtocolStore8(&msg, strlen(conf->snmpConf->readComm));
			CWProtocolStore8(&msg, strlen(conf->snmpConf->writeComm));
			CWProtocolStore8(&msg, 0); // reserved field
			CWProtocolStore32(&msg, conf->snmpConf->trapIP);
			CWProtocolStoreStr(&msg, conf->snmpConf->readComm);
			CWProtocolStoreStr(&msg, conf->snmpConf->writeComm);
			APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
				CW_VSP_SUBELM_CONF_SNMP, msg, size);
		}
	}

	if (gWTPProtocolUser && gWTPProtocolUser->get_vsp_vap_conf) {
		if (gWTPProtocolUser->get_vsp_vap_conf(conf->vapConf)) {
			int i;
			int size = (4 * conf->vapConf->vapCount);
			CW_CREATE_PROTOCOL_MESSAGE(msg, size, goto out;);
			for (i = 0; i < conf->vapConf->vapCount; i++) {
				unsigned char val = 0;
				SET_CW_VSP_4IP_VAP_EN(val, (conf->vapConf->vaps)[i].vapEnabled);
				SET_CW_VSP_4IP_VAP_TUN_EN(val, (conf->vapConf->vaps)[i].vapTun);
				CWProtocolStore8(&msg, (conf->vapConf->vaps)[i].vapId);
				CWProtocolStore8(&msg, val);
				CWProtocolStore16(&msg, (conf->vapConf->vaps)[i].vapVlanId);
			}
			APPEND_VSP(valPtr->subElmCount, valPtr->subElms, 
				CW_VSP_SUBELM_CONF_VAP, msg, size);
		}
	}
out:
	CW_CLEAN_VSP4IPCONF_OBJECT(conf);
	return CW_TRUE;
}
#undef APPEND_VSP

__inline__ void CWWTPDestroyVSP(CW4IPVendorSpecificPayload *valPtr)
{
	int i;

	if (valPtr == NULL) return;

	for (i = 0; i < valPtr->subElmCount; i++) {
		CW_FREE_OBJECT((valPtr->subElms)[i].valuePtr);
	}

	CW_FREE_OBJECT(valPtr->subElms);
}

CWBool CWWTPGetBoardData(CWWTPVendorInfos *valPtr) {
	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

	if (gWTPProtocolUser && gWTPProtocolUser->get_board_data)
		return gWTPProtocolUser->get_board_data(valPtr);

	valPtr->vendorInfosCount = 2; // we fill 2 information (just the required ones)
	CW_CREATE_ARRAY_ERR((valPtr->vendorInfos), valPtr->vendorInfosCount, CWWTPVendorInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	(valPtr->vendorInfos)[0].vendorIdentifier = 23456;
	(valPtr->vendorInfos)[0].type = CW_WTP_MODEL_NUMBER;
	(valPtr->vendorInfos)[0].length = sizeof(long int); // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[0] ).valuePtr), (valPtr->vendorInfos)[0].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[0]).valuePtr) = 123456; // MODEL NUMBER

	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	(valPtr->vendorInfos)[1].vendorIdentifier = 23456;
	(valPtr->vendorInfos)[1].type = CW_WTP_SERIAL_NUMBER;
	(valPtr->vendorInfos)[1].length = sizeof(long int); // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[1] ).valuePtr), (valPtr->vendorInfos)[1].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[1]).valuePtr) = 123456; // SERIAL NUMBER
	
	return CW_TRUE;
}

CWBool CWWTPGetVendorInfos(CWWTPVendorInfos *valPtr) {
	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_vendor_infos)
		return gWTPProtocolUser->get_vendor_infos(valPtr);

	valPtr->vendorInfosCount = 3; // we fill 3 information (just the required ones)
	CW_CREATE_ARRAY_ERR((valPtr->vendorInfos), valPtr->vendorInfosCount, CWWTPVendorInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	(valPtr->vendorInfos)[0].vendorIdentifier = 23456;
	(valPtr->vendorInfos)[0].type = CW_WTP_HARDWARE_VERSION;
	(valPtr->vendorInfos)[0].length = sizeof(long int); // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[0] ).valuePtr), (valPtr->vendorInfos)[0].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[0]).valuePtr) = 123456; // HW version
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	((valPtr->vendorInfos)[1]).vendorIdentifier = 23456;
	((valPtr->vendorInfos)[1]).type = CW_WTP_SOFTWARE_VERSION;
	((valPtr->vendorInfos)[1]).length = sizeof(long int); // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[1] ).valuePtr), (valPtr->vendorInfos)[1].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[1]).valuePtr) = 12347; // SW version
	
	// my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
	(valPtr->vendorInfos)[2].vendorIdentifier = 23456;
	(valPtr->vendorInfos)[2].type = CW_BOOT_VERSION;
	(valPtr->vendorInfos)[2].length = sizeof(long int); // just one int
	CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[2] ).valuePtr), (valPtr->vendorInfos)[2].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	*(((valPtr->vendorInfos)[2]).valuePtr) = 1234568; // Boot version
	
	return CW_TRUE;
}

__inline__ void CWWTPDestroyVendorInfos(CWWTPVendorInfos *valPtr) {
	int i;
	
	if(valPtr == NULL) return;

	if (gWTPProtocolUser && gWTPProtocolUser->destroy_vendor_infos)
		return gWTPProtocolUser->destroy_vendor_infos(valPtr);
	
	for(i = 0; i < valPtr->vendorInfosCount; i++) {
		CW_FREE_OBJECT((valPtr->vendorInfos)[i].valuePtr);
	}
	
	CW_FREE_OBJECT(valPtr->vendorInfos);
}

__inline__ int CWWTPGetFrameTunnelMode() {
	//it may be also 802.3_FrameTunnelMode - NativeFrameTunnelMode - All
	if (gWTPProtocolUser && gWTPProtocolUser->get_frame_tunnel_mode)
		return gWTPProtocolUser->get_frame_tunnel_mode();

	return CW_LOCAL_BRIDGING;
}

__inline__ int CWWTPGetMACType() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_mac_type)
		return gWTPProtocolUser->get_mac_type();

	return CW_LOCAL_MAC;
}

__inline__ char *CWWTPGetLocation() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_location)
		return gWTPProtocolUser->get_location();

	return gWTPLocation;
}

__inline__ int  *CWWTPGetSessionID(unsigned int *session_id) {
	int i;
	if (gWTPProtocolUser && gWTPProtocolUser->get_session_id)
		return gWTPProtocolUser->get_session_id(session_id);
	for(i=0;i<4;i++)
	{
	    session_id[i] = CWRandomIntInRange(0, INT_MAX);
	}
	return session_id;
	//return CWRandomIntInRange(0, INT_MAX);
}

__inline__ int CWWTPGetIPv4Address() {
	struct sockaddr_in myAddr;
	unsigned int len = sizeof(myAddr);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_ipv4_address)
		return gWTPProtocolUser->get_ipv4_address();

	//CWDebugLog("WTPGetIPv4Address");

	/* assume the socket is connected */
	getsockname(gWTPSocket, (struct sockaddr*) &myAddr, &len);
	
	return ntohl(myAddr.sin_addr.s_addr); 	// TO-DO: this is garbage if we are an IPv6 client
}

__inline__ void CWWTPGetIPv6Address(struct sockaddr_in6* myAddr) {
	
	unsigned int len = sizeof(*myAddr);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_ipv6_address)
		gWTPProtocolUser->get_ipv6_address(myAddr);

	/* assume the socket is connected */
	getsockname(gWTPSocket, (struct sockaddr*) myAddr, &len);
}

__inline__ int CWWTPGetIPv4StatusDuplicate() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_ipv4_status_duplicate)
		return gWTPProtocolUser->get_ipv4_status_duplicate();

	return gIPv4StatusDuplicate;
}

__inline__ int CWWTPGetIPv6StatusDuplicate() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_ipv6_status_duplicate)
		return gWTPProtocolUser->get_ipv6_status_duplicate();

	return gIPv6StatusDuplicate;
}

__inline__ char *CWWTPGetName() {
	if (gWTPProtocolUser && gWTPProtocolUser->get_name)
		return gWTPProtocolUser->get_name();

	return gWTPName;
}

/*CWBool CWWTPGetRadiosInformation(CWRadiosInformation *valPtr) {
	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	valPtr->radiosCount = 2;
	
	CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioInformationValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	(valPtr->radios)[0].ID = 0; // first radio
	(valPtr->radios)[0].type = CW_802_DOT_11b;
	
	(valPtr->radios)[1].ID = 1; // second radio
	(valPtr->radios)[1].type = CW_802_DOT_11b;
	
	return CW_TRUE;
}
*/

/* L'AC ha la funzione ridefinita */
CWBool CWGetWTPRadiosAdminState(CWRadiosAdminInfo *valPtr) 
{
	int i;

	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_radio_admin_state)
		return gWTPProtocolUser->get_radio_admin_state(valPtr);

	valPtr->radiosCount = gRadiosInfo.radioCount;
	
	CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioAdminInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	for (i=0; i<gRadiosInfo.radioCount; i++)
	{
		(valPtr->radios)[i].ID = gRadiosInfo.radiosInfo[i].radioID; // first radio
		(valPtr->radios)[i].state = gRadiosInfo.radiosInfo[i].adminState;
		(valPtr->radios)[i].cause = gRadiosInfo.radiosInfo[i].adminCause;
	}
		
	return CW_TRUE;
}

CWBool CWGetWTPRadiosOperationalState(int radioID, CWRadiosOperationalInfo *valPtr)
{
	int i;
	CWBool found = CW_FALSE;

	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_radio_operational_state)
		return gWTPProtocolUser->get_radio_operational_state(radioID, valPtr);

	if(radioID<0) {
		
		valPtr->radiosCount = gRadiosInfo.radioCount;
		
		CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioOperationalInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
		for (i=0; i<gRadiosInfo.radioCount; i++)
		{	
			(valPtr->radios)[i].ID = gRadiosInfo.radiosInfo[i].radioID;
			(valPtr->radios)[i].state = gRadiosInfo.radiosInfo[i].operationalState;
			(valPtr->radios)[i].cause = gRadiosInfo.radiosInfo[i].operationalCause;
		}
		return CW_TRUE;	
	}	
	else {
		for (i=0; i<gRadiosInfo.radioCount; i++)
		{
			if(gRadiosInfo.radiosInfo[i].radioID == radioID)
			{
				found = CW_TRUE;
				valPtr->radiosCount = 1;
				CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWRadioOperationalInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
				(valPtr->radios)[0].ID =  gRadiosInfo.radiosInfo[i].radioID;
				(valPtr->radios)[0].state = gRadiosInfo.radiosInfo[i].operationalState;
				(valPtr->radios)[0].cause = gRadiosInfo.radiosInfo[i].operationalCause;
				break;
			}
		}
		return found;
	}
}

CWBool CWGetDecryptErrorReport(int radioID, CWDecryptErrorReportInfo *valPtr)
{
	int i;
	CWBool found = CW_FALSE;
	
	/*
	CWMACAddress add, add2;
	for(i=0; i<6; i++) add[i]=i;
	for(i=0; i<6; i++) add2[i]=99;
	CWListElement elem,elem2;
	elem.data = add;
	elem.next = &elem2;
	elem2.data = &add2;
	elem2.next = NULL; 
	gRadiosInfo.radiosInfo[0].decryptErrorMACAddressList = &elem;
	*/

	if(valPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_decrypt_error_report)
		return gWTPProtocolUser->get_decrypt_error_report(radioID, valPtr);

	valPtr->radios=NULL;
	
	if(radioID<0) {
		
		valPtr->radiosCount = gRadiosInfo.radioCount;
		
		CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWDecryptErrorReportValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
		
		for (i=0; i<gRadiosInfo.radioCount; i++)
		{	
			(valPtr->radios)[i].ID = gRadiosInfo.radiosInfo[i].radioID;
			(valPtr->radios)[i].numEntries = CWCountElementInList(gRadiosInfo.radiosInfo[i].decryptErrorMACAddressList);
			(valPtr->radios[i]).decryptErrorMACAddressList = NULL;
			CW_CREATE_ARRAY_ERR((valPtr->radios[i]).decryptErrorMACAddressList, (valPtr->radios[i]).numEntries, CWMACAddress, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
			int j;
			CWListElement *temp;
			temp = gRadiosInfo.radiosInfo[i].decryptErrorMACAddressList;
			for(j=0; j<(valPtr->radios[i]).numEntries; j++)
			{
				CW_COPY_MEMORY((valPtr->radios[i]).decryptErrorMACAddressList[j],temp->data, sizeof(CWMACAddress));
				temp = temp->next;
			}
		}
		return CW_TRUE;	
	}	
	else {
		for (i=0; i<gRadiosInfo.radioCount; i++)
		{
			if(gRadiosInfo.radiosInfo[i].radioID == radioID)
			{
				found = CW_TRUE;
				valPtr->radiosCount = 1;
				CW_CREATE_ARRAY_ERR(valPtr->radios, valPtr->radiosCount, CWDecryptErrorReportValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
				(valPtr->radios)[0].ID = gRadiosInfo.radiosInfo[i].radioID;
				(valPtr->radios)[0].numEntries = CWCountElementInList(gRadiosInfo.radiosInfo[i].decryptErrorMACAddressList);
				(valPtr->radios[0]).decryptErrorMACAddressList = NULL;
				CW_CREATE_ARRAY_ERR((valPtr->radios[0]).decryptErrorMACAddressList, (valPtr->radios[0]).numEntries, CWMACAddress, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
				int j;
				CWListElement *temp;
				temp = gRadiosInfo.radiosInfo[0].decryptErrorMACAddressList;
				for(j=0; j<(valPtr->radios[0]).numEntries; j++)
				{
					CW_COPY_MEMORY((valPtr->radios[0]).decryptErrorMACAddressList[j],temp->data, 6);	
					temp = temp->next;
				}
			}
		}
		return found;
	}
}

int CWWTPGetACIndex()
{
	if (gWTPProtocolUser && gWTPProtocolUser->get_ac_index)
		return gWTPProtocolUser->get_ac_index();

	return 1; //valore predefinito
}

char* CWWTPGetACName()
{
	if (gWTPProtocolUser && gWTPProtocolUser->get_ac_name)
		return gWTPProtocolUser->get_ac_name();

	return gACInfoPtr->name;
}

int CWWTPGetStatisticsTimer ()
{
	if (gWTPProtocolUser && gWTPProtocolUser->get_statistics_timer)
		return gWTPProtocolUser->get_statistics_timer();

	return gWTPStatisticsTimer;
}

int CWOpenDataChannel(CWACInfoValues *ACInfoPtr)
{
	int tunnel_mode = CWWTPGetFrameTunnelMode();

	if (!(tunnel_mode & CW_802_DOT_3_BRIDGING))
		return -1;
	if (gWTPProtocolUser && gWTPProtocolUser->open_data_channel)
		return gWTPProtocolUser->open_data_channel(ACInfoPtr);
	return 0;
}

int CWCloseDataChannel(CWACInfoValues *ACInfoPtr)
{
	int tunnel_mode = CWWTPGetFrameTunnelMode();

	if (!(tunnel_mode & CW_802_DOT_3_BRIDGING))
		return -1;
	if (gWTPProtocolUser && gWTPProtocolUser->close_data_channel)
		return gWTPProtocolUser->close_data_channel(ACInfoPtr);
	return 0;
}

CWBool CWWTPGetACNameWithIndex (CWACNamesWithIndex *ACsInfo)
{
	if(ACsInfo == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);
	
	if (gWTPProtocolUser && gWTPProtocolUser->get_ac_index)
		return gWTPProtocolUser->get_ac_name_with_index(ACsInfo);

	ACsInfo->count = 2;
	
	CW_CREATE_ARRAY_ERR(ACsInfo->ACNameIndex, ACsInfo->count, CWACNameWithIndexValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	(ACsInfo->ACNameIndex)[0].index = 0; // first radio
	CW_CREATE_STRING_FROM_STRING_ERR((ACsInfo->ACNameIndex)[0].ACName, "ACPrimary", return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	
	
	(ACsInfo->ACNameIndex)[1].index = 1; // first radio
	CW_CREATE_STRING_FROM_STRING_ERR((ACsInfo->ACNameIndex)[1].ACName, "ACSecondary", return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););;
	
	return CW_TRUE;
}
