#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/select.h>
#include <sys/time.h>
#include <ctype.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <arpa/inet.h>

#define DEF_UNIX_PATH	"/tmp/wtp_status"

#define OUTPUT_SIZE 8

const char *state_str[] = {
    "SULKING",
    "DISCOVERY",
    "JOIN",
    "CONFIGURE",
    "DATA_CHECK",
    "RUN",
    "RESET",
    "QUIT",
    "REBOOT",
};

#define N(arr)	(sizeof(arr)/sizeof(arr[0]))

const char * idx2state(int idx)
{
    if (idx < N(state_str))
	return state_str[idx];
    else
	return "N/A";
}

static void usage(void)
{
    fprintf(stderr, "Usage: check_wtp [-h] [-p sock]\n");
}

void check_wtp(const char *path)
{
    int sock, len, ret;
    unsigned char cmd_msg = 1;
    struct sockaddr_un cliaddr, srvaddr;
    char buffer[OUTPUT_SIZE];
    char tmpfile[BUFSIZ];
    fd_set rfds;
    struct timeval tv;

    sock = socket(AF_UNIX, SOCK_DGRAM, 0);
    /* client addr */
    bzero(&cliaddr, sizeof(cliaddr));
    cliaddr.sun_family = AF_UNIX;
    strcpy(cliaddr.sun_path, tmpnam_r(tmpfile));
    if (bind(sock, (const struct sockaddr *)&cliaddr, sizeof(cliaddr)) < 0) {
	fprintf(stderr, "can't bind\n");
	goto out;
    }
    /* server addr */
    bzero(&srvaddr, sizeof(srvaddr));
    srvaddr.sun_family = AF_UNIX;
    strcpy(srvaddr.sun_path, path);
    /* setup select parameters */
    FD_ZERO(&rfds);
    FD_SET(sock, &rfds);
    tv.tv_sec = 1;   // timeout = 1 sec
    tv.tv_usec = 0;

    if (sendto(sock, &cmd_msg, 1, 0, 
		(struct sockaddr *)&srvaddr, sizeof(srvaddr)) < 0) {
	fprintf(stderr, "Can't send msg.\n");
	goto out;
    }

    ret = select(sock + 1, &rfds, NULL, NULL, &tv);
    if (ret > 0 && FD_SET(sock, &rfds))
	len = recvfrom(sock, buffer, OUTPUT_SIZE, 0, NULL, NULL);
    else
	len = 0;

    if (len == OUTPUT_SIZE)
	printf("%s %s\n", idx2state(ntohl(*((int *)buffer))), 
		inet_ntoa(*((struct in_addr *)(buffer+4))));
out:
    unlink(tmpfile);
}

int main(int argc, char *argv[])
{
    int opt;
    char *unix_path = DEF_UNIX_PATH;

    while ((opt = getopt(argc, argv, "p:h")) != -1) {
	switch(opt) {
	    case 'p':
		unix_path = optarg;
		break;
	    case 'h':
	    default:
		usage();
		exit(EXIT_SUCCESS);
	}
    }

    if (optind > argc) {
	usage();
	exit(EXIT_FAILURE);
    }

    check_wtp(unix_path);

    exit(EXIT_SUCCESS);
}
