#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sqlite.h>
#include "CWAC.h"
#include "ACProtocol_User.h"

#define BUF_SIZE 1024
#define SYSTEM_NAME_DB_PATH "/db/system_name"
#define MAX_STATION_DB_PATH "/db/capwap/max_station"
#define AC_CLUSTER_IP_DB_PATH "/db/capwap/ac_cluster_ip"
#define SZ_MAP_DIR  "/db/capwap/map_tmpl"
#define SZ_MAP_ENABLE_PATH SZ_MAP_DIR "/map_tmpl_en"
#define WAN_NUM	2
#define CONF_MAX_DEV "max_devs="
#define VLAN_TMPL_BASE	1000

#define FILE_NAME_LEN 32
#define FILE_PATH_LEN 64
#define SQ_CMD_LEN 128
#define SQ_CMD_LONG_LEN 256
#define DIGIT_LEN 12
#define DB_TABLE_NUM 6
#define MAX_DB_FILE 10
#define ATTRS_LEN 64
#define MAC_LEN 18
#define IP_LEN 16
// total 20 seconds of try lock
#define LOCK_QUANTUM            500000L
#define MAX_RETRIES_LOCKDB      40

#define ERROR_ATTRS_FORMAT -2
#define ERROR_IP_FORMAT -3
#define ERROR_RENAME -4

#define CDBG_NONE       0
#define CDBG_ERROR      1
#define CDBG_WARN       2
#define CDBG_MDUMP      3
#define CDBG_TRACE      4

#define capwap_calloc(s...)   calloc(s)
#define capwap_free(s...)     free(s)

#define VLAN_ID_NUM 4096 
#define AC_CLUSTER_NUM 5

#define OD_EMCD_CONF_PATH "/conf/od_emcd/od_emcd.conf"
#define DB_DEV_PATH "db_dev_path="
#define DB_EMS_PATH "db_ems_path="
#define DB_META_PATH "db_meta_path="
#define DB_LOCK_DIR "db_lock_dir="

typedef struct db_info
{
   char *value;
   int num;
   struct db_info *p;
} db_info;

struct vapMapValue 
{
    int vm_rfidx;
    int vm_vapidx;
    int vm_enabled;
    int vm_vlanid;
    int vm_tun;
    int vm_szid;
};

typedef struct vapMap_template {
    int mt_count;
    struct vapMapValue *mt_value;
} VAPMAP_T;

int dbac_unlockdb(int fd);
int dbac_lockdb(char *db_name, int op);
static char ems_path[128],dev_path[128],config_lock_dir[128],meta_path[128];
char*** dbac_query(char *db_name, char *sql);
static int sql_callback(void *data_p, int argc, char **argv, char **azColName);
void dbac_free_db_info(db_info *data);
int dbac_result_free(char ***p);
void w6000_add_new_dev(CWWTPProtocolManager *WTPProtocolManager );
int set_vm_update_tunnel_status(int dev_id);
void vlan_mapping_update_check(int dev_id,CWWTPProtocolManager *WTPProtocolManager);
void w6000_job_restore(int dev_id);
int check_vm_update_attribute(int dev_id);
CWBool set_vm_update_attribute(int dev_id);

/* Converts an integer value to its hex character*/
char to_hex(char code) {
    static char hex[] = "0123456789abcdef";
    return hex[code & 15];
}
/* Returns a url-encoded version of str */
/* IMPORTANT: be sure to free() the returned string after use */
char *url_encode(char *str) {
    char *pstr = str, *buf = malloc(strlen(str) * 3 + 1), *pbuf = buf;
    while (*pstr) {
	if (isalnum(*pstr) || *pstr == '-' || *pstr == '_' || *pstr == '.' || *pstr == '~') 
	    *pbuf++ = *pstr;
	else if (*pstr == ' ') 
	    *pbuf++ = '+';
	else 
	    *pbuf++ = '%', *pbuf++ = to_hex(*pstr >> 4), *pbuf++ = to_hex(*pstr & 15);
	pstr++;
    }
    *pbuf = '\0';
    return buf;
}


int read_config_item(char *config, char* key, char *result, int len)
{
    char *start, *end;

    start = strstr(config, key);
    if(!start) return -1;
    start += strlen(key);
    end = strchr(start, '\n');
    if(!end) return -1;
    if( (end - start) >= len ) return -1;
    strncpy(result, start, end-start);
    result[end-start]='\0';
    return 0;
}

int dbac_report_usedb(int dev_id, char dbname[], char filepath[])
{
    sprintf(dbname, "dev%d", dev_id);
    strcat(filepath, "/");
    strcat(filepath, dbname);
    strcat(filepath, ".db");

    return 0;
}

int get_station_info(int dev_id,char *dev_path)
{
    char filepath[FILE_PATH_LEN], dbname[32+1];
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    char ***find;
    int lock_id=-1;
    int num=0;
    
    strcpy(filepath,dev_path);
    if(dbac_report_usedb(dev_id, dbname, filepath)!=0)
        return -1;
    sprintf(sq_cmd, "select count(*) from station_info;");
    lock_id=dbac_lockdb(dbname, LOCK_EX);
    if (lock_id==-1)
        return -1;
    find=dbac_query(filepath, sq_cmd);
    dbac_unlockdb(lock_id);
    if(find==NULL || find[0] == NULL){
        dbac_result_free(find);
        return -1;
    }
    
    if(atoi(find[0][0])>0){
	num=atoi(find[0][0]);
    }

    dbac_result_free(find);
    return num;
}

int get_device_type(char *model_name)
{
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    char ***find;
    int lock_id=-1;
    int dev_type=0;

    sprintf(sq_cmd, "select dev_type from device_meta where dev_type_name='%s';",model_name);
    lock_id=dbac_lockdb("meta", LOCK_EX);
    if (lock_id==-1)
        return -1;
    find=dbac_query(meta_path, sq_cmd);
    dbac_unlockdb(lock_id);
    if(find==NULL || find[0] == NULL){
        dbac_result_free(find);
        return -1;
    }

    if(atoi(find[0][0])>0){
        dev_type=atoi(find[0][0]);
    }

    dbac_result_free(find);
    return dev_type;
}

/*
                        added by discovery(even)|   added by CAPWAP(odd)|   remark
add new dev                     0                           1               add information into DB
restore configuration           2                           3               no need to add DB entry but do restore configure
we can know the device is added by discovery or CAPWAP by the value is even or odd
*/
int check_capwap_attribute(int dev_id)
{
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    char ***find;
    int lock_id=-1,result;

    sprintf(sq_cmd, "select capwap from device where dev_id='%d';",dev_id);
    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if (lock_id==-1)
        return CW_FALSE;
    find=dbac_query(ems_path, sq_cmd);
    dbac_unlockdb(lock_id);
    if(find==NULL || find[0] == NULL){
        dbac_result_free(find);
        return CW_FALSE;
    }

    result=atoi(find[0][0]);

    dbac_result_free(find);

    /* add by discovery,and not restore configuration by CAPWAP yet */
    if(result == 0)
	return CW_TRUE;
    else
	return CW_FALSE;
}

int check_vm_update_attribute(int dev_id)
{
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    char ***find;
    int lock_id=-1,result;

    sprintf(sq_cmd, "select vm_update from device_state where dev_id='%d';",dev_id);
    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if (lock_id==-1)
        return CW_FALSE;
    find=dbac_query(ems_path, sq_cmd);
    dbac_unlockdb(lock_id);
    if(find==NULL || find[0] == NULL){
        dbac_result_free(find);
        return CW_FALSE;
    }

    result=atoi(find[0][0]);

    dbac_result_free(find);

    if(result == 0)
	return CW_TRUE;
    else
	return CW_FALSE;
}

CWBool set_vm_update_attribute(int dev_id)
{
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    int lock_id=-1;

    sprintf(sq_cmd, "UPDATE device_state set vm_update=1 where dev_id='%d';",dev_id);
    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if (lock_id==-1)
        return CW_FALSE;
    dbac_query(ems_path, sq_cmd);
    dbac_unlockdb(lock_id);

    return CW_TRUE;
}
/**
*  it is invoked once for each row in the result of a dbac_query by sqlite_exec() 
* @param  data_p copy of the fourth argument to sqlite_exec()
* @param  argc  number of columns in the dbac_query result
* @param  argv  array of pointers to strings where each string is a single column of the result for that record
* @param  azColName  names of the result columns
* @return  0 on success , non-zero on error
*/
static int sql_callback(void *data_p, int argc, char **argv, char **azColName)
{
    int i;

    db_info *data;
    db_info *temp;
 
    if(data_p==NULL)
        return(0);
    data=data_p;
    
    ((db_info *)data_p)->num++;      // number of rows
    
    while(data->p!=NULL)
        data=data->p;
    
    for(i=0; i<argc; i++){
        temp=capwap_calloc(sizeof(db_info),1);

        if(i==0)
            temp->num=argc;     // number of columns

        if(argv[i]==NULL){
            temp->value=capwap_calloc(strlen("NULL")+1, 1);
            strncpy(temp->value, "NULL", strlen("NULL"));
            
        }
        else{
            temp->value=capwap_calloc(strlen(argv[i])+1, 1);   
            strncpy(temp->value, argv[i], strlen(argv[i]));
        }
        data->p=temp;      //link list
        data=temp;
    }
   
    return(0);
}

void dbac_free_db_info(db_info *data)
{
    if (data) {
        if (data->p) {
            dbac_free_db_info(data->p);
        }
        capwap_free(data->value);
        capwap_free(data);
    }
}

/**
*  release  char*** pointer
* @param char*** p pointer
* @return  0
*/
int dbac_result_free(char ***p)
{
    int i,j;

    if(p!=NULL){
        for(i=0 ; p[i]!=NULL ; i++){
            for(j=0 ; p[i][j]!=NULL ; j++){
                capwap_free(p[i][j]);
            }
            capwap_free(p[i]);
        }
        capwap_free(p);
    }

    return(0);
}

/** 
*  get value from database
* requires: sql_callback
* @param db_name  file path & file name
* @param sql  SQlite command
* @return  pointer on success, NULL on error
*/

char*** dbac_query(char *db_name, char *sql)
{
    sqlite *db=NULL;
    char *zErrMsg = 0;
    int rc=-1, col_num, value_num=0, i;
    char ***db_p=NULL;
    char **Col_p=NULL;
    db_info *data=NULL,*p=NULL;
    
    if(db_name==NULL)
        return(NULL);
  
    for(i=0 ; i<5 ; i++){
        db=sqlite_open(db_name,0,&zErrMsg);
        if(db!=NULL){
            break;
        }
        else{
            sleep(1);
        }    
    }

    if(db==NULL){
        CWLog("Can't open database: '%s'\n", zErrMsg);
        capwap_free(zErrMsg);
        return(NULL);
    }
 
    data=(db_info *)capwap_calloc(1, sizeof(db_info));
    if (data==NULL) {	// memory allocation error
    	sqlite_close(db);
        return(NULL);
    }

    for(i=0 ; i<5 ; i++){
        rc=sqlite_exec(db, sql, sql_callback, data, &zErrMsg);
       
        if(rc==SQLITE_OK)
            break;
        else{
            CWLog("SQL error: %s, time:%d\n", zErrMsg, i+1);    
            
            sleep(1);  
            sqlite_close(db);
            dbac_free_db_info(data);
            data=(db_info *)capwap_calloc(1, sizeof(db_info));
            db=sqlite_open(db_name,0,&zErrMsg);
            if(db==NULL){
                printf("Can't open database: '%s'\n", zErrMsg);
                capwap_free(zErrMsg);
                return(NULL);
            }
        }
          
    }
    if(rc!=SQLITE_OK){
        CWLog("SQL error: %s\n", zErrMsg);
        capwap_free(data);
        sqlite_close(db);
        return(NULL);
    }
    
    sqlite_close(db);
    
    p=data;
    
    db_p=capwap_calloc(p->num+1, sizeof(char *));
    if (db_p==NULL) {
        capwap_free(data);
        return(NULL);
    }
    db_p[p->num]=NULL;
    
    p=p->p;
    
    col_num=0;
        
    while(p!=NULL){
        if(p->num>0){             //  number of columns   (row)
            Col_p=capwap_calloc(sizeof(char *), p->num+1);
	    if (Col_p==NULL) {
	        capwap_free(data);
        	return(NULL);
    	    }
            Col_p[p->num]=NULL;
            db_p[col_num]=Col_p;
            col_num++;
            value_num=0;
        }
        Col_p[value_num]=p->value;    // value
        value_num++;
       
        p=p->p;   
    }
    
    p=data->p;
                       // free sql_callback()'s  link list
    while(p!=NULL){
        capwap_free(data);
        data=p;
        p=p->p;   
    }
    if(data!=NULL)
        capwap_free(data);

    return(db_p);
}
/**
* lock database file
* @param db_name database name
* @param op operation
* @return >0 on success, -1 on error
*/
int dbac_lockdb(char *db_name, int op)
{
    int fd, ru, i;
    char lock_path[FILE_PATH_LEN];
    char *sub_filename=".lock";
    unsigned int intval, stime = 0;

    snprintf(lock_path, FILE_PATH_LEN, "%s/%s%s", config_lock_dir, db_name,
             sub_filename);

    if ((fd = open(lock_path, O_CREAT | O_RDONLY, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0) {
        return(-1);
    }

    intval = LOCK_QUANTUM * (0.1 + 0.9*rand()/RAND_MAX);
    for (i=1; i<=MAX_RETRIES_LOCKDB; i++) {
        if ((ru=flock(fd, op|LOCK_NB))!=0) {
        stime += intval;
            usleep(stime);
        }
        else {  // success
            break;
        }
    }

    if(ru==0) {
        return(fd);
    }
    else {
        close(fd);
        return(-1);
    }
}


/**
*  unlock database file
* @param fd  file descriptors
* @return  0 on success, -1 on error
*/
int dbac_unlockdb(int fd)
{
    int rc;
    struct timeval now;

    gettimeofday(&now, NULL);

    rc=flock(fd, LOCK_UN);
    close(fd);
    if (rc) {
        CWLog("dbac_unlockdb");
    }

    return(rc);
}

/**
*  get all devices id
*requires: dbac_lockdb, dbac_unlockdb, dbac_usedb, dbac_query, dbac_result_free
*@param  n  number of device
*@return device id on success, NULL on error
*/
int* dbac_get_devid(int *n ,char *filepath)
{
    char sq_cmd[SQ_CMD_LEN]="select distinct dev_id from device";
    char ***find;
    int lock_id, *devid = NULL;
    int counter;

    *n=-1;

    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if(lock_id==-1)
        return(NULL);

    find=dbac_query(filepath,sq_cmd);

    dbac_unlockdb(lock_id);

    counter = 0;
    if(find!=NULL) {
        for(counter=0 ; find[counter]!=NULL ; counter++);
        if (counter > 0 ) {
            devid=capwap_calloc(sizeof(int), counter);
            for(counter=0 ; find[counter]!=NULL ; counter++){
               devid[counter]=atoi(find[counter][0]);
            }
        }
        dbac_result_free(find);
    }
    else{
        return(NULL);
    }
    *n=counter;

    return(devid);
}

int getStationNum() {
    int num, station_num=0;
    int total, *devs, i;

    devs = dbac_get_devid(&total,ems_path);

    for(i=0;i<total;i++){
	num = get_station_info(devs[i],dev_path);
	if (num > 0)
	    station_num += num;
    }
    if(devs) capwap_free(devs);
    return station_num;
}

/** 
 * @brief Get value form DB
 * 
 * @param path relative path or absolute path (start with "/")
 * @param value buffer to store value (the buffer size should larger enough)
 * @param size buffer size
 * 
 * @return NULL for failure, otherwise return pointer of value
 */
static char *w6000_get_value_from_db(char *path, char *value, size_t size)
{
    FILE *fp = NULL;
    char realpath[BUFSIZ];
    int len;

    if (path[0] == '/')
	sprintf(realpath, "%s", path);
    else
	sprintf(realpath, "/db/%s", path);

    if ((fp = fopen(realpath, "r")) == NULL) {
	CWLog("Can't open db file %s", path);
	return NULL;
    }
    memset(value, 0, size);
    if (!fgets(value, size-1, fp)) {
	fclose(fp);
	return NULL;
    }
    fclose(fp);

    len = strlen(value);
    if (value[len-1] == '\n')
	value[len-1] = '\0';

    return value;
}

static char *w6000_get_name(void)
{
    /* XXX: We use static variable here, NOT thread safe!!! */
    static char value[BUFSIZ];
    
    if (w6000_get_value_from_db(SYSTEM_NAME_DB_PATH, value, BUFSIZ) == NULL)
	return NULL;
    return value;
}

static int w6000_get_stations(void)
{
    return getStationNum();
}

static int w6000_get_max_station(void)
{
    char value[BUFSIZ];
    
    if (w6000_get_value_from_db(MAX_STATION_DB_PATH, value, BUFSIZ) == NULL)
	return -1;
    return atoi(value);
}

int w6000_get_wan_ip_by_idx(int index, char *ip_addr)
{
    char wan_ip_path[BUFSIZ];
    
    if (index == 0)
	sprintf(wan_ip_path, "/db/wan/ip/");
    else
	sprintf(wan_ip_path, "/db/wan%d/ip", index+1);

    if (w6000_get_value_from_db(wan_ip_path, ip_addr, IP_LEN) == NULL)
	return -1;
    return 0;
}

int w6000_get_wan_ifname_by_idx(int index, char *ifname)
{
    char wan_if_path[BUFSIZ];
    
    if (index == 0)
	sprintf(wan_if_path, "/db/wan/if");
    else
	sprintf(wan_if_path, "/db/wan%d/if", index+1);

    if (w6000_get_value_from_db(wan_if_path, ifname, IFNAMSIZ) == NULL)
	return -1;
    return 0;
}

static int w6000_get_max_WTPs(void)
{
    char dev_max[128];
    char *config;
    int fsize;
    FILE *fp;

    /* Read firmware header */
    fp = fopen(OD_EMCD_CONF_PATH, "r");
    if(fp == NULL) {
	return -1;
    }

    fseek(fp, 0, SEEK_END);
    fsize = ftell(fp);
    config = malloc((fsize + 1) * sizeof(char));
    rewind(fp);
    fread(config, sizeof(char), fsize, fp);
    config[fsize]='\0';
    fclose(fp);

    if(read_config_item(config, CONF_MAX_DEV, dev_max, 128)<0) {
	capwap_free(config);
	return -1;
    }
    capwap_free(config);
    return atoi(dev_max);
}

static int get_devid_by_pair_ip_mac(char *ip,char *mac)
{
    char sq_cmd[SQ_CMD_LEN]= "";
    char ***find;
    int lock_id, result;

    sprintf(sq_cmd,"select dev_id from device where ip='%s' and mac='%s';",ip,mac);
    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if(lock_id==-1)
        return -1;

    find=dbac_query(ems_path,sq_cmd);

    dbac_unlockdb(lock_id);

    if(find==NULL || find[0]==NULL){
         dbac_result_free(find);
	 /* Add a new entry in DB */
	 return 0;
    }

    result=atoi(find[0][0]);

    dbac_result_free(find);
    return result;
}

static char *get_ip_of_wtp(CWWTPProtocolManager *WTPProtocolManager,
	char *ip, int len)
{
    CWWTPManager 	*WTPManagerPtr = NULL;
    struct sockaddr_in 	*sin;

    WTPManagerPtr = 
	container_of(WTPProtocolManager, CWWTPManager, WTPProtocolManager);
    sin = (struct sockaddr_in *)(&WTPManagerPtr->address);
    memset(ip, 0, len);
    inet_ntop(AF_INET, &sin->sin_addr, ip, len);
    return ip;
}

static char *get_mac_of_wtp(CWWTPProtocolManager *WTPProtocolManager,
	char *mac_str)
{
    unsigned char		mac[6];
    int				i;
    memset(mac,0,6);
    memset(mac_str,0,MAC_LEN);
    for(i = 0; i < (WTPProtocolManager->WTPBoardData).vendorInfosCount; i++) {
	if(((WTPProtocolManager->WTPBoardData).vendorInfos)[i].type == CW_BASE_MAC_ADDRESS)
		CW_COPY_MEMORY(mac,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].valuePtr,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].length);
    }
    sprintf(mac_str, "%02X:%02X:%02X:%02X:%02X:%02X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);

    return mac_str;
}

static int get_devid_by_wtp(CWWTPProtocolManager *WTPProtocolManager)
{
    char 	wtp_ip[20];
    char	mac_str[MAC_LEN];
    get_ip_of_wtp(WTPProtocolManager, wtp_ip, 20);
    get_mac_of_wtp(WTPProtocolManager, mac_str);
    return get_devid_by_pair_ip_mac(wtp_ip,mac_str);
}

/*
    check ip/mac in DB , if not exist , add a new entry and add a restore job
    if exist, restore configuration when capwap attribute is 0
*/
static int w6000_add_wtp(CWWTPProtocolManager *WTPProtocolManager)
{
    int devid = get_devid_by_wtp(WTPProtocolManager);

    if (devid == 0) {
	w6000_add_new_dev(WTPProtocolManager);
	devid = get_devid_by_wtp(WTPProtocolManager);
    }
    
    if (devid <= 0)
	return -1;

    /* check the capwap attribute in ems.db 
       if capwap = 0 , need to add a restore job */
    if(check_capwap_attribute(devid) == CW_TRUE)
	w6000_job_restore(devid);

    /* check if need to update vlan mapping */
    vlan_mapping_update_check(devid, WTPProtocolManager);

    if(WTPProtocolManager->frameTunnelMode & CW_802_DOT_3_BRIDGING)
	set_vm_update_tunnel_status(devid);
    return 1;
}

int set_vm_update_tunnel_status(int dev_id)
{
    char sq_cmd[SQ_CMD_LONG_LEN] = "";
    int lock_id=-1;

    sprintf(sq_cmd, "UPDATE device_tunnel set status=2 where dev_id='%d';",dev_id);
    lock_id=dbac_lockdb("od_ems", LOCK_EX);
    if (lock_id==-1)
	return CW_FALSE;
    dbac_query(ems_path, sq_cmd);
    dbac_unlockdb(lock_id);
	return CW_TRUE;
} 

int ac_lib_init(void)
{
    char *config;
    int fsize;
    FILE *fp;
    
    /* get env variables form od_emcd.conf */
    fp = fopen(OD_EMCD_CONF_PATH, "r");
    if(fp == NULL) {
	return -1;
    }

    fseek(fp, 0, SEEK_END);
    fsize = ftell(fp);
    config = malloc((fsize + 1) * sizeof(char));
    rewind(fp);
    fread(config, sizeof(char), fsize, fp);
    config[fsize]='\0';
    fclose(fp);

    if(read_config_item(config, DB_EMS_PATH, ems_path, 128)<0) {
	capwap_free(config);
	return -1;
    }
    if(read_config_item(config, DB_DEV_PATH, dev_path, 128)<0) {
	capwap_free(config);
	return -1;
    }
    if(read_config_item(config, DB_META_PATH, meta_path, 128)<0) {
	capwap_free(config);
	return -1;
    }
    if(read_config_item(config, DB_LOCK_DIR, config_lock_dir, 128)<0) {
	capwap_free(config);
	return -1;
    }
    capwap_free(config);
    return 0;
}

void w6000_job_restore(int dev_id)
{
    FILE *pp;
    char str[4096];
    sprintf(str, "/bin/curl -v -k -s 'http://127.0.0.1/OutdoorEquipmentManagement/CAPWAP_CMD.shtml?cmd=restore_config&dev_id=%d'>/dev/null 2>&1",dev_id);
    if((pp = popen(str, "r"))) 
	pclose(pp);
    else {
	CWLog("Add a restore job fail");
	exit(1);
    }
}

static CWBool w6000_get_vlan_sz_map(VSPVapConf *vapConf, VAPMAP_T *mapPtr)
{
    int enabled_tmpl = 0;
    int i;
    char buf[BUFSIZ];

    mapPtr->mt_count = 0;
    mapPtr->mt_value = NULL;

    if (vapConf == NULL || vapConf->vapCount == 0)
	return CW_FALSE;

    if (w6000_get_value_from_db(SZ_MAP_ENABLE_PATH, buf, BUFSIZ) == NULL)
	return CW_FALSE;
    enabled_tmpl = (strcmp(buf, "Enabled") ? 0 : 1);
    
    CW_CREATE_ARRAY_ERR((mapPtr->mt_value), vapConf->vapCount, struct vapMapValue, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    mapPtr->mt_count = vapConf->vapCount;

    if (enabled_tmpl) {
	/* 
	 * Folder schema:
	 * <DB_PATH>
	 * `-- capwap 
	 *     `-- map_tmpl 
	 *         `-- <rfidx> 
	 *             `-- <vapidx>
	 *                 |--- vap_en
	 *                 `--- vap_szid
	 */
	for (i = 0; i < mapPtr->mt_count; i++) {
	    char tmpl_path[BUFSIZ];
	    int szid;

	    /* Read configuration database */
	    mapPtr->mt_value[i].vm_rfidx = GET_CW_VSP_4IP_RF_IDX(vapConf->vaps[i].vapId);
	    mapPtr->mt_value[i].vm_vapidx = GET_CW_VSP_4IP_VAP_IDX(vapConf->vaps[i].vapId);
	    /* Get vm_enabled */
	    sprintf(tmpl_path, "capwap/map_tmpl/%d/%d/vap_en", 
		    mapPtr->mt_value[i].vm_rfidx, mapPtr->mt_value[i].vm_vapidx);
	    if (w6000_get_value_from_db(tmpl_path, buf, BUFSIZ))
		mapPtr->mt_value[i].vm_enabled = (strcmp(buf, "Enabled") ? 0 : 1);
	    else
		mapPtr->mt_value[i].vm_enabled = 0;
	    /* Get vm_tun */
	    sprintf(tmpl_path, "capwap/map_tmpl/%d/%d/vap_gre", 
		    mapPtr->mt_value[i].vm_rfidx, mapPtr->mt_value[i].vm_vapidx);
	    if (w6000_get_value_from_db(tmpl_path, buf, BUFSIZ))
		mapPtr->mt_value[i].vm_tun = (strcmp(buf, "Enabled") ? 0 : 1);
	    else
		mapPtr->mt_value[i].vm_tun = 0;
	    /* Get vm_szid : We only override the VLAN ID when GRE is enabled */
	    sprintf(tmpl_path, "capwap/map_tmpl/%d/%d/vap_szid", 
		    mapPtr->mt_value[i].vm_rfidx, mapPtr->mt_value[i].vm_vapidx);
	    if (w6000_get_value_from_db(tmpl_path, buf, BUFSIZ))
		szid = atoi(buf);
	    else
		szid = 0;
	    mapPtr->mt_value[i].vm_vlanid = (mapPtr->mt_value[i].vm_tun ? (szid + VLAN_TMPL_BASE) : vapConf->vaps[i].vapVlanId);
	    mapPtr->mt_value[i].vm_szid = szid;
	}
    } else {
	for (i = 0; i < mapPtr->mt_count; i++) {
	    mapPtr->mt_value[i].vm_rfidx = GET_CW_VSP_4IP_RF_IDX(vapConf->vaps[i].vapId);
	    mapPtr->mt_value[i].vm_vapidx = GET_CW_VSP_4IP_VAP_IDX(vapConf->vaps[i].vapId);
	    mapPtr->mt_value[i].vm_enabled = vapConf->vaps[i].vapEnabled;
	    mapPtr->mt_value[i].vm_vlanid = vapConf->vaps[i].vapVlanId;
	    mapPtr->mt_value[i].vm_tun = vapConf->vaps[i].vapTun;
	    mapPtr->mt_value[i].vm_szid = 0;
	}
    }
    return CW_TRUE;
}

static void w6000_free_vlan_sz_map(VAPMAP_T *mapPtr)
{
    CW_FREE_OBJECT(mapPtr->mt_value);
    mapPtr->mt_count = 0;
}

static int get_vid_list(VSPVapConf *pvapConf, char *vid_list, int list_len, 
	int list_type)
{
    int vap_check[CW_VSP_4IP_MAX_RF][VLAN_ID_NUM];
    VAPMAP_T map_tmpl;
    int i, pre_rf_idx, rf_idx, vid;
    char str[BUFSIZ];

    map_tmpl.mt_count = 0;
    for( i = 0; i < CW_VSP_4IP_MAX_RF * VLAN_ID_NUM; i++)
	*((int *)vap_check + i) = 0;

    /* vid_list format #0 : rf1-rf2-rf3, rfX=vlan:sz,vlan:sz.... */
    /* vid_list format #1 : rf1-rf2-rf3, rfX=sz:vlan#sz:vlan.... */
    memset(vid_list, 0, list_len);
    if(w6000_get_vlan_sz_map(pvapConf, &map_tmpl)) {
	for (i = 0; i < map_tmpl.mt_count; i++) {
	    struct vapMapValue *map = &(map_tmpl.mt_value[i]);
	    if (i == 0)
		pre_rf_idx = map->vm_rfidx;
	    else
		pre_rf_idx = map_tmpl.mt_value[i-1].vm_rfidx;
	    rf_idx = map->vm_rfidx;
	    vid = map->vm_vlanid;
	    if (map->vm_enabled && map->vm_tun) {
		if (pre_rf_idx != rf_idx) {
		    vid_list[strlen(vid_list)-1]='\0';
		    strcat(vid_list,"-");
		}
		if(vap_check[rf_idx][vid] == 0) {
		    if (list_type == 0)
			sprintf(str,"%d:%d,", 
				(vid == 4096 ? 0 : vid), map->vm_szid);
		    else 
			sprintf(str,"%d:%d#", 
				map->vm_szid, (vid == 4096 ? 0 : vid));
		    strcat(vid_list,str);
		    vap_check[rf_idx][vid]=1;
		}
	    }
	}
	vid_list[strlen(vid_list)-1]='\0';
	w6000_free_vlan_sz_map(&map_tmpl);
	return 1;
    } else {
	return 0;
    }
}

void vlan_mapping_update_check(int dev_id,CWWTPProtocolManager *WTPProtocolManager) 
{
    VSPVapConf *pvapConf;
    char vid_list[2048];
    FILE *pp;
    char str[4096];
    VSPGreConf *pGreConf;
    pGreConf = WTPProtocolManager->vsp4ipConf.greConf;

    pvapConf = WTPProtocolManager->vsp4ipConf.vapConf;

    if(check_vm_update_attribute(dev_id)) {
	/* set vm_update attribute to 1 */
	set_vm_update_attribute(dev_id);
	if (get_vid_list(pvapConf, vid_list, sizeof(vid_list), 0)) {
	    sprintf(str, "/bin/curl -v -k -s 'http://127.0.0.1/OutdoorEquipmentManagement/CAPWAP_CMD.shtml?cmd=update_vlan_mapping&dev_id=%d&vid_list=%s&gre_state=%d'>/dev/null 2>&1",dev_id, vid_list,(WTPProtocolManager->frameTunnelMode & CW_802_DOT_3_BRIDGING ? 2 : (pGreConf ? pGreConf->greEnabled : 0)));
    
	    if((pp = popen(str, "r"))) 
		pclose(pp);
	    else {
		CWLog("Update VLAN mapping fail");
		exit(1);
	    }
	}
    }
    
    return;
}

void w6000_add_new_dev(CWWTPProtocolManager *WTPProtocolManager )
{
    char wtp_ip[20];
    char str[4096];
    char vid_list[2048];
    char *dev_name=NULL;
    FILE *pp;
    int i,device_type;
    unsigned char mac[6];
    unsigned char ver[10];
    char model_name[FILE_NAME_LEN];
    char mac_str[MAC_LEN];
    VSPGreConf *pGreConf;
    VSPAcctConf *pacctConf;
    VSPSnmpConf *psnmpConf;
    VSPVapConf *pvapConf;
    CWWTPManager 	*WTPManagerPtr = NULL;
    WTPManagerPtr = 
	container_of(WTPProtocolManager, CWWTPManager, WTPProtocolManager);

    /* Get device GRE setting */
    /* Only enabled GRE when pGreConf->greEnabled = 1 */
    pGreConf = WTPProtocolManager->vsp4ipConf.greConf;
    /* Get device account setting */
    pacctConf = WTPProtocolManager->vsp4ipConf.acctConf;
    /* Get device snmp setting */
    psnmpConf = WTPProtocolManager->vsp4ipConf.snmpConf;
    /* Get device vap setting */
    pvapConf = WTPProtocolManager->vsp4ipConf.vapConf;

    if (pvapConf) {
	get_vid_list(pvapConf, vid_list, sizeof(vid_list), 0);
    }

    /* Get device ip */
    memset(wtp_ip,0,20);
    inet_ntop(AF_INET, &((struct sockaddr_in *)((struct sockaddr*)&(WTPManagerPtr->address)))->sin_addr ,wtp_ip,20);

    /* Get device name and do url_encode */
    dev_name=url_encode(WTPProtocolManager->name);

    /* Get device SW version */
    memset(ver,0,8);
    for(i = 0; i < WTPProtocolManager->descriptor.vendorInfos.vendorInfosCount; i++) {
	switch ((WTPProtocolManager->descriptor.vendorInfos.vendorInfos)[i].type ) {
	    case CW_WTP_SOFTWARE_VERSION:
		CW_COPY_MEMORY(ver,(WTPProtocolManager->descriptor.vendorInfos.vendorInfos)[i].valuePtr,(WTPProtocolManager->descriptor.vendorInfos.vendorInfos)[i].length);
		break;
	    case CW_WTP_HARDWARE_VERSION:
	    case CW_BOOT_VERSION:
	    case CW_WTP_OTHER_SW_VERSION:
	    default:
		break;
	}
    }

    /* Get device model name and device MAC */
    memset(model_name,0,FILE_NAME_LEN);
    memset(mac,0,6);
    memset(mac_str,0,MAC_LEN);
    for(i = 0; i < (WTPProtocolManager->WTPBoardData).vendorInfosCount; i++) {
	switch(((WTPProtocolManager->WTPBoardData).vendorInfos)[i].type ) {
	    case CW_WTP_MODEL_NUMBER:
		CW_COPY_MEMORY(model_name,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].valuePtr,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].length); 
		break;
	    case CW_BASE_MAC_ADDRESS:
		CW_COPY_MEMORY(mac,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].valuePtr,((WTPProtocolManager->WTPBoardData).vendorInfos)[i].length);
		break;
	    case CW_WTP_SERIAL_NUMBER:
	    case CW_BOARD_ID:
	    case CW_BOARD_REVISION:
	    default:
		break;
	}
    
    }

    device_type=get_device_type(model_name);
    if (device_type < 0) {
	CWLog("Can't support model: %s", model_name);
	return;
    }

    sprintf(mac_str, "%02X:%02X:%02X:%02X:%02X:%02X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    sprintf(str, "/bin/curl -v -k -s 'http://127.0.0.1/OutdoorEquipmentManagement/CAPWAP_CMD.shtml?"
		    "cmd=new_add&ip=%s&mac=%s&version=%s&dev_type=%d&name=%s&admin=%s&passwd=%s&"
		    "snmp_read=%s&vid_list=%s&gre_state=%d&behindNAT=%d'>/dev/null 2>&1",
		    wtp_ip, mac_str, ver, device_type, dev_name, pacctConf->username, pacctConf->password, 
		    psnmpConf->readComm,vid_list, 
		    (WTPProtocolManager->frameTunnelMode & CW_802_DOT_3_BRIDGING ? 2 : (pGreConf ? pGreConf->greEnabled : 0)),
		    gACProtocolUser->get_nat_check(NULL, WTPProtocolManager));

    capwap_free(dev_name);

    if((pp = popen(str, "r"))) 
	pclose(pp);
    else {
	CWLog("Add a entry to DB fail");
	exit(1);
    }

    return;
}

CWBool w6000_get_ac_ipv4_list(int **listPtr, int *countPtr)
{
    struct in_addr addr;
    char ip_buf[IP_LEN];
    char ip_list[AC_CLUSTER_NUM][IP_LEN];
    FILE *fp;
    char *t,*delim=" \n",*pos=NULL;
    int index=0,i=0;

    if(listPtr == NULL || countPtr == NULL) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

    fp = fopen(AC_CLUSTER_IP_DB_PATH, "r");
    if(fp) {
	while ( fgets(ip_buf, IP_LEN ,fp )) {
	    if(strlen(ip_buf) >1) {
		t=ip_buf;
		t=strtok_r(ip_buf,delim,&pos);
		strcpy(ip_list[index] , t);
		//t=strtok_r(NULL , delim ,&pos);
		index++;
	    }
	}
	fclose(fp);
	*countPtr = index;

	CW_CREATE_ARRAY_ERR((*listPtr), (*countPtr), int, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););

	for(i=0; i < index ; i++) {
	    inet_pton(AF_INET, ip_list[i], &addr);
	    (*listPtr)[i] = ntohl(addr.s_addr);
	}

	return CW_TRUE;
    }
    return CW_FALSE;
}

/** 
 * @brief Check if the WTP is in the same subnet of AC interface
 * 
 * @param WTPManagerPtr The pointer of WTP manager
 * 
 * @return True for yes.
 */
static CWBool w6000_wtp_in_same_subnet(CWWTPManager *WTPManagerPtr)
{
    //unsigned int incoming_ipaddr;
    //unsigned int wtp_ipaddr;
    //struct ifreq ifr;

    return CW_FALSE;

    /* TODO: check subnet mask of incoming address */
    /*
    incoming_ipaddr = CWACGetInterfaceIPv4AddressAtIndex(WTPManagerPtr->interfaceIndex);
    wtp_ipaddr = ntohl(((struct sockaddr_in *)&(WTPManagerPtr->address))->sin_addr);

    if (ioctl(WTPManagerPtr->sock, SIOCGIFNETMASK, &ifr) < 0) {
	CWLog("Get get netmask.");
	return CW_FALSE;
    }
    */
}

/** 
 * @brief Check if the interface index is WAN interface
 * 
 * @param ifindex Interface index
 * 
 * @return True for WAN interface, otherwise return False
 */
static CWBool w6000_is_wan_if(int ifindex)
{
    char wan_if[IFNAMSIZ], incoming_if[IFNAMSIZ];
    int i;

    if (if_indextoname(ifindex, incoming_if) == NULL) {
	CWLog("Can't get interface name for ifindex=%d", ifindex);
	return CW_FALSE;
    }
    CWLog("Incoming interface name is %s", incoming_if);

    for (i = 0; i < WAN_NUM; i++) {
	if (w6000_get_wan_ifname_by_idx(i, wan_if) < 0) {
	    CWLog("Can't get interface name of WAN%d", i);
	    continue;
	}
	if (strcmp(wan_if, incoming_if) == 0)
	    return CW_TRUE;
    }
    return CW_FALSE;
}

/*
 * Disabled GRE tunnel for AP which attached to LAN
 * 1. Get the incoming interface name by interface index
 * 2. Check if this interface is WAN interface
 * 3. If this interface is not WAN interface, check if this interface is in the same subnet of incoming interface
 * 4. If it belongs to the same subnet => disable GRE (also set pGreConf->greEnabled to 0)
 * 5. Other cases => Enable GRE (also set pGreConf->greEnabled to 1)
 */
static CWBool w6000_get_gre_conf(VSPGreConf *greConf, CWWTPProtocolManager *WTPProtocolManager)
{
    
    VSPGreConf *pGreConf;
    pGreConf = WTPProtocolManager->vsp4ipConf.greConf;
    CWWTPManager *WTPManagerPtr=NULL;
    unsigned int incoming_ipaddr;
    unsigned int incoming_ifindex;

    if(greConf == NULL ) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

    /* GRE config setting is null */
    if(pGreConf == NULL ) return CW_FALSE;

    CW_ZERO_MEMORY(greConf, sizeof(VSPGreConf));

    WTPManagerPtr = container_of(WTPProtocolManager, CWWTPManager, WTPProtocolManager);
    incoming_ipaddr = CWACGetInterfaceIPv4AddressAtIndex(WTPManagerPtr->interfaceIndex);
    incoming_ifindex = CWNetworkGetSystemIndexFromInterfaceIndex(&gACSocket, WTPManagerPtr->interfaceIndex);

    if (w6000_is_wan_if(incoming_ifindex) || !w6000_wtp_in_same_subnet(WTPManagerPtr)) {
	greConf->greEnabled=1;
    } else {
	greConf->greEnabled=0;
    }
    pGreConf->greEnabled = greConf->greEnabled;

    greConf->key=0;
    greConf->remoteIP = incoming_ipaddr;

    return CW_TRUE;
}

static CWBool w6000_get_acct_conf(VSPAcctConf *acctConf, CWWTPProtocolManager *WTPProtocolManager)
{
	return CW_FALSE;
	
}

static CWBool w6000_get_snmp_conf(VSPSnmpConf *snmpConf, CWWTPProtocolManager *WTPProtocolManager)
{
    char read_tpl[BUF_SIZE];
    char write_tpl[BUF_SIZE];
    VSPSnmpConf *psnmpConf;
    psnmpConf = WTPProtocolManager->vsp4ipConf.snmpConf;

    if(snmpConf == NULL ) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

    CW_ZERO_MEMORY(snmpConf, sizeof(VSPSnmpConf));
    /*TO-DO check the setting in template */
    sprintf(read_tpl,"public");
    sprintf(write_tpl,"private");
    if(psnmpConf->snmpEnabled == 0) {
	CWLog("SNMP did not enabled");
	snmpConf->snmpEnabled=1;
	CW_CREATE_STRING_FROM_STRING_ERR(snmpConf->readComm, (strlen(read_tpl) ? read_tpl: ""), CWLog("Can't allocate memory."););
	CW_CREATE_STRING_FROM_STRING_ERR(snmpConf->writeComm, (strlen(write_tpl) ? write_tpl: ""), CWLog("Can't allocate memory."););
	return CW_TRUE;
    }
    else {
	CWLog("SNMP was enabled");
	return CW_FALSE;
    }
    
    return CW_FALSE;
}

static CWBool w6000_get_vap_conf(VSPVapConf *vapConf, CWWTPProtocolManager *WTPProtocolManager)
{
    VSPVapConf *pVapConf = WTPProtocolManager->vsp4ipConf.vapConf;
    VAPMAP_T map_tmpl;
    CWWTPManager *WTPManagerPtr=NULL;
    unsigned int incoming_ifindex;
    int i;

    if(vapConf == NULL ) return CWErrorRaise(CW_ERROR_WRONG_ARG, NULL);

    /* Don't override VAP basic configuration if GRE tunnel doesn't need to enable */
    WTPManagerPtr = container_of(WTPProtocolManager, CWWTPManager, WTPProtocolManager);
    incoming_ifindex = CWNetworkGetSystemIndexFromInterfaceIndex(&gACSocket, WTPManagerPtr->interfaceIndex);
    if (!w6000_is_wan_if(incoming_ifindex) && w6000_wtp_in_same_subnet(WTPManagerPtr))
	return CW_FALSE;

    /* Reutn VAPConf to override WTP */
    if (w6000_get_vlan_sz_map(pVapConf, &map_tmpl)) {
	CW_CREATE_ARRAY_ERR((vapConf->vaps), pVapConf->vapCount, VSPVapValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
	vapConf->vapCount = pVapConf->vapCount;

	for (i = 0; i < map_tmpl.mt_count; i++) {
	    SET_CW_VSP_4IP_VAP_ID(vapConf->vaps[i].vapId, map_tmpl.mt_value[i].vm_rfidx, map_tmpl.mt_value[i].vm_vapidx);
	    vapConf->vaps[i].vapEnabled = map_tmpl.mt_value[i].vm_enabled;
	    vapConf->vaps[i].vapVlanId = map_tmpl.mt_value[i].vm_vlanid;
	    vapConf->vaps[i].vapTun = map_tmpl.mt_value[i].vm_tun;
	}
	w6000_free_vlan_sz_map(&map_tmpl);
	return CW_TRUE;
    }

    return CW_FALSE;
}


static CWBool w6000_get_nat_check(CWProtocolJoinRequestValues *joinRequest, CWWTPProtocolManager *WTPProtocolManager)
{
    CWWTPManager *WTPManagerPtr=NULL;
    WTPManagerPtr = container_of(WTPProtocolManager, CWWTPManager, WTPProtocolManager);
    struct sockaddr_in	*dev_sin,*nat_sin;

    dev_sin = (struct sockaddr_in *)&WTPManagerPtr->address;
    nat_sin = (struct sockaddr_in *)&WTPProtocolManager->ipv4Address;
    if(memcmp(&nat_sin->sin_addr, &dev_sin->sin_addr, sizeof(struct in_addr)))
	return CW_TRUE;
    else 
	return CW_FALSE;
}

static CWBool w6000_check_wtp_hw(CWWTPProtocolManager *WTPProtocolManagerPtr)
{
    int i;
    char model_name[BUFSIZ];
    CWWTPVendorInfos *bdata = &WTPProtocolManagerPtr->WTPBoardData;

    memset(model_name, 0, sizeof(model_name));
    for(i = 0; i < bdata->vendorInfosCount; i++) {
	if ((bdata->vendorInfos)[i].type == CW_WTP_MODEL_NUMBER) {
	    CW_COPY_MEMORY(model_name, (bdata->vendorInfos)[i].valuePtr, 
		    (bdata->vendorInfos)[i].length); 
	    break;
	}
    }

    if (strlen(model_name) == 0)
	return CW_FALSE;

    if (get_device_type(model_name) < 0)
	return CW_FALSE;

    return CW_TRUE;
}

int w6000_open_udptap(CWWTPProtocolManager *WTPProtocolManagerPtr)
{
    char 			wtp_ip[20];
    int 			devid;
    char 			sessionID[BUFSIZ];
    char 			cmd[BUFSIZ];
    char 			vid_list[2048]; /* size issue ? */
    VSPVapConf 			*pvapConf;
    char			mac_str[MAC_LEN];

    pvapConf = WTPProtocolManagerPtr->vsp4ipConf.vapConf;
    get_vid_list(pvapConf, vid_list, sizeof(vid_list), 1);
    if (strlen(vid_list) == 0) {
	    CWLog("%s: No VLAN mapping has been set, skip tunnel setup. (BUG!!)", __func__);
	    return -1;
    }

    get_ip_of_wtp(WTPProtocolManagerPtr, wtp_ip, 20);
    get_mac_of_wtp(WTPProtocolManagerPtr,mac_str);
    devid = get_devid_by_pair_ip_mac(wtp_ip,mac_str);
    sprintf(sessionID, "0x%x-0x%x-0x%x-0x%x",
	    WTPProtocolManagerPtr->sessionID[0],
	    WTPProtocolManagerPtr->sessionID[1],
	    WTPProtocolManagerPtr->sessionID[2],
	    WTPProtocolManagerPtr->sessionID[3]);

    if(devid > 0) {
	/* Close old */
	memset(cmd, 0, sizeof(cmd));
	sprintf(cmd, "/ramfs/od_emcd/bin/manipulateUdpTunnel.sh 3 2 %d", devid);
	system(cmd);

	/* Create new */
	memset(cmd, 0, sizeof(cmd));
	sprintf(cmd, "/ramfs/od_emcd/bin/manipulateUdpTunnel.sh 1 2 "
		"%d %s %s %s",
		devid, wtp_ip, vid_list, sessionID);
	system(cmd);

	CWLog("%s: Udp tunnel opened for %s (%s)", __func__, wtp_ip, sessionID);

    }
    else {
	CWLog("%s: No such device id:%d", __func__, devid);
	return -1;
    }
    return 0;
}

#if 0
int w6000_close_udptap(CWWTPProtocolManager *WTPProtocolManagerPtr)
{
    int devid = get_devid_by_wtp(WTPProtocolManagerPtr);
    char cmd[BUFSIZ];

    sprintf(cmd, "/ramfs/od_emcd/bin/manipulateUdpTunnel.sh 3 2 %d", devid);
    system(cmd);
    return 0;
}
#endif

static struct ACProtocol_User w6000 = {
    .name			= "W6000",
    .lib_init			= ac_lib_init,
    .get_name			= w6000_get_name,
    .get_stations		= w6000_get_stations,
    //.get_limit		= w6000_get_max_station,
    .get_max_WTPs		= w6000_get_max_WTPs,
    .get_ac_ipv4_list		= w6000_get_ac_ipv4_list,
    .get_vsp_gre_conf		= w6000_get_gre_conf,
    .get_vsp_acct_conf		= w6000_get_acct_conf,
    .get_vsp_snmp_conf		= w6000_get_snmp_conf,
    .get_vsp_vap_conf		= w6000_get_vap_conf,
    .get_nat_check		= w6000_get_nat_check,
    .check_wtp_hw		= w6000_check_wtp_hw,
    .add_wtp			= w6000_add_wtp,
    .open_data_channel		= w6000_open_udptap,
    //.close_data_channel		= w6000_close_udptap,
};


static void w6000_init(void)
{
    if (!CWErr(ACRegisterModule(&w6000))) {
	CWLog("Can't register AC protocol module \"%s\".", w6000.name);
    }
    if(w6000.lib_init()){
	CWLog("w6000 lib init fail");
	exit(1);
    }
}

static void w6000_exit(void)
{
    if (!CWErr(ACUnregisterModule(&w6000))) {
	CWLog("Can't unregister AC protocol module \"%s\".", w6000.name);
    }
}

AC_MOD_INIT(w6000_init);
AC_MOD_EXIT(w6000_exit);
