#include "CWWTP.h"
#include "WTPProtocol_User.h"
#include "cipcfg.h"
#include "wlan.h"
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>  
#include <netinet/in.h>  
#include <arpa/inet.h>   

extern char *current_config;
extern const int gid_vap[MAX_RF][MAX_VAP];

#define BINDING_80211_ENC_CAP_TKIP_S	2
#define BINDING_80211_ENC_CAP_AES_S	3
#define TUN_IFNAME	"udpETH"

static int eap_get_discovery_type(void)
{
    /* TODO: check the prefer Addres, need to hack WTPDiscoveryState.c */
    return CW_MSG_ELEMENT_DISCOVERY_TYPE_CONFIGURED;
}

static int eap_get_max_radios(void)
{
    return max_rf;
}

static int eap_get_radio_in_use(void)
{
    int i, count = 0;
    CIPCFG *cipcfg = NULL;
    char group[MAX_VALUE_LEN];
    char value[MAX_VALUE_LEN];

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	return 0;
    }

    for (i = 0; i < max_rf; i++) {
	sprintf(group, "rf%d", i+1);
	if (get_cfg_item_by_str(cipcfg, group, "state", value) 
		== CIPCFG_SUCCESS) {
	    if (strcmp(value, "enable") == 0)
		count++;
	} else {
	    CWLog("Can't get state of %s.", group);
	}
    }

    close_cipcfg(cipcfg);
    return count;
}

static int eap_get_enc_capabilities(void)
{
    return (1 << BINDING_80211_ENC_CAP_TKIP_S) | 
	(1 << BINDING_80211_ENC_CAP_AES_S);
}

static int eap_get_frame_tunnel_mode(void)
{
    return CW_802_DOT_3_BRIDGING;
}

static CWBool eap_get_board_data(CWWTPVendorInfos *valPtr)
{
    char model[MAX_VALUE_LEN];
    char mac_str[18];
    unsigned char mac[6];

    CW_ZERO_MEMORY(model, sizeof(model));
    get_sys_item("model", model);
    CW_ZERO_MEMORY(mac_str, sizeof(mac_str));
    get_sys_item("wan_mac", mac_str);
    sscanf(mac_str, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX", 
	    &mac[0], &mac[1], &mac[2], &mac[3], &mac[4], &mac[5]);

    valPtr->vendorInfosCount = 3; // we fill 3 information (just the required ones)
    CW_CREATE_ARRAY_ERR((valPtr->vendorInfos), valPtr->vendorInfosCount, CWWTPVendorInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    (valPtr->vendorInfos)[0].vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    (valPtr->vendorInfos)[0].type = CW_WTP_MODEL_NUMBER;
    (valPtr->vendorInfos)[0].length = strlen(model); // string length without NULL terminate
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[0] ).valuePtr), (valPtr->vendorInfos)[0].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    CW_COPY_MEMORY(((valPtr->vendorInfos)[0]).valuePtr, model, (valPtr->vendorInfos)[0].length); // MODEL NUMBER

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    (valPtr->vendorInfos)[1].vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    (valPtr->vendorInfos)[1].type = CW_WTP_SERIAL_NUMBER;
    (valPtr->vendorInfos)[1].length = sizeof(long int); // just one int
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[1] ).valuePtr), (valPtr->vendorInfos)[1].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    *(((valPtr->vendorInfos)[1]).valuePtr) = 123456; // SERIAL NUMBER

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    (valPtr->vendorInfos)[2].vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    (valPtr->vendorInfos)[2].type = CW_BASE_MAC_ADDRESS;
    (valPtr->vendorInfos)[2].length = sizeof(mac);  // mac address length
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[2] ).valuePtr), (valPtr->vendorInfos)[2].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    CW_COPY_MEMORY(((valPtr->vendorInfos)[2]).valuePtr, mac, (valPtr->vendorInfos)[2].length); // WAN MAC

    return CW_TRUE;
}

static CWBool eap_get_vendor_infos(CWWTPVendorInfos *valPtr)
{
    char sw_ver[MAX_VALUE_LEN];

    CW_ZERO_MEMORY(sw_ver, MAX_VALUE_LEN);
    get_sys_item("sw_ver", sw_ver);

    valPtr->vendorInfosCount = 3; // we fill 3 information (just the required ones)
    CW_CREATE_ARRAY_ERR((valPtr->vendorInfos), valPtr->vendorInfosCount, CWWTPVendorInfoValues, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    (valPtr->vendorInfos)[0].vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    (valPtr->vendorInfos)[0].type = CW_WTP_HARDWARE_VERSION;
    (valPtr->vendorInfos)[0].length = sizeof(long int); // just one int
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[0] ).valuePtr), (valPtr->vendorInfos)[0].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    *(((valPtr->vendorInfos)[0]).valuePtr) = 123456; // HW version

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    ((valPtr->vendorInfos)[1]).vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    ((valPtr->vendorInfos)[1]).type = CW_WTP_SOFTWARE_VERSION;
    ((valPtr->vendorInfos)[1]).length = strlen(sw_ver); // string length without NULL terminate
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[1] ).valuePtr), (valPtr->vendorInfos)[1].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    CW_COPY_MEMORY(((valPtr->vendorInfos)[1]).valuePtr, sw_ver, (valPtr->vendorInfos)[1].length); // SW version

    // my vendor identifier (IANA assigned "SMI Network Management Private Enterprise Code")
    (valPtr->vendorInfos)[2].vendorIdentifier = INAN_PRIVATE_NUM_4IPNET;
    (valPtr->vendorInfos)[2].type = CW_BOOT_VERSION;
    (valPtr->vendorInfos)[2].length = sizeof(long int); // just one int
    CW_CREATE_OBJECT_SIZE_ERR(( ( (valPtr->vendorInfos)[2] ).valuePtr), (valPtr->vendorInfos)[2].length, return CWErrorRaise(CW_ERROR_OUT_OF_MEMORY, NULL););
    *(((valPtr->vendorInfos)[2]).valuePtr) = 1234568; // Boot version

    return CW_TRUE;
}

static char *eap_get_location(void)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];
    char *oldLocation = NULL;

    memset(value, 0, MAX_VALUE_LEN);
    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }
    if (get_cfg_item_by_str(cipcfg, "device", "location", value) 
	    != CIPCFG_SUCCESS) {
	CWLog("Can't get location from cipcfg.");
	goto fail;
    }
    oldLocation = gWTPLocation;
    gWTPLocation = NULL;
    CW_CREATE_STRING_FROM_STRING_ERR(gWTPLocation, value, CWLog("Can't allocate memory."););
    if (gWTPLocation) {
	CW_FREE_OBJECT(oldLocation);
    } else
	gWTPLocation = oldLocation;
fail:
    close_cipcfg(cipcfg);
    return gWTPLocation;
}

static char *eap_get_name(void)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];
    char *oldName = NULL;

    memset(value, 0, MAX_VALUE_LEN);
    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }
    if (get_cfg_item_by_str(cipcfg, "device", "name", value) 
	    != CIPCFG_SUCCESS) {
	CWLog("Can't get name from cipcfg.");
	goto fail;
    }
    oldName = gWTPName;
    gWTPName = NULL;
    CW_CREATE_STRING_FROM_STRING_ERR(gWTPName, value, CWLog("Can't allocate memory."););
    if (gWTPName) {
	CW_FREE_OBJECT(oldName);
    } else
	gWTPName = oldName;

fail:
    close_cipcfg(cipcfg);
    return gWTPName;
}

struct ac_list {
    int ac_prio;
    char *ac_name;
    in_addr_t ac_addr;
};

static struct ac_list *get_aclist_cache(CIPCFG *cipcfg, int *size)
{
    unsigned int tbl_size, i;
    char value[MAX_VALUE_LEN], attr[MAX_VALUE_LEN];
    struct ac_list *list = NULL;

    if (get_cfg_item_by_str(cipcfg, "capwap", "aclist.table_size", value) !=
	    CIPCFG_SUCCESS) {
	CWLog("Can't get name from cipcfg.");
	return NULL;
    }
    tbl_size = atoi(value);
    *size = tbl_size;

    list = calloc(tbl_size, sizeof(struct ac_list));
    if (!list) {
	CWLog("Can't allocate memroy for ac list.\n");
	return NULL;
    }

    for (i = 0; i < tbl_size; i++) {
	sprintf(attr, "aclist.ac_entry:%d.priority", i);
	get_cfg_item_by_str(cipcfg, "capwap", attr, value);
	list[i].ac_prio = atoi(value);
	sprintf(attr, "aclist.ac_entry:%d.name", i);
	get_cfg_item_by_str(cipcfg, "capwap", attr, value);
	list[i].ac_name = strdup(value);
	sprintf(attr, "aclist.ac_entry:%d.ip", i);
	get_cfg_item_by_str(cipcfg, "capwap", attr, value);
	list[i].ac_addr = ntohl(inet_addr(value));
	if (list[i].ac_addr == INADDR_NONE)
	    list[i].ac_addr = 0;
    }

    return list;
}

static void free_aclist_cache(struct ac_list *list, int size)
{
    int i;

    for (i = 0; i < size; i++) {
	if (list[i].ac_name)
	    free(list[i].ac_name);
    }
    free(list);
}

static int find_idx_by_addr(struct ac_list *list, int size, in_addr_t addr)
{
    int i;

    for (i = 0; i < size; i++) {
	if (list[i].ac_addr && addr == list[i].ac_addr)
	    return -1;
	else if (list[i].ac_addr)
	    continue;
	if (list[i].ac_name && strlen(list[i].ac_name) > 0)
	    continue;
	return i;
    }
    return -1;
}

static int eap_save_ac_info(CWACInfoValues *ACInfoPtr)
{
    CIPCFG *cipcfg = NULL;
    struct ac_list *list = NULL;
    struct sockaddr_in *sa = (struct sockaddr_in *)&ACInfoPtr->incomingAddress;
    char *ac_name = ACInfoPtr->name;
    char attr[MAX_VALUE_LEN];
    int i, ac_num;
    ITEM_LIST *cfg_list = NULL;

    if ((cipcfg = init_cipcfg()) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    list = get_aclist_cache(cipcfg, &ac_num);
    for (i = 0; i < ac_num; i++) {
	if (strcmp(ac_name, list[i].ac_name) == 0 && list[i].ac_addr == 0) {
	    sprintf(attr, "aclist.ac_entry:%d.ip", i);
	    cfg_list = append_cfg_item(cfg_list, GID_CAPWAP, attr, 
		    inet_ntoa(sa->sin_addr));
	    CWLog("Map AC Name \"%s\" to %s.", ac_name, inet_ntoa(sa->sin_addr));
	    break;
	}
    }
    free_aclist_cache(list, ac_num);

    if (set_cfg_items_lite(cipcfg, cfg_list) != CIPCFG_SUCCESS) {
	CWLog("Can't save ac list configuration in cipfg.\n");
	goto fail;
    }
    if (save_cipcfg(cipcfg, current_config) != CIPCFG_SUCCESS) {
	CWLog("Can't save to %s.\n", current_config);
	goto fail;
    }

    close_cipcfg(cipcfg);
    return 0;
fail:
    if (cfg_list)
	free_item_list(cfg_list);
    close_cipcfg(cipcfg);
    return -1;
}

static int eap_save_aclist(CWACInfoValues *ACInfoPtr)
{
    CIPCFG *cipcfg = NULL;
    ACIPv4ListValues *aclist = &ACInfoPtr->ACIPv4ListInfo;
    char value[MAX_VALUE_LEN], attr[MAX_VALUE_LEN];
    int i, ac_num;
    struct ac_list *list = NULL;
    ITEM_LIST *cfg_list = NULL;

    if ((cipcfg = init_cipcfg()) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    list = get_aclist_cache(cipcfg, &ac_num);
    for (i = 0; i < aclist->ACIPv4ListCount; i++) {
	int idx;
	struct in_addr addr;

	if ((idx = find_idx_by_addr(list, ac_num, aclist->ACIPv4List[i])) < 0)
	    continue;
	list[idx].ac_addr = aclist->ACIPv4List[i];
	/* Save to cipcfg */
	sprintf(attr, "aclist.ac_entry:%d.priority", idx);
	sprintf(value, "%d", 500+idx);
	cfg_list = append_cfg_item(cfg_list, GID_CAPWAP, attr, value);
	sprintf(attr, "aclist.ac_entry:%d.type", idx);
	cfg_list = append_cfg_item(cfg_list, GID_CAPWAP, attr, "referral");
	sprintf(attr, "aclist.ac_entry:%d.ip", idx);
	addr.s_addr = htonl(aclist->ACIPv4List[i]);
	cfg_list = append_cfg_item(cfg_list, GID_CAPWAP, attr, inet_ntoa(addr));
	CWLog("Save AC %s in cipcfg.", inet_ntoa(addr));
    }
    free_aclist_cache(list, ac_num);

    if (set_cfg_items_lite(cipcfg, cfg_list) != CIPCFG_SUCCESS) {
	CWLog("Can't save ac list configuration in cipfg.\n");
	goto fail;
    }
    if (save_cipcfg(cipcfg, current_config) != CIPCFG_SUCCESS) {
	CWLog("Can't save to %s.\n", current_config);
	goto fail;
    }

    close_cipcfg(cipcfg);
    return 0;
fail:
    close_cipcfg(cipcfg);
    return -1;
}

static CWBool eap_get_gre_conf(VSPGreConf *greConf)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];
    struct in_addr remote;

    CW_ZERO_MEMORY(greConf, sizeof(VSPGreConf));

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    // Get gre.state
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "gre", "state", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get gre.state from cipcfg.");
	goto fail;
    }
    if (strlen(value) > 0)
	greConf->greEnabled = (strcmp("enable", value) ? 0 : 1);

    // Get gre.remote
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "gre", "remote", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get gre.remote from cipcfg.");
	goto fail;
    }
    if (strlen(value) > 0 && inet_aton(value, &remote) != 0)
	greConf->remoteIP = ntohl(remote.s_addr);

    // Get gre.key
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "gre", "key", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get gre.key from cipcfg.");
	goto fail;
    }
    if (strlen(value) > 0)
	greConf->key = atoi(value);

    close_cipcfg(cipcfg);
    return CW_TRUE;
fail:
    close_cipcfg(cipcfg);
    return CW_FALSE;
}

static CWBool eap_get_acct_conf(VSPAcctConf *acctConf)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];

    CW_ZERO_MEMORY(acctConf, sizeof(VSPAcctConf));

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    // Get usernmae
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "admin", "name", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get admin.name from cipcfg.");
	goto fail;
    }
    CW_CREATE_STRING_FROM_STRING_ERR(acctConf->username, 
	    (strlen(value) ? value : ""), CWLog("Can't allocate memory."););

    // Get password
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "admin", "password", value) != 
	    CIPCFG_SUCCESS) {
	CWLog("Can't get admin.name from cipcfg.");
	goto fail;
    }
    CW_CREATE_STRING_FROM_STRING_ERR(acctConf->password, 
	    (strlen(value) ? value : ""), CWLog("Can't allocate memory."););

    close_cipcfg(cipcfg);
    return CW_TRUE;
fail:
    close_cipcfg(cipcfg);
    return CW_FALSE;
}

static CWBool eap_get_snmp_conf(VSPSnmpConf *snmpConf)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];

    CW_ZERO_MEMORY(snmpConf, sizeof(VSPSnmpConf));

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    // Get state
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "snmp", "state", value) 
	    != CIPCFG_SUCCESS) {
	CWLog("Can't get snmp.state from cipcfg.");
	goto fail;
    }
    if (strlen(value) > 0)
	snmpConf->snmpEnabled = (strcmp("enable", value) ? 0 : 1);

    // Get read community
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "snmp", "read", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get snmp.read from cipcfg.");
	goto fail;
    }
    CW_CREATE_STRING_FROM_STRING_ERR(snmpConf->readComm, 
	    (strlen(value) ? value : ""), CWLog("Can't allocate memory."););

    // Get write community
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "snmp", "write", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get snmp.write from cipcfg.");
	goto fail;
    }
    CW_CREATE_STRING_FROM_STRING_ERR(snmpConf->writeComm, 
	    (strlen(value) ? value : ""), CWLog("Can't allocate memory."););

    // Get trap server address
    memset(value, 0, MAX_VALUE_LEN);
    if (get_cfg_item_by_str(cipcfg, "snmp", "trap", value) != CIPCFG_SUCCESS) {
	CWLog("Can't get snmp.trap from cipcfg.");
	goto fail;
    }
    if (strlen(value) > 0 && strcmp("enable", value) == 0) {
	struct in_addr trap_addr;
	memset(value, 0, MAX_VALUE_LEN);
	if (get_cfg_item_by_str(cipcfg, "snmp", "trap_server", value) 
		!= CIPCFG_SUCCESS) {
	    CWLog("Can't get snmp.trap_server from cipcfg.");
	    goto fail;
	}
	if (strlen(value) > 0 && inet_aton(value, &trap_addr) != 0)
	    snmpConf->trapIP = ntohl(trap_addr.s_addr);
    }

    close_cipcfg(cipcfg);
    return CW_TRUE;
fail:
    close_cipcfg(cipcfg);
    return CW_FALSE;
}

static CWBool eap_get_vap_conf(VSPVapConf *vapConf)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];
    int vapCount, i;
    int rfidx = 0; // TODO: support multiple RF

    CW_ZERO_MEMORY(vapConf, sizeof(VSPVapConf));

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
	CWLog("Can't open cipcfg.");
	goto fail;
    }

    // FIXME: Get Maximum number of VAP
    for (vapCount = 0; 
	    get_ifname_by_idx(IF_ROLE_VAP, rfidx, vapCount, value); 
	    vapCount++);
    
    if (!vapCount) {
	CWLog("No VAP avaiable.");
	goto fail;
    }
    if (vapCount >= CW_VSP_4IP_MAX_VAP_PER_RF) {
	CWLog("Can't not support more than %d VAPs.", CW_VSP_4IP_MAX_VAP_PER_RF);
	goto fail;
    }

    vapConf->vapCount = vapCount;
    CW_CREATE_ARRAY_ERR(vapConf->vaps, vapCount, VSPVapValues, goto fail;);
    for (i = 0; i < vapCount; i++) {
	char group[MAX_VALUE_LEN];
	sprintf(group, "rf%d_vap%d", rfidx+1, i+1);
	SET_CW_VSP_4IP_VAP_ID((vapConf->vaps)[i].vapId, rfidx, i);
	if (get_cfg_item_by_str(cipcfg, group, "state", value) != CIPCFG_SUCCESS) {
	    CWLog("Can't get %s.state from cipcfg.", group);
	    goto fail;
	}
	(vapConf->vaps)[i].vapEnabled = (strcmp("enable", value) ? 0 : 1);
	if (get_cfg_item_by_str(cipcfg, group, "vid", value) != CIPCFG_SUCCESS) {
	    CWLog("Can't get %s.vid from cipcfg.", group);
	    goto fail;
	}
	(vapConf->vaps)[i].vapVlanId = atoi(value);
	if (get_cfg_item_by_str(cipcfg, group, "tun", value) != CIPCFG_SUCCESS) {
	    CWLog("Can't get %s.tun from cipcfg.", group);
	    goto fail;
	}
	(vapConf->vaps)[i].vapTun = (strcmp("enable", value) ? 0 : 1);
    }

    close_cipcfg(cipcfg);
    return CW_TRUE;
fail:
    close_cipcfg(cipcfg);
    return CW_FALSE;
}

static int eap_save_vsp_conf(VSP4IPConf *newConf)
{
    CIPCFG *cipcfg = NULL;
    char value[MAX_VALUE_LEN];
    ITEM_LIST *list = NULL;
    VSP4IPConf vsp4ipConf, *oldConf;

    oldConf = &vsp4ipConf;
    CW_PREPARE_VSP4IPCONF_OBJECT(oldConf, goto fail;);

    // Account configuration
    if (newConf->acctConf && eap_get_acct_conf(oldConf->acctConf)) {
	/* XXX Now AP can't modify account username */
	//if (strcmp(oldConf->acctConf->username, newConf->acctConf->username)) {
	//    list = append_cfg_item(list, GID_USER1, "name", 
	//	    newConf->acctConf->username);
	//}
	if (strcmp(oldConf->acctConf->username, newConf->acctConf->username)) {
	    list = append_cfg_item(list, GID_USER1, "password", 
		    newConf->acctConf->password);
	}
    }

    // SNMP configuration
    if (newConf->snmpConf && eap_get_snmp_conf(oldConf->snmpConf)) {
	if (oldConf->snmpConf->snmpEnabled != newConf->snmpConf->snmpEnabled) {
	    list = append_cfg_item(list, GID_SNMP, "state",
		    (newConf->snmpConf->snmpEnabled ? "enable" : "disable"));
	}
	if (newConf->snmpConf->snmpEnabled) {
	    /* XXX: Now we don't override the write community and trap server */
	    if (strcmp(oldConf->snmpConf->readComm, newConf->snmpConf->readComm)) {
		list = append_cfg_item(list, GID_SNMP, "read", 
			newConf->snmpConf->readComm);
	    }
	}
    }

    // VAP Configuration
    if (newConf->vapConf && eap_get_vap_conf(oldConf->vapConf)) {
	int i, j, rfidx, vapidx, gid;
	char vid[MAX_VALUE_LEN];
	VSPVapValues *newVap, *oldVap;

	for (i = 0; i < newConf->vapConf->vapCount; i++) {
	    for (j = 0; j < oldConf->vapConf->vapCount; j++) {
		newVap = &(newConf->vapConf->vaps[i]);
		oldVap = &(oldConf->vapConf->vaps[j]);
		if (newVap->vapId == oldVap->vapId) {
		    rfidx = GET_CW_VSP_4IP_RF_IDX(newVap->vapId);
		    vapidx = GET_CW_VSP_4IP_VAP_IDX(newVap->vapId);
		    gid = gid_vap[rfidx][vapidx];
		    if (newVap->vapEnabled != oldVap->vapEnabled) {
			list = append_cfg_item(list, gid, "state", 
				(newVap->vapEnabled ? "enable" : "disable"));
		    }
		    if (newVap->vapTun != oldVap->vapTun) {
			list = append_cfg_item(list, gid, "tun",
				(newVap->vapTun ? "enable" : "disable"));
		    }
		    /* Only override the VLAN ID of VAP when tunnel is enabled */
		    if (newVap->vapTun && 
			    newVap->vapVlanId != oldVap->vapVlanId) {
			sprintf(vid, "%d", newVap->vapVlanId);
			list = append_cfg_item(list, gid, "vid", vid);
		    }
		    break;
		}
	    }
	}
    }

    if (list) {
	if ((cipcfg = init_cipcfg()) == NULL) {
	    CWLog("Can't open cipcfg.");
	    goto fail;
	}
	if (set_cfg_items_lite(cipcfg, list) != CIPCFG_SUCCESS) {
	    CWLog("Can't save configuration in cipfg.\n");
	    goto fail;
	}
	if (save_cipcfg(cipcfg, current_config) != CIPCFG_SUCCESS) {
	    CWLog("Can't save to %s.\n", current_config);
	    goto fail;
	}
	/* Set the Reboot flag here, this flag is checked in Data check state */
	WTPRebootAfterConfigure = CW_TRUE;
	close_cipcfg(cipcfg);
    }
    CW_CLEAN_VSP4IPCONF_OBJECT(oldConf);
    return 0;
fail:
    CW_CLEAN_VSP4IPCONF_OBJECT(oldConf);
    if (list)
	free_item_list(list);
    close_cipcfg(cipcfg);
    return -1;
}

static int eap_open_udptap(CWACInfoValues *ACInfoPtr)
{
    char cmd[BUFSIZ];
    struct sockaddr_in peer_addr;

    CW_COPY_NET_ADDR_PTR(&peer_addr, &(ACInfoPtr->preferredAddress));
    if (peer_addr.sin_family != AF_INET) {
	CWLog("Not support non-IPv4 address.");
	return 1;
    }
    
    /* Close udptap first */
    system("ifconfig " TUN_IFNAME "down");
    system("ip link delete " TUN_IFNAME);

    CWLog("Setup CAPWAP data channel to %s.", 
	    inet_ntoa(peer_addr.sin_addr));
    sprintf(cmd, "ip link add %s type udptap client "
		    "remote %s capwap 0x%x 0x%x 0x%x 0x%x",
	    TUN_IFNAME, inet_ntoa(peer_addr.sin_addr), 
	    ACInfoPtr->SessionID[0], ACInfoPtr->SessionID[1],
	    ACInfoPtr->SessionID[2], ACInfoPtr->SessionID[3]);
    system(cmd);

    system("brctl addif brTun0 " TUN_IFNAME);
    sleep(5);	/* Wait for AC to ready */
    system("ifconfig " TUN_IFNAME " up");

    return 0;
}

static int eap_close_udptap(CWACInfoValues *ACInfoPtr)
{
    CWLog("Remove CAPWAP data channel.");
    system("ifconfig " TUN_IFNAME "down");
    system("ip link delete " TUN_IFNAME);
    return 0;
}

static struct WTPProtocol_User eap = {
    .name		    = "EAP",
    .get_discovery_type	    = eap_get_discovery_type,
    .get_max_radios	    = eap_get_max_radios,
    .get_radio_in_use	    = eap_get_radio_in_use,
    //.get_enc_capabilities   = eap_get_enc_capabilities,
    .get_frame_tunnel_mode  = eap_get_frame_tunnel_mode,
    .get_board_data	    = eap_get_board_data,
    .get_vendor_infos	    = eap_get_vendor_infos,
    .get_location	    = eap_get_location,
    .get_name		    = eap_get_name,
    //.save_ac_info	    = eap_save_ac_info,
    .save_aclist	    = eap_save_aclist,
    //.get_vsp_gre_conf	    = eap_get_gre_conf,
    .get_vsp_acct_conf	    = eap_get_acct_conf,
    .get_vsp_snmp_conf	    = eap_get_snmp_conf,
    .get_vsp_vap_conf	    = eap_get_vap_conf,
    .save_vsp_conf	    = eap_save_vsp_conf,
    .open_data_channel	    = eap_open_udptap,
    .close_data_channel     = eap_close_udptap,
};

static void eap_init(void)
{
    if (!CWErr(WTPRegisterModule(&eap))) {
	CWLog("Can't register WTP protocol module \"%s\".", eap.name);
    }
}

static void eap_exit(void)
{
    if (!CWErr(WTPUnregisterModule(&eap))) {
	CWLog("Can't unregister WTP protocol module \"%s\".", eap.name);
    }
}

WTP_MOD_INIT(eap_init);
WTP_MOD_EXIT(eap_exit);
