#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#ifndef CONFIG_KEY_OFFSET
#define CONFIG_KEY_OFFSET 2048
#endif

#ifndef CONFIG_FWP_ID_PARTITION // A300 : /dev/mtdblock6
#define CONFIG_FWP_ID_PARTITION "/dev/mtdblock1"      
#endif

#ifndef CONFIG_FWP_KEY_PARTITION // A300 : /dev/mtdblock6
#define CONFIG_FWP_KEY_PARTITION "/dev/mtdblock1"
#endif

#ifndef FWP_MODEL_HASH_FILE
#define FWP_MODEL_HASH_FILE "/etc/model.hash"   
#endif

#ifndef CONFIG_FWP_ID_OFFSET // A300 : 102 
#define CONFIG_FWP_ID_OFFSET 8 
#endif

#ifndef FWP_DATA_SIZE
#define FWP_DATA_SIZE 1024
#endif

#ifndef FWP_ID_SIZE
#define FWP_ID_SIZE 6 
#endif

#ifndef FWP_HASH_SIZE
#define FWP_HASH_SIZE 16 
#endif

#undef MAX_VALUE_LEN
#define MAX_VALUE_LEN 256

/* copy from syscall.c in cipcfg to remove dependency */
static char *get_rc_env(char *name, char *value, int len)
{
    FILE *fp;
    char buf[MAX_VALUE_LEN];
    if (!name || !*name || !value || len <= 0)
	return NULL;
    *value = '\0';
    sprintf(buf, ". /etc/rc.d/rc.env; echo -n \"$%s\"", name);
    if (!(fp = popen(buf, "r")))
	return NULL;
    *buf = '\0';
    fgets(buf, sizeof(buf), fp);
    pclose(fp);

    if (len < sizeof(buf))
	buf[len-1] = '\0';
    strcpy(value, buf);

    return value;
}

/* copy from syscall.c in cipcfg to remove dependency */
static int cip_system(char *cmd)
{
    if (cmd == 0) return 1;
    return system(cmd);
}

int key_verify_fail()
{
    char *mgnt[] = {"stop_syslog", "stop_sshd", "stop_snmp", "stop_crond"};
    char buf[MAX_VALUE_LEN], cmd[MAX_VALUE_LEN];
    char *ptr, *pch;
    int i, num = 0;

    /* Stop syslog, sshd, snmp and crond */
    for(i=0; i < 4; i++) {
        sprintf(cmd, "/etc/rc.d/rc.mgnt %s", mgnt[i]);
        cip_system(cmd);
    }

    /* Get ATH_IRQS from rc.env */
    if(get_rc_env("ATH_IRQS", buf, sizeof(buf)) != NULL) {

        /* Get the number of IRQS which is the number of RF cards */
        pch = strtok_r(buf, " ", &ptr);
        while (pch != NULL) {
            if(isdigit(pch[0]))
                num++;
            pch = strtok_r(NULL, " ", &ptr);
        }
        /* Stop all RF cards */
        for (i=0; i<num; i++) {
            sprintf(cmd,"/etc/rc.d/rc.wlan stop %d", i);
            cip_system(cmd);
        }
        sprintf(cmd,"/etc/rc.d/rc.wlan stop");
        cip_system(cmd);
    }

    /* Suppose the file system is writable and we use an overlay filesystem:
       Move home2.asp to home.asp to show key invalid message. 
     */
    rename("/web/html/home2.asp", "/web/html/home.asp");
    return 0;   
}

int firmware_protection_check()
{
    extern int fwp_key_verify(unsigned char *key, unsigned char *id, int size, unsigned char *hash);
    unsigned char product_key[FWP_DATA_SIZE];
    unsigned char *product_id, *decrypted_hash;
    FILE *fp;
    int fd;
    int ret;

    product_id=malloc(FWP_ID_SIZE * sizeof(char));
    decrypted_hash=malloc( FWP_HASH_SIZE * sizeof(char));

    if((fd = open(CONFIG_FWP_KEY_PARTITION, O_RDONLY)) >= 0) {
        lseek(fd, CONFIG_KEY_OFFSET, SEEK_SET);   // product key offset = 2048
        read(fd, product_key, FWP_DATA_SIZE);
        close(fd);
    }
    /* Get product ID (MAC or some unique id) */
    if((fd = open(CONFIG_FWP_ID_PARTITION, O_RDONLY)) >= 0) {
        lseek(fd, CONFIG_FWP_ID_OFFSET, SEEK_SET);      // mac offset = 8
        read(fd,product_id, FWP_ID_SIZE);
        close(fd);
    }
    /* Hash file */
    if((fp = fopen(FWP_MODEL_HASH_FILE, "r")) != NULL) {
        fread(decrypted_hash, 1, FWP_HASH_SIZE, fp);
        fclose(fp);
    }
    /* Key verification: pass the production key, mac, mac size and hash
       Return 0 means key verification success, otherwise, the verification
       fail, and we have to stop some services and print the key invalid 
       message on web page.
     */
    ret = fwp_key_verify(product_key, product_id, FWP_ID_SIZE, decrypted_hash);
//        key_verify_fail();

    free(product_id);
    free(decrypted_hash);

    return ret;
}
