#!/bin/sh

# Error code list
# 1 -- tarball format error
# 2 -- Don't support customize
# 3 -- File checking error

CUSTOMIZE_LIST=/web/customize_list.txt
CUSTOMIZE_DEST=/conf/customize
CUSTOMIZE_TMP=/tmp/customize

if [ "$1" == "reset" ]; then
    rm -rf $CUSTOMIZE_DEST
    # Config archiving
    [ -x /sbin/cfgarcv.sh ] && /sbin/cfgarcv.sh misc
    exit 0;
fi

tarball=$1

clean_and_exit()
{
    local exit_code=$1
    rm -rf $tarball $CUSTOMIZE_TMP
    exit $exit_code
}

# Step 1 -- Create temp folder
mkdir -p $CUSTOMIZE_TMP

# Step 2 -- untar tarball
tar zxf $tarball -C $CUSTOMIZE_TMP
if [ "$?" != "0" ]; then
    clean_and_exit 1	# Format error
fi

# Step 3 -- read list file and check file
files=""
if [ ! -f $CUSTOMIZE_LIST ]; then
    clean_and_exit 2	# Don't support customize
fi
while read file prog; do
    if [ -z "$(echo $file|grep '#')" ] && [ -e $CUSTOMIZE_TMP/$file ]; then
	if [ -n "$prog" ]; then
	    $prog $CUSTOMIZE_TMP/$file
	    if [ "$?" != "0" ]; then
		clean_and_exit 3
	    fi
	fi
	files="$files $CUSTOMIZE_TMP/$file"
    fi
done < $CUSTOMIZE_LIST

# Step 4 -- Copy customize files to rom
if [ ! -d $CUSTOMIZE_DEST ]; then
    mkdir -p $CUSTOMIZE_DEST
fi
for file in $files; do
    cp $file $CUSTOMIZE_DEST/
done
# Config archiving
[ -x /sbin/cfgarcv.sh ] && /sbin/cfgarcv.sh misc

clean_and_exit 0
