/**
 *  \file web_get.c
 *
 *  This is the cgi modules for A800 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2007 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/28/2006     0.1       Jimmy Yang     Initial version
 *
 * 
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#ifdef WITH_ACC_MGMT
#include "accmgmt_curusr.h"
#endif
#include "appweb/appweb.h"
#include "id.h"
#include "cipcfg.h"
#include "cipcfg_related_common.h"
#include "syscall.h"
#include "wlan.h"
#include "web_get.h"
#ifdef SUPPORT_REPORT_AGENT
#include "agent.h"
#endif
extern const unsigned int max_rf;
extern const char *rf_name[];
extern const int gid_rf[];
extern const int gid_wds_link[][MAX_WDS_LINK];
extern const int gid_vap[][MAX_VAP];
extern const STRING_PAIR zone_info[];
extern const int gid_nas[]; 
extern const int gid_radius[];
extern const int gid_radius_nas[]; 
extern const int gid_radius_map[];
extern const int gid_frule[];
extern const int gid_policy_route[]; 
extern const int gid_vlan[];
extern const int gid_vlan_dhcp[];
extern const int gid_vlan_lan[];
extern const int gid_blacklist[];
extern const char *current_config;

static int strcapitalize(int eid, webs_t wp, int argc, char_t **argv) ;

#ifndef CUSTOMIZE_PATH
#define CUSTOMIZE_PATH	"/conf/customize"
#endif

static MprVarTriggerStatus cipcfg_trigger(MprVarTriggerOp op,
	MprProperties *parentProperties, MprVar *prop, MprVar *newValue, int copyRef)
{
    MprVar *mp = NULL;

    mp = espGetProperty(prop, "cipcfg", 0);
    if (mp && mprVarIsObject(mprGetVarType(mp))) {
	MprVar *vp = NULL;
	MprVarTrigger trigger = NULL;
	CIPCFG *cipcfg = NULL;
	char name[16];
	int i;

	vp = espGetProperty(mp, "trigger", 0);
	if (vp && mprGetVarType(vp) == MPR_TYPE_INT)
	    trigger = (MprVarTrigger) mprVarToInteger(vp);
	switch (op) {
	case MPR_VAR_WRITE:
	case MPR_VAR_READ:
	case MPR_VAR_CREATE_PROPERTY:
	case MPR_VAR_DELETE_PROPERTY:
	    break;
	case MPR_VAR_DELETE:
	    mprSetVarName(mp, "");
	    mprAddVarTrigger(prop, trigger);
            /* TODO: more config type support? */
	    for (i=0; i<=1; i++) {
		sprintf(name, "cfg%d", i);
		vp = espGetProperty(mp, name, 0);
		if (vp && mprGetVarType(vp) == MPR_TYPE_INT) {
		    cipcfg = (CIPCFG *) mprVarToInteger(vp);
		    if (cipcfg != NULL)
			close_cipcfg(cipcfg);
		}
	    }
	    break;
	default:
	    break;
	}
	if (trigger != NULL)
	    return trigger(op, parentProperties, prop, newValue, copyRef);
    }
    return MPR_TRIGGER_PROCEED;
}

static MprVar *init_cipcfg_trigger(EjsId eid)
{
    MprVar *gp = NULL, *mp = NULL, *vp = NULL;
    MprVarTrigger trigger = NULL;
    char *name = NULL;

    gp = ejsGetGlobalObject(eid);
    if (!gp)
	goto fail;
    name = "cipcfg";
    mp = espGetProperty(gp, name, 0);
    if (!mp || !mprVarIsObject(mprGetVarType(mp))) {
	MprVar tmp=espCreateArrayVar(name, 0);
	mp = espSetPropertyValue(gp, name, tmp);
	espDestroyVar(&tmp);
	if (!mp || !mprVarIsObject(mprGetVarType(mp)))
	    goto fail;
    }
    name = "trigger";
    vp = espGetProperty(mp, name, 0);
    if (!vp || mprGetVarType(vp) != MPR_TYPE_INT) {
	trigger = mprAddVarTrigger(gp, cipcfg_trigger);
	vp = espSetPropertyValue(mp, name, mprCreateIntegerVar((int)trigger));
	if (!vp || mprGetVarType(vp) != MPR_TYPE_INT ||
		(trigger != (MprVarTrigger) mprVarToInteger(vp))) {
	    mprAddVarTrigger(gp, trigger);
	    goto fail;
	}
	mprSetVarReadonly(vp, 1);
    }
    return mp;
fail:
    return NULL;
}

static CIPCFG *init_cipcfg_w(EjsId eid, int type)
{
    //extern const char *config_ram;
    //extern const char *config_rom;
    //extern const char *restored_cfg_file;
    //extern const char *backup_cfg_file;
    extern const char *current_config;
    MprVar *mp = NULL, *vp = NULL;
    CIPCFG *cipcfg = NULL;
    char name[16], *fname = NULL;

    /* TODO: more config type support? */
    if (type < 0 || type > 1)
	goto fail;
    switch (type) {
    case 1:
	fname = current_config;
	break;
    default:
	fname = NULL;
	break;
    }
    mp = init_cipcfg_trigger(eid);
    if (!mp)
	goto fail;
    sprintf(name, "cfg%d", type);
    vp = espGetProperty(mp, name, 0);
    if (vp && mprGetVarType(vp) == MPR_TYPE_INT)
	cipcfg = (CIPCFG *) mprVarToInteger(vp);
    if (!cipcfg) {
	cipcfg = (!fname) ? init_cipcfg() : init_cipcfg_f(fname);
	if (!cipcfg)
	    goto fail;
	vp = espSetPropertyValue(mp, name, mprCreateIntegerVar((int)cipcfg));
	if (!vp || mprGetVarType(vp) != MPR_TYPE_INT)
	    goto fail;
	if (cipcfg != (CIPCFG *) mprVarToInteger(vp)) {
	    espDeleteProperty(mp, name);
	    goto fail;
	}
	mprSetVarReadonly(vp, 1);
    }
    return cipcfg;
fail:
    if (cipcfg)
	close_cipcfg(cipcfg);
    return NULL;
}

/**
 * This is a ASP function for 
 * getting the configuration from configuration file
 * Note: The input item_name should be same as name of read_cfg display.
 *
 */
static int getCfg(EspRequest *ep, int argc, char **argv)
{
    char *group_name, *item_name, item_value[MAX_VALUE_LEN];
    CIPCFG *cipcfg_w = NULL, *cipcfg = NULL;
    int ret, newcfg = 0;

    if (ejsParseArgs(argc, argv, "%s %s %d", &group_name, &item_name, &newcfg) < 2) {
	espError(ep, "Insufficient args\n");
	return -1;
    }

    if (newcfg == 1) {
	extern const char *restored_cfg_file;
	cipcfg = init_cipcfg_f(restored_cfg_file);
    } else
	cipcfg_w = init_cipcfg_w(espGetScriptHandle(ep), 0);

    if (!cipcfg_w && !cipcfg) {
	cipcfg = init_cipcfg();
	if (!cipcfg) {
	    printf("initial cipcfg error.\n");
	    return 0;
	}
    } else if (cipcfg_w != NULL) {
	cipcfg = cipcfg_w;
    }

    ret = get_cfg_item_by_str(cipcfg, group_name, item_name, item_value);
    if (ret != CIPCFG_SUCCESS) {
	sprintf(item_value, "N/A");
	printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
    }

    if (cipcfg && !cipcfg_w)
	close_cipcfg(cipcfg);

    espSetReturnString(ep, item_value);
    return 0;
}

static int getVlanByIP(EspRequest *ep, int argc, char **argv)
{
    char *reqip;
    int vlan = 0;

    if (ejsParseArgs(argc, argv, "%s", &reqip) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    vlan = getVlanbyIP(reqip);
    if (vlan < 0)
	vlan = 0;
    espSetReturn(ep, mprCreateIntegerVar(vlan));
    return 0;
}

static int getMgmtIPListJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    int ret, max, i;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, GID_REMOTE_MANAGE, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum mapping entries.\n");
	return 0;
    }
    max = atoi(item_value);
    espWriteFmt(ep, "{\"MGMTIP\":[");
    for (i = 0; i < max; i++) {
	sprintf(item_name, "entry:%d.ip", i);
	ret = get_cfg_item(cipcfg, GID_REMOTE_MANAGE, item_name, item_value);
	ptr = value;
	ptr += sprintf(ptr, "{\"ip\":\"%s\"}", item_value);
	if (i != max - 1)
	    sprintf(ptr, ",");
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}
#ifdef GW_FUNC
static int getBlackListJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    char remark[MAX_VALUE_LEN];
    int ret, max, i, list;

    if (ejsParseArgs(argc, argv, "%d", &list) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item_by_str(cipcfg, "blacklist", "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum black list entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    if (list < 0 || list >= atoi(item_value)) { //check if blacklist number is out side of the range
	close_cipcfg(cipcfg);
	return 0;
    }
    ret = get_cfg_item(cipcfg, gid_blacklist[list], "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum black list entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    max = atoi(item_value);
    espWriteFmt(ep, "{\"LIST\":[");
    for (i = 0; i < max; i++) {
	sprintf(item_name, "entry:%d.name", i);
	ret = get_cfg_item(cipcfg, gid_blacklist[list], item_name, item_value);
	sprintf(item_name, "entry:%d.remark", i);
	ret = get_cfg_item(cipcfg, gid_blacklist[list], item_name, remark);
	ptr = value;
	ptr += sprintf(ptr, "{\"name\":\"%s\",\"remark\":\"%s\"}", item_value, remark);
	if (i != max - 1)
	    sprintf(ptr, ",");
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}
#endif
static int getPrivilegeIPJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    int page, entry_perpage, max, i, ret;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    char remark[MAX_VALUE_LEN];

    if (ejsParseArgs(argc, argv, "%d %d", &page, &entry_perpage) < 2) {
	espError(ep, "Insufficient args\n");
	return -1;
    }

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, GID_PRIVILEGEIP, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum Privilege IP entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    max = atoi(item_value);
    if (page * entry_perpage > max || page * entry_perpage <= 0) {
	close_cipcfg(cipcfg);
	return 0;
    }
    espWriteFmt(ep, "{\"PRIVILEGEIP\":[");
    for (i = (page - 1) * entry_perpage; i < page * entry_perpage; i++) {
	sprintf(item_name, "entry:%d.ip", i);
	ret = get_cfg_item(cipcfg, GID_PRIVILEGEIP, item_name, item_value);
	sprintf(item_name, "entry:%d.remark", i);
	ret = get_cfg_item(cipcfg, GID_PRIVILEGEIP, item_name, remark);
	ptr = value;
	ptr += sprintf(ptr, "{\"ip\":\"%s\",\"remark\":\"%s\"}", item_value, remark);
	if (i != page * entry_perpage - 1)
	    sprintf(ptr, ",");
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}

static int getPrivilegeMACJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    int page, entry_perpage, max, i, ret;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    char remark[MAX_VALUE_LEN];

    if (ejsParseArgs(argc, argv, "%d %d", &page, &entry_perpage) < 2) {
	espError(ep, "Insufficient args\n");
	return -1;
    }

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, GID_PRIVILEGEMAC, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum Privilege MAC entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    max = atoi(item_value);
    if (page * entry_perpage > max || page * entry_perpage <= 0) {
	close_cipcfg(cipcfg);
	return 0;
    }
    espWriteFmt(ep, "{\"PRIVILEGEMAC\":[");
    for (i = (page - 1) * entry_perpage; i < page * entry_perpage; i++) {
	sprintf(item_name, "entry:%d.mac", i);
	ret = get_cfg_item(cipcfg, GID_PRIVILEGEMAC, item_name, item_value);
	sprintf(item_name, "entry:%d.remark", i);
	ret = get_cfg_item(cipcfg, GID_PRIVILEGEMAC, item_name, remark);
	ptr = value;
	ptr += sprintf(ptr, "{\"mac\":\"%s\",\"remark\":\"%s\"}", item_value, remark);
	if (i != page * entry_perpage - 1)
	    sprintf(ptr, ",");
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}

static int getWallAdListJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    char topic[MAX_VALUE_LEN], desc[MAX_VALUE_LEN], display[MAX_VALUE_LEN];
    int ret, max, i;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum walled garden ad list entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    max = atoi(item_value);
    espWriteFmt(ep, "{\"WALLAD\":[");
    for (i = 0; i < max; i++) {
	sprintf(item_name, "entry:%d.url", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, item_value);
	sprintf(item_name, "entry:%d.topic", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, topic);
	sprintf(item_name, "entry:%d.desc", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, desc);
	sprintf(item_name, "entry:%d.display", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, display);
	ptr = value;
	ptr += sprintf(value, "{\"url\":\"%s\",\"topic\":\"%s\",\"desc\":\"%s\",\"display\":\"%s\"}", 
		item_value, topic, desc, display);
	if (i != max - 1)
	    sprintf(ptr, ",");
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}

static int getEnableWallAdListJson(EspRequest *ep, int argc, char **argv)
{
    CIPCFG *cipcfg = NULL;
    char value[BUFSIZ], item_name[32], item_value[MAX_VALUE_LEN], *ptr;
    char topic[MAX_VALUE_LEN], desc[MAX_VALUE_LEN], display[MAX_VALUE_LEN];
    int ret, max, i, multiple = 0;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initialize cipcfg error.\n");
	return 0;
    }
    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum walled garden ad list entries.\n");
	close_cipcfg(cipcfg);
	return 0;
    }
    max = atoi(item_value);
    espWriteFmt(ep, "{\"WALLAD\":[");
    for (i = 0; i < max; i++) {
	sprintf(item_name, "entry:%d.display", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, display);
	if (strcmp(display, "enable"))
	    continue;
	sprintf(item_name, "entry:%d.url", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, item_value);
	sprintf(item_name, "entry:%d.topic", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, topic);
	sprintf(item_name, "entry:%d.desc", i);
	ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, item_name, desc);
	ptr = value;
	if (multiple > 0)
	    ptr += sprintf(value, ",");
	else
	    multiple = 1;
	sprintf(ptr, "{\"url\":\"%s\",\"topic\":\"%s\",\"desc\":\"%s\"}", 
		item_value, topic, desc);
	espWriteFmt(ep, "%s", value);
    }
    espWriteFmt(ep, "]}");
    close_cipcfg(cipcfg);
    return 0;
}

static int getRcEnv(EspRequest *ep, int argc, char **argv)
{
    if (argc != 1 || !argv || !*argv || !**argv)
	goto fail;
    else {
	char *s = *argv;
	MprVar env, *vp;
	int rc;
	rc = espGetVar(ep, ESP_APPLICATION_OBJ, "rc.env", &env);
	if (rc != 0 || !mprVarIsObject(mprGetVarType(&env))) {
	    MprVar tmp = espCreateArrayVar("rc.env", 0);
	    espSetVar(ep, ESP_APPLICATION_OBJ, "rc.env", tmp);
	    espDestroyVar(&tmp);
	    rc = espGetVar(ep, ESP_APPLICATION_OBJ, "rc.env", &env);
	    if (rc != 0 || !mprVarIsObject(mprGetVarType(&env)))
		goto fail;
	}
	vp = espGetProperty(&env, s, 0);
	if (!vp || !mprVarIsString(mprGetVarType(vp))) {
	    FILE *fp;
	    char buf[MAX_VALUE_LEN];
	    if (!get_rc_env(s, buf, sizeof(buf)))
		goto fail;
	    vp = espSetPropertyValue(&env, s, mprCreateStringVar(buf, 0));
	    if (!vp || !mprVarIsString(mprGetVarType(vp)))
		goto fail;
	}
    	espSetReturn(ep, *vp);
	return 0;
    }
fail:
    espSetReturn(ep, mprCreateNullVar());
    return 0;
}

static int getFtime(EspRequest *ep, int argc, char **argv)
{
    struct timeval tp;
    double val = 0.0;

    if (gettimeofday(&tp, 0) < 0)
	goto fail;
    val = tp.tv_usec / ((double) 1000000.0);
    if (val >= 1.0) val -= (long) val;
    val += tp.tv_sec;
    espSetReturn(ep, mprCreateFloatVar(val));
    return 0;
fail:
    espSetReturn(ep, mprCreateNullVar());
    return 0;
}

static int getTime(EspRequest *ep, int argc, char **argv)
{
    struct timeval tp;
    double val = 0.0;

    if (gettimeofday(&tp, 0) < 0)
	goto fail;
    val = ((long)(tp.tv_usec / 1000)) / ((double)1000.0);
    if (val >= 1.0) val -= (long) val;
    val = (val + (double) tp.tv_sec) * ((double)1000.0);
    espSetReturn(ep, mprCreateFloatVar(val));
    return 0;
fail:
    espSetReturn(ep, mprCreateNullVar());
    return 0;
}


/*
   Get the report - 
   function and struct are defined at agent.h
*/

#ifdef SUPPORT_REPORT_AGENT
void getReport(int eid, webs_t wp, int argc, char_t **argv)
{
    struct repreq req;
    char_t *type; 
    int output_len;
    static int i;

    if (ejArgs(argc, argv, T("%s"), &type) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    
    // call getReportType to get the report type.
    req.rep_type=getReportType(type);

    req.rep_format = REP_JSON_FORMAT;
    sprintf(req.rep_uid, "caller");

    req.rep_method = REP_METHOD_PASSIVE;

    report_agent_get(&req);

    output_len=strlen(req.rep_output);

    for(i=0;i < output_len; i+=WEBS_MAX_BUFSIZE)
    {
        websWrite(wp, T("%.900s"), &req.rep_output[i]);
    }
    free(req.rep_output);

    return;
}
#endif

/*  for HTML Special Characters Encoding.
 *  Special Characters like "<", ">" and "&"
 */
void htmlSpecialCharEncode(char *src, char *dst)
{
    size_t dlen = 0;
    if (!src || !dst)
	return;
    while (*src) {
        switch (*src) {
            case '<' : dlen += sprintf(dst + dlen, "&lt;"); break;
            case '>' : dlen += sprintf(dst + dlen, "&gt;"); break;
            case '"' : dlen += sprintf(dst + dlen, "&quot;"); break;
            case '&' : dlen += sprintf(dst + dlen, "&amp;"); break;
            default  : dst[dlen++] = *src;
        }
        src++;
    }
    dst[dlen++] = '\0';
    return;
}

/*
 * This funcion will put escape char "\" in front of special chars : "\", """, "'"
 * to prevent the string variable cause error in JavaScript area.
 */
void javascriptEscapeSequence(char *src, char *dst)
{
    size_t dlen = 0;
    if (!src || !dst)
	return;
    while (*src) {
        switch (*src) {
            case '\\':
            case '\'':
            case '"' :
                dlen += sprintf(dst + dlen, "\\"); 
                dst[dlen++] = *src;
                break;
            //case '\0': dst[dlen++] = *src; return;
            default  : dst[dlen++] = *src;
        }
        src++;
    }
    dst[dlen++] = '\0';
    return;
}

void jasonStringEscapeSequence(char *src, char *dst)
{
    size_t dlen = 0;
    if (!src || !dst)
	return;
    while (*src) {
        switch (*src) {
            case '\\':
                dlen += sprintf(dst + dlen, "\\"); 
            case '"' :
                dlen += sprintf(dst + dlen, "\\"); 
            case '\'':
                dlen += sprintf(dst + dlen, "\\"); 
                dst[dlen++] = *src;
                break;
            //case '\0': dst[dlen++] = *src; return;
            default  : dst[dlen++] = *src;
        }
        src++;
    }
    dst[dlen++] = '\0';
    return;
}

int rfcards_w(EjsId eid, char *func, char *value)
{
    MprVar *gp = NULL, *mp = NULL, *vp = NULL;
    char *name = NULL;
    int ret = 0;
    if (!value)
	return -1;
    gp = ejsGetGlobalObject(eid);
    if (!gp)
	goto fail;
    name = "rfcards";
    mp = espGetProperty(gp, name, 0);
    if (!mp || !mprVarIsObject(mprGetVarType(mp))) {
	MprVar tmp=espCreateArrayVar(name, 0);
	mp = espSetPropertyValue(gp, name, tmp);
	espDestroyVar(&tmp);
	if (!mp || !mprVarIsObject(mprGetVarType(mp)))
	    goto fail;
    }
    if (!func) {
	func = "";
	name = "/";
    } else {
	name = (!*func) ? "." : func;
    }
    vp = espGetProperty(mp, name, 0);
    if (!vp || !mprVarIsString(mprGetVarType(vp))) {
	char cards[max_rf+1];
	ret = rfcards(func, value);
	strncpy(cards, value, max_rf);
	cards[max_rf] = '\0';
	vp = espSetPropertyValue(mp, name, mprCreateStringVar(cards, 0));
	if (!vp || !mprVarIsString(mprGetVarType(vp)))
	    espDeleteProperty(mp, name);
    } else {
	mprVarToString(&name, MPR_MAX_STRING, 0, vp);
	if (!name)
	    goto fail;
	strncpy(value, name, max_rf);
	mprFree(name);
    }
    return ret;
fail:
    if (!func)
	func = "";
    ret = rfcards(func, value);
    return ret;
}

/**
 * This is a ASP function for 
 * getting parameters from the html form
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getValue(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *item_name, item_value[MAX_VALUE_LEN];

    if (ejArgs(argc, argv, T("%s"), &item_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    
    if (!strcmp(item_name, "cur_cate")) {
        strcpy(item_value, websGetVar(wp, "selected", "status"));
    }
    else if (!strcmp(item_name, "cur_wds_rf")) {
        strcpy(item_value, websGetVar(wp, "rf", "0"));
        if(!strncmp("0", item_value, 1)){
            int i;
            char cards[max_rf];
            rfcards_w((EjsId)eid, "wds", cards );
            for ( i=1; i <= max_rf; i++ ){
                if ( '1' == cards[i-1] ){
                    sprintf(item_value, "%d", i);
                    break;
                }
            }
        }
    }
    else if (!strcmp(item_name, "cur_wds_link")) {
        strcpy(item_value, websGetVar(wp, "wds", "1"));
    }
    else if (!strcmp(item_name, "cur_vap")) {
        strcpy(item_value, websGetVar(wp, "vap", "0"));
    }
    else if (!strcmp(item_name, "cur_vlan")) {
        strcpy(item_value, websGetVar(wp, "vlan", "0"));
    }
    else if (!strcmp(item_name, "cur_policy")) {
        strcpy(item_value, websGetVar(wp, "policy", "0"));
    }
    else if (!strcmp(item_name, "cur_radius")) {
        strcpy(item_value, websGetVar(wp, "radius", "0"));
    }
    else if (!strcmp(item_name, "search_item")) {
        strcpy(item_value, websGetVar(wp, "search", "0"));
    }
    else if (!strcmp(item_name, "cur_scan")) {
        strcpy(item_value, websGetVar(wp, "now", "0"));
    }
    else if (!strcmp(item_name, "err_code")) {
        strcpy(item_value, websGetVar(wp, "err_code", "0"));
    }
    else if (!strcmp(item_name, "errMsg")) {
       strcpy(item_value, websGetVar(wp, "errMsg", "0"));
    }
    else if (!strcmp(item_name, "uname")) {
       strcpy(item_value, websGetVar(wp, "uname", "0"));
    }
    else if (!strcmp(item_name, "session")) {
       strcpy(item_value, websGetVar(wp, "session", "0"));
    }
    else if (!strcmp(item_name, "ReturnUrl")) {
       strcpy(item_value, websGetVar(wp, "ReturnUrl", "0"));
    }
    else if (!strcmp(item_name, "curr_bill")) {
        strcpy(item_value, websGetVar(wp, "curr_bill", "0"));
    }
    else if(!strcmp(item_name, "umac")) {
        strcpy(item_value, websGetVar(wp, "umac", "0"));
    }
    else if(!strcmp(item_name, "timeStr")) {
        strcpy(item_value, websGetVar(wp, "timeStr", "00:00"));
    }
    else if (!strcmp(item_name, "reboot")) {
        strcpy(item_value, websGetVar(wp, "reboot", "0"));
    }
    else if (!strcmp(item_name, "cur_rule")) {
	strcpy(item_value, websGetVar(wp, "rule", "1"));
    }
    else if (!strcmp(item_name, "insert_rule")) {
	strcpy(item_value, websGetVar(wp, "insert", "-1"));
    }
    else
        item_value[0] = '\0';

    ejSetResult(eid, item_value);
    return 0;
}


int getUpperCase(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *item_name, item_value[MAX_VALUE_LEN];
    if (ejArgs(argc, argv, T("%s"), &item_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    int i=0,len=0;
    len = strlen(item_name);
    for (i ; i < len ;i ++)
    {
	if((*(item_name+i) >= 97) && (*(item_name+i) <= 122))
	    item_value[i]=*(item_name+i) - ' ';
	else
	    item_value[i]=*(item_name+i);
    }
    item_value[len]='\0';

    ejSetResult(eid, item_value);
    return 0;
					    
}




/**
 * This is a ASP function for 
 * getting the imformation from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getStatus(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *item_name, item_value[MAX_VALUE_LEN];
    int ret;
    if (ejArgs(argc, argv, T("%s"), &item_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    
    memset(item_value, 0, sizeof(item_value));

    ret = get_sys_item(item_name, item_value);
    if (ret != CIPCFG_SUCCESS) {
        item_value[0]='\0';
    }
    ejSetResult(eid, item_value);
    return 0;
}

/**
 * This is a ASP function for 
 * getting the configuration from configuration file
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  ep:
 * @param  query:
 * @resturn item_name:
 */
static int getCurrentCfg(EspRequest *ep, int argc, char **argv)
{
    extern const char *current_config;
    char *group_name, *item_name, item_value[MAX_VALUE_LEN], *fullname;
    CIPCFG *cipcfg_w = NULL, *cipcfg = NULL;
    int ret;

    if (ejsParseArgs(argc, argv, T("%s"), &fullname) < 1) {
	//websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    
    /* Seperate fullname to group_name and item_name */
    if((item_name = strchr( fullname, '.')) == NULL) {
	//websError(wp, 400, T("Invalid arg"));
        return -1;
    }

    cipcfg_w = init_cipcfg_w(espGetScriptHandle(ep), 1);
    if (!cipcfg_w) {
	cipcfg = init_cipcfg_f(current_config);
	if (!cipcfg) {
	    printf("initial cipcfg error.\n");
	    return 0;
	}
    } else {
	cipcfg = cipcfg_w;
    }

    *item_name = '\0';
    item_name++;

    group_name = fullname;
    if (get_cfg_item_by_str(cipcfg, group_name, item_name, item_value) != CIPCFG_SUCCESS) {
        sprintf(item_value, "N/A");
        printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
    }

    if (cipcfg && !cipcfg_w)
	close_cipcfg(cipcfg);

    ejsSetReturnValue(ep->eid, mprCreateStringVar(item_value, 0));
    return 0;
}

static int strcapitalize(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *str, result[MAX_VALUE_LEN+1] ;

    if (ejArgs(argc, argv, T("%s"), &str) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    result[MAX_VALUE_LEN] = '\0' ;
    strncpy(result, str, MAX_VALUE_LEN) ;
    result[0] = toupper(result[0]) ;

    ejSetResult(eid, result);

    return 0;
}

int getCustomizeString(const char *name, const char *defval, 
	char *value, int maxlen)
{
    FILE *fp = NULL;
    char filename[BUFSIZ];

    bzero(value, maxlen);
    sprintf(filename, "%s/%s.txt", CUSTOMIZE_PATH, name);
    fp = fopen(filename, "r");
    if (fp == NULL) {
	/* Using defaul value */
	printf("Can't get customized string for \"%s\".\n", name);
	strncpy(value, defval, maxlen);
    } else {
	fgets(value, maxlen - 1, fp);
	fclose(fp);
    }

    return 0;
}

int getCustomizeCss(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name, *attribute, item_value[MAX_VALUE_LEN];
    char_t tmp[BUFSIZ];
    FILE *fp = NULL;
    char filename[BUFSIZ];

    if (ejArgs(argc, argv, T("%s %s"), &name, &attribute) < 2) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    bzero(item_value, MAX_VALUE_LEN);
    sprintf(filename, "%s/%s.txt", CUSTOMIZE_PATH, name);
    fp = fopen(filename, "r");
    if (fp) {
	fgets(item_value, MAX_VALUE_LEN - 1, fp);
	fclose(fp);
	sprintf(tmp, "\t%s: %s;\n", attribute, item_value);
	websWrite(wp,T(tmp));
    }

    return 0;
}

#ifdef CAPWAP
int getACList(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char item_value[MAX_VALUE_LEN], str[256];
    char prio[MAX_VALUE_LEN], name[MAX_VALUE_LEN], addr[MAX_VALUE_LEN];
    int i, tbl_size, isFirst = 1, num;

    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "capwap", "aclist.table_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp, T("["));
    for (i = 0; i < tbl_size; i++) {
	sprintf(str, "aclist.ac_entry:%d.priority", i);
	get_cfg_item_by_str(cipcfg, "capwap", str, prio);
	sprintf(str, "aclist.ac_entry:%d.name", i);
	get_cfg_item_by_str(cipcfg, "capwap", str, name);
	sprintf(str, "aclist.ac_entry:%d.ip", i);
	get_cfg_item_by_str(cipcfg, "capwap", str, addr);
	if (!isFirst)
	    websWrite(wp, T(","));
	num = sprintf(str, "{\"priority\":%s, \"name\":\"%s\", \"ip\":\"%s\"}",
		prio, name, addr);
	if (num > 0) {
	    websWrite(wp, T(str));
	    isFirst = 0;
	} else 
	    continue;
    }
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}

int getWtpStatus(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t state[MAX_VALUE_LEN];
    char value[BUFSIZ], wtp_state[BUFSIZ], ac_addr[BUFSIZ];
    CIPCFG *cipcfg = NULL;
    int ret, i, len;
    FILE *fp = NULL;

    cipcfg=init_cipcfg_f(current_config);
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    ret = get_cfg_item(cipcfg, GID_CAPWAP, "state", state);
    close_cipcfg(cipcfg);
    if (ret != CIPCFG_SUCCESS || strcmp(state, "disable") == 0) {
	websWrite(wp,T("Disabled"));
	return 0;
    }

    if ((fp = popen(PATH_CHECK_WTP, "r")) == NULL) {
	websWrite(wp,T("Disabled"));
	return 0;
    }
    if (fgets(value, BUFSIZ, fp)) {
	if (sscanf(value, "%s %s", wtp_state, ac_addr) == 2) {
	    len = strlen(wtp_state);
	    if (!strcmp(wtp_state, "SULKING") || !strcmp(wtp_state, "RESET") ||
		    !strcmp(wtp_state, "QUIT") || !strcmp(wtp_state, "REBOOT")) {
		for (i = 1; i < len; i++) 
		    wtp_state[i] = tolower(wtp_state[i]);
		websWrite(wp, T(wtp_state));
	    } else {
		char output[BUFSIZ];
		for (i = 1; i < len; i++) 
		    wtp_state[i] = tolower(wtp_state[i]);
		sprintf(output, "%s (%s)", wtp_state, ac_addr);
		websWrite(wp, T(output));
	    }
	} else {
	    websWrite(wp,T("Disabled"));
	}
    } else {
	websWrite(wp,T("Disabled"));
    }
    pclose(fp);
    return 0;
}

int getTunStatus(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t state[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    int s, ret;
    struct ifreq ifr;

    cipcfg=init_cipcfg_f(current_config);
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    ret = get_cfg_item(cipcfg, GID_CAPWAP, "state", state);
    close_cipcfg(cipcfg);
    if (ret != CIPCFG_SUCCESS || strcmp(state, "disable") == 0) {
	websWrite(wp,T("Disabled"));
	return 0;
    }

    memset(&ifr, 0, sizeof(ifr));
    s = socket(PF_INET, SOCK_DGRAM, 0);
    strcpy(ifr.ifr_name, "udpETH");
    if (ioctl(s, SIOCGIFFLAGS, &ifr) < 0) {
	websWrite(wp,T("Disabled"));
	close(s);
	return 0;
    }

    if (ifr.ifr_flags & IFF_RUNNING)
	websWrite(wp, T("Active"));
    else 
	websWrite(wp, T("Disconnected"));

    close(s);
    return 0;
}
#endif /* CAPWAP */

#ifdef WITH_ACC_MGMT
int getOdUserJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *macAddr;
    char value[BUFSIZ];
    CUR_USER *current_user_list = NULL;

    if (ejArgs(argc, argv, T("%s"), &macAddr) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }

    websWrite(wp,T("{\"USER\":["));
    current_user_list = get_all_cur_user(macAddr);

    sprintf(value, "{\"name\":\"%s\", \"ip\":\"%s\", \"utype\":\"%s\", \"mac\":\"%s\", \"sessionlen\":\"%lu\", \"byteamount\":\"%Lu\", \"btype\":\"%s\", \"logintime\":\"%lu\"}",current_user_list->name, current_user_list->ip, current_user_list->user_type, current_user_list->mac, current_user_list->sessionlength, current_user_list->byteamount, current_user_list->charge_type,current_user_list->login_time);
   websWrite(wp,T(value));
   free_cur_user_list(current_user_list);
   websWrite(wp,T("]}"));

   return 0;
}
#endif

/**
 * This is a ASP function for 
 * getting the WDS info from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getAPListJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t secret[MAX_VALUE_LEN], c_ch[MAX_VALUE_LEN], 
           c_mode[MAX_VALUE_LEN], item[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    AP_LIST *ap_list = NULL;
    enum { EMPTY, NOT_EMPTY }; 
    enum { OTHER, BUDDY };
    char rf_wds[max_rf], tmp[1024], mac[17];
    int ch, mode, rf, ret, i, j, count=1 , p = OTHER ;

    if (ejArgs(argc, argv, T("%d"), &rf) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    if ( rf == 0 ){
        websWrite(wp,T("&nbsp;"));
        return 0;
    }
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }
 
    ret = get_cfg_item(cipcfg, gid_rf[rf-1], "wds_discover_secret", secret);
    ret = get_cfg_item(cipcfg, gid_rf[rf-1], "channel", c_ch);
    ret = get_cfg_item(cipcfg, gid_rf[rf-1], "mode", c_mode);
    
    if (ret != CIPCFG_SUCCESS) {
        secret[0]='\0';
    }
    if ( !strncmp(c_ch, "auto", 4) ){
        ch = 0;
    }else{
        ch = atoi(c_ch);
    }
    
    if ( !strncmp(c_mode, "auto", 4) ){
        mode = AUTO ;
    }else if ( !strncmp(c_mode, "11g", 3) ){
        mode = _802_11g ;
    }else if ( !strncmp(c_mode, "11b", 3) ){
        mode = _802_11b ;
    }else if ( !strncmp(c_mode, "11a", 3) ){
        mode = _802_11a ;
    }

    printf("\nStart Searching WDS Peer......\n") ;
    printf("rf=%d, secret=%s, ch=%d, mode=%d \n", rf, secret, ch, mode );
    ap_list=wdsd_scan(rf-1, secret, ch, mode);
    if(ap_list == NULL){
        printf("ap_list is NULL\n") ;
    }
    print_aps(ap_list);

    rfcards_w((EjsId)eid, "wds", rf_wds);
    websWrite(wp,T("["));
    for ( ; ap_list != NULL ; ap_list = ap_list->next ){
        for (i=0; i < max_rf; i++){
            // compare with own RF cards
            sprintf( item, "rf%d_mac", i+1 ); 
            ret = get_sys_item( item, mac);
            if( !strncasecmp(ap_list->mac, mac, 17) ){
                goto next_item ;
            }
            // compare with exist wds link list
            if ( '1' == rf_wds[i] ){
                for (j=0; j < MAX_WDS_LINK; j++){
                    ret = get_cfg_item(cipcfg, gid_wds_link[i][j], "mac" , mac);
                    if( !strncasecmp(ap_list->mac, mac, 17) ){
                        sprintf(tmp, "{\"stat\":\"Buddy\",\"mac\":\"%s\",\"ch\":%d,\"snr\":%d}", ap_list->mac, ap_list->ch, ap_list->signal );
                        goto next_item ;
                    }
                }
            }
        }
	if(count > 1)
		websWrite(wp,T(","));
        sprintf(tmp, "{\"stat\":%d,\"mac\":\"%s\",\"ch\":%d,\"snr\":%d}", rf, ap_list->mac, ap_list->ch, ap_list->signal );
	count ++;
        websWrite(wp, T(tmp));
next_item : // do nothing and goto ap_list->next
	; //XXX: Do *NOT* remove this semicolon for compiler compatible purpose.
    }

    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    free_ap_list(ap_list);
    return 0;
}


/**
 * This is a ASP function for 
 * getting the peer AP info from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getScanResult(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    AP_LIST *ap_list = NULL;
    char *rf, tmp[1024], t[MAX_VALUE_LEN], mac[17], iden[32];
    char rf_tmp[max_rf], rf_fun[max_rf];
    char vap_name[MAX_VALUE_LEN], bg[MAX_VALUE_LEN];
    int now, count=0, ret, i, j;
/*
    int vap_gid[][MAX_VAP] = 
                { {GID_RF1_VAP1, GID_RF1_VAP2, GID_RF1_VAP3, GID_RF1_VAP4}, 
                  {GID_RF2_VAP1, GID_RF2_VAP2, GID_RF2_VAP3, GID_RF2_VAP4},
                  {GID_RF3_VAP1, GID_RF3_VAP2, GID_RF3_VAP3, GID_RF3_VAP4},
                  {GID_RF4_VAP1, GID_RF4_VAP2, GID_RF4_VAP3, GID_RF4_VAP4}
    };
    int wds_gid[][MAX_WDS_LINK] =
                { {GID_RF1_WDS1, GID_RF1_WDS2, GID_RF1_WDS3, GID_RF1_WDS4}, 
                  {GID_RF2_WDS1, GID_RF2_WDS2, GID_RF2_WDS3, GID_RF2_WDS4},
                  {GID_RF3_WDS1, GID_RF3_WDS2, GID_RF3_WDS3, GID_RF3_WDS4},
                  {GID_RF4_WDS1, GID_RF4_WDS2, GID_RF4_WDS3, GID_RF4_WDS4}
    };
*/
    if (ejArgs(argc, argv, T("%s %d"), &rf, &now) < 2) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    
    if ( now == 0 ){ 
        printf("Get Last Scan Result.\n") ; 
    }else{ 
        printf("Start Scanning AP Now......\n") ; 
    }
    
    memset(t, '\0', MAX_VALUE_LEN);
    if ( !strncmp(rf, "scan", 4) ){
        ap_list=rf_scan( 3, now, t);
    }
    if(ap_list == NULL)
        printf("ap_list is NULL\n") ;
    print_aps(ap_list);

    // initial cipcfg
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    memset(rf_fun, 0, max_rf);
    // initial WDS link list
    memset(rf_fun, 0, max_rf);
    rfcards_w((EjsId)eid, "wds", rf_tmp);
    for (i=0; i < max_rf; i++){
        if ( '1' == rf_tmp[i] ){
            rf_fun[i] = 'W';
        }
    }
    // initial VAP list
    rfcards_w((EjsId)eid, "ap", rf_tmp);
    for (i=0; i < max_rf; i++){
        if ( '1' == rf_tmp[i] ){
            rf_fun[i] = 'A';
        }
    }

    sprintf(tmp, "{\"time\":\"%s\",\"list\":[", t);
    websWrite(wp, T(tmp));
    if (ap_list != NULL){
        for ( i=0; ap_list != NULL ; ap_list = ap_list->next, i++ ){
            memset(iden, 0, 32);
            if (i > 0)
                websWrite(wp,T(","));
            for (i=0; i < max_rf; i++){
                // compare with VAP
                if ( 'A' == rf_fun[i] ){
                    for (j=0; j < MAX_VAP; j++){
                        sprintf( vap_name, "vap_%d_mac", j+1 ); 
                        ret = get_sys_item( vap_name, mac);
                        if( !strncasecmp(ap_list->mac, mac, 17) ){
                            strncpy( iden, "Own VAP: ", 10 );
                            ret = get_cfg_item(cipcfg, gid_vap[i][j], "name", vap_name );
                            strncat( iden, vap_name, strlen(vap_name) );
                            goto show_page ;
                        }
                    }
                }else if ( 'W' == rf_fun[i] ){ // compare with exist wds link list
                    for (j=0; j < MAX_WDS_LINK; j++){
                        ret = get_cfg_item(cipcfg, gid_wds_link[i][j] , "mac" , mac);
                        if( !strncasecmp(ap_list->mac, mac, 17) ){
                            strncpy( iden, "Buddy", 6 );
                            goto show_page ;
                        }
                    }
                }
                // compare with own RF cards
                sprintf( t, "rf%d_mac", i+1 ); 
                ret = get_sys_item( t, mac);
                if( !strncasecmp(ap_list->mac, mac, 17) ){
                    strncpy( iden, "Own: ", 6 );
                    strncat( iden, rf_name[i], strlen(rf_name[i]) );
                    goto show_page ;
                }
            }
show_page:
            sprintf(tmp, "{\"essid\":\"%s\",\"mac\":\"%s\",\"ch\":\"%d\",\"snr\":\"%d\",\"rate\":\"%d\",\"iden\":\"%s\"}", ap_list->ssid, ap_list->mac, ap_list->ch, ap_list->signal, ap_list->rate, iden);
            websWrite(wp, T(tmp));
        }
    }
    websWrite(wp,T("]}"));
    close_cipcfg(cipcfg);
    free_ap_list(ap_list);
    return 0;
}

/**
 * This is a ASP function for 
 * getting the information from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getFunc(int eid, webs_t wp, int argc, char_t **argv)
{
    int i, rf;
    // int gid[] = { GID_RF1, GID_RF2, GID_RF3, GID_RF4 };
    char role[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    if (ejArgs(argc, argv, T("%d"), &rf) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    cipcfg = init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }
 
    get_cfg_item(cipcfg, gid_rf[rf-1], "roles", role);

    if (!strcmp(role, "wds,wdsd") || !strcmp(role, "wdsd,wds")) {
        strcpy(role, "wds");
    }

    for(i=0; role[i]!='\0'; i++){
        role[i] = toupper(role[i]);
    }

    websWrite(wp, T(role));
    close_cipcfg(cipcfg);
    return 0;
}

/**
 * This is a ASP function for 
 * getting the WDS candidate peer info from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getCandidate(int eid, webs_t wp, int argc, char_t **argv)
{
    AP_LIST *ap_list = NULL;
    char tmp[1024];
    int rf;
    int count=0;

    if (ejArgs(argc, argv, T("%d"), &rf) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    
    printf("Get Candidate Peer...\n") ; 
    ap_list=wdsd_connected(rf);
    if(ap_list == NULL)
    printf("No Candidate Peer Exist.\n") ;
    print_aps(ap_list);

    if (ap_list != NULL){
        websWrite(wp, T("<table align=\"center\" class=\"list_table\" width=\"80%%\">\n \
                <tr class=\"tbl_title\">\n  \
                    <td>Item</td>\n     \
                    <td>Mac Address</td>\n  \
                    <td>SNR (dB)</td>\n   \
                    <td>Security</td>\n   \
                    <td><input type=\"button\" value=\"Connect\"></td>\n  \
                </tr>\n \
                <tr class=\"tbl_value\">\n  \
                "));
        while( ap_list != NULL) {
            count++;
            sprintf(tmp, "<td>%d</td>\n \
                    <td>%s</td>\n \
                    <td>%d</td>\n \
                    <td>%s</td>\n \
                    <td><input type=\"checkbox\" name=\"wds%d\" value=\"%s\" ></td>\n \
                </tr>\n<tr>\n", count, ap_list->mac, ap_list->signal, ap_list->security, count, ap_list->mac);
            websWrite(wp, T(tmp));
            ap_list = ap_list->next;
        }
        websWrite(wp,T("</tr></table>\n"));
    }else{
        websWrite(wp,T("<!-- No Candidate Peer Exist. -->\n"));
    }
    free_ap_list(ap_list);
    return 0;
}


/**
 * This is a ASP function for 
 * checking if WDS function is enable on this RF card.
 * Note: The input item_name should be same as name of read_cfg display.
 */
int getRF(int eid, webs_t wp, int argc, char_t **argv)
{
    int rf;
    if (ejArgs(argc, argv, T("%d"), &rf) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    ejSetResult(eid, rf_name[rf-1]);
    return 0;
}


/**
 * This is a ASP function for 
 * get the index number of all WDS links.
 */
int getWDSidx(int eid, webs_t wp, int argc, char_t **argv)
{
    int i, rf, wds, idx = -1 ;
    char value[2], cards[max_rf];
    if (ejArgs(argc, argv, T("%d %d"), &rf, &wds) < 2) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    printf ("rf=%d, wds=%d\n", rf, wds);
    rfcards_w((EjsId)eid, "wds", cards );
    if ( rf==0 ){
        idx = 0;
    }else{
        for ( i=1; i <= max_rf; i++ ){
            if ( '1' == cards[i-1] ){
                if ( rf!=i ){
                    idx = idx + MAX_WDS_LINK ;
                }else{
                    idx = idx + wds ; 
                    break;
                }          
            }
        }
    }
    memset(value, '\0', 2);
    sprintf(value, "%d", idx);
    printf("idx= %c%c\n", value[0], value[1]);
    websWrite(wp,T(value));
    return 0;
}

/*
 *   check if any function (ex. AP, WDS, SCAN ) supported
 */
static int ifFunc(EspRequest *ep, int argc, char **argv)
{
    int ret, rf = 0, c = 0;
    char *func = "", cards[max_rf+1], tmp[MAX_VALUE_LEN];
    memset(tmp, 0, MAX_VALUE_LEN);
    CIPCFG *cipcfg = NULL;

    cards[max_rf] = 0;
    if (argc > 0 && argv && argv[0]) {
        func = argv[0];
        if (argc > 1 && argv[1])
            rf = atoi(argv[1]);
    }
    if (!strncmp(func, "WIRELESS", 8)){
#ifdef TOP_IS_WIRELESS
	/* Use under_wireless() to decide if this webpage is under wireless category.
	 * It is implemented in the extension.*/
	if (1 == under_wireless())
	    c = 1;
	else 
#endif
	if ( 0 != get_product_info("PRODUCT", tmp) ){
	    return 0;
	}
        if (!strncmp("WCB1000", tmp, 7)) {
            c = 1;
        }
    }else if(!strncmp(func, "FIREWALL", 8)){

        cipcfg = init_cipcfg();
        if(!cipcfg){
            printf("initial cipcfg error.\n");
            return 0;
        }
        ret = get_cfg_item(cipcfg, GID_OPMODE, "type", tmp);
        if ((ret == CIPCFG_SUCCESS)&&(!strncmp(tmp, "CPE", 3))) {
            c = 1;
        }
        close_cipcfg(cipcfg);

#ifdef WITH_ACC_MGMT
    }else if(!strncmp(func, "USER", 4)){

        cipcfg = init_cipcfg();
        ret = get_cfg_item(cipcfg, GID_OPMODE, "type", tmp);

        /* Show user button only when GW mode */
        if((ret == CIPCFG_SUCCESS) && (!strncmp(tmp, "GW", 2))) {
            c = 1;
        }
        close_cipcfg(cipcfg);
#endif
    }else{
        rfcards_w(espGetScriptHandle(ep), func, cards);
        if ( max_rf < rf ){
            // illegal access
        }else if ( 1 > rf ){ // NOT specify rf card
            if ( strchr(cards, '1') ) {
                c = 1;
            }
        }else{
            if ( '1' == cards[rf-1] ){
                c = 1;
            }
        }
    }
    ejsSetReturnValue(ep->eid, mprCreateIntegerVar(c));    
    return 0;
}

/**
 * This is a ASP function for 
 * getting the RF info from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 */
int getRFList(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t item[MAX_VALUE_LEN], mac[MAX_VALUE_LEN], band[MAX_VALUE_LEN], ch[MAX_VALUE_LEN], txp[MAX_VALUE_LEN];
    char cards[max_rf], buf[1024];
    int i, ret;

    rfcards_w((EjsId)eid, "ALL", cards);

    websWrite(wp,T("["));
    for( i=0; i<max_rf; i++){
        if( '0' != cards[i] ){
            *mac = *band = *ch = *txp = '\0';
            sprintf( item, "rf%d_mac", i+1 ); 
            ret = get_sys_item( item, mac);
            sprintf( item, "rf%d_band", i+1 ); 
            ret = get_sys_item( item, band);
            sprintf( item, "rf%d_ch", i+1 ); 
            ret = get_sys_item( item, ch);
            sprintf( item, "rf%d_txpwr", i+1 ); 
            ret = get_sys_item( item, txp);

            if(i > 0)
                websWrite(wp,T(","));
            sprintf(buf, "{\"rf_name\":\"%s\",\"mac\":\"%s\",\"band\":\"%s\",\"ch\":\"%s\",\"txp\":\"%s\"}",
                            rf_name[i], mac, band, ch, txp );
            websWrite(wp,T(buf));
        }
    }
    websWrite(wp,T("]"));
    return 0;
}


/**
 * This is a ASP function for 
 * getting the RF info of WDS function
 * Note: The input item_name should be same as name of read_cfg display.
 */
int getRFofWDS(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *c_rf, value[MAX_VALUE_LEN];
    char *attrs[] = { "state",       "mode",       "rate",
                    "channel",      "txpwr",   "distance",
                   "slottime", "acktimeout", "ctstimeout", NULL };
    char buf[1024];
    int rf, i, ret;
    CIPCFG *cipcfg = NULL;

    // rf range : 1 ~ 4
    c_rf = websGetVar(wp, T("rf"), T("0"));
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    if (!strncmp(c_rf, "0", 1)) { 
        // Can't get RF number from web client
        char cards[max_rf];
        rfcards_w((EjsId)eid, "wds", cards);
        for( i=0; i<max_rf; i++){
            if( '0' != cards[i] ){
                rf = i;
                break;
            }
        }
    }else{
        rf = atoi(c_rf);
        rf--;
    }

    websWrite(wp,T("{"));
    for ( i=0; attrs[i]; i++){
        if( i > 0 )
            websWrite(wp,T(","));
        ret = get_cfg_item(cipcfg, gid_rf[rf], attrs[i], value);
        sprintf(buf, "\"%s\":\"%s\"", attrs[i], value);
        websWrite(wp,T(buf));
    }
    websWrite(wp,T("}"));
    close_cipcfg(cipcfg);
    return 0;
}

/** 
 * @brief This function is used to get WDS attributes.
 * 
 * @param eid 
 * @param wp 
 * @param argc 
 * @param *argv 
 * 
 * @return 
 */
int getWDS(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *group_name, *item_name, item_value[MAX_VALUE_LEN];
    int rf_idx, link_idx, attr;

    if (ejArgs(argc, argv, T("%s %s"), &group_name, &item_name) < 2) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    if (sscanf(group_name, "rf%d_wds%d", &rf_idx, &link_idx) < 2) {
	printf("Wrong gid:'%s' for WDS links\n", group_name);
	goto fail;
    }
    /* index offset is 1 */
    rf_idx--;
    link_idx--;

    if (strcmp(item_name, "state") == 0) {
	attr = WDSLINK_STATE;
    } else if (strcmp(item_name, "rate") == 0) {
	attr = WDSLINK_RATE;
    } else if (strcmp(item_name, "quality") == 0) {
	attr = WDSLINK_QUALITY;
    } else if (strcmp(item_name, "health") == 0) {
	attr = WDSLINK_HEALTH;
    } else if (strcmp(item_name, "tx_total") == 0) {
	attr = WDSLINK_TXTOTAL;
    } else if (strcmp(item_name, "tx_fail") == 0) {
	attr = WDSLINK_TXFAIL;
    } else {
	printf("Wrong item name:'%s' for WDS links\n", item_name);
	goto fail;
    }

    if (getWDSLinkAttr(rf_idx, link_idx, attr, item_value) == 0)
	goto fail;
    goto success;

fail:
    sprintf(item_value, "N/A");
success:
    ejSetResult(eid, item_value);
    return 0;
}


/**
 * This is a ASP function for 
 * getting the imformation from the platform
 * Note: The input item_name should be same as name of read_cfg display.
 */
int getPKGinfo(int eid, webs_t wp, int argc, char_t **argv)
{
    int ret, f_size = 2048;
    char_t value[f_size];

    memset(value, 0, f_size);
    ret = pkg_info(f_size, value);
    if (ret != CIPCFG_SUCCESS) {
        value[0]='\0';
    }
    websWrite(wp,T(value));
    return 0;
}

int getCalibrationInfo(int eid, webs_t wp, int argc, char_t **argv)
{
    unsigned int flash_cck, flash_ofdm, cur_cck, cur_ofdm;
    FILE *fp;
    char buf[BUFSIZ], output[BUFSIZ], *pos;

#define RTL8186_HW_CONF	"/sbin/rtl8186_hw_config"
    if (access(RTL8186_HW_CONF, X_OK) != 0) {
	printf("Can't find %s.\n", RTL8186_HW_CONF);
	goto fail;
    }

    if ((fp = popen(RTL8186_HW_CONF " txPowerCCK", "r")) == NULL) {
	printf("Can't get CCK value from flash.\n");
	goto fail;
    }
    fgets(buf, 3, fp);
    buf[2] = '\0';
    pclose(fp);
    sscanf(buf, "%02x", &flash_cck);

    if ((fp = popen(RTL8186_HW_CONF " txPowerOFDM", "r")) == NULL) {
	printf("Can't get OFDM value from flash.\n");
	goto fail;
    }
    fgets(buf, 3, fp);
    buf[2] = '\0';
    pclose(fp);
    sscanf(buf, "%02x", &flash_ofdm);

    if ((fp = popen(PATH_IWPRIV " wlan0 get_mib TxPowerCCK", "r")) == NULL) {
	printf("Can't get CCK value from current state.\n");
	goto fail;
    }
    fgets(buf, BUFSIZ, fp);
    buf[BUFSIZ-1] = '\0';
    pclose(fp);
    if ((pos = strstr(buf, "get_mib:")) == NULL) {
	printf("Can't find CCK value from iwpriv coomand.\n");
	goto fail;
    }
    cur_cck = atoi(pos+8);

    if ((fp = popen(PATH_IWPRIV " wlan0 get_mib TxPowerOFDM", "r")) == NULL) {
	printf("Can't get OFDM value from current state.\n");
	goto fail;
    }
    fgets(buf, BUFSIZ, fp);
    buf[BUFSIZ-1] = '\0';
    pclose(fp);
    if ((pos = strstr(buf, "get_mib:")) == NULL) {
	printf("Can't find OFDM value from iwpriv coomand.\n");
	goto fail;
    }
    cur_ofdm = atoi(pos+8);

    sprintf(output, "(CCK/OFDM)\nFLASH: (%d/%d)\nCUR: (%d/%d)\n", 
	    flash_cck, flash_ofdm, cur_cck, cur_ofdm);
    websWrite(wp, T(output));
    return 0;

fail:
	websWrite(wp, T(""));
	return 0;
#undef RTL8186_HW_CONF
}

int getTitle(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t value[MAX_VALUE_LEN], model[MAX_VALUE_LEN], company[MAX_VALUE_LEN];
    char_t title[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    int ret;
    memset(company, 0, MAX_VALUE_LEN);
    memset(model, 0, MAX_VALUE_LEN);
    memset(value, 0, MAX_VALUE_LEN);
    cipcfg = init_cipcfg_f(current_config);
    if (!cipcfg)
	*value = '\0';
    else {
	if (get_cfg_item_by_str(cipcfg, "device", "name", value) != CIPCFG_SUCCESS)
	    *value = '\0';
	close_cipcfg(cipcfg);
    }
    getCustomizeString("title", value, title, MAX_VALUE_LEN);
    websWrite(wp,T(title));
    return 0;
}

int getFooter(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t value[MAX_VALUE_LEN];
    getCustomizeString("footer", "@Footer", 		    
	    value, MAX_VALUE_LEN);
    websWrite(wp,T(value));
    return 0;
}

/*
 *  generate a javascript array for selection
 */
int genOptions(int eid, webs_t wp, int argc, char_t **argv)
{
    int i;
    char *arrName, value[MAX_VALUE_LEN];
    const STRING_PAIR *data;
    if (ejArgs(argc, argv, T("%s"), &arrName) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    memset(value, 0, MAX_VALUE_LEN);
    sprintf(value, "var %s = new Array();\n", arrName);
    websWrite(wp,T(value));

    if ( !strcmp(arrName, "TimeZoneArray") ){
        data = zone_info;
    }else{
        // XXX: other data , like country code
        // data = country_info;
    }

    for (i=0; data[i].key; i++){
        sprintf(value, "%s[%s.length]=new Array(\"%s\",\"%s\");\n", arrName, arrName, data[i].key, data[i].value);
        websWrite(wp,T(value));
    }
    return 0;
}

#ifdef GW_FUNC
int getStaticRouteJson(int eid, webs_t wp, int argc, char_t **argv)
{
    int ret, i, j, isFirst = 1;
    char item_name[32];
    char item_value[MAX_VALUE_LEN];
    char value[BUFSIZ], *ptr;
    int max_num, policy_no;
    char *name_list[] = {
	    "destip", "netmask", "gateway", NULL,
	};

    if (ejArgs(argc, argv, T("%d"), &policy_no) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, gid_policy_route[policy_no], "max", item_value);
    if(ret != CIPCFG_SUCCESS)
    {
        printf("Can't get num\n");
        return 0;
    }
    max_num = atoi(item_value);
    for (i=0; i<max_num; i++){
	ptr = value;
	if (isFirst == 1) {
	    ptr += sprintf(ptr, "{\"ROUTE\":[");
	    isFirst = 0;
	} else
	    websWrite(wp, T("%s,"), value);

	ptr += sprintf(ptr, "{");
	for (j=0; name_list[j] != NULL; j++) {
	    sprintf(item_name, "route:%d.%s", i, name_list[j]);
	    ret = get_cfg_item(cipcfg, gid_policy_route[policy_no], item_name, item_value);
	    ptr += sprintf(ptr, "%s\"%s\":\"%s\"", (j == 0) ? "" : ", ", name_list[j],
			(ret == CIPCFG_SUCCESS) ? item_value : "");
	}
	ptr += sprintf(ptr, "}");
    }
    websWrite(wp,T("%s]}"), value);
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef GW_FUNC
int getReservedIPJson(int eid, webs_t wp, int argc, char_t **argv)
{
    int ret, i, isFirst = 1;
    char item_name[32];
    char ip[MAX_VALUE_LEN], mac[MAX_VALUE_LEN], remark[MAX_VALUE_LEN];
    char item_value[MAX_VALUE_LEN];
    char value[BUFSIZ];
    int vlan_no = 0;

    if (ejArgs(argc, argv, T("%d"), &vlan_no) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }

    memset(item_value, 0, MAX_VALUE_LEN);

    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    if(!cipcfg) {
        printf("initial cipcfg error.\n");
        return 0;
    }

    websWrite(wp,T("{\"RESERVED\":["));

    for (i=0; i<=9; i++) {
        if (isFirst == 0)
            websWrite(wp, T(","));

        sprintf(item_name, "reserved_ip:%d.ip", i);
        ret = get_cfg_item(cipcfg, gid_vlan_dhcp[vlan_no], item_name, ip);
        sprintf(item_name, "reserved_ip:%d.mac", i);
        ret = get_cfg_item(cipcfg, gid_vlan_dhcp[vlan_no], item_name, mac);
        sprintf(item_name, "reserved_ip:%d.remark", i);
        ret = get_cfg_item(cipcfg, gid_vlan_dhcp[vlan_no], item_name, remark);

        sprintf(value, "{\"ip\":\"%s\", \"mac\":\"%s\", \"remark\" :\"%s\"}", ip, mac, remark);

        websWrite(wp,T(value));
        isFirst = 0;
    }
    websWrite(wp,T("]}"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif

int getWalledListJson(int eid, webs_t wp, int argc, char_t **argv)
{
    int ret, i, isFirst = 1;
    int gid = GID_NET_WALL, max = 0;
    char item_name[32], item_value[MAX_VALUE_LEN];
    char value[BUFSIZ], *ptr;
    CIPCFG *cipcfg = NULL;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	return 0;
    }

    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, gid, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get num\n");
	return 0;
    }
    max = atoi(item_value);
    for (i=0; i<max; i++) {
	ptr = value;
	if (isFirst == 1) {
	    ptr += sprintf(ptr, "{\"WALLED\":[");
	    isFirst = 0;
	} else
	    websWrite(wp, T("%s,"), ptr);

	sprintf(item_name, "wall:%d.dest", i);
	ret = get_cfg_item(cipcfg, gid, item_name, item_value); 
	ptr += sprintf(ptr, "{\"dest\":\"%s\"}", (ret == CIPCFG_SUCCESS) ? item_value : "");
    }
    websWrite(wp, T("%s]}"), value);
    close_cipcfg(cipcfg);
    return 0;
}

#ifdef GW_FUNC
int getFirewallJson(int eid, webs_t wp, int argc, char_t **argv)
{
    int i, j, policy_no=0, isFirst = 1, max_num = 0;
    char value[BUFSIZ], *ptr;
    int ret;
    char item_name[32];
    char item_value[MAX_VALUE_LEN];
    char *name_list[] = {
            "srcip", "srcmask", "dstip", "dstmask", "proto", "action",
            "sport", "dport", "icmptype", "proto_no", NULL,
        };

    if (ejArgs(argc, argv, T("%d"), &policy_no) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }

    memset(value, 0, BUFSIZ);

    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, gid_frule[policy_no], "max", item_value);
    if(ret != CIPCFG_SUCCESS)
    {
        printf("Can't get num\n");
        return 0;
    }
    max_num = atoi(item_value);
    for (i=0; i<max_num; i++){
        ptr = value;
        if (isFirst == 1) {
            ptr += sprintf(ptr, "{\"FIREWALL\":[");
            isFirst = 0;
        } else
            websWrite(wp, T("%s,"), ptr);

        ptr += sprintf(ptr, "{");
        for (j=0; name_list[j] != NULL; j++) {
            sprintf(item_name, "frule:%d.%s", i, name_list[j]);
            ret = get_cfg_item(cipcfg, gid_frule[policy_no], item_name, item_value);
            ptr += sprintf(ptr, "%s\"%s\":\"%s\"", (j == 0) ? "" : ", ", name_list[j],
                        (ret == CIPCFG_SUCCESS) ? item_value : "");
        }
        ptr += sprintf(ptr, "}");
    }
    websWrite(wp, T("%s]}"), value);
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef GW_FUNC
// get RADIUS class to local policy mapping table
int getMappingJson(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char item_name[32];
    char item_value[MAX_VALUE_LEN], attr[MAX_VALUE_LEN], policyid[MAX_VALUE_LEN], remark[MAX_VALUE_LEN];
    char value[BUFSIZ], *ptr, *binattr;
    int radius_no = 1, max = 0, isFirst = 1, i, ret;

    if (ejArgs(argc, argv, T("%d"), &radius_no) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	return 0;
    }

    memset(value, 0, BUFSIZ);
    memset(item_value, 0, MAX_VALUE_LEN);
    ret = get_cfg_item(cipcfg, gid_radius_map[radius_no], "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum mapping entries.\n");
	return 0;
    }
    max = atoi(item_value);
    for (i=0; i<max; i++) {
	ptr = value;
	if (isFirst == 1) {
	    ptr += sprintf(ptr, "{\"MAPPING\":[");
	    isFirst = 0;
	} else
	    websWrite(wp, T("%s,"), ptr);
        sprintf(item_name, "entry:%d.attr", i);
        ret = get_cfg_item(cipcfg, gid_radius_map[radius_no], item_name, attr);
	binattr = NULL;
	if (ret == CIPCFG_SUCCESS) {
	    binattr = hexbinconvert(attr, 2);
	    jasonStringEscapeSequence(binattr, attr);
	}
        sprintf(item_name, "entry:%d.policyid", i);
        ret = get_cfg_item(cipcfg, gid_radius_map[radius_no], item_name, policyid);
        sprintf(item_name, "entry:%d.remark", i);
        ret = get_cfg_item(cipcfg, gid_radius_map[radius_no], item_name, remark);

        ptr += sprintf(ptr, "{\"attr\":\"%s\", \"policyid\":\"%s\", \"remark\":\"%s\"}", 
		    (binattr == NULL) ? "" : attr, policyid, remark);
	free(binattr);
    }
    websWrite(wp, T("%s]}"), value);
    close_cipcfg(cipcfg);
    return 0;
}
#endif

/*
 *  generate a JSON string for selection
 */

int getJsonString(int eid, webs_t wp, int argc, char_t **argv)
{
    int i, rfno, vapno, isFirst = 1;
    char *arrName, value[BUFSIZ];

    if (ejArgs(argc, argv, T("%s"), &arrName) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    memset(value, 0, BUFSIZ);

    if ( !strncmp(arrName, "Channel", 7) ){ // generate Channel by Country Code
        char ch[8];
        COUNTRY_CHANNEL cc;
        if ( 0 != get_allowed_ch(&cc) ){
            // Can NOT get channel information,
            // Create Default Channel 
            sprintf(value, "{\"cc\":0,\"ch_11a\":[],\"ch_11b\":[\"Auto\",1,2,3,4,5,6,7,8,9,10,11,12,13],\"ch_11g\":[\"Auto\",1,2,3,4,5,6,7,8,9,10,11,12,13]}");
        }else{
            sprintf(value, "{\"cc\":%u,\"ch_11a\":[", cc.cc);
            if( -1 != cc.ch_11a[0] ){
                strncat(value, "\"Auto\"", 6);
                for(i=0; (-1 != cc.ch_11a[i]) ; i++){
                    sprintf(ch, ",%d", cc.ch_11a[i]);
                    strcat(value, ch);
                }
            }
            strncat(value, "],\"ch_11b\":[", 12);
            if( -1 != cc.ch_11b[0] ){
                strncat(value, "\"Auto\"", 6);
                for(i=0; (-1 != cc.ch_11b[i]) ; i++){
                    sprintf(ch, ",%d", cc.ch_11b[i]);
                    strcat(value, ch);
                }
            }
            strncat(value, "],\"ch_11g\":[", 12);
            if( -1 != cc.ch_11g[0] ){
                strncat(value, "\"Auto\"", 6);
                for(i=0; (-1 != cc.ch_11g[i]) ; i++){
                    sprintf(ch, ",%d", cc.ch_11g[i]);
                    strcat(value, ch);
                }
            }
            strncat(value, "],\"ch_11ng\":[", 13);
	    if (-1 != cc.ch_11ng[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11ng[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11ng[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "],\"ch_11ngu\":[", 14);
	    if (-1 != cc.ch_11ngu[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11ngu[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11ngu[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "],\"ch_11ngl\":[", 14);
	    if (-1 != cc.ch_11ngl[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11ngl[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11ngl[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "],\"ch_11na\":[", 13);
	    if (-1 != cc.ch_11na[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11na[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11na[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "],\"ch_11nau\":[", 14);
	    if (-1 != cc.ch_11nau[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11nau[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11nau[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "],\"ch_11nal\":[", 14);
	    if (-1 != cc.ch_11nal[0]) {
		strncat(value, "\"Auto\"", 6);
		for(i=0; (-1 != cc.ch_11nal[i]); i++) {
		    sprintf(ch, ",%d", cc.ch_11nal[i]);
		    strcat(value, ch);
		}
	    }
            strncat(value, "]}", 2);
        }
        // for DEBUG: printf("---%s---\n", value);
        websWrite(wp,T(value));
    }else if ( !strncmp(arrName, "URLs", 4) ){
        int ret;
        char item_name[32], url[MAX_VALUE_LEN], state[MAX_VALUE_LEN];
        CIPCFG *cipcfg = NULL;
        cipcfg=init_cipcfg();
        if(!cipcfg){
            printf("initial cipcfg error.\n");
            return 0;
        }
        websWrite(wp,T("{"));
        for (i=0; i<20; i++){
            if (isFirst == 0)
                websWrite(wp, T(","));
            sprintf(item_name, "url_filter_entry:%d.url", i);
            ret = get_cfg_item(cipcfg, GID_URL_FILTER, item_name, url);
            sprintf(item_name, "url_filter_entry:%d.state", i);
            ret = get_cfg_item(cipcfg, GID_URL_FILTER, item_name, state);
            sprintf(value, "\"URL%d\":[\"%s\", \"%s\"]", i+1, url, state);
            websWrite(wp,T(value));
            isFirst = 0;
        }
        websWrite(wp,T("}"));

        close_cipcfg(cipcfg);
    }else if ( !strncmp(arrName, "TimeZone", 8) ){
        // TODO:move Time Zone from function "genOptions()"to here.
    } else if (sscanf(arrName, "SCAN_rf%d", &rfno) == 1){
	char tmp[MAX_VALUE_LEN], ssid[MAX_VALUE_LEN], t[MAX_VALUE_LEN];
	int num;
	if (--rfno >= 0 && (getOperationPermission(wp, OID_SCAN) & PERM_EXEC)) {
	    AP_LIST *node, *ap_list = rf_scan(rfno, 1, t);
	    websWrite(wp,T("["));
	    foreach_ap_get(node, ap_list) {
                if (isFirst == 0)
                    websWrite(wp, T(","));
                    htmlSpecialCharEncode(node->ssid, tmp);
                    javascriptEscapeSequence(tmp, ssid);
		num = sprintf(value, "{"
			"\"ssid\":\"%s\","
			"\"mac\":\"%s\","
			"\"channel\":%d,"
			"\"rate\":%d,"
			"\"signal\":%d,"
			"\"security\":\"%s\""
			"}", 
			ssid, node->mac, node->ch, node->rate, 
			node->signal, node->security);
		if (num > 0){
		    websWrite(wp,T(value));
                    isFirst = 0;
            	}else 
		    continue;
	    }
	    websWrite(wp,T("]"));
	    free_ap_list(ap_list);
	}
    } else if (sscanf(arrName, "STALIST_rf%d_vap%d", &rfno, &vapno) == 2){
	int rf_idx = rfno - 1, vap_idx = vapno - 1, num;
	char if_name[16], name[MAX_VALUE_LEN], ssid[MAX_VALUE_LEN];
	char group_name[MAX_VALUE_LEN], tmp[MAX_VALUE_LEN];
	CIPCFG *cipcfg_w = NULL, *cipcfg = NULL;
	ASSOC_STA_LIST *sta_list = NULL, *sta = NULL;

	sprintf(group_name, "rf%d_vap%d", rfno, vapno);
	if (get_vap_ifname_by_idx(rf_idx, vap_idx, if_name) == NULL) {
	    printf("Can't get interface name for RF=%d VAP=%d\n", 
		    rf_idx, vap_idx);
	    return 0;
	}
	cipcfg_w = init_cipcfg_w(eid, 0);
	if (!cipcfg_w) {
	    cipcfg = init_cipcfg();
	    if (!cipcfg) {
		printf("initial cipcfg error.\n");
		return 0;
	    }
	} else {
	    cipcfg = cipcfg_w;
	}
	websWrite(wp,T("["));
	sta_list = get_sta_list(if_name);
	foreach_sta_get(sta, sta_list) {
            if (isFirst == 0)
                websWrite(wp, T(","));
	    get_cfg_item_by_str(cipcfg, group_name, "name", name);
	    get_cfg_item_by_str(cipcfg, group_name, "ssid", ssid);
            htmlSpecialCharEncode(ssid, tmp);
            javascriptEscapeSequence(tmp, ssid);
	    num = sprintf(value, "{"
		    "\"group\":\"%s\","
		    "\"name\":\"%s\","
		    "\"ssid\":\"%s\","
		    "\"mac\":\"%s\","
		    "\"rssi\":%d,"
		    "\"idle\":%d"
		    "}",
		    group_name, name, ssid, sta->mac, sta->rssi, 
		    (int)sta->last_rx);
	    if (num > 0) {
		websWrite(wp,T(value));
                isFirst = 0;
            } else 
		continue;
	}
	websWrite(wp,T("]"));
	if (cipcfg && !cipcfg_w)
	    close_cipcfg(cipcfg);
	free_sta_list(sta_list);
    }else if ( !strncmp(arrName, "REDIRECT", 8) ){
        int ret;
        char item_name[32], name[MAX_VALUE_LEN], from_port[MAX_VALUE_LEN], to_ip[MAX_VALUE_LEN], 
             to_port[MAX_VALUE_LEN], protocol[MAX_VALUE_LEN], state[MAX_VALUE_LEN];
        CIPCFG *cipcfg = NULL;
        cipcfg=init_cipcfg();
        if(!cipcfg){
            printf("initial cipcfg error.\n");
            return 0;
        }
        websWrite(wp,T("{\"RED\":["));
        for (i=0; i<20; i++){
            if (isFirst == 0)
                websWrite(wp, T(","));
            sprintf(item_name, "redir_entry:%d.name", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, name);                
            sprintf(item_name, "redir_entry:%d.from_port", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, from_port);
            sprintf(item_name, "redir_entry:%d.to_ip", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, to_ip);
            sprintf(item_name, "redir_entry:%d.to_port", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, to_port);
            sprintf(item_name, "redir_entry:%d.protocol", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, protocol);
            sprintf(item_name, "redir_entry:%d.state", i);
            ret = get_cfg_item(cipcfg, GID_IP_REDIRECT, item_name, state);
            sprintf(value, "{\"name\":\"%s\", \"from_port\":\"%s\", \"to_ip\":\"%s\", \"to_port\":\"%s\", \"protocol\":\"%s\", \"state\":\"%s\"}", name, from_port, to_ip, to_port, protocol, state);
            websWrite(wp,T(value));
            isFirst = 0;
        }
        websWrite(wp,T("]}"));
        close_cipcfg(cipcfg);
#ifdef GW_FUNC
    }else if( !strncmp(arrName, "NASLIST", 7) ){
        int ret;
        char item_name[32];
        char type[MAX_VALUE_LEN], ipaddr[MAX_VALUE_LEN], mask[MAX_VALUE_LEN], secret[MAX_VALUE_LEN]; 

        CIPCFG *cipcfg = NULL;
        cipcfg=init_cipcfg();
        if(!cipcfg){
            printf("initial cipcfg error.\n");
            return 0;
        }
        websWrite(wp,T("{\"NASLIST\":["));
        for (i=0; i<10; i++){
            if (isFirst == 0)
                websWrite(wp, T(","));

            ret = get_cfg_item(cipcfg, gid_nas[i], "type", type);                
            ret = get_cfg_item(cipcfg, gid_nas[i], "ipaddr", ipaddr);
            ret = get_cfg_item(cipcfg, gid_nas[i], "mask", mask);
            ret = get_cfg_item(cipcfg, gid_nas[i], "secret", secret);
            sprintf(value, "{\"type\":\"%s\", \"ipaddr\":\"%s\", \"mask\":\"%s\", \"secret\":\"%s\"}", 
                    type, ipaddr, mask, secret);
            websWrite(wp,T(value));
            isFirst = 0;
        }
        websWrite(wp,T("]}"));
        close_cipcfg(cipcfg);
#endif	
#ifdef GW_FUNC
    }else if( !strncmp(arrName, "ROUTE", 5) ){
        int ret;
        char item_name[32];
        char destip[MAX_VALUE_LEN], netmask[MAX_VALUE_LEN], gateway[MAX_VALUE_LEN];

        CIPCFG *cipcfg = NULL;
        cipcfg=init_cipcfg();
        if(!cipcfg){
            printf("initial cipcfg error.\n");
            return 0;
        }
        websWrite(wp,T("{\"ROUTE\":["));
        for (i=0; i<17; i++){
            if (isFirst == 0)
                websWrite(wp, T(","));

            ret = get_cfg_item(cipcfg, gid_policy_route[i], "destip", destip);
            ret = get_cfg_item(cipcfg, gid_policy_route[i], "netmask", netmask);
            ret = get_cfg_item(cipcfg, gid_policy_route[i], "gateway", gateway);

            sprintf(value, "{\"destip\":\"%s\", \"netmask\":\"%s\", \"gateway\":\"%s\"}"
                    , destip, netmask, gateway);

            websWrite(wp,T(value));
            isFirst = 0;
        }
        websWrite(wp,T("]}"));
        close_cipcfg(cipcfg);
#endif	
    }else{
        // XXX: other data , like country code
        // data = country_info;
    }
    return 0;
}

int getDhcpLeaseJson(int eid, webs_t wp, int argc, char_t **argv)
{
    FILE    *fd = NULL;
    char    ip[32], mac[32], et[64], value[BUFSIZ], ifName[32];
    int     i = 0;
    
    get_lan_ifname(ifName);
    sprintf(et, PATH_LEASE_FILE, ifName);
    sprintf(value, "%s -f %s > %s", PATH_DUMPLEASES, et, PATH_TMP_LEASE_FILE);
    system(value);
    fd = popen(DUMP_LEASE_CMD, "r");
    websWrite(wp,T("["));
    while(fgets(value, BUFSIZ, fd)){
        if(i == 0){
            i++;
            continue;
        }
        if(i > 1)
            websWrite(wp,T(","));
        sscanf(value, "%s %s %s", mac, ip, et);
        // Format: [{"ip":"IP","mac":"MAC","et":"ET"}, {"ip":"IP","mac":"MAC","et":"ET"}]
        sprintf(value, "{\"ip\":\"%s\",\"mac\":\"%s\",\"et\":\"%s\"}", ip, mac, et);
        websWrite(wp,T(value));
        i++;
    }
    if(fd)
        pclose(fd);
    websWrite(wp,T("]"));
    return 0;
}    
/**
 * This is a ASP function for 
 * get the Max number of item.
 */
int getMax(int eid, webs_t wp, int argc, char_t **argv)
{
    char *item;
    int ret = 0;
    if (ejArgs(argc, argv, T("%s"), &item) < 1) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }
    if (!strncmp(item, "RF", 2)){
        ret = max_rf;
    }else if (!strncmp(item, "VAP", 3)){
        ret = MAX_VAP;
    }else if (!strncmp(item, "WDS", 3)){
        ret = MAX_WDS_LINK;
    }else if (!strncmp(item, "REP", 3)){
        ret = MAX_REP;
    }else if (!strncmp(item, "STA", 3)){
        ret = MAX_STA;
    }

    ejsSetReturnValue(eid, mprCreateIntegerVar(ret));
    return 0;
}

/*
 * get web protocol :  HTTP or HTTPS
 */
int getProtocol(int eid, webs_t wp, int argc, char_t **argv)
{
    MaRequest *rq = (MaRequest*)wp;
    maWriteStr(rq, maGetVar(rq, MA_REQUEST_OBJ, "REQUEST_TRANSPORT", ""));
    
    return 0;
}

int getReboot(int eid, webs_t wp, int argc, char_t **argv)
{
    struct stat st;
    CIPCFG *cipcfg;
    char item_value[MAX_VALUE_LEN];
    
    if(stat(PATH_REBOOT_FLAG, &st) == 0) {
	cipcfg = init_cipcfg();
	get_cfg_item(cipcfg, GID_OPMODE, "type", item_value);
	if (strcmp(item_value, "GW") == 0)
	    cip_system("[ -x /etc/rc.d/rc.radius ] && /etc/rc.d/rc.radius stop_acct");
	close_cipcfg(cipcfg);
        system("(logger \"Rebooting from Web UI...\";sleep 5;/sbin/reboot)&");
    }
    return 0;
}

int getTouchRebootFlag(int eid, webs_t wp, int argc, char_t **argv)
{
    char cmd[64];
    
    sprintf(cmd, "%s %s", PATH_TOUCH, PATH_REBOOT_FLAG);
    system(cmd);
    return 0;   
}
/**
 *  This ASP function is used to get the config access permission
 */
void getAccessRight(EspRequest *ep, int argc, char **argv)
{
    char            *g_name;
    char            *ret = ASP_PERM_N;
    int             group = 0;
    unsigned int    permission = 0;
    
    MaRequest *rq = espGetRequestHandle(ep);
    char ret_str[MAX_VALUE_LEN];
    memset(ret_str, '\0', MAX_VALUE_LEN);
    
    if (ejsParseArgs(argc, argv, "%s", &g_name) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }

    getCookieByName(rq, "username", ret_str);
    
    group       = getUserGroup(ret_str);
    if(group != USER_GROUP_NONE){
	permission  = getConfigPermissionByName(g_name);
	switch(GET_GRP_PERM(group, permission)){
            case PERM_NONE:
                ret = ASP_PERM_N;
                break;
            case PERM_READ:
                ret = ASP_PERM_R;
                break;
            case PERM_WRITE:
            case PERM_READ|PERM_WRITE:
                ret = ASP_PERM_W;
                break;
        }
    }
	
    ejsSetReturnString(ep->eid, ret);
}


/**
 *  This ASP function is used to get the operation permission
 */
void getOperRight(EspRequest *ep, int argc, char **argv)
{
    char            *o_name;
    char            *ret = ASP_PERM_N;
    int             group = 0;
    unsigned int    permission = 0;
    
    MaRequest *rq = espGetRequestHandle(ep);
    char ret_str[MAX_VALUE_LEN];
    memset(ret_str, '\0', MAX_VALUE_LEN);

    if (ejsParseArgs(argc, argv, "%s", &o_name) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
	
    getCookieByName(rq, "username", ret_str);
    
    group       = getUserGroup(ret_str);
    if(group != USER_GROUP_NONE){
	permission  = getOperationPermissionByName(o_name);
	switch(GET_GRP_PERM(group, permission)&PERM_EXEC){
	    case PERM_EXEC:
	    ret = ASP_PERM_X;
	    break;

	    default:
	    ret = ASP_PERM_N;
	    break;
        }
    }
	
    ejsSetReturnString(ep->eid, ret);
}
/**
 *  This ASP function is used to get the txpower level
 */
int getTxPwrLv(int eid, webs_t wp, int argc, char_t **argv)
{
    TXPWR_T *list = get_txpwr_level();
    int isFirst = 1, num, i;
    char value[256];

    websWrite(wp,T("["));
    for (i = 0; list[i].desc; i++) {
	if (isFirst == 0)
	    websWrite(wp, ",");
	num = sprintf(value, "{\"desc\":\"%s\", \"level\":%d}", 
		list[i].desc, list[i].level);
	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else {
	    continue;
	}
    }
    websWrite(wp,T("]"));

    return 0;
}


/**
 *  This ASP function is used to get the WPS settings of VAP
 */
int getVapWps(int eid, webs_t wp, int argc, char_t **argv)
{
    extern const char *current_config;
    char *type = NULL;
    CIPCFG *cipcfg = NULL;
    int i, is_empty = 1;
    char_t mode[MAX_VALUE_LEN];
    char_t eap[MAX_VALUE_LEN];
    char_t group_name[MAX_VALUE_LEN]; 
    char_t state[MAX_VALUE_LEN];
    char_t vap_name[MAX_VALUE_LEN];
    char_t enabled[MAX_VALUE_LEN];
    char_t configurable[MAX_VALUE_LEN];
    char_t ap_pin[MAX_VALUE_LEN];
    char tmp[BUFSIZ];

    if (ejArgs(argc, argv, T("%s"), &type) == 1) {
	if (type && strcmp(type, "activated") == 0)
	    cipcfg = init_cipcfg_f(current_config);
	else
	    cipcfg = init_cipcfg();
    } else {
	cipcfg = init_cipcfg();
    }

    websWrite(wp, T("["));
    for (i = 0; i < MAX_VAP; i++) {
	sprintf(group_name, "rf1_vap%d", i + 1);
	get_cfg_item_by_str(cipcfg, group_name, "name", vap_name);
	get_cfg_item_by_str(cipcfg, group_name, "state", state);
	if (strcmp(state, "enable") != 0)
	    continue;
	sprintf(group_name, "rf1_vap%d_sec", i + 1);
	get_cfg_item_by_str(cipcfg, group_name, "mode", mode);
	get_cfg_item_by_str(cipcfg, group_name, "wpa.eap", eap);
	if (strcmp(mode, "shared") == 0 || strcmp(mode, "8021x") == 0 || 
		(strcmp(mode, "wpa") == 0 && strcmp(eap, "eap") == 0))
	    continue;
	get_cfg_item_by_str(cipcfg, group_name, "wps.enabled", enabled);
	get_cfg_item_by_str(cipcfg, group_name, "wps.configure", configurable);
	get_cfg_item_by_str(cipcfg, group_name, "wps.ap_pin", ap_pin);
	sprintf(tmp, "{\"name\":\"%s\", \"idx\":%d, \"enabled\":%s, \"configurable\":%s,"" \"pin\":\"%s\"}", vap_name, i,
		(strcmp(enabled, "enable") == 0 ? "true" : "false"), 
		(strcmp(configurable, "enable") == 0 ? "true" : "false"), 
		ap_pin);
	if (!is_empty)
	    websWrite(wp, T(","));
	is_empty = 0;
	websWrite(wp, T(tmp));
    }
    websWrite(wp, T("]"));
    close_cipcfg(cipcfg);
    return 0;
}

/*
 * This function returns an XMLHttpRequest of ping result
 */
static void getPing(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    int ret = 0, pid = 0;
    char  *url, *pid_c;

    if(getOperationPermission(rq, OID_PING ) & PERM_EXEC){
        url = maGetVar(rq, MA_FORM_OBJ, T("url"), T(""));
        pid_c = maGetVar(rq, MA_FORM_OBJ, T("pid"), T(""));
        printf("ping : \"%s\" ......\n", url);
	if (*url != '\0') {
            ret = cip_ping_p(url, &pid);
	    if (ret != CIPCFG_SUCCESS){
		printf("[PING] Illegal Characters of URL.\n");
		maWriteFmt(rq, "%s", "Illegal Characters of URL.\nOK");
	    }
	    else {
		maWriteFmt(rq, "%d", pid);
	    }
	}
	else {
            char *p, buf[80];
            FILE *fp = NULL;
            if( (fp = fopen(PATH_PING_RESULT, "r")) != NULL ){
                while (fgets(buf, 80, fp)){
		    maWriteFmt(rq, "%s", buf);
                }
		if (kill(atoi(pid_c), 0) < 0)
		    maWriteFmt(rq, "\n%s", "OK");
                fclose(fp);
            }else{
                printf("ERROR!! Can not get ping result!!\n");
            }
        }
   }
    maSetResponseCode(rq, 200);
    return;
}

#ifdef FULL_NET_UTILS
static void getTracert(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    int ret = 0, pid = 0;
    char  *url, *pid_c, *stop;

    url = maGetVar(rq, MA_FORM_OBJ, T("url"), T(""));
    pid_c = maGetVar(rq, MA_FORM_OBJ, T("pid"), T(""));
    stop = maGetVar(rq, MA_FORM_OBJ, T("stop"), T(""));
    printf("tracert : \"%s\" ......\n", url);
    if (*url != '\0') {
	ret = cip_tracert_p(url, &pid);
	if (ret != CIPCFG_SUCCESS){
	    printf("[PING] Illegal Characters of URL.\n");
	    maWriteFmt(rq, "%s", "Illegal Characters of URL.\nOK");
	}
	else
	    maWriteFmt(rq, "%d", pid);
    }
    else {
	char buf[80];
	FILE *fp = NULL;
	if( (fp = fopen(PATH_TRACERT_RESULT, "r")) != NULL ){
	    while (fgets(buf, 80, fp)){
		maWriteFmt(rq, "%s", buf);
	    }
	    if (kill(atoi(pid_c), 0) < 0)
		maWriteFmt(rq, "\n%s", "OK");
	    fclose(fp);
	}else{
	    printf("ERROR!! Can not get tracert result!!\n");
	}
	if (!strcmp(stop, "true")) {
	    sprintf(buf, "/proc/%s/cmdline", pid_c);
	    if ((fp = fopen(buf, "r")) != NULL) {
		fgets(buf, 80, fp);
		if (strcmp(buf, PATH_TRACERT) == 0)
		    kill(atoi(pid_c), 15);
		pclose(fp);
	    }
	    maWriteFmt(rq, "\n%s", "OK");
	}
    }
    maSetResponseCode(rq, 200);
    return;
}

static void getARPTable(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    FILE *fp = NULL;
    char buf[MAX_VALUE_LEN];

    if ((fp = popen(PATH_ARP, "r")) != NULL) {
	while (fgets(buf, MAX_VALUE_LEN, fp) != NULL)
	    maWriteFmt(rq, "%s", buf);
	maWriteFmt(rq, "\n%s", "OK");
	pclose(fp);
    }
    else {
	maWriteFmt(rq, "%s", "Cannot finish the request.");
    }
    maSetResponseCode(rq, 200);
    return;
}
#endif

int getIgdPortMapJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char    *cmd = "/usr/sbin/iptables -t nat -L MINIUPNPD -nv|awk '{if (NR>2) print $4, $11, $12}'|sed 's/dpt://g;s/to://g;s/:/\ /g'";
    FILE    *fd = NULL;
    char    protocal[16], ext_port[16], ip[17], in_port[16], buf[BUFSIZ];
    char    *ptr = NULL;
    int     i=0;
    
    memset(buf, 0, BUFSIZ);
    websWrite(wp,T("["));
    fd = popen(cmd, "r");
    if(fd){
        while(fgets(buf, BUFSIZ, fd)){
            if(i>0)
                websWrite(wp,T(","));
            sscanf(buf, "%s %s %s %s", protocal, ext_port, ip, in_port);
	    ptr = strchr(in_port, '-');
	    if(ptr != NULL)
		*ptr = '\0';
            sprintf(buf, "{\"protocal\":\"%s\",\"in_port\":\"%s\",\"ext_port\":\"%s\",\"ip\":\"%s\"}",
                    protocal, in_port, ext_port, ip);
            websWrite(wp,T(buf));
            i++;
        }
        pclose(fd);
    }
    websWrite(wp,T("]"));
    return 0;
}

/*
 * This funcion will put escape char "\" in front of special chars : "\", """, "'"
 * to prevent the string variable cause error in JavaScript area.
 */
int _J(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *src, dst[MAX_VALUE_LEN];

    if (ejArgs(argc, argv, T("%s"), &src) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    javascriptEscapeSequence(src, dst);

    ejSetResult(eid, dst);
    return 0;
}

/*  for HTML Special Characters Encoding.
 *  Special Characters like "<", ">" and "&"
 */
int _H(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *src, dst[MAX_VALUE_LEN];

    if (ejArgs(argc, argv, T("%s"), &src) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    htmlSpecialCharEncode(src, dst);

    ejSetResult(eid, dst);
    return 0;
}

int feature(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *name, value[1];
    int ret, ncode = 0, fcode = 0;
    if ((ret = ejsParseArgs(argc, argv, "%s %d %d", &name, &ncode, &fcode)) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    if (ret < 3)
	fcode = ncode;

    ret = chkFeature(name, value);
    if (ret != CIPCFG_SUCCESS) {
	ret = (ret == CIPCFG_ERR_UNKNOWN_ITEM) ? ncode : fcode;
    }else{
        if ( value[0] == 'Y' ){
            ret = 1;
        }else{
            ret = 0;
        }
    }
    ejsSetReturnValue(eid, mprCreateIntegerVar(ret));
    return 0;
}

#ifdef GW_FUNC
int getVlanList(int eid, webs_t wp, int argc, char_t **argv)
{
    int i, j, policy_no=0, isFirst = 1, max_num = 0;
    char value[BUFSIZ], *ptr;
    int ret;
    char ip[MAX_VALUE_LEN], tag[MAX_VALUE_LEN], status[MAX_VALUE_LEN];
    char *name_list[] = { "ip", "tag", "status", NULL };

    memset(value, 0, BUFSIZ);

    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return 0;
    }

    /* TODO: Max_num should be get from rc.env or xml file */
    max_num = 9;
    for (i=0; i<max_num; i++){
	ptr = value;
	if (isFirst == 1) {
	    ptr += sprintf(ptr, "{\"VLAN\":[");
	    isFirst = 0;
	} else
	    websWrite(wp, T("%s,"), ptr);

        ret = get_cfg_item(cipcfg, gid_vlan_lan[i], "ip", ip);
        ret = get_cfg_item(cipcfg, gid_vlan[i], "vtag", tag);
        ret = get_cfg_item(cipcfg, gid_vlan[i], "status", status);

        if(!strcmp(status, "enable")) 
            strncpy(status, "Enabled", 7);
        else 
            strncpy(status, "Disabled", 8);

        ptr += sprintf(ptr, "{\"ip\":\"%s\",\"tag\":\"%s\",\"status\":\"%s\"}",
                    ip, tag, status);
    }
    websWrite(wp, T("%s]}"), value);
    close_cipcfg(cipcfg);
    return 0;
}
#endif

static int fresh_online(void)
{
    int fildes, status;
    struct stat st;
    time_t now;

    if (stat("/tmp/status/online_user.txt.raw", &st) < 0)
        return 0;

    time(&now);   
    if (abs(now - st.st_mtime) < 3)
	return 1;
    return 0;
}

static int refreshUserList(EspRequest *ep, int argc, char **argv)
{
    char cmd[256] = "kill -USR1 `cat /tmp/MPID`";
    int i;

    cip_system(cmd);
    for (i = 1; i < 5; i++) {
	usleep(i * 1000000);
	if (fresh_online() == 1)
	    break;
    }
    return 0;
}

int ipmask(char *in_netmask, char *ip, char *start_ip, char *end_ip)
{
unsigned long netmask, ipaddr, netaddr, broadcast;
int in[4], j, num_netmask;
unsigned char bc[4],na[4];

	/* Check netmask */
	if (4 == sscanf(in_netmask, "%d.%d.%d.%d", &in[0],&in[1],&in[2],&in[3])) {
	    for (j=0; j<4; ++j) {
		if (in[j]<0 || in[j]>255) {
			printf("Invalid octet %d in netmask \"%s\".\n",
				j+1, in_netmask);
			return -1;
		}
	    }
	    netmask = in[3] + 256 * (in[2] + 256 * (in[1] + 256 * in[0]));
	}
	else {
	    if (1 != sscanf(in_netmask, "%d", &num_netmask)) {
		printf("Invalid netmask \"%s\".\n", in_netmask);
		return -1;
	    }
	    netmask = 0;
	    for(j=1; j<= num_netmask; j++)
		netmask = (netmask >> 1) | 2147483648;
	}

	/* Check IP address */
	if (4 != sscanf(ip, "%d.%d.%d.%d", &in[0],&in[1],&in[2],&in[3])) {
		printf("Invalid IP address \"%s\".\n", ip);
		return -1;
	}
	for (j=0; j<4; ++j) {
		if (in[j]<0 || in[j]>255) {
			printf("Invalid octet %d in IP address \"%s\".\n",
				j+1, ip);
			return -1;
		}
	}
	ipaddr = in[3] + 256 * (in[2] + 256 * (in[1] + 256 * in[0]));

	broadcast = ipaddr | (~ netmask);
	bc[0] = broadcast / 256 / 256 / 256;
	bc[1] = (broadcast / 256 / 256) % 256;
	bc[2] = (broadcast / 256) % 256;
	bc[3] = broadcast % 256;

	netaddr = ipaddr & netmask;
	na[0] = netaddr / 256 / 256 / 256;
	na[1] = (netaddr / 256 / 256) % 256;
	na[2] = (netaddr / 256) % 256;
	na[3] = netaddr % 256;

	sprintf(end_ip, "%d.%d.%d.%d", bc[0], bc[1], bc[2], bc[3]);
	sprintf(start_ip, "%d.%d.%d.%d", na[0], na[1], na[2], na[3]);
	return 0;
}

#ifdef MGMT_IP
static int checkMgmtIP(EspRequest *ep, int argc, char **argv)
{
    char *reqip, buf[MAX_VALUE_LEN], *pch, *mask, *tmp;
    FILE *fp = NULL;
    int i;
    struct in_addr req_l, mgmtip_l, mgmt_start, mgmt_end;

    if (ejsParseArgs(argc, argv, "%s", &reqip) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    if (inet_aton(reqip, &req_l) == 0) {
	printf("The incoming IP is invalid.\n");
	espSetReturn(ep, mprCreateIntegerVar(0));
	return -1;
    }
    fp = fopen("/tmp/mgmtip", "r");
    if (fp != NULL) {
	*buf = 0;
	while(fgets(buf, MAX_VALUE_LEN, fp) != NULL) {
	    pch = strchr(buf, '/');
	    if (pch != NULL) {
		mask = pch + 1;
		*pch = '\0';
	    }
	    else
		mask = NULL;
	    if (inet_aton(buf, &mgmtip_l) == 0) {
		printf("Invalid management IP address.\n");
		*buf = 0;
		continue;
	    }
	    if (mask == NULL) {
		if (req_l.s_addr == mgmtip_l.s_addr) {
		    espSetReturn(ep, mprCreateIntegerVar(1));
		    fclose(fp);
		    return 0;
		}
	    }
	    else {
		char start_ip[MAX_VALUE_LEN], end_ip[MAX_VALUE_LEN];
		if ((tmp = strchr(mask, '\n')) != NULL)
		    *tmp = '\0';
		if (ipmask(mask, buf, start_ip, end_ip) == -1) {
		    printf("Invalid management IP/subnet address.\n");
		    *buf = 0;
		    continue;
		}
		if (inet_aton(start_ip, &mgmt_start) == 0) {
		    printf("Invalid management IP start address.\n");
		    *buf = 0;
		    continue;
		}
		if (inet_aton(end_ip, &mgmt_end) == 0) {
		    printf("Invalid management IP end address.\n");
		    *buf = 0;
		    continue;
		}
		if (req_l.s_addr >= mgmt_start.s_addr && req_l.s_addr <= mgmt_end.s_addr) {
		    espSetReturn(ep, mprCreateIntegerVar(1));
		    fclose(fp);
		    return 0;
		}
	    }
	    *buf = 0;
	}
	fclose(fp);
    }
    espSetReturn(ep, mprCreateIntegerVar(0));
    return 0;
}
#endif

static int checkLogoStat(EspRequest *ep, int argc, char **argv)
{
    int reqvlan;
    struct stat buf;
    bool ret = 0;
    char reqfile[MAX_VALUE_LEN];

    if (ejsParseArgs(argc, argv, "%d", &reqvlan) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    sprintf(reqfile, "/conf/tmpl_img/%d/tpl_logo.gif", reqvlan);
    if (stat(reqfile, &buf) == 0)
	ret = 1;
    espSetReturn(ep, mprCreateBoolVar(ret));
    return 0;
}

static int getRandomStr(EspRequest *ep, int argc, char **argv)
{
    struct timeval t;
    struct timezone z;
    unsigned int x;
    char buf[10];

    gettimeofday(&t, &z);
    x = ((t.tv_sec & 0xffff) << 16) + (t.tv_usec & 0xffff);
    sprintf(buf, "%8x", rand_r(&x));
    espSetReturnString(ep, buf);
    return 0;
}

static int Return403(EspRequest *ep, int argc, char **argv)
{
    char *html403 = "<HTML>\n"
		"<HEAD><TITLE>403 Forbidden</TITLE></HEAD>\n"
		"<BODY><H1>Forbidden</H1>You don't have permission to access this page on this server.<P></BODY>\n"
		"</HTML>\n";
    espSetResponseCode(ep, 403);
    espWriteFmt(ep, "%s", html403);
    espEvalScript(ep, "exit(0);", NULL, NULL);
    return 0;
}

void initGetAspLite(void)
{
    espDefineStringCFunction(0, "ifFunc", ifFunc, 0);
    espDefineStringCFunction(0, "getCurrentCfg", getCurrentCfg, 0);
    espDefineStringCFunction(0, "getCfg", getCfg, 0);
    espDefineStringCFunction(0, "getRcEnv", getRcEnv, 0);
    espDefineStringCFunction(0, "getFtime", getFtime, 0);
    espDefineStringCFunction(0, "getTime", getTime, 0);
#ifdef MGMT_IP
    espDefineStringCFunction(0, "checkMgmtIP", checkMgmtIP, 0);
#endif
    espDefineStringCFunction(0, "getOperRight", getOperRight, 0);
    espDefineStringCFunction(0, "getAccessRight", getAccessRight, 0);
    espDefineCFunction(0, "return403", Return403, 0);
    websAspDefine(T("getValue"), getValue);
    websAspDefine(T("getUpperCase"), getUpperCase);
    websAspDefine(T("getStatus"), getStatus);
    websAspDefine(T("getTitle"), getTitle);
    websAspDefine(T("getFooter"), getFooter);
    websAspDefine(T("getProtocol"), getProtocol);
    websAspDefine(T("getReboot"), getReboot);
    websAspDefine(T("strcapitalize"), strcapitalize);
    websAspDefine(T("getCustomizeCss"), getCustomizeCss);
    websAspDefine(T("_J"), _J);
    websAspDefine(T("_H"), _H);
    websAspDefine(T("feature"), feature);
}

/**
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initGetAsp(void)
{
    initGetAspLite();

    espDefineStringCFunction(0, "refreshUserList", refreshUserList, 0);
    espDefineStringCFunction(0, "getMgmtIPListJson", getMgmtIPListJson, 0);
    espDefineStringCFunction(0, "getWallAdListJson", getWallAdListJson, 0);
    espDefineStringCFunction(0, "getEnableWallAdListJson", getEnableWallAdListJson, 0);
    espDefineStringCFunction(0, "checkLogoStat", checkLogoStat, 0);
    espDefineStringCFunction(0, "getVlanByIP", getVlanByIP, 0);
    espDefineStringCFunction(0, "getRandomStr", getRandomStr, 0);
#ifdef GW_FUNC    
    espDefineStringCFunction(0, "getBlackListJson", getBlackListJson, 0);
#endif    
    espDefineStringCFunction(0, "getPrivilegeIPJson", getPrivilegeIPJson, 0);
    espDefineStringCFunction(0, "getPrivilegeMACJson", getPrivilegeMACJson, 0);
#ifdef WITH_ACC_MGMT
    websAspDefine(T("getOdUserJson"), getOdUserJson);
#endif    
    websAspDefine(T("getAPListJson"), getAPListJson);
    websAspDefine(T("getScanResult"), getScanResult);
    websAspDefine(T("getFunc"), getFunc);
    websAspDefine(T("getCandidate"), getCandidate);
    websAspDefine(T("getRF"), getRF);
    websAspDefine(T("getWDSidx"), getWDSidx);
#ifdef GW_FUNC    
    websAspDefine(T("getFirewallJson"), getFirewallJson);
    websAspDefine(T("getVlanList"), getVlanList);
    websAspDefine(T("getStaticRouteJson"), getStaticRouteJson);
    websAspDefine(T("getReservedIPJson"), getReservedIPJson);
    websAspDefine(T("getMappingJson"), getMappingJson);
#endif    
    websAspDefine(T("getJsonString"), getJsonString);
    websAspDefine(T("getWalledListJson"), getWalledListJson);
    websAspDefine(T("getRFList"), getRFList);
    websAspDefine(T("getRFofWDS"), getRFofWDS);
    websAspDefine(T("getWDS"), getWDS);
    websAspDefine(T("getPKGinfo"), getPKGinfo);
    websAspDefine(T("getCalibrationInfo"), getCalibrationInfo);
    websAspDefine(T("genOptions"), genOptions);
    websAspDefine(T("getDhcpLeaseJson"), getDhcpLeaseJson);
    websAspDefine(T("getMax"), getMax);
    websAspDefine(T("getTouchRebootFlag"), getTouchRebootFlag);
    websAspDefine(T("getTxPwrLv"), getTxPwrLv);
    websAspDefine(T("getVapWps"), getVapWps);
    websAspDefine(T("getIgdPortMapJson"), getIgdPortMapJson);
#ifdef CAPWAP
    websAspDefine(T("getACList"), getACList);
    websAspDefine(T("getWtpStatus"), getWtpStatus);
    websAspDefine(T("getTunStatus"), getTunStatus);
#endif
//    websFormDefine(T("getPing"), getPing);
#ifdef SUPPORT_REPORT_AGENT
    websAspDefine(T("getReport"), getReport);
#endif
    maDefineEgiForm("/getPing.egi", getPing);
#ifdef FULL_NET_UTILS
    maDefineEgiForm("/getTracert.egi", getTracert);
    maDefineEgiForm("/getARPTable.egi", getARPTable);
#endif
}
