/**
 *  \file web_mgnt.c
 *
 *  This is the cgi modules for A300 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/21/2006     0.1       Jimmy Yang     Initial version
 *
 */

#include "id.h"
#include "cipcfg.h"
#include "syscall.h"
#include "web_mgnt.h"
#include "webfunc.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


/**
 * set Management Accounts
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyMgntAccount(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *user_name, *u1_old_pwd, *u1_new_pwd, *u2_old_pwd, *u2_new_pwd;
    char_t *u3_new_pwd;
    char tmp[256], u1_pwd[MAX_VALUE_LEN], u2_pwd[MAX_VALUE_LEN], u3_pwd[MAX_VALUE_LEN];
    ITEM_LIST *list= NULL;
    CIPCFG *cipcfg = NULL;
    int ret, apply_u1=0, apply_u2=0;

    cipcfg=init_cipcfg();
    app = websGetVar(wp, T("apply"), T("true"));
    user_name = websGetVar(wp, T("user_name"), T(""));

    if(getUserGroupPermission(wp, GID_USER3) & PERM_WRITE){
        u3_new_pwd = websGetVar(wp, T("operator_new_pwd"), T(""));

        if (*u3_new_pwd != '\0'){ 
            ret = get_cfg_item(cipcfg, GID_USER3, "password", u3_pwd);
            if ( getOperationPermission(wp, OID_CHANGE_ALL_PWD) & PERM_EXEC ) {
                list=append_cfg_item(list, GID_USER3, "password", u3_new_pwd);
            }else{
                close_cipcfg(cipcfg);
                websRedirect(wp, "/utilities/password.asp?err_code=1" ); 
                return;
            }
        }
    }
    // User 2, less permission
    if(getUserGroupPermission(wp, GID_USER2) & PERM_WRITE){
        u2_new_pwd = websGetVar(wp, T("u2_new_pwd"), T(""));
        if(*u2_new_pwd == '\0') {
            u2_new_pwd = websGetVar(wp, T("manager_new_pwd"), T(""));
        }
        if (*u2_new_pwd != '\0'){ 
            ret = get_cfg_item(cipcfg, GID_USER2, "password", u2_pwd);
            if ( getOperationPermission(wp, OID_CHANGE_ALL_PWD) & PERM_EXEC ) { 
                list=append_cfg_item(list, GID_USER2, "password", u2_new_pwd);
                apply_u2 = 1;
            }else{
                close_cipcfg(cipcfg);
                websRedirect(wp, "/utilities/password.asp?err_code=1" ); 
                return;
            }
        }
    }

    // User 1, Top power
    if( (getUserGroupPermission(wp, GID_USER1) & PERM_WRITE) ||
      (getOperationPermission(wp, OID_CHANGE_ALL_PWD) & PERM_EXEC) ){
        u1_new_pwd = websGetVar(wp, T("u1_new_pwd"), T(""));
        if (*u1_new_pwd != '\0'){
            u1_old_pwd = websGetVar(wp, T("u1_old_pwd"), T(""));
            ret = get_cfg_item(cipcfg, GID_USER1, "password", u1_pwd);
            if (!strcmp(u1_pwd, u1_old_pwd)){
                list=append_cfg_item(list, GID_USER1, "password", u1_new_pwd);
                apply_u1 = 1;
            }else{
                close_cipcfg(cipcfg);
                websRedirect(wp, "/utilities/password.asp?err_code=1" ); 
                return;
            }
        }
    }
    
    print_list(list);
    ret=set_cfg_items_lite(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        close_cipcfg(cipcfg);
        websRedirect(wp, "/utilities/password.asp?err_code=3" ); 
        return;
    }

    close_cipcfg(cipcfg);
    websRedirect(wp, "/utilities/password.asp" ); 
    return;
}

void applyMgnt(webs_t wp, char_t *path, char_t *query)
{
    char_t *state, *ip, *port, *level, *snmp_state, *snmp_rocmty, *snmp_rwcmty, *rb_time;
    char_t *snmp_trap, *snmp_trap_receiver, *mgmt_vid;
    char_t *app;
    char_t *report_agent_state, *report_manager;
    char tmp[256];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx= 0;

    app         = websGetVar(wp, T("apply"), T("true"));
    if(getUserGroupPermission(wp, GID_SYSLOG) & PERM_WRITE){
        state       = websGetVar(wp, T("state"), T("disable"));
        if(!strncmp(state, "enable", 6)){
        	ip          = websGetVar(wp, T("server_ip"), T("0.0.0.0"));
        	port        = websGetVar(wp, T("server_port"), T("514"));
	}
        level       = websGetVar(wp, T("level"), T("3"));

        list        = append_cfg_item(list, GID_SYSLOG, "state", state);

        if(!strncmp(state, "enable", 6)){
        	list        = append_cfg_item(list, GID_SYSLOG, "server_ip", ip);
        	list        = append_cfg_item(list, GID_SYSLOG, "server_port", port);
	}
        list        = append_cfg_item(list, GID_SYSLOG, "level", level);
    }
    if(getUserGroupPermission(wp, GID_SNMP) & PERM_WRITE){
        snmp_state  = websGetVar(wp, T("snmp_state"), T("disable"));
        snmp_rocmty = websGetVar(wp, T("snmp_rocomty"), T("public"));
        snmp_rwcmty = websGetVar(wp, T("snmp_rwcomty"), T("private"));
        snmp_trap   = websGetVar(wp, T("snmp_trap"), T("disable"));
        if(!strncmp(snmp_trap, "enable", 6))
        	snmp_trap_receiver = websGetVar(wp, T("trap_receiver"), T("0.0.0.0"));
        
        list=append_cfg_item(list, GID_SNMP, "state", snmp_state);
	list=append_cfg_item(list, GID_SNMP, "trap", snmp_trap);
        if(!strncmp(snmp_state, "enable", 6)){
            list=append_cfg_item(list, GID_SNMP, "read", snmp_rocmty);
            list=append_cfg_item(list, GID_SNMP, "write", snmp_rwcmty);
            if(!strncmp(snmp_trap, "enable", 6))
            	list=append_cfg_item(list, GID_SNMP, "trap_server", snmp_trap_receiver);
        }
    }
#ifdef REPORT_AGENT
    if(getUserGroupPermission(wp, GID_REPORT_AGENT) & PERM_WRITE){
	report_agent_state	= websGetVar(wp, T("report_agent_state"), T("disable"));
	report_manager 	= websGetVar(wp, T("report_manager"), T("127.0.0.1"));
	list		= append_cfg_item(list, GID_REPORT_AGENT, "state", report_agent_state); 
	list		= append_cfg_item(list, GID_REPORT_AGENT, "report_manager", report_manager); 
    }
#endif
#ifdef VLAN_MGNT
    chkFeature("VLAN_MGNT", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, GID_WAN_IP) & PERM_WRITE)){
        mgmt_vid    = websGetVar(wp, T("mgmt_vid"), T("4096"));
        list        = append_cfg_item(list, GID_WAN_IP, "mgmt_vid", mgmt_vid);
    }
#endif
#ifdef UPNP_IGD
    chkFeature("UPNP_IGD", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, GID_UPNP) & PERM_WRITE)){
        state = websGetVar(wp, T("upnp"), T("disable"));
        list  = append_cfg_item(list, GID_UPNP, "state", state);
    }
#endif
#ifdef AUTO_REBOOT
    chkFeature("AUTO_REBOOT", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, GID_AUTO_REBOOT) & PERM_WRITE)){
        state   = websGetVar(wp, T("auto_reboot"), T("disable"));
        rb_time = websGetVar(wp, T("reboot_time"), T("03:00"));
        list  = append_cfg_item(list, GID_AUTO_REBOOT, "state", state);
        list  = append_cfg_item(list, GID_AUTO_REBOOT, "reboot_time", rb_time);
    }
#endif
#ifdef HOMEPAGE_REDIR
    chkFeature("HOMEPAGE_REDIR", tmp);
    if(tmp[0]=='Y') {
	char_t *url;
	state = websGetVar(wp, T("ho_state"), T("disable"));
	url = websGetVar(wp, T("ho_url"), T(""));
	list = append_cfg_item(list, GID_HOMEPAGE_REDIR, "state", state);
	list = append_cfg_item(list, GID_HOMEPAGE_REDIR, "url", url);
    }
#endif
#ifdef IPPNP
    chkFeature("IPPNP", tmp);
    if(tmp[0]=='Y') {
	state = websGetVar(wp, T("ippnp_state"), T("disable"));
	list = append_cfg_item(list, GID_IPPNP, "state", state);
    }
#endif
#ifdef SMTP_REDIR
    chkFeature("SMTP_REDIR", tmp);
    if(tmp[0]=='Y') {
	char_t *smtpip, *smtpport;
	state = websGetVar(wp, T("smtp_state"), T("disable"));
	smtpip = websGetVar(wp, T("smtp_ip"), T(""));
	smtpport = websGetVar(wp, T("smtp_port"), T(""));
	list = append_cfg_item(list, GID_SMTP_REDIR, "state", state);
	list = append_cfg_item(list, GID_SMTP_REDIR, "ip", smtpip);
	list = append_cfg_item(list, GID_SMTP_REDIR, "port", smtpport);
    }
#endif
    print_list(list);

    cipcfg=init_cipcfg();

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid; 
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, "/system/management.asp" );
}

/* (A800G:A800L) AP Bonding Operation Mode */
static void applyABOpmode(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int ret, applylist[] = {GID_BONDING, GID_END};
    char *app, *lan_style, *wds_style;
    char tmp[MAX_VALUE_LEN];
    char *returl = NULL;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "false");
    lan_style = maGetVar(rq, MA_FORM_OBJ, "lan_style", "0");
    wds_style = maGetVar(rq, MA_FORM_OBJ, "wds_style", "0");

    memset(tmp, 0, sizeof(tmp));
    chkFeature("AP_BONDING", tmp);
    if (*tmp != 'Y') {
	returl = "/system/opmode.asp?err_code=500";
	goto end;
    }
    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	returl = "/system/opmode.asp?err_code=501";
	goto end;
    }
    memset(tmp, 0, sizeof(tmp));
    ret = get_cfg_item(cipcfg, GID_OPMODE, "type", tmp);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get opmode type\n");
	returl = "/system/opmode.asp?err_code=502";
	goto end;
    }
    if (strcmp(tmp, "AP") != 0) {
	returl = "/system/opmode.asp?err_code=101";
	goto end;
    }

    // lan_style = (*lan_style == '1') ? "1" : "0";
    // wds_style = (*wds_style == '2') ? "2" : ((*wds_style == '1') ? "1" : "0");
    list = append_cfg_item(list, GID_BONDING, "lan_style", lan_style);
    list = append_cfg_item(list, GID_BONDING, "wds_style", wds_style);
    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	printf("Can't set items");
	returl = "/system/opmode.asp?err_code=503";
	goto end;
    }

    if (!strncmp("true", app, 4)) {
	/* FIXME: No relative actions for apply_cipcfg() */
	if (can_apply(applylist) == CIPCFG_SUCCESS) {
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	} else {
	    returl = "/utilities/reboot.asp";
	}
    }
end:
    if (cipcfg)
	close_cipcfg(cipcfg);
    if (!returl)
	returl = "/system/opmode.asp";
    maRedirect(rq, 302, returl);
}

/* (A800G) Gateway/AP Operation Mode */
void applyGWOpmode(webs_t wp, char_t *path, char_t *query)
{
    extern const unsigned int max_rf;
    extern const int gid_rf[];
    extern const char *config_rom;
    extern const char *restored_cfg_file;
    char_t *app, *opmode, *setmode, *rf_mode;
    char tmp[MAX_VALUE_LEN];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int i, ret, newmode = 0;

    app  = websGetVar(wp, T("apply"), T("true"));
    opmode = websGetVar(wp, T("opmode"), T(""));
    setmode = websGetVar(wp, T("setmode"), T(""));

    cipcfg = init_cipcfg();
    if (!cipcfg) {
        printf("initial cipcfg error.\n");
        //websRedirect(wp, T("/system/opmode.asp?err_code=501"));
        return;
    }
    *tmp = '\0';
    ret = get_cfg_item(cipcfg, GID_OPMODE, "type", tmp);
    // ret = get_sys_item("operating_mode", tmp);
    if (ret != CIPCFG_SUCCESS) {
        printf("Can't get opmode type\n");
        close_cipcfg(cipcfg);
        //websRedirect(wp, T("/system/opmode.asp?err_code=503"));
        return;
    }

    if (strcmp(opmode, tmp) != 0) {
        close_cipcfg(cipcfg);
        websRedirect(wp, T("/system/opmode.asp?err_code=101"));
        return;
    } else if (strcmp(setmode, tmp) != 0) {
        close_cipcfg(cipcfg);
        sprintf(tmp, "/etc/default-config_%s.xml", setmode);
        cipcfg = init_cipcfg_f(tmp);
        if (!cipcfg) {
            printf("initial cipcfg (%s) error.\n", tmp);
#if 0
            //websRedirect(wp, T("/system/opmode.asp?err_code=504"));
            return;
#else
            cipcfg = init_cipcfg_f(DEFAULT_CONFIG);
            if (!cipcfg) {
                printf("initial cipcfg (%s) error.\n", DEFAULT_CONFIG);
                //websRedirect(wp, T("/system/opmode.asp?err_code=505"));
                return;
            }
            //list = append_cfg_item(list, GID_OPMODE, "type", setmode);
#endif
        }
        newmode = 1;
        //list = append_cfg_item(list, GID_OPMODE, "type", setmode);
    }
    list = append_cfg_item(list, GID_OPMODE, "type", setmode);
    for (i=0; i<max_rf; i++) {
        sprintf(tmp, "rf%d_mode", i+1);
        rf_mode = websGetVar(wp, T(tmp), T(""));
        if (rf_mode && *rf_mode) {
            if (strcmp(rf_mode, "AP") == 0) {
                rf_mode = "ap";
            } else if ((i == (max_rf - 1)) && (strcmp(rf_mode, "SCAN") == 0)) {
                rf_mode = "scan";
            } else {
                rf_mode = "wds,wdsd";
            }
            list = append_cfg_item(list, gid_rf[i], "roles", rf_mode);
        }
    }
    print_list(list);

    if (!newmode)
        ret = set_cfg_items(cipcfg, list);
    else {
        ret = set_cfg_items_f(cipcfg, list, restored_cfg_file);
	sprintf(tmp, "rm -fr /conf/certificate/; rm -fr /conf/tmpl_img; rm -f /conf/user_db");
	ret = cip_system(tmp);
	if (WEXITSTATUS(ret) == 1)
	    printf("ERROR reset to default certificate, template images and user database.");
    }
    // free_item_list(list);
    close_cipcfg(cipcfg);
    if (ret != CIPCFG_SUCCESS) {
        printf("Can't set items");
        // websRedirect(wp, T("/system/opmode.asp?err_code=502"));
        return;
    }

    if (newmode || !strncmp("true", app, 4))
        websRedirect(wp, T("/utilities/pre_reboot.asp"));
    else
        websRedirect(wp, T("/system/opmode.asp"));
    return;
}

#ifdef OP_MODE
/* (WCB1000/A520/A560) CPE/AP Operation Mode */
void applyOpmode(webs_t wp, char_t *path, char_t *query)
{
    char_t *om, *app;
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char tmp[256];
    int ret;

    app  = websGetVar(wp, T("apply"), T("true"));

    if(getUserGroupPermission(wp, GID_OPMODE) & PERM_WRITE){
        om     = websGetVar(wp, T("optmode"), T("CPE"));
        list   = append_cfg_item(NULL, GID_OPMODE, "type", om);
	print_list(list);
#ifdef RESET_CFG_CHANGE_MODE
	ret = reset_cipcfg();
	if (ret != CIPCFG_SUCCESS){
	    printf("ERROR reset to default. error code : %d\n", ret);
	}
#endif
        cipcfg = init_cipcfg();
        ret = set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return;
        }
        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
	    // Do nothing
        }
        close_cipcfg(cipcfg);
        if ( !strncmp("true", app, 4 ) )
            websRedirect(wp, "/utilities/pre_reboot.asp" );
        else
            websRedirect(wp, "/system/optmode.asp" );
    } else
        websRedirect(wp, "/system/optmode.asp" );
}
#endif

static void applyUserLog(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    char item_value[MAX_VALUE_LEN];
    char *userlog_syslog_enable = NULL, *sessionlog_syslog_enable = NULL, *app = NULL;
    char *userlog_syslog_ip = NULL, *userlog_syslog_port = NULL;
    char *sessionlog_syslog_ip = NULL, *sessionlog_syslog_port = NULL;
    ITEM_LIST *list = NULL;
    int ret, reboot = 0;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1)); 
    int applist_idx= 0;

    cipcfg = init_cipcfg();
    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    userlog_syslog_enable = maGetVar(rq, MA_FORM_OBJ, "userlog_syslogd_enable", "Disable");
    get_cfg_item(cipcfg, GID_USERLOG, "syslog_en", item_value);
    if (strcmp(item_value, userlog_syslog_enable))
	list = append_cfg_item(list, GID_USERLOG, "syslog_en", userlog_syslog_enable);
    sessionlog_syslog_enable = maGetVar(rq, MA_FORM_OBJ, "sessionlog_syslogd_enable", "Disable");
    get_cfg_item(cipcfg, GID_SESSIONLOG, "syslog_en", item_value);
    if (strcmp(item_value, sessionlog_syslog_enable)) {
	list = append_cfg_item(list, GID_SESSIONLOG, "syslog_en", sessionlog_syslog_enable);
	reboot = 1;
    }
    if (!strcmp(userlog_syslog_enable, "Enable")) {
	userlog_syslog_ip = maGetVar(rq, MA_FORM_OBJ, "userlog_syslogd_ip", NULL);
	if (userlog_syslog_ip == NULL) {
	    get_cfg_item(cipcfg, GID_USERLOG_SYSLOG, "server_ip", item_value);
	    userlog_syslog_ip = T(item_value);
	}
	list = append_cfg_item(list, GID_USERLOG_SYSLOG, "server_ip", userlog_syslog_ip);
	userlog_syslog_port = maGetVar(rq, MA_FORM_OBJ, "userlog_syslogd_port", NULL);
	if (userlog_syslog_port == NULL) {
	    get_cfg_item(cipcfg, GID_USERLOG_SYSLOG, "server_port", item_value);
	    userlog_syslog_port = T(item_value);
	}
	list = append_cfg_item(list, GID_USERLOG_SYSLOG, "server_port", userlog_syslog_port);
    }
    if (!strcmp(sessionlog_syslog_enable, "Enable")) {
	sessionlog_syslog_ip = maGetVar(rq, MA_FORM_OBJ, "sessionlog_syslogd_ip", NULL);
	if (sessionlog_syslog_ip == NULL) {
	    get_cfg_item(cipcfg, GID_SESSIONLOG_SYSLOG, "server_ip", item_value);
	    sessionlog_syslog_ip = T(item_value);
	}
	list = append_cfg_item(list, GID_SESSIONLOG_SYSLOG, "server_ip", sessionlog_syslog_ip);
	sessionlog_syslog_port = maGetVar(rq, MA_FORM_OBJ, "sessionlog_syslogd_port", NULL);
	if (sessionlog_syslog_port == NULL) {
	    get_cfg_item(cipcfg, GID_SESSIONLOG_SYSLOG, "server_port", item_value);
	    sessionlog_syslog_port = T(item_value);
	}
	list = append_cfg_item(list, GID_SESSIONLOG_SYSLOG, "server_port", sessionlog_syslog_port);
    }
    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return;
    }
    while(list != NULL) {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0) {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS && reboot == 0){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
            free(applylist);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
            free(applylist);
	    return;
	}
    }
    close_cipcfg(cipcfg);
    maRedirect(rq, 302, "/status/user_log.asp");
}


static void applyMgmtIP(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    int max_entry = 10, i, ret;
    ITEM_LIST *list = NULL;
    char *ip, *app, buf[MAX_VALUE_LEN];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1)); 
    int applist_idx= 0;

    cipcfg = init_cipcfg();
    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    for(i = 0; i < max_entry; i++) {
	sprintf(buf, "ip%d", i);
	ip = maGetVar(rq, MA_FORM_OBJ, buf, NULL);
	if (ip != NULL) {
	    sprintf(buf, "entry:%d.ip", i);
	    list = append_cfg_item(list, GID_REMOTE_MANAGE, buf, ip);
	}
    }
    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	free(applylist);
	close_cipcfg(cipcfg);
	return;
    }
    while(list != NULL) {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0) {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
            free(applylist);
	    return;
	}
    }
    free(applylist);
    close_cipcfg(cipcfg);
    maRedirect(rq, 302, "/system/RemoteMgmtIP.asp");
}
/**
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initMgntForm(void)
{
    websFormDefine(T("applyMgntAccount"), applyMgntAccount);
    websFormDefine(T("applyMgnt"), applyMgnt);
    websFormDefine(T("applyGWOpmode"), applyGWOpmode);
#ifdef OP_MODE
    websFormDefine(T("applyOpmode"), applyOpmode);
#endif
    maDefineEgiForm("/applyABOpmode.egi", applyABOpmode);
    maDefineEgiForm("/applyUserLog.egi", applyUserLog);
    maDefineEgiForm("/applyMgmtIP.egi", applyMgmtIP);
}
