/**
 *  \file web_net.c
 *
 *  This is the cgi modules for A300 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/20/2006     0.1       Jimmy Yang     Initial version
 *
 */

#include "id.h"
#include "cipcfg.h"
#include "syscall.h"
#include "web_net.h"
#include "webfunc.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


/**
 * Get Network configuration form cipcfg
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int getNet(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t *group_name, *item_name, item_value[256];
    CIPCFG *cipcfg = NULL;

    if (ejArgs(argc, argv, T("%s %s"), &group_name, &item_name) < 2) {
        websError(wp, 400, T("Insufficient args\n"));
        return -1;
    }

    cipcfg=init_cipcfg();

    if (!strcmp(item_name, "type")) {
        get_cfg_item_by_str(cipcfg, group_name, "type", item_value);
    }
    else if (!strcmp(item_name, "addr")) {
        get_cfg_item_by_str(cipcfg, group_name, "addr", item_value);
    }
    else if (!strcmp(item_name, "netmask")) {
        get_cfg_item_by_str(cipcfg, group_name, "netmask", item_value);
    }
    else if (!strcmp(item_name, "gateway")) {
        get_cfg_item_by_str(cipcfg, group_name, "gateway", item_value);
    }
    else if (!strcmp(item_name, "dns1")) {
        get_cfg_item_by_str(cipcfg, group_name, "dns1", item_value);
    }
    else if (!strcmp(item_name, "dns2")) {
        get_cfg_item_by_str(cipcfg, group_name, "dns2", item_value);
    }
    else {
        item_value[0] = '\0';
    }

    close_cipcfg(cipcfg);

    return websWrite(wp, T(item_value));
}

/**
 * Renew the IP, restart DHCP client
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyRenewIp(webs_t wp, char_t *path, char_t *query)
{
	extern const char *current_config;
	ITEM_LIST *list = NULL;
	CIPCFG *cipcfg = NULL;
	int ret;
	char tmp[256];
		
	if(getUserGroupPermission(wp, GID_WAN_IP) & PERM_WRITE){
		cipcfg = init_cipcfg_f(current_config);	
		get_cfg_item(cipcfg, GID_WAN_IP, "type", tmp);
		
		if(!strncmp(tmp, "dhcp", 4))
			apply_sys(CMD_RENEW_IP);
		else
		{
			list=append_cfg_item(list, GID_WAN_IP, "type", "dhcp");			
			print_list(list);		
			ret=set_cfg_items(cipcfg, list);
			if (ret != CIPCFG_SUCCESS) {
				return;
			}	    			
			apply_cfg(cipcfg);
		}		
	    
		close_cipcfg(cipcfg); 
	}
	
	websRedirect(wp, "/system/network.asp"); 
}

/**
 * set WAN interface (IP, netmask, GW, DNS) to device
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyNetWan(webs_t wp, char_t *path, char_t *query)
{
    char_t *app;
    char tmp[256], item_value[MAX_VALUE_LEN];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret, i;
    char_t *val;
    char   *wan_ct[] = {"type", "addr", "netmask", "gateway", "dns1", "dns2", NULL};
    char   *wan_ht[] = {"type", "addr", "netmask", "gateway", "dns1", "dns2", NULL};
    char   *wan_def[] = {"fixed", "192.168.1.1", "255.255.255.0", "192.168.1.254", "", "", NULL};
    char opmode[MAX_VALUE_LEN];
    int err_code = 0, nif_code = 0;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	// websRedirect(wp, "/system/network.asp?err_code=501"); 
	return;
    }
    get_cfg_item(cipcfg, GID_OPMODE, "type", opmode);

    app = websGetVar(wp, T("apply_t"), T("0"));
    if(getUserGroupPermission(wp, GID_WAN_IP) & PERM_WRITE){
	if (!strcmp(opmode, "AP")) {
	    for (i=0; wan_ht[i]; i++) {
		val = websGetVar(wp, T(wan_ht[i]), T(wan_def[i]));
		list = append_cfg_item(list, GID_WAN_IP, wan_ct[i], val);
	    }
	}
	else if(!strcmp(opmode, "CPE")) {
	    for (i=0; wan_ht[i]; i++) {
		val = websGetVar(wp, T(wan_ht[i]), NULL);
		if(val == NULL) {
		    get_cfg_item(cipcfg, GID_WAN_IP, wan_ht[i], item_value);
		    val = T(item_value);
		}
	    list = append_cfg_item(list, GID_WAN_IP, wan_ct[i], val);
	    }	
	}
#ifdef GW_FUNC
	else {
	    val = websGetVar(wp, T("type"), T(""));
	    if (!strcmp(opmode, "GW") && !strcmp(val, "fixed")) {
		extern int gid_vlan[], gid_vlan_lan[];
		int max_vlan = 8, j;
		char *wan_ip, *wan_mask;
		char ipaddr[MAX_VALUE_LEN], ipmask[MAX_VALUE_LEN];
		unsigned long wan_bcast, wan_naddr;
		unsigned long bcast, naddr;

		wan_ip = websGetVar(wp, T("addr"), T(""));
		wan_mask = websGetVar(wp, T("netmask"), T(""));
		if (get_ipmask(wan_ip, wan_mask, &wan_bcast, &wan_naddr) <= 0) {
		    err_code = 101;
		    nif_code = 1;
		    goto fail;
		}

		if (get_rc_env("MAX_VLAN", tmp, sizeof(tmp)) != NULL)
		    max_vlan = atoi(tmp);
		for (j = 0; j <= max_vlan; j++) {
		    get_cfg_item(cipcfg, gid_vlan[j], "status", tmp);
		    if (strcmp(tmp, "enable") != 0)
			continue;
		    get_cfg_item(cipcfg, gid_vlan_lan[j], "ip", ipaddr);
		    get_cfg_item(cipcfg, gid_vlan_lan[j], "netmask", ipmask);
		    if (get_ipmask(ipaddr, ipmask, &bcast, &naddr) <= 0)
			continue;
		    if (naddr <= wan_bcast && bcast >= wan_naddr) {
			err_code = 102;
			nif_code = j;
			goto fail;
		    }
		}
	    }
	    for (i=0; wan_ht[i]; i++) {
		val = websGetVar(wp, T(wan_ht[i]), NULL);
		if(val == NULL) {
		    get_cfg_item(cipcfg, GID_WAN_IP, wan_ht[i], item_value);
		    val = T(item_value);
		}
		list = append_cfg_item(list, GID_WAN_IP, wan_ct[i], val);
	    }
	}
#endif
#ifdef PPPOE 
        chkFeature("PPPOE", tmp);
        if (tmp[0] == 'Y') {
            char_t *username, *password, *mtu, *mss, *ondemand, *ondemand_time;
            int ondemand_s;

            username = websGetVar(wp, T("pppoe_username"), "");
            password = websGetVar(wp, T("pppoe_password"), "");
            mtu = websGetVar(wp, T("mtu"), "1492");
            mss = websGetVar(wp, T("mss"), "1400");
            ondemand = websGetVar(wp, T("ondemand"), "disable");
            ondemand_time = websGetVar(wp, T("ondemand_time"), "0");
            ondemand_s = atoi(ondemand_time);
            ondemand_s = ondemand_s * 60;

            list = append_cfg_item(list, GID_WAN_IP, "pppoe.username", username);
            list = append_cfg_item(list, GID_WAN_IP, "pppoe.password", password);
            list = append_cfg_item(list, GID_WAN_IP, "pppoe.mtu", mtu);
            list = append_cfg_item(list, GID_WAN_IP, "pppoe.clampmss", mss);
            if(!strcmp(ondemand, "disable"))
                list = append_cfg_item(list, GID_WAN_IP, "pppoe.ondemand", "no");
            else {
                sprintf(ondemand_time, "%d", ondemand_s);
                list = append_cfg_item(list, GID_WAN_IP, "pppoe.ondemand", ondemand_time);
            }
        }
#endif

#ifdef BANDWIDTH_CTRL
        chkFeature("BANDWIDTH_CTRL", tmp);
	if (tmp[0]=='Y' && !strcmp(opmode, "CPE")) {	// CPE mode
            val  = websGetVar(wp, "bandwidth_up", "none");
            list = append_cfg_item(list, GID_WAN_IP, "bandwidth_up", val);
            val  = websGetVar(wp, "bandwidth_down", "none");
            list = append_cfg_item(list, GID_WAN_IP, "bandwidth_down", val);
        } else if (tmp[0]=='Y' && !strcmp(opmode, "GW")) {	// GW mode
            val  = websGetVar(wp, "up_rate", "0");
            list = append_cfg_item(list, GID_POLICY0_BANDWIDTH, "up_rate", val);
            val  = websGetVar(wp, "down_rate", "0");
            list = append_cfg_item(list, GID_POLICY0_BANDWIDTH, "down_rate", val);
        }
#endif
    }
#ifdef STP
    chkFeature("STP", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, GID_NET_L2) & PERM_WRITE)){
        val  = websGetVar(wp, T("stp"), T("disable"));
        list = append_cfg_item(list, GID_NET_L2, "stp", val);
    }
#endif
#ifdef DDNS
    chkFeature("DDNS", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, GID_DDNS) & PERM_WRITE)){
        char   *ddns_ct[] = {"state", "provider", "host", "username", "password",NULL};
        char   *ddns_ht[] = {"ddns", "provider", "host", "username", "password",NULL};
        char   *ddns_def[]= {"enable", "dyndns.org(dynamic)", "", "", "",NULL};
        for(i=0;ddns_ht[i];i++){
            val  = websGetVar(wp, T(ddns_ht[i]), T(ddns_def[i]));
            list = append_cfg_item(list, GID_DDNS, ddns_ct[i], val);
            if((strcmp(ddns_ct[i], "state") == 0) && (strcmp(val, T("disable"))==0))
                break;
        }
    }
#endif
#ifdef FAILACT
    chkFeature("FAILACT", tmp);
    if(tmp[0]=='Y'){
	val  = websGetVar(wp, T("failact"), T("disable"));
	list = append_cfg_item(list, GID_FAILACT, "status", val);
	if (!strcmp(val, "enable")) {
	    int i, max;
	    char buf[MAX_VALUE_LEN];
	    val  = websGetVar(wp, T("fa_message"), T("Sorry, the Server is temporarily unavailable."));
	    list = append_cfg_item(list, GID_FAILACT, "message", val);
	    get_cfg_item(cipcfg, GID_FAILURL, "max", item_value);
	    max = atoi(item_value);
	    for(i = 0; i < max; i++) {
		sprintf(buf, "fa_url%d", i);
		val  = websGetVar(wp, T(buf), T(""));
		sprintf(buf, "entry:%d.url", i);
		list = append_cfg_item(list, GID_FAILURL, buf, val);
	    }
	}
    }
#endif
#ifdef LAN
    chkFeature("LAN", tmp);
    if(tmp[0]=='Y'){
        if(getUserGroupPermission(wp, GID_LAN) & PERM_WRITE){
            char   *lan_ct[] = {"addr", "netmask",NULL};
            char   *lan_ht[] = {"lan_addr", "lan_netmask",NULL};
            char   *lan_def[]= {"192.168.10.1", "255.255.255.0",NULL};
            for(i=0;lan_ht[i];i++){
                val  = websGetVar(wp, T(lan_ht[i]), T(lan_def[i]));
                list = append_cfg_item(list, GID_LAN, lan_ct[i], val);
            }
        }
        if(getUserGroupPermission(wp, GID_LAN_DHCP) & PERM_WRITE){
            char   *dhcp_ct[] = {"state", "start_ip", "end_ip", "dns1", "dns2", "wins", "domain", "lease_time",NULL};
            char   *dhcp_ht[] = {"dhcp_serv", "dhcp_ip_start", "dhcp_ip_end", "dns_serv_1", "dns_serv_2", "wins_serv", "domain_name", "dhcp_lease",NULL};
            char   *dhcp_def[]= {"enable", "192.168.10.2", "192.168.10.254", "192.168.1.1", "", "192.168.1.1", "", "86400",NULL};
            for(i=0;dhcp_ht[i];i++){
                val  = websGetVar(wp, T(dhcp_ht[i]), T(dhcp_def[i]));
                list = append_cfg_item(list, GID_LAN_DHCP, dhcp_ct[i], val);
                if((strcmp(dhcp_ct[i], "state") == 0) && (strcmp(val, T("disable"))==0))
                    break;
            }
        }
    }
#endif

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    // free_item_list(list);
    close_cipcfg(cipcfg);

    if (ret != CIPCFG_SUCCESS) {
	// websRedirect(wp, "/system/network.asp?err_code=502"); 
	return;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("1", app, 1 ) ){
        websRedirect(wp, "/utilities/pre_reboot.asp" );
        return;
    }

    websRedirect(wp, "/system/network.asp"); 
    return;

fail:
    // free_item_list(list);
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/network.asp?err_code=%d&nif_code=%d", err_code, nif_code);
    websRedirect(wp, T(tmp)); 
    return;
}

/*
 * set up DMZ 
 */
void applyDMZ(webs_t wp, char_t *path, char_t *query)
{
    char_t  *state, *dmz_ip, *app;
    char tmp[256];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int ret;
    int applylist[] = {GID_DMZ, GID_END};

    if(getUserGroupPermission(wp, GID_DMZ) & PERM_WRITE){
        app = websGetVar(wp, T("apply"), T("true"));
        dmz_ip = websGetVar(wp, T("dmz_ip"), T("192.168.1.1"));
        state = websGetVar(wp, T("state"), T("disable"));

        list=append_cfg_item(NULL, GID_DMZ, "dmz_ip", dmz_ip);
        list=append_cfg_item(list, GID_DMZ, "state", state);

        print_list(list);

        cipcfg=init_cipcfg();

        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return ;
        }

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                return;
            }
        }

        close_cipcfg(cipcfg);
    }
    websRedirect(wp, "/firewall/dmz.asp" ); 
}


/*
 * set up URL Filter
 */
void applyURLFilter(webs_t wp, char_t *path, char_t *query)
{
    char_t  *act, *new_url, *state, *del_item, *app;
    char tmp[256], item_value[MAX_VALUE_LEN];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int i, ret;
    int applylist[] = {GID_URL_FILTER, GID_END};

    if(getUserGroupPermission(wp, GID_URL_FILTER) & PERM_WRITE){
        act = websGetVar(wp, T("act"), T(""));
        app = websGetVar(wp, T("apply"), T("true"));
        cipcfg = init_cipcfg();
        if (!strncmp("add", act, 3)){
            new_url = websGetVar(wp, T("url"), T(""));
            // find an empty entry to add.
            item_value[0] = '1';
            for(i=0; *item_value!='\0'; i++){
                sprintf(tmp, "url_filter_entry:%d.url", i);
                get_cfg_item(cipcfg, GID_URL_FILTER, tmp, item_value);
            }
            sprintf(tmp, "url_filter_entry:%d.url", i-1 );
            list=append_cfg_item(NULL, GID_URL_FILTER, tmp, new_url);
            sprintf(tmp, "url_filter_entry:%d.state", i-1 );
            list=append_cfg_item(list, GID_URL_FILTER, tmp, "enable");
        }else if (!strncmp("delete", act, 6)){
            del_item = websGetVar(wp, T("del_item"), T(""));
            i = atoi(del_item);
            sprintf(tmp, "url_filter_entry:%d.url", i-1 );
            list=append_cfg_item(NULL, GID_URL_FILTER, tmp, "");
            sprintf(tmp, "url_filter_entry:%d.state", i-1 );
            list=append_cfg_item(list, GID_URL_FILTER, tmp, "disable");
        }else if (!strncmp("save", act, 4)){
            state = websGetVar(wp, T("state1"), T("disable"));
            list=append_cfg_item(NULL, GID_URL_FILTER, "url_filter_entry:0.state", state);
            for (i=1; i<20; i++){
                sprintf(tmp, "state%d", i+1);
                state = websGetVar(wp, T(tmp), T("disable"));
                sprintf(tmp, "url_filter_entry:%d.state", i);
                list=append_cfg_item(list, GID_URL_FILTER, tmp, state);
            }
        }

        print_list(list);

        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return ;
        }

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                return;
            }
        }

        close_cipcfg(cipcfg);
    }

    websRedirect(wp, "/firewall/urlfilter.asp" ); 
}


/*
 * set up IP Redirect
 */
void applyRedirect(webs_t wp, char_t *path, char_t *query)
{
    char_t  *act, *name, *from_port, *to_port, *to_ip, *protocol, *state, *entry, *app;
    char tmp[256], item_value[MAX_VALUE_LEN], edit_state[MAX_VALUE_LEN];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int i, ret;
    int applylist[] = {GID_IP_REDIRECT, GID_END};

    if(getUserGroupPermission(wp, GID_IP_REDIRECT) & PERM_WRITE){
        act = websGetVar(wp, T("act"), T(""));
        app = websGetVar(wp, T("apply"), T("true"));
        cipcfg = init_cipcfg();
	strcpy(edit_state, "enable");
        if ( !strncmp("edit", act, 4) || !strncmp("add", act, 3) ){
            if (!strncmp("edit", act, 4)) {
                entry = websGetVar(wp, T("entry"), T(""));
                i = atoi(entry);
		sprintf(tmp, "redir_entry:%d.state", i );
		get_cfg_item(cipcfg, GID_IP_REDIRECT, tmp, edit_state);
            }else if (!strncmp("add", act, 3)){
                // find an empty entry to add.
                item_value[0] = '1';
                for(i=0; *item_value!='\0'; i++){
                    sprintf(tmp, "redir_entry:%d.to_ip", i);
                    get_cfg_item(cipcfg, GID_IP_REDIRECT, tmp, item_value);
                }
                i-- ;
            }
            name = websGetVar(wp, T("name"), T(""));
            from_port = websGetVar(wp, T("from_port"), T(""));
              to_port = websGetVar(wp, T("to_port"), T(""));
              to_ip   = websGetVar(wp, T("to_ip"), T(""));
            protocol  = websGetVar(wp, T("protocol"), T(""));
            sprintf(tmp, "redir_entry:%d.state", i );
            list=append_cfg_item(NULL, GID_IP_REDIRECT, tmp, edit_state);
            sprintf(tmp, "redir_entry:%d.name", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, name);            
            sprintf(tmp, "redir_entry:%d.from_port", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, from_port);
            sprintf(tmp, "redir_entry:%d.to_port", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, to_port);
            sprintf(tmp, "redir_entry:%d.to_ip", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, to_ip);
            sprintf(tmp, "redir_entry:%d.protocol", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, protocol);
        }else if (!strncmp("delete", act, 6)){
            entry = websGetVar(wp, T("entry"), T(""));
            i = atoi(entry);
            sprintf(tmp, "redir_entry:%d.state", i );
            list=append_cfg_item(NULL, GID_IP_REDIRECT, tmp, "disable");
            sprintf(tmp, "redir_entry:%d.name", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, "");            
            sprintf(tmp, "redir_entry:%d.from_port", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, "");
            sprintf(tmp, "redir_entry:%d.to_port", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, "");
            sprintf(tmp, "redir_entry:%d.to_ip", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, "");
            sprintf(tmp, "redir_entry:%d.protocol", i );
            list=append_cfg_item(list, GID_IP_REDIRECT, tmp, "");
        }else if (!strncmp("save", act, 4)){
            state = websGetVar(wp, T("state0"), T("disable"));
            list=append_cfg_item(NULL, GID_IP_REDIRECT, "redir_entry:0.state", state);
            for (i=1; i<20; i++){
                sprintf(tmp, "state%d", i);
                state = websGetVar(wp, T(tmp), T("disable"));
                sprintf(tmp, "redir_entry:%d.state", i);
                list=append_cfg_item(list, GID_IP_REDIRECT, tmp, state);
            }
        }

        print_list(list);

        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return ;
        }

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                return;
            }
        }

        close_cipcfg(cipcfg);
    }

    websRedirect(wp, "/firewall/redirect.asp" ); 
}

static void applyVlanQosMap(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *qosac, *remark, *status;
    char buf[MAX_VALUE_LEN];
    int i, applylist[2] = {GID_VLAN_QOS_MAP, GID_END}, ret;
    int max_entry;
    ITEM_LIST *list = NULL;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    max_entry = atoi(maGetVar(rq, MA_FORM_OBJ, "max_entry", "1"));
    status = maGetVar(rq, MA_FORM_OBJ, "status", "1");
    list = append_cfg_item(list, GID_VLAN_QOS_MAP, "state", status);
    if (strcmp(status, "enable") == 0) {
	for (i = 0; i < max_entry; i++) {
	    sprintf(buf, "vlanid%d", i);
	    vlanid = maGetVar(rq, MA_FORM_OBJ, buf, "");
	    if (!strcmp(vlanid, "")) {
		sprintf(buf, "entry:%d.vlanid", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, vlanid);
		sprintf(buf, "entry:%d.qosac", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, "0x0");
		sprintf(buf, "entry:%d.remark", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, "");
	    }
	    else {
		sprintf(buf, "entry:%d.vlanid", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, vlanid);
		sprintf(buf, "qosac%d", i);
		qosac = maGetVar(rq, MA_FORM_OBJ, buf, "0x0");
		sprintf(buf, "entry:%d.qosac", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, qosac);
		sprintf(buf, "remark%d", i);
		remark = maGetVar(rq, MA_FORM_OBJ, buf, "");
		sprintf(buf, "entry:%d.remark", i);
		list = append_cfg_item(list, GID_VLAN_QOS_MAP, buf, remark);
	    }
	}
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("Initialize cipcfg error.\n");
	return;
    }
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return;
    }
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
	    return;
	}
    }
    close_cipcfg(cipcfg);
    maRedirect(rq, 302, "/system/vlan_setting.asp");
}

/**
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initNetForm(void)
{
    websFormDefine(T("applyNetWan"), applyNetWan);
    websFormDefine(T("applyRenewIp"), applyRenewIp);
    websAspDefine(T("getNet"), getNet);
    websFormDefine(T("applyDMZ"), applyDMZ);
    websFormDefine(T("applyURLFilter"), applyURLFilter);
    websFormDefine(T("applyRedirect"), applyRedirect);
    maDefineEgiForm("/applyVlanQosMap.egi", applyVlanQosMap);
}
