#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include "cipcfg.h"
#include "syscall.h"
#include "id.h"

#include "web_permission.h"

static int check_permission(char *cookie_username, char *cookie_password) 
{
    char opmode[MAX_VALUE_LEN], allowmode[MAX_VALUE_LEN];
    char username[MAX_VALUE_LEN], password[MAX_VALUE_LEN];
    extern const int gid_manager_userid[];
    CIPCFG *cipcfg = NULL;
    int ret = 1, i;

    if (!cookie_username || !cookie_password)
        return ret;
    if((cipcfg = init_cipcfg()) == NULL)
        return ret;

    for(i=0; gid_manager_userid[i] != GID_END; i++) {
        get_cfg_item(cipcfg, gid_manager_userid[i], "name", username);
        /* Find the corresponding user name */
        if( !strcmp(cookie_username, username) ) {
            get_cfg_item(cipcfg, gid_manager_userid[i], "password", password);

            /* Check allow mode */
            get_cfg_item(cipcfg, gid_manager_userid[i], "allowmode", allowmode);
            if (*allowmode) {
		char *ptr, *p;
		int match = 0;

		get_cfg_item(cipcfg, GID_OPMODE, "type", opmode);
                p = strtok_r(allowmode, ",", &ptr);
                while(p != NULL) { 
		    if(!strcmp(p, opmode)) {
			match = 1;
			break;
                    }
		    p = strtok_r(NULL, ",", &ptr);
                }

                /* allowmode is not empty and opmode is not in allowmode.
		 * so we break and return 1
		 */
		if( match == 0)
		    break;
	    }

	    if (!strcmp(crypt(password, cookie_password), cookie_password))
		ret = 0;
	    break;
        }
    }

    close_cipcfg(cipcfg);
    return ret;
}
/*
 *  argument 1 : user name
 *  argument 2 : password
 */
static int chkPermission(EspRequest *ep, int argc, char **argv)
{
    int ret = 1;

    if (argv && argv[0] && argv[1]) {
        ret = check_permission(argv[0], argv[1]);
    }

    ejsSetReturnValue(ep->eid, mprCreateIntegerVar(ret));
    return 0;
}

unsigned int getUserGroupPermission(MaRequest *rq, unsigned int g_id)
{
#ifdef PERMISSION_CONTROL_BY_GID
    unsigned int    perm = PERM_NONE;
    unsigned int    ug   = USER_GROUP_NONE;

    char username[MAX_VALUE_LEN]; 
    memset(username, '\0', MAX_VALUE_LEN);

    getCookieByName(rq, "username", username);

    ug   = getUserGroup(username);
    perm = getConfigPermissionByGID(g_id);

    return GET_GRP_PERM(ug, perm);
#else
    return (PERM_READ|PERM_WRITE|PERM_EXEC);
#endif
}

unsigned int getOperationPermission(MaRequest *rq, unsigned int o_id)
{
#ifdef PERMISSION_CONTROL_BY_GID
    unsigned int    perm = PERM_NONE;
    unsigned int    ug   = USER_GROUP_NONE;
    char username[MAX_VALUE_LEN]; 
    memset(username, '\0', MAX_VALUE_LEN);

    getCookieByName(rq, "username", username);
    
    ug   = getUserGroup(username);
    perm = getOperationPermissionByOID(o_id);
    
    return GET_GRP_PERM(ug, perm);
#else
    return (PERM_READ|PERM_WRITE|PERM_EXEC); 
#endif
}

int getCookieByName(MaRequest *rq, char *cookie_name, char *ret_str)
{
    char *cookies, *cookie_content = NULL;
    int ret = 0;

    if (ret_str != NULL)
	*ret_str = '\0';
    if (!rq || !cookie_name || !ret_str)
	return -1;
    
    cookies = maGetCookie(rq); 
    if (cookies != NULL) { 
        cookie_content = strstr(cookies, cookie_name);
    }

    if (cookie_content != NULL) {
	int i = 0, j = 0, len;
        while (cookie_content[i] != ';' && i < strlen(cookie_content)) {
            if (cookie_content[i] == '=') {
                j=i+1;
            }
            i++;
        }
	len = ((i-j) < MAX_VALUE_LEN) ? (i-j) : (MAX_VALUE_LEN - 1);
        strncpy(ret_str, cookie_content+j, len);
	ret_str[len] = '\0';
	ret = 1;
    }
    return ret;
}

/* Get Cookies: The first argument is cookie's name */
void getCookies(EspRequest *ep, int argc, char **argv)
{
    MaRequest *rq = espGetRequestHandle(ep);
    char ret_str[MAX_VALUE_LEN]; 

    memset(ret_str, '\0', MAX_VALUE_LEN);
    
    if(argv[0] != NULL)
        getCookieByName(rq, argv[0], ret_str);

    ejsSetReturnString(ep->eid, ret_str);    
}

/* 
   argument 1: cookie name
   argument 2: value
   argument 3: 0, [+-]lifetime, time
 */
void setCookies(MaRequest *rq, int argc, char **argv)
{
    int n = 0, t = 0;
    char *name, *value;
    char *path = NULL;

    if (argc < 2 || !argv || !argv[0] || !argv[1])
        return;
    name = argv[0];
    value = argv[1];
    if(argc > 2 && argv[2]) {
        n = argv[2][0];
        if (n == '0') {
            n = 0;
        } else {
            if (n == '+' || n == '-')
                t = time(0);
            t += atoi(argv[2]);
            n = 1;
        }
        if (argc > 3 && argv[3] && *argv[3])
            path = argv[3];
    }

    if (n == 0) {
        maSetCookie(rq, name, value, t, path, 0);
    } else {
        struct tm tm;
        char dateStr[64];
        char cookie[4096];

        mprGmtime(&t, &tm);
        mprRfcTime(dateStr, sizeof(dateStr), &tm);
        sprintf(cookie, "Set-Cookie: %s=%s; path=%s; Expires=%s; ", 
            name, value, path ? path : "/", dateStr);
        maSetHeader(rq, cookie, 1);
        maSetHeader(rq, "Cache-control: no-cache=\"set-cookie\"", 0);
    }
}

void permissionControlByPage(EspRequest *ep, int argc, char **argv)
{
    int ret;
 
    /* permission control by page(cookie) or by group id */
#ifdef PERMISSION_CONTROL_BY_PAGE 
    ret = 1;
#else
    ret = 0;
#endif
    ejsSetReturnValue(ep->eid, mprCreateIntegerVar(ret));
}

#ifdef PERMISSION_CONTROL_BY_PAGE 
int permissionChkByCookie(MaRequest *rq)
{
    char username[MAX_VALUE_LEN], password[MAX_VALUE_LEN];

    getCookieByName(rq, "username", username);
    getCookieByName(rq, "password", password);

    return check_permission(username, password);
}
#endif

void initPermission(void)
{
    maDefineEsp(T("setCookies"), setCookies);
    espDefineStringCFunction(0, "getCookies", getCookies, 0);
    espDefineStringCFunction(0, "chkPermission", chkPermission, 0);
    espDefineStringCFunction(0, "permissionControlByPage", permissionControlByPage, 0);
}
