/**
 *  \file web_status.c
 *
 *  This is the cgi modules for A300 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   12/29/2006     0.1       Jimmy Yang     Initial version
 *
 */
#include "cipcfg.h"
#include "syscall.h"
#include "web_status.h"

#define MTDOOPS_DEV "log"

/**
 * Display oops log message on Web
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int genOopsLog(int eid, webs_t wp, int argc, char_t **argv)
{
    char buf[80];
    FILE *fp = NULL;
    char mtd_name[15], cmd[80];
    
    if (find_mtdblock(MTDOOPS_DEV, mtd_name) < 0){
	printf("There is no mtd block named %s\n", MTDOOPS_DEV);
	return 1;
    }

    sprintf(cmd, PATH_OOPSLOG" %s > "PATH_OOPSLOG_FILE, mtd_name);
    printf("cmd: %s\n", cmd);
    if (cip_system(cmd)) {
	printf("Can't read oops log from flash\n");
	return 1;
    }
    
    if ((fp=fopen(PATH_OOPSLOG_FILE, "r")) == NULL) {
        printf("Can't open  file\n");
        return 1;
    }
    
    while (fgets(buf, sizeof(buf), fp) != NULL) {
        websWrite(wp, T(buf));
	printf("%s", buf);
    }
    pclose(fp);
    return 0;
}

/**
 * Display CAPWAP log on Web
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int genCapwapLog(int eid, webs_t wp, int argc, char_t **argv)
{
    char buf[80];
    FILE *fp = NULL;

    if ((fp=fopen(PATH_CAPWAP_LOG, "r")) == NULL) {
        printf("Can't open syslog file\n");
        return 1;
    }
    
    while (fgets(buf, sizeof(buf), fp) != NULL) {
        websWrite(wp, T(buf));
    }
    fclose(fp);
    return 0;
}

/**
 * Display Syslog message on Web
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int genLog(int eid, webs_t wp, int argc, char_t **argv)
{
    char buf[80];
    FILE *fp = NULL;

    if ((fp=popen(PATH_LOGREAD, "r")) == NULL) {
        printf("Can't open syslog file\n");
        return 1;
    }
    
    while (fgets(buf, sizeof(buf), fp) != NULL) {
        websWrite(wp, T(buf));
    }
    pclose(fp);
    return 0;
}

/**
 * Save the Syslog message to remote 
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void backupLog(webs_t wp, char_t *path, char_t *query)
{
    FILE            *fd = NULL;
    char            buf[1024];
    int             ret = 0;
    struct stat     fst;
    
    apply_sys(CMD_GEN_SYSLOG);
    ret = stat("/web/html/status/syslog_messages", &fst);
    if((ret >= 0) && (fst.st_size != 0)){
        fd = fopen("/web/html/status/syslog_messages", "r");
        if(fd != NULL){
            websWrite(wp, "HTTP/1.0 200 OK\r\n");
            websWrite(wp, "Content-type: text/plain\r\n");
            websWrite(wp, "Content-Disposition: attachment; filename=\"EventLog.txt\"\r\n");
            websWrite(wp, "Content-length: %d\r\n", fst.st_size);
            websWrite(wp, "\r\n\r\n");
            while(!feof(fd)){
                ret = fread(buf, 1, 1024, fd);
                websWriteBlock(wp, buf, ret);
            }
        }
    } else
        websRedirect(wp, "/status/log.asp");
    if(fd)
        fclose(fd);
}

/**
 * Clear syslog message file
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void clearLog(webs_t wp, char_t *path, char_t *query)
{
    apply_sys(CMD_CLEAR_SYSLOG);
    websRedirect(wp, "/status/log.asp");
}

/**
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initStatusForm(void)
{
    websFormDefine(T("backupLog"), backupLog);
    websFormDefine(T("clearLog"), clearLog);
    websAspDefine(T("genLog"), genLog);
    websAspDefine(T("genOopsLog"), genOopsLog);
    websAspDefine(T("genCapwapLog"), genCapwapLog);
}
