/**
 *  \file web_sys.c
 *
 *  This is the cgi modules for A800 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/20/2006     0.1       Jimmy Yang     Initial version
 *
 */
#include "id.h"
#include "cipcfg.h"
#include "syscall.h"
#include "web_sys.h"
#include "string.h"
#include "webfunc.h"
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

extern unsigned int magic;
extern char *config_rom, *config_ram;
extern char *backup_cfg_file;
extern const int gid_vap[][MAX_VAP];
extern const int gid_wds_link[][MAX_WDS_LINK];

#ifdef GW_FUNC
extern int gid_vlan[];
extern int gid_vlan_lan[];
extern int gid_vlan_dhcp[];
extern int gid_vlan_dhcprelay[];
extern int gid_vlan_policy[];
extern int gid_uplogin[];
extern int gid_uploginsuccess[];
extern int gid_uploginfail[];
extern int gid_uploginsuccessod[];
extern int gid_uplogoutsuccess[];
#endif

/**
 * Set/save the system device information to device
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
//TODO : add the system time stting 
void applySystem(webs_t wp, char_t *path, char_t *query)
{
    char_t *name, *desc, *location, *type, *ntp1, *ntp2, *zone, *m_set;
#ifdef INTERNAL_DN
    char_t *internal_dn;
#endif
    char_t *year, *month, *day, *hour, *min, *sec;
    char_t *app;
    char buf[19], tmp[256];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret;
    int applylist[] = {GID_DEVICE, GID_SYSTIME, GID_END};

    app = websGetVar(wp, T("apply"), T("true"));
    if(getUserGroupPermission(wp, GID_DEVICE) & PERM_WRITE){
        name = websGetVar(wp, T("name"), T("A800"));
        desc = websGetVar(wp, T("desc"), T("Cipherium Ltd."));
        location = websGetVar(wp, T("location"), T("Taipei, Tiawan"));
    
        list=append_cfg_item(NULL, GID_DEVICE, "name", name);
        list=append_cfg_item(list, GID_DEVICE, "desc", desc);
        list=append_cfg_item(list, GID_DEVICE, "location", location);
#ifdef INTERNAL_DN
	chkFeature("INTERNAL_DN", tmp);
	if(tmp[0] == 'Y'){
	    internal_dn = websGetVar(wp, T("internal_dn"), 
		    T("mygateway.mycompany.com"));
	    list=append_cfg_item(list, GID_DEVICE, "internal_dn", internal_dn);
	}
#endif
    }
    // for System Time
    if(getUserGroupPermission(wp, GID_SYSTIME) & PERM_WRITE){
        zone = websGetVar(wp, T("timezone"), T("Asia/Taipei"));
        type = websGetVar(wp, T("synch"), T("manual"));
        m_set= websGetVar(wp, T("manual_set"), T("false"));
    
        list=append_cfg_item(list, GID_SYSTIME, "timezone", zone);
        list=append_cfg_item(list, GID_SYSTIME, "type", type);
    
        if ( !strncmp(type, "ntp", 3 ) ){
            ntp1 = websGetVar(wp, T("ntp_server1"), T("tock.usno.navy.mil"));
            ntp2 = websGetVar(wp, T("ntp_server2"), T(""));
            list=append_cfg_item(list, GID_SYSTIME, "ntp_server1", ntp1);
            list=append_cfg_item(list, GID_SYSTIME, "ntp_server2", ntp2);
        }else{
            if ( !strncmp(m_set, "true", 4 ) ){
                year = websGetVar(wp, T("year"), T("2007"));
                month = websGetVar(wp, T("month"), T("01"));
                day = websGetVar(wp, T("day"), T("01"));
                hour = websGetVar(wp, T("hour"), T("00"));
                min = websGetVar(wp, T("min"), T("00"));
                sec = websGetVar(wp, T("sec"), T("00"));
                memset(buf, 0, 19);
                strncat(buf, year, 4);
                strncat(buf, "." , 1);
                strncat(buf, month,2);
                strncat(buf, "." , 1);
                strncat(buf,  day, 2);
                strncat(buf, "-" , 1);
                strncat(buf, hour, 2);
                strncat(buf, ":" , 1);
                strncat(buf,  min, 2);
                strncat(buf, ":" , 1);
                strncat(buf,  sec, 2);
                set_systime( buf, zone );
            }
        }
    }
    if(list != NULL){
        print_list(list);
    
        cipcfg=init_cipcfg();
    
        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return;
        }
   	/* 
        if ( !strncmp(type, "ntp", 3 ) ){
            sprintf(tmp, "%s start", PATH_RC_SYSTIME);
            cip_system(tmp);
        }
	*/
        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                return;
            }
        }
    
        close_cipcfg(cipcfg);
    }
    websRedirect( wp, "/system/sysinfo.asp" );
}

/**
 * Reboot the system
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyReboot(webs_t wp, char_t *path, char_t *query)
{
//    apply_sys(CMD_REBOOT);
    websRedirect(wp, "/utilities/pre_reboot.asp");
}

/**
 * Restore the configuration to default and reboot system.
 * cp /etc/default-config.xml to /etc/config/a300-config.xml
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
// TODO : Does this function need reboot system?
void resetCfg(webs_t wp, char_t *path, char_t *query)
{
    int ret;
    char cmd[BUFSIZ];
    CIPCFG *cipcfg = NULL;
    extern const char *restored_cfg_file;

    // calling reset_cipcfg will cause some problem like bug 5039.
    //ret=reset_cipcfg();

    /* Using customized config file if exist */
    if (access(CUSTOMIZE_CONFIG, F_OK) == 0) {
	cipcfg = init_cipcfg_f(CUSTOMIZE_CONFIG);
    } else {
	cipcfg = init_cipcfg_f(DEFAULT_CONFIG);
    }

    if (!cipcfg) {
	printf("initial cipcfg error.\n");
        websRedirect(wp, "/utilities/backup.asp");
	return;
    }
    ret = save_cipcfg(cipcfg, restored_cfg_file);
    close_cipcfg(cipcfg);
    if (ret != CIPCFG_SUCCESS){
        printf("ERROR reset to default. error code : %d\n", ret);
        websRedirect(wp, "/utilities/backup.asp");
        return ;
    }

    sprintf(cmd, "rm -fr /conf/certificate/; rm -fr /conf/tmpl_img; rm -f /conf/user_db");
    ret = cip_system(cmd);
    if (WEXITSTATUS(ret) == 1)
        printf("ERROR reset to default certificate, template images and user database.");

    printf("SUCCESS reset to default. Start Rebooting.......\n");
//    apply_sys(CMD_REBOOT);
    websRedirect(wp, "/utilities/pre_reboot.asp");
}

/**
 * Restore the configuration to default
 * cp /etc/default-config.xml to /etc/config/a300-config.xml
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */

static void backupCfg(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    int ret, len;
    char buf[4096];
    FILE *fp; 
    int fd;

    /* Permission Check */
#ifdef PERMISSION_CONTROL_BY_PAGE
    ret = permissionChkByCookie(rq);
    if(ret != 0)
        maRedirect(rq, 302, "/utilities/backup.asp");
#endif 
#ifdef PERMISSION_CONTROL_BY_GID
    if(!(getOperationPermission(rq, OID_BACKUP_CONFIG) & PERM_EXEC))
        maRedirect(rq, 302, "/utilities/backup.asp");
#endif    
    ret = backup_cipcfg(BACKUP_CIPCFG_FILE_NAME);

    if (ret != SYS_BACKUP_SUCCESS) {
        printf("config backup fail.\n");
        maRedirect(rq, 302, "/utilities/backup.asp");
    }else{
        fp = fopen(BACKUP_CIPCFG_FILE_NAME, "r"); 
        if(fp == NULL) {
            maRedirect(rq, 302, "/utilities/backup.asp");
        }
        fd = fileno(fp);

        if (flock(fd, LOCK_SH) != 0) {
            maRedirect(rq, 302, "/utilities/backup.asp");
        } else {
            maSetHeader(rq, "Cache-Control: no-cache", 0);
            maSetHeader(rq, "Content-type: application/octet-stream", 0);
            maSetHeader(rq, "Content-Disposition: attachment;filename=" BACKUP_CIPCFG_FILE_NAME, 0);
            while ((len=fread(buf, sizeof(char), 4096, fp)) > 0)
                maWrite(rq, buf, len);
            flock(fd, LOCK_UN);
        }
        fclose(fp);
    }
}

/** 
 * @brief Reset Customized items
 * 
 * @param wp 
 * @param path 
 * @param query 
 */
void resetCustomize(webs_t wp, char_t *path, char_t *query)
{
    char cmd[BUFSIZ];

    sprintf(cmd, "%s reset", PATH_CUSTOMIZE_SH);
    cip_system(cmd);
    websRedirect(wp, "/utilities/pre_reboot.asp");
}

#ifdef GW_FUNC
void applyVlan(webs_t wp, char_t *path, char_t *query)
{
    char_t *ip, *mask, *start_ip, *end_ip, *dns1, *dns2, *vtag, *state, *qosac;
    char_t *domain_name, *wins_server, *lease_time, *dhcp_state, *defauth;
    char_t *auth1, *auth2, *auth3, *auth4, *auth5, *auth6;
    char_t *current_vlan, *remark; 
    char_t *app; 
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    char tmp[256];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret, i = 0;
    int max_vlan = 8, err_code = 0, nif_code = 0;

    app = websGetVar(wp, T("apply"), T("false"));

    current_vlan = websGetVar(wp, T("vlan"), T(""));
    i = atoi(current_vlan);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	free(applylist);
	// sprintf(tmp, "/system/vlan_config.asp?vlan=%d&err_code=501", i);
	// websRedirect(wp, T(tmp));
	return;
    }
    get_cfg_item(cipcfg, GID_OPMODE, "type", tmp);
    if (strcmp(tmp, "GW") != 0)
	goto do_nothing;
    if (get_rc_env("MAX_VLAN", tmp, sizeof(tmp)) != NULL)
	max_vlan = atoi(tmp);
    
    if (i < 0 || i > max_vlan) {
	i = 0;
	goto do_nothing;
    }

    vtag = websGetVar(wp, T("vid"), T("0"));
    qosac = websGetVar(wp, T("qosac"), T("0x0"));
    state = websGetVar(wp, T("state"), T(""));
    remark = websGetVar(wp, T("remark"), T(""));
    ip = websGetVar(wp, T("ip"), T(""));
    mask = websGetVar(wp, T("mask"), T(""));
    dhcp_state = websGetVar(wp, T("dhcp_state"), T(""));
    start_ip = websGetVar(wp, T("start_ip"), T(""));
    end_ip = websGetVar(wp, T("end_ip"), T(""));
    dns1 = websGetVar(wp, T("dns1"), T(""));
    dns2 = websGetVar(wp, T("dns2"), T(""));

    // Authentication Type
    auth1 = websGetVar(wp, T("auth1_policy"), T("disable"));
    auth2 = websGetVar(wp, T("auth2_policy"), T("disable"));
    auth3 = websGetVar(wp, T("auth3_policy"), T("disable"));
    auth4 = websGetVar(wp, T("auth4_policy"), T("disable"));
    auth5 = websGetVar(wp, T("auth5_policy"), T("disable"));
    auth6 = websGetVar(wp, T("auth6_policy"), T("disable"));

    domain_name = websGetVar(wp, T("domain_name"), T(""));
    wins_server = websGetVar(wp, T("wins_server"), T(""));
    lease_time = websGetVar(wp, T("lease_time"), T(""));
    defauth = websGetVar(wp, T("defauth"), T("0"));

    if (!*state || !*ip || !*mask || !*dhcp_state)
	goto do_nothing;
    if (strcmp(state, "enable") != 0) {
	state = "disable";
    } else {
	int j;
	char ipaddr[MAX_VALUE_LEN], ipmask[MAX_VALUE_LEN];
	unsigned long cur_bcast, cur_naddr;
	unsigned long bcast, naddr;

	if (get_ipmask(ip, mask, &cur_bcast, &cur_naddr) <= 0) {
	    err_code = 101;
	    nif_code = i;
	}
	for (j = 0; j <= max_vlan; j++) {
	    if (i == j)
		continue;
	    get_cfg_item(cipcfg, gid_vlan[j], "status", tmp);
	    if (strcmp(tmp, "enable") != 0)
		continue;
	    get_cfg_item(cipcfg, gid_vlan[j], "vtag", tmp);
	    if (atoi(vtag) == atoi(tmp)) {
		err_code = 104;
		nif_code = j;
		goto fail;
	    }
	    if (err_code != 0)
		continue;
	    get_cfg_item(cipcfg, gid_vlan_lan[j], "ip", ipaddr);
	    get_cfg_item(cipcfg, gid_vlan_lan[j], "netmask", ipmask);
	    if (get_ipmask(ipaddr, ipmask, &bcast, &naddr) <= 0)
		continue;
	    if (naddr <= cur_bcast && bcast >= cur_naddr) {
		err_code = 102;
		nif_code = j;
	    }
	}
	if (err_code == 101)
	    goto fail;
	get_cfg_item(cipcfg, GID_WAN_IP, "type", tmp);
	if (!strcmp(tmp, "fixed")) {
	    get_cfg_item(cipcfg, GID_WAN_IP, "addr", ipaddr);
	    get_cfg_item(cipcfg, GID_WAN_IP, "netmask", ipmask);
	    if (get_ipmask(ipaddr, ipmask, &bcast, &naddr) > 0) {
		if (naddr <= cur_bcast && bcast >= cur_naddr) {
		    err_code = 103;
		    nif_code = 1;
		}
	    }
	}
	if (err_code != 0)
	    goto fail;
    }

    list=append_cfg_item(list, gid_vlan[i], "status", state);
    list=append_cfg_item(list, gid_vlan[i], "remark", remark);

    list=append_cfg_item(list, gid_vlan_lan[i], "ip", ip);
    list=append_cfg_item(list, gid_vlan_lan[i], "netmask", mask);
    list=append_cfg_item(list, gid_vlan_lan[i], "dhcp_mode", dhcp_state);
    list=append_cfg_item(list, gid_vlan[i], "vtag", vtag);
    list=append_cfg_item(list, gid_vlan[i], "qosac", qosac);

    if (!strcmp(dhcp_state, "DHCP")) {
	list=append_cfg_item(list, gid_vlan_dhcp[i], "status", dhcp_state);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "start_ip", start_ip);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "end_ip", end_ip);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "dns1", dns1);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "dns2", dns2);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "domain", domain_name);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "wins_ip", wins_server);
	list=append_cfg_item(list, gid_vlan_dhcp[i], "lease_time", lease_time);
    }
    else if (!strcmp(dhcp_state, "RELAY")) {
	char *dhcprelay_ip;
	dhcprelay_ip = websGetVar(wp, T("dhcprelay_ip"), T(""));
	list=append_cfg_item(list, gid_vlan_dhcprelay[i], "ip", dhcprelay_ip);
    }
    list=append_cfg_item(list, gid_vlan_policy[i], "defauth", defauth);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth1", auth1);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth2", auth2);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth3", auth3);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth4", auth4);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth5", auth5);
    list=append_cfg_item(list, gid_vlan_policy[i], "auth6", auth6);
    
    if(list != NULL){
        print_list(list);

        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
	    // free_item_list(list);
	    close_cipcfg(cipcfg);
	    free(applylist);
	    // sprintf(tmp, "/system/vlan_config.asp?vlan=%d&err_code=502", i);
	    // websRedirect(wp, T(tmp));
            return;
        }
    
        while(list != NULL)
        {
            if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
            {
               applylist[applist_idx] = list->gid;
               applist_idx++;
            }
            list = list->next;
        }
        applylist[applist_idx] = GID_END;

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
            }else{
		// free_item_list(list);
                close_cipcfg(cipcfg);
                free(applylist);
                websRedirect(wp, "/utilities/reboot.asp");
                return;
            }
        }
    
    }

do_nothing:
    // free_item_list(list);
    close_cipcfg(cipcfg);
    free(applylist);
    sprintf(tmp, "/system/vlan_config.asp?vlan=%d", i);
    websRedirect(wp, T(tmp));
    return;

fail:
    // free_item_list(list);
    close_cipcfg(cipcfg);
    free(applylist);
    sprintf(tmp, "/system/vlan_config.asp?vlan=%d&err_code=%d&nif_code=%d", i, err_code, nif_code);
    websRedirect(wp, T(tmp));
    return;
}
#endif

#ifdef GW_FUNC
void applyWalledGarden(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *dest;
    char item_value[MAX_VALUE_LEN], tmp[256];
    int ret, i, gid = GID_NET_WALL, max = 0;
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int applylist[] = {GID_NET_WALL, GID_END};

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// websRedirect(wp, T("/error.asp?page=walled_garden&err_code=501"));
	return;
    }

    ret = get_cfg_item(cipcfg, gid, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get num\n");
	close_cipcfg(cipcfg);
	// websRedirect(wp, T("/error.asp?page=walled_garden&err_code=503"));
	return;
    }
    max = atoi(item_value);

    for (i=0; i<max; i++) {
	sprintf(tmp, "dest%d", i);
	dest = websGetVar(wp, T(tmp), T(""));
	sprintf(tmp, "wall:%d.dest", i);
	list = append_cfg_item(list, gid, tmp, dest);
    }
    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't set items");
	// free_item_list(list);
	close_cipcfg(cipcfg);
	// websRedirect(wp, T("/error.asp?page=walled_garden&err_code=502"));
	return;
    }
    // free_item_list(list);

    app = websGetVar(wp, T("apply"), T("true"));
    if (!strncmp("true", app, 4)) {
/*
	// TODO: move into aply_cfg_group() ?
	sprintf(tmp, ". /etc/rc.d/rc.env; $SCRIPTS_PATH/walled_garden.sh");
	cip_system(tmp);
*/
/*
	if (can_apply(applylist) == CIPCFG_SUCCESS) {
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	} else {
	    close_cipcfg(cipcfg);
	    websRedirect(wp, "/utilities/reboot.asp");
	    return;
	}
*/
    }

    close_cipcfg(cipcfg);
    websRedirect(wp, T("/system/walled_garden.asp"));
}
#endif

#if defined(GW_FUNC) || defined(CAPWAP)
static int Restore_Cert(EspRequest *ep, int argc, char **argv)
{
    char cmd[BUFSIZ];
    int ret;
    FILE *fp = NULL;

    sprintf(cmd, "/bin/rm -fr /conf/certificate/");
    ret = cip_system(cmd);
    if (WEXITSTATUS(ret) == 1)
	ret = 107;
    else {
	fp = fopen(PATH_SYNC_CFG, "a");
	if (fp != NULL) {
	    fwrite("200\n", 1, strlen("200\n"), fp);
	    fclose(fp);
	}
	ret = 0;
    }
    ejsSetReturnValue(ep->eid, mprCreateIntegerVar(ret));
    return 0;
}
#endif

#ifdef GW_FUNC
static int delTmplImg(EspRequest *ep, int argc, char **argv)
{
    char cmd[BUFSIZ];
    int ret, vlanid;

    if (ejsParseArgs(argc, argv, "%d", &vlanid) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    sprintf(cmd, "rm -fr /conf/tmpl_img/%d/tpl_logo.gif", vlanid);
    ret = cip_system(cmd);
    if (WEXITSTATUS(ret) == 1)
	ret = 4;
    else
	ret = 6;
    ejsSetReturnValue(ep->eid, mprCreateIntegerVar(ret));
    return 0;
}
#endif

#ifdef GW_FUNC
static void applyReservIP(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *entry, *app, *ip, *mac, *vlan_ptr, *remark;
    char *act;
    char item_value[MAX_VALUE_LEN], item_name[32], tmp[256];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int i, j, ret;
    int applylist[] = {gid_vlan_dhcp[0], GID_END};
    int vlan; 

    act = maGetVar(rq, MA_FORM_OBJ, "act", "");
    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    cipcfg = init_cipcfg();

    vlan_ptr = maGetVar(rq, MA_FORM_OBJ, "vlan", "0");
    vlan = atoi(vlan_ptr);

    sprintf(tmp, "vlan%d_dhcp", vlan);
    sprintf(item_name, "reserved_ip:%d.ip", 0);
    if (get_cfg_item_by_str(cipcfg, tmp, item_name, item_value) != CIPCFG_SUCCESS)
        vlan = 0;
    applylist[0] = gid_vlan_dhcp[vlan];

    if ( !strncmp("edit", act, 4) || !strncmp("add", act, 3) ){
        if (!strncmp("edit", act, 4)) {
            entry = maGetVar(rq, MA_FORM_OBJ, "entry", "");
            i = atoi(entry);
        }else if (!strncmp("add", act, 3)){
            // find an empty entry to add.
            item_value[0] = '1';
            for(i=0; *item_value!='\0'; i++){
                sprintf(item_name, "reserved_ip:%d.ip", i);
                ret = get_cfg_item(cipcfg, gid_vlan_dhcp[vlan], item_name, item_value);
            }
            i-- ;
        }

        ip = maGetVar(rq, MA_FORM_OBJ, "ip", "");
        mac = maGetVar(rq, MA_FORM_OBJ, "mac", "");
        remark = maGetVar(rq, MA_FORM_OBJ, "remark", "");

        sprintf(item_name, "reserved_ip:%d.ip", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, ip);
        sprintf(item_name, "reserved_ip:%d.mac", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, mac);
        sprintf(item_name, "reserved_ip:%d.remark", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, remark);

    }else if (!strncmp("delete", act, 6)){
        entry = maGetVar(rq, MA_FORM_OBJ, "entry", "");
        i = atoi(entry);
        sprintf(item_name, "reserved_ip:%d.ip", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, "");
        sprintf(item_name, "reserved_ip:%d.mac", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, "");
        sprintf(item_name, "reserved_ip:%d.remark", i);
        list=append_cfg_item(list, gid_vlan_dhcp[vlan], item_name, "");

    }else{
        sprintf(tmp, "/system/reserved_ip.asp?vlan=%d", vlan);
        maRedirect(rq, 302, 
                maGetVar(rq, MA_GLOBAL_OBJ, "HTTP_REFERER", tmp));
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }

    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/reserved.asp?vlan=%d", vlan);
    maRedirect(rq, 302, 
              maGetVar(rq, MA_GLOBAL_OBJ, "HTTP_REFERER", tmp));
}
#endif

#ifdef GW_FUNC
static void applyWallAdList(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int applylist[] = {GID_NET_WALL_AD, GID_END};
    int i, max, ret;
    char item_value[MAX_VALUE_LEN], item_name[32];
    char *app, *url, *desc, *topic, *display;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=walled_garden_adlist&err_code=501");
	return;
    }
    ret = get_cfg_item(cipcfg, GID_NET_WALL_AD, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	printf("Can't get maximum walled garden ad list entries.\n");
	// maRedirect(rq, 302, "/error.asp?page=walled_garden_adlist&err_code=501");
	close_cipcfg(cipcfg);
	return;
    }
    max = atoi(item_value);

    for(i = 0; i < max; i++) {
	sprintf(item_name, "url%d", i);
	url = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "topic%d", i);
	topic = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "desc%d", i);
	desc = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "active%d", i);
	display = maGetVar(rq, MA_FORM_OBJ, item_name, "disable");
	sprintf(item_name, "entry:%d.url", i);
	list=append_cfg_item(list, GID_NET_WALL_AD, item_name, url);
	sprintf(item_name, "entry:%d.topic", i);
	list=append_cfg_item(list, GID_NET_WALL_AD, item_name, topic);
	sprintf(item_name, "entry:%d.desc", i);
	list=append_cfg_item(list, GID_NET_WALL_AD, item_name, desc);
	sprintf(item_name, "entry:%d.display", i);
	list=append_cfg_item(list, GID_NET_WALL_AD, item_name, display);
    }
    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }
    
    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }

    close_cipcfg(cipcfg);
    maRedirect(rq, 302, "/system/walled_garden_adlist.asp");
}
#endif

#ifdef GW_FUNC
static void applyUpLogin(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *logintype, *item_value, tmp[MAX_VALUE_LEN];
    int vlan, ret;
    int applylist[] = {gid_uplogin[0], GID_END};

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    vlanid = maGetVar(rq, MA_FORM_OBJ, "vlan_id", "0");
    vlan = atoi(vlanid);
    if (vlan > 8 || vlan < 0)
	vlan = 0;
    applylist[0] = gid_uplogin[vlan];
    logintype = maGetVar(rq, MA_FORM_OBJ, "login_type", "default");
    list = append_cfg_item(list, gid_uplogin[vlan], "type", logintype);
    if (!strcmp(logintype, "template")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_bg_color", "80C0FF");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.title_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_txt_color", "000080");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.title_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_txt_color", "000080");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.page_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_bg_color", "FFFFFF");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.page_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title", "User Login Page");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.title", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "welcome", "Welcome to User Login Page");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.welcome", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information", "Please Enter Your Name and Password to Sign in");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.information", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "username", "Username");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.username", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "password", "Password");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.password", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "submit", "Submit");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.submit", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "clear", "Clear");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.clear", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "copyright", "Copyright(c)");
	list = append_cfg_item(list, gid_uplogin[vlan], "template.copyright", item_value);
    }
    else if (!strcmp(logintype, "external")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "externalurl", "");
	list = append_cfg_item(list, gid_uplogin[vlan], "external.url", item_value);
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=uplogin&err_code=501");
	return;
    }
    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/uplogin.asp?vlan=%d", vlan);
    maRedirect(rq, 302, tmp);
}
#endif

#ifdef GW_FUNC
static void applyUpLoginSuccess(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *logintype, *item_value, tmp[MAX_VALUE_LEN];
    int vlan, ret;
    int applylist[] = {gid_uploginsuccess[0], GID_END};

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    vlanid = maGetVar(rq, MA_FORM_OBJ, "vlan_id", "0");
    vlan = atoi(vlanid);
    if (vlan > 8 || vlan < 0)
	vlan = 0;
    applylist[0] = gid_uploginsuccess[vlan];
    logintype = maGetVar(rq, MA_FORM_OBJ, "login_type", "default");
    list = append_cfg_item(list, gid_uploginsuccess[vlan], "type", logintype);
    if (!strcmp(logintype, "template")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_bg_color", "");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.title_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_txt_color", "");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.title_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_txt_color", "");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.page_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_bg_color", "");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.page_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title", "Login Success Page");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.title", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "welcome", "Hello");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.welcome", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information", "Please click this button to");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.information", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "logout", "Logout");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.logout", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information2", "Thank you");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.information2", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "logintime", "Login Time");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "template.logintime", item_value);
    }
    else if (!strcmp(logintype, "external")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "externalurl", "");
	list = append_cfg_item(list, gid_uploginsuccess[vlan], "external.url", item_value);
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=uplogin&err_code=501");
	return;
    }
    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/upsucceedlogin.asp?vlan=%d", vlan);
    maRedirect(rq, 302, tmp);
}
#endif

#ifdef GW_FUNC
static void applyUpLogoutSuccess(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *logintype, *item_value, tmp[MAX_VALUE_LEN];
    int vlan, ret;
    int applylist[] = {gid_uplogoutsuccess[0], GID_END};

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    vlanid = maGetVar(rq, MA_FORM_OBJ, "vlan_id", "0");
    vlan = atoi(vlanid);
    if (vlan > 8 || vlan < 0)
	vlan = 0;
    applylist[0] = gid_uplogoutsuccess[vlan];
    logintype = maGetVar(rq, MA_FORM_OBJ, "login_type", "default");
    list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "type", logintype);
    if (!strcmp(logintype, "template")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_bg_color", "");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.title_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_txt_color", "");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.title_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_txt_color", "");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.page_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_bg_color", "");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.page_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title", "Logout Success Page");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.title", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information", "Logout successfully");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "template.information", item_value);
    }
    else if (!strcmp(logintype, "external")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "externalurl", "");
	list = append_cfg_item(list, gid_uplogoutsuccess[vlan], "external.url", item_value);
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=uplogin&err_code=501");
	return;
    }
    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/upsucceedlogout.asp?vlan=%d", vlan);
    maRedirect(rq, 302, tmp);
}
#endif

#ifdef GW_FUNC
static void applyUpLoginSuccessOD(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *logintype, *item_value, tmp[MAX_VALUE_LEN];
    int vlan, ret;
    int applylist[] = {gid_uploginsuccessod[0], GID_END};

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    vlanid = maGetVar(rq, MA_FORM_OBJ, "vlan_id", "0");
    vlan = atoi(vlanid);
    if (vlan > 8 || vlan < 0)
	vlan = 0;
    applylist[0] = gid_uploginsuccessod[vlan];
    logintype = maGetVar(rq, MA_FORM_OBJ, "login_type", "default");
    list = append_cfg_item(list, gid_uploginsuccessod[vlan], "type", logintype);
    if (!strcmp(logintype, "template")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_bg_color", "");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.title_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_txt_color", "");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.title_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_txt_color", "");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.page_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_bg_color", "");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.page_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title", "Login Success Page");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.title", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "welcome", "Hello");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.welcome", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information", "Please click this button to");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.information", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "logout", "Logout");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.logout", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information2", "Thank you");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.information2", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "logintime", "Login Time");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.logintime", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "remaining", "Remaining Usage");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.remaining", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "day", "Day");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.day", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "hour", "Hour");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.hour", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "min", "Min");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.min", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "sec", "Sec");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "template.sec", item_value);
    }
    else if (!strcmp(logintype, "external")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "externalurl", "");
	list = append_cfg_item(list, gid_uploginsuccessod[vlan], "external.url", item_value);
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=uplogin&err_code=501");
	return;
    }
    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/upsucceedloginondemand.asp?vlan=%d", vlan);
    maRedirect(rq, 302, tmp);
}
#endif

#ifdef GW_FUNC
static void applyUpLoginFail(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char *app, *vlanid, *logintype, *item_value, tmp[MAX_VALUE_LEN];
    int vlan, ret;
    int applylist[] = {gid_uploginfail[0], GID_END};

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    vlanid = maGetVar(rq, MA_FORM_OBJ, "vlan_id", "0");
    vlan = atoi(vlanid);
    if (vlan > 8 || vlan < 0)
	vlan = 0;
    applylist[0] = gid_uploginfail[vlan];
    logintype = maGetVar(rq, MA_FORM_OBJ, "login_type", "default");
    list = append_cfg_item(list, gid_uploginfail[vlan], "type", logintype);
    if (!strcmp(logintype, "template")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_bg_color", "");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.title_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title_txt_color", "");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.title_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_txt_color", "");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.page_txt_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "page_bg_color", "");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.page_bg_color", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "title", "Login Failed Page");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.title", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "information", "Error Message");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.information", item_value);
	item_value = maGetVar(rq, MA_FORM_OBJ, "back", "Try Again");
	list = append_cfg_item(list, gid_uploginfail[vlan], "template.back", item_value);
    }
    else if (!strcmp(logintype, "external")) {
	item_value = maGetVar(rq, MA_FORM_OBJ, "externalurl", "");
	list = append_cfg_item(list, gid_uploginfail[vlan], "external.url", item_value);
    }
    print_list(list);

    cipcfg = init_cipcfg();
    if (!cipcfg) {
	printf("initial cipcfg error.\n");
	// maRedirect(rq, 302, "/error.asp?page=uplogin&err_code=501");
	return;
    }
    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
        return ;
    }

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            maRedirect(rq, 302, "/utilities/reboot.asp" );
            return;
        }
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/system/upfaillogin.asp?vlan=%d", vlan);
    maRedirect(rq, 302, tmp);
}
#endif

#ifdef GRE
void applyGre(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *state, *gre_remote, *gre_key, *if_gre_state;
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char tmp[256], interface[BUFSIZ], item_value[MAX_VALUE_LEN];
    int i, rfidx, gid = GID_GRE;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, ret, lan_num = 0;

    app = websGetVar(wp, T("apply"), T("true"));

    cipcfg = init_cipcfg();

    chkFeature("GRE", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, gid) & PERM_WRITE)){
	state	    = websGetVar(wp, T("gre_state"), T("disable"));
	gre_remote  = websGetVar(wp, T("gre_remote"), T(""));
	gre_key	    = websGetVar(wp, T("gre_key"), T(""));
	list  = append_cfg_item(list, gid, "state", state);
	if(!strncmp(state, "enable", 6)) {
	    list  = append_cfg_item(list, gid, "remote", gre_remote);
	    list  = append_cfg_item(list, gid, "key", gre_key);
	    
	    if(get_cfg_item_by_str(cipcfg, "gre", "lan_num", item_value) == CIPCFG_SUCCESS)
		lan_num = atoi(item_value);

	    for(i = 0; i< lan_num; i++){
		sprintf(interface, "lan%d", i);
		if_gre_state = websGetVar(wp, T(interface), T(""));
		sprintf(tmp, "lan:%d.gre_state", i);
		list = append_cfg_item(list, gid, tmp, 
			(strcmp("on", if_gre_state) ? "disable" : "enable"));
	    }

	    rfidx = 0; /* TODO: Now we only support single RF */
	    for (i = 0; i < MAX_VAP; i++) {
		sprintf(interface, "vap%d", i);
		get_cfg_item(cipcfg, gid_vap[rfidx][i], "state", item_value);
		if (strcmp(item_value, "enable"))
		    continue;
		if_gre_state = websGetVar(wp, T(interface), T(""));
		list = append_cfg_item(list, gid_vap[rfidx][i], "gre", 
			(strcmp("on", if_gre_state) ? "disable" : "enable"));
	    }
	    for (i = 0; i < MAX_WDS_LINK; i++) {
		sprintf(interface, "wds%d", i);
		get_cfg_item(cipcfg, gid_wds_link[rfidx][i], "state", item_value);
		if (strcmp(item_value, "enable"))
		    continue;
		if_gre_state = websGetVar(wp, T(interface), T(""));
		list = append_cfg_item(list, gid_wds_link[rfidx][i], "gre", 
			(strcmp("on", if_gre_state) ? "disable" : "enable"));
	    }
	}
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/system/gre.asp"))); 
}
#endif

#ifdef CAPWAP
void applyCapwap(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *state, *check_cert, *dns_srv, *srv_suffix, *dhcp_opt, *unicast;
    char_t *broadcast, *multicast, *if_tun_state;
    char_t value[MAX_VALUE_LEN], tmp[MAX_VALUE_LEN];
    char_t priority[MAX_VALUE_LEN], *acname, *acaddr;
    char interface[BUFSIZ], item_value[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int gid = GID_CAPWAP, tbl_size, i, j, rfidx;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, ret, lan_num = 0;

    app = websGetVar(wp, T("apply"), T("true"));

    cipcfg = init_cipcfg();

    state = websGetVar(wp, T("state"), T("disable"));

    if (getUserGroupPermission(wp, gid) & PERM_WRITE) {
	if (!strcmp(state, "disable")) {
	    list = append_cfg_item(list, gid, "state", "disable");
	} else {
	    list = append_cfg_item(list, gid, "state", "enable");

	    /* Tunnel configuration for LAN */
	    if(get_cfg_item_by_str(cipcfg, "network_L2", "lan_num", value) == CIPCFG_SUCCESS)
		lan_num = atoi(value);
	    for(i = 0; i< lan_num; i++){
		sprintf(tmp, "lan:%d.controllable", i);
		if (get_cfg_item_by_str(cipcfg, "network_L2", tmp, item_value) == CIPCFG_SUCCESS && 
			strcmp(item_value, "enable") == 0) {
		    sprintf(interface, "lan%d", i);
		    if_tun_state = websGetVar(wp, T(interface), T(""));
		    sprintf(tmp, "lan:%d.tun", i);
		    list = append_cfg_item(list, GID_NET_L2, tmp, 
			    (strcmp("on", if_tun_state) ? "disable" : "enable"));
		}
	    }

	    /* Tunnel configuration for AP */
	    rfidx = 0; /* TODO: Now we only support single RF */
	    for (i = 0; i < MAX_VAP; i++) {
		sprintf(interface, "vap%d", i);
		get_cfg_item(cipcfg, gid_vap[rfidx][i], "state", value);
		if (strcmp(value, "enable"))
		    continue;
		if_tun_state = websGetVar(wp, T(interface), T(""));
		list = append_cfg_item(list, gid_vap[rfidx][i], "tun", 
			(strcmp("on", if_tun_state) ? "disable" : "enable"));
	    }

	    /* Tunnel configuration for WDS */
	    for (i = 0; i < MAX_WDS_LINK; i++) {
		sprintf(interface, "wds%d", i);
		get_cfg_item(cipcfg, gid_wds_link[rfidx][i], "state", value);
		if (strcmp(value, "enable"))
		    continue;
		if_tun_state = websGetVar(wp, T(interface), T(""));
		list = append_cfg_item(list, gid_wds_link[rfidx][i], "tun", 
			(strcmp("on", if_tun_state) ? "disable" : "enable"));
	    }

	    check_cert = websGetVar(wp, T("check_cert"), T(""));
	    list = append_cfg_item(list, gid, "check_cert", 
		    (strcmp(check_cert, "enable") ? "disable" : "enable"));
	    dns_srv = websGetVar(wp, T("dns_srv"), T(""));
	    list = append_cfg_item(list, gid, "discovery.dns_srv", (strcmp(dns_srv, "enable") ? "disable" : "enable"));
	    if (!strcmp(dns_srv, "enable")) {
		    srv_suffix = websGetVar(wp, T("srv_suffix"), "");
		    list = append_cfg_item(list, gid, "discovery.srv_suffix", srv_suffix);
	    }
	    dhcp_opt = websGetVar(wp, T("dhcp_opt"), T(""));
	    list = append_cfg_item(list, gid, "discovery.dhcp_opt", (strcmp(dhcp_opt, "enable") ? "disable" : "enable"));
	    broadcast = websGetVar(wp, T("broadcast"), T(""));
	    list = append_cfg_item(list, gid, "discovery.broadcast", broadcast);
	    multicast = websGetVar(wp, T("multicast"), T(""));
	    list = append_cfg_item(list, gid, "discovery.multicast", multicast);

	    unicast = websGetVar(wp, T("unicast"), T(""));
	    list = append_cfg_item(list, gid, "discovery.unicast", (strcmp(unicast, "enable") ? "disable" : "enable"));
	    get_cfg_item(cipcfg, gid, "aclist.table_size", value);
	    tbl_size = atoi(value);
	    if (!strcmp(unicast, "enable")) {
	    	for (i  = 0, j = 0; i < ((tbl_size == 0) ? 10: tbl_size); i++) {
		    sprintf(value, "acname%d", i);
		    acname = websGetVar(wp, T(value), T(""));
		    sprintf(value, "acaddr%d", i);
		    acaddr = websGetVar(wp, T(value), T(""));
		    sprintf(value, "aclist.ac_entry:%d.priority", i);
		    if (strlen(acname) == 0 && strlen(acaddr) == 0)
		        sprintf(priority, "%d", 999);
		    else
		    	sprintf(priority, "%d", i*10);
		    list = append_cfg_item(list, gid, value, priority);
		    sprintf(value, "aclist.ac_entry:%d.name", i);
		    list = append_cfg_item(list, gid, value, acname);
		    sprintf(value, "aclist.ac_entry:%d.ip", i);
		    list = append_cfg_item(list, gid, value, acaddr);
		    sprintf(value, "aclist.ac_entry:%d.type", i);
		    list = append_cfg_item(list, gid, value, T("static"));
	    	}
	    }
	}
    }
    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/system/capwap.asp"))); 
}
#endif

#ifdef IPV6 
void applyIPv6(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *state, *mode, *addr, *prefix, *gateway, *dns1, *dns2;
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char tmp[256];
    int gid = GID_IPV6;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, ret;

    app = websGetVar(wp, T("apply"), T("true"));

    cipcfg = init_cipcfg();

    chkFeature("IPV6", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, gid) & PERM_WRITE)){
	state	    = websGetVar(wp, T("ipv6_state"), T("disable"));
	mode	    = websGetVar(wp, T("ipv6_mode"), T("dhcp"));
	list  = append_cfg_item(list, gid, "state", state);
	if(!strncmp(state, "enable", 6)) {
	    list  = append_cfg_item(list, gid, "mode", mode);
	    if(!strncmp(mode, "static", 6)){
		// get v6 address
		addr = websGetVar(wp, T("addr"), T(""));
		prefix = websGetVar(wp, T("prefix"), T("64"));
		gateway = websGetVar(wp, T("gateway"), T(""));
		dns1 = websGetVar(wp, T("dns1"), T(""));
		dns2 = websGetVar(wp, T("dns2"), T(""));
		list  = append_cfg_item(list, gid, "addr", addr);
		list  = append_cfg_item(list, gid, "prefix", prefix);
		list  = append_cfg_item(list, gid, "gateway", gateway);
		list  = append_cfg_item(list, gid, "dns1", dns1);
		list  = append_cfg_item(list, gid, "dns2", dns2);
	    }
	}
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/system/ipv6.asp"))); 
}
#endif

#ifdef VLAN_SW
int genLanPortList(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char output[BUFSIZ], item_name[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN];
    int i, j, lan_num = 0;

    cipcfg = init_cipcfg();

    if(get_cfg_item_by_str(cipcfg, "network_L2", "lan_num", item_value) == CIPCFG_SUCCESS)
	lan_num = atoi(item_value);

    websWrite(wp,T("["));
    for (i = 0, j = 0; i < lan_num; i++) {
	sprintf(item_name, "lan:%d.controllable", i);
	if (get_cfg_item_by_str(cipcfg, "network_L2", item_name, item_value) == CIPCFG_SUCCESS &&
		strcmp(item_value, "enable") == 0) {
	    if (j++)
		websWrite(wp,T(","));
	    sprintf(output, "%d", i);
	    websWrite(wp,T(output));
	}
    }
    websWrite(wp,T("]"));

    close_cipcfg(cipcfg);
    return 0;
}

void applyPort(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *port, *vid;
    char tmp[MAX_VALUE_LEN], attr_vid[BUFSIZ], item_value[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int gid = GID_NET_L2;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, ret;

    app = websGetVar(wp, T("apply"), T("false"));

    cipcfg = init_cipcfg();

    chkFeature("VLAN_SW", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, gid) & PERM_WRITE)) {
	port = websGetVar(wp, T("port"), T("1"));
	sprintf(tmp, "lan:%s.controllable", port);
	if (get_cfg_item_by_str(cipcfg, "network_L2", tmp, item_value) == CIPCFG_SUCCESS && 
		strcmp(item_value, "enable") == 0) {
	    vid = websGetVar(wp, T("vid"), T("4096"));
	    sprintf(attr_vid, "lan:%s.vid", port);
	    list = append_cfg_item(list, gid, attr_vid, vid);
	}
    }
    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/system/port_config.asp"))); 
}
#endif

/*
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initSysForm(void)
{
    websFormDefine(T("applySystem"), applySystem);
    websFormDefine(T("applyReboot"), applyReboot);
#ifdef GW_FUNC
    websFormDefine(T("applyVlan"), applyVlan);
    websFormDefine(T("applyWalledGarden"), applyWalledGarden);
#endif
    websFormDefine(T("resetCfg"), resetCfg);
    websFormDefine(T("resetCustomize"), resetCustomize);
#ifdef GRE
    websFormDefine(T("applyGre"), applyGre);
#endif
#ifdef IPV6 
    websFormDefine(T("applyIPv6"), applyIPv6);
#endif
#ifdef CAPWAP
    websFormDefine(T("applyCapwap"), applyCapwap);
#endif
 
    maDefineEgiForm("/backupCfg.egi", backupCfg);
#ifdef GW_FUNC
    maDefineEgiForm("/applyReservIP.egi", applyReservIP);
    maDefineEgiForm("/applyWallAdList.egi", applyWallAdList);
    maDefineEgiForm("/applyUpLogin.egi", applyUpLogin);
    maDefineEgiForm("/applyUpLoginSuccess.egi", applyUpLoginSuccess);
    maDefineEgiForm("/applyUpLoginFail.egi", applyUpLoginFail);
    maDefineEgiForm("/applyUpLoginSuccessOD.egi", applyUpLoginSuccessOD);
    maDefineEgiForm("/applyUpLogoutSuccess.egi", applyUpLogoutSuccess);
    espDefineStringCFunction(0, "delTmplImg", delTmplImg, 0);
#endif
#if defined(GW_FUNC) || defined(CAPWAP)
    espDefineStringCFunction(0, "Restore_Cert", Restore_Cert, 0);
#endif
#ifdef VLAN_SW
    websAspDefine(T("genLanPortList"), genLanPortList);
    websFormDefine(T("applyPort"), applyPort);
#endif
}
