/**
 *  \file web_user.c
 *
 *  This is the cgi modules for A800 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/20/2006     0.1       Jimmy Yang     Initial version
 *
 */

#include "accmgmt_user.h"
#include "accmgmt_curusr.h"
#include "ondemand_idgen.h"

#include "id.h"
#include "cipcfg.h"
#include "syscall.h"
#include "web_usr.h"
#include "string.h"
#include "webfunc.h"
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <regex.h>

extern const int gid_nas[];
extern const int gid_radius[];
extern const int gid_radius_nas[];
extern const int gid_radius_map[];
extern const int gid_frule[];
extern const int gid_policy_route[];
extern const int gid_policy_sess_limit[];
extern const int gid_policy_bandwidth[];
extern const int gid_bill[];
extern const int gid_ticket_term[];
extern const int gid_blacklist[];


int getLocalUserCount(int eid, webs_t wp, int argc, char_t **argv)
{
    int i;
    char *search_item;

    if (ejArgs(argc, argv, T("%s"), &search_item) < 1)
	i = getLocalUserRowCount(NULL);
    else
	i = getLocalUserRowCount(search_item);
    printf("Local User Count = %d\n", i);

    ejsSetReturnValue(eid, mprCreateIntegerVar(i));
    return 0;
}

int getOnDemandUserCount(int eid, webs_t wp, int argc, char_t **argv)
{
    int i;
    char *search_item = NULL;

    if (ejArgs(argc, argv, T("%s"), &search_item) < 1)
	i = getOnDemandUserRowCount(NULL);
    else
	i = getOnDemandUserRowCount(search_item);
    printf("OnDemand User Count = %d\n", i);

    ejsSetReturnValue(eid, mprCreateIntegerVar(i));
    return 0;
}

int getLocalJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char value[BUFSIZ], isFirst = 1;
    LOCAL_USER *local_user_list = NULL, *local_user_list_head = NULL;

    websWrite(wp,T("{\"LOCAL\":["));

    local_user_list = getLocalUser();
    local_user_list_head = local_user_list;

    while(local_user_list != NULL)
    {
        if (isFirst == 0)
            websWrite(wp, T(","));
        sprintf(value, "{\"name\":\"%s\", \"password\":\"%s\", \"gpidx\":\"%d\", \"mac\":\"%s\", \"remark\":\"%s\", \"ipsec\":\"%d\", \"allowed_sz\":\"%d\"}",local_user_list->name, local_user_list->password, local_user_list->gpidx, local_user_list->mac, local_user_list->remark, local_user_list->ipsec, local_user_list->allowed_sz);
        websWrite(wp,T(value));
        isFirst = 0;
        local_user_list = local_user_list->next;
    }
    free_local_usr_list(local_user_list_head);
    websWrite(wp,T("]}"));

    return 0;
}

// TODO: To be moved
typedef enum {
    ONDEMAND_UNKONWN = -100,  // Unknown  
    ONDEMAND_EXPIRED = -1,    // Expired  
    ONDEMAND_ALIVE = 0,       // Alive
    ONDEMAND_ONLINE = 1    // Online
} onedmend_status;

int getOnDemandJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char value[BUFSIZ], status[10]; 
    int isFirst = 1, nowstatus;
    ONDEMAND_USER *user_list = NULL, *user_list_head = NULL;
    time_t tnow;
    unsigned long ttmp;
    int pagenumber, entrynumber;

    if (ejArgs(argc, argv, T("%d %d"), &pagenumber, &entrynumber) < 2) {
       websError(wp, 400, T("Insufficient args\n"));
       return -1;
    }
    websWrite(wp,T("{\"ONDEMAND\":["));

    user_list = getOnDemandUser(pagenumber, entrynumber);
    user_list_head = user_list;
    time(&tnow);
    ttmp = tnow;

    while(user_list != NULL) {
        if (isFirst == 0)
            websWrite(wp, T(","));
	nowstatus = user_list->status;
	if (nowstatus == ONDEMAND_ALIVE) {
	    if (user_list->validatetime > 0 && ttmp > user_list->validatetime)
		nowstatus = ONDEMAND_EXPIRED;
	    if (user_list->expiretime > 0 && ttmp > user_list->expiretime)
		nowstatus = ONDEMAND_EXPIRED;
	}

        switch( nowstatus ) {
        case ONDEMAND_ALIVE:
            strcpy(status, "Alive");
            break;
        case ONDEMAND_ONLINE:
            strcpy(status, "Online");
            break;
        case ONDEMAND_EXPIRED:
            strcpy(status, "Expired");
            break;
        default:
            strcpy(status, "Unknown");
            break;
        }
        sprintf(value, "{\"name\":\"%s\", \"passwd\":\"%s\", \"type\":\"%d\"," 
		" \"status\":\"%s\", \"avilableunit\":\"%lu\", "
		"\"validatetime\":\"%lu\"," " \"acttype\":\"%d\"," 
		"\"expiretime\":\"%lu\", \"durationdays\":\"%d\","
		 "\"unit\":\"%d\", \"grace\":\"%d\", \"remark\":\"%s\"}"
		, user_list->name, user_list->passwd, user_list->type
		, status, user_list->availableunit
		, user_list->validatetime, user_list->acttype
		, user_list->expiretime, user_list->durationdays
		, user_list->unit, user_list->grace, user_list->remark);
        websWrite(wp,T(value));
        isFirst = 0;
        user_list = user_list->next;
    }
    free_ondemand_usr_list(user_list_head);
    websWrite(wp,T("]}"));

    return 0;
}


int getLocalSearchJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char value[BUFSIZ], isFirst = 1;
    LOCAL_USER *local_user_list = NULL, *local_user_list_head = NULL;
    char *search_item;
    int pagenumber, entrynumber;

    if (ejArgs(argc, argv, T("%s %d %d"), &search_item, &pagenumber, &entrynumber) < 1) {
       websError(wp, 400, T("Insufficient args\n"));
       return -1;
    }

    websWrite(wp,T("{\"LOCAL\":["));

    local_user_list = searchLocalUserbyNameAndRemark(search_item, pagenumber, entrynumber);
    local_user_list_head = local_user_list;

    while(local_user_list != NULL)
    {
        if (isFirst == 0)
            websWrite(wp, T(","));
        sprintf(value, "{\"name\":\"%s\", \"password\":\"%s\", \"gpidx\":\"%d\", \"mac\":\"%s\", \"remark\":\"%s\", \"ipsec\":\"%d\", \"allowed_sz\":\"%d\"}",local_user_list->name, local_user_list->password, local_user_list->gpidx, local_user_list->mac, local_user_list->remark, local_user_list->ipsec, local_user_list->allowed_sz);
        websWrite(wp,T(value));
        isFirst = 0;
        local_user_list = local_user_list->next;
    }
    free_local_usr_list(local_user_list_head);
    websWrite(wp,T("]}"));

    return 0;
}

int getOnDemandSearchJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char value[BUFSIZ], status[20];
    int isFirst = 1, nowstatus;
    ONDEMAND_USER *user_list = NULL, *user_list_head = NULL;
    char *search_item;
    time_t tnow;
    unsigned long ttmp;
    int pagenumber, entrynumber;

    if (ejArgs(argc, argv, T("%s %d %d"), &search_item, &pagenumber, &entrynumber) < 3) {
       websError(wp, 400, T("Insufficient args\n"));
       return -1;
    }
    websWrite(wp,T("{\"ONDEMAND\":["));

    user_list = searchOnDemandUserbyNameAndRemark(search_item, pagenumber, entrynumber);
    user_list_head = user_list;

    time(&tnow);
    ttmp = tnow;

    while(user_list != NULL) {
	nowstatus = user_list->status;
	if (nowstatus == ONDEMAND_ALIVE) {
	    if (user_list->validatetime > 0 && ttmp > user_list->validatetime)
		nowstatus = ONDEMAND_EXPIRED;
	    if (user_list->expiretime > 0 && ttmp > user_list->expiretime)
		nowstatus = ONDEMAND_EXPIRED;
	}
        switch( nowstatus ) {
        case ONDEMAND_ALIVE:
            strcpy(status, "Alive");
            break;
        case ONDEMAND_ONLINE:
            strcpy(status, "Online");
            break;
        case ONDEMAND_EXPIRED:
            strcpy(status, "Expired");
            break;
        default:
            strcpy(status, "Unknown");
            break;
        }
        if (isFirst == 0)
            websWrite(wp, T(","));

        sprintf(value, "{\"name\":\"%s\", \"passwd\":\"%s\", \"type\":\"%d\","
		" \"status\":\"%s\", \"avilableunit\":\"%lu\","
		" \"validatetime\":\"%lu\", \"acttype\":\"%d\"," 
		"\"expiretime\":\"%lu\", \"durationdays\":\"%d\"," 
		"\"unit\":\"%d\", \"grace\":\"%d\", \"remark\":\"%s\"}"
		, user_list->name, user_list->passwd, user_list->type
		, status, user_list->availableunit
		, user_list->validatetime, user_list->acttype
		, user_list->expiretime, user_list->durationdays
		, user_list->unit, user_list->grace, user_list->remark);

        websWrite(wp,T(value));
        isFirst = 0;
        user_list = user_list->next;
    }
    free_ondemand_usr_list(user_list_head);
    websWrite(wp,T("]}"));

    return 0;
}

/**
 * Set/save the system device information to device
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
//TODO : add the system time stting 
void applyUserRadius(webs_t wp, char_t *path, char_t *query)
{
    char_t *postfix, *status_8021x, *fullname;
    char_t *ip0, *authport0, *accport0, *secret0, *acctenabled0, *authproto0;
    char_t *ip1, *authport1, *accport1, *secret1, *acctenabled1, *authproto1;
    char_t *current_radius, *blacklist;
    char_t *app;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    char tmp[256];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret;

    app = websGetVar(wp, T("apply"), T("true"));

    // TODO : Add Permission Control
    current_radius = websGetVar(wp, T("radius"), T(""));
    int i = atoi(current_radius);

    postfix = websGetVar(wp, T("postfix"), T(""));
    status_8021x = websGetVar(wp, T("status_8021x"), T("disable"));
    fullname = websGetVar(wp, T("fullname"), T("disable"));
    blacklist = websGetVar(wp, T("blacklist"), T("0"));

    ip0 = websGetVar(wp, T("ip0"), T(""));
    authport0 = websGetVar(wp, T("port0"), T(""));
    secret0 = websGetVar(wp, T("secret0"), T(""));
    acctenabled0 = websGetVar(wp, T("acctenabled0"), T(""));
    accport0 = websGetVar(wp, T("acctport0"), T(""));
    authproto0 = websGetVar(wp, T("auth_proto0"), T("PAP"));

    ip1 = websGetVar(wp, T("ip1"), T(""));
    authport1 = websGetVar(wp, T("port1"), T(""));
    secret1 = websGetVar(wp, T("secret1"), T(""));
    acctenabled1 = websGetVar(wp, T("acctenabled1"), T(""));
    accport1 = websGetVar(wp, T("acctport1"), T(""));
    authproto1 = websGetVar(wp, T("auth_proto1"), T("PAP"));

    list=append_cfg_item(NULL, gid_radius[i], "postfix", postfix);
    list=append_cfg_item(list, gid_radius_nas[i], "dot1x", 
	    (strcmp(status_8021x, "enable") == 0 ? "Enabled" : "Disabled"));
    list=append_cfg_item(list, gid_radius_nas[i], "fullname", 
	    (strcmp(fullname, "enable") == 0 ? "Enabled" : "Disabled"));
    list=append_cfg_item(list, gid_radius[i], "blacklist", blacklist);
    list=append_cfg_item(list, gid_radius[i], "serverip1", ip0);
    list=append_cfg_item(list, gid_radius[i], "authport1", authport0);
    list=append_cfg_item(list, gid_radius[i], "authsecret1", secret0);
    list=append_cfg_item(list, gid_radius[i], "acct1", acctenabled0);
    list=append_cfg_item(list, gid_radius[i], "acctport1", accport0);
    list=append_cfg_item(list, gid_radius[i], "authtype1", authproto0);

    list=append_cfg_item(list, gid_radius[i], "serverip2", ip1);
    list=append_cfg_item(list, gid_radius[i], "authport2", authport1);
    list=append_cfg_item(list, gid_radius[i], "authsecret2", secret1);
    list=append_cfg_item(list, gid_radius[i], "acct2", acctenabled1);
    list=append_cfg_item(list, gid_radius[i], "acctport2", accport1);
    list=append_cfg_item(list, gid_radius[i], "authtype2", authproto1);


    if(list != NULL){
        const char *auth_gid[6] = {"auth1", "auth2", "auth3", "auth4", "auth5", "auth6"};
        char other_postfix[MAX_VALUE_LEN];
        int j;

        print_list(list);

        cipcfg=init_cipcfg();

        for(j=0; j<6; j++) {
            get_cfg_item_by_str(cipcfg, auth_gid[j], "postfix", other_postfix);
            if(((i+1) != j) && !strcmp(postfix, other_postfix)) {
                sprintf(tmp, "/user/authradius.asp?radius=%s&err_code=1"
			, websGetVar(wp, "radius", "0"));
                websRedirect(wp,
			websGetVar(wp, T("HTTP_REFERER"), T(tmp)));
                return;
            } 
        }
        
        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return;
        }

        while(list != NULL)
        {
            if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
            {
               applylist[applist_idx] = list->gid;
               applist_idx++;
            }
            list = list->next;
        }
        applylist[applist_idx] = GID_END;

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                free(applylist);
                return;
            }
        }

        close_cipcfg(cipcfg);
    }

    sprintf(tmp, "/user/authradius.asp?radius=%s", websGetVar(wp, "radius", "0"));
    websRedirect(wp,
            websGetVar(wp, T("HTTP_REFERER"), T(tmp)));

}

void applyUsr8021x(webs_t wp, char_t *path, char_t *query)
{
    char_t  *act, *type, *ipaddr, *mask, *secret, *entry, *app;
    char item_value[MAX_VALUE_LEN];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int i, ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    act = websGetVar(wp, T("act"), T(""));

    app = websGetVar(wp, T("apply"), T("true"));
    cipcfg = init_cipcfg();
    if ( !strncmp("edit", act, 4) || !strncmp("add", act, 3) ){
        if (!strncmp("edit", act, 4)) {
            entry = websGetVar(wp, T("entry"), T(""));
            i = atoi(entry);
        }else if (!strncmp("add", act, 3)){
            // find an empty entry to add.
            item_value[0] = '1';
            for(i=0; *item_value!='\0'; i++){
                ret = get_cfg_item(cipcfg, gid_nas[i], "ipaddr", item_value);
                if( ret == CIPCFG_ERR_NODE_NOT_FOUND)
                {
                    i++;
                    break;
                }
            }
            i-- ;
        }
        type = websGetVar(wp, T("type"), T(""));
        ipaddr = websGetVar(wp, T("ip"), T(""));
        mask = websGetVar(wp, T("mask"), T(""));
        secret = websGetVar(wp, T("secret"), T(""));

        list=append_cfg_item(NULL, gid_nas[i], "type", type);
        list=append_cfg_item(list, gid_nas[i], "ipaddr", ipaddr);            
        list=append_cfg_item(list, gid_nas[i], "mask", mask);
        list=append_cfg_item(list, gid_nas[i], "secret", secret);
    }else if (!strncmp("delete", act, 6)){
        entry = websGetVar(wp, T("entry"), T(""));
        i = atoi(entry);

        list=append_cfg_item(NULL, gid_nas[i], "type", "0");
        list=append_cfg_item(list, gid_nas[i], "ipaddr", "");            
        list=append_cfg_item(list, gid_nas[i], "mask", "");
        list=append_cfg_item(list, gid_nas[i], "secret", "");
    }else if (!strncmp("save", act, 4)){
        for (i=1; i<10; i++){
        }
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        { 
           applylist[applist_idx] = list->gid;
           applist_idx++; 
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);
    websRedirect(wp, "/user/8021x.asp" ); 
}

void applyUsrPolicyFirewall(webs_t wp, char_t *path, char_t *query)
{
    char_t *entry, *app;
    char_t *act;
    char item_value[MAX_VALUE_LEN], item_name[32], tmp[256];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int i, j, ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int cur_policy = 0 ;
    char_t *policy = websGetVar(wp, T("policy"), T("0"));
    char *name_list[] = {
	    "srcip", "srcmask", "dstip", "dstmask", "proto", "action",
	    "sport", "dport", "icmptype", "proto_no", NULL,
	};

    cur_policy = atoi(policy);

    act = websGetVar(wp, T("act"), T(""));

    app = websGetVar(wp, T("apply"), T("true"));
    cipcfg = init_cipcfg();
    if ( !strncmp("edit", act, 4) || !strncmp("add", act, 3) ){
        if (!strncmp("edit", act, 4)) {
            entry = websGetVar(wp, T("entry"), T(""));
            i = atoi(entry);
        }else if (!strncmp("add", act, 3)){
            // find an empty entry to add.
            item_value[0] = '1';
            for(i=0; *item_value!='\0'; i++){
                sprintf(item_name, "frule:%d.srcip", i);
                ret = get_cfg_item(cipcfg, gid_frule[cur_policy], item_name, item_value);
                if( ret != CIPCFG_SUCCESS)
                {
                    i++;
                    break;
                }
            }
            i-- ;
        }

        sprintf(item_name, "frule:%d.idx", i);
        sprintf(item_value, "%d", i); 
        list=append_cfg_item(NULL, gid_frule[cur_policy], item_name, item_value);
        sprintf(item_name, "frule:%d.name", i);
        list=append_cfg_item(list, gid_frule[cur_policy], item_name, "");
	for (j=0; name_list[j] != NULL; j++) {
	    entry = websGetVar(wp, T(name_list[j]), T(""));
	    sprintf(item_name, "frule:%d.%s", i, name_list[j]);
	    list = append_cfg_item(list, gid_frule[cur_policy], item_name, entry);
	}
    }else if (!strncmp("delete", act, 6)){
        entry = websGetVar(wp, T("entry"), T(""));
        i = atoi(entry);
        sprintf(item_name, "frule:%d.idx", i);
        sprintf(item_value, "%d", i); 
        list=append_cfg_item(NULL, gid_frule[cur_policy], item_name, item_value);
        sprintf(item_name, "frule:%d.name", i);
        list=append_cfg_item(list, gid_frule[cur_policy], item_name, "");
	for (j=0; name_list[j] != NULL; j++) {
	    sprintf(item_name, "frule:%d.%s", i, name_list[j]);
	    list = append_cfg_item(list, gid_frule[cur_policy], item_name, "");
	}
    }else{
        sprintf(tmp, "/user/policy_firewall.asp?policy=%d", cur_policy);
        websRedirect(wp,
                websGetVar(wp, T("HTTP_REFERER"), T(tmp)));
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
// FIXME : Add error redirect
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        { 
           applylist[applist_idx] = list->gid;
           applist_idx++; 
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    sprintf(tmp, "/user/policy_firewall.asp?policy=%d", cur_policy);
    websRedirect(wp,
            websGetVar(wp, T("HTTP_REFERER"), T(tmp)));

}

void applyUsrPolicyRoute(webs_t wp, char_t *path, char_t *query)
{
    char_t *entry, *app;
    char_t *act;
    char item_value[MAX_VALUE_LEN], item_name[32], tmp[256];
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int i, j, ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int cur_policy = 0;
    char_t *policy = websGetVar(wp, T("policy"), T("0"));
    char *name_list[] = {
	    "destip", "netmask", "gateway", NULL,
	};

    cur_policy = atoi(policy);

    act = websGetVar(wp, T("act"), T(""));

    app = websGetVar(wp, T("apply"), T("true"));
    cipcfg = init_cipcfg();
    if ( !strncmp("edit", act, 4) || !strncmp("add", act, 3) ){
        if (!strncmp("edit", act, 4)) {
            entry = websGetVar(wp, T("entry"), T(""));
            i = atoi(entry);
        }else if (!strncmp("add", act, 3)){
            // find an empty entry to add.
            item_value[0] = '1';
            for(i=0; *item_value!='\0'; i++){
                sprintf(item_name, "route:%d.destip", i);
                ret = get_cfg_item(cipcfg, gid_policy_route[cur_policy], item_name, item_value);
                if( ret != CIPCFG_SUCCESS)
                {
                    i++;
                    break;
                }
            }
            i-- ;
        }

        sprintf(item_name, "route:%d.idx", i);
        sprintf(item_value, "%d", i); 
        list=append_cfg_item(NULL, gid_policy_route[cur_policy], item_name, item_value);
	for (j=0; name_list[j] != NULL; j++) {
	    entry = websGetVar(wp, T(name_list[j]), T(""));
	    sprintf(item_name, "route:%d.%s", i, name_list[j]);
	    list=append_cfg_item(list, gid_policy_route[cur_policy], item_name, entry);
	}
    }else if (!strncmp("delete", act, 6)){
        entry = websGetVar(wp, T("entry"), T(""));
        i = atoi(entry);
        sprintf(item_name, "route:%d.idx", i);
        sprintf(item_value, "%d", i); 
        list=append_cfg_item(NULL, gid_policy_route[cur_policy], item_name, item_value);
	for (j=0; name_list[j] != NULL; j++) {
	    sprintf(item_name, "route:%d.%s", i, name_list[j]);
	    list=append_cfg_item(list, gid_policy_route[cur_policy], item_name, "");
	}
    }else{
        sprintf(tmp, "/user/policy_route.asp?policy=%d", cur_policy);
        websRedirect(wp,
                websGetVar(wp, T("HTTP_REFERER"), T(tmp)));
    }

        print_list(list);

        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return ;
        }

        while(list != NULL)
        {
            if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
            { 
               applylist[applist_idx] = list->gid;
               applist_idx++; 
            }
            list = list->next;
        }
        applylist[applist_idx] = GID_END;

        // check if apply or just save settings into config file.
        if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
            }else{
                close_cipcfg(cipcfg);
                websRedirect( wp, "/utilities/reboot.asp" );
                free(applylist);
                return;
            }
        }

        close_cipcfg(cipcfg);

    sprintf(tmp, "/user/policy_route.asp?policy=%d", cur_policy);
    websRedirect(wp,
            websGetVar(wp, T("HTTP_REFERER"), T(tmp)));
}

void applyLocalUser(webs_t wp, char_t *path, char_t *query)
{
    char_t *name, *password, *mac, *remark, *policy, *postfix, *status_8021x, *status_roam, *multilogin;
    char_t *app, *rosession, *roidle, *rointerim;
    char_t *act;
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int ret, session=0, idle=0, interim=0;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, total_num;
    char buf[MAX_VALUE_LEN];

    act = websGetVar(wp, T("act"), T(""));

    app = websGetVar(wp, T("apply"), T("true"));
    cipcfg = init_cipcfg();
    if ( !strcmp("edit", act) || !strcmp("add", act) ){

        name = websGetVar(wp, T("user_name"), T(""));
        password = websGetVar(wp, T("password"), T(""));
        mac = websGetVar(wp, T("mac"), T(""));
        remark = websGetVar(wp, T("remark"), T(""));
        policy = websGetVar(wp, T("policy"), T("0"));

        LOCAL_USER *addusr = (LOCAL_USER *) malloc(sizeof(LOCAL_USER));

        strcpy(addusr->name, name);
        strcpy(addusr->password, password);
        addusr->gpidx = atoi(policy);
        strcpy(addusr->mac, mac);
        strcpy(addusr->remark, remark);
        addusr->ipsec=0;
        addusr->allowed_sz=0;

        if(strcmp("edit", act) == 0)
            ret = updateLocalUser(addusr);
        else if(!strncmp("add", act, 3)) {
	    total_num = getLocalUserRowCount(NULL);
	    if(total_num + 1 > getMaxLocalUser())
		ret = -2;	// exceed max num 
	    else
		ret = addLocalUser(addusr);
	}

        free(addusr);
        if(ret != 0) {
	    free(applylist);
	    close_cipcfg(cipcfg);
/*
	    if (ret == 1)
		websRedirect(wp, "/user/local.asp?err_code=1" ); 
	    else if (ret == -2)
		websRedirect(wp, "/user/local.asp?err_code=2" ); 
*/
	    sprintf(buf, "/user/local.asp?err_code=%d", ret);
	    websRedirect(wp, buf); 
            return;
        }

    }else if (!strcmp("delete", act)){ // Delete a Local User
        name = websGetVar(wp, T("user_name"), T(""));
        deleteLocalUserbyName(name);

    }else if (!strcmp("deleteall", act)){ // Delete a Local User
        deleteLocalUserbyName(NULL);
    }else{  // Save or Apply
        const char *auth_gid[6] = {"auth1", "auth2", "auth3", "auth4", "auth5", "auth6"};
        char other_postfix[MAX_VALUE_LEN], *blacklist;
        int j;

        postfix = websGetVar(wp, T("postfix"), T(""));
        status_8021x = websGetVar(wp, T("status_8021x"), T(""));
        multilogin = websGetVar(wp, T("multilogin"), T("disable"));
        status_roam = websGetVar(wp, T("roamingout"), T(""));
	rosession = websGetVar(wp, T("sessionto"), T("120"));
	roidle = websGetVar(wp, T("idleto"), T("10"));
	rointerim = websGetVar(wp, T("interimup"), T("5"));
	blacklist = websGetVar(wp, T("blacklist"), T("0"));
	session = atoi(rosession) * 60;
	idle = atoi(roidle) * 60;
	interim = atoi(rointerim) * 60;
        for(j=1; j<6; j++) {
            get_cfg_item_by_str(cipcfg, auth_gid[j], "postfix", other_postfix);
            if(!strcmp(postfix, other_postfix)) {
                websRedirect(wp, "/user/local.asp?err_code=-3"); 
                return;
            } 
        }

        list=append_cfg_item(NULL, GID_MGMT_AUTH1, "postfix", postfix);
        list=append_cfg_item(list, GID_MGMT_AUTH1, "dot1x", 
		(strcmp(status_8021x, "enable") == 0 ? "Enabled" : "Disabled"));
        list=append_cfg_item(list, GID_MGMT_AUTH1, "multilogin", multilogin);
        list=append_cfg_item(list, GID_MGMT_AUTH1, "localroam", 
		(strcmp(status_roam, "enable") == 0 ? "Enabled" : "Disabled"));
        list=append_cfg_item(list, GID_MGMT_AUTH1, "blacklist", blacklist);
	if (strcmp(status_roam, "enable") == 0) {
	    sprintf(buf, "%d", session);
	    list=append_cfg_item(list, GID_MGMT_AUTH1_RO, "session", buf);
	    sprintf(buf, "%d", idle);
	    list=append_cfg_item(list, GID_MGMT_AUTH1_RO, "idle", buf);
	    sprintf(buf, "%d", interim);
	    list=append_cfg_item(list, GID_MGMT_AUTH1_RO, "interim", buf);
	}
    }

    if(list != NULL) 
    {
        print_list(list);

        // TODO - Error Redirect
        ret=set_cfg_items(cipcfg, list);
        if (ret != CIPCFG_SUCCESS) {
            return ;
        }
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        { 
           applylist[applist_idx] = list->gid;
           applist_idx++; 
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);

    websRedirect(wp, "/user/local.asp" ); 
}

int getCurrentUserJson(int eid, webs_t wp, int argc, char_t **argv)
{
    char value[BUFSIZ], isFirst = 1;
    CUR_USER *current_user_list = NULL, *current_user_list_head = NULL;

    websWrite(wp,T("{\"CURUSR\":["));

    current_user_list = get_all_cur_user("ALL");
    current_user_list_head = current_user_list;

    while(current_user_list != NULL)
    {
        if (isFirst == 0)
            websWrite(wp, T(","));

        sprintf(value, "{\"name\":\"%s\", \"ip\":\"%s\", \"mac\":\"%s\","
		" \"idle\":\"%d\"}",current_user_list->fname, 
		current_user_list->ip, current_user_list->mac, 
		current_user_list->idle_time);
        websWrite(wp,T(value));
        isFirst = 0;
        current_user_list = current_user_list->next;
    }
    free_cur_user_list(current_user_list_head);
    websWrite(wp,T("]}"));

    return 0;
}

void applyOnDemandGeneral(webs_t wp, char_t *path, char_t *query) 
{

  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0;
  char_t *postfix, *currency, *currname, *totTicket, *baudRate, *sync_int;
  char_t *act, *blacklist;
  int ret;

     act = websGetVar(wp, T("apply"), T("false")); 
 
     cipcfg = init_cipcfg();
 
     postfix  = websGetVar(wp, T("postfix"), T("ondemand"));
     currency = websGetVar(wp, T("currency"), T("US"));
     currname = websGetVar(wp, T("currname"), T("none"));
     totTicket= websGetVar(wp, T("totTicket"), T("one"));
     baudRate = websGetVar(wp, T("baudRate"), T("9600"));
     sync_int = websGetVar(wp, T("syncint"), T("15"));
     blacklist = websGetVar(wp, T("blacklist"), T("0"));

     list = append_cfg_item(NULL, GID_MGMT_AUTH6, "postfix", postfix);
     list = append_cfg_item(list, GID_MGMT_AUTH6, "currency", currency);
     list = append_cfg_item(list, GID_MGMT_AUTH6, "othercurr", currname);
     list = append_cfg_item(list, GID_MGMT_AUTH6, "baudrate", baudRate);
     list = append_cfg_item(list, GID_MGMT_AUTH6, "tickets", totTicket);
     list = append_cfg_item(list, GID_MGMT_AUTH6, "syncint", sync_int); 
     list = append_cfg_item(list, GID_MGMT_AUTH6, "blacklist", blacklist); 

     print_list(list);
     ret=set_cfg_items(cipcfg, list);
     if (ret != CIPCFG_SUCCESS) {
            return;
     }

     while(list != NULL)
     {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
      }
      applylist[applist_idx] = GID_END;

      if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
      }
      close_cipcfg(cipcfg);
      websRedirect(wp, "/user/OnDemandGeneralSetting.asp");
} 

void applyTickConf(webs_t wp, char_t *path, char_t *query)
{
  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0;
  char_t *header1, *header2, *header3, *remark, *ssid, *wkey, *footer1, *footer2, *footer3;
  char_t *act;
  int ret;

  header1 = websGetVar(wp, T("header1"), T("")); 
  header2 = websGetVar(wp, T("header2"), T(""));
  header3 = websGetVar(wp, T("header3"), T(""));
  ssid = websGetVar(wp, T("ssid"), T(""));
  wkey = websGetVar(wp, T("wkey"), T(""));
  remark = websGetVar(wp, T("remark"), T(""));
  footer1 = websGetVar(wp, T("footer1"), T(""));
  footer2 = websGetVar(wp, T("footer2"), T(""));
  footer3 = websGetVar(wp, T("footer3"), T(""));

  cipcfg = init_cipcfg();
  list = append_cfg_item(NULL, GID_MGMT_AUTH6_TICKET,"header1", header1);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"header2", header2);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"header3", header3);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"remark", remark); 
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"ssid", ssid);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"wkey", wkey);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"footer1", footer1);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"footer2", footer2);
  list = append_cfg_item(list, GID_MGMT_AUTH6_TICKET,"footer3", footer3);

  act = websGetVar(wp, T("apply"), T("false"));
  print_list(list);

  ret=set_cfg_items(cipcfg, list);
  if (ret != CIPCFG_SUCCESS) {
           return;
  }

  while(list != NULL)
  {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
   }
   applylist[applist_idx] = GID_END;

   if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
   }
   close_cipcfg(cipcfg);
   websRedirect(wp, "/user/OnDemandTicketConf.asp");
}

void enableBillingPlan(webs_t wp, char_t *path, char_t *query)
{
  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist =  malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0, i ;
  char_t *benabled, *act;
  int bgid, ret;
  char tmp[256];

  cipcfg = init_cipcfg();
  act = websGetVar(wp, T("apply"), T("false")); 
  for (i=1; i<=10; i++) {
       bgid = gid_bill[i-1];
       sprintf(tmp, "benabled%d", i);
       benabled = websGetVar(wp, T(tmp), T("Disabled"));
       if ( !strncmp( benabled, "Enabled", 7))
          list = append_cfg_item(NULL, bgid, "status", benabled);
       else
	  list = append_cfg_item(NULL, bgid, "status", "Disabled");

       print_list(list);
       ret=set_cfg_items(cipcfg, list);
       if (ret != CIPCFG_SUCCESS) {
           return;
       }
  }
  while(list != NULL)
  {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
   }
   applylist[applist_idx] = GID_END;

   if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
   }
   close_cipcfg(cipcfg);
   websRedirect(wp, "/user/BillingConf.asp"); 
} 
                
void applyBillingConf(webs_t wp, char_t *path, char_t *query)
{
  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0;
  char_t *billNo, *btype, *bhrs, *bmts, *adays, *ahrs, *edays, *price, *acttype, *expire;
  char_t *act;
  int ret, gid;

 
  billNo = websGetVar(wp, T("curr_bill"), T("1"));
  acttype = websGetVar(wp, T("acttype"), T("1"));
  btype = websGetVar(wp, T("btype"), T("TIME"));
  bhrs = websGetVar(wp, T("bhrs"), T("0"));
  bmts = websGetVar(wp, T("bmts"), T("0"));
  adays = websGetVar(wp, T("expiration1"), T("0"));
  ahrs = websGetVar(wp, T("expiration_hour1"), T("2"));
  edays = websGetVar(wp, T("edays"), T("0"));
  price = websGetVar(wp, T("price"), T("0"));

  gid = gid_bill[atoi(billNo)-1];
  printf("Billing Plan No. : %d\n", atoi(billNo));

  cipcfg = init_cipcfg();  
  list = append_cfg_item(list, gid, "btype", btype);
  list = append_cfg_item(list, gid, "acttype", acttype);
  list = append_cfg_item(list, gid, "qhrs", bhrs);
  list = append_cfg_item(list, gid, "qmins", bmts);
  list = append_cfg_item(list, gid, "actdays", adays);
  list = append_cfg_item(list, gid, "acthrs", ahrs);

  list = append_cfg_item(list, gid, "edays", edays);

  list = append_cfg_item(list, gid, "price", price);

  act = websGetVar(wp, T("apply"), T("false"));
  print_list(list);

  ret=set_cfg_items(cipcfg, list);
  if (ret != CIPCFG_SUCCESS) {
           return;
  }

  while(list != NULL)
  {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
   }
   applylist[applist_idx] = GID_END;

   if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
   }
   close_cipcfg(cipcfg);
   websRedirect(wp, "/user/BillingConf.asp");

}

void applyTicketTerminal(webs_t wp, char_t *path, char_t *query)
{
  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0;
  int ret, gid_term, i;
  char_t *ipAddr, *portNo, *location, *remark, *act;
  char tmp[256];
 
  cipcfg = init_cipcfg(); 
  for (i=1; i<=10; i++) {
       memset(&tmp, 0x0, 256);
       gid_term = gid_ticket_term[i-1];
       sprintf(tmp, "ipAddr%d", i);
       ipAddr = websGetVar(wp, T(tmp), T(""));
       list = append_cfg_item(list, gid_term, "ipaddr", ipAddr);  
       sprintf(tmp, "portNo%d", i);
       portNo = websGetVar(wp, T(tmp), T(""));
       list = append_cfg_item(list, gid_term, "port", portNo);
       sprintf(tmp, "location%d", i);
       location = websGetVar(wp, T(tmp), T(""));
       list = append_cfg_item(list, gid_term, "location", location);
       sprintf(tmp, "remark%d", i);
       remark = websGetVar(wp, T(tmp), T(""));
       list = append_cfg_item(list, gid_term, "remarks", remark);

  }

  act = websGetVar(wp, T("apply"), T("false"));

  print_list(list);
  ret=set_cfg_items(cipcfg, list);
  if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return;
  }
  while(list != NULL)
  {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
   }
   applylist[applist_idx] = GID_END;

   if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
   }
   close_cipcfg(cipcfg);

   websRedirect(wp, "/user/OnDemandTickTerminal.asp");
}
 
void ConfirmOnDemandAccount(webs_t wp, char_t *path, char_t *query)
{
  CIPCFG *cipcfg = NULL;
  ITEM_LIST *list = NULL;
  int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
  int applist_idx = 0;
  int ret, gid, hrs, mins, i;
  char_t *selectall, *act;
  char item_value[MAX_VALUE_LEN];

  selectall = websGetVar(wp, T("selectall"), T("false"));

  cipcfg = init_cipcfg(); 
  if( !strncmp( selectall, "true", 4) )
  {
      for (i=1; i<=10; i++) {
         gid = gid_bill[i]; 
         ret = get_cfg_item(cipcfg, gid, "bhrs", item_value);
         if (*item_value != '\0')
		hrs = atoi(item_value);
	 ret = get_cfg_item(cipcfg, gid, "mins", item_value);
	 if (*item_value != '\0')
		mins = atoi(item_value);
         list = append_cfg_item(NULL, gid, "status", "Enabled");
      }
  }

  cipcfg = init_cipcfg();
  act = websGetVar(wp, T("apply"), T("false"));
  print_list(list);

  ret=set_cfg_items(cipcfg, list);
  if (ret != CIPCFG_SUCCESS) {
           return;
  }

  while(list != NULL)
  {
         if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
         {
               applylist[applist_idx] = list->gid;
               applist_idx++;
         }
         list = list->next;
   }
   applylist[applist_idx] = GID_END;

   if ( !strncmp("true", act, 4 ) ) {
         if(can_apply(applylist) == CIPCFG_SUCCESS){
                apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
         }
   }
   close_cipcfg(cipcfg);

   websRedirect(wp, "/user/OnDemandBillingConf.asp");
}

void CreateOnDemandAccount(webs_t wp, char_t *path, char_t *query)
{
    CIPCFG *cipcfg = NULL;
    char status[MAX_VALUE_LEN];
    char username[25], tmp[512];
    char_t *billNo, *remark, *units;
    OD_GENERATED_USER *oduser;
    ONDEMAND_USER *user;
    int gid, ret;

    cipcfg = init_cipcfg();
    billNo = websGetVar(wp, T("curr_bill"), T("0"));
    remark = websGetVar(wp, T("remark"), T(""));
    units = websGetVar(wp, T("units"), T("1"));

    gid = gid_bill[atoi(billNo)-1];

    printf("Billing Plan No. : %d\n", atoi(billNo));
    ret = get_cfg_item(cipcfg, gid, "status", status);
    if (!strncmp(status , "Enabled", 7)) {
        printf("Calling IDGEN tool to generate Username and Password.\n");
        oduser = (OD_GENERATED_USER *) calloc(1, sizeof(OD_GENERATED_USER));
        ret = generateODuser(atoi(billNo), remark, oduser);
        strcpy(username, oduser->name);
        free(oduser);
        close_cipcfg(cipcfg);

        if( ret <= 0) {
            sprintf(tmp, "/user/generateODUser.asp?err_code=-1");
            websRedirect(wp, tmp);
            return;
        }
        user = (ONDEMAND_USER *) malloc(sizeof(ONDEMAND_USER));
        user = searchOnDemandUserbyName(username);
        user->unit = atoi(units);
        user->availableunit *= (unsigned long)user->unit;
        if( user->acttype == 0)
                user->validatetime = getexptime(0) + user->availableunit;
        updateOnDemandUser(user);
        free(user);

        if( ret > 0 ) {
           sprintf(tmp, "/user/OnDemandReceipt.asp?uname=%s&curr_bill=%d", username, atoi(billNo));
           websRedirect(wp, tmp);
        }
    }
    else {
    	close_cipcfg(cipcfg);
    	free(user);
    	sprintf(tmp, "/user/OnDemandReceipt.asp?error=%s", "Billing Plan has been disabled.");
    	websRedirect(wp, tmp);
    } 
} 


int localUsrToCSV(char *csv_path)
{   
    LOCAL_USER *head, *tmp;
    char buf[LOCALCSV_MAXLEN], wbuf[LOCALCSV_MAXLEN];
    FILE *fp;
    int i=0, j, fd;
    int ret = -1;

    fd = open(csv_path, O_RDWR | O_CREAT);
    if(fd < 0) 
        return -1;
    
    // Lock the file 
    if(flock(fd, LOCK_EX) != 0) {
        close(fd);
        return -1; 
    }

    fp = fdopen(fd, "w");
    if(fp == NULL) {
        flock(fd, LOCK_UN);
        close(fd);
        return -1;
    }
    ftruncate(fd, 0);
    fprintf(fp, "#*Username,*Password,*MAC Address,*Policy ID,*Remark\r\n");

    head = getLocalUser();
    tmp = head;

    ret = 0;
    while(head != NULL) {
        memset(buf, 0, LOCALCSV_MAXLEN);
        memset(wbuf, 0, LOCALCSV_MAXLEN);

        sprintf(buf, "%s,%s,%s,%d,%s\n", head->name, head->password,
                        head->mac, head->gpidx, head->remark);

        for(i=j=0; buf[i] != '\0'; i++) {
            if(i == 0 || buf[i] == ',' || buf[i] == '"' || buf[i] == '\n')
                wbuf[j++] = '"';
            if (buf[i] == '\n')
                wbuf[j++] = '\r';
            wbuf[j++] = buf[i];
            if( buf[i] == ',' )
                wbuf[j++] = '"';
        }
        wbuf[j] = '\0';
        if (fwrite(wbuf, sizeof(char), j, fp) < j) {
            ret = -1;
            break;
        }
        head = head->next;
    }

    free_local_usr_list(tmp);
    flock(fd, LOCK_UN);
    fclose(fp);
    return ret;
}

int ondemandUsrToCSV(char *csv_path)
{   
    ONDEMAND_USER *head, *tmp;
    char buf[ONDEMANDCSV_MAXLEN], wbuf[ONDEMANDCSV_MAXLEN];
    FILE *fp;
    int i=0, j, fd;
    int ret = -1;
    time_t tmp_t;
    char *ptr, tbuf[MAX_VALUE_LEN];
    struct tm tm;

    fd = open(csv_path, O_RDWR | O_CREAT);
    if(fd < 0) 
        return -1;
    
    // Lock the file 
    if(flock(fd, LOCK_EX) != 0) {
        close(fd);
        return -1; 
    }

    fp = fdopen(fd, "w");
    if(fp == NULL) {
        flock(fd, LOCK_UN);
        close(fd);
        return -1;
    }
    ftruncate(fd, 0);
    fprintf(fp, "#*Username,*Password,*Type,*Status,*Remaining Quota (sec),*Account Activation"
		",*Validity Period,*Unit,Grace,*1st Login Expiration Time,*Account Expired Time,*Remark\r\n");

    head = getOnDemandUser(0, 0);
    tmp = head;

    ret = 0;
    while(head != NULL) {
        memset(buf, 0, ONDEMANDCSV_MAXLEN);
        memset(wbuf, 0, ONDEMANDCSV_MAXLEN);
	
	ptr = buf;
	ptr += sprintf(ptr, "%s,%s,%d,%d,%lu,%d,%d,%d,%d", head->name, head->passwd,
			head->type, head->status, head->availableunit,
			head->acttype, head->durationdays, head->unit,
			head->grace);
	tmp_t = (time_t) head->validatetime;
	gmtime_r(&tmp_t, &tm);
	if (strftime(tbuf, MAX_VALUE_LEN, "%Y/%m/%d %H:%M:%S", &tm) == 0)
	    *tbuf = '\0';
	ptr += sprintf(ptr, ",%s", tbuf);
	tmp_t = (time_t) head->expiretime;
	gmtime_r(&tmp_t, &tm);
	if (strftime(tbuf, MAX_VALUE_LEN, "%Y/%m/%d %H:%M:%S", &tm) == 0)
	    *tbuf = '\0';
	ptr += sprintf(ptr, ",%s", tbuf);
	sprintf(ptr, ",%s\n", head->remark);

        for(i=j=0; buf[i] != '\0'; i++) {
            if(i == 0 || buf[i] == ',' || buf[i] == '"' || buf[i] == '\n')
                wbuf[j++] = '"';
            if (buf[i] == '\n')
                wbuf[j++] = '\r';
            wbuf[j++] = buf[i];
            if( buf[i] == ',' )
                wbuf[j++] = '"';
        }
        wbuf[j] = '\0';
        if (fwrite(wbuf, sizeof(char), j, fp) < j) {
            ret = -1;
            break;
        }
        head = head->next;
    }

    free_ondemand_usr_list(tmp);
    flock(fd, LOCK_UN);
    fclose(fp);
    return ret;
}

static void exportLocalUser(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *file = LOCAL_DB_CSV_PATH;
    FILE *fp;
    int ret;

    /* Permission Check */
#ifdef PERMISSION_CONTROL_BY_PAGE
    ret = permissionChkByCookie(rq);
    if(ret != 0)
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-104");
#endif 
#ifdef PERMISSION_CONTROL_BY_GID
    if(!(getOperationPermission(rq, OID_EXPORT_LOCAL) & PERM_EXEC))
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-104");
#endif    

    ret = localUsrToCSV(file);
    if (ret < 0) {
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-101");
    } else if (!(fp = fopen(file, "r"))) {
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-102");
    } else {
        int fd = fileno(fp);

        if (flock(fd, LOCK_SH) != 0) {
            maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-103");
        } else {
            char buf[4096];
            int len;

            maSetHeader(rq, "Cache-Control: no-cache", 0);
            maSetHeader(rq, "Content-type: application/octet-stream", 0);
            maSetHeader(rq, "Content-Disposition: attachment;filename=" LOCAL_DB_CSV_NAME, 0);
            while ((len=fread(buf, sizeof(char), 4096, fp)) > 0)
                maWrite(rq, buf, len);
            flock(fd, LOCK_UN);
        }
        fclose(fp);
    }
}

static void exportOndemandUser(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *file = ONDEMAND_DB_CSV_PATH;
    FILE *fp;
    int ret;

    /* Permission Check */
#ifdef PERMISSION_CONTROL_BY_PAGE
    ret = permissionChkByCookie(rq);
    if(ret != 0)
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-104");
#endif 
#ifdef PERMISSION_CONTROL_BY_GID
    if(!(getOperationPermission(rq, OID_EXPORT_ONDEMAND) & PERM_EXEC))
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-104");
#endif    

    ret = ondemandUsrToCSV(file);
    if (ret < 0) {
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-101");
    } else if (!(fp = fopen(file, "r"))) {
        maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-102");
    } else {
        int fd = fileno(fp);

        if (flock(fd, LOCK_SH) != 0) {
            maRedirect(rq, 302, "/utilities/backup_user.asp?err_code=-103");
        } else {
            char buf[4096];
            int len;

            maSetHeader(rq, "Cache-Control: no-cache", 0);
            maSetHeader(rq, "Content-type: application/octet-stream", 0);
            maSetHeader(rq, "Content-Disposition: attachment;filename=" ONDEMAND_DB_CSV_NAME, 0);
            while ((len=fread(buf, sizeof(char), 4096, fp)) > 0)
                maWrite(rq, buf, len);
            flock(fd, LOCK_UN);
        }
        fclose(fp);
    }
}

/*
  Rules of csv format:
    * Fields that contain commas, double-quotes must be quoted.
    * A quote within a field must be escaped with an additional quote 
      immediately preceding the literal quote.
    * No line-break.
*/
char *parse_csv_value(char *item)
{
    int i=0, j; 
    int head_quote_flag = 0; // flag if the head is double-quotes
    int num_con_quote = 0;   // the number of continous double-quotes

    /* If the head and tail are double-quotes, remove them */
    if( item[i] == '"' ) {
        head_quote_flag = 1;
        if( item[strlen(item)-1] == '"') {
            item[strlen(item)-1] = '\0';
            item++;
        } else {
            /* Error: Fields that contain double-quotes must be quoted */
            return NULL;
        }
    }
    while( i < strlen(item) ) {
        if( item[i] == '\n' ) {
            /* Error: No line-break */
            return NULL;
        }
        /* If there are continous double-quotes, remove one of them */
        if( item[i] == '"' ) {
            /* Error: Fields that contain double-quotes must be quoted */
            if(head_quote_flag == 0)
                return NULL;
            
            j = i;
            num_con_quote = 0;
            while( item[j] == '"' && j < strlen(item)) {
                num_con_quote++;
                j++;
            }

            if((num_con_quote % 2) != 0) 
                return NULL;
            
            for(j = i; j<strlen(item); j++) {
                item[j] = item[j+1];
            }
        }
        i++;
    }
    return item;
}

#ifdef CIPCFG_ITEM_VALIDATION
int LocalUsrValidate(LOCAL_USER *addUsr)
{
    if(cipcfg_item_regex_check("^[A-Za-z0-9._-]{1,20}$", addUsr->name) != 0)
        return -1;

    if(cipcfg_item_regex_check("^[A-Za-z0-9._-]{1,20}$", addUsr->password) != 0)
        return -1;

    if(cipcfg_item_mac_check(addUsr->mac) != CIPCFG_SUCCESS) 
        return -1;
    
    return 0;
}

int OndemandUsrValidate(ONDEMAND_USER *addUsr)
{
    if(cipcfg_item_regex_check("^[A-Za-z0-9._-]{1,20}$", addUsr->name) != 0)
        return -1;

    if(cipcfg_item_regex_check("^[A-Za-z0-9._-]{1,20}$", addUsr->password) != 0)
        return -1;
    return 0;
}
#endif

/* return 0 : success
 * return -1: file format err
 * return -2: exceed max num
 */
int importLocalUser(int fd, int *err_type, int *err_line, int *length)
{
    LOCAL_USER *lusr = NULL, *head = NULL, *tmp1 = NULL;
    FILE *fp = NULL;
    char buf[LOCALCSV_MAXLEN];
    int count = 0;
    char *pch;
    char *tmp;
    int mem = 0, total_policy = 0;
    char *ptr;
    int line=1, total_num, ret = -1;
    char mac_rule[] = "^[0-9a-fA-F]{2}[:][0-9a-fA-F]{2}[:][0-9a-fA-F]{2}[:][0-9a-fA-F]{2}[:][0-9a-fA-F]{2}[:][0-9a-fA-F]{2}$";
    char name_rule[] = "^[a-zA-Z0-9_.-]+$";
    char remark_rule[] = "^[a-zA-Z0-9#@_?:*|. ,-]+$";
    char passwd_rule[] = "^[^][\"'<>\\`~]+$";
    regex_t preg_mac, preg_name, preg_passwd, preg_remark;

    if(err_type != NULL)
	*err_type = IMPLU_ERR_GENERAL;
    if (err_line != NULL)
	*err_line = 0;
    if (fd < 0)
	return ret;
    fd = dup(fd);
    if (fd < 0) 
	return ret;
    fp = fdopen(fd, "r");
    if(fp == NULL) {
	close(fd);
        return ret;
    }
    if(regcomp(&preg_mac, mac_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if(regcomp(&preg_name, name_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if(regcomp(&preg_remark, remark_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if(regcomp(&preg_passwd, passwd_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }

    if (!get_rc_env("TOTAL_POLICY", buf, LOCALCSV_MAXLEN)) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    total_policy = atoi(buf);
    if (err_type != NULL)
	*err_type = 0;
    while(fgets(buf, LOCALCSV_MAXLEN, fp)) {
        // Should not have any '\n'
        strtok_r(buf, "\n\r", &ptr); 
	if (!strncmp(buf, "#", 1) || !strncmp(buf, ";", 1) || !strncmp(buf, "//", 2)) {
	    line++;
	    continue;
	}
	//if(!*buf) goto skip;
	ptr = buf;
        pch = strsep(&ptr, ",");
        tmp = parse_csv_value(pch);

        if(tmp == NULL) {
            // format error
	    if(err_line) *err_line = line;
	    if(err_type) *err_type = IMPLU_ERR_FORMAT; 
	    goto end;
        }

        tmp1 = lusr;
        lusr = calloc(1, sizeof(LOCAL_USER));

        if(tmp1 != NULL)
            tmp1->next = lusr;
        if(head == NULL)
            head = lusr;
        lusr->next = NULL;

        count = 0;
        while (pch != NULL) {
            switch(count) {
                case 0:
		    if (strcmp(tmp, "") == 0 || strlen(tmp) >= sizeof(lusr->name)
			|| regexec(&preg_name, tmp, (size_t) 0, NULL, 0) != 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_NAME; 
			if(length) *length = sizeof(lusr->name) - 1;
			goto end;
		    }
                    strncpy(lusr->name, tmp, sizeof(lusr->name)-1);
                    break;
                case 1:
		    if (strcmp(tmp, "") == 0 || strlen(tmp) >= sizeof(lusr->password)
			|| regexec(&preg_passwd, tmp, (size_t) 0, NULL, 0) != 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_PASSWD; 
			if(length) *length = sizeof(lusr->password) - 1;
			goto end;
		    }
                    strncpy(lusr->password, tmp, sizeof(lusr->password)-1);
                    break;
                case 2:
		    if (strcmp(tmp, "") && regexec(&preg_mac, tmp, (size_t) 0, NULL, 0) != 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_MAC; 
			goto end;
		    }
                    strncpy(lusr->mac, tmp, sizeof(lusr->mac)-1);
                    break;
                case 3:
                    lusr->gpidx = atoi(tmp);
		    if (lusr->gpidx < 0 || lusr->gpidx > total_policy) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_POLICY; 
			goto end;
		    }
                    break;
                case 4:
		    if (strlen(tmp) >= sizeof(lusr->remark) || (strcmp(tmp, "") && regexec(&preg_remark, tmp, (size_t) 0, NULL, 0) != 0)) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_REMARK; 
			if(length) *length = sizeof(lusr->remark) - 1;
			goto end;
		    }
                    strncpy(lusr->remark, tmp, sizeof(lusr->remark)-1);
                    break;
/*
                case 5:
                    lusr->ipsec = atoi(tmp);
                    break;
                case 6:
                    lusr->allowed_sz = atoi(tmp);
                    break;
*/
            }
            pch = strsep(&ptr, ",");
            if(pch == NULL)
		break;
            tmp = parse_csv_value(pch);
            if(tmp == NULL) { 
	        if(err_line) *err_line = line;
	        if(err_type) *err_type = IMPLU_ERR_FORMAT; 
		goto end;
            }
            count++;
        }


        if( count != 4 ) {
            if(err_line) *err_line = line;
            if(err_type) *err_type = IMPLU_ERR_FORMAT; 
	    goto end;
        }
#ifdef CIPCFG_ITEM_VALIDATION
        if(LocalUsrValidate(lusr) != 0) {
            if(err_line) *err_line = line;
            if(err_type) *err_type = IMPLU_ERR_VALIDATE; 
	    goto end;
        }
#endif
        line++;
    }
    line = line - 1;

    // Check Total Number 
    total_num = getLocalUserRowCount(NULL);
    if(total_num + line > getMaxLocalUser()) {
        if(err_line) *err_line = line;
        if(err_type) *err_type = IMPLU_ERR_EXCEED; 
	goto end;
    }

    ret = addMultipleLocalUsers(head, &mem);
    if(ret < 0) {
	if (mem != 0) {
            if(err_line) *err_line = mem;
            if(err_type) *err_type = IMPLU_ERR_DUPLICATE; 
	} else {
            if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	}
    }

end:
    fclose(fp);
    free_local_usr_list(head);
    return ret;
}

int importOndemandUser(int fd, int *err_type, int *err_line, int *length)
{
    ONDEMAND_USER *lusr = NULL, *head = NULL, *tmp1 = NULL;
    FILE *fp = NULL;
    char buf[ONDEMANDCSV_MAXLEN];
    int count = 0;
    char *pch;
    char *tmp;
    int mem = 0;
    char *ptr;
    int line=1, total_num, ret = -1, max;
    char *tmpptr, item_value[MAX_VALUE_LEN];
    struct tm tm;
    char name_rule[] = "^[a-zA-Z0-9]+$";
    char remark_rule[] = "^[a-zA-Z0-9#@_?:*|. ,-]+$";
    char passwd_rule[] = "^[^][\"'<>\\`~]+$";
    regex_t preg_name, preg_passwd, preg_remark;
    CIPCFG *cipcfg = NULL;

    if(err_type != NULL)
	*err_type = IMPLU_ERR_GENERAL;
    if (err_line != NULL)
	*err_line = 0;
    if (fd < 0)
	return ret;
    fd = dup(fd);
    if (fd < 0) 
	return ret;
    fp = fdopen(fd, "r");
    if(fp == NULL) {
	close(fd);
        return ret;
    }

    if(regcomp(&preg_name, name_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if(regcomp(&preg_remark, remark_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if(regcomp(&preg_passwd, passwd_rule, REG_EXTENDED | REG_NOSUB) != 0) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    if (err_type != NULL)
	*err_type = 0;
    while(fgets(buf, ONDEMANDCSV_MAXLEN, fp)) {
        // Should not have any '\n'
        strtok_r(buf, "\n\r", &ptr); 
	//if(!*buf) goto skip;
	if (!strncmp(buf, "#", 1) || !strncmp(buf, ";", 1) || !strncmp(buf, "//", 2)) {
	    line++;
	    continue;
	}
	ptr = buf;
        pch = strsep(&ptr, ",");
        tmp = parse_csv_value(pch);

        if(tmp == NULL) {
            // format error
	    if(err_line) *err_line = line;
	    if(err_type) *err_type = IMPLU_ERR_FORMAT; 
	    goto end;
        }

        tmp1 = lusr;
        lusr = calloc(1, sizeof(ONDEMAND_USER));

        if(tmp1 != NULL)
            tmp1->next = lusr;
        if(head == NULL)
            head = lusr;
        lusr->next = NULL;

        count = 0;
        while (pch != NULL) {
            switch(count) {
                case 0:
		    if (strcmp(tmp, "") == 0 || strlen(tmp) >= sizeof(lusr->name)
			|| regexec(&preg_name, tmp, (size_t) 0, NULL, 0) != 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_NAME; 
			if(length) *length = sizeof(lusr->name) - 1;
			goto end;
		    }
                    strncpy(lusr->name, tmp, sizeof(lusr->name)-1);
                    break;
                case 1:
		    if (strcmp(tmp, "") == 0 || strlen(tmp) >= sizeof(lusr->passwd)
			|| regexec(&preg_passwd, tmp, (size_t) 0, NULL, 0) != 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_PASSWD; 
			if(length) *length = sizeof(lusr->passwd) - 1;
			goto end;
		    }
                    strncpy(lusr->passwd, tmp, sizeof(lusr->passwd)-1);
                    break;
                case 2:
		    lusr->type = atoi(tmp);
		    if (lusr->type != 0){
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_TYPE; 
			goto end;
		    }
                    break;
                case 3:
                    lusr->status = atoi(tmp);
		    if (lusr->status != 0 && lusr->status != -1){
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_STATUS; 
			goto end;
		    }
                    break;
                case 4:
		    lusr->availableunit = strtoul(tmp, &tmpptr, 10);
                    break;
                case 5:
                    lusr->acttype = atoi(tmp);
		    if (lusr->acttype != 0 && lusr->acttype != 1){
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_ACTTYPE; 
			goto end;
		    }
                    break;
		case 6:
		    lusr->durationdays = atoi(tmp);
		    break;
		case 7:
		    lusr->unit = atoi(tmp);
		    break;
		case 8:
		    lusr->grace = 0;
		    break;
                case 9:
		    if (strptime(tmp, "%Y/%m/%d %H:%M:%S", &tm) == 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_TIMEFORMAT; 
			goto end;
		    }
                    lusr->validatetime = timegm(&tm);
		    if (lusr->validatetime < 0)
			lusr->validatetime = 0;
                    break;
		case 10:
		    if (strptime(tmp, "%Y/%m/%d %H:%M:%S", &tm) == 0) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_TIMEFORMAT; 
			goto end;
		    }
		    lusr->expiretime = timegm(&tm);
		    if (lusr->expiretime < 0)
			lusr->expiretime = 0;
		    break;
		case 11:
		    if (strlen(tmp) >= sizeof(lusr->remark) || (strcmp(tmp, "") && regexec(&preg_remark, tmp, (size_t) 0, NULL, 0) != 0)) {
			if(err_line) *err_line = line;
			if(err_type) *err_type = IMPLU_ERR_REMARK; 
			if(length) *length = sizeof(lusr->remark) - 1;
			goto end;
		    }
                    strncpy(lusr->remark, tmp, sizeof(lusr->remark)-1);
		    break;
            }
            pch = strsep(&ptr, ",");
            if(pch == NULL)
		break;
            tmp = parse_csv_value(pch);
            if(tmp == NULL) { 
	        if(err_line) *err_line = line;
	        if(err_type) *err_type = IMPLU_ERR_FORMAT; 
		goto end;
            }
            count++;
        }


        if( count != 11 ) {
            if(err_line) *err_line = line;
            if(err_type) *err_type = IMPLU_ERR_FORMAT; 
	    goto end;
        }
#ifdef CIPCFG_ITEM_VALIDATION
        if(OndemandUsrValidate(lusr) != 0) {
            if(err_line) *err_line = line;
            if(err_type) *err_type = IMPLU_ERR_VALIDATE; 
	    goto end;
        }
#endif
        line++;
    }
    line = line - 1;

    // Check Total Number 
    total_num = getOnDemandUserRowCount(NULL);
    cipcfg = init_cipcfg();
    if (cipcfg == NULL) {
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    ret = get_cfg_item(cipcfg, GID_MGMT_AUTH6, "max_users", item_value);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	if(err_line) *err_line = line;
	if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	goto end;
    }
    close_cipcfg(cipcfg);
    max = atoi(item_value);
    ret = -1;
    if(total_num + line > max) {
        if(err_line) *err_line = line;
        if(err_type) *err_type = IMPLU_ERR_EXCEED; 
	goto end;
    }

    ret = addMultipleOndemandUsers(head, &mem);
    if(ret < 0) {
	if (mem != 0) {
            if(err_line) *err_line = mem;
            if(err_type) *err_type = IMPLU_ERR_DUPLICATE; 
	} else {
            if(err_type) *err_type = IMPLU_ERR_GENERAL; 
	}
    }

end:
    fclose(fp);
    free_ondemand_usr_list(head);
    return ret;
}

static void applyUsrPolicy(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *app = NULL, *policy = NULL, *up_rate = NULL, *down_rate = NULL, *sess_limit = NULL, *idle_min = NULL;
    ITEM_LIST *list = NULL;
    int curpolicy, ret;
    CIPCFG *cipcfg = NULL;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1)); 
    int applist_idx= 0, idle_sec;
    char tmp[256], idletimeout[MAX_VALUE_LEN];

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    policy = maGetVar(rq, MA_FORM_OBJ, "policy", "0");
    up_rate = maGetVar(rq, MA_FORM_OBJ, "up_rate", "0");
    down_rate = maGetVar(rq, MA_FORM_OBJ, "down_rate", "0");
    sess_limit = maGetVar(rq, MA_FORM_OBJ, "user_ses_limit", "-1");

    curpolicy = atoi(policy);
    if (curpolicy == 0) {
	idle_min = maGetVar(rq, MA_FORM_OBJ, "idletimeout", "10");
	idle_sec = atoi(idle_min) * 60;
	sprintf(idletimeout, "%d", idle_sec);
	list = append_cfg_item(list, GID_MGMT_AUTH, "idletimeout", idletimeout);
    }
    list = append_cfg_item(list, gid_policy_bandwidth[curpolicy], "up_rate", up_rate);
    list = append_cfg_item(list, gid_policy_bandwidth[curpolicy], "down_rate", down_rate);
    list = append_cfg_item(list, gid_policy_sess_limit[curpolicy], "max_sessions", sess_limit);
    
    print_list(list);
    cipcfg = init_cipcfg();
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return;
    }
    while(list != NULL) {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0) {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
            free(applylist);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
            free(applylist);
	    return;
	}
    }
    close_cipcfg(cipcfg);
    sprintf(tmp, "/user/policy.asp?policy=%d", curpolicy);
    maRedirect(rq, 302, tmp);
}

static void kickOutCurrentuser(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *kick_item= NULL;
    char cmd[BUFSIZ];

    // kick_item is the MAC of current login user.
    kick_item = maGetVar(rq, MA_FORM_OBJ, "kick_item", "");

    sprintf(cmd, "/ramfs/bin/nam_kick %s %s", kick_item, "Force logout");
    cip_system(cmd);

    maRedirect(rq, 302, "/status/cur_login.asp");
}

static void applyUserManagement(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *app = NULL, *idle_min = NULL, idletimeout[MAX_VALUE_LEN];
    int idle_sec, ret;
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    idle_min = maGetVar(rq, MA_FORM_OBJ, "idletimeout", "10");
    idle_sec = atoi(idle_min) * 60;
    sprintf(idletimeout, "%d", idle_sec);
    list = append_cfg_item(list, GID_MGMT_AUTH, "idletimeout", idletimeout);

    print_list(list);
    cipcfg = init_cipcfg();
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return;
    }
    close_cipcfg(cipcfg);
    maRedirect(rq, 302, "/user/management.asp");
}


static void applyMapping(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *app = NULL, *cur_server = NULL, *attr, *policy, *remark, *hexattr;
    char buf[MAX_VALUE_LEN], *mapstate = NULL;
    int ret, i, max_entry = 10, radius_no;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1)); 
    int applist_idx= 0;
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    cur_server = maGetVar(rq, MA_FORM_OBJ, "cur_server", "0");
    radius_no = atoi(cur_server);
    mapstate = maGetVar(rq, MA_FORM_OBJ, "state", "Disabled");

    list = append_cfg_item(list, gid_radius_map[radius_no], "state", mapstate);
    if (strcmp(mapstate, "Enabled") == 0) {
        memset(buf, 0, MAX_VALUE_LEN);
	for(i = 0; i < max_entry; i++) {
	    sprintf(buf, "attr%d", i);
	    attr = maGetVar(rq, MA_FORM_OBJ, buf, "");
	    hexattr = hexbinconvert(attr, 1);
	//    printf("%s%s\n", buf, hexattr);
	    sprintf(buf, "entry:%d.attr", i);
	    list = append_cfg_item(list, gid_radius_map[radius_no], buf, (hexattr == NULL) ? "" : hexattr);
	    sprintf(buf, "attr%dpolicy", i);
	    policy = maGetVar(rq, MA_FORM_OBJ, buf, "");
	    sprintf(buf, "entry:%d.policyid", i);
	    list = append_cfg_item(list, gid_radius_map[radius_no], buf, policy);
	    sprintf(buf, "attr%dremark", i);
	    remark = maGetVar(rq, MA_FORM_OBJ, buf, "");
	    sprintf(buf, "entry:%d.remark", i);
	    list = append_cfg_item(list, gid_radius_map[radius_no], buf, remark);
	    free(hexattr);
	}
    }
    print_list(list);
    cipcfg = init_cipcfg();
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    while(list != NULL) {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0) {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;
    // free_item_list(list);
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
            free(applylist);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
            free(applylist);
	    return;
	}
    }
    close_cipcfg(cipcfg);
    sprintf(buf, "/user/classmapping.asp?radius=%d", radius_no);
    maRedirect(rq, 302, buf);
}

static void applyBlackList(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int listid = 0, list_max, ret, i;
    int applylist[] = {gid_blacklist[0], GID_END};
    char *app = NULL, *list_number = NULL, *act = NULL, *name = NULL, *entry = NULL;
    char *remark = NULL;
    char item_value[MAX_VALUE_LEN], buf[MAX_VALUE_LEN];

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    list_number = maGetVar(rq, MA_FORM_OBJ, "list", "0");
    listid = atoi(list_number);
    applylist[0] = gid_blacklist[listid];
    act = maGetVar(rq, MA_FORM_OBJ, "act", "add");
    name = maGetVar(rq, MA_FORM_OBJ, "username", "");
    remark = maGetVar(rq, MA_FORM_OBJ, "remark", "");
    entry = maGetVar(rq, MA_FORM_OBJ, "entry", "0");
    cipcfg = init_cipcfg();
    if (cipcfg == NULL)
	return;
    ret = get_cfg_item(cipcfg, gid_blacklist[listid], "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    list_max = atoi(item_value);
    if (!strcmp(act, "add")) {
	for (i = 0; i < list_max; i++) {
	    sprintf(buf, "entry:%d.name", i);
	    if (get_cfg_item(cipcfg, gid_blacklist[listid], buf, item_value) != CIPCFG_SUCCESS) {
		// free_item_list(list);
		if (i > 0)
		    break;
	    }
	    if (!strcmp(item_value, ""))
		break;
	}
	if (i >= list_max) {
	    // free_item_list(list);
	    sprintf(buf, "/user/blacklist.asp?list=%d&err_code=101", listid);
	    maRedirect(rq, 302, buf);
	    close_cipcfg(cipcfg);
	    return;
	}
	list = append_cfg_item(list, gid_blacklist[listid], buf, name);
	sprintf(buf, "entry:%d.remark", i);
	list = append_cfg_item(list, gid_blacklist[listid], buf, remark);
    }
    else if (!strcmp(act, "edit")) {
	sprintf(buf, "entry:%d.name", atoi(entry));
	list = append_cfg_item(list, gid_blacklist[listid], buf, name);
	sprintf(buf, "entry:%d.remark", atoi(entry));
	list = append_cfg_item(list, gid_blacklist[listid], buf, remark);
    }
    else if (!strcmp(act, "delete")) {
	sprintf(buf, "entry:%d.name", atoi(entry));
	list = append_cfg_item(list, gid_blacklist[listid], buf, "");
	sprintf(buf, "entry:%d.remark", atoi(entry));
	list = append_cfg_item(list, gid_blacklist[listid], buf, "");
    }
    else {
	close_cipcfg(cipcfg);
	return;
    }
    print_list(list);
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    // free_item_list(list);
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
	    return;
	}
    }
    close_cipcfg(cipcfg);
    sprintf(buf, "/user/blacklist.asp?list=%d", listid);
    maRedirect(rq, 302, buf);
}

static void applyPrivilegeIP(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char *app, *page_str, *entry_perpage_str, *ip, *remark;
    int applylist[] = {GID_PRIVILEGEIP, GID_END};
    char item_value[MAX_VALUE_LEN], item_name[MAX_VALUE_LEN];
    int max, i, page, entry_perpage, ret, j;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    page_str = maGetVar(rq, MA_FORM_OBJ, "page", "0");
    entry_perpage_str = maGetVar(rq, MA_FORM_OBJ, "entry_perpage", "0");
    page = atoi(page_str);
    entry_perpage = atoi(entry_perpage_str);
    if (page * entry_perpage <= 0)
	return;
    cipcfg = init_cipcfg();
    if (cipcfg == NULL)
	return;
    ret = get_cfg_item(cipcfg, GID_PRIVILEGEIP, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    max = atoi(item_value);
    if (page * entry_perpage > max) {
	close_cipcfg(cipcfg);
	return;
    }
    for (i = (page - 1) * entry_perpage, j = 0; i < page * entry_perpage; i++, j++) {
	sprintf(item_name, "ip%d", j);
	ip = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "remark%d", j);
	remark = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "entry:%d.ip", i);
	list = append_cfg_item(list, GID_PRIVILEGEIP, item_name, ip);
	sprintf(item_name, "entry:%d.remark", i);
	list = append_cfg_item(list, GID_PRIVILEGEIP, item_name, remark);
    }
    print_list(list);
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    // free_item_list(list);
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
	    return;
	}
    }
    close_cipcfg(cipcfg);
    sprintf(item_name, "/user/privilege_ip.asp?page=%d", page);
    maRedirect(rq, 302, item_name);
}

static void applyPrivilegeMAC(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char *app, *page_str, *entry_perpage_str, *mac, *remark;
    int applylist[] = {GID_PRIVILEGEMAC, GID_END};
    char item_value[MAX_VALUE_LEN], item_name[MAX_VALUE_LEN];
    int max, i, page, entry_perpage, ret, j;

    app = maGetVar(rq, MA_FORM_OBJ, "apply", "true");
    page_str = maGetVar(rq, MA_FORM_OBJ, "page", "0");
    entry_perpage_str = maGetVar(rq, MA_FORM_OBJ, "entry_perpage", "0");
    page = atoi(page_str);
    entry_perpage = atoi(entry_perpage_str);
    if (page * entry_perpage <= 0)
	return;
    cipcfg = init_cipcfg();
    if (cipcfg == NULL)
	return;
    ret = get_cfg_item(cipcfg, GID_PRIVILEGEMAC, "max", item_value);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    max = atoi(item_value);
    if (page * entry_perpage > max) {
	close_cipcfg(cipcfg);
	return;
    }
    for (i = (page - 1) * entry_perpage, j = 0; i < page * entry_perpage; i++, j++) {
	sprintf(item_name, "mac%d", j);
	mac = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "remark%d", j);
	remark = maGetVar(rq, MA_FORM_OBJ, item_name, "");
	sprintf(item_name, "entry:%d.mac", i);
	list = append_cfg_item(list, GID_PRIVILEGEMAC, item_name, mac);
	sprintf(item_name, "entry:%d.remark", i);
	list = append_cfg_item(list, GID_PRIVILEGEMAC, item_name, remark);
    }
    print_list(list);
    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	return;
    }
    // free_item_list(list);
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
	    apply_cfg(cipcfg);
	    unlink(PATH_SYNC_CFG);
	}else{
	    close_cipcfg(cipcfg);
	    maRedirect( rq, 302, "/utilities/reboot.asp" );
	    return;
	}
    }
    close_cipcfg(cipcfg);
    sprintf(item_name, "/user/privilege_mac.asp?page=%d", page);
    maRedirect(rq, 302, item_name);
}

static void DeleteOnDemandAccount(MaRequest *rq, char *script, char *uri, char *query, char *postData, int postLen)
{
    char *delname = NULL, *action = NULL;

    action = maGetVar(rq, MA_FORM_OBJ, "action", "delete");
    delname = maGetVar(rq, MA_FORM_OBJ, "delName", "");

    if (strcmp(delname, "") != 0 && strcmp(action, "delete") == 0)
	deleteOnDemandUserbyName(delname);
    else if (strcmp(action, "deleteall") == 0)
	deleteOnDemandUserbyName(NULL);
    maRedirect(rq, 302, "/user/OnDemandUserList.asp");
}

static int isonline(EspRequest *ep, int argc, char **argv)
{
    char ipAddr[12], macAddr[24], *session;
    int ret = 0;

    if (ejsParseArgs(argc, argv, "%s", &session) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    checkSession(session, ipAddr, macAddr);
    ret = check_online(macAddr);
    espSetReturn(ep, mprCreateIntegerVar(ret));
    return 0;
}

static int getReturnUrl(EspRequest *ep, int argc, char **argv)
{
    char HpageURL[128], *returnurl;
    int ret = 0;

    if (ejsParseArgs(argc, argv, "%s", &returnurl) < 1) {
	espError(ep, "Insufficient args\n");
	return -1;
    }
    ret = dec(returnurl, HpageURL);
    if (ret == -1)
	HpageURL[0] = '\0';
    
    espSetReturnString(ep, HpageURL);
    return 0;
}

/*
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initUsrForm(void)
{
    /* Set Functions */
    websFormDefine(T("applyUserRadius"), applyUserRadius);
    websFormDefine(T("applyUsr8021x"), applyUsr8021x);
    websFormDefine(T("applyUsrPolicyFirewall"), applyUsrPolicyFirewall);
    websFormDefine(T("applyUsrPolicyRoute"), applyUsrPolicyRoute);
    websFormDefine(T("applyLocalUser"), applyLocalUser);
    websFormDefine(T("applyOnDemandGeneral"), applyOnDemandGeneral);
    websFormDefine(T("applyTicketTerminal"), applyTicketTerminal);
    websFormDefine(T("applyTickConf"), applyTickConf);
    websFormDefine(T("applyBillingConf"), applyBillingConf);
    websFormDefine(T("enableBillingPlan"), enableBillingPlan);
    websFormDefine(T("ConfirmOnDemandAccount"), ConfirmOnDemandAccount);
    websFormDefine(T("CreateOnDemandAccount"), CreateOnDemandAccount);
    maDefineEgiForm("/exportLocalUser.egi", exportLocalUser);
    maDefineEgiForm("/exportOndemandUser.egi", exportOndemandUser);
    maDefineEgiForm("/applyUsrPolicy.egi", applyUsrPolicy);
    maDefineEgiForm("/kickOutCurrentUser.egi", kickOutCurrentuser);
    maDefineEgiForm("/applyUserManagement.egi", applyUserManagement);
    maDefineEgiForm("/DeleteOnDemandAccount.egi", DeleteOnDemandAccount);
    maDefineEgiForm("/applyMapping.egi", applyMapping);
    maDefineEgiForm("/applyBlackList.egi", applyBlackList);
    maDefineEgiForm("/applyPrivilegeIP.egi", applyPrivilegeIP);
    maDefineEgiForm("/applyPrivilegeMAC.egi", applyPrivilegeMAC);

    /* Get Functions */
    espDefineStringCFunction(0, "isonline", isonline, 0);
    espDefineStringCFunction(0, "getReturnUrl", getReturnUrl, 0);
    websAspDefine(T("getLocalUserCount"), getLocalUserCount);
    websAspDefine(T("getLocalJson"), getLocalJson);
    websAspDefine(T("getLocalSearchJson"), getLocalSearchJson);
    websAspDefine(T("getCurrentUserJson"), getCurrentUserJson);
    websAspDefine(T("getOnDemandUserCount"), getOnDemandUserCount);
    websAspDefine(T("getOnDemandJson"), getOnDemandJson);
    websAspDefine(T("getOnDemandSearchJson"), getOnDemandSearchJson);
}
