/**
 *  \file web_wlan.c
 *
 *  This is the cgi modules for A300 to action System related function
 *
 *  Depends on:
 *
 *  Copyright (c) 2004-2006 by Cipherium System Co., Ltd. All rights reserved.
 *
 *  No part may be reproduced except as authorized by written permission of the
 *  contributing companies.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 *  CONTRIBUTORS "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 *  OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*   Revision      History
 *   Date          Version     Author         Remarks
 *   -------------------------------------------------------------------------
 *   11/20/2006     0.1       Jimmy Yang     Initial version
 *   01/30/2007     0.2       Leo Chen       for A800
 */

#include "id.h"
#include "wlan.h"
#include "cipcfg.h"
#include "syscall.h"
#include "web_wlan.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef LAYER2_FIREWALL
#include "service.h"
#endif
extern const int gid_rf[];
extern const char *if_vap_2d[][MAX_VAP];
extern const int gid_vap[][MAX_VAP];
extern const int gid_vap_sec[][MAX_VAP];
extern const int gid_vap_acl[][MAX_VAP];
extern const int gid_wds_link[][MAX_WDS_LINK];
static time_t last_ap_scan_time = 0;
static AP_LIST *ap_scan_list = NULL;
extern const char *current_config;
#define SCAN_INTERVAL 15

/**
 * Generate the Country code list for WLAN web page
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int genWlanCC(int eid, webs_t wp, int argc, char_t **argv)
{    
    FILE *fp = NULL;
    char buf[128];
    char str[256];
    int i =1;

    if ((fp = fopen( COUNTRY_CODE, "r")) == NULL) {
        printf("Can't open the country code\n");
        return -1;
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        if (!strncmp(buf, "NA=0", 4)) break;
    }

    /* Create Default Country Code */
    sprintf(str, "{\"cc\":[\"NONE\",\"NA\",");

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        websWrite(wp, T(str));
        char *p;
        buf[strlen(buf)-1] = '\0'; // remoe the new line char
        p = strstr(buf, "=$CTRY_");
        *p = '\0';
        sprintf(str, "\"%s\",\"%s\",",
            (p+7), buf);
        i++;
    }
    str[strlen(str)-1] = '\0'; //remove the last ","
    websWrite(wp, T(str));
    fclose(fp);
    sprintf(str, "]}");
    websWrite(wp, T(str));
    return 0;
}

/**
 * Generate the MAC ACL tables
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
int genWlanAcl(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[256], item_value[256], value[256], *group_name;
    char mac_buf[256], enable_buf[256];
    int i = 1, tbl_size, num, isFirst = 1;

    if (ejArgs(argc, argv, T("%s"), &group_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "table_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp,T("["));
    for (i = 0; i < tbl_size; i++) {
        sprintf(str, "acl_entry:%d.mac", i);
        get_cfg_item_by_str(cipcfg, group_name, str, mac_buf);
        sprintf(str, "acl_entry:%d.enabled", i);
        get_cfg_item_by_str(cipcfg, group_name, str, enable_buf);
	if (isFirst == 0)
	    websWrite(wp, T(","));
	num = sprintf(value, "{\"mac\":\"%s\", \"state\":\"%s\"}", 
		mac_buf, enable_buf);
	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else 
	    continue;
    }
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}

/**
 * Enable/Disable WLAN Card and set country code
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWdsInterface(webs_t wp, char_t *path, char_t *query)
{
    char_t *secret, *var_value, *app, *tmpvar, *diversity, *rf;
    char var_name[64], attr_name[64], tmp[256],
         *attr_names[] = {     "state",       "mode",         "rate",
                             "channel",      "txpwr",     "distance", 
                            "slottime", "acktimeout",   "ctstimeout"},
         *def_values[]={"enable", "auto", "auto", "auto", "auto", "0", "0", "0", "0"};
    int i, j, gid, ret, rf_id;
    ITEM_LIST *list = NULL;
    CIPCFG  *cipcfg = NULL;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    memset(tmp, 0, 256);
    memset(var_name, 0, 64);
    memset(attr_name, 0, 64);
    app = websGetVar(wp, T("apply"), T("true"));
    secret = websGetVar(wp, T("secret"), T("discovery"));
    for (i=0; i<max_rf; i++){
        list=append_cfg_item(list, gid_rf[i], "wds_discover_secret", secret);
    }
    rf = websGetVar(wp, T("rf"), T("1"));
    rf_id = atoi(rf);
    if (rf_id < 1 || rf_id > max_rf)
        rf_id = 1;
    gid = gid_rf[atoi(rf)-1];
    for(j=0; j< MAX_ATTRS; j++) {
        if(!strcmp(attr_names[j], "slottime"))
        {
                tmpvar=websGetVar(wp, T("adv"), T("off"));
                var_value = websGetVar(wp, T(attr_names[j]), T(def_values[j])) ;
                if(!strncmp(tmpvar, "on", 2))
                     var_value=strcat(var_value,":d");
        }
        else
        	var_value = websGetVar(wp, T(attr_names[j]), T(def_values[j])) ;
	if (!strcmp(attr_names[j], "channel") || !strcmp(attr_names[j], "rate"))
		var_value[0] = tolower(var_value[0]); // Auto -> auto
        list=append_cfg_item(list, gid, attr_names[j], var_value);
    }
#ifdef RF_DIVERSITY
    chkFeature("RF_DIVERSITY", tmp);
    if(tmp[0]=='Y'){
	diversity = websGetVar(wp, T("diversity"), T("disable"));
	list=append_cfg_item(list, gid, "diversity", diversity);
    }
#endif

    print_list(list);

    cipcfg=init_cipcfg();

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("Set cipcfg fault..%d\n",ret);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);
/*
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/wds/wds_rf.asp"))); 
*/
    sprintf(tmp, "/wds/wds_rf.asp?rf=%d", rf_id);
    websRedirect(wp, tmp); 
}

/**
 * Enable/Disable WLAN Card and set country code
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlan(webs_t wp, char_t *path, char_t *query)
{
    char_t *app, *state, *cc, *mode, *superag, *shpreamble, *rate, *ch, *txpwr;
    char_t *ssid, *acktimeout, *channel_width, *diversity, *shortgi, *chwidth_ext;
    char_t *burst, *compression, *fast_frames, *dynamic_turbo, *preferbssid;
    char_t *bintval, *puren; 
    char tmp[256], opmode[MAX_VALUE_LEN];
    ITEM_LIST *list= NULL;
    CIPCFG *cipcfg = NULL;
    int ret;
    struct stat st;
    int rf_no = 0;

    memset(tmp, 0, 256);
    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "opmode", "type", opmode);

    app = websGetVar(wp, T("apply"), T("true"));
    state = websGetVar(wp, T("state"), T("enable"));
    mode = websGetVar(wp, T("mode"), T("auto"));
    shpreamble = websGetVar(wp, T("shpreamble"), T("enable"));
    rate = websGetVar(wp, T("rate"), T("auto"));
    txpwr = websGetVar(wp, T("txpwr"), T("auto"));
#ifdef MULTIPLE_RF
    rf_no = atoi(websGetVar(wp, T("rf"), T("1"))) - 1;
#endif

    if (getUserGroupPermission(wp, gid_rf[rf_no]) & PERM_WRITE) {
	list=append_cfg_item(list, gid_rf[rf_no], "state", state);
	if (!strcmp(state, "enable")) {
            list=append_cfg_item(list, gid_rf[rf_no], "mode", mode);
	}
	list=append_cfg_item(list, gid_rf[rf_no], "shpreamble", shpreamble);
        rate[0]=tolower(rate[0]); // Auto -> auto
	list=append_cfg_item(list, gid_rf[rf_no], "rate", rate);
	list=append_cfg_item(list, gid_rf[rf_no], "txpwr", txpwr);
#ifdef RF_TURBO
        chkFeature("RF_TURBO", tmp);
        if(tmp[0]=='Y'){
            if (!strcmp(rate, "auto")) {
                list=append_cfg_item(list, gid_rf[rf_no], "turbo", "auto");
            } else {
                list=append_cfg_item(list, gid_rf[rf_no], "turbo", "off");
            }
        }
#endif
#ifdef RF_CH_SELECT
        chkFeature("RF_CH_SELECT", tmp);
        if(tmp[0]=='Y'){
            ch = websGetVar(wp, T("channel"), T("auto"));
            ch[0]=tolower(ch[0]); // Auto -> auto
	    list=append_cfg_item(list, gid_rf[rf_no], "channel", ch);
        }
#endif
#ifdef RF_SUPER_AG
        chkFeature("RF_SUPER_AG", tmp);
        if(tmp[0]=='Y'){
            superag = websGetVar(wp, T("superag"), T(""));
	    list=append_cfg_item(list, gid_rf[rf_no], "superag", superag);
        }
#endif
#ifdef RF_SUPER_G
	burst = websGetVar(wp, T("burst"), T(""));
	list=append_cfg_item(list, gid_rf[rf_no], "burst", burst);
	compression = websGetVar(wp, T("compression"), T(""));
	// FIXME compression always disable or it fail on Qos packet
	list=append_cfg_item(list, gid_rf[rf_no], "compression", "");
	fast_frames = websGetVar(wp, T("fast_frames"), T(""));
	list=append_cfg_item(list, gid_rf[rf_no], "fast_frames", fast_frames);
	dynamic_turbo = websGetVar(wp, T("dynamic_turbo"), T(""));
	list=append_cfg_item(list, gid_rf[rf_no], "dynamic_turbo", dynamic_turbo);
#endif
#ifdef RF_ACKTIMEOUT
        chkFeature("RF_ACKTIMEOUT", tmp);
        if(tmp[0]=='Y'){
	    acktimeout = websGetVar(wp, T("acktimeout"), T("0"));
	    list=append_cfg_item(list, gid_rf[rf_no], "acktimeout", acktimeout);
        }
#endif
#ifdef RF_DIVERSITY
        chkFeature("RF_DIVERSITY", tmp);
        if(tmp[0]=='Y'){
	    diversity = websGetVar(wp, T("diversity"), T("disable"));
	    list=append_cfg_item(list, gid_rf[rf_no], "diversity", diversity);
        }
#endif
#ifdef RF_11N_CHANNEL_WIDTH 
        chkFeature("RF_11N_CHANNEL_WIDTH", tmp);
        if(tmp[0]=='Y'){
            channel_width = websGetVar(wp, T("channelwidth"), T("40"));
            list=append_cfg_item(list, gid_rf[rf_no], "ch_width", channel_width);

	    chwidth_ext = websGetVar(wp, T("chwidth_ext"), T("u"));
	    list = append_cfg_item(list, gid_rf[rf_no], "chwidth_ext", chwidth_ext);
        }
#endif
#ifdef RF_SHORT_GI
        chkFeature("RF_SHORT_GI", tmp);
        if(tmp[0]=='Y'){
            shortgi = websGetVar(wp, T("shortgi"), T(""));
            list=append_cfg_item(list, gid_rf[rf_no], "shortgi", shortgi);
        }
#endif
#ifdef RF_PUREN
	chkFeature("RF_PUREN", tmp);
	if(tmp[0]=='Y'){
	    puren = websGetVar(wp, T("puren_btn"), T("disable"));
	    list = append_cfg_item(list, gid_rf[rf_no], "puren", puren);
	}
#endif
#ifdef RF_BEACON_INTERVAL
        chkFeature("RF_BEACON_INTERVAL", tmp);
        if(tmp[0]=='Y'){
	    int i;
            bintval = websGetVar(wp, T("beacon"), T("100"));
	    for (i = 0; i < MAX_VAP; i++)
		list=append_cfg_item(list, gid_vap[rf_no][i], "beacon", bintval);
        }
#endif

    }

    // if MAX_VAP is not 1, SSID will be set in 
    // applyWlanVap() & ap/vap_config.asp
    if ((strcmp(opmode, "AP") == 0)){
	if ((MAX_VAP == 1) && (getUserGroupPermission(wp, GID_RF1_VAP1) & PERM_WRITE)){		
            ssid = websGetVar(wp, T("ssid"), T(""));
	    list=append_cfg_item(list, GID_RF1_VAP1, "ssid", ssid);
	}
    }else if((strcmp(opmode, "CPE") == 0) && (getUserGroupPermission(wp, GID_RF1_STA1) & PERM_WRITE)) {
	ssid = websGetVar(wp, T("ssid"), T(""));
	list=append_cfg_item(list, GID_RF1_STA1, "ssid", ssid);
    }
#ifdef STA_PREFERRED_BSSID 
    chkFeature("STA_PREFERRED_BSSID", tmp);
    if(tmp[0]=='Y'){
	preferbssid = websGetVar(wp, T("bssid2join"), T(""));
	list = append_cfg_item(list, GID_RF1_STA1, "bssid2join", preferbssid);
    }
#endif

    print_list(list);

    ret=set_cfg_items(cipcfg, list);

    printf("%d\n", ret);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    // No matter users press "Save" or "Apply", all the configurations will apply after reboot. 
    if ( !strncmp("true", app, 4 ) ){
        close_cipcfg(cipcfg);
        websRedirect( wp, "/utilities/reboot.asp" );
        return;
    }

    close_cipcfg(cipcfg);

#ifdef MULTIPLE_RF
    sprintf(tmp, "/ap/ap_rf.asp?rf=%d", rf_no+1);
#else
    sprintf(tmp, "/ap/ap_rf.asp");
#endif
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T(tmp))); 
}

/**
 * Set Virual AP
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanVap(webs_t wp, char_t *path, char_t *query)
{
    char_t *rf_vap, *state, *name, *ssid, *vid;
    char_t *app;
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int ret, gid, rf_no, vap_no;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    app = websGetVar(wp, T("apply"), T("true"));
    rf_vap = websGetVar(wp, T("vap"), T("10"));
    sscanf(rf_vap, "%1d%d", &rf_no, &vap_no);
    state = websGetVar(wp, T("state"), T("disable"));
    name = websGetVar(wp, T("name"), T("A300"));
    ssid = websGetVar(wp, T("ssid"), T("a300"));
    vid = websGetVar(wp, T("vid"), T("4096"));

    printf("Input VID = %s\n", vid);

    /*
    switch( atoi(vap) ){
        case 0: gid = GID_RF1_VAP1 ; break ;
        case 1: gid = GID_RF1_VAP2 ; break ;
        case 2: gid = GID_RF1_VAP3 ; break ;
        case 3: gid = GID_RF1_VAP4 ;
    }
    */

//    gid = gid_vap[0][atoi(vap)];
    gid = gid_vap[rf_no-1][vap_no];

    list=append_cfg_item(NULL, gid, "state", state);
    if (strcmp(state, "enable") == 0) {
	list=append_cfg_item(list, gid, "name", name);
	list=append_cfg_item(list, gid, "ssid", ssid);
	list=append_cfg_item(list, gid, "vid", vid);
    }

    print_list(list);

    cipcfg=init_cipcfg();

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }
#ifdef MULTIPLE_RF
    sprintf(tmp, "/ap/vap_config.asp?rf=%d&vap=%d", rf_no, vap_no);
#else
    sprintf(tmp, "/ap/vap_config.asp?vap=%d", vap_no);
#endif
    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T(tmp))); 
}

/**
 * Set Virual AP (Advanced)
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanVapAdv(webs_t wp, char_t *path, char_t *query)
{
    char_t *rf_vap, *wmm, *rts, *iapp, *protection, *dtim;
    char_t *frag, *hidessid, *sta_isolate, *mcast_rate, *igmp_snooping;
    char_t *app;
    ITEM_LIST *list=NULL;
    CIPCFG *cipcfg = NULL;
    int ret, gid;
    char tmp[256], opmode[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, rf_no, vap_no;

    app = websGetVar(wp, T("apply"), T("true"));
    rf_vap = websGetVar(wp, T("vap"), T("10"));
    sscanf(rf_vap, "%1d%d", &rf_no, &vap_no);

    rts = websGetVar(wp, T("rts_threshold"), T("2346"));
    frag = websGetVar(wp, T("frag_threshold"), T("2346"));
    gid = gid_vap[rf_no-1][vap_no];

    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "opmode", "type", opmode);

    if (getUserGroupPermission(wp, gid) & PERM_WRITE) {
	if (strcmp(opmode, "CPE") == 0) {
	    if (getUserGroupPermission(wp, GID_RF1_STA1) & PERM_WRITE) {
		list=append_cfg_item(list, GID_RF1_STA1, "rts_threshold", rts);
		list=append_cfg_item(list, GID_RF1_STA1, "frag_threshold", frag);
	    }
	} else { // AP Mode & GW Mode
	    list=append_cfg_item(list, gid, "rts_threshold", rts);
	    list=append_cfg_item(list, gid, "frag_threshold", frag);
	}
#ifdef BROADCAST_SSID
        chkFeature("BROADCAST_SSID", tmp);
        if(tmp[0]=='Y'){
            hidessid = websGetVar(wp, T("hidessid"), T("disable"));
            list=append_cfg_item(list, gid, "hidessid", hidessid);
        }
#endif
#ifdef RF_STA_ISOLATION
        chkFeature("RF_STA_ISOLATION", tmp);
        if(tmp[0]=='Y'){
            sta_isolate = websGetVar(wp, T("sta_isolate"), T("disable"));
            list=append_cfg_item(list, gid, "sta_isolate", sta_isolate);
        }
#endif
#ifdef RF_QOS_WMM
        chkFeature("RF_QOS_WMM", tmp);
        if(tmp[0]=='Y'){
            wmm = websGetVar(wp, T("wmm"), T("disable"));
            list=append_cfg_item(list, gid, "wmm", wmm);
        }
#endif
#ifdef RF_IAPP
        chkFeature("RF_IAPP", tmp);
        if(tmp[0]=='Y'){
            iapp = websGetVar(wp, T("iapp"), T("disable"));
            list=append_cfg_item(list, gid, "iapp", iapp);
        }
#endif
#ifdef RF_11G_PROTECT
        chkFeature("RF_11G_PROTECT", tmp);
        if(tmp[0]=='Y'){
            protection = websGetVar(wp, T("protection"), T("disable"));
            list=append_cfg_item(list, gid, "protection", protection);
        }
#endif

#ifdef RF_DTIM
        chkFeature("RF_DTIM", tmp);
        if(tmp[0]=='Y'){
            dtim = websGetVar(wp, T("dtim"), T("1"));
            list=append_cfg_item(list, gid, "dtim_period", dtim);
        }
#endif
#ifdef RF_MCAST_RATE
        chkFeature("RF_MCAST_RATE", tmp);
        if(tmp[0]=='Y'){
            mcast_rate = websGetVar(wp, T("mcast_rate"), T("1000"));
            list=append_cfg_item(list, gid, "mcast_rate", mcast_rate);
        }
#endif
#ifdef RF_IGMP_SNOOPING
        chkFeature("RF_IGMP_SNOOPING", tmp);
        if(tmp[0]=='Y'){
            igmp_snooping = websGetVar(wp, T("igmp_snooping"), T("disable"));
            list=append_cfg_item(list, gid, "igmp_snooping", igmp_snooping);
        }
#endif
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }
#ifdef MULTIPLE_RF
    sprintf(tmp, "/ap/ap_advanced.asp?rf=%d&vap=%d", rf_no, vap_no);
#else
    sprintf(tmp, "/ap/ap_advanced.asp?vap=%d", vap_no);
#endif
    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T(tmp))); 
}

/**
 * Virtual AP's security
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanVapSec(webs_t wp, char_t *path, char_t *query)
{
    char_t *rf_vap, *sec_type, *auth, *cipher;
    char_t *method, *keylen, *txidx, *key0, *key1, *key2, *key3;
    char_t *psktype, *grprekey, *passphrase;
    char_t *dynwep, *dynkeylen, *dynrekey;
    char_t *acctenabled0, *acctenabled1, *ip0, *port0, *acctport0, *interimIntr0, *secret0;
    char_t *ip1, *port1, *acctport1, *interimIntr1, *secret1;
    char_t *mode[7] = {"open", "shared", "8021x", "wpa", "auto", "mac", "wapi"};
    char_t *enabled[2] = {"enable", "disable"};
    char_t *eap[2] = {"psk", "eap"};
    char_t *version[3] = {"1", "2", "1:2"};
    char_t *enc[3] = {"rc4-tkip", "aes-ccmp", "mixed"};
#ifdef WAPI
    char_t *wapi_method, *wapi_key, *as_ip, *as_port;
#endif
    char_t *app;
    // int vap_sec_gids[MAX_VAP] = {GID_RF1_VAP1_SEC, GID_RF1_VAP2_SEC, GID_RF1_VAP3_SEC, GID_RF1_VAP4_SEC};
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    int ret, gid, i, rf_no, vap_no;
    char tmp[256], opmode[MAX_VALUE_LEN], roles[MAX_VALUE_LEN];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    app = websGetVar(wp, T("apply"), T("true"));
    rf_vap = websGetVar(wp, T("vap"), T("10"));
    sscanf(rf_vap, "%1d%d", &rf_no, &vap_no);

    cipcfg = init_cipcfg();
    get_cfg_item_by_str(cipcfg, "opmode", "type", opmode);
    sprintf(tmp, "rf%d", rf_no);
    get_cfg_item_by_str(cipcfg, tmp, "roles", roles);

    if (!strcmp(opmode, "AP") || !strcmp(opmode, "GW")) {
	gid = gid_vap_sec[rf_no-1][vap_no];
    } else if (!strcmp(opmode, "CPE")) {
	gid = GID_RF1_STA1_SEC;
    }

    if (!(getUserGroupPermission(wp, gid) & PERM_WRITE))
	goto do_nothing;
    /*
    switch( atoi(vap) ){
        case 0: gid = GID_RF1_VAP1_SEC ; break ;
        case 1: gid = GID_RF1_VAP2_SEC ; break ;
        case 2: gid = GID_RF1_VAP3_SEC ; break ;
        case 3: gid = GID_RF1_VAP4_SEC ;
    }

    printf("Gid = %d\n", gid);
    */
    /* input mode will be none, wep, 8021x, wpa:psk, wpa:eap 
     * they are mapping to configuration item: mode, wep.enabled and wpa.eap
     * none : mode=open, wep.enabled=disable, wep.eap=psk
     * wep : mode=(auth), wep.enabled=enable, wpa.eap=psk
     * 8021x : mode=8021x, wep.enabled=disable, wpa.eap=psk
     * wpa:psk : mode=wpa, wep.enabled=disable, wpa.eap=psk
     * wpa:eap : mode=wpa, wep.enabled=disable, wpa.eap=eap
    */
    sec_type = websGetVar(wp, T("secType"), T("none"));
    auth = websGetVar(wp, T("auth"), T("open"));
    if (!strcmp(sec_type, "none")) {
        list=append_cfg_item(NULL, gid, "mode", mode[0]);
        list=append_cfg_item(list, gid, "wep.enabled", enabled[1]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[0]);
    } else if (!strcmp(sec_type, "wep")) {
        if (!strcmp(auth, "open")) {
            list=append_cfg_item(NULL, gid, "mode", mode[0]);
        } else if (!strcmp(auth, "shared")) {
            list=append_cfg_item(NULL, gid, "mode", mode[1]);
        } else if (!strcmp(auth, "auto")) {
            list=append_cfg_item(NULL, gid, "mode", mode[4]);
        }
        list=append_cfg_item(list, gid, "wep.enabled", enabled[0]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[0]);
    } else if (!strcmp(sec_type, "8021x")) {
        list=append_cfg_item(NULL, gid, "mode", mode[2]);
        list=append_cfg_item(list, gid, "wep.enabled", enabled[1]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[0]);
    } else if (!strcmp(sec_type, "wpa:psk")) {
        list=append_cfg_item(NULL, gid, "mode", mode[3]);
        list=append_cfg_item(list, gid, "wep.enabled", enabled[1]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[0]);
    } else if (!strcmp(sec_type, "wpa:eap")) {
        list=append_cfg_item(NULL, gid, "mode", mode[3]);
        list=append_cfg_item(list, gid, "wep.enabled", enabled[1]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[1]);
    } else if (!strcmp(sec_type, "mac")) {
        list=append_cfg_item(NULL, gid, "mode", mode[5]);
        list=append_cfg_item(list, gid, "wep.enabled", enabled[1]);
        list=append_cfg_item(list, gid, "wpa.eap", eap[0]);
    }
#ifdef WAPI
    else if (!strcmp(sec_type, "wapi:psk")) {
        list=append_cfg_item(NULL, gid, "mode", mode[6]);
        list=append_cfg_item(list, gid, "wapi.wapi_type", "psk");
    }
    else if (!strcmp(sec_type, "wapi:cert")) {
        list=append_cfg_item(NULL, gid, "mode", mode[6]);
        list=append_cfg_item(list, gid, "wapi.wapi_type", "cert");
    }
#endif

    method = websGetVar(wp, T("method"), T("ascii"));
    keylen = websGetVar(wp, T("keylen"), T("64"));
    txidx = websGetVar(wp, T("txidx"), T("1"));
    key0 = websGetVar(wp, T("key0"), T(""));
    key1 = websGetVar(wp, T("key1"), T(""));
    key2 = websGetVar(wp, T("key2"), T(""));
    key3 = websGetVar(wp, T("key3"), T(""));
    
    cipher = websGetVar(wp, T("cipher"), T("1:tkip"));
    if (!strcmp(cipher, "1:tkip")) {
        list=append_cfg_item(list, gid, "wpa.version", version[0]);
        list=append_cfg_item(list, gid, "wpa.enc", enc[0]);
    } else if (!strcmp(cipher, "1:aes")) {
        list=append_cfg_item(list, gid, "wpa.version", version[0]);
        list=append_cfg_item(list, gid, "wpa.enc", enc[1]);
    } else if (!strcmp(cipher, "2:tkip")) {
        list=append_cfg_item(list, gid, "wpa.version", version[1]);
        list=append_cfg_item(list, gid, "wpa.enc", enc[0]);
    } else if (!strcmp(cipher, "2:aes")) {
        list=append_cfg_item(list, gid, "wpa.version", version[1]);
        list=append_cfg_item(list, gid, "wpa.enc", enc[1]);
    } else if (!strcmp(cipher, "1:2:tkip:aes")) {
        list=append_cfg_item(list, gid, "wpa.version", version[2]);
        list=append_cfg_item(list, gid, "wpa.enc", enc[2]);
    }
    
    grprekey = websGetVar(wp, T("grprekey"), T("600"));
    psktype = websGetVar(wp, T("psktype"), T("passphrase"));
    passphrase = websGetVar(wp, T("passphrase"), T(""));

    dynwep = websGetVar(wp, T("dynwep"), T("enable"));
    dynkeylen = websGetVar(wp, T("dynkeylen"), T("64"));
    dynrekey = websGetVar(wp, T("dynrekey"), T("300"));

    acctenabled0 = websGetVar(wp, T("acctenabled0"), T("disable"));
    acctenabled1 = websGetVar(wp, T("acctenabled1"), T("disable"));
    ip0 = websGetVar(wp, T("ip0"), T("0.0.0.0"));
    port0 = websGetVar(wp, T("port0"), T("1812"));
    acctport0 = websGetVar(wp, T("acctport0"), T("1813"));
    interimIntr0 = websGetVar(wp, T("interimIntr0"), T("60"));
    secret0 = websGetVar(wp, T("secret0"), T(""));
    ip1 = websGetVar(wp, T("ip1"), T(""));
    port1 = websGetVar(wp, T("port1"), T(""));
    acctport1 = websGetVar(wp, T("acctport1"), T(""));
    interimIntr1 = websGetVar(wp, T("interimIntr1"), T("60"));
    secret1 = websGetVar(wp, T("secret1"), T(""));

    list=append_cfg_item(list, gid, "wep.txidx", txidx);
    /* Madwifi 0.9.0, all VAP share same 4 group keys.
     * We foce set same key value into all VAPs
     */
    if (!strcmp(sec_type, "wep")) {
	for (i = 0; i < MAX_VAP; i++) {
	    int wep_gid = gid_vap_sec[rf_no-1][i];
	    if (!strcmp(opmode, "CPE")) {
		wep_gid = GID_RF1_STA1_SEC;
	    }
	    list=append_cfg_item(list, wep_gid, "wep.method", method);
	    list=append_cfg_item(list, wep_gid, "wep.keylen", keylen);
	    list=append_cfg_item(list, wep_gid, "wep.key0", key0);
	    list=append_cfg_item(list, wep_gid, "wep.key1", key1);
	    list=append_cfg_item(list, wep_gid, "wep.key2", key2);
	    list=append_cfg_item(list, wep_gid, "wep.key3", key3);
	}
    }

    if (!strcmp(sec_type, "wpa:eap") || !strcmp(sec_type, "wpa:psk")) {
	list=append_cfg_item(list, gid, "wpa.grprekey", grprekey);
	list=append_cfg_item(list, gid, "wpa.type", psktype);
	list=append_cfg_item(list, gid, "wpa.passphrase", passphrase);
    }

    if (!strcmp(sec_type, "8021x")) {
	list = append_cfg_item(list, gid, "dynwep.enabled", dynwep);
	list = append_cfg_item(list, gid, "dynwep.keylen", dynkeylen);
	list = append_cfg_item(list, gid, "dynwep.rekey", dynrekey);
    }

    if (!strcmp(sec_type, "8021x") || !strcmp(sec_type, "wpa:eap") 
	    || !strcmp(sec_type, "mac")) {
	list=append_cfg_item(list, gid, "radius.server:0.ip", ip0);
	list=append_cfg_item(list, gid, "radius.server:0.port", port0);
	list=append_cfg_item(list, gid, "radius.server:0.acctenabled", acctenabled0);
	list=append_cfg_item(list, gid, "radius.server:0.acctport", acctport0);
        list=append_cfg_item(list, gid, "radius.server:0.interimIntr", interimIntr0);
	list=append_cfg_item(list, gid, "radius.server:0.secret", secret0);
#ifdef RADIUS_2
        chkFeature("RADIUS_2", tmp);
        if(tmp[0]=='Y'){
            list=append_cfg_item(list, gid, "radius.server:1.ip", ip1);
            list=append_cfg_item(list, gid, "radius.server:1.port", port1);
            list=append_cfg_item(list, gid, "radius.server:1.acctenabled", acctenabled1);
            list=append_cfg_item(list, gid, "radius.server:1.acctport", acctport1);
	    list=append_cfg_item(list, gid, "radius.server:1.interimIntr", interimIntr1);
            list=append_cfg_item(list, gid, "radius.server:1.secret", secret1);
        }
#endif
    }
#ifdef WAPI
    if (!strcmp(sec_type, "wapi:psk")) {
	wapi_method = websGetVar(wp, T("wapi_method"), T("ascii"));
	wapi_key = websGetVar(wp, T("wapi_key"), T(""));
	list=append_cfg_item(list, gid, "wapi.wapi_method", wapi_method);
	list=append_cfg_item(list, gid, "wapi.wapi_key", wapi_key);
    } else if (!strcmp(sec_type, "wapi:cert")) {
	as_ip = websGetVar(wp, T("as_ip"), T(""));
	as_port = websGetVar(wp, T("as_port"), T("3810"));
	list=append_cfg_item(list, gid, "wapi.as_ip", as_ip);
	list=append_cfg_item(list, gid, "wapi.as_port", as_port);
    }
#endif

    /* Set Universal Repeater */
    if ((!strcmp(opmode, "AP") || !strcmp(opmode, "GW")) && 
	    !strcmp(roles, "ap,rep")) {
	int repGid = GID_RF1_REP1_SEC;
	if (!strcmp(sec_type, "wep")) {
#ifndef REP_IS_ATHEROS	
	    list=append_cfg_item(list, gid, "mode", auth);
	    list=append_cfg_item(list, gid, "wep.enabled", "enable");
#endif	
	    list=append_cfg_item(list, repGid, "wep.txidx", txidx);
	    list=append_cfg_item(list, repGid, "wep.method", method);
	    list=append_cfg_item(list, repGid, "wep.keylen", keylen);
	    list=append_cfg_item(list, repGid, "wep.key0", key0);
	    list=append_cfg_item(list, repGid, "wep.key1", key1);
	    list=append_cfg_item(list, repGid, "wep.key2", key2);
	    list=append_cfg_item(list, repGid, "wep.key3", key3);
	    
	} 
#ifndef REP_IS_ATHEROS	
	else {
	    char rep_mode[MAX_VALUE_LEN], rep_wep[MAX_VALUE_LEN];
	    get_cfg_item_by_str(cipcfg, "rf1_rep1_sec", "mode", rep_mode);
	    get_cfg_item_by_str(cipcfg, "rf1_rep1_sec", "wep.enabled", rep_wep);
	    if (!strcmp(rep_wep, "enable") && (
			!strcmp(rep_mode, "open") || 
			!strcmp(rep_mode, "shared") ||
			!strcmp(rep_mode, "auto")
			)) {
		list=append_cfg_item(list, GID_RF1, "roles", "ap");
	    }
	}
#endif	
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

do_nothing:
#ifdef MULTIPLE_RF
    sprintf(tmp, "/ap/security.asp?rf=%d&vap=%d", rf_no, vap_no);
#else
    sprintf(tmp, "/ap/security.asp?vap=%d", vap_no);
#endif
    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T(tmp))); 
}

/**
 * Set VAP's MAC ACL list
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanVapAcl(webs_t wp, char_t *path, char_t *query)
{
    char_t *max_sta, *rf_vap, *acl_type;
    char_t *mac, *enabled;
    char_t *ip0, *port0, *secret0;
    char_t *ip1, *port1, *secret1;
    char_t *app;
    ITEM_LIST *list= NULL;
    CIPCFG *cipcfg = NULL;
    int ret, vap_gid, acl_gid, sec_gid, i;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0, rf_no, vap_no;

    memset(tmp, 0, 256);

    app = websGetVar(wp, T("apply"), T("true"));
    rf_vap = websGetVar(wp, T("vap"), T("10"));
    sscanf(rf_vap, "%1d%d", &rf_no, &vap_no);

    vap_gid = gid_vap[rf_no-1][vap_no];
    acl_gid = gid_vap_acl[rf_no-1][vap_no];
    sec_gid = gid_vap_sec[rf_no-1][vap_no];
    
    cipcfg=init_cipcfg();

#ifdef MAX_CLIENTS
    chkFeature("MAX_CLIENTS", tmp);
    if((tmp[0]=='Y')&&(getUserGroupPermission(wp, vap_gid) & PERM_WRITE)) {
	max_sta = websGetVar(wp, T("maxSta"), T("0"));
	list=append_cfg_item(list, vap_gid, "max_sta", max_sta);
    }
#endif

    acl_type = websGetVar(wp, T("aclType"), T("disable"));

    if (getUserGroupPermission(wp, acl_gid) & PERM_WRITE) {
	if (!strcmp(acl_type, "disable")) {
	    list=append_cfg_item(list, acl_gid, "state", "disable");
	    list=append_cfg_item(list, acl_gid, "policy", "allow");
	}else if ( !strcmp(acl_type, "allow") || !strcmp(acl_type, "deny") ) {
	    int acl_num = 0;
	    list=append_cfg_item(list, acl_gid, "state", "enable");
	    list=append_cfg_item(list, acl_gid, "policy", acl_type);
	    if (cipcfg) {
		get_cfg_item(cipcfg, acl_gid, "table_size", tmp);
		acl_num = atoi(tmp);
	    }

	    for (i=0; i< ((acl_num == 0) ? 10: acl_num); i++) {
		sprintf(tmp, "mac%d", i);
		mac = websGetVar(wp, T(tmp), T("00:00:00:00:00:00"));
		upperCase(mac);
		sprintf(tmp, "enabled%d", i);
		enabled = websGetVar(wp, T(tmp), T("disable"));
		sprintf(tmp, "acl_entry:%d.mac", i);
		list=append_cfg_item(list, acl_gid, tmp, mac);
		sprintf(tmp, "acl_entry:%d.enabled", i);
		list=append_cfg_item(list, acl_gid, tmp, enabled);
	    }
	}else if (!strcmp(acl_type, "radius")) {
#ifdef RADIUS_ACL
            chkFeature("RADIUS_ACL", tmp);
            if(tmp[0]=='Y'){
                list=append_cfg_item(list, acl_gid, "state", "enable");
                list=append_cfg_item(list, acl_gid, "policy", "radius");
                /* Get RADIUS value from web */
                ip0 = websGetVar(wp, T("ip0"), T("0.0.0.0"));
                port0 = websGetVar(wp, T("port0"), T("1812"));
                secret0 = websGetVar(wp, T("secret0"), T(""));
                ip1 = websGetVar(wp, T("ip1"), T(""));
                port1 = websGetVar(wp, T("port1"), T(""));
                secret1 = websGetVar(wp, T("secret1"), T(""));

                /* Save RADIUS setting to config */
                if (getUserGroupPermission(wp, sec_gid) & PERM_WRITE) {
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:0.ip", ip0);
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:0.port", port0);
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:0.secret", secret0);
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:1.ip", ip1);
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:1.port", port1);
                    list=append_cfg_item(list, sec_gid, 
                            "radius.server:1.secret", secret1);
                }
            }
#endif
	}
    }

    print_list(list);


    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

#ifdef MULTIPLE_RF
    sprintf(tmp, "/ap/acl.asp?rf=%d&vap=%d", rf_no, vap_no);
#else
    sprintf(tmp, "/ap/acl.asp?vap=%d", vap_no);
#endif
    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T(tmp))); 
}

static void apply_wlan_repeater(webs_t wp, char_t *path, char_t *query, int with_unirep)
{
    char_t *app, *rep_type, *wds, *distance, *acktimeout, *ctstimeout, *slottime, *rf;
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char opmode[MAX_VALUE_LEN], tmp[256];
    int ret;
    int doAddMac = 0, doDelWds = 0;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int wds_idx = 0, rf_idx = 0;
/*
    int gids[4][4]={ {GID_RF1_WDS1, GID_RF1_WDS2, GID_RF1_WDS3, GID_RF1_WDS4}, 
                     {GID_RF2_WDS1, GID_RF2_WDS2, GID_RF2_WDS3, GID_RF2_WDS4},
                     {GID_RF3_WDS1, GID_RF3_WDS2, GID_RF3_WDS3, GID_RF3_WDS4},
                     {GID_RF4_WDS1, GID_RF4_WDS2, GID_RF4_WDS3, GID_RF4_WDS4} };
*/
    app = websGetVar(wp, T("apply"), T("true"));

    cipcfg = init_cipcfg();

    get_cfg_item_by_str(cipcfg, "opmode", "type", opmode);
    if (strcmp(opmode, "AP") != 0 && strcmp(opmode, "GW") != 0 && 
	    !(getUserGroupPermission(wp, GID_RF1) & PERM_WRITE))
	goto do_nothing;

    rep_type = websGetVar(wp, T("rep_type"), T("none"));
    if (strcmp(rep_type, "wds") == 0 && 
	    (getUserGroupPermission(wp, GID_RF1_WDS1) & PERM_WRITE)) {

	char_t *mac, *enabled, *del;
	char_t *wep, *keylen, *method, *key, *item;
	char mac_buf[BUFSIZ], enable_buf[BUFSIZ], del_buf[BUFSIZ];
	int i, gid;

	list = append_cfg_item(NULL, GID_RF1, "roles", "ap,wds");
	doAddMac = atoi(websGetVar(wp, T("doAddMac"), "0"));
	doDelWds = atoi(websGetVar(wp, T("doDelWds"), "0"));

	if (doDelWds) {
	    for (i = 0; i < MAX_WDS_LINK; i++) {
		gid = gid_wds_link[0][i];
		sprintf(del_buf, "wds%d_del", i);
		del = websGetVar(wp, del_buf, T(""));
		if (strcmp(del, "on") == 0) {
		    list = append_cfg_item(list, gid, "mac", "");
		    list = append_cfg_item(list, gid, "state", "disable");
		}
	    }
	} else {
#ifdef WDS_INDEPENDENT_SECURITY
	    {
		wds = websGetVar(wp, T("wds"), T("1")); 
		rf = websGetVar(wp, T("rf"), T("1")); 

		wds_idx = atoi(wds);
		rf_idx = atoi(rf);
		wds_idx = wds_idx - 1;
		rf_idx = rf_idx - 1;

		if (wds_idx < 0) wds_idx = 0;
		if (rf_idx < 0) rf_idx = 0;

		gid = gid_wds_link[rf_idx][wds_idx];

		mac = websGetVar(wp, "macInput", T(""));
		enabled = websGetVar(wp, "state", T(""));

		if (mac[0] != '\0') {
		    list = append_cfg_item(list, gid, "mac", mac);
		    list = append_cfg_item(list, gid, "state", 
				(strcmp(enabled, "on") == 0 ? "enable" : "disable"));
		} else {
		    list = append_cfg_item(list, gid, "mac", "");
		    list = append_cfg_item(list, gid, "state", "disable");
		}
	    }
#else
	    for (i = 0; i < MAX_WDS_LINK; i++) {
		gid = gid_wds_link[0][i];
		sprintf(mac_buf, "wds%d_mac", i);
		sprintf(enable_buf, "wds%d", i);
		mac = websGetVar(wp, mac_buf, T(""));
		enabled = websGetVar(wp, enable_buf, T(""));
		if (mac[0] != '\0') {
		    list = append_cfg_item(list, gid, "mac", mac);
		    list = append_cfg_item(list, gid, "state", 
			    (strcmp(enabled, "on") == 0 ? "enable" : "disable"));
		} else {
		    list = append_cfg_item(list, gid, "mac", "");
		    list = append_cfg_item(list, gid, "state", "disable");
		}
	    }
#endif
	}
//    rf  = websGetVar(wp, T("rf"),  T("0"));
//    printf("rf=%s\n", rf);

//    item = websGetVar(wp, T("pathcost"), T("100"));
//    list = append_cfg_item( list, gid, "pathcost", item);
    // uppercase of mac address before save

	wep = websGetVar(wp, T("wdsWepEnabled"), T("disable"));
	keylen = websGetVar(wp, T("wdsWepKeylen"), T("64"));
	method = websGetVar(wp, T("wdsWepMethod"), T("ascii"));
	key = websGetVar(wp, T("wdsWepKey"), T(""));

#ifdef WDS_INDEPENDENT_SECURITY
	gid = gid_wds_link[rf_idx][wds_idx];
	if (!strcmp(wep, "wep")) {
	    list = append_cfg_item(list, gid, "wep.enabled", "enable");
	    list = append_cfg_item(list, gid, "wep.keylen", keylen);
	    list = append_cfg_item(list, gid, "wep.txidx", "1");
	    list = append_cfg_item(list, gid, "wep.method", method);
	    list = append_cfg_item(list, gid, "wep.key0", key);
	} else {
	    list = append_cfg_item(list, gid, "wep.enabled", "disable");
	}
#ifdef WDS_WPA_PSK
	if (!strcmp(wep, "wpa")) {
	    list = append_cfg_item(list, gid, "wpa.enabled", "enable");
	    item = websGetVar(wp, T("wdscipher"), T("aes-ccmp"));
	    list = append_cfg_item(list, gid, "wpa.enc", item);
	    item = websGetVar(wp, T("wdspsktype"), T("passphrase"));
	    list = append_cfg_item( list, gid, "wpa.type", item);
	    item = websGetVar(wp, T("wdspassphrase"), T(""));
	    list = append_cfg_item( list, gid, "wpa.passphrase", item);
	} else {
	    list = append_cfg_item( list, gid, "wpa.enabled", "disable");
	}
#endif /* WDS_WPA_PSK */
#ifdef RF_DISTANCE
	item = websGetVar(wp, T("distance"), T("0"));
	list = append_cfg_item( list, gid_rf[0], "distance", item);
	item = websGetVar(wp, T("acktimeout"), T("0"));
	list = append_cfg_item( list, gid_rf[0], "acktimeout", item);
	item = websGetVar(wp, T("ctstimeout"), T("0"));
	list = append_cfg_item( list, gid_rf[0], "ctstimeout", item);
	item = websGetVar(wp, T("slottime"), T("0"));
	list = append_cfg_item( list, gid_rf[0], "slottime", item);
#endif /* RF_DISTANCE */
#else /* WDS_INDEPENDENT_SECURITY */
	for (i = 0; i < MAX_WDS_LINK; i++) {
	    gid = gid_wds_link[rf_idx][i];
	    if (!strcmp(wep, "wep")) {
		list = append_cfg_item(list, gid, "wep.enabled", "enable");
		list = append_cfg_item(list, gid, "wep.keylen", keylen);
		list = append_cfg_item(list, gid, "wep.txidx", "1");
		list = append_cfg_item(list, gid, "wep.method", method);
		list = append_cfg_item(list, gid, "wep.key0", key);
	    }
	    else {
		list = append_cfg_item( list, gid, "wep.enabled", "disable");
	    }
#ifdef WDS_WPA_PSK
	    if (!strcmp(wep, "wpa")) {
		list = append_cfg_item( list, gid, "wpa.enabled", "enable");
		item = websGetVar(wp, T("wdscipher"), T("aes-ccmp"));
		list = append_cfg_item( list, gid, "wpa.enc", item);
		item = websGetVar(wp, T("wdspsktype"), T("passphrase"));
		list = append_cfg_item( list, gid, "wpa.type", item);
		item = websGetVar(wp, T("wdspassphrase"), T(""));
		list = append_cfg_item( list, gid, "wpa.passphrase", item);
	    }
	    else {
		list = append_cfg_item( list, gid, "wpa.enabled", "disable");
	    }
#endif /* WDS_WPA_PSK */
#ifdef RF_DISTANCE
	    item = websGetVar(wp, T("distance"), T("0"));
	    list = append_cfg_item( list, gid_rf[0], "distance", item);
	    item = websGetVar(wp, T("acktimeout"), T("0"));
	    list = append_cfg_item( list, gid_rf[0], "acktimeout", item);
	    item = websGetVar(wp, T("ctstimeout"), T("0"));
	    list = append_cfg_item( list, gid_rf[0], "ctstimeout", item);
	    item = websGetVar(wp, T("slottime"), T("0"));
	    list = append_cfg_item( list, gid_rf[0], "slottime", item);
#endif
	}
#endif /* WDS_INDEPENDENT_SECURITY */

/*
#ifdef WDS_HT
        chkFeature("WDS_HT", tmp);
        if(tmp[0]=='Y'){
            item = websGetVar(wp, T("wds_ht"), T("disable"));
	    list = append_cfg_item(list, gid, "ht", item);
        }
#endif
*/
    } else if (with_unirep == 1 && strcmp(rep_type, "unirep") == 0 && 
	    (getUserGroupPermission(wp, GID_RF1_REP1) & PERM_WRITE)) {

	char_t *ssid, *sec;
	char_t *auth, *keylen, *method, *txidx, *key0, *key1, *key2, *key3;
	char_t *version, *cipher, *psktype, *passphrase;
	int gid = GID_RF1_REP1_SEC;
	int vapGid = GID_RF1_VAP1_SEC;

	list = append_cfg_item(NULL, GID_RF1, "roles", "ap,rep");
	list = append_cfg_item(list, GID_RF1_REP1, "state", "enable");

	ssid = websGetVar(wp, T("ssid"), "");
	list = append_cfg_item(list, GID_RF1_REP1, "ssid", ssid);

	sec = websGetVar(wp, T("UniRepSecType"), "none");
	if (strcmp(sec, "wep") == 0) {
	    auth = websGetVar(wp, T("auth"), "open");
	    keylen = websGetVar(wp, T("keylen"), "64");
	    method = websGetVar(wp, T("method"), "ascii");
	    txidx = websGetVar(wp, T("txidx"), "1");
	    key0 = websGetVar(wp, T("key0"), "");
	    key1 = websGetVar(wp, T("key1"), "");
	    key2 = websGetVar(wp, T("key2"), "");
	    key3 = websGetVar(wp, T("key3"), "");

	    /* TODO: check value */
	    list = append_cfg_item(list, gid, "mode", auth);
	    list = append_cfg_item(list, gid, "wep.enabled", "enable");
	    list = append_cfg_item(list, gid, "wep.keylen", keylen);
	    list = append_cfg_item(list, gid, "wep.method", method);
	    list = append_cfg_item(list, gid, "wep.txidx", txidx);
	    list = append_cfg_item(list, gid, "wep.key0", key0);
	    list = append_cfg_item(list, gid, "wep.key1", key1);
	    list = append_cfg_item(list, gid, "wep.key2", key2);
	    list = append_cfg_item(list, gid, "wep.key3", key3);
	    /* Also apply to vap1 */
#ifndef REP_IS_ATHEROS
	    list = append_cfg_item(list, vapGid, "mode", auth);
	    list = append_cfg_item(list, vapGid, "wep.enabled", "enable");
#endif
	    list = append_cfg_item(list, vapGid, "wep.txidx", txidx);
	    list = append_cfg_item(list, vapGid, "wep.keylen", keylen);
	    list = append_cfg_item(list, vapGid, "wep.method", method);
	    list = append_cfg_item(list, vapGid, "wep.key0", key0);
	    list = append_cfg_item(list, vapGid, "wep.key1", key1);
	    list = append_cfg_item(list, vapGid, "wep.key2", key2);
	    list = append_cfg_item(list, vapGid, "wep.key3", key3);
	} else if (strcmp(sec, "wpa:psk") == 0) {
	    cipher = websGetVar(wp, T("cipher"), "1:rc4-tkip");
	    psktype = websGetVar(wp, T("psktype"), "passphrase");
	    passphrase = websGetVar(wp, T("passphrase"), "");

	    if (!strcmp(cipher, "1:rc4-tkip")) {
		list=append_cfg_item(list, gid, "wpa.version", "1");
		list=append_cfg_item(list, gid, "wpa.enc", "rc4-tkip");
	    } else if (!strcmp(cipher, "1:aes-ccmp")) {
		list=append_cfg_item(list, gid, "wpa.version", "1");
		list=append_cfg_item(list, gid, "wpa.enc", "aes-ccmp");
	    } else if (!strcmp(cipher, "2:rc4-tkip")) {
		list=append_cfg_item(list, gid, "wpa.version", "2");
		list=append_cfg_item(list, gid, "wpa.enc", "rc4-tkip");
	    } else if (!strcmp(cipher, "2:aes-ccmp")) {
		list=append_cfg_item(list, gid, "wpa.version", "2");
		list=append_cfg_item(list, gid, "wpa.enc", "aes-ccmp");
	    }

	    list = append_cfg_item(list, gid, "mode", "wpa");
	    list = append_cfg_item(list, gid, "wep.enabled", "disable");
	    list = append_cfg_item(list, gid, "wpa.eap", "psk");
	    list = append_cfg_item(list, gid, "wpa.type", psktype);
	    list = append_cfg_item(list, gid, "wpa.passphrase", passphrase);
	} else {
	    list = append_cfg_item(list, gid, "mode", "open");
	    list = append_cfg_item(list, gid, "wep.enabled", "disable");
	}
    } else if (strcmp(rep_type, "none") == 0) {
	list = append_cfg_item(NULL, GID_RF1, "roles", "ap");
    } else {
	goto do_nothing;
    }
#ifdef CIPWES 
    char_t *wes;
    wes = websGetVar(wp, T("wes_btn"), T("disable"));
    list = append_cfg_item( list, GID_WES, "state", wes);
#endif

    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	printf("Error set cipcfg items.... Error Code: %d\n", ret);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) || doAddMac == 1 || doDelWds == 1){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

do_nothing:
    close_cipcfg(cipcfg);

    {
	char_t *def_url = (with_unirep == 1) ? T("/wireless/repeater_config.asp") : T("/wireless/wds_config.asp");
	websRedirect(wp, strcmp(rep_type, "wds") ? def_url : websGetVar(wp, T("REFERER"), def_url)); 
    }
}

/**
 * Set WLAN WDS configuration
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanWds(webs_t wp, char_t *path, char_t *query)
{
    apply_wlan_repeater(wp, path, query, 0);
}

/**
 * Set Repeater configuration
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWlanRepeater(webs_t wp, char_t *path, char_t *query)
{
    apply_wlan_repeater(wp, path, query, 1);
}

/**
 * Set Scan configuration
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyScan(webs_t wp, char_t *path, char_t *query)
{
    char_t *state, *interval;
    char_t *app;
    ITEM_LIST *list;
    CIPCFG *cipcfg = NULL;
    char tmp[256];
    int ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    cipcfg = init_cipcfg();

    *tmp = '\0';
    get_cfg_item(cipcfg, GID_RF4, "roles", tmp);
    if (!strstr(tmp, "scan"))
	goto do_nothing;
    
    app = websGetVar(wp, T("apply"), T("true"));
    state = websGetVar(wp, T("state"), T("disable"));
    printf("set scan state \"%s\"\n", state );
    list=append_cfg_item(NULL, GID_RF4, "state", state);
    list=append_cfg_item(list, GID_RF4_SCAN, "state", state);
    
    if ( !strncmp(state, "dis", 3) ){
        
    }else{
        interval = websGetVar(wp, T("interval"), T("60"));
        printf("set scan interval %s sec\n", interval );
        list=append_cfg_item(list, GID_RF4_SCAN, "interval", interval);
    }
    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
	// free_item_list(list);
	close_cipcfg(cipcfg);
	free(applylist);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;
 
    // free_item_list(list);

    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
        if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
        }else{
            close_cipcfg(cipcfg);
            free(applylist);
            websRedirect(wp, "/utilities/reboot.asp");
            return;
        }
    }

do_nothing:
    close_cipcfg(cipcfg);
    free(applylist);
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/utilities/scan.asp"))); 
}

/**
 * Connect to AP
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyScanConnect(webs_t wp, char_t *path, char_t *query)
{
    char_t *ssid, *channel, *security;
    char_t *auth, *method, *keylen, *txidx, *key0, *key1, *key2, *key3;
    char_t *cipher, *psktype, *passphrase;
    char_t *version[2] = {"1", "2"};
    char_t *preferbssid;
    int wpa;
    char opmode[MAX_VALUE_LEN];
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    int gid, secGid, ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;

    ssid = websGetVar(wp, T("ssid"), T("NO-SSID"));
    channel = websGetVar(wp, T("channel"), T("0"));
    security = websGetVar(wp, T("security"), T("None"));
    auth = websGetVar(wp, T("auth"), T("open"));
    method = websGetVar(wp, T("method"), T("ascii"));
    keylen = websGetVar(wp, T("keylen"), T("64"));
    txidx = websGetVar(wp, T("txidx"), T("1"));
    key0 = websGetVar(wp, T("key0"), T(""));
    key1 = websGetVar(wp, T("key1"), T(""));
    key2 = websGetVar(wp, T("key2"), T(""));
    key3 = websGetVar(wp, T("key3"), T(""));
    wpa = atoi(websGetVar(wp, T("wpa"), T("1")));
    if (wpa < 1 || wpa > 2) 
	wpa = 2;
    cipher =  websGetVar(wp, T("cipher"), T("rc4-tkip"));
    psktype = websGetVar(wp, T("psktype"), T("passphrase"));
    passphrase = websGetVar(wp, T("passphrase"), T(""));

    cipcfg = init_cipcfg();

    get_cfg_item_by_str(cipcfg, "opmode", "type", opmode);
    if (strcmp(opmode, "CPE") == 0) {
	list = append_cfg_item(NULL, GID_RF1, "channel", "auto");
	gid = GID_RF1_STA1;
	secGid = GID_RF1_STA1_SEC;
#ifdef STA_PREFERRED_BSSID
	preferbssid =websGetVar(wp, T("bssid2join"), T(""));
	list = append_cfg_item(list, gid, "bssid2join", preferbssid);
#endif
    } else {
	list = append_cfg_item(NULL, GID_RF1, "channel", channel);
	gid = GID_RF1_REP1;
	secGid = GID_RF1_REP1_SEC;
    }
    list = append_cfg_item(list, gid, "state", "enable");
    list = append_cfg_item(list, gid, "ssid", ssid);

    if (strcmp(security, "WEP") == 0) {
	list = append_cfg_item(list, secGid, "mode", auth);
	list = append_cfg_item(list, secGid, "wep.enabled", "enable");
	list = append_cfg_item(list, secGid, "wep.method", method);
	list = append_cfg_item(list, secGid, "wep.keylen", keylen);
	list = append_cfg_item(list, secGid, "wep.txidx", txidx);
	list = append_cfg_item(list, secGid, "wep.key0", key0);
	list = append_cfg_item(list, secGid, "wep.key1", key1);
	list = append_cfg_item(list, secGid, "wep.key2", key2);
	list = append_cfg_item(list, secGid, "wep.key3", key3);
	if (strcmp(opmode, "AP") == 0 || strcmp(opmode, "GW") == 0) {
	    int vapGid = GID_RF1_VAP1_SEC;
	    list = append_cfg_item(list, vapGid, "mode", auth);
	    list = append_cfg_item(list, vapGid, "wep.enabled", "enable");
	    list = append_cfg_item(list, vapGid, "wep.method", method);
	    list = append_cfg_item(list, vapGid, "wep.keylen", keylen);
	    list = append_cfg_item(list, vapGid, "wep.txidx", txidx);
	    list = append_cfg_item(list, vapGid, "wep.key0", key0);
	    list = append_cfg_item(list, vapGid, "wep.key1", key1);
	    list = append_cfg_item(list, vapGid, "wep.key2", key2);
	    list = append_cfg_item(list, vapGid, "wep.key3", key3);
	}
    } else if (strcmp(security, "WPA-PSK") == 0) {
	list = append_cfg_item(list, secGid, "mode", "wpa");
	list = append_cfg_item(list, secGid, "wep.enabled", "disable");
	list = append_cfg_item(list, secGid, "wpa.eap", "psk");
	list = append_cfg_item(list, secGid, "wpa.version", version[wpa-1]);
	list = append_cfg_item(list, secGid, "wpa.enc", cipher);
	list = append_cfg_item(list, secGid, "wpa.type", psktype);
	list = append_cfg_item(list, secGid, "wpa.passphrase", passphrase);
    } else {
	list = append_cfg_item(list, secGid, "mode", "open");
	list = append_cfg_item(list, secGid, "wep.enabled", "disable");
    }
    print_list(list);

    ret = set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    if(can_apply(applylist) == CIPCFG_SUCCESS){
        apply_cfg(cipcfg);
        unlink(PATH_SYNC_CFG);
        free(applylist);
    }else{
        close_cipcfg(cipcfg);
        websRedirect( wp, "/utilities/reboot.asp" );
        free(applylist);
        return;
    }

    close_cipcfg(cipcfg);
    websRedirect(wp, "/status/overview.asp" ); 
}

/**
 * WDS Discovery
 *
 * @param  wp:
 * @param  path:
 * @param  query:
 * @resturn none:
 */
void applyWdsDiscover(webs_t wp, char_t *path, char_t *query)
{
    char_t  *mac, tmp[MAX_WDS_LINK][MAX_VALUE_LEN];
    ITEM_LIST *list = NULL;
    CIPCFG  *cipcfg = NULL;
    int rf, ret, i;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx=0;
    /*
    int gid[4][4]={ {GID_RF1_WDS1, GID_RF1_WDS2, GID_RF1_WDS3, GID_RF1_WDS4}, 
                    {GID_RF2_WDS1, GID_RF2_WDS2, GID_RF2_WDS3, GID_RF2_WDS4},
                    {GID_RF3_WDS1, GID_RF3_WDS2, GID_RF3_WDS3, GID_RF3_WDS4},
                    {GID_RF4_WDS1, GID_RF4_WDS2, GID_RF4_WDS3, GID_RF4_WDS4} };
    */
    rf = atoi(websGetVar(wp, T("wds_rf"), T("0")));
    mac = websGetVar(wp, T("mac"), T(""));
    printf("Adding mac  %s on RF%d\n", mac, rf);
    memset(tmp, 0, MAX_WDS_LINK * MAX_VALUE_LEN );
    cipcfg=init_cipcfg();
    // check where to add mac
    for (i=0; i < MAX_WDS_LINK; i++){
        ret = get_cfg_item(cipcfg, gid_wds_link[rf-1][i] , "mac" , tmp[i]);
        if( tmp[i][0] == '\0' ){
            break;
        }
    }
    if ( i == MAX_WDS_LINK ){
        printf("WDS full. Can not add wds link.\n ");
        websWrite(wp, T("WDS full. Can not add wds link.\n"));
        close_cipcfg(cipcfg);
        return ;
    }else{
        list=append_cfg_item(NULL, gid_wds_link[rf-1][i], "state", "enable");
        list=append_cfg_item(list, gid_wds_link[rf-1][i], "mac", mac);
    }

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("Set cipcfg fault..%d\n",ret);
        close_cipcfg(cipcfg);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    if(can_apply(applylist) == CIPCFG_SUCCESS){
        apply_cfg(cipcfg);
        unlink(PATH_SYNC_CFG);
    }else{
        close_cipcfg(cipcfg);
        websRedirect( wp, "/utilities/reboot.asp" );
        return;
    }

    close_cipcfg(cipcfg);
    
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/wds/overview.asp"))); 
}


/**
 * Delete WDS Link 
 */
void applyDeleteWDS(webs_t wp, char_t *path, char_t *query)
{
    char_t  *wds;
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;
    char wds_name[5];
    int i, rf, ret;
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    
    /*
    int gid[4][4]={ {GID_RF1_WDS1, GID_RF1_WDS2, GID_RF1_WDS3, GID_RF1_WDS4}, 
                    {GID_RF2_WDS1, GID_RF2_WDS2, GID_RF2_WDS3, GID_RF2_WDS4},
                    {GID_RF3_WDS1, GID_RF3_WDS2, GID_RF3_WDS3, GID_RF3_WDS4},
                    {GID_RF4_WDS1, GID_RF4_WDS2, GID_RF4_WDS3, GID_RF4_WDS4} };
    */
    rf = atoi(websGetVar(wp, T("rf"), T("0")));
    for (i=0; i < MAX_WDS_LINK; i++){
        memset(wds_name, 0, 5);
        sprintf(wds_name, "wds%d%d", rf, i+1) ;
        wds = websGetVar(wp, T(wds_name), T("NONE"));
        if ( !strncmp( wds, "delete", 6) ){
            list=append_cfg_item(list, gid_wds_link[rf-1][i] , "state", "disable");
            list=append_cfg_item(list, gid_wds_link[rf-1][i] , "mac", "");
#ifdef WDS_HT
            list=append_cfg_item(list, gid_wds_link[rf-1][i] , "ht", "disable");
#endif /* WDS_HT */
            list=append_cfg_item(list, gid_wds_link[rf-1][i] , "wep.enabled", "disable");
            list=append_cfg_item(list, gid_wds_link[rf-1][i] , "wpa.enabled", "disable");
        }
    }

    print_list(list);
    cipcfg=init_cipcfg();

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
        list = list->next;
    }
    applylist[applist_idx] = GID_END;

    if(can_apply(applylist) == CIPCFG_SUCCESS){
        apply_cfg(cipcfg);
        unlink(PATH_SYNC_CFG);
        free(applylist);
    }else{
        close_cipcfg(cipcfg);
        websRedirect( wp, "/utilities/reboot.asp" );
        free(applylist);
        return;
    }

    close_cipcfg(cipcfg);
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/wds/overview.asp"))); 
}

/**
 * Apply WPS settings & operations
 */
void applyWPS(webs_t wp, char_t *path, char_t *query)
{
#define CMD_PBC_STR	"hostapd_cli -i %s wps_pbc"
#define CMD_PIN_STR     "hostapd_cli -i %s wps_pin any %s"
    char_t *app, *wps_state, *target_vap, *pin, *method, *act_vap, *sta_pin;
    char *wps_states[] = {"disable", "unconfigure", "configured"};
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    int i, gid, vap_idx = 0, ret = 0;
    int rf_idx = 0; /* XXX: we assume that VAPs run on the first RF */ 
    char cmd[BUFSIZ], iface[BUFSIZ];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx=0;

    app = websGetVar(wp, T("apply"), T("true"));
    method = websGetVar(wp, T("method"), T(""));
    act_vap = websGetVar(wp, T("activated_vap"), T(""));

    if (strcmp(method, "PBC") == 0 && strcmp(act_vap, "") != 0) {
	if ((getOperationPermission(wp, OID_WPS_OPERATION) & PERM_EXEC)) {
	    sprintf(iface, "ath%dap%d", rf_idx, atoi(act_vap));
	    sprintf(cmd, CMD_PBC_STR, iface);
	    cip_system(cmd);
	}
    } else if (strcmp(method, "PIN") == 0 && strcmp(act_vap, "") != 0) {
	sta_pin = websGetVar(wp, T("sta_pin"), T(""));
	if (strcmp(sta_pin, "") != 0 && 
		(getOperationPermission(wp, OID_WPS_OPERATION) & PERM_EXEC)) {
	    sprintf(iface, "ath%dap%d", rf_idx, atoi(act_vap));
	    sprintf(cmd, CMD_PIN_STR, iface, sta_pin);
	    cip_system(cmd);
	}
    } else {
	wps_state = websGetVar(wp, T("wps_state"), T(wps_states[0]));
	target_vap = websGetVar(wp, T("target_vap"), T("0"));
	pin = websGetVar(wp, T("pin"), T("12345670"));
	vap_idx = atoi(target_vap);
	gid = gid_vap_sec[rf_idx][vap_idx];
	
	if (getUserGroupPermission(wp, gid) & PERM_WRITE) {
	    if (strcmp(wps_state, wps_states[1]) == 0) { // unconfigure
		list = append_cfg_item(list, gid, "wps.enabled", "enable");
		list = append_cfg_item(list, gid, "wps.configure", "enable");
		list = append_cfg_item(list, gid, "wps.ap_pin", pin);
	    } else if (strcmp(wps_state, wps_states[2]) == 0) { // configured
		list = append_cfg_item(list, gid, "wps.enabled", "enable");
		list = append_cfg_item(list, gid, "wps.configure", "disable");
		list = append_cfg_item(list, gid, "wps.ap_pin", pin);
	    }
	}
	/* Disabled WPS of others VAPs, we only active one for them */
	for (i = 0; i < MAX_VAP; i++) {
	    int sec_gid = gid_vap_sec[rf_idx][i];
	    if (i != vap_idx || strcmp(wps_state, wps_states[0]) == 0)
		if (getUserGroupPermission(wp, sec_gid) & PERM_WRITE)
                {
		    list = append_cfg_item(list, sec_gid, 
			    "wps.enabled", "disable");
                }
	}

	print_list(list);

	cipcfg = init_cipcfg();

	ret = set_cfg_items(cipcfg, list);
	if (ret != CIPCFG_SUCCESS) {
	    printf("Set cipcfg fault..%d\n",ret);
	    return ;
	}

        while(list != NULL)
        {
            if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
            {
                applylist[applist_idx] = list->gid;
                applist_idx++;
            }
            list = list->next;
        }
        applylist[applist_idx] = GID_END;

	// check if apply or just save settings into config file.
	if ( !strncmp("true", app, 4 ) ){
            if(can_apply(applylist) == CIPCFG_SUCCESS){
		apply_cfg(cipcfg);
                unlink(PATH_SYNC_CFG);
                free(applylist);
	    }else{
		close_cipcfg(cipcfg);
		websRedirect( wp, "/utilities/reboot.asp" );
                free(applylist);
		return;
	    }
	}
	close_cipcfg(cipcfg);
    }

    websRedirect(wp, websGetVar(wp, T("REFERER"), T("/ap/wps.asp"))); 
}

/** 
 * @brief Kick out station of specified MAC address
 * 
 * @param wp 
 * @param path 
 * @param query 
 */
void kickStation(webs_t wp, char_t *path, char_t *query)
{
    char_t *vap, *macaddr;
    char *iface = NULL, buf[MAX_VALUE_LEN];
    int rf_no, vap_no;

    vap = websGetVar(wp, T("vap"), T(""));
    macaddr = websGetVar(wp, T("macAddr"), T(""));

    if (strcmp(vap, "") != 0 && strcmp(macaddr, "") != 0) {
	if (sscanf(vap, "rf%d_vap%d", &rf_no, &vap_no) > 0)
	    iface = if_vap_2d[rf_no-1][vap_no-1];
	if (iface) {
#ifndef RF_RALINK
	    sprintf(buf, "%s %s kickmac %s", PATH_IWPRIV, iface, macaddr);
#else
	    sprintf(buf, "%s %s set DisConnectSta=%s", PATH_IWPRIV, iface, macaddr);
#endif
	    cip_system(buf);
	}
    }
    websRedirect(wp, 
	    websGetVar(wp, T("REFERER"), T("/status/clients.asp"))); 
}

int setPower(int eid, webs_t wp, int argc, char_t **argv)
{
    char *txpwr;
    ITEM_LIST *list= NULL;
    CIPCFG *cipcfg = NULL;

    txpwr = websGetVar(wp, T("set"), T(""));
    if(strcmp(txpwr,"")==0){
        websWrite(wp,T("Invalid operation.<br>"));
        return 0;
    }

    list = append_cfg_item(list, GID_RF1, "txpwr", txpwr);
    cipcfg = init_cipcfg();
    if(set_cfg_items(cipcfg, list) == CIPCFG_SUCCESS) {
        unlink(PATH_SYNC_CFG);
        set_txpwr(txpwr);
    }
    close_cipcfg(cipcfg);
    return 0;
}

int setCSA(int eid, webs_t wp, int argc, char_t **argv)
{
    char *channel_str, *count_str, ret_str[BUFSIZ];
    int channel, count, ret;

    channel_str = count_str = 0;
    channel_str = websGetVar(wp, T("channel"), T(""));
    count_str = websGetVar(wp, T("count"), T(""));
    channel = atoi(channel_str);
    count = atoi(count_str);

    if(channel > 0 && count > 0){
	ret = set_channel_switch(channel,count);
	sprintf(ret_str,"%d",ret);
        websWrite(wp,T(ret_str));
    }
    else
        websWrite(wp,T("0"));
    return 0;
}

#ifdef GRE
int setGRE(int eid, webs_t wp, int argc, char_t **argv)
{
    char *state, *remote, *key;
    ITEM_LIST *list = NULL;
    CIPCFG *cipcfg = NULL;

    state = websGetVar(wp, T("state"), T(""));
    remote = websGetVar(wp, T("remote"), T(""));
    key = websGetVar(wp, T("key"), T(""));

    if (!strcmp(state, "disable") || remote[0] == '\0') {
	list = append_cfg_item(list, GID_GRE, "state", "disable");
    } else {
	list = append_cfg_item(list, GID_GRE, "state", "enable");
	list = append_cfg_item(list, GID_GRE, "remote", remote);
	list = append_cfg_item(list, GID_GRE, "key", key);
    }

    cipcfg = init_cipcfg();
    set_cfg_items(cipcfg, list);
    close_cipcfg(cipcfg);

    return 0;
}
#endif

#ifdef CAPWAP
int genTunInterfaceList(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char group_name[MAX_VALUE_LEN]; 
    char state[MAX_VALUE_LEN], tun_state[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], interface[MAX_VALUE_LEN];
    char output[BUFSIZ];
    char desc[BUFSIZ];
    char tmp[BUFSIZ];
    int i, rfidx=0, lan_num=0;

    cipcfg=init_cipcfg();

    sprintf(group_name, "tun", rfidx+1, i+1);
    if(get_cfg_item_by_str(cipcfg, "network_L2", "lan_num", item_value) == CIPCFG_SUCCESS)
	lan_num = atoi(item_value);

    websWrite(wp,T("{"));

    if(lan_num > 0){
        websWrite(wp,T("\"lan\":["));
        for (i = 0; i < lan_num; i++){
            sprintf(group_name, "lan:%d.controllable", i);
            get_cfg_item_by_str(cipcfg, "network_L2", group_name, state);
            sprintf(group_name, "lan:%d.tun", i);
            get_cfg_item_by_str(cipcfg, "network_L2", group_name, tun_state);
            sprintf(output, "\"state\":%s, \"tun_state\":%s", "true",
                strcmp(tun_state, "enable") ? "false" : "true");
	    sprintf(group_name, "lan:%d.desc", i);
	    memset(tmp, 0, sizeof(tmp));
	    get_cfg_item_by_str(cipcfg, "network_L2", group_name, tmp);
	    if (strlen(tmp) != 0) {
		sprintf(desc, "\"desc\":\"%s\"", tmp);
	    } else {
		if (i == 0)
		    sprintf(desc, "\"desc\":\"Uplink\"");
		else
		    sprintf(desc, "\"desc\":\"LAN%d\"", i);
	    }
            websWrite(wp, T((i ? ",{" : "{")));
	    if(strcmp(state,"disable") != 0) {
		websWrite(wp,T(output));
		websWrite(wp,T(", "));
	    }
	    websWrite(wp,T(desc));
            websWrite(wp,T("}"));
        }
        websWrite(wp,T("],"));
    }

    websWrite(wp,T("\"vap\":["));
    for (i = 0; i < MAX_VAP; i++) {
	sprintf(group_name, "rf%d_vap%d", rfidx+1, i+1);
	get_cfg_item_by_str(cipcfg, group_name, "state", state);
	get_cfg_item_by_str(cipcfg, group_name, "tun", tun_state);
	sprintf(output, "\"state\":%s, \"tun_state\":%s", 
		strcmp(state, "enable") ? "false" : "true", 
		strcmp(tun_state, "enable") ? "false" : "true");
	websWrite(wp, T((i ? ",{" : "{")));
	websWrite(wp,T(output));
	websWrite(wp,T("}"));
    }
    websWrite(wp,T("], \"wds\":["));
    for (i = 0; i < MAX_WDS_LINK; i++) {
	sprintf(group_name, "rf%d_wds%d", rfidx+1, i+1);
	get_cfg_item_by_str(cipcfg, group_name, "state", state);
	get_cfg_item_by_str(cipcfg, group_name, "tun", tun_state);
	sprintf(output, "\"state\":%s, \"tun_state\":%s", 
		strcmp(state, "enable") ? "false" : "true", 
		strcmp(tun_state, "enable") ? "false" : "true");
	websWrite(wp, T((i ? ",{" : "{")));
	websWrite(wp,T(output));
	websWrite(wp,T("}"));
    }
    websWrite(wp,T("]}"));

    close_cipcfg(cipcfg);
    return 0;
}
#endif

#define BAND_11B	1 
#define BAND_11G	2
#define BAND_11A	3 
#define BAND_11NA	4
#define BAND_11NAU	5
#define BAND_11NAL	6
#define BAND_11NG	7
#define BAND_11NGU	8
#define BAND_11NGL	9
int print_channel(int mode, COUNTRY_CHANNEL cc ,webs_t wp )
{
    char ch[BUFSIZ];
    int i = 0;
    memset(ch, 0, BUFSIZ);
    sprintf(ch, "channel={");
    switch(mode){
	case BAND_11B:
	    if( -1 != cc.ch_11b[0] ){
                for(i=0; (-1 != cc.ch_11b[i]) ; i++){
                    sprintf(ch, "%s%d,", ch, cc.ch_11b[i]);
                }
            }
	break;
	case BAND_11G:
	    if( -1 != cc.ch_11g[0] ){
                for(i=0; (-1 != cc.ch_11g[i]) ; i++){
                    sprintf(ch, "%s%d,", ch, cc.ch_11g[i]);
                }
            }
	break;
	case BAND_11A:
	    if( -1 != cc.ch_11a[0] ){
                for(i=0; (-1 != cc.ch_11a[i]) ; i++){
                    sprintf(ch, "%s%d,", ch, cc.ch_11a[i]);
                }
            }
	break;
	case BAND_11NA:
            if (-1 != cc.ch_11na[0]) {
                for(i=0; (-1 != cc.ch_11na[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11na[i]);
                }
            }
	break;
	case BAND_11NAU:
	    if (-1 != cc.ch_11nau[0]) {
                for(i=0; (-1 != cc.ch_11nau[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11nau[i]);
                }
            }
	break;
	case BAND_11NAL:
	    if (-1 != cc.ch_11nal[0]) {
                for(i=0; (-1 != cc.ch_11nal[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11nal[i]);
                }
            }
	break;
	case BAND_11NG:
            if (-1 != cc.ch_11ng[0]) {
                for(i=0; (-1 != cc.ch_11ng[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11ng[i]);
                }
            }
	break;
	case BAND_11NGU:
	    if (-1 != cc.ch_11ngu[0]) {
                for(i=0; (-1 != cc.ch_11ngu[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11ngu[i]);
                }
            }
	break;
	case BAND_11NGL:
	    if (-1 != cc.ch_11ngl[0]) {
                for(i=0; (-1 != cc.ch_11ngl[i]); i++) {
                    sprintf(ch, "%s%d,", ch, cc.ch_11ngl[i]);
                }
            }
	break;
    }
	if( strlen(ch) > 0 && ch[strlen(ch)-1] == ',')
	    ch[strlen(ch)-1] = '\0';
	sprintf(ch, "%s};", ch);
        websWrite(wp, T(ch));
}


int scanAP(int eid, webs_t wp, int argc, char_t **argv)
{
    AP_LIST *ap;
    char out[BUFSIZ], rf_mode[256], ch_width[256], chwidth_ext[256];
    time_t time_now = time(NULL);
    CIPCFG *cipcfg = NULL;

    if(time_now - last_ap_scan_time > SCAN_INTERVAL){
        if(ap_scan_list != NULL)
            free_ap_list(ap_scan_list);
        ap_scan_list = ap_scan();
        last_ap_scan_time = time_now;
    }

    ap = ap_scan_list;
    while(ap){
        memset(out,0,BUFSIZ);
        sprintf(out,"log={%s,%s,%d,%d,%d,%s,%s};",
            ap->mac, ap->ssid, ap->ch, ap->rate, ap->signal, ap->type, ap->security);
        websWrite(wp,T(out));
        ap = ap->next;
    }

    //available channel list
    cipcfg = init_cipcfg_f(current_config);
    COUNTRY_CHANNEL cc;
    if ( 0 != get_allowed_ch(&cc) ){
	goto end;
    }
    if(get_cfg_item_by_str(cipcfg, "rf1", "mode", rf_mode) == CIPCFG_SUCCESS){
	if(!strcmp(rf_mode, "11ng") || !strcmp(rf_mode, "11na")){
	    if(get_cfg_item_by_str(cipcfg, "rf1", "ch_width", ch_width) == CIPCFG_SUCCESS){
		if(strcmp(ch_width, "20")){
		    if(get_cfg_item_by_str(cipcfg, "rf1", "chwidth_ext", chwidth_ext) == CIPCFG_SUCCESS){
			if(!strcmp(chwidth_ext, "l")){
			    if(!strcmp(rf_mode, "11ng"))
				print_channel(BAND_11NGL, cc, wp);
			    else
				print_channel(BAND_11NAL, cc, wp);
			}else{
			    if(!strcmp(rf_mode, "11ng"))
				print_channel(BAND_11NGU, cc, wp);
			    else
				print_channel(BAND_11NAU, cc, wp);
			}
		    }
		}else
		    print_channel(BAND_11G, cc, wp);
	    }
	}else if (!strcmp(rf_mode, "11a")){
	    print_channel(BAND_11A, cc, wp);
	}else if (!strcmp(rf_mode, "11b")){
	    print_channel(BAND_11B, cc, wp);
	}else if (!strcmp(rf_mode, "11g")){
	    print_channel(BAND_11G, cc, wp);
	}else if (!strcmp(rf_mode, "auto")){
	    print_channel(BAND_11G, cc, wp);
	}
    }
end:
    close_cipcfg(cipcfg);
    return 0;
}

#ifdef LAYER2_FIREWALL
char * genBfwMainEntry(int rule_id, char* value)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN],group_name[MAX_VALUE_LEN];
    char rule_name_buf[MAX_VALUE_LEN], enable_buf[MAX_VALUE_LEN], action_buf[MAX_VALUE_LEN]; 
    char etherType_buf[MAX_VALUE_LEN], remark_buf[MAX_VALUE_LEN];
    char sMac_buf[MAX_VALUE_LEN], dMac_buf[MAX_VALUE_LEN], sMacMask_buf[MAX_VALUE_LEN]; 
    char dMacMask_buf[MAX_VALUE_LEN], sInterface_buf[MAX_VALUE_LEN], dInterface_buf[MAX_VALUE_LEN];
    	
    int tbl_size, num;

    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();
        
	//		bfw entry	
	
	sprintf(str, "bfw_entry:%d.enabled", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, enable_buf);
        
	sprintf(str, "bfw_entry:%d.action", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, action_buf);
        
	sprintf(str, "bfw_entry:%d.rule_name", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rule_name_buf);
	
	sprintf(str, "bfw_entry:%d.etherType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, etherType_buf);
        
	sprintf(str, "bfw_entry:%d.remark", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, remark_buf);

	sprintf(str, "bfw_entry:%d.sMac", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sMac_buf);

	sprintf(str, "bfw_entry:%d.dMac", rule_id);
	get_cfg_item_by_str(cipcfg, group_name, str, dMac_buf);
	
	sprintf(str, "bfw_entry:%d.sMacMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sMacMask_buf);

	sprintf(str, "bfw_entry:%d.dMacMask", rule_id);
	get_cfg_item_by_str(cipcfg, group_name, str, dMacMask_buf);
	
	sprintf(str, "bfw_entry:%d.sInterface", rule_id);
	get_cfg_item_by_str(cipcfg, group_name, str, sInterface_buf);
	
	sprintf(str, "bfw_entry:%d.dInterface", rule_id);
	get_cfg_item_by_str(cipcfg, group_name, str, dInterface_buf);
	
	num = sprintf(value, "\"enabled\":\"%s\", \"action\":\"%s\", \"rule_name\":\"%s\", \"etherType\":\"%s\", \"remark\":\"%s\", \"sMac\":\"%s\", \"dMac\":\"%s\", \"sMacMask\":\"%s\", \"dMacMask\":\"%s\", \"sInterface\":\"%s\", \"dInterface\":\"%s\"  ",enable_buf, action_buf, rule_name_buf, etherType_buf, remark_buf, sMac_buf, dMac_buf, sMacMask_buf, dMacMask_buf, sInterface_buf, dInterface_buf);

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwDaiPvg(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[256], item_value[256], value[512], *group_name;
    char mac_buf[256], ip_buf[256], state_buf[256];
    int i = 1, tbl_size, num, isFirst = 1;

    if (ejArgs(argc, argv, T("%s"), &group_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "ptbl_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp,T("["));
    for (i = 0; i < tbl_size; i++) {
	if (isFirst == 0)
	    websWrite(wp, T(","));
        
        sprintf(str, "privilege_entry:%d.mac", i);
	get_cfg_item_by_str(cipcfg, group_name, str, mac_buf);
        sprintf(str, "privilege_entry:%d.ip", i);
	get_cfg_item_by_str(cipcfg, group_name, str, ip_buf);
        sprintf(str, "privilege_entry:%d.state", i);
	get_cfg_item_by_str(cipcfg, group_name, str, state_buf);
 	 
    	//websWrite(wp,T("{"));
	//sprintf(value,genBfwMainEntry(i) );
	
	num = sprintf(value, "{ \"mac\":\"%s\", \"ip\":\"%s\", \"state\":\"%s\"}", mac_buf, ip_buf, state_buf);

	if(num > 0){	
    	    websWrite(wp, T(value));
            isFirst = 0;
        }
	else
	    continue;
    } // for	
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwDaiDhcp(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[256], item_value[256], value[512], *group_name;
    char dhcpMacAddr_buf[256], dhcpIpAddr_buf[256], remark_buf[256];
    int i = 1, tbl_size, num, isFirst = 1;

    if (ejArgs(argc, argv, T("%s"), &group_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "dtbl_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp,T("["));
    for (i = 0; i < tbl_size; i++) {
	if (isFirst == 0)
	    websWrite(wp, T(","));
        
        sprintf(str, "dhcp_entry:%d.dhcpMacAddr", i);
	get_cfg_item_by_str(cipcfg, group_name, str, dhcpMacAddr_buf);
        sprintf(str, "dhcp_entry:%d.dhcpIpAddr", i);
	get_cfg_item_by_str(cipcfg, group_name, str, dhcpIpAddr_buf);
        sprintf(str, "dhcp_entry:%d.remark", i);
	get_cfg_item_by_str(cipcfg, group_name, str, remark_buf);
 	 
	
	num = sprintf(value, "{ \"dhcpMacAddr\":\"%s\", \"dhcpIpAddr\":\"%s\", \"remark\":\"%s\"}", dhcpMacAddr_buf, dhcpIpAddr_buf, remark_buf);

	if(num > 0){	
    	    websWrite(wp, T(value));
            isFirst = 0;
        }
	else
	    continue;
    } // for	
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef LAYER2_FIREWALL
char * genBfwIPv4Entry(int rule_id, char* value)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], group_name[MAX_VALUE_LEN];
    char sourceIp_buf[MAX_VALUE_LEN], sourceIpMask_buf[MAX_VALUE_LEN], destIp_buf[MAX_VALUE_LEN], destIpMask_buf[MAX_VALUE_LEN];
    char protocol_buf[MAX_VALUE_LEN], service_buf[MAX_VALUE_LEN], sourcePort_buf[MAX_VALUE_LEN], destPort_buf[MAX_VALUE_LEN];	
    	
    int tbl_size, num;
    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();

	//		IPV4			
	
	sprintf(str, "bfw_entry:%d.IPv4.protocol", rule_id); 
        get_cfg_item_by_str(cipcfg, group_name, str, protocol_buf);
	
	sprintf(str, "bfw_entry:%d.IPv4.service", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, service_buf);
	
	sprintf(str, "bfw_entry:%d.IPv4.sIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sourceIp_buf);
	
	sprintf(str, "bfw_entry:%d.IPv4.dIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, destIp_buf);

	sprintf(str, "bfw_entry:%d.IPv4.sPort", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sourcePort_buf);
	
	sprintf(str, "bfw_entry:%d.IPv4.dPort", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, destPort_buf);
        
	sprintf(str, "bfw_entry:%d.IPv4.sIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sourceIpMask_buf);

	sprintf(str, "bfw_entry:%d.IPv4.dIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, destIpMask_buf);
        

	num = sprintf(value, "\"protocol\":\"%s\", \"service\":\"%s\", \"sIp\":\"%s\", \"sIpMask\":\"%s\", \"dIp\":\"%s\", \"dIpMask\":\"%s\" ",protocol_buf, service_buf, sourceIp_buf, sourceIpMask_buf, destIp_buf, destIpMask_buf);

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
char * genBfw8023Entry(int rule_id, char* value)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], group_name[MAX_VALUE_LEN];
    char sap_buf[MAX_VALUE_LEN], type_buf[MAX_VALUE_LEN];
    	
    int  tbl_size, num;
    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();
        
	//  		IEEE_8023			

	sprintf(str, "bfw_entry:%d.IEEE_8023.sap", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sap_buf);
        
	sprintf(str, "bfw_entry:%d.IEEE_8023.type", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, type_buf);


	num = sprintf(value, "\"sap\":\"%s\", \"type\":\"%s\"   ",sap_buf, type_buf);

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
char * genBfwVlanEntry(int rule_id, char* value)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], group_name[MAX_VALUE_LEN];
    char vlanId_buf[MAX_VALUE_LEN], priority_buf[MAX_VALUE_LEN], vType_buf[MAX_VALUE_LEN];
    	
    int tbl_size, num;
    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();

        
	//  		VLAN			
	
	sprintf(str, "bfw_entry:%d.VLAN.id", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, vlanId_buf);
         
	sprintf(str, "bfw_entry:%d.VLAN.priority", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, priority_buf);
         
	sprintf(str, "bfw_entry:%d.VLAN.vType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, vType_buf);
	
	num = sprintf(value, "\"id\":\"%s\", \"priority\":\"%s\", \"vType\":\"%s\" ",vlanId_buf, priority_buf, vType_buf);

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
char * genBfwArpEntry(int rule_id, char* value)
{
    printf("Arp entry  \n");
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], group_name[MAX_VALUE_LEN];
    char arpOpcode_buf[MAX_VALUE_LEN], arpHType_buf[MAX_VALUE_LEN], arpPType_buf[MAX_VALUE_LEN], arpSIp_buf[MAX_VALUE_LEN]; 
    char arpDIp_buf[MAX_VALUE_LEN], arpSMac_buf[MAX_VALUE_LEN], arpDMac_buf[MAX_VALUE_LEN];
    char sArpIpMask_buf[MAX_VALUE_LEN], dArpIpMask_buf[MAX_VALUE_LEN], sArpMacMask_buf[MAX_VALUE_LEN], dArpMacMask_buf[MAX_VALUE_LEN];	
    int tbl_size, num;

    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();
	//  		ARP			
	
	sprintf(str, "bfw_entry:%d.ARP.opcode", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpOpcode_buf);
         
	sprintf(str, "bfw_entry:%d.ARP.hType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpHType_buf);
        
	sprintf(str, "bfw_entry:%d.ARP.pType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpPType_buf);
	
	sprintf(str, "bfw_entry:%d.ARP.sArpIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpSIp_buf);

	sprintf(str, "bfw_entry:%d.ARP.dArpIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpDIp_buf);

	sprintf(str, "bfw_entry:%d.ARP.sArpIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sArpIpMask_buf);

	sprintf(str, "bfw_entry:%d.ARP.dArpIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, dArpIpMask_buf);

	sprintf(str, "bfw_entry:%d.ARP.sArpMac", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpSMac_buf);

	sprintf(str, "bfw_entry:%d.ARP.dArpMac", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, arpDMac_buf);
	
	sprintf(str, "bfw_entry:%d.ARP.sArpMacMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sArpMacMask_buf);
	
	sprintf(str, "bfw_entry:%d.ARP.dArpMacMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, dArpMacMask_buf);


	num = sprintf(value, "\"opcode\":\"%s\", \"hType\":\"%s\", \"pType\":\"%s\", \"sArpIp\":\"%s\", \"sArpIpMask\":\"%s\", \"sArpMac\":\"%s\", \"sArpMacMask\":\"%s\", \"dArpIp\":\"%s\", \"dArpIpMask\":\"%s\", \"dArpMac\":\"%s\", \"dArpMacMask\":\"%s\"  ",arpOpcode_buf, arpHType_buf, arpPType_buf, arpSIp_buf, sArpIpMask_buf, arpSMac_buf, sArpMacMask_buf, arpDIp_buf, dArpIpMask_buf, arpDMac_buf, dArpMacMask_buf );

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
char * genBfwRarpEntry(int rule_id, char* value)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], group_name[MAX_VALUE_LEN];
    char rOpcode_buf[MAX_VALUE_LEN], rHType_buf[MAX_VALUE_LEN], rPType_buf[MAX_VALUE_LEN];
    char rSIp_buf[MAX_VALUE_LEN], rDIp_buf[MAX_VALUE_LEN], rSMac_buf[MAX_VALUE_LEN], rDMac_buf[MAX_VALUE_LEN];
    char sRarpIpMask_buf[MAX_VALUE_LEN], dRarpIpMask_buf[MAX_VALUE_LEN], sRarpMacMask_buf[MAX_VALUE_LEN], dRarpMacMask_buf[MAX_VALUE_LEN];	
    	
    int tbl_size, num;
    sprintf(group_name,"bfirewall");
    cipcfg=init_cipcfg();


	//  		RARP		
	
	sprintf(str, "bfw_entry:%d.RARP.opcode", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rOpcode_buf);
         
	sprintf(str, "bfw_entry:%d.RARP.hType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rHType_buf);
        
	sprintf(str, "bfw_entry:%d.RARP.pType", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rPType_buf);
	
	sprintf(str, "bfw_entry:%d.RARP.sRarpIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rSIp_buf);

	sprintf(str, "bfw_entry:%d.RARP.dRarpIp", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rDIp_buf);

	sprintf(str, "bfw_entry:%d.RARP.sRarpIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sRarpIpMask_buf);

	sprintf(str, "bfw_entry:%d.RARP.dRarpIpMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, dRarpIpMask_buf);

	sprintf(str, "bfw_entry:%d.RARP.sRarpMac", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rSMac_buf);

	sprintf(str, "bfw_entry:%d.RARP.dRarpMac", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, rDMac_buf);
	
	sprintf(str, "bfw_entry:%d.RARP.sRarpMacMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, sRarpMacMask_buf);
	
	sprintf(str, "bfw_entry:%d.RARP.dRarpMacMask", rule_id);
        get_cfg_item_by_str(cipcfg, group_name, str, dRarpMacMask_buf);


	num = sprintf(value, "\"opcode\":\"%s\", \"hType\":\"%s\", \"pType\":\"%s\", \"sRarpIp\":\"%s\", \"sRarpIpMask\":\"%s\", \"sRarpMac\":\"%s\", \"sRarpMacMask\":\"%s\", \"dRarpIp\":\"%s\", \"dRarpIpMask\":\"%s\", \"dRarpMac\":\"%s\", \"dRarpMacMask\":\"%s\"  ",rOpcode_buf, rHType_buf, rPType_buf, rSIp_buf, sRarpIpMask_buf, rSMac_buf, sRarpMacMask_buf, rDIp_buf, dRarpIpMask_buf, rDMac_buf, dRarpMacMask_buf );

    close_cipcfg(cipcfg);
    return value;
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwService(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], value[512], *group_name;
    char name_buf[MAX_VALUE_LEN], protocol_buf[MAX_VALUE_LEN];
    char sport_buf[MAX_VALUE_LEN], dport_buf[MAX_VALUE_LEN], *uDefined_buf;
    int i = 1, tbl_size, num, isFirst = 1;

    if (ejArgs(argc, argv, T("%s"), &group_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    websWrite(wp,T("["));

    for(i=0; i<predefined_service_table_size; i++){
	sprintf(name_buf,"%s",predefined_service_table[i].name);
	sprintf(protocol_buf,"%s",predefined_service_table[i].protocol);
	sprintf(sport_buf,"%s",predefined_service_table[i].sport);
	sprintf(dport_buf,"%s",predefined_service_table[i].dport);
	uDefined_buf="false";
	
	if (isFirst == 0)
	    websWrite(wp, T(","));
	
	num = sprintf(value, "{ \"name\":\"%s\", \"protocol\":\"%s\", \"sport\":\"%s\", \"dport\":\"%s\", \"uDefined\":\"%s\"}", name_buf, protocol_buf, sport_buf, dport_buf, uDefined_buf);

	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else 
	    continue;
    } // for



    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "table_size", item_value);
    tbl_size = atoi(item_value);
    for (i = 0; i < tbl_size; i++) {
        sprintf(str, "uds_entry:%d.name", i);
        get_cfg_item_by_str(cipcfg, group_name, str, name_buf);
	if(!strcmp(name_buf,"")){ 
	    continue;
	}

	sprintf(str, "uds_entry:%d.protocol", i);
        get_cfg_item_by_str(cipcfg, group_name, str, protocol_buf);
        
	sprintf(str, "uds_entry:%d.sport", i);
        get_cfg_item_by_str(cipcfg, group_name, str, sport_buf);
        
	sprintf(str, "uds_entry:%d.dport", i);
        get_cfg_item_by_str(cipcfg, group_name, str, dport_buf);
	
	
	uDefined_buf="true";
 	
	if (isFirst == 0)
	    websWrite(wp, T(","));

	num = sprintf(value, "{ \"name\":\"%s\", \"protocol\":\"%s\", \"sport\":\"%s\", \"dport\":\"%s\", \"uDefined\":\"%s\"}", name_buf, protocol_buf, sport_buf, dport_buf, uDefined_buf);
	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else 
	    continue;
    } // for
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
    
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwServiceAdd(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], value[512], *group_name;
    char name_buf[MAX_VALUE_LEN];
    int i = 1, tbl_size, num, isFirst = 1;

    if (ejArgs(argc, argv, T("%s"), &group_name) < 1) {
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }

    websWrite(wp,T("["));

    for(i=0; i<predefined_service_table_size; i++){
	sprintf(name_buf,"%s",predefined_service_table[i].name);
	
	if (isFirst == 0)
	    websWrite(wp, T(","));
	
	num = sprintf(value, "{ \"name\":\"%s\"}", name_buf);

	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else 
	    continue;
    } // for


    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "table_size", item_value);
    tbl_size = atoi(item_value);
    for (i = 0; i < tbl_size; i++) {
        sprintf(str, "uds_entry:%d.name", i);
        get_cfg_item_by_str(cipcfg, group_name, str, name_buf);
	if(!strcmp(name_buf,"")){ 
	    continue;
	}
	if (isFirst == 0)
	    websWrite(wp, T(","));

	num = sprintf(value, "{ \"name\":\"%s\"}", name_buf);
	if (num > 0) {
	    websWrite(wp, T(value));
	    isFirst = 0;
	} else 
	    continue;
    } // for
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef LAYER2_FIREWALL
ITEM_LIST * shiftBfw(int source,int dest)
{
    char_t *tmp,str[256],tmp_buf[256],group_name[256];
    ITEM_LIST *list=NULL;
    int ret,i=0,x=0,y=0;
    int bfw_gid = GID_BFW;
    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    sprintf(group_name,"bfirewall");
    i=source;


    //printf("before while i=%d source=%d dest=%d \n",i,source,dest);
    while(1){
	if(source<dest){
	    x=i;
	    y=i+1;
	    //printf("i=%d,x=%d,y=%d \n",i,x,y);
	    if(i==dest-1){
		x=dest-1;
		y=source;
	    } 
	    else if(i>dest-1){
		break;
	    }
	    i++;
	}
	else{
	    x=i;
	    y=i-1;
	    //printf("i=%d,x=%d,y=%d \n",i,x,y);
	    if(i==dest){
		y=source;
	    }
	    else if(i<dest){
		break;	
	    }
	    i--;
	}
		sprintf(str, "bfw_entry:%d.enabled", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.enabled", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);      

		sprintf(str, "bfw_entry:%d.action", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.action", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);      
 
		sprintf(str, "bfw_entry:%d.rule_name", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.rule_name", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.etherType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.etherType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.sInterface", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.sInterface", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.dInterface", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.dInterface", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.sMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.sMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.dMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.dMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.sMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.sMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.dMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.dMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.remark", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.remark", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.IPv4.protocol", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.protocol", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.service", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.service", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.sIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.sIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.dIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.dIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.sIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.sIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.dIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.dIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.sPort", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.sPort", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IPv4.dPort", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IPv4.dPort", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.IEEE_8023.sap", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IEEE_8023.sap", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.IEEE_8023.type", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.IEEE_8023.type", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       

		sprintf(str, "bfw_entry:%d.VLAN.id", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);	
		sprintf(str, "bfw_entry:%d.VLAN.id", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.VLAN.priority", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.VLAN.priority", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.VLAN.vType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.VLAN.vType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.opcode", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.opcode", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.hType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.hType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.pType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.pType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.sArpIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.sArpIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.dArpIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.dArpIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.sArpIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.sArpIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.dArpIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.dArpIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.sArpMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.sArpMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.dArpMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.dArpMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
	
		sprintf(str, "bfw_entry:%d.ARP.sArpMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.sArpMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.ARP.dArpMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.ARP.dArpMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
	
		sprintf(str, "bfw_entry:%d.RARP.opcode", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.opcode", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.hType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.hType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.pType", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.pType", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.sRarpIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.sRarpIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.dRarpIp", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.dRarpIp", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.sRarpIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.sRarpIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.dRarpIpMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.dRarpIpMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);       
		
		sprintf(str, "bfw_entry:%d.RARP.sRarpMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.sRarpMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);  
		     
		sprintf(str, "bfw_entry:%d.RARP.dRarpMac", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.dRarpMac", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);      
			
		sprintf(str, "bfw_entry:%d.RARP.sRarpMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.sRarpMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);  
		     
		sprintf(str, "bfw_entry:%d.RARP.dRarpMacMask", y);
        	get_cfg_item_by_str(cipcfg, group_name, str, tmp_buf);
		sprintf(str, "bfw_entry:%d.RARP.dRarpMacMask", x);
		list=append_cfg_item(list, bfw_gid, str, tmp_buf);      
			
		
	}

    print_list(list);

    close_cipcfg(cipcfg);
    return list;
}
#endif

#ifdef LAYER2_FIREWALL
void applyWlanBfwList(webs_t wp, char_t *path, char_t *query)
{
    char_t *bfw_type;
    char_t *enabled;
    char_t *app;
    ITEM_LIST *list= NULL, *tmp_list= NULL;
    CIPCFG *cipcfg = NULL;
    cipcfg=init_cipcfg();
    int ret, i, bfw_gid;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int tbl_size=0, delete_rule, rule_index;
    memset(tmp, 0, 256);

    app = websGetVar(wp, T("apply"), T("true"));

    bfw_gid = GID_BFW;
    get_cfg_item_by_str(cipcfg, "bfirewall", "table_size", tmp);
    tbl_size = atoi(tmp);
     
    delete_rule = atoi(websGetVar(wp, T("delete_flag"), T("-1")));
    if(delete_rule==-1){ 
   
        bfw_type = websGetVar(wp, T("Bfw_Type"), T("disable"));
        if (!strcmp(bfw_type, "disable")) {
 	    list=append_cfg_item(list, bfw_gid, "state", "disable");
        }
    	else if ( !strcmp(bfw_type, "enable")) {
	    list=append_cfg_item(list, bfw_gid, "state", "enable");
			
    	    rule_index = atoi(websGetVar(wp, T("rule_index"), T("0")));
	    for (i=rule_index; i<rule_index+10; i++) {
	    	sprintf(tmp, "enabled%d", i);
	    	enabled = websGetVar(wp, T(tmp), T("nofound"));
	    	if(strcmp(enabled,"nofound")==0){
		    continue;
	    	}
	    	sprintf(tmp, "bfw_entry:%d.enabled", i);
	    	list=append_cfg_item(list, bfw_gid, tmp, enabled);      // put enabled 
		
       	    }//for
        }
    }// if
    else{   //delete rule
		
        close_cipcfg(cipcfg);
	list=shiftBfw(delete_rule,tbl_size);
    	cipcfg=init_cipcfg();
	i=tbl_size-1;
//	list=NULL;

	sprintf(tmp, "bfw_entry:%d.enabled", i);
	list=append_cfg_item(list, bfw_gid, tmp, "disable");      

	sprintf(tmp, "bfw_entry:%d.action", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");      
 
	sprintf(tmp, "bfw_entry:%d.rule_name", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.etherType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.sInterface", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.dInterface", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.sMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.dMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.sMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.dMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.remark", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
       
	sprintf(tmp, "bfw_entry:%d.IPv4.protocol", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.service", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.sIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.dIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.sIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.dIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.sPort", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IPv4.dPort", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.IEEE_8023.sap", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.IEEE_8023.type", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       

	sprintf(tmp, "bfw_entry:%d.VLAN.id", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.VLAN.priority", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.VLAN.vType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.opcode", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.hType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.pType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.sArpIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.dArpIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.sArpIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.dArpIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.sArpMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.dArpMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
	
	sprintf(tmp, "bfw_entry:%d.ARP.sArpMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.ARP.dArpMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
	
	sprintf(tmp, "bfw_entry:%d.RARP.opcode", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.hType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.pType", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.sRarpIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.dRarpIp", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.sRarpIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.dRarpIpMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.sRarpMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.dRarpMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
	
	sprintf(tmp, "bfw_entry:%d.RARP.sRarpMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
		
	sprintf(tmp, "bfw_entry:%d.RARP.dRarpMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, "");       
	

    }// else
    print_list(list);


    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("set cfg item fail ! ret = %d \n",ret);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;

        }
	tmp_list = list;
        list = list->next;
	free(tmp_list);
    }
    applylist[applist_idx] = GID_END;
    // check if apply or just save settings into config file.
   
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    close_cipcfg(cipcfg);
    
    websRedirect(wp, "/firewall/bfirewall.asp"); 
}
#endif

#ifdef LAYER2_FIREWALL
void applyWlanBfwMove(webs_t wp, char_t *path, char_t *query)
{
    char_t *bfw_type;
    char_t *enabled;
    char_t *app;
    ITEM_LIST *list= NULL, *tmp_list;
    CIPCFG *cipcfg = NULL;
    int i, ret;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int source_rule, dest_rule, rule_index;
	
    memset(tmp, 0, 256);

    app = websGetVar(wp, T("apply"), T("true"));

    //bfw_gid = GID_BFW;
    source_rule = atoi(websGetVar(wp, T("source"), T("-1")))-1;
    //printf("source rule is %d \n",source_rule);
    dest_rule = atoi(websGetVar(wp, T("dest"), T("-1")))-1;
   
    list=shiftBfw(source_rule,dest_rule);
    print_list(list);

    cipcfg=init_cipcfg();

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("set cfg item fail ! ret = %d \n",ret);
        return ;
    }

    while(list != NULL){
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0){
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
	tmp_list= list;
        list = list->next;
	free(tmp_list);
    }
    applylist[applist_idx] = GID_END;
    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }
    if(source_rule>dest_rule){
    	rule_index=dest_rule;
    }
    else{ 
	rule_index=dest_rule-1;    
    }
    sprintf(tmp, "/firewall/bfirewall.asp?rule=%d",rule_index+1);
    close_cipcfg(cipcfg);
    websRedirect(wp,tmp); 
}
#endif

#ifdef LAYER2_FIREWALL
void applyBfwDAI(webs_t wp, char_t *path, char_t *query)
{
    char_t *block_dhcp, *block_arp, *Snooping, *Inspection, *enhancement, *privilege, *proxyARP;
    char_t *mac, *ip, *state, *dhcpIpAddr, *dhcpMacAddr, *remark, *ForceDhcp;
    char_t *app, *vapState;
    ITEM_LIST *list= NULL, *tmp_list;
    CIPCFG *cipcfg = NULL;
    int i, ret, bfw_gid, tbl_size;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
	
    memset(tmp, 0, 256);
    cipcfg=init_cipcfg();

    app = websGetVar(wp, T("apply"), T("true"));
    bfw_gid = GID_BFW_DAI;
    
    vapState = websGetVar(wp, T("vapInterfaceState"), T("00000000"));     
    list=append_cfg_item(list, bfw_gid, "trust_interface.entry:0.vap", vapState);      // put vap state 
    
    proxyARP = websGetVar(wp, T("proxy_arp"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "proxy_arp", proxyARP);      // put proxy_arp 

    Snooping = websGetVar(wp, T("dhcp_snooping"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "Snooping", Snooping);      // put Snooping 
    
    Inspection = websGetVar(wp, T("arp_inspection"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "Inspection", Inspection);      // put Inspection 
   
    enhancement = websGetVar(wp, T("enhancement"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "enhancement", enhancement);      // put enhancement 
    
    privilege = websGetVar(wp, T("privilege"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "privilege", privilege);      // put Inspection 
    
    ForceDhcp = websGetVar(wp, T("dhcp_force"), T("disable"));     
    list=append_cfg_item(list, bfw_gid, "force_dhcp", ForceDhcp);      // put force_dhcp 
    
    
    if(strcmp(privilege,"enable")==0){
   					   
        get_cfg_item_by_str(cipcfg, "bfw_dai", "ptbl_size", tmp);
        tbl_size = atoi(tmp);
    	for (i=0; i<tbl_size; i++) {
	    sprintf(tmp, "mac%d", i);
	    mac = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "privilege_entry:%d.mac", i);
	    list=append_cfg_item(list, bfw_gid, tmp, mac);      // put mac 
            
	    sprintf(tmp, "ip%d", i);
	    ip = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "privilege_entry:%d.ip", i);
	    list=append_cfg_item(list, bfw_gid, tmp, ip);      // put ip 
	    
	    sprintf(tmp, "state%d", i);
	    state = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "privilege_entry:%d.state", i);
	    list=append_cfg_item(list, bfw_gid, tmp, state);      // put state 
        }	
    } // if privilege enable
    
    // dhcp table    
    if(strcmp(Snooping,"enable")==0){
	    
        get_cfg_item_by_str(cipcfg, "bfw_dai", "dtbl_size", tmp);
        tbl_size = atoi(tmp);
    	for (i=0; i<tbl_size; i++) {
	    sprintf(tmp, "dhcpMacAddr%d", i);
	    dhcpMacAddr = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "dhcp_entry:%d.dhcpMacAddr", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dhcpMacAddr);      // put dhcp Mac address 
            
	    sprintf(tmp, "dhcpIpAddr%d", i);
	    dhcpIpAddr = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "dhcp_entry:%d.dhcpIpAddr", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dhcpIpAddr);      // put dhcp Ip Addr 
	    
	    sprintf(tmp, "remark%d", i);
	    remark = websGetVar(wp, T(tmp), T(""));
	    sprintf(tmp, "dhcp_entry:%d.remark", i);
	    list=append_cfg_item(list, bfw_gid, tmp, remark);      // put remark 
        }
    }//dhcp snooping	
    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("set cfg item fail ! ret = %d \n",ret);
        return ;
    }

    while(list != NULL){
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0){
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
	tmp_list= list;
        list = list->next;
	free(tmp_list);
    }
    applylist[applist_idx] = GID_END;
    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }
    sprintf(tmp, "/firewall/DAI.asp");
    close_cipcfg(cipcfg);
    websRedirect(wp,tmp); 
}
#endif

#ifdef LAYER2_FIREWALL
void applyWlanBfwEdit(webs_t wp, char_t *path, char_t *query)
{
	
    char_t *action, *rule_name, *etherType, *sInterface, *dInterface, *sMac, *dMac, *sMacMask, *dMacMask, *remark ;
    char_t *sIp, *dIp, *sIpMask, *dIpMask, *protocol, *service, *sPort, *dPort;
    char_t *sap, *type;
    char_t *vlanId, *priority, *vType;
    char_t *opcode, *hType, *pType, *sArpIp, *dArpIp, *sArpIpMask, *dArpIpMask, *sArpMac, *dArpMac, *sArpMacMask, *dArpMacMask;
    char_t *app;
    ITEM_LIST *list= NULL, *tmp_list;
    CIPCFG *cipcfg = NULL;
    int ret, vap_gid, acl_gid, sec_gid, i, bfw_gid;
    char tmp[256];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int tbl_size, rule_index, insert_index;
    memset(tmp, 0, 256);

    app = websGetVar(wp, T("apply"), T("true"));

    bfw_gid = GID_BFW;
    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, "bfirewall", "table_size", tmp);
    tbl_size = atoi(tmp);
     
    rule_index = atoi(websGetVar(wp, T("rule_index"), T(-1)));
    insert_index = atoi(websGetVar(wp, T("insert_index"), T(-1)));
    rule_index--;
  
    if(insert_index!=-1){  //need to shift the rule
    	//printf("call shift(%d,%d) \n",tbl_size-1, rule_index);
        close_cipcfg(cipcfg);
    	list= shiftBfw( tbl_size-1, rule_index);
        cipcfg=init_cipcfg();
	ret=set_cfg_items(cipcfg, list);
    	if (ret != CIPCFG_SUCCESS) {
	    printf("set cfg item fail ! ret = %d \n",ret);
	    return ;
        }
						
    }
    	i=rule_index;
	// main entry
	action = websGetVar(wp, T("action"), T("disable"));
	sprintf(tmp, "bfw_entry:%d.action", i);
	list=append_cfg_item(list, bfw_gid, tmp, action);      // put action 

	rule_name = websGetVar(wp, T("rule_name"), T(""));
	sprintf(tmp, "bfw_entry:%d.rule_name", i);
	list=append_cfg_item(list, bfw_gid, tmp, rule_name);      // put rule name

	etherType = websGetVar(wp, T("etherTypeList"), T(""));
	sprintf(tmp, "bfw_entry:%d.etherType", i);
	list=append_cfg_item(list, bfw_gid, tmp, etherType);      // put etherType 
		
	sInterface = websGetVar(wp, T("sInterfaceList"), T(""));
	sprintf(tmp, "bfw_entry:%d.sInterface", i);
	list=append_cfg_item(list, bfw_gid, tmp, sInterface);      // put source interface 
		
	dInterface = websGetVar(wp, T("dInterfaceList"), T(""));
	sprintf(tmp, "bfw_entry:%d.dInterface", i);
	list=append_cfg_item(list, bfw_gid, tmp, dInterface);      // put dest interface 
		
	sMac = websGetVar(wp, T("sMac"), T(""));
	sprintf(tmp, "bfw_entry:%d.sMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, sMac);      // put source Mac 
		
	dMac = websGetVar(wp, T("dMac"), T(""));
	sprintf(tmp, "bfw_entry:%d.dMac", i);
	list=append_cfg_item(list, bfw_gid, tmp, dMac);      // put dest Mac

	sMacMask = websGetVar(wp, T("sMacMask"), T(""));
	sprintf(tmp, "bfw_entry:%d.sMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, sMacMask);      // put source Mac Mask
		
	dMacMask = websGetVar(wp, T("dMacMask"), T(""));
	sprintf(tmp, "bfw_entry:%d.dMacMask", i);
	list=append_cfg_item(list, bfw_gid, tmp, dMacMask);      // put dest Mac Mask

	remark = websGetVar(wp, T("remark"), T(""));
	sprintf(tmp, "bfw_entry:%d.remark", i);
	list=append_cfg_item(list, bfw_gid, tmp, remark);      // put remark 

        // IPv4
	if(!strcmp(etherType,"IPv4")){
		
	    protocol = websGetVar(wp, T("protocol"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.protocol", i);
	    list=append_cfg_item(list, bfw_gid, tmp, protocol);      // put protocol 
	
            service = websGetVar(wp, T("serviceList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.service", i);
	    list=append_cfg_item(list, bfw_gid, tmp, service);       // put service      

	    sIp = websGetVar(wp, T("sIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.sIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sIp);       // put sIp

	    dIp = websGetVar(wp, T("dIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.dIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dIp);       // put dIp

	    sIpMask = websGetVar(wp, T("sIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.sIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sIpMask);       // put sIpMask

	    dIpMask = websGetVar(wp, T("dIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.dIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dIpMask);       // put dIpMask

	    sPort = websGetVar(wp, T("sPort"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.sPort", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sPort);      // put sPort 

	    dPort = websGetVar(wp, T("dPort"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IPv4.dPort", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dPort);      // put dPort 
	} // eof IPv4
	else if(!strcmp(etherType,"IEEE_8023")){

	    sap = websGetVar(wp, T("sap"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IEEE_8023.sap", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sap);      // put sap

	    type = websGetVar(wp, T("type"), T(""));
	    sprintf(tmp, "bfw_entry:%d.IEEE_8023.type", i);
	    list=append_cfg_item(list, bfw_gid, tmp, type);      // put type 
	} // eof IEEE 802.3
	else if(!strcmp(etherType,"VLAN")){
		
            vlanId = websGetVar(wp, T("vlanId"), T(""));
	    sprintf(tmp, "bfw_entry:%d.VLAN.id", i);
	    list=append_cfg_item(list, bfw_gid, tmp, vlanId);      // put valn Id 
		
	    priority = websGetVar(wp, T("priority"), T(""));
	    sprintf(tmp, "bfw_entry:%d.VLAN.priority", i);
	    list=append_cfg_item(list, bfw_gid, tmp, priority);      // put valn priority 
		
	    vType = websGetVar(wp, T("vType"), T(""));
	    sprintf(tmp, "bfw_entry:%d.VLAN.vType", i);
	    list=append_cfg_item(list, bfw_gid, tmp, vType);      // put valn type
	}// eof VLAN	
	else if(!strcmp(etherType,"ARP")){
	    opcode = websGetVar(wp, T("opcodeList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.opcode", i);
	    list=append_cfg_item(list, bfw_gid, tmp, opcode);      // put opcode 
		
	    hType = websGetVar(wp, T("hType"), T(""));
 	    sprintf(tmp, "bfw_entry:%d.ARP.hType", i);
	    list=append_cfg_item(list, bfw_gid, tmp, hType);      // put hType 
		
	    pType = websGetVar(wp, T("pType"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.pType", i);
	    list=append_cfg_item(list, bfw_gid, tmp, pType);      // put pType 
		
	    sArpIp = websGetVar(wp, T("sArpIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.sArpIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpIp);      // put source ARP Ip 
		
	    sArpIpMask = websGetVar(wp, T("sArpIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.sArpIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpIpMask);      // put source ARP Ip Mask 
		
	    dArpIp = websGetVar(wp, T("dArpIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.dArpIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpIp);      // put dest ARP Ip 
		
	    dArpIpMask = websGetVar(wp, T("dArpIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.dArpIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpIpMask);      // put dest ARP Ip Mask 
		
	    sArpMac = websGetVar(wp, T("sArpMac"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.sArpMac", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpMac);      // put source Mac 
		
	    dArpMac = websGetVar(wp, T("dArpMac"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.dArpMac", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpMac);      // put dest Mac

	    sArpMacMask = websGetVar(wp, T("sArpMacMask"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.sArpMacMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpMacMask);      // put source Mac Mask
		
	    dArpMacMask = websGetVar(wp, T("dArpMacMask"), T(""));
	    sprintf(tmp, "bfw_entry:%d.ARP.dArpMacMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpMacMask);      // put dest Mac Mask
	} //eof ARP
	else if(!strcmp(etherType,"RARP")){
	    opcode = websGetVar(wp, T("opcodeList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.opcode", i);
	    list=append_cfg_item(list, bfw_gid, tmp, opcode);      // put opcode 
		
	    hType = websGetVar(wp, T("hType"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.hType", i);
	    list=append_cfg_item(list, bfw_gid, tmp, hType);      // put hType 
		
	    pType = websGetVar(wp, T("pType"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.pType", i);
	    list=append_cfg_item(list, bfw_gid, tmp, pType);      // put pType 
		
	    sArpIp = websGetVar(wp, T("sArpIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.sRarpIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpIp);      // put source ARP Ip 
		
	    sArpIpMask = websGetVar(wp, T("sArpIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.sRarpIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpIpMask);      // put source ARP Ip Mask 
		
	    dArpIp = websGetVar(wp, T("dArpIp"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.dRarpIp", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpIp);      // put dest ARP Ip 
		
	    dArpIpMask = websGetVar(wp, T("dArpIpMaskList"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.dRarpIpMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpIpMask);      // put dest ARP Ip Mask 
		
	    sArpMac = websGetVar(wp, T("sArpMac"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.sRarpMac", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpMac);      // put source Mac 
		
	    dArpMac = websGetVar(wp, T("dArpMac"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.dRarpMac", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpMac);      // put dest Mac

	    sArpMacMask = websGetVar(wp, T("sArpMacMask"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.sRarpMacMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, sArpMacMask);      // put source Mac Mask
		
	    dArpMacMask = websGetVar(wp, T("dArpMacMask"), T(""));
	    sprintf(tmp, "bfw_entry:%d.RARP.dRarpMacMask", i);
	    list=append_cfg_item(list, bfw_gid, tmp, dArpMacMask);      // put dest Mac Mask
	} //eof ARP


    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("set cfg item fail ! ret = %d \n",ret);
        return ;
    }
    
    while(list != NULL){
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0){
           applylist[applist_idx] = list->gid;
           applist_idx++;

        }
	tmp_list = list;
        list = list->next;
        free(tmp_list);
    }
    applylist[applist_idx] = GID_END;
    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }

    sprintf(tmp, "/firewall/bfirewall.asp?rule=%d",rule_index+1);
    close_cipcfg(cipcfg);

    websRedirect(wp, 
	    websGetVar(wp, T(tmp), T(tmp))); 
}
#endif

#ifdef LAYER2_FIREWALL
void applyBfwService(webs_t wp, char_t *path, char_t *query)
{

    char_t *name, *protocol, *sport, *dport;
    char_t code_buf[MAX_VALUE_LEN], type_buf[MAX_VALUE_LEN];
    char_t sport_buf[MAX_VALUE_LEN], dport_buf[MAX_VALUE_LEN];
    char_t *app, name_buf[MAX_VALUE_LEN], protocol_buf[MAX_VALUE_LEN];
    char_t *DelSvcName;
    char_t *service;	                   	
    char_t *save_tok;	                   	
    ITEM_LIST *list= NULL, *tmp_list;
    CIPCFG *cipcfg = NULL;
    int ret, i, j, uds_gid;
    char tmp[MAX_VALUE_LEN];
    int *applylist = malloc(sizeof(int) * (SMALL_BUF_LEN+1));
    int applist_idx = 0;
    int tbl_size, tbl_index=0, target=0;
    int sort_ary[10]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};	
    int mark_ary[10]={-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};	
    memset(tmp, 0, 256);
    app = websGetVar(wp, T("apply"), T("true"));
    uds_gid = GID_BFW_SERVICE;
    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, "bfw_service", "table_size", tmp);
    tbl_size = atoi(tmp);
    
    DelSvcName = websGetVar(wp, T("del_name"), T(""));          // get the service name which to remove  ie: "FTP;TCP;NTP" 
	
    if(strcmp(DelSvcName,"")!=0){                                 // some service has to remove  
	service=strtok_r(DelSvcName,";",&save_tok);
	while(service!=NULL){
	    for(j=0;j<tbl_size;j++){
	        sprintf(tmp, "uds_entry:%d.name", j);
       		get_cfg_item_by_str(cipcfg, "bfw_service", tmp, name_buf);
	        // mark_ary save the data that name!="" and not the delete target
		// ie:  mark_ary=  1 0 1 0 1 0 0 1 0 1 
		// the 2,4,5,6 and 8 entry will be remove
		if(strcmp(name_buf,"") && strcmp(service,name_buf) && mark_ary[j]!=0){           // prepare shift 
		    mark_ary[j]=1;
		}
		else {    
		    mark_ary[j]=0;
		}
	    }

	    service=strtok_r(NULL,";",&save_tok);
	}
	
	/*    
	 *    mark_ary=  1 0 1 0 1 0 0 1 0 1 
	 *       - - - - after shift - - - -
	 *    sort_ary=  0 2 4 7 9-1-1-1-1-1
	 */
	for(j=0;j<tbl_size;j++){                                       
	    if(mark_ary[j]==1 ){           //shift 
		sort_ary[target]=j;
		target++;
	    }	
	}
	//  sort the service list   
	for(i=0;i<tbl_size;i++){		 	
	    // sort_ary[i]==i means the entry i does not need to modify	 and sort_ary[i]==-1 means the entry i need to be remove
	    if(sort_ary[i]!=i && sort_ary[i]!=-1 ){ 			// need to shift service
		sprintf(tmp, "uds_entry:%d.name", sort_ary[i]);
       		get_cfg_item_by_str(cipcfg, "bfw_service", tmp, name_buf);
		sprintf(tmp, "uds_entry:%d.name", i);
		list=append_cfg_item(list, uds_gid, tmp,name_buf);         
			        
		sprintf(tmp, "uds_entry:%d.protocol", sort_ary[i]);
       		get_cfg_item_by_str(cipcfg, "bfw_service", tmp, protocol_buf);
		sprintf(tmp, "uds_entry:%d.protocol", i);
		list=append_cfg_item(list, uds_gid, tmp,protocol_buf);         
				
		sprintf(tmp, "uds_entry:%d.sport", sort_ary[i]);
       		get_cfg_item_by_str(cipcfg, "bfw_service", tmp, sport_buf);
		sprintf(tmp, "uds_entry:%d.sport", i);
		list=append_cfg_item(list, uds_gid, tmp,sport_buf);         
				
		sprintf(tmp, "uds_entry:%d.dport", sort_ary[i]);
       		get_cfg_item_by_str(cipcfg, "bfw_service", tmp, dport_buf);
		sprintf(tmp, "uds_entry:%d.dport", i);
	       	list=append_cfg_item(list, uds_gid, tmp,dport_buf);     
	    }	
     	    else if(sort_ary[i]==-1){					// remove this service
	       	sprintf(tmp, "uds_entry:%d.name", i);
		list=append_cfg_item(list, uds_gid, tmp,"");        
	    }
       	}//for
    }
    else{ 		// we got a service to save
	// step 1  find the place to put new service
	for(i=0; i < tbl_size; i++){
	    sprintf(tmp, "uds_entry:%d.name", i);
       	    get_cfg_item_by_str(cipcfg, "bfw_service", tmp, name_buf);
	    if(!strcmp(name_buf,"")){	
		target=i;
		break;	
	    }	
	}
	if(i==10){
	    printf("The service is full !\n");	
	    return;
	}
	// step 2  get the service value    ie:  name, protocol, sport, dport, type, code
	name = websGetVar(wp, T("name"), T(""));
	protocol = websGetVar(wp, T("protocol"), T(""));
	sport = websGetVar(wp, T("sport"), T(""));
	dport = websGetVar(wp, T("dport"), T(""));

        // step 3  append value to cipcfg
	sprintf(tmp, "uds_entry:%d.name", target);
	list=append_cfg_item(list, uds_gid, tmp, name);      // put name
 
	sprintf(tmp, "uds_entry:%d.protocol", target);
	list=append_cfg_item(list, uds_gid, tmp, protocol);      // put protocol 
		
	if (!strcmp(protocol, "UDP") || !strcmp(protocol, "TCP")) {
	    sprintf(tmp, "uds_entry:%d.sport", target);
	    list=append_cfg_item(list, uds_gid, tmp, sport);      // put sport 
		    
	    sprintf(tmp, "uds_entry:%d.dport", target);
	    list=append_cfg_item(list, uds_gid, tmp, dport);      // put dport 
	}
	
    }//else

    print_list(list);

    ret=set_cfg_items(cipcfg, list);
    if (ret != CIPCFG_SUCCESS) {
        printf("set cfg item fail ! ret = %d \n",ret);
        return ;
    }

    while(list != NULL)
    {
        if(chkGIDdup(applylist, applist_idx, list->gid) == 0)
        {
           applylist[applist_idx] = list->gid;
           applist_idx++;
        }
	tmp_list = list;
        list = list->next;
        free(tmp_list);
    }
    
    applylist[applist_idx] = GID_END;
    // check if apply or just save settings into config file.
    if ( !strncmp("true", app, 4 ) ){
	if(can_apply(applylist) == CIPCFG_SUCCESS){
            apply_cfg(cipcfg);
            unlink(PATH_SYNC_CFG);
            free(applylist);
        }else{
            close_cipcfg(cipcfg);
            websRedirect( wp, "/utilities/reboot.asp" );
            free(applylist);
            return;
        }
    }
    
    sprintf(tmp, "/firewall/bfirewallService.asp");
    close_cipcfg(cipcfg);

    websRedirect(wp, "/firewall/bfirewallService.asp");
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwEditEntry(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], value[512], group_name[MAX_VALUE_LEN], etherType_buf[MAX_VALUE_LEN];
    char *etherType;	
    int tbl_size, num;
    int rule_id;
    sprintf(group_name,"bfirewall");
    rule_id=atoi(argv[0]);
    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "table_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp,T("{"));
        
    sprintf(value,genBfwMainEntry(rule_id,value) );
    websWrite(wp, T(value));
    sprintf(str, "bfw_entry:%d.etherType", rule_id);
    get_cfg_item_by_str(cipcfg, group_name, str, etherType_buf);
 
    if(!strcmp("IPv4",etherType_buf)){	
    	websWrite(wp, T(","));
    	sprintf(value,genBfwIPv4Entry(rule_id,value) );
    	websWrite(wp, T(value));
    }
    else if(!strcmp("IEEE_8023",etherType_buf)){	
    	websWrite(wp, T(","));
    	sprintf(value,genBfw8023Entry(rule_id,value) );
    	websWrite(wp, T(value));
    }
    else if(!strcmp("VLAN",etherType_buf)){	
    	websWrite(wp, T(","));
    	sprintf(value,genBfwVlanEntry(rule_id,value) );
    	websWrite(wp, T(value));
    }
    else if(!strcmp("ARP",etherType_buf)){	
    	websWrite(wp, T(","));
    	sprintf(value,genBfwArpEntry(rule_id,value) );
    	websWrite(wp, T(value));
    }
    else if(!strcmp("RARP",etherType_buf)){	
    	websWrite(wp, T(","));
   	sprintf(value,genBfwRarpEntry(rule_id,value) );
    	websWrite(wp, T(value));
    }
    websWrite(wp,T("}"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif

#ifdef LAYER2_FIREWALL
int genBfwList(int eid, webs_t wp, int argc, char_t **argv)
{
    CIPCFG *cipcfg = NULL;
    char str[MAX_VALUE_LEN], item_value[MAX_VALUE_LEN], value[512], *group_name;
    int i = 1, tbl_size, num, isFirst = 1;
    int rule_index;
    if (ejArgs(argc, argv, T("%s %d"), &group_name, &rule_index) < 2){
	websError(wp, 400, T("Insufficient args\n"));
	return -1;
    }
    cipcfg=init_cipcfg();
    get_cfg_item_by_str(cipcfg, group_name, "table_size", item_value);
    tbl_size = atoi(item_value);
    websWrite(wp,T("["));
    rule_index=(rule_index/10)*10;
    for (i = rule_index;i < rule_index + 10 && i < tbl_size; i++) {
        
	if (isFirst == 0)
	    websWrite(wp, T(","));
        
    	websWrite(wp,T("{"));
	sprintf(value,genBfwMainEntry(i,value) );
    	websWrite(wp, T(value));
    	websWrite(wp,T("}"));
	isFirst = 0;
    
    }
    websWrite(wp,T("]"));
    close_cipcfg(cipcfg);
    return 0;
}
#endif


/**
 * Define the form action for system releated function
 *
 * @param  none:
 * @return none
 */
void initWlanForm(void)
{
    websFormDefine(T("applyWlan"), applyWlan);
    websFormDefine(T("applyWlanVap"), applyWlanVap);
    websFormDefine(T("applyWlanVapAdv"), applyWlanVapAdv);
    websFormDefine(T("applyWlanVapSec"), applyWlanVapSec);
    websFormDefine(T("applyWlanVapAcl"), applyWlanVapAcl);
    websFormDefine(T("applyWlanWds"), applyWlanWds);
    websFormDefine(T("applyWlanRepeater"), applyWlanRepeater);
    websFormDefine(T("applyWdsInterface"), applyWdsInterface);
    websFormDefine(T("applyWdsDiscover"), applyWdsDiscover);
    websFormDefine(T("applyScan"), applyScan);
    websFormDefine(T("applyScanConnect"), applyScanConnect);
    websFormDefine(T("applyDeleteWDS"), applyDeleteWDS);
    websFormDefine(T("applyWPS"), applyWPS);
#ifdef LAYER2_FIREWALL
    websFormDefine(T("applyWlanBfwList"), applyWlanBfwList);
    websFormDefine(T("applyWlanBfwMove"), applyWlanBfwMove);
    websFormDefine(T("applyWlanBfwEdit"), applyWlanBfwEdit);
    websFormDefine(T("applyBfwService"), applyBfwService);
    websFormDefine(T("applyBfwDAI"), applyBfwDAI);
#endif

    websAspDefine(T("genWlanCC"), genWlanCC);
    websAspDefine(T("genWlanAcl"), genWlanAcl);
#ifdef LAYER2_FIREWALL
    websAspDefine(T("genBfwList"), genBfwList);
    websAspDefine(T("genBfwEditEntry"), genBfwEditEntry);
    websAspDefine(T("genBfwService"), genBfwService);
    websAspDefine(T("genBfwServiceAdd"), genBfwServiceAdd);
    websAspDefine(T("genBfwDaiPvg"), genBfwDaiPvg);
    websAspDefine(T("genBfwDaiDhcp"), genBfwDaiDhcp);
#endif
// For ap management
    websAspDefine(T("scanAP"), scanAP);
    websAspDefine(T("setPower"),setPower); 
    websAspDefine(T("setCSA"),setCSA); 
#ifdef GRE
    websAspDefine(T("setGRE"), setGRE);
#endif
#ifdef CAPWAP
    websAspDefine(T("genTunInterfaceList"), genTunInterfaceList);
#endif

    websFormDefine(T("kickStation"), kickStation);
}
