/* cipcfg.c */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>
#include <unistd.h>
#include <ctype.h>
#include <signal.h>
#include "id.h"
#include "syscall.h"
#include "cipcfg.h"
#include "cipcfg_validate.h"
#include "cipcfg_related_common.h"


extern unsigned int magic;
extern char* config_rom;
extern char* config_ram;
extern char* current_config;
extern char* backup_cfg_file;
extern char* restored_cfg_file;
extern group_row_t group_table[];
extern user_group_row_t user_table[];
extern permission_row_t conf_perm_table[];
extern permission_row_t oper_perm_table[];

const STRING_PAIR zone_info[] = 
{
    {"Pacific/Kiritimati","(GMT+14:00)Kiritimati,Christmas Is."},
    {"Pacific/Tongatapu","(GMT+13:00)Nuku'alofa,Tonga"},
    {"Pacific/Fiji","(GMT+12:00)Fiji,Kamchatka,Marshall Is."},
    {"Pacific/Auckland","(GMT+12:00)Auckland,Wellington"},
    {"Asia/Magadan","(GMT+11:00)Magadan,Solomon Is.,New Caledonia"},
    {"Asia/Vladivostok","(GMT+10:00)Vladivostok"},
    {"Australia/Hobart","(GMT+10:00)Hobart"},
    {"Pacific/Guam","(GMT+10:00)Guam,Port Moresby"},
    {"Australia/Canberra","(GMT+10:00)Canberra,Melbourne,Sydney"},
    {"Australia/Brisbane","(GMT+10:00)Brisbane"},
    {"Australia/Darwin","(GMT+09:30)Darwin"},
    {"Australia/Adelaide","(GMT+09:30)Adelaide"},
    {"Asia/Yakutsk","(GMT+09:00)Yakutsk"},
    {"Asia/Seoul","(GMT+09:00)Seoul"},
    {"Asia/Tokyo","(GMT+09:00)Osaka,Sapporo,Tokyo"},
    {"Asia/Taipei","(GMT+08:00)Taipei"},
    {"Australia/Perth","(GMT+08:00)Perth"},
    {"Asia/Singapore","(GMT+08:00)Kuala Lumpur,Singapore"},
    {"Asia/Ulaanbaatar","(GMT+08:00)Ulaanbataar"},
    {"Asia/HongKong","(GMT+08:00)Beijing,Chongqing,Hong Kong,Urumqi"},
    {"Asia/Krasnoyarsk","(GMT+07:00)Krasnoyarsk"},
    {"Asia/Jakarta","(GMT+07:00)Bangkok,Hanoi,Jakarta"},
    {"Asia/Rangoon","(GMT+06:30)Rangoon"},
    {"Asia/Dhaka","(GMT+06:00)Astana,Dhaka"},
    {"Asia/Novosibirsk","(GMT+06:00)Almaty,Novosibirsk"},
    {"Asia/Calcutta","(GMT+05:30)Calcutta,Chennai,Mumbai,New Delhi"},
    {"Asia/Karachi","(GMT+05:00)Islamabad,Karachi"},
    {"Asia/Tashkent","(GMT+05:00)Tashkent"},
    {"Asia/Yekaterinburg","(GMT+05:00)Ekaterinburg"},
    {"Asia/Kabul","(GMT+04:30)Kabul"},
    {"Asia/Yerevan","(GMT+04:00)Baku,Tbilisi,Yerevan"},
    {"Asia/Muscat","(GMT+04:00)Abu Dhabi,Muscat"},
    {"Asia/Tehran","(GMT+03:30)Tehran"},
    {"Africa/Nairobi","(GMT+03:00)Nairobi"},
    {"Europe/Moscow","(GMT+03:00)Moscow,St. Petersburg,Volgograd"},
    {"Asia/Kuwait","(GMT+03:00)Kuwait,Riyadh"},
    {"Asia/Baghdad","(GMT+03:00)Baghdad"},
    {"Asia/Jerusalem","(GMT+02:00)Jerusalem"},
    {"Europe/Helsinki","(GMT+02:00)Helsinki,Riga,Tallinn"},
    {"Africa/Harare","(GMT+02:00)Harare,Pretoria"},
    {"Africa/Cairo","(GMT+02:00)Cairo"},
    {"Europe/Bucharest","(GMT+02:00)Bucharest"},
    {"Europe/Athens","(GMT+02:00)Athens,Istanbul,Minsk"},
    {"Europe/Sarajevo","(GMT+01:00)Sarajevo,Sofija,Vilnius,Warsaw,Zagreb"},
    {"Europe/Brussels","(GMT+01:00)Brussels,Copenhagen,Madrid,Paris"},
    {"Europe/Prague","(GMT+01:00)Belgrade,Bratislava,Budapest,Prague"},
    {"Europe/Amsterdam","(GMT+01:00)Amsterdam,Berlin,Bern,Rome"},
    {"Europe/Dublin","(GMT 00:00)Greenwich Mean Time:Dublin,Lisbon,London"},
    {"Africa/Casablanca","(GMT 00:00)Casablanca,Monrovia"},
    {"Atlantic/Cape_Verde","(GMT-01:00)Cape Verde Is."},
    {"Atlantic/Azores","(GMT-01:00)Azores"},
    {"Atlantic/South_Georgia","(GMT-02:00)Mid-Atlantic"},
    {"America/Buenos_Aires","(GMT-03:00)Buenos Aires,Georgetown"},
    {"Brazil/East","(GMT-03:00)Brasilia"},
    {"US/Newfoundland_Time","(GMT-03:30)Newfoundland"},
    {"America/Santiago","(GMT-04:00)Santiago"},
    {"America/Caracas","(GMT-04:00)Caracas,La Paz"},
    {"US/Atlantic_Time","(GMT-04:00)Atlantic Time(Canada)"},
    {"America/Indianapolis","(GMT-05:00)Indiana(East)"},
    {"US/Eastern_Time","(GMT-05:00)Eastern Time(US&Canada)"},	 
    {"US/Central_Time","(GMT-06:00)Central Time(US&Canada)"},   
    {"US/Mountain_Time","(GMT-07:00)Mountain Time(US&Canada)"},    
    {"US/Arizona","(GMT-07:00)Mountain Time (Arizona, no DST)"},	     
    {"US/Pacific_Time","(GMT-08:00)Pacific Time(US&Canada),Tijuana"},    
    {"US/Alaska_Time","(GMT-09:00)Alaska"},	 
    {"US/Hawaii_Time","(GMT-10:00)Hawaii"},
    {"US/Samoa","(GMT-11:00)Midway Island,Samoa"},
    {"Etc/GMT+12","(GMT-12:00)International Date Line West"},
    { NULL, NULL } // End of array
};

CFG_RULE cli_cfg_type_list[] ={
    {CFG_TYPE_INT, "int"},
    {CFG_TYPE_STRING, "string"},
    {CFG_TYPE_MAC, "mac"},
    {CFG_TYPE_IP, "ip"},
    {CFG_TYPE_DOMAIN, "domain"},
    {CFG_TYPE_BAND, "band"},
    {CFG_TYPE_CHANNEL, "channel"},
    {CFG_TYPE_TIMEZONE, "timezone"},
    {CFG_TYPE_RESTRICT, "restricted"},
    {-1, NULL}
};

int apply_rf0 = FALSE; //The flag to apply rf0 config

/*  Lock cipcfg file. 
 *  target is lock target, 
 *   flag  is lock mode : EX_LOCK or SHARE_LOCK
 */
int lock_cipcfg(int target, int flag)
{
    int fd = -1;
    char *file = NULL;
    switch(target)
    {
        case LOCK_CIPCFG_RAM:
            file = LOCK_RAM_FILE;
            break;
        case LOCK_CIPCFG_ROM:
            file = LOCK_ROM_FILE;
            break;
        case LOCK_CIPCFG_BACKUP:
            file = LOCK_BACKUP_FILE;
            break;
        case LOCK_CIPCFG_RESTORED:
            file = LOCK_RESTORED_FILE;
            break;
        case LOCK_CIPCFG_CURRENT:
            file = LOCK_CURRENT_FILE;
            break;
        default:
            file = LOCK_DEFAULT_FILE;
    }
    if (file != NULL)
	fd = open(file, O_RDWR|O_CREAT|O_TRUNC);
    // open() return -1 when error
    // O_TRUNC makes file size -> 0
    if( -1 == fd ){
        return CIPCFG_ERR_OPENFILE;
    }
    if( 0 != flock(fd, flag) ){
        close(fd);
        return CIPCFG_ERR_LOCK;
    }else{
        // return non-negative fd for unlock_cipcfg
        return fd;
    }
}

void unlock_cipcfg(int fd)
{
    flock(fd, LOCK_UN);
    close(fd);
}

void print_list(ITEM_LIST *list)
{
    int count =0;

    while (list != NULL)
    {
        count++;
        cipcfg_print("%d. Gid = %d, Name=%s Value=%s\n", 
                count, list->gid, list->item_name, list->item_value);
        list = list->next;
    }
}

int free_item_list(ITEM_LIST *list)
{
    ITEM_LIST *tmp;
    while (list != NULL) {
        tmp = list;
        list = list->next;
        free(tmp->item_name);
        free(tmp->item_value);
        free(tmp);
    }
    return 0;
}

/**
 * search group row in group table by prefix
 * @param prefix prefix of group
 * @return	pointer to the row in group_table, NULL if not found
 */   
group_row_t *group_table_prefix(char *prefix)
{
    int i, len;
    if (!prefix)
	return NULL;
    len = strlen(prefix);
    for (i=0; group_table[i].gid; i++) {
	if (strlen(group_table[i].prefix) == len) {
            if (!strcmp(group_table[i].prefix, prefix)) {
        	return &group_table[i];
            }
	}
    }
    return NULL;
}

/**
 * search group row in group table by group_name
 * @param prefix prefix of group
 * @return	pointer to the row in group_table, NULL if not found
 */   
group_row_t *group_table_row_str(char *group_name)
{
    int i, len;

    if (!group_name)
	return NULL;
    // search alias table
/*
    for (j=0;  grp_alias[j].group_name ; j++) {
        if (strlen(group_name) == strlen(grp_alias[j].alias_name)) {
            if (!strcmp(grp_alias[j].alias_name, group_name)) {
                group_name = grp_alias[j].group_name;
                break;
            }
        }
    }
*/
    len = strlen(group_name);
    // search in original group table
    for (i=0; group_table[i].gid; i++) {
	if (strlen(group_table[i].group_name) == len) {
            if (!strcmp(group_table[i].group_name, group_name)) {
                return &group_table[i];
            }
        }
    }
    
    return NULL;
}

/**
 * search group row in group table by gid
 * @param gid	group id
 * @return	pointer to the row in group_table, NULL if not found
 */   
group_row_t *group_table_row(int gid)
{
    int i;
    
    for (i=0; group_table[i].gid; i++) {
    	if (gid==group_table[i].gid) {
	    return &group_table[i];
	}
    }
    return NULL;
}

/**
 * searches for specific node from root according to path in name
 *  
 * @param root	root
 * @param name	absolute path name delimeted by dot. a colon specifies the index
 * 		if multiple nodes with the same name exist 
 * @return	pointer	to destination node, NULL if not found
 */ 
CIPCFG *cipcfg_search(CIPCFG *root, char name[])
{
    char *buf, *node;
    CIPCFG *p=root;
    int found=0;
    int i;
    char *ptr;

    if (!root || !name)
	return NULL;
    buf = strdup(name);
    if (!buf)
	return NULL;
    node=strtok_r(buf, ".", &ptr);
    if (!node) {
	free(buf);
    	return NULL;
    }
    while (p) {
    	if (strcmp(ezxml_name(p), node)) {
	    p=p->sibling;
	}
	else {
	    found=1;
	    break;
	}
    }
    if (!found) {
	free(buf);
    	return NULL;
    }
    
    while (1) {
    	char *colon;
    	int idx=0;
    	
    	node=strtok_r(NULL, ".", &ptr);
    	if (!node)
	    break;
	colon=strchr(node, ':');
	if (colon) {
	    int len;
	    *colon='\0';
	    colon++;	    
	    len = strlen(colon);
	    for(i=0; i<len; i++)
            { 
                if(isdigit(colon[i]) == 0)
                {
	            free(buf);
                    return NULL;
                }
            }
	    idx=atoi(colon);
	}
	p = ezxml_child(p, node);
	if (p && idx)
	    p = ezxml_idx(p, idx);
	if (!p)
	    break;
    }
    free(buf);
    return p;
}

#if 1
struct attribute *cipcfg_list_all_child(CIPCFG *root)
{
    CIPCFG *child, *child_head;
    struct attribute *head = NULL, *prev = NULL, *tmp;
    int idx;

    if (!root)
	return NULL;
    child_head = root->child;
    while (child_head != NULL) {
	child = child_head;
	idx = (child->next != NULL) ? 0 : -1;
	while (child != NULL && child->name != NULL) {
	    tmp = get_new_attr(child->name, NULL);
	    if (!tmp) {
		free_attr_list(head);
		return NULL;
	    }
	    tmp->idx = idx;
	    idx++;

	    if (head == NULL)
		head = tmp;
	    if (prev != NULL)
		prev->next = tmp;
	    prev = tmp;

	    child = child->next;
	}
	child_head = child_head->sibling;
    }
    return head;
}

struct attribute *list_child_by_path(CIPCFG *cipcfg, char *path)
{
    CIPCFG *cur_cipcfg = NULL;

    cur_cipcfg = cipcfg_search(cipcfg, path);
    if (!cur_cipcfg) {
	cipcfg_print("ERROR: Fail to find item.\n");
	return NULL;
    }
    return cipcfg_list_all_child(cur_cipcfg);
}
#else
/**
 * Get all child nodes at current path 
 *
 * @param path          current path.
 * @return              the list of child nodes at current path. 
 */

CIPCFG *list_child_by_path(CIPCFG *cipcfg, char *path)
{
    CIPCFG *cur_cipcfg = NULL;

    cur_cipcfg = cipcfg_search(cipcfg, path);
    if(!cur_cipcfg)
    {
        cipcfg_print("ERROR: Fail to find item.\n");
        return NULL;
    }
    return cipcfg_list_all_child(cur_cipcfg);
}

/**
 * Get all child nodes of current node. 
 * 
 * @param node          current cipcfg node. 
 * @return              all child nodes of inputed node. 
 */

CIPCFG *cipcfg_list_all_child(CIPCFG *root)
{
    CIPCFG *head, *child_list, *tmp;
    int i=1;

    head = root->child;
    child_list = head;

    while(head != NULL)
    {
        i=1;
        tmp = head;
        if(head->next)
        {
            while( ezxml_idx(head, i) != NULL)
            {
                tmp->next = ezxml_idx(head, i);
                tmp = tmp->next;
                i++;
            }    
        }
        if(i > 1)
        {
            tmp -> next = head->sibling;
	    head = tmp -> next;
        }
        else
        {
            head -> next = head->sibling;
            head = head->next;
        }
    }
    return child_list;
}
#endif

static ITEM_LIST *get_new_item_f(int gid, char *item_name, char *item_value, int force_len)
{
    ITEM_LIST *item;

    item = (ITEM_LIST *) malloc(sizeof(*item));
    if (!item)
	return NULL;
    memset(item, 0, sizeof(*item));
    item->gid = gid;
    item->item_name = item->item_value = NULL;
    item->next = NULL;
    if (item_name != NULL) {
	item->item_name = strdup(item_name);
	if (!item->item_name)
	    goto fail;
    }
    if (item_value != NULL && force_len >= 0) {
	if (force_len > 0) {
	    // item_value should allocate enough space for all possible value
	    item->item_value = (char *) malloc(force_len);
	    if (!item->item_value)
		goto fail;
	    memset(item->item_value, 0, force_len);
	    strncpy(item->item_value, item_value, force_len-1);
	    item->item_value[force_len-1]='\0';
	} else {
	    item->item_value = strdup(item_value);
	    if (!item->item_value)
		goto fail;
	}
    }
    return item;
fail:
    free_item_list(item);
    return NULL;
}

ITEM_LIST *get_new_item(int gid, char *item_name, char *item_value)
{
    return get_new_item_f(gid, item_name, item_value, 0);
}

static ITEM_LIST *dup_item(ITEM_LIST *item)
{
    ITEM_LIST *new_item;

    if (!item)
	return NULL;
    new_item = get_new_item(item->gid, item->item_name, item->item_value);
    return new_item;
}

/**
 * Reset the config to factory default value
 * The factory config file is on "/etc/default-config.xml"
 * @return 0: success, 0<: error
 */ 
int reset_cipcfg()
{
    FILE *sfp, *dfp;
    char buf[MAX_VALUE_LEN];
    int fd;

    /* Using customized config file if exist */
    if (access(CUSTOMIZE_CONFIG, F_OK) == 0)
	sfp=fopen(CUSTOMIZE_CONFIG, "r");
    else
	sfp=fopen(DEFAULT_CONFIG, "r");
    if (!sfp) {
	return CIPCFG_ERR_OPENFILE;
    }

    fd = lock_cipcfg(LOCK_CIPCFG_ROM, EX_LOCK);
    if (0 > fd) {
	fclose(sfp);
	return fd;
    }
    dfp = fopen(config_rom, "w");
    if (!dfp) {
	fclose(sfp);
	unlock_cipcfg(fd);
	return CIPCFG_ERR_OPENFILE;
    }

    while (fgets(buf, MAX_VALUE_LEN, sfp) != NULL) {
        fputs(buf, dfp);
    }
    
    // XXX:instead fsync() ??
    fflush(dfp); //force write data to flash
    fclose(sfp);
    fclose(dfp);

    if((dfp=fopen(config_rom, "r")) == NULL){
	unlock_cipcfg(fd);
        return CIPCFG_ERR_OPENFILE;
    }
    fclose(dfp);
    unlock_cipcfg(fd);

    fd = lock_cipcfg(LOCK_CIPCFG_RAM, EX_LOCK);
    if( 0 > fd){
        return fd;
    }
    sprintf(buf, "/bin/rm -f %s", config_ram);
    cip_system(buf);
    unlock_cipcfg(fd);
#if defined(CIPCFG_CFGARCV) && CIPCFG_CFGARCV == 1
    do_cfgarcv() ;// Do config archiving (defined in extension)
#endif

    return CIPCFG_SUCCESS;
}

/**
 * Initialize a CIPCFG session. Prepare session-wise data and get current config
 * from config file and return it as a session handle.
 * @return current config, NULL on error
 */ 
CIPCFG* init_cipcfg()
{
    FILE *fp=NULL, *ram_fp=NULL;
    CIPCFG *cipcfg;
    char buf[MAX_VALUE_LEN];
    int ram_fd, rom_fd;

    /* Not always read the config from flash to imptorve preformence
     * Only read the config at first time when access config
     */

    if (access(config_ram, F_OK) != 0) {
        rom_fd = lock_cipcfg(LOCK_CIPCFG_ROM, SHARE_LOCK);
        if( 0 > rom_fd){
            return NULL;
        }
        if ((fp=fopen(config_rom, "r")) != NULL) {
            /* duplicate the config for flash to ram */
            ram_fd = lock_cipcfg(LOCK_CIPCFG_RAM, EX_LOCK);
            if( 0 > ram_fd){
                unlock_cipcfg(rom_fd);
                return NULL;
            }
            if ((ram_fp = fopen(config_ram, "w")) == NULL) {
                unlock_cipcfg(ram_fd);
                unlock_cipcfg(rom_fd);
                return NULL;
            }

            /* check the file is correctly xml file */
            *buf = '\0';
            fgets(buf, MAX_VALUE_LEN, fp); 
            if (!strstr(buf, "<cipconfig")) {
                cipcfg_print("No config in flash, reset to default\n");
                unlock_cipcfg(ram_fd);
                unlock_cipcfg(rom_fd);
                reset_cipcfg();
                return NULL;
            }
            fputs(buf, ram_fp);

            while (!feof(fp)) {
                *buf = '\0';
                fgets(buf, MAX_VALUE_LEN, fp);
                fputs(buf, ram_fp);
            }

            fclose(fp);
            unlock_cipcfg(rom_fd);
            fflush(ram_fp);
            fclose(ram_fp);
            unlock_cipcfg(ram_fd);
        }else{
            unlock_cipcfg(rom_fd);
            return NULL;
        }
    }

    ram_fd = lock_cipcfg(LOCK_CIPCFG_RAM, SHARE_LOCK);
    if( 0 > ram_fd){
        return NULL;
    }

    if ((fp=fopen(config_ram, "r")) == NULL) {
        unlock_cipcfg(ram_fd);
        return NULL;
    }

    cipcfg = ezxml_parse_fp(fp);    
    fclose(fp);
    unlock_cipcfg(ram_fd);

    return cipcfg;
}

static int get_cipcfg_lock_target(char *config_file)
{
    int lock_target = LOCK_DEFAULT;
    if (config_file && *config_file) {
        if (!strcmp(config_file, restored_cfg_file))
            lock_target = LOCK_CIPCFG_RESTORED;
        else if (!strcmp(config_file, config_rom))
            lock_target = LOCK_CIPCFG_ROM;
        else if (!strcmp(config_file, config_ram))
            lock_target = LOCK_CIPCFG_RAM;
        else if (!strcmp(config_file, backup_cfg_file))
            lock_target = LOCK_CIPCFG_BACKUP;
        else if (!strcmp(config_file, current_config))
            lock_target = LOCK_CIPCFG_CURRENT;
    }
    return lock_target;
}

/**
 * Initialize a CIPCFG session. Prepare session-wise data and get current config
 * from input file and return it as a session handle.
 * @return current config, NULL on error
 */ 
CIPCFG* init_cipcfg_f(char *config_file)
{
    FILE *fp=NULL;
    CIPCFG *cipcfg;
    int fd, lock_target;

    if (!config_file || !*config_file)
        config_file = config_rom;
    lock_target = get_cipcfg_lock_target(config_file);

    fd = lock_cipcfg(lock_target, SHARE_LOCK);
    if( 0 > fd){
        return NULL;
    }

    if ((fp=fopen(config_file, "r")) == NULL) {
        unlock_cipcfg(fd);
        return NULL;
    }

    cipcfg = ezxml_parse_fp(fp);    
    fclose(fp);
    unlock_cipcfg(fd);

    return cipcfg;
}


/**
 * Close a CIPCFG session and free memory located for CIPCFG structure.
 * @return 0 if success, <0 on error
 */  
int close_cipcfg(CIPCFG *cipcfg)
{
    if (!cipcfg) {
    	return CIPCFG_ERR_MEM;
    }
    ezxml_free(cipcfg);
    return CIPCFG_SUCCESS;
}

/**
 * Append an item to item_list. The item's gid, name and value are specified in 
 * gid, item_name and item_value arguments. The new item's item_name and 
 * item_value field will be allocated by this function and have to be freed 
 * using free_item_list().
 * 
 * @param item_list	item list
 * @param gid     	group id of the new item
 * @param item_name	name of the new item
 * @param item_value  	value of the new item
 * @return 		head of item list after append
 */ 
ITEM_LIST *append_cfg_item(ITEM_LIST *item_list, int gid, char *item_name, char *item_value)
{
    ITEM_LIST *item = NULL, *list_head = item_list, *prev_item = NULL;
    int count =  0;
    
    if (!item_name || !item_value)
	return NULL;
    if ((item = get_new_item_f(gid, item_name, item_value, MAX_VALUE_LEN)) == NULL) {
	cipcfg_print("Memory allocate error, can't get new item\n");
	return NULL;
    }
    if (!item_list)
	return item;

    while (item_list != NULL) {
        if (gid > item_list->gid) {
            if (item_list->next == NULL) {// last item in list
                item_list->next = item;
                break;
            }
            prev_item = item_list;
            item_list = item_list->next;
            count++;
        }
        else
        {
            if (prev_item == NULL) { // first item in list
                item->next = list_head;
                list_head = item;
            }
            else {
                item->next = prev_item->next;
                prev_item->next = item;
            }
            break;
        }
    }

    return list_head;
}

ITEM_LIST *check_item_exist(ITEM_LIST *head, char *item_name)
{
    if (!item_name)
	return NULL;
    while(head != NULL)
    {
        if(strcmp(head->item_name, item_name) == 0)
            return head;
        head = head->next;
    }
    return NULL;
}

int free_attr_list(struct attribute *head)
{
    struct attribute *next;
    while (head != NULL) {
        next = head->next;
        free(head->attr_name);
        free(head->attr_value);
        free(head);
        head = next;
    }
    return 0;
} 

/**
 * Get item name and value at specific path.
 * 
 * @param item          Item name. 
 * @param path          Specific path wich item is located.
 * @return              Item value and item name. 
 */
struct attribute *get_item_by_path(char *item, char *path)
{
    CIPCFG *cipcfg = NULL;
    CIPCFG *cur_cipcfg = NULL;
    struct attribute *ret = NULL;

    if (!path)
	return NULL;
    cipcfg = init_cipcfg();
    if(!cipcfg)
    {
        cipcfg_print("ERROR: Fail to init configuration file.\n");
        return NULL;
    }

    cur_cipcfg = cipcfg_search(cipcfg, path);
    if(cur_cipcfg == NULL)
    {
        //cipcfg_print("ERROR: Item not found.\n");
	close_cipcfg(cipcfg);
        return NULL;
    }
    ret = get_all_attr(cur_cipcfg, item);

    close_cipcfg(cipcfg); 
    return ret; 
}

struct attribute *get_new_attr(char *attr_name, char *attr_value)
{
    struct attribute *attr;

    attr = (struct attribute *) malloc(sizeof(*attr));
    if (!attr)
	return NULL;
    memset(attr, 0, sizeof(*attr));
    attr->idx = -1;
    attr->attr_name = attr->attr_value = NULL;
    attr->next = NULL;
    if (attr_name) {
	attr->attr_name = strdup(attr_name);
	if (!attr->attr_name)
	    goto fail;
    }
    if (attr_value) {
	attr->attr_value = strdup(attr_value);
	if (!attr->attr_value)
	    goto fail;
    }
    return attr;
fail:
    free_attr_list(attr);
    return NULL;
}

/**
 * Get a list item attribute value.
 * 
 * @param node          the CIPCFG handle
 * @param item          the item name which would be listed, if NULL is filled, it would
 *                      return all attributes of current cipcfg node. 
 * @return              the list of attributes. 
 */
struct attribute * get_all_attr(CIPCFG *node, char *item)
{
    int i = 0;
    struct attribute *head = NULL, *prev = NULL, *attr = NULL;

    if (!node)
	return NULL;
    if (item != NULL) {
	while (node->attr[i] != NULL && node->attr[i+1] != NULL) {
	    if (strcmp(item, node->attr[i]) == 0) {
		attr = get_new_attr(node->attr[i], node->attr[i+1]);
		head = attr;
		break;
	    }
	    i = i+2;
	}
    } else {
	while (node->attr[i] != NULL && node->attr[i+1] != NULL) {
	    attr = get_new_attr(node->attr[i], node->attr[i+1]);
	    if (!attr) {
		free_attr_list(head);
		head = NULL;
		break;
	    }
	    if(prev != NULL)
	        prev -> next = attr; 
	    if(head == NULL)
		head = attr;
	    prev = attr; 
	    i = i+2;
	}
    }
    return head;
}

int get_cfg_item_by_prefix(CIPCFG *cipcfg, char *prefix, char *item_name, char *item_value)
{
    CIPCFG *node;
    char *full_item_name, *attr_name; 
    const char *attr;

    if (item_value != NULL)
	*item_value = '\0';
    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    if (!prefix)
	return CIPCFG_ERR_UNKNOWN_GROUP;
    if (!item_name || !item_value)
	return CIPCFG_ERR_UNKNOWN_ITEM;

    full_item_name = malloc(strlen(prefix)+strlen(item_name)+2);
    if (!full_item_name) {
	cipcfg_print("%s>full_item_name malloc error\n", __func__);
	return CIPCFG_ERR_MEM;
    }
    sprintf(full_item_name, "%s.%s", prefix, item_name);

    // separate attribute from item_name
    attr_name = strrchr(full_item_name, '.');
    *attr_name = '\0';
    attr_name++;

    node = cipcfg_search(cipcfg, full_item_name);
    if (!node) {
	cipcfg_print("%s>Can't find node=%s\n", __func__, full_item_name);
	free(full_item_name);
	return CIPCFG_ERR_NODE_NOT_FOUND;
    }

    if ((attr = ezxml_attr(node, attr_name)) == NULL) {
        cipcfg_print("%s>Can't find the item_name=%s.%s\n", __func__, full_item_name, attr_name);
        free(full_item_name);
        return CIPCFG_ERR_UNKNOWN_ITEM;
    }

    strncpy(item_value, attr, MAX_VALUE_LEN-1);
    item_value[MAX_VALUE_LEN-1] = 0;
    free(full_item_name);
    return CIPCFG_SUCCESS;
}

/**
 * Get a single config value of cipcfg specified by gid and item_name. Retrieved
 * value will be stored in item_value.
 * 
 * @param cipcfg	the CIPCFG handle
 * @param gid		group id of the item
 * @param item_name	name of the item
 * @param item_value	buffer to store the item value, which should be
 * 			allocated by caller, the length must >= MAX_VLAUE_LEN
 * @return		0 on success, <0 on error
 */    
int get_cfg_item(CIPCFG *cipcfg, int gid, char *item_name, char *item_value)
{
    group_row_t *group;
    int ret = CIPCFG_ERR_NODE_NOT_FOUND;

    if (item_value != NULL)
	*item_value = '\0';
    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    group = group_table_row(gid);
    if (!group) {	// no matched group !
        cipcfg_print("%s>Can't find group ID=%d\n", __func__, gid);
        return CIPCFG_ERR_UNKNOWN_GROUP;
    }
    ret = get_cfg_item_by_prefix(cipcfg, group->prefix, item_name, item_value);
    if (ret != CIPCFG_SUCCESS)
	cipcfg_print("%s>return value of calling get_cfg_item_by_prefix() = %d\n", __func__, ret);
    return ret;
}

/**
 * Get a single config value of cipcfg specified by group_name and item_name. Retrieved
 * value will be stored in item_value.
 * 
 * @param cipcfg	the CIPCFG handle
 * @param group_name	string group name of the item
 * @param item_name	name of the item
 * @param item_value	buffer to store the item value, which should be
 * 			allocated by caller, the length must >= MAX_VLAUE_LEN
 * @return		0 on success, <0 on error
 */    
int get_cfg_item_by_str(CIPCFG *cipcfg, char *group_name, char *item_name, char *item_value)
{
    group_row_t *group;
    int ret = CIPCFG_ERR_NODE_NOT_FOUND;

    if (item_value != NULL)
	*item_value = '\0';
    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    group = group_table_row_str(group_name);
    if (!group) {	// no matched group !
        cipcfg_print("%s>Can't find group name=%s\n", __func__, group_name);
        return CIPCFG_ERR_UNKNOWN_GROUP;
    }
    ret = get_cfg_item_by_prefix(cipcfg, group->prefix, item_name, item_value);
    if (ret != CIPCFG_SUCCESS)
	cipcfg_print("%s>return value of calling get_cfg_item_by_prefix() = %d\n", __func__, ret);
    return ret;
}

/**
 * Get a list of item values. This is a wrapper of get_cfg_item.
 * 
 * @param cipcfg	the CIPCFG handle
 * @param group_list	item list. the item_value field of each item should be 
 * 			allocated enough space
 * @return		0 on success, <0 on error
 */      
int get_cfg_items(CIPCFG *cipcfg, ITEM_LIST *group_list)
{
    ITEM_LIST *p;
    int ret = CIPCFG_ERR_NODE_NOT_FOUND;

    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    if (!group_list) {
	return CIPCFG_SUCCESS;
    }

    /* set the group value to cipcfg */
    for (p=group_list; p; p=p->next) {
    	ret=get_cfg_item(cipcfg, p->gid, p->item_name, p->item_value);
    	if (ret) {
	    return ret;
	}
    }

    return CIPCFG_SUCCESS;
}

char *path_comma_filter (char *string) {
    int i, j=0, k, len;

    if (!string)
	return NULL;

    len = strlen(string);
    for (i=0; i<len; i++) {
        if( string[i] == ':' ) {
            j = i;
            while( j < len && string[j] != '.') {
                j++;
            }
	    for ( k=i; k < len; k++ ) {
                string[k] = string[k+j-i];
		if (string[k+j-i] == '\0')
		    len = k;
            }
        }
    
    }
    
    return string;
}

/** 
 * Mapping configuration type string to type index. 
 * 
 * @param type          Input type string
 * @return              type index declared in cli_cfg_type_list table 
 */
int cfg_type_map(char *type)
{
    int i=0;

    if (!type)
	return -1;

    while(cli_cfg_type_list[i].type_string != NULL && cli_cfg_type_list[i].cfg_type != -1)
    {
        if(!strcmp(cli_cfg_type_list[i].type_string, type))
            break;
        i++;
    }

    if(cli_cfg_type_list[i].type_string != NULL)
        return cli_cfg_type_list[i].cfg_type;
    else
        return -1;
}

/** 
 * Parse the configuration rule string. The delim is three characters "|" "&" "|" 
 * 
 * @param string        Rule string 
 * @param flag          Flags that indicate which parts of rule string would be returned. 
 * @return              return value which includes : name, type, rule, description 
 */
char *parse_cfg_rule(char *string, int flag)
{
    int i=0, j=0, idx=1, len;
    char *rule=string;
    char *ret;

    if (!string)
	return NULL;

    ret = malloc(sizeof(char)* CLI_CFG_RULE_MAXLEN);
    if (!ret)
	return NULL;
    memset(ret, '\0', CLI_CFG_RULE_MAXLEN);

    len = strlen(string);
    while (i < len) {
        if(rule[j] == '|' && rule[j+1] == '&' && rule[j+2] == '|'){
            if(idx == flag){
                strncpy(ret, rule, j);
                goto ext;
            }
            idx++;
            rule+=j+3;
            j=0;
            i+=3;
        }
        else{
            j++;
            i++;
        }
    }
    strcpy(ret, rule);

ext:
    if(idx == flag && strlen(ret) > 0)
        return ret;
    free(ret);
    return NULL;
}

static char *get_cfg_item_attr_value(CIPCFG *cur_cipcfg, char *item_name, int flag)
{
    struct attribute *attr;
    char *value;

    attr = get_all_attr(cur_cipcfg, item_name);
    if (!attr)
	return NULL;
    value = parse_cfg_rule(attr->attr_value, flag);
    free_attr_list(attr);
    return value;
}

/** 
 * Get configuration item name in configuration rule file by it's path. 
 * 
 * @param cur_cipcfg    CIPCFG handle
 * @param item_name     The item name   
 * @return              Readable item name. 
 */
char *get_cfg_item_name(CIPCFG *cur_cipcfg, char *item_name)
{
    char *name;

    name = get_cfg_item_attr_value(cur_cipcfg, item_name, CLI_CFG_GET_NAME);
    return name;
}

/** 
 * Get configuration item rule in configuration rule file by it's path. 
 * 
 * @param cur_cipcfg    CIPCFG handle
 * @param item_name     The item name   
 * @return              item validation rule. 
 */
char *get_cfg_item_rule(CIPCFG *cur_cipcfg, char *item_name)
{
    char *rule;

    rule = get_cfg_item_attr_value(cur_cipcfg, item_name, CLI_CFG_GET_RULE);
    return rule;
}

/** 
 * Get configuration item description in configuration rule file by it's path. 
 * 
 * @param cur_cipcfg    CIPCFG handle
 * @param item_name     The item name   
 * @return              Descriptions. 
 */
char *get_cfg_item_desc(CIPCFG *cur_cipcfg, char *item_name)
{
    char *desc;

    desc = get_cfg_item_attr_value(cur_cipcfg, item_name, CLI_CFG_GET_DESC);
    return desc;
}

/** 
 * Get configuration item type in configuration rule file by it's path. 
 * 
 * @param cur_cipcfg    CIPCFG handle
 * @param item_name     The item name   
 * @return              Validation type 
 */
int get_cfg_item_type(CIPCFG *cur_cipcfg, char *item_name)
{
    char *type_str;
    int type;

    type_str = get_cfg_item_attr_value(cur_cipcfg, item_name, CLI_CFG_GET_TYPE);
    type = cfg_type_map(type_str);
    if (type_str != NULL)
	free(type_str);
    return type;
}

/** 
 * Get configuration item rule in configuration rule file by it's group name. 
 * 
 * @param item_name     The item name   
 * @param path          The item group name. 
 * @return              Readable item name. 
 */
char *get_cfg_item_rule_by_group_name(CIPCFG *cfg_rule, char *item_name, char *group_name)
{
    char *rule;
    CIPCFG *cur_rule_cfg = NULL;
    group_row_t *group;
    char *full_item_name, *attr_name;

    if (!cfg_rule || !item_name || !group_name)
	return NULL;

    group=group_table_row_str(group_name);
    if (!group) {       // no matched group !
        cipcfg_print("ERROR: Cannot find group.\n");
        return NULL;
    }
    full_item_name=malloc(strlen(group->prefix)+strlen(item_name)+2);
    if (!full_item_name)
	return NULL;
    sprintf(full_item_name, "%s.%s", group->prefix, item_name);

    // separate attribute from item_name
    attr_name=strrchr(full_item_name, '.');
    *attr_name='\0';
    attr_name++;
    
    full_item_name = path_comma_filter(full_item_name); 

    cur_rule_cfg=cipcfg_search(cfg_rule, full_item_name);
    if(!cur_rule_cfg)
    {
        cipcfg_print("ERROR: Cannot find NODE.\n");
	free(full_item_name);
        return NULL;
    }

    rule = get_cfg_item_rule(cur_rule_cfg, attr_name);
    if (!rule)
	cipcfg_print("ERROR: Cannot get rule of attribute.\n");
    free(full_item_name);
    return rule;
}

/** 
 * Get configuration item type in configuration rule file by it's group name. 
 * 
 * @param item_name     The item name   
 * @param path          The item group name. 
 * @return              Readable item name. 
 */
int get_cfg_item_type_by_group_name(CIPCFG *cfg_rule, char *item_name, char *group_name)
{
    int type;
    CIPCFG *cur_rule_cfg = NULL;
    group_row_t *group;
    char *full_item_name, *attr_name;

    if (!cfg_rule)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    if (!group_name)
	return CIPCFG_ERR_UNKNOWN_GROUP;
    if (!item_name)
	return CIPCFG_ERR_UNKNOWN_ITEM;

    group=group_table_row_str(group_name);
    if (!group) {       // no matched group !
	cipcfg_print("ERROR: Cannot find group.\n");
        return CIPCFG_ERR_UNKNOWN_GROUP;
    }

    full_item_name=malloc(strlen(group->prefix)+strlen(item_name)+2);
    if (!full_item_name)
	return CIPCFG_ERR_MEM;
    sprintf(full_item_name, "%s.%s", group->prefix, item_name);

    // separate attribute from item_name
    attr_name=strrchr(full_item_name, '.');
    *attr_name='\0';
    attr_name++;

    full_item_name = path_comma_filter(full_item_name); 
#ifdef CLI_DEBUG
    cipcfg_print("%s:%d FULL_ITEM_NAME=%s\n", __FILE__, __LINE__, full_item_name);
#endif
    cur_rule_cfg=cipcfg_search(cfg_rule, full_item_name);
    if(!cur_rule_cfg)
    {
        cipcfg_print("ERROR: Cannot find NODE.\n");
	free(full_item_name);
        return CIPCFG_ERR_VALIDATION_FAIL;
    }

    type = get_cfg_item_type(cur_rule_cfg, attr_name);
    if (type == -1) {
	cipcfg_print("ERROR: Cannot get type of attribute.\n");
	type = CIPCFG_ERR_VALIDATION_FAIL;
    }
    free(full_item_name);
    return type;
}

/**
 * Set config items in cipcfg according to item list specified in item_list. The
 * group been set will have dirty bit set in group table. All items in item_list
 * should be in the same group.
 *  
 * @param cipcfg	XML tree structure to set
 * @param item_list	items name/value pairs to set
 * @return		0 if successful, <0 if error occurred
 */
int set_cfg_group(CIPCFG *cipcfg, ITEM_LIST *group_list)
{
    ITEM_LIST *p;
    char *item_name, *attr_name;
    CIPCFG *node;
    group_row_t *group;
    int dirty, ret = CIPCFG_ERR_NODE_NOT_FOUND;

    if (!cipcfg)
	return ret;

    if (!group_list) {
	return CIPCFG_SUCCESS;
    }

    group = group_table_row(group_list->gid);
    if (!group) {	// no matched group !
	return CIPCFG_ERR_UNKNOWN_GROUP;
    }

#if 0
    /* Now we don't implement any validate function so I disabled this check */
    ret = validate_input(group_list);
    if (ret != CIPCFG_SUCCESS) {
	return ret;
    }
#endif

    dirty = FALSE;

    /* set the group value to cipcfg */
    for (p = group_list; p; p = p->next) {
	item_name = malloc(strlen(group->prefix) + strlen(p->item_name) + 2);
	if (!item_name) {
	    return CIPCFG_ERR_MEM;
	}
	sprintf(item_name, "%s.%s", group->prefix, p->item_name);

	// separate attribute from item_name
	attr_name = strrchr(item_name, '.');
	*attr_name = '\0';
	attr_name++;

	node = cipcfg_search(cipcfg, item_name);
	/* If the node did not exist, we try to add the tag */
	if (!node) {
	    CIPCFG *pnode, *rnode;
	    const char *tag_max;
	    char *tag_name, *tag_idx;
	    int max, idx, i, j;
	    size_t off;

	    tag_name = strrchr(item_name, '.');
	    if (!tag_name) {
		cipcfg_print("%s>Can't find subtag of item_name=%s.\n", __func__, item_name);
		free(item_name);
		return CIPCFG_ERR_NODE_NOT_FOUND;
	    }
	    *tag_name = '\0';
	    tag_name++;

	    pnode = cipcfg_search(cipcfg, item_name);
	    if (!pnode) {
		cipcfg_print("%s>Can't find the parent node.\n", __func__);
		free(item_name);
		return CIPCFG_ERR_NODE_NOT_FOUND;
	    }

	    /* Get the max number of child nodes. */
	    tag_max = ezxml_attr(pnode, "max");
	    if (!tag_max) {
		cipcfg_print("%s>Can't find max in the parent node.\n", __func__);
		free(item_name);
		return CIPCFG_ERR_NODE_NOT_FOUND;
	    }
	    max = atoi(tag_max);

	    // Find the index of child node.
	    tag_idx = strrchr(tag_name, ':');
	    if (!tag_idx) {
		cipcfg_print("%s>Can't find index of tag_name=%s.\n", __func__, tag_name);
		free(item_name);
		return CIPCFG_ERR_NODE_NOT_FOUND;
	    }
	    *tag_idx = '\0';
	    tag_idx++;
	    idx = atoi(tag_idx);

	    rnode = ezxml_child(pnode, tag_name);
	    if (!rnode) {
		cipcfg_print("%s>Can't find the first tag node.\n", __func__);
		free(item_name);
		return CIPCFG_ERR_NODE_NOT_FOUND;
	    }

	    if (max <= idx) {
		cipcfg_print("%s>Exceed the max number of the child nodes\n", __func__);
		free(item_name);
		return CIPCFG_ERR_EXCEED_MAX_CHILD;
	    }

	    off = rnode->off;
	    for (i=0; i<=idx; i++) {
		node = (CIPCFG *) ezxml_idx(rnode, i);
		if (!node) {
		    node = (CIPCFG *) ezxml_add_child_d(pnode, tag_name, off);
		    if (!node)
			break;
		    /* Initial all attributes and values */
		    for (j=0; rnode->attr[j] && rnode->attr[j+1]; j+=2)
			ezxml_set_attr_d(node, rnode->attr[j], "");
		    dirty = TRUE;
		} else
		    off = node->off;
	    }
	    if (!node) {
		free(item_name);
		return CIPCFG_ERR_MEM;
	    }

	    /* Set the cur_num if exists */
	    if (ezxml_attr(pnode, "cur_num") != NULL) {
		char tmp[MAX_VALUE_LEN];

		sprintf(tmp, "%d", idx+1);
		ezxml_set_attr_d(pnode, "cur_num", tmp);
		dirty = TRUE;
	    }
	}

	// If attr did not exist, we add attr
	if (ezxml_attr(node, attr_name) == NULL) {
	    cipcfg_print("%s>Can't find the item_name=%s\n", __func__, p->item_name);
	    free(item_name);
	    return  CIPCFG_ERR_UNKNOWN_ITEM;
	}
	if (strcmp(p->item_value, ezxml_attr(node, attr_name))) {
	    dirty = TRUE;
	    ezxml_set_attr_d(node, attr_name, p->item_value);
	}
	free(item_name);
    }

    if (dirty) {
	group->dirty = TRUE;
    }

    return CIPCFG_SUCCESS;
}

int int_compare( const void* a, const void* b ) 
{
    int* arg1 = (int*) a;
    int* arg2 = (int*) b;
    if( *arg1 < *arg2 ) return -1;
    else if( *arg1 == *arg2 ) return 0;
    else return 1;
}   

static int put_cfg_items(CIPCFG *cipcfg, ITEM_LIST *set_list, int *gid_list, int *gid_idxp)
{
    ITEM_LIST *p, *tmp, *grp_head, *related, *related_tail;
    int gid_idx = 0, gid_max = gid_idxp ? *gid_idxp : 0;
    int gid, ret;

    if (gid_idxp)
        *gid_idxp = gid_idx;
    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    if (!set_list)
        return CIPCFG_SUCCESS;

    if (gid_list != NULL) {
        if (gid_max <= 0)
            return CIPCFG_ERR_INVALID_LENGTH;
        memset(gid_list, 0, sizeof(int)*gid_max);
    }

    ret = CIPCFG_SUCCESS;
    p = set_list;
    gid = p->gid;
    grp_head = NULL;
    while (TRUE) {
        if (p && p->gid==gid) {
            tmp = dup_item(p);
            tmp->next = grp_head;
            grp_head = tmp;
            if ((related = set_related_items(p->gid, p->item_name, p->item_value))) {
                related_tail = related;
                while (related_tail->next != NULL)
                    related_tail = related_tail->next;
                related_tail->next = p->next;
                p->next = related;
            }
        } else {
            if (gid_list) {
                /* Save group id to PATH_SYNC_CFG */
                gid_list[gid_idx] = grp_head->gid;
                gid_idx++;
                if (gid_idx >= gid_max) {
                    ret = CIPCFG_ERR_INVALID_LENGTH;
                    break;
                }
            }
            // save current group
            ret = set_cfg_group(cipcfg, grp_head);
            free_item_list(grp_head);
            if (ret != CIPCFG_SUCCESS)
                break;
            if (p) {    // proceed to next group
                grp_head = dup_item(p);
                gid = grp_head->gid;
                grp_head->next = NULL;
            } else {    // end of list
                break;
            }
        }
        p = p->next;
    }
    if (gid_list && gid_idxp)
        *gid_idxp = gid_idx;
    return ret;
}

/* Check gid_list is in PATH_SYNC_CFG */
static int set_sync_file(int *gid_list, int gid_idx)
{
    int match = 0, count = 0, ret;

    ret = check_sync_file(gid_list, &match, &count);
    if (ret < 0)
	return 0;
    if (ret == 0 || match < 1) {
        FILE *fp = fopen(PATH_SYNC_CFG, "a");
        if (fp != NULL) {
            int i;
            for (i=0; i < gid_idx; i++)
                fprintf(fp, "%d ", gid_list[i]);
            fprintf(fp, "\n");
            fclose(fp);
            return 1;
        } else
            return -1;
    }
    return 0;
}

static int save_cipcfg_xml(char *xml_str, char *file, int len)
{
    int fd, target;
    FILE *fp;

    if (!xml_str)
        return CIPCFG_ERR_ILLEGAL_CFGFILE;
    if (!file || !*file)
        return CIPCFG_ERR_OPENFILE;
    target = get_cipcfg_lock_target(file);
    fd = lock_cipcfg(target, EX_LOCK);
    if (fd < 0)
        return CIPCFG_ERR_LOCK;
    fp = fopen(file, "w");
    if (!fp) {
        unlock_cipcfg(fd);
        return CIPCFG_ERR_OPENFILE;
    }
    if (len >= 0) {
	if (len > 0)
	    fwrite(xml_str, 1, len, fp);
	else
	    fputs(xml_str, fp);
    }
    fflush(fp);
    fclose(fp);
    unlock_cipcfg(fd);
    return CIPCFG_SUCCESS;
}

int save_cipcfg(CIPCFG *cipcfg, char *file)
{
    char *xml_str = NULL;
    int ret;

    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    xml_str = ezxml_toxml(cipcfg);
    ret = save_cipcfg_xml(xml_str, file, 0);
    if (xml_str != NULL)
        free(xml_str);
#if defined(CIPCFG_CFGARCV) && CIPCFG_CFGARCV == 1
        do_cfgarcv() ; // Do config archiving (defined in extension)
#endif
    return ret;
}

/**
 * Set config. Split set_list into multiple list for each group.
 * 
 * @param cipcfg	CIPCFG handle
 * @param set_list	items to be set
 * @param file          output file;
                        if  file == NULL => the same as set_cfg_items(cipcfg, set_list).
                           *file == '\0' => update cipcfg without save.
 * @return		0 on success, <0 on error
 */ 
int set_cfg_items_f(CIPCFG *cipcfg, ITEM_LIST *set_list, char *file)
{
    int ret = CIPCFG_SUCCESS, target;
    char *xml_str = NULL;

    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    if (!file) {
        int gid_list[MAX_GRPLIST_NUM], gid_idx = MAX_GRPLIST_NUM;
        if (!set_list)
            return ret;

        ret = put_cfg_items(cipcfg, set_list, gid_list, &gid_idx);
        if (ret != CIPCFG_SUCCESS)
            return ret;
        set_sync_file(gid_list, gid_idx);
        file = config_ram;
    } else if (set_list != NULL) {
        ret = put_cfg_items(cipcfg, set_list, NULL, NULL);
        if (ret != CIPCFG_SUCCESS)
            return ret;
    }

    if (!*file)
        return ret;

    xml_str = ezxml_toxml(cipcfg);

    ret = save_cipcfg_xml(xml_str, file, 0);
    if (ret != CIPCFG_SUCCESS)
        goto fail;

    target = get_cipcfg_lock_target(file);
    if (target == LOCK_CIPCFG_RAM) {
        ret = save_cipcfg_xml(xml_str, config_rom, 0);
    } else if (target == LOCK_CIPCFG_ROM) {
        int fd = lock_cipcfg(LOCK_CIPCFG_RAM, EX_LOCK);
        if (fd < 0) {
            ret = CIPCFG_ERR_LOCK;
        } else {
            unlink(config_ram);
            unlock_cipcfg(fd);
        }
    }

#if defined(CIPCFG_CFGARCV) && CIPCFG_CFGARCV == 1
    if (ret != CIPCFG_SUCCESS)
        goto fail;
    if (target == LOCK_CIPCFG_RAM || target == LOCK_CIPCFG_ROM)
        do_cfgarcv() ;// Do config archiving (defined in extension)
#endif

fail:
    if (xml_str != NULL)
        free(xml_str);
    return ret;
}

int set_cfg_items_dependency(CIPCFG *cipcfg, ITEM_LIST *set_list)
{
    int ret = CIPCFG_SUCCESS;
#ifdef CIPCFG_DEPENDENCY_CHECK
    if ((ret = dependency_check(set_list)) != CIPCFG_SUCCESS)
	return ret;
#endif
    return set_cfg_items_f(cipcfg, set_list, config_ram);
}

int set_cfg_items_lite(CIPCFG *cipcfg, ITEM_LIST *set_list)
{
    return set_cfg_items_f(cipcfg, set_list, config_ram);
}

/**
 * Set config. Split set_list into multiple list for each group.
 * 
 * @param cipcfg	CIPCFG handle
 * @param set_list	items to be set
 * @return		0 on success, <0 on error
 */ 
int set_cfg_items(CIPCFG *cipcfg, ITEM_LIST *set_list)
{
    return set_cfg_items_f(cipcfg, set_list, NULL);
}

int apply_cfg_group(CIPCFG *cipcfg, int gid)
{
    extern char *get_lan_ifname(char *);
    char buf[MAX_VALUE_LEN], tmp[MAX_VALUE_LEN], opmode[MAX_VALUE_LEN];
    int ret = CIPCFG_SUCCESS, pid;
    FILE *fp = NULL;

    if (!cipcfg)
	return CIPCFG_ERR_NODE_NOT_FOUND;

    get_cfg_item(cipcfg, GID_OPMODE, "type", opmode);
    cipcfg_print("apply gid %d\n", gid);
    memset(buf, 0, MAX_VALUE_LEN);
    memset(tmp, 0, MAX_VALUE_LEN);
    switch (gid)
    {
        case GID_DEVICE: //cipconfig.system.device
            cipcfg_print("Commit System Device information\n");
            break;
        case GID_SYSTIME: //cipconfig.system.systime
            cipcfg_print("Commit System Time information\n");
            sprintf(buf, "%s sync_ntp", PATH_RC_SYSTIME );
            cip_system(buf);
            break;
        case GID_OPMODE:
            cipcfg_print("Commit Operation Mode.\n");
            break;
        case GID_AUTO_REBOOT:
            cipcfg_print("Apply GID_AUTO_REBOOT\n");
            sprintf(buf, "%s restart_crond", PATH_RC_MGNT );
            cip_system(buf);
            break;
        case GID_LAN:
            cipcfg_print("Apply GID_LAN..\n");
            sprintf(buf, "%s restart_lan", PATH_RC_NET );
            cip_system(buf);
            break;
        case GID_WAN_IP: //cipconfig.network.wan.ip
            sprintf(buf, "%s setup_dns", PATH_RC_NET );
            cip_system(buf);
            // TODO:deside which rf run vap
            sprintf(buf, "%s restart 0", PATH_RC_WLAN );
            cip_system(buf);
            sprintf(buf, "%s restart", PATH_RC_BRIDGE);
            cip_system(buf);
            sprintf(buf, "%s restart_wan", PATH_RC_NET );
            cip_system(buf);
            /* hostapd relys on WLAN and bridge */
            sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
            cip_system(buf);
            // Bandwidth Control
            sprintf(buf, "%s BWC", PATH_FIREWALL_MOD);
            cip_system(buf);

            break;
        case GID_LAN_DHCP:
            cipcfg_print("Apply GID_LAN_DHCP..\n");
            get_lan_ifname(buf);
            cipcfg_print("LAN if: %s\n", buf);
            if(buf[0] != '\0'){
                sprintf(tmp, "%s %s restart", PATH_UDHCPD, buf);
                cipcfg_print("Exec: %s\n", tmp);
                cip_system(tmp);
            }
            break;
        case GID_UPNP:
            cipcfg_print("Apply GID_UPNP\n");
            sprintf(buf, "%s UPNP", PATH_FIREWALL_MOD);
            cip_system(buf);
            break;
        case GID_URL_FILTER:
            sprintf(buf, "%s URLF", PATH_FIREWALL_MOD );
            cip_system(buf);
            break;
        case GID_DMZ:
            sprintf(buf, "%s DMZ", PATH_FIREWALL_MOD );
            cip_system(buf);
            break;
        case GID_IP_REDIRECT:
            sprintf(buf, "%s RED", PATH_FIREWALL_MOD );
            cip_system(buf);
            break;
        case GID_NET_L2: //cipconfig.network.l2
            sprintf(buf, "%s restart", PATH_RC_BRIDGE);
            cip_system(buf);
            sprintf(buf, "%s restart_wan", PATH_RC_NET );
            cip_system(buf);
            /* hostapd relys on WLAN and bridge */
            sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
            cip_system(buf);
            break;
        case GID_DDNS:
            cipcfg_print("Apply DDNS...\n");
            cip_system(PATH_RC_DDNS);
            break;
        case GID_BFW:
	     cipcfg_print("Apply firewall...\n");
            sprintf(buf, "%s", PATH_BFW );
	     cip_system(buf);
	     break;
	 case GID_BFW_SERVICE:
	     cipcfg_print("Commit firewall service...\n");	    
	     break;
	 case GID_BFW_DAI:
	     cipcfg_print("Apply DAI...\n");	    
            sprintf(buf, "%s", PATH_DAI );
	     cip_system(buf);
	     break;
        case GID_ACCOUNT: //cipconfig.managmgent.accounts
        case GID_USER1: //cipconfig.managmgent.accounts.user:0   root
        case GID_USER2: //cipconfig.managmgent.accounts.user:1  admin
            /* it will implement on Web CGI */
            break;
        case GID_SNMP: //cipconfig.management.rmt_srv.snmp
            sprintf(buf, "%s restart_snmp", PATH_RC_MGNT );
            cip_system(buf);
	    /* Restart syslog because of the snmp trap */
            sprintf(buf, "%s restart_syslog", PATH_RC_MGNT );
            cip_system(buf);
            break;
	case GID_REMOTE_MANAGE: //cipconfig.management.remote_manage
            sprintf(buf, "%s write_mgmtip", PATH_RC_MGNT );
            cip_system(buf);
	    break;
	case GID_SMTP_REDIR: //cipconfig.management.smtp_red
	    sprintf(buf, "/ramfs/bin/smtp_redir.sh");
            cip_system(buf);
	    break;
	case GID_OD_TERM1:
	case GID_OD_TERM2:
	case GID_OD_TERM3:
	case GID_OD_TERM4:
	case GID_OD_TERM5:
	case GID_OD_TERM6:
	case GID_OD_TERM7:
	case GID_OD_TERM8:
	case GID_OD_TERM9:
	case GID_OD_TERM10:
	    sprintf(buf, "%s restart_ether_print", PATH_RC_MGNT);
	    cip_system(buf);
	    break;
        case GID_SYSLOG: //cipconfig.management.syslog
	case GID_USERLOG: //cipconfig.management.userlog
	case GID_USERLOG_SYSLOG: //cipconfig.management.userlog
	case GID_SESSIONLOG_SYSLOG: //cipconfig.management.sessionlog.syslog
            sprintf(buf, "%s restart_syslog", PATH_RC_MGNT );
            cip_system(buf);
            break;
	case GID_SESSIONLOG: //cipconfig.management.sessionlog
	    break;
	case GID_POLICY0_SESSION_LIMIT:
	case GID_POLICY1_SESSION_LIMIT:
	case GID_POLICY2_SESSION_LIMIT:
	case GID_POLICY3_SESSION_LIMIT:
	case GID_POLICY4_SESSION_LIMIT:
	case GID_POLICY5_SESSION_LIMIT:
	case GID_POLICY6_SESSION_LIMIT:
	case GID_POLICY7_SESSION_LIMIT:
	case GID_POLICY8_SESSION_LIMIT:
	case GID_POLICY9_SESSION_LIMIT:
	case GID_POLICY10_SESSION_LIMIT:
	case GID_POLICY11_SESSION_LIMIT:
	case GID_POLICY12_SESSION_LIMIT:
	case GID_POLICY13_SESSION_LIMIT:
	case GID_POLICY14_SESSION_LIMIT:
	case GID_POLICY15_SESSION_LIMIT:
	case GID_POLICY16_SESSION_LIMIT:
	    sprintf(buf, "/ramfs/bin/session_limit.sh");
	    cip_system(buf);
	    break;
	case GID_POLICY0_BANDWIDTH:
	case GID_POLICY1_BANDWIDTH:
	case GID_POLICY2_BANDWIDTH:
	case GID_POLICY3_BANDWIDTH:
	case GID_POLICY4_BANDWIDTH:
	case GID_POLICY5_BANDWIDTH:
	case GID_POLICY6_BANDWIDTH:
	case GID_POLICY7_BANDWIDTH:
	case GID_POLICY8_BANDWIDTH:
	case GID_POLICY9_BANDWIDTH:
	case GID_POLICY10_BANDWIDTH:
	case GID_POLICY11_BANDWIDTH:
	case GID_POLICY12_BANDWIDTH:
	case GID_POLICY13_BANDWIDTH:
	case GID_POLICY14_BANDWIDTH:
	case GID_POLICY15_BANDWIDTH:
	case GID_POLICY16_BANDWIDTH:
	    break;
	case GID_REPORT_AGENT: //cipconfig.management.report_agent
            sprintf(buf, "%s restart_agent", PATH_RC_MGNT );
            cip_system(buf);
            break;
        case GID_RF1: //cipconfig.wlan.rfcard:0
            if (apply_rf0 == FALSE) {
                sprintf(buf, "%s restart 0", PATH_RC_WLAN );
                cip_system(buf);
                /* bridge will depend on WLAN status */
                sprintf(buf, "%s restart", PATH_RC_BRIDGE );
                cip_system(buf);
                /* wan relys on WLAN and bridge */
                sprintf(buf, "%s restart_wan", PATH_RC_NET );
                cip_system(buf);
                /* hostapd relys on WLAN and bridge */
                sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
                cip_system(buf);
		/* cpe mode and universal repeater mode should get IP after start_hostapd */
                sprintf(buf, "%s check_dhcpc", PATH_RC_NET );
                cip_system(buf);
		
		if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		    fgets(buf, MAX_VALUE_LEN, fp);
		    fclose(fp);
		    pid = atoi(buf);
		    if (pid > 1)
			kill(pid, SIGUSR2);
		}
                apply_rf0 = TRUE;
            }
            break;
        case GID_RF2: //cipconfig.wlan.rfcard:1
            sprintf(buf, "%s restart 1", PATH_RC_WLAN );
            cip_system(buf);
            /* bridge will depend on WLAN status */
            sprintf(buf, "%s restart", PATH_RC_BRIDGE );
            cip_system(buf);
            /* wan relys on WLAN and bridge */
            sprintf(buf, "%s restart_wan", PATH_RC_NET );
            cip_system(buf);
	    /* hostapd relys on WLAN and bridge */
	    sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
	    cip_system(buf);
	    if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		fgets(buf, MAX_VALUE_LEN, fp);
		fclose(fp);
		pid = atoi(buf);
		if (pid > 1)
		    kill(pid, SIGUSR2);
	    }
            break;
        case GID_RF3: //cipconfig.wlan.rfcard:2
            sprintf(buf, "%s restart 2", PATH_RC_WLAN );
            cip_system(buf);
            /* bridge will depend on WLAN status */
            sprintf(buf, "%s restart", PATH_RC_BRIDGE );
            cip_system(buf);
            /* wan relys on WLAN and bridge */
            sprintf(buf, "%s restart_wan", PATH_RC_NET );
            cip_system(buf);
	    /* hostapd relys on WLAN and bridge */
	    sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
	    cip_system(buf);
	    if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		fgets(buf, MAX_VALUE_LEN, fp);
		fclose(fp);
		pid = atoi(buf);
		if (pid > 1)
		    kill(pid, SIGUSR2);
	    }
            break;
        case GID_RF1_VAP1: //cipconfig.wlan.rfcard:0.vap:0
        case GID_RF1_VAP2: //cipconfig.wlan.rfcard:0.vap:1
        case GID_RF1_VAP3: //cipconfig.wlan.rfcard:0.vap:2
        case GID_RF1_VAP4: //cipconfig.wlan.rfcard:0.vap:3
        case GID_RF1_VAP5: //cipconfig.wlan.rfcard:0.vap:4
        case GID_RF1_VAP6: //cipconfig.wlan.rfcard:0.vap:5
        case GID_RF1_VAP7: //cipconfig.wlan.rfcard:0.vap:6
        case GID_RF1_VAP8: //cipconfig.wlan.rfcard:0.vap:7
	case GID_RF1_STA1: //cipconfig.wlan.rfcard:0.sta:0
	case GID_RF1_REP1: //cipconfig.wlan.rfcard:0.rep:0
        case GID_RF1_VAP1_ACL: //cipconfig.wlan.rfcard:0.vap:0.macacl
        case GID_RF1_VAP2_ACL: //cipconfig.wlan.rfcard:0.vap:1.macacl
        case GID_RF1_VAP3_ACL: //cipconfig.wlan.rfcard:0.vap:2.macacl
        case GID_RF1_VAP4_ACL: //cipconfig.wlan.rfcard:0.vap:3.macacl
        case GID_RF1_VAP5_ACL: //cipconfig.wlan.rfcard:0.vap:4.macacl
        case GID_RF1_VAP6_ACL: //cipconfig.wlan.rfcard:0.vap:5.macacl
        case GID_RF1_VAP7_ACL: //cipconfig.wlan.rfcard:0.vap:6.macacl
        case GID_RF1_VAP8_ACL: //cipconfig.wlan.rfcard:0.vap:7.macacl
        case GID_RF1_VAP1_SEC: //cipconfig.wlan.rfcard:0.vap:0.security
        case GID_RF1_VAP2_SEC: //cipconfig.wlan.rfcard:0.vap:1.security
        case GID_RF1_VAP3_SEC: //cipconfig.wlan.rfcard:0.vap:2.security
        case GID_RF1_VAP4_SEC: //cipconfig.wlan.rfcard:0.vap:3.security
        case GID_RF1_VAP5_SEC: //cipconfig.wlan.rfcard:0.vap:4.security
        case GID_RF1_VAP6_SEC: //cipconfig.wlan.rfcard:0.vap:5.security
        case GID_RF1_VAP7_SEC: //cipconfig.wlan.rfcard:0.vap:6.security
        case GID_RF1_VAP8_SEC: //cipconfig.wlan.rfcard:0.vap:7.security
        case GID_RF1_STA1_SEC: //cipconfig.wlan.rfcard:0.sta:0.security
        case GID_RF1_REP1_SEC: //cipconfig.wlan.rfcard:0.rep:0.security
        case GID_RF1_WDS1: //cipcfongi.wlan.rfcard:0.wds:0
        case GID_RF1_WDS2: //cipcfongi.wlan.rfcard:0.wds:1
        case GID_RF1_WDS3: //cipcfongi.wlan.rfcard:0.wds:2
        case GID_RF1_WDS4: //cipcfongi.wlan.rfcard:0.wds:3
        case GID_RF1_WDS5: //cipcfongi.wlan.rfcard:0.wds:4
        case GID_RF1_WDS6: //cipcfongi.wlan.rfcard:0.wds:5
        case GID_RF1_WDS7: //cipcfongi.wlan.rfcard:0.wds:6
        case GID_RF1_WDS8: //cipcfongi.wlan.rfcard:0.wds:7
            ret = get_cfg_item(cipcfg, GID_RF1, "state", buf);
            if (ret) {return ret;}
            if (!strcmp(buf, "enable") && apply_rf0 == FALSE) {
                sprintf(buf, "%s restart 0", PATH_RC_WLAN );
                cip_system(buf);
                /* bridge will depend on WLAN status */
                sprintf(buf, "%s restart", PATH_RC_BRIDGE );
                cip_system(buf);
		if (!strcmp(opmode, "GW")) {
                    sprintf(buf, "%s restart_lan", PATH_RC_NET );
                    cip_system(buf);
		} else {
                    /* wan relys on WLAN and bridge */
                    sprintf(buf, "%s restart_wan", PATH_RC_NET );
                    cip_system(buf);
		}
                /* hostapd relys on WLAN and bridge */
                sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
                cip_system(buf);
		/* cpe mode and universal repeater mode should get IP after start_hostapd */
                sprintf(buf, "%s check_dhcpc", PATH_RC_NET );
                cip_system(buf);
		// snmp need to be restarted because there might be new interface
		if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		    fgets(buf, MAX_VALUE_LEN, fp);
		    fclose(fp);
		    pid = atoi(buf);
		    if (pid > 1)
			kill(pid, SIGUSR2);
		}
                apply_rf0 = TRUE;
            }
            break;
        case GID_RF2_WDS1: //cipcfongi.wlan.rfcard:1.wds:0
        case GID_RF2_WDS2: //cipcfongi.wlan.rfcard:1.wds:1
        case GID_RF2_WDS3: //cipcfongi.wlan.rfcard:1.wds:2
        case GID_RF2_WDS4: //cipcfongi.wlan.rfcard:1.wds:3
        case GID_RF2_WDS5: //cipcfongi.wlan.rfcard:1.wds:4
        case GID_RF2_WDS6: //cipcfongi.wlan.rfcard:1.wds:5
        case GID_RF2_WDS7: //cipcfongi.wlan.rfcard:1.wds:6
        case GID_RF2_WDS8: //cipcfongi.wlan.rfcard:1.wds:7
            ret = get_cfg_item(cipcfg, GID_RF2, "state", buf);
            if (ret) {return ret;}
            if (!strcmp(buf, "enable")) {
                sprintf(buf, "%s restart 1", PATH_RC_WLAN );
                cip_system(buf);
		/* bridge will depend on WLAN status */
		sprintf(buf, "%s restart", PATH_RC_BRIDGE );
		cip_system(buf);
		if (!strcmp(opmode, "GW")) {
                    sprintf(buf, "%s restart_lan", PATH_RC_NET );
                    cip_system(buf);
		} else {
                    /* wan relys on WLAN and bridge */
                    sprintf(buf, "%s restart_wan", PATH_RC_NET );
                    cip_system(buf);
		}
		/* hostapd relys on WLAN and bridge */
		sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
		cip_system(buf);
		// snmp need to be restarted because there might be new interface
		if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		    fgets(buf, MAX_VALUE_LEN, fp);
		    fclose(fp);
		    pid = atoi(buf);
		    if (pid > 1)
			kill(pid, SIGUSR2);
		}
            }
            break;
        case GID_RF3_WDS1: //cipcfongi.wlan.rfcard:2.wds:0
        case GID_RF3_WDS2: //cipcfongi.wlan.rfcard:2.wds:1
        case GID_RF3_WDS3: //cipcfongi.wlan.rfcard:2.wds:2
        case GID_RF3_WDS4: //cipcfongi.wlan.rfcard:2.wds:3
        case GID_RF3_WDS5: //cipcfongi.wlan.rfcard:2.wds:4
        case GID_RF3_WDS6: //cipcfongi.wlan.rfcard:2.wds:5
        case GID_RF3_WDS7: //cipcfongi.wlan.rfcard:2.wds:6
        case GID_RF3_WDS8: //cipcfongi.wlan.rfcard:2.wds:7
            ret = get_cfg_item(cipcfg, GID_RF3, "state", buf);
            if (ret) {return ret;}
            if (!strcmp(buf, "enable")) {
                sprintf(buf, "%s restart 2", PATH_RC_WLAN );
                cip_system(buf);
		/* bridge will depend on WLAN status */
		sprintf(buf, "%s restart", PATH_RC_BRIDGE );
		cip_system(buf);
		if (!strcmp(opmode, "GW")) {
                    sprintf(buf, "%s restart_lan", PATH_RC_NET );
                    cip_system(buf);
		} else {
                    /* wan relys on WLAN and bridge */
                    sprintf(buf, "%s restart_wan", PATH_RC_NET );
                    cip_system(buf);
		}
		/* hostapd relys on WLAN and bridge */
		sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
		cip_system(buf);
		// snmp need to be restarted because there might be new interface
		if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		    fgets(buf, MAX_VALUE_LEN, fp);
		    fclose(fp);
		    pid = atoi(buf);
		    if (pid > 1)
			kill(pid, SIGUSR2);
		}
            }
            break;
        case GID_RF4_WDS1: //cipcfongi.wlan.rfcard:3.wds:0
        case GID_RF4_WDS2: //cipcfongi.wlan.rfcard:3.wds:1
        case GID_RF4_WDS3: //cipcfongi.wlan.rfcard:3.wds:2
        case GID_RF4_WDS4: //cipcfongi.wlan.rfcard:3.wds:3
        case GID_RF4_WDS5: //cipcfongi.wlan.rfcard:3.wds:4
        case GID_RF4_WDS6: //cipcfongi.wlan.rfcard:3.wds:5
        case GID_RF4_WDS7: //cipcfongi.wlan.rfcard:3.wds:6
        case GID_RF4_WDS8: //cipcfongi.wlan.rfcard:3.wds:7
            ret = get_cfg_item(cipcfg, GID_RF4, "state", buf);
            if (ret) {return ret;}
            if (!strcmp(buf, "enable")) {
                sprintf(buf, "%s restart 3", PATH_RC_WLAN );
                cip_system(buf);
		/* bridge will depend on WLAN status */
		sprintf(buf, "%s restart", PATH_RC_BRIDGE );
		cip_system(buf);
		if (!strcmp(opmode, "GW")) {
                    sprintf(buf, "%s restart_lan", PATH_RC_NET );
                    cip_system(buf);
		} else {
                    /* wan relys on WLAN and bridge */
                    sprintf(buf, "%s restart_wan", PATH_RC_NET );
                    cip_system(buf);
		}
		/* hostapd relys on WLAN and bridge */
		sprintf(buf, "%s start_hostapd", PATH_RC_WLAN );
		cip_system(buf);
		// snmp need to be restarted because there might be new interface
		if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		    fgets(buf, MAX_VALUE_LEN, fp);
		    fclose(fp);
		    pid = atoi(buf);
		    if (pid > 1)
			kill(pid, SIGUSR2);
		}
            }
            break;
        case GID_RF4: //cipconfig.wlan.rfcard:3
        case GID_RF4_SCAN: //cipcfongi.wlan.rfcard:3.scan
            sprintf(buf, "%s restart 3", PATH_RC_WLAN );
            cip_system(buf);
	    if ((fp=fopen(SNMPD_PID_FILE, "r")) != NULL) {
		fgets(buf, MAX_VALUE_LEN, fp);
		fclose(fp);
		pid = atoi(buf);
		if (pid > 1)
		    kill(pid, SIGUSR2);
	    }
            break;
	case GID_MGMT_AUTH:
	    break;
	case GID_MGMT_AUTH2_MAP:
	case GID_MGMT_AUTH3_MAP:
	case GID_MGMT_AUTH4_MAP:
	case GID_MGMT_AUTH5_MAP:
	    sprintf(buf, "/etc/rc.d/rc.radius write_map");
            cip_system(buf);
	    break;
	case GID_MGMT_AUTH6:
	case GID_MGMT_AUTH6_TICKET:
	    sprintf(buf, "%s reload_ether_print", PATH_RC_MGNT);
	    cip_system(buf);
	    break;
	case GID_BLACKLIST1:
	case GID_BLACKLIST2:
	case GID_BLACKLIST3:
	case GID_BLACKLIST4:
	case GID_BLACKLIST5:
	case GID_BLACKLIST6:
	    sprintf(buf, "%s write_blacklist", PATH_RC_MGNT);
	    cip_system(buf);
	    break;
	case GID_PRIVILEGEIP:
	    sprintf(buf, "/ramfs/bin/ip_pass.sh");
	    cip_system(buf);
	    break;
	case GID_PRIVILEGEMAC:
	    sprintf(buf, "/ramfs/bin/mac_pass.sh");
	    cip_system(buf);
	    break;
 
        default:
	    return CIPCFG_ERR_UNKNOWN_GROUP;
    }    

    return CIPCFG_SUCCESS;
}

int apply_cfg(CIPCFG *cipcfg)
{
/* Save the cipcfg to xml file on ram disk to represent 
 * the current status on system. 
 * In this function apply_cfg() now, it will apply ALL saved config groups.
 * Note: This must be done before any group been applied because scripts need
 * current configuration.
 */
    int i, ret, cfg_change = FALSE;

    ret = save_cipcfg(cipcfg, current_config);
    if (ret != CIPCFG_SUCCESS)
	return ret;

    for (i=0; group_table[i].gid; i++) {
        if (group_table[i].dirty == TRUE) {
            group_table[i].dirty = FALSE;
	    if (apply_cfg_group(cipcfg, group_table[i].gid)!=CIPCFG_SUCCESS) {
	        cipcfg_print("<error>apply_cfg_group gid %d\n", group_table[i].gid);
	        ret = CIPCFG_ERR_UNKNOWN_GROUP;
	    }
            cfg_change = TRUE;
        }
    }

    if (cfg_change == TRUE) {
        apply_rf0 = FALSE;
    }
    return ret;
}


/**
 * Encrypt xml format string cfgstr and store the result to buffer allocated.
 * Caller is responsible to free the buffer.
 * 
 * Currently no encryption is done hence simply copy cfgstr to buffer.   
 *  
 * @param cipstr	string represented the configuration in XML format
 * @param buffer	buffer used to store config file, has to be freed by
 * 			caller
 * @return		length of config file if success, <0 on error
 */
char *encrypt_cipcfg(unsigned char *cfg, int len)
{
    int i;

    if (!cfg)
	return NULL;
    for (i=0; i<len; i++)
    {
        cfg[i] = ~cfg[i] + i%64;
    }
    return cfg;
}

/**
 * Decrypt encrypted config file stored in buffer and return string represent
 * XML format of the configuration.
 * 
 * Currently no encryption is done hence returns a copy of buffer.   
 *  
 * @param buffer	buffer storing encrypted config file
 * @param len		length of buffer
 * @return		a string represent XML configuration on success, NULL on
 * 			error
 */
char *decrypt_cipcfg(unsigned char *cfg, int len)
{
    int i;

    if (!cfg)
	return NULL;
    for (i=0; i<len; i++)
    {
        cfg[i] = ~(cfg[i] - i%64);
    }
    return cfg;
}

/**
 * Retrieve configuration and translate to user-level config file format.
 *  
 * @param cipcfg	the configuration
 * @param buffer	buffer used to store config file, has to be freed by
 * 			caller
 * @return		length of config file if success, <0 on error
 */
int backup_cipcfg(char *backup_file)
{
    CIPCFG *cipcfg = NULL; 
    unsigned char *cfgstr, magbuf[MAGIC_LEN + 1];
    const char *devtype, *version;
    int fd, len, ret;
    FILE *fp = NULL;
    CFG_HEADER cfg_hdr;

    if (!backup_file)
	return CIPCFG_ERR_OPENFILE;

    cipcfg = init_cipcfg();
    if (!cipcfg) {
        cipcfg_print("initial cipcfg error.\n");
        return CIPCFG_ERR_OPENFILE;
    }

    memset(magbuf, 0, MAGIC_LEN + 1);
    memset(&cfg_hdr, 0, sizeof(CFG_HEADER));
    cfgstr = ezxml_toxml(cipcfg);
    if (!cfgstr) {
	close_cipcfg(cipcfg);
	return CIPCFG_ERR_OPENFILE;
    }
    len = strlen(cfgstr);
    cipcfg_print("backupCfg cfg len = %d\n", len);

    devtype = ezxml_attr(cipcfg, "devtype");
    version = ezxml_attr(cipcfg, "version");
    if (!devtype || !version) {
	free(cfgstr);
	close_cipcfg(cipcfg);
	return CIPCFG_ERR_OPENFILE;
    }
    // create CFG_HEADER
    strcpy(cfg_hdr.hw_type, devtype);
    strcpy(cfg_hdr.hw_ver, SOFTWARE_VERSION);
    strcpy(cfg_hdr.cfg_ver, version);
    close_cipcfg(cipcfg);

    cfg_hdr.cfg_len = len;
    cfg_hdr.cfg_sum = checksum(cfgstr, len);
    ret = prd_tbl_lookup(magic, magbuf, NULL);
    if (ret != CIPCFG_SUCCESS) {
        //error
	free(cfgstr);
        return ret;
    }
    strncpy(cfg_hdr.magic, magbuf, MAGIC_LEN);

    fd = lock_cipcfg(LOCK_CIPCFG_BACKUP, EX_LOCK);
    if (fd < 0) {
	free(cfgstr);
        return CIPCFG_ERR_LOCK;
    }

    // remove file first
    remove(backup_file);

    fp = fopen(backup_file, "wb");
    if (fp == NULL) {
	free(cfgstr);
        unlock_cipcfg(fd);
        cipcfg_print("Can't open \"%s\"\n", backup_file);
        return CIPCFG_ERR_OPENFILE;
    }
    fwrite(&cfg_hdr, 1, sizeof(cfg_hdr), fp);
    encrypt_cipcfg(cfgstr, len);
    fwrite(cfgstr, 1, len, fp);

    free(cfgstr);
    fclose(fp);
    unlock_cipcfg(fd);
    
    return SYS_BACKUP_SUCCESS;
}

static unsigned int invert_value(unsigned int val)
{
	int i;
	unsigned int ret = 0;

	for (i = 0; i < sizeof(val); i++) {
#if 0
		ret |= (((val >> (i*8)) & 0xff) << ((sizeof(val)-1-i)*8));
#else
		ret <<= 8;
		ret |= (val & 0xff);
		val >>= 8;
#endif
	}
	return ret;
}

/**
 * Decrypt configuration file
 *  
 * @param buffer	contains the configuration file
 * @param len		buffer length
 * @return		0 on success, <0 on error
 */
int decrypt_uploaded_cipcfg(unsigned char *upload_cfg, int len)
{
    int ret;
    unsigned char *cfgstr, *cfg, magbuf[MAGIC_LEN + 1];
    CFG_HEADER *cfg_hdr;
    unsigned int cfg_len, cfg_sum;
    int invert = 0;

    if (!upload_cfg)
	return CIPCFG_ERR_ILLEGAL_CFGFILE;
    cipcfg_print("upload file len = %d\n", len);
    if (len < sizeof(CFG_HEADER)) {
        cipcfg_print("upload file too small, length is %d\n", len);
        return CIPCFG_ERR_INVALID_LENGTH;
    }

    cfg_hdr = (CFG_HEADER *)upload_cfg;
    cfg_len = cfg_hdr->cfg_len;
    if (len != (sizeof(CFG_HEADER)+cfg_len)) {
	cfg_len = invert_value(cfg_len);
	if (len != (sizeof(CFG_HEADER)+cfg_len))
	    return CIPCFG_ERR_INVALID_LENGTH;
	invert = 1;
    }

    cfg     = upload_cfg + sizeof(CFG_HEADER);
    cipcfg_print("uploadCfg cfg len: %d\n", cfg_len);

    cfgstr = decrypt_cipcfg(cfg, cfg_len);
    if (!cfgstr) {
    	return CIPCFG_ERR_ILLEGAL_CFGFILE;
    }
    cfg_sum = (invert != 0) ? invert_value(cfg_hdr->cfg_sum) : cfg_hdr->cfg_sum;
    if (cfg_sum != checksum(cfg, cfg_len)) {
        cipcfg_print("Checksum mismatch!!!, correct checksum is %d\n", cfg_sum);
        return SYS_ERR_CHECK_SUM;
    }


    memset(magbuf, 0, MAGIC_LEN + 1); // add a char to the end to set '\0'
    ret = prd_tbl_lookup(magic, magbuf, NULL);
    if( ret != CIPCFG_SUCCESS ){
        return ret;
    }
    if (strncmp(cfg_hdr->magic, magbuf, MAGIC_LEN)) {
        cipcfg_print("Magic number mismatch!!!, the wrong magic is %s\n", cfg_hdr->magic);
        return SYS_ERR_MAGIC_NUM;
    }

    if (invert != 0) {
	cfg_hdr->cfg_len = cfg_len;
	cfg_hdr->cfg_sum = cfg_sum;
    }

    return SYS_UPLOAD_SUCCESS;
}

/**
 * Decrypt configuration file and store it.
 *  
 * @param buffer	contains the configuration file
 * @param len		buffer length
 * @return		0 on success, <0 on error
 */
int restore_cipcfg(unsigned char *upload_cfg, int len)
{
    int ret;
    unsigned char *cfg;
    CFG_HEADER *cfg_hdr;

    ret = decrypt_uploaded_cipcfg(upload_cfg, len);
    if (ret != SYS_UPLOAD_SUCCESS)
	return ret;

    cfg_hdr = (CFG_HEADER *)upload_cfg;
    cfg = upload_cfg + sizeof(CFG_HEADER);

    ret = save_cipcfg_xml(cfg, restored_cfg_file, (cfg_hdr->cfg_len > 0) ? cfg_hdr->cfg_len : -1);
    if (ret != CIPCFG_SUCCESS)
	return ret;

#if defined(CIPCFG_CFGARCV) && CIPCFG_CFGARCV == 1
    do_cfgarcv() ; // Do config archiving (defined in extension)
#endif

    return SYS_UPLOAD_SUCCESS;
}

void upperCase( char *str)
{
    int i, len;
    if (!str)
	return;
    len = strlen(str);
    for(i=0; i<len; i++) {
        str[i]=toupper(str[i]);
    }
    return;
}

/**
 * Get the user group.
 *  
 * @param name	user name
 * @return		> 0 : User group
 *              < 0 : Error
 */
int getUserGroup(const char *name)
{
    int                 ret = USER_GROUP_NONE;
    user_group_row_t    *u;

    if (!name)
	return ret;
    for(u = user_table;u->u_gid;u++){
        if(strcmp(u->u_name, name) == 0){
            ret = u->u_gid;
            break;
        }
    }
    return ret;
}

/**
 * Get Config permission by group name
 *  
 * @param       group name
 * @return		group permission
 */
unsigned int getConfigPermissionByName(const char *g_name)
{
    unsigned int        ret = PERM_NONE;
    permission_row_t    *p;
    
    if (!g_name)
	return ret;
    for(p = conf_perm_table;p->gid;p++){
        if(strcmp(p->name, g_name) == 0){
            ret = p->permission;
            break;
        }
    }
    return ret;
}

/**
 * Get Config permission by group ID
 *  
 * @param       group name
 * @return		group permission
 */
unsigned int getConfigPermissionByGID(const unsigned int g_id)
{
    unsigned int        ret = PERM_NONE;
    permission_row_t    *p;
    
    for(p = conf_perm_table;p->gid;p++){
        if(p->gid == g_id){
            ret = p->permission;
            break;
        }
    }
    return ret;
}

/**
 * Get Operation permission by name
 *  
 * @param       group name
 * @return		group permission
 */
unsigned int getOperationPermissionByName(const char *o_name)
{
    unsigned int        ret = PERM_NONE;
    permission_row_t    *p;
    
    if (!o_name)
	return ret;
    for(p = oper_perm_table;p->gid;p++){
        if(strcmp(p->name, o_name) == 0){
            ret = p->permission;
            break;
        }
    }
    return ret;
}

/**
 * Get Operation permission by operation ID
 *  
 * @param       group name
 * @return		group permission
 */
unsigned int getOperationPermissionByOID(const unsigned int o_id)
{
    unsigned int        ret = PERM_NONE;
    permission_row_t    *p;
    
    for(p = oper_perm_table;p->gid;p++){
        if(p->gid == o_id){
            ret = p->permission;
            break;
        }
    }
    return ret;
}

ITEM_LIST *set_related_items(int grpgid, char* itemname, char* itemvalue)
{
    ITEM_LIST *tmp = NULL;

    if (!itemname || !itemvalue)
	return tmp;
    switch(grpgid)
    {
#ifdef SET_RFCARD_ROLES
        case GID_OPMODE:
            if(!strcmp(itemname, "type"))
            {
                tmp = set_rfcard_roles(itemvalue);
            }
            break;
#endif
        case GID_RF1:
#ifdef LEAVE_REP
            if(!strcmp(itemname, "roles") && (!strcmp(itemvalue, "ap,wds") || !strcmp(itemvalue, "ap")))
            {
                leave_rep();
            }
#endif
        case GID_RF2:
        case GID_RF3:
        case GID_RF4:
#ifdef SET_TIMEOUT_RELATED
            if(!strcmp(itemname, "distance"))
            {
                tmp = set_timeout(grpgid, itemvalue); 
            }
#endif
#ifdef SET_SCAN_RELATED
            if(!strcmp(itemname, "state"))
            {
                tmp = set_scan(grpgid, itemvalue); 
            }
#endif
            break;
        case GID_RF1_VAP1_SEC: 
        case GID_RF1_VAP2_SEC: 
        case GID_RF1_VAP3_SEC: 
        case GID_RF1_VAP4_SEC: 
        case GID_RF1_VAP5_SEC: 
        case GID_RF1_VAP6_SEC: 
        case GID_RF1_VAP7_SEC: 
        case GID_RF1_VAP8_SEC:
        case GID_RF2_VAP1_SEC: 
        case GID_RF2_VAP2_SEC: 
        case GID_RF2_VAP3_SEC: 
        case GID_RF2_VAP4_SEC: 
        case GID_RF2_VAP5_SEC: 
        case GID_RF2_VAP6_SEC: 
        case GID_RF2_VAP7_SEC: 
        case GID_RF2_VAP8_SEC: 
        case GID_RF3_VAP1_SEC: 
        case GID_RF3_VAP2_SEC: 
        case GID_RF3_VAP3_SEC: 
        case GID_RF3_VAP4_SEC: 
        case GID_RF3_VAP5_SEC: 
        case GID_RF3_VAP6_SEC: 
        case GID_RF3_VAP7_SEC: 
        case GID_RF3_VAP8_SEC: 
        case GID_RF4_VAP1_SEC: 
        case GID_RF4_VAP2_SEC: 
        case GID_RF4_VAP3_SEC: 
        case GID_RF4_VAP4_SEC: 
        case GID_RF4_VAP5_SEC: 
        case GID_RF4_VAP6_SEC: 
        case GID_RF4_VAP7_SEC: 
        case GID_RF4_VAP8_SEC: 
#ifdef SET_SEC_TYPE_RELATED 
            if(!strcmp(itemname, "mode"))
                tmp = set_security_type(grpgid, itemvalue);
#endif

// FIXME : Solve multi-group dependency problem in set_cfg_items()
#if 0
#ifdef SET_WEP_RELATED
            if(!strncmp(itemname, "wep.key", 7) || 
               !strcmp(itemname, "wep.method") || 
               !strcmp(itemname, "wep.keylen"))
                tmp = set_wep(grpgid, itemname, itemvalue);
#endif
#endif
            break;
#ifdef SET_REP_RELATED
	case GID_RF1_REP1:
            if(!strcmp(itemname, "state") && !strcmp(itemvalue, "enable"))
            {
                tmp = set_rep_related();
            }
	    break;
#endif
        default:
            break;
    }
    return tmp;
}

/* TODO : Can't hadle the case: 
 * In same group id, it has two or more same child tag, that has also child tag
 *
 * This function would get all attributes of nodes which is in the same group as CIPCFG *node.
 * (It is converted by original recursive version.)
 *
 * @param     node      The cipcfg node.
 * @param     prefix    The node path prefix in configuration file.
 * @param     count     Sequence number of node which have the same name with others.(ezxml next node)
 * @return    head      List of all attributes.
 */
struct attribute *get_attr(CIPCFG *node, char *prefix, int count, int next_flag)
{
    int i, is_group=0, idx;
    struct attribute *head= NULL, *ret= NULL, *prev= NULL;
    /* Stack index */
    int tail=0;
    /* Stacks */
    char *prefix_stack[STACK_MAX_NUM];
    int idx_stack[STACK_MAX_NUM];
    CIPCFG *cfg_node_stack[STACK_MAX_NUM];
    /* Current parameters */
    int cur_idx;
    char *cur_prefix = NULL;
    CIPCFG *cur_node;

    if (!node || !prefix)
	return NULL;
    prefix_stack[tail] = strdup(prefix);
    if (!prefix_stack[tail])
	return NULL;
    idx_stack[tail] = count;
    cfg_node_stack[tail] = node;

    tail++;
    while (tail > 0) {
        tail--;
        cur_node = cfg_node_stack[tail];
        cur_idx = idx_stack[tail];
        cur_prefix = prefix_stack[tail];

	if (cur_idx >= 0) {
	    is_group = 0;
	    /* if the prefix is group prefix and count=0, exit */
	    for (i=0; group_table[i].gid; i++) {
		if (strstr(group_table[i].prefix, cur_prefix)) {
		    is_group = 1;
		    break;
		}
	    }
	    if (is_group != 0) {
		free(cur_prefix);
		continue;
	    }
	}

	idx = -1;
	if ((cur_node->next || cur_idx > 0) && cur_idx != -1)
	    idx = cur_idx ; /* use count as index for output */
        i=0;
        while (cur_node->attr[i] != NULL && cur_node->attr[i+1] != NULL)
        {
            ret = get_new_attr(cur_node->attr[i], cur_node->attr[i+1]);
	    if (!ret)
		goto fail;
	    ret->idx = idx;

            if(prev != NULL)
                prev -> next = ret;
            if(head == NULL)
                head = ret;
            prev = ret;

            i = i+2;
        }

        if (cur_node->sibling) {
	    char *ptr;
	    int plen;
	    ptr = strrchr(cur_prefix, '.');
	    if (!ptr) {
		free(cur_prefix);
		continue;
	    }
	    plen = strlen(cur_prefix) - strlen(ptr);
	    prefix_stack[tail] = malloc(plen+strlen(cur_node->sibling->name)+2);
	    if (!prefix_stack[tail])
		goto fail;
	    strncpy(prefix_stack[tail], cur_prefix, plen);
	    ptr = prefix_stack[tail]+plen;
	    sprintf(ptr, ".%s", cur_node->sibling->name);
            idx_stack[tail] = 0;
            cfg_node_stack[tail] = cur_node->sibling;
            tail++;
        }

        if (cur_node->next) {
	    prefix_stack[tail] = strdup(cur_prefix);
	    if (!prefix_stack[tail])
		goto fail;
            idx_stack[tail] = cur_idx+1;
            cfg_node_stack[tail] = cur_node->next;
            tail++;
        }

        if (cur_node->child) {
	    prefix_stack[tail] = malloc(strlen(cur_prefix)+strlen(cur_node->child->name)+2);
	    if (!prefix_stack[tail])
		goto fail;
	    sprintf(prefix_stack[tail], "%s.%s", cur_prefix, cur_node->child->name);
            idx_stack[tail] = 0;
            cfg_node_stack[tail] = cur_node->child;
            tail++;
        }
        free(cur_prefix);
    }
    return head;
fail:
    free_attr_list(head);
    for (i=0; i<tail; i++)
	free(prefix_stack[i]);
    if (cur_prefix != NULL)
	free(cur_prefix);
    return NULL;
}

int check_sync_file(int *gid, int *match, int *count) 
{
    FILE *fp;
    char buf[MAX_GRPLIST_STRING_LEN];
    char *gid_str_head, *gid_str;
    int i=0;

    if (match != NULL)
	*match = 0;
    if (count != NULL)
	*count = 0;
    if (!gid || !match || !count)
	return -1;

    gid_str_head = gid_str = malloc(sizeof(char) * MAX_GRPLIST_STRING_LEN);
    if (!gid_str_head)
	return -1;

    for(i=0; gid[i] > GID_END; i++); 
    qsort(gid, i, sizeof(int), int_compare);  
    for(i=0; gid[i] > GID_END; i++) {
        sprintf(gid_str, "%d ", gid[i]);
        gid_str += strlen(gid_str);
    }
    sprintf(gid_str, "\n");

    fp = fopen(PATH_SYNC_CFG, "r");
    if (fp == NULL) {
	free(gid_str_head);
	return 0;
    }

    while(fgets(buf, MAX_GRPLIST_STRING_LEN, fp)) {
        /* Skip the '\n' in buf */
        if(strcmp(buf, gid_str_head) == 0)
            *match = *match + 1;

        *count = *count + 1;
    }
    fclose(fp);
    free(gid_str_head);

    return 1;
}
/*
  @param    gid 	A list of group IDs that are allowed to be applied. 
*/
int can_apply(int *gid)
{
    int match = 0, count = 0;
    int ret;
    ret = check_sync_file(gid, &match, &count);

    if (ret < 0)
	return CIPCFG_ERR_CFG_SYNC;
    if (ret == 0) 
        return CIPCFG_SUCCESS;

    if((match < 1 && count != 0) || count > 1) 
        return CIPCFG_ERR_CFG_SYNC;
    else 
        return CIPCFG_SUCCESS;
}

int chkGIDdup(int *applylist, int num_applist, int gid)
{
    int i=0;

    if (!applylist)
	return -1;

    for(i=0; i<num_applist; i++)
    {
        if(applylist[i] == gid)
            break;
    }
    if(i == num_applist)
        return 0;
    else
        return -1;
}

int cipcfg_print(char *fmt, ...)
{
    static int std_print = -1;
    int ret = 0;
    va_list ap;

    if (std_print == -1) {
	if ((getenv(CIPCFG_PRINT_ERR) == NULL) || strcmp(getenv(CIPCFG_PRINT_ERR), "0"))
	    std_print = 1;
	else
	    std_print = 0;
    }
    if (std_print != 0) {
	va_start(ap, fmt);
	ret = vfprintf(stderr, fmt, ap);
	va_end(ap);
    }
    return ret;
}

