#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cipcfg.h"
#include "id.h"
extern const int gid_vap_sec[][MAX_VAP]; 
extern const int max_rf;
extern const int gid_scan[];
extern const int gid_rf[];
extern const char *current_config;

#ifndef EXT_SET_RFCARD_ROLES
#ifdef SET_RFCARD_ROLES
ITEM_LIST *set_rfcard_roles(char *itemvalue)
{

    ITEM_LIST *tmp=NULL;

    if (!itemvalue)
	return NULL;
    if(!strcmp(itemvalue, "AP")) {
        tmp = append_cfg_item(tmp, GID_RF1, "roles", "ap");
    }
    else if(!strcmp(itemvalue, "CPE")) {
        tmp = append_cfg_item(tmp, GID_RF1, "roles", "sta");
    }
    return tmp;
}
#endif
#endif
    
#ifdef SET_SEC_TYPE_RELATED 
/* 32 security groups use the same function */
ITEM_LIST *set_security_type(int grpgid, char *itemvalue)
{
    ITEM_LIST *tmp, *head = NULL;

    if (!itemvalue)
	return NULL;
    if(!strcmp(itemvalue, "open") || !strcmp(itemvalue, "auto") || !strcmp(itemvalue, "shared")) 
    {
	tmp = get_new_item(grpgid, "wep.enabled", "enable");
	if (!tmp)
	    return NULL;
        // TODO - Find a way to move to upper level(the function caller)
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
	tmp = get_new_item(grpgid, "wpa.eap", "psk");
	if (!tmp) {
	    free_item_list(head);
	    return NULL;
	}
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
    }
    else if(!strcmp(itemvalue, "8021x")) 
    {
	tmp = get_new_item(grpgid, "wep.enabled", "disable");
	if (!tmp)
	    return NULL;
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
	tmp = get_new_item(grpgid, "wpa.eap", "psk");
	if (!tmp) {
	    free_item_list(head);
	    return NULL;
	}
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
    }
    else if(!strcmp(itemvalue, "wpa")) 
    {
	tmp = get_new_item(grpgid, "wep.enabled", "disable");
	if (!tmp)
	    return NULL;
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
    }
    return head;
}
#endif

#ifdef SET_WEP_RELATED 
ITEM_LIST *set_wep(int grpgid, char *itemname, char *itemvalue)
{
    ITEM_LIST *tmp, *head = NULL;
    int i;
    int rfidx=0;

    if (!itemname || !itemvalue)
	return NULL;
    for(i=0;i< MAX_VAP;i++)
    {
	tmp = get_new_item(gid_vap_sec[rfidx][i], itemname, itemvalue);
	if (!tmp) {
	    free_item_list(head);
	    return NULL;
	}
        printf("%s: Set %s to %s\n", __func__, tmp->item_name, tmp->item_value);
        tmp->next = head;
        head = tmp;
    }
    return head;
}
#endif

#ifdef SET_TIMEOUT_RELATED 
ITEM_LIST *set_timeout(int grpgid, char *distance_value)
{
    ITEM_LIST *tmp, *head = NULL;
    char item_value[MAX_VALUE_LEN];
    int d, s;

    if (!distance_value)
	return NULL;
    d = atoi(distance_value);
    s = 9+(d/300);
    if(d%300 != 0)
        s+=1;

    sprintf(item_value, "%d", s);    
    tmp = get_new_item(grpgid, "slottime", item_value);
    if (!tmp)
	return NULL;
    tmp->next = head;
    head = tmp;

    sprintf(item_value, "%d", ((s*2)+3));
    tmp = get_new_item(grpgid, "acktimeout", item_value);
    if (!tmp) {
	free_item_list(head);
	return NULL;
    }
    tmp->next = head;
    head = tmp;

    sprintf(item_value, "%d", ((s*2)+3));
    tmp = get_new_item(grpgid, "ctstimeout", item_value);
    if (!tmp) {
	free_item_list(head);
	return NULL;
    }
    tmp->next = head;
    head = tmp;
    return head;
}
#endif

#ifdef SET_SCAN_RELATED
ITEM_LIST *set_scan(int grpgid, char *itemvalue)
{
    ITEM_LIST *head = NULL;
    int rfidx=0;
    CIPCFG *cipcfg = NULL;
    char roles[MAX_VALUE_LEN];
    int i;

    if (!itemvalue)
	return NULL;
    /* get rfidx */
    for(i=0; i<max_rf; i++)
    {
        if(gid_rf[i] == grpgid)
            rfidx = i;
    }

    if ((cipcfg = init_cipcfg_f(current_config)) == NULL) {
        printf("Init cipcfg error.\n");
        return head;
    }

    if((get_cfg_item(cipcfg, grpgid, "roles", roles)) != CIPCFG_SUCCESS)
    {
        printf("Unable to get roles from cipcfg.\n");
        close_cipcfg(cipcfg);
        return head; 
    }

    /* If the role is scan */
    if(!strcmp(roles, "scan"))
    {
        printf("Automatically change scan.\n");
	head = get_new_item(gid_scan[rfidx], "state", (!strcmp(itemvalue, "enable")) ? "enable" : "disable");
	if (head != NULL)
	    head->next = NULL;
    }
    close_cipcfg(cipcfg);
    return head;
}
#endif
