#include <string.h>
#include <ctype.h>
#include <regex.h>
#include "id.h"
#include "cipcfg_validate.h"

extern const STRING_PAIR zone_info[];
char err_msg[CIPCFG_ERR_MSG_MAXLEN];
struct ascii_pair {
    char *hex;
    char character;
};

static struct ascii_pair ascii_tbl[]={
    { "20", ' '}, { "21", '!'}, { "22", '\"'}, { "23", '#'}, { "24", '$'}, { "25", '%'}, { "26", '&'}, 
    { "27", '\''}, { "28", '('}, { "29", ')'}, { "2a", '*'}, { "2b", '+'}, { "2c", ','}, { "2d", '-'},
    { "2e", '.'}, { "2f", '/'}, { "30", '0'}, { "31", '1'}, { "32", '2'}, { "33", '3'}, { "34", '4'}, 
    { "35", '5'}, { "36", '6'}, { "37", '7'}, { "38", '8'}, { "39", '9'}, { "3a", ':'}, { "3b", ';'},
    { "3c", '<'}, { "3d", '='}, { "3e", '>'}, { "3f", '?'}, { "40", '@'}, { "41", 'A'}, { "42", 'B'},
    { "43", 'C'}, { "44", 'D'}, { "45", 'E'}, { "46", 'F'}, { "47", 'G'}, { "48", 'H'}, { "49", 'I'},
    { "4a", 'J'}, { "4b", 'K'}, { "4c", 'L'}, { "4d", 'M'}, { "4e", 'N'}, { "4f", 'O'}, { "50", 'P'},
    { "51", 'Q'}, { "52", 'R'}, { "53", 'S'}, { "54", 'T'}, { "55", 'U'}, { "56", 'V'}, { "57", 'W'},
    { "58", 'X'}, { "59", 'Y'}, { "5a", 'Z'}, { "5b", '['}, { "5c", '\\'}, { "5d", ']'}, { "5e", '^'},
    { "5f", '_'}, { "60", '`'}, { "61", 'a'}, { "62", 'b'}, { "63", 'c'}, { "64", 'd'}, { "65", 'e'},
    { "66", 'f'}, { "67", 'g'}, { "68", 'h'}, { "69", 'i'}, { "6a", 'j'}, { "6b", 'k'}, { "6c", 'l'},
    { "6d", 'm'}, { "6e", 'n'}, { "6f", 'o'}, { "70", 'p'}, { "71", 'q'}, { "72", 'r'}, { "73", 's'},
    { "74", 't'}, { "75", 'u'}, { "76", 'v'}, { "77", 'w'}, { "78", 'x'}, { "79", 'y'}, { "7a", 'z'},
    { "7b", '{'}, { "7c", '|'}, { "7d", '}'}, { "7e", '~'}, 
    { NULL, '\0'}
};

/* Map hex ascii code to character */
static char ascii_mapping(char *hexnum)
{
    int i=0;
    while(ascii_tbl[i].hex != NULL)
    {
        if(!strcmp(ascii_tbl[i].hex, hexnum))
            return ascii_tbl[i].character;
        i++;
    }
    return '\0';
}

/* Get the number after \x */
static char *get_ascii_hex(char *input, int i)
{
    char *a = NULL;
    a = malloc(sizeof(int));
    strncpy(a, input+i+2, sizeof(int));
    a[2] = '\0';
    return a;
}

/* Parse \x C escape sequence, so that we can use \xoo 
   in text to describe ascii codes. 
 */
char *parse_ascii_esc_seq(char *input)
{
    int i=0;
    char *output = malloc(sizeof(char) * CLI_CFG_RULE_MAXLEN);
    char *hexnum = NULL;
    char character;
    int out_idx=0;

    memset(output, '\0', CLI_CFG_RULE_MAXLEN);
    while(i < strlen(input))
    {
        if(input[i] == '\\' && input[i+1] == 'x')
        {
            hexnum = get_ascii_hex(input, i);
            character = ascii_mapping(hexnum);
            if(character != '\0')
                output[out_idx] = character;
            else
            {
                cipcfg_print("Unknow ascii code.\n");
                exit(1);
            }
            free(hexnum);
            out_idx++;
            i = i+4;
        }
        else
        {
            output[out_idx] = input[i];
            out_idx++;
            i++;
        }
    }
    return output;
}

#ifdef CIPCFG_ITEM_VALIDATION
/* Check input date's validation */
int cipcfg_date_input_validation(char *input)
{
    char *chk_str = malloc(sizeof(char) * MAX_VALUE_LEN);

    strcpy(chk_str, input);
    if(cipcfg_item_regex_check(CFG_ITEM_DATE_RULE, chk_str))
    {
//        sprintf(err_msg, "The format of date should be <yyyy>.<mm>.<dd>-<hh>:<mm>:<ss>\n");
	free(chk_str);
        return CIPCFG_ERR_VALIDATION_FAIL;
    }

    free(chk_str);
    return CIPCFG_SUCCESS;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_item_timezone_check(char *input)
{
    int i, match = 0;

    for (i=0; zone_info[i].key != NULL; i++) {
        if ( !strcmp(input, zone_info[i].key) ) {
            match = 1;
            break;
        }
    }

    if( match == 0 )
	return CIPCFG_ERR_VALIDATION_FAIL;
    else
        return CIPCFG_SUCCESS;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_item_domain_check(char *input)
{
    char *chk_string;

    /* domain name length cannot be longer than 256 */
    if(strlen(input) > MAX_DOMAIN_LEN)
    {
//        sprintf(err_msg, "Domain name cannot be longer than 256\n"); 
	return CIPCFG_ERR_VALIDATION_FAIL;
    }
    chk_string = malloc(sizeof(char) * MAX_DOMAIN_LEN);
    strcpy(chk_string, input);
    /* domain type can be ip address or domain name */
    if(cipcfg_item_regex_check(CFG_ITEM_DOMAIN_RULE, chk_string) != 0)
    {
//        sprintf(err_msg, "It is not a valid domain name\n"); 
	free(chk_string);
	return CIPCFG_ERR_VALIDATION_FAIL;
    }
    free(chk_string);
    return CIPCFG_SUCCESS;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_channel_check(char *input, char *rule)
{
    COUNTRY_CHANNEL cc;
    int i;

    /* If there is no allowed_channel file found, use default band/channel from web UI */
    if ( CIPCFG_SUCCESS != get_allowed_ch(&cc))
    {
        /* The default valid channel are : auto, 1-13 */
        if(!strcmp(input, "auto"))
            return CIPCFG_SUCCESS;

        if(atoi(input) >= 1 || atoi(input) <=13 )
            return CIPCFG_SUCCESS;

        return CIPCFG_ERR_VALIDATION_FAIL;
    }
    else if(rule != NULL)
    {
        if(cipcfg_item_regex_check(rule, input) == 0)
            return CIPCFG_SUCCESS;
        else 
        {
//            sprintf(err_msg, "It is not a valid channel\n");
            return CIPCFG_ERR_VALIDATION_FAIL;
        }
    }
    else
    {
        /* Check channel */
        if(!strcmp(input, "auto"))
            return CIPCFG_SUCCESS;

        for(i=0; (-1 != cc.ch_11a[i]) ; i++){
            if(cc.ch_11a[i] == atoi(input))
                return CIPCFG_SUCCESS;
        }
        for(i=0; (-1 != cc.ch_11b[i]) ; i++){
            if(cc.ch_11b[i] == atoi(input))
                return CIPCFG_SUCCESS;
        }

        for(i=0; (-1 != cc.ch_11g[i]) ; i++){
            if(cc.ch_11g[i] == atoi(input))
                return CIPCFG_SUCCESS;
        }
    }

//    sprintf(err_msg, "It is not a valid channel\n");
    return CIPCFG_ERR_VALIDATION_FAIL;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_band_check(char *input, char *rule)
{
    COUNTRY_CHANNEL cc;
    int ret=CIPCFG_ERR_VALIDATION_FAIL;

    if ( 0 != get_allowed_ch(&cc))
    {
        /* The default valid band are : 11b, 11g, 11bg */
        if(!strcmp(input, "11b") || !strcmp(input, "11g") || !strcmp(input, "11bg"))
            return CIPCFG_SUCCESS;

    }
    else if(rule != NULL)
    {
        ret = cipcfg_item_regex_check(rule, input);
    }
    else
    {
        if(!strcmp(input, "11a") && -1 != cc.ch_11a[0])
            return CIPCFG_SUCCESS;
        else if(!strcmp(input, "11b") && -1 != cc.ch_11b[0])
            return CIPCFG_SUCCESS;
        else if(!strcmp(input, "11g") && -1 != cc.ch_11g[0])
            return CIPCFG_SUCCESS;
        else if(!strcmp(input, "11na") && -1 != cc.ch_11a[0] && -1 != cc.ch_11na[0])
            return CIPCFG_SUCCESS;
        else if(!strcmp(input, "11ng") && -1 != cc.ch_11g[0] && -1 != cc.ch_11ng[0])
            return CIPCFG_SUCCESS;
        else if(!strcmp(input, "11bg") && -1 != cc.ch_11b[0] && -1 != cc.ch_11g[0])
            return CIPCFG_SUCCESS;
    } 
//    sprintf(err_msg, "It is not a valid band\n");
    return ret;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_item_int_check(char *rule, char *input)
{
    char *pch;
    int valid = 0;
    char string[MAX_VALUE_LEN];
    int min, max;
    int value;
    char *tmp;
    int i;

    if( strlen(input) == 0 || !strcmp(input, "") )
        return CIPCFG_ERR_VALIDATION_FAIL;    

    value = atoi(input);
    tmp = malloc(sizeof(char) * MAX_VALUE_LEN);
    strcpy(string, rule);
    strcpy(tmp, input);

    pch = strtok(string,",");
    while (pch != NULL)
    {
        min=0;
        max=0;
        for(i=0;i < strlen(pch);i++)
        {
            if(pch[i] == '~')
	    {
                max = atoi(pch+i+1);
            }
        }
        min = atoi(pch);
        if(max ==0)
        {
            if(min == value)
                valid = 1;
        }
        else if(value <= max && value >=min)
            valid =1;
        pch = strtok(NULL, ",");
    }
    free(tmp);
    if(valid == 1)
        return CIPCFG_SUCCESS;
    else
    {
//        sprintf(err_msg, "The input is not valid, it should be %s\n", rule);
        return CIPCFG_ERR_VALIDATION_FAIL;
    }
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
// These rules are copy from web UI
int cipcfg_item_mac_check(char *input)
{

    /*  The valid type of mac should be desided by upper level*/
    if(cipcfg_item_regex_check(CFG_ITEM_MAC_RULE1, input) == 0 || 
	cipcfg_item_regex_check(CFG_ITEM_MAC_RULE2, input) == 0 || 
	cipcfg_item_regex_check(CFG_ITEM_MAC_RULE3, input) == 0)
        return CIPCFG_SUCCESS;
    else
    {
//        sprintf(err_msg, "It is not a valid mac\n");
        return CIPCFG_ERR_VALIDATION_FAIL;
    }
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_item_ip_check(char *input)
{   
    char *string;
    char *tmp;
    int invalid=0;
    string = malloc(sizeof(char)*CLI_IP_LEN);
    strcpy(string, input);

    // the valid format is xx.xx.xx.xx
    if(cipcfg_item_regex_check(CFG_ITEM_IP_RULE, string))
    {
//        sprintf(err_msg, "It is not a valid IP\n");
        return CIPCFG_ERR_VALIDATION_FAIL;
    }

    // Check every element is less than 255
    tmp = strtok(string, ".");
    while(tmp !=NULL)
    {
        if(atoi(tmp) > IP_MAX_NUM){
            invalid=1;
            break;
        }
        tmp = strtok(NULL, ".");
    }

    free(string);
    if(invalid == 0)
        return CIPCFG_SUCCESS;
    else
    {
//        sprintf(err_msg, "It is not a valid IP\n");
        return CIPCFG_ERR_VALIDATION_FAIL;
    }
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int cipcfg_item_regex_check(const char *regex, char *string)
{   
    regex_t preg;
    char *regex_rule = NULL;
    
    regex_rule = parse_ascii_esc_seq(regex);

    if(regcomp(&preg, regex_rule, REG_EXTENDED | REG_NOSUB ) != 0){
        free(regex_rule);
        return CIPCFG_ERR_VALIDATION_FAIL;
    }else{
        free(regex_rule);
        if(regexec(&preg, string, (size_t) 0, NULL, 0) == 0)
            return 0;
        else
        {
//            sprintf(err_msg, "The input is not valid, it should be %s\n", regex);
            return CIPCFG_ERR_VALIDATION_FAIL;
        }
    }
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
/* There are 8 types of configuration item */
int check_input_validity(int type, char *input, char *rule)
{
    int ret=CIPCFG_SUCCESS;

    switch(type)
    {
	case CFG_TYPE_INT:
  	    ret=cipcfg_item_int_check(rule, input);
	    break;
	case CFG_TYPE_STRING:
	    ret=cipcfg_item_regex_check(rule, input);
	    break;
	case CFG_TYPE_MAC:
	    ret=cipcfg_item_mac_check(input);	
	    break;
	case CFG_TYPE_IP:
	    ret=cipcfg_item_ip_check(input);
	    break;
	case CFG_TYPE_BAND:
	    ret = cipcfg_band_check(input, rule);
	    break;
	case CFG_TYPE_CHANNEL:
	    ret=cipcfg_channel_check(input, rule);
	    break;
	case CFG_TYPE_DOMAIN:
	    ret=cipcfg_item_domain_check(input);
	    break;
	case CFG_TYPE_TIMEZONE:
	    ret=cipcfg_item_timezone_check(input);
	    break;
	default:
	    ret=CIPCFG_ERR_VALIDATION_FAIL;	
    }
    return ret;
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
int input_validation_by_item(char *item_name, char *item_value, char *group_name)
{
    int type;
    char *rule = NULL;
    CIPCFG *cipcfg = NULL;

    cipcfg = init_cipcfg_f(CLI_CFG_FILE);
    if(!cipcfg)
    {
        cipcfg_print("ERROR: Fail to init configuration file.\n");
        return NULL;
    }

    /* Get item type */
    type = get_cfg_item_type_by_group_name(cipcfg, item_name, group_name);
    if(type == CIPCFG_ERR_VALIDATION_FAIL)
	return CIPCFG_ERR_VALIDATION_FAIL;

    /* Get item validation rule */
    rule = get_cfg_item_rule_by_group_name(cipcfg, item_name, group_name);

    /* Check type is valid or not */
    switch(type_pre_check(type, item_value))
    {
	case CLI_CHK_TYPE_FAILURE:
  	    sprintf(err_msg, "ERROR: The type is not correct.\n");
	    return CIPCFG_ERR_VALIDATION_FAIL;
	case CLI_CHK_TYPE_RESTRICT:
	    sprintf(err_msg, "ERROR: This item cannot be set.\n");
	    return CIPCFG_ERR_VALIDATION_FAIL;
	default:
	    break;
    } 

    /* Check input is valid or not */
    if(check_input_validity( type, item_value, rule))
	return CIPCFG_ERR_VALIDATION_FAIL; 

    close_cipcfg(cipcfg);
    return CIPCFG_SUCCESS; 
}
#endif

#ifdef CIPCFG_ITEM_VALIDATION
/* Filter out invalid input type */
int type_pre_check(int type, char *item_value)
{
    int i;
    int ret = CLI_CHK_TYPE_SUCCESS;

    switch(type)
    {
	case CFG_TYPE_INT:
	    i=0;
       	    while(i < strlen(item_value))
            {
                if(!isdigit(item_value[i]) && item_value[i] != '-')
                {
                    ret = CLI_CHK_TYPE_FAILURE;
                    break;
                }
                i++;
            }  
	    break;
	case CFG_TYPE_CHANNEL:
	    if(strcmp(item_value, "auto"))
	    {
		i=0;
		while(i < strlen(item_value))
		{
		    if(!isdigit(item_value[i]))
                    {
                        ret = CLI_CHK_TYPE_FAILURE;
                        break;
                    }
		    i++;
		}
	    }
	    break;
	case CFG_TYPE_RESTRICT:
	    ret = CLI_CHK_TYPE_RESTRICT; 
            break;
	default:
	    break;
    }
    return ret;
}
#endif

#ifdef CIPCFG_DEPENDENCY_CHECK
int dep_fun_mapping(char *id, ITEM_LIST *list, char *group_name)
{
    int i;
    int ret=0;

    id++;       /* skip the first character */
    for(i =0; dr_function_list[i].name; i++)
    {
        if(!strcmp(dr_function_list[i].name, id)){
            ret = dr_function_list[i].fp(list, group_name);
        }
    }
    
    return ret;
}   
#endif

#ifdef CIPCFG_DEPENDENCY_CHECK
/* Check there is no conflict in our configuration */
int dependency_check(ITEM_LIST *item_list_head)
{
    int i=0, j=0, k=0, type, the_same_group;
    ITEM_LIST *dep_check_str = NULL;
    ITEM_LIST *head = NULL;
    int correct = 1, ret = CIPCFG_SUCCESS;
    struct attribute *attr = NULL;		
    char *cmp_str = NULL;
    char code[MAX_VALUE_LEN];
    group_row_t *cur_group_row, *item_list_group_row;
    CIPCFG *cipcfg = NULL, *rule_cfg = NULL;
    char **rule_item = NULL;              /* store dependency items */
	
    cipcfg = init_cipcfg();
    rule_cfg = init_cipcfg_f(CLI_CFG_FILE);
    if(!cipcfg || !rule_cfg) {
        sprintf(err_msg, "ERROR: Fail to init CIPCFG.\n");
	ret = CIPCFG_ERR_ILLEGAL_CFGFILE;
	goto end;
    }

    rule_item = malloc(sizeof(char *) * CLI_MAX_DEP_ITEM);
    if (rule_item == NULL) {
	ret = CIPCFG_ERR_MEM;
	goto end;
    }

    /* Per Rule */
    for(i=0;dependency_chk_list[i].check_list[0].group;i++)
    {
        /* Group filter */
        the_same_group=0;
        head = item_list_head;
        while(head != NULL && the_same_group != 1)
        {
            /* get item_list group - there may be several groups in item_list, 
               so we have to check item_list group one by one. 
             */
            item_list_group_row = group_table_row(head->gid);
            for(j=0;dependency_chk_list[i].check_list[j].group;j++)
            {
                if(!strcmp(dependency_chk_list[i].check_list[j].group, item_list_group_row->group_name))
                {
                    the_same_group=1;
                    break;
                }
            }
            if(dependency_chk_list[i].check_item.group != NULL)
            {
                if(!strcmp(dependency_chk_list[i].check_item.group, item_list_group_row->group_name))
                {
                    the_same_group=1;
                }
            }
            head = head->next;
        }

        /* Continue check if there is at least an item which is the same group with 
           Committed items.	 	
         */
        if(the_same_group == 0)
            continue;

	if( dependency_chk_list[i].check_list[0].op[0] == '@') {
	    if(dep_fun_mapping(dependency_chk_list[i].check_list[0].op, 
		item_list_head, dependency_chk_list[i].check_list[0].group) != 0) {
		ret = CIPCFG_ERR_DEPENDENCY;
		goto end;
	    } else {
                    /* We would not check any other item in this rule, 
                       just set correct to 0 and check another rule */
		continue;
	    }
	}	

        correct=1;		
        for(j=0;dependency_chk_list[i].check_list[j].group;j++)
        {
	    if (dependency_chk_list[i].check_list[j].name == NULL)
		break;

            cur_group_row = group_table_row_str(dependency_chk_list[i].check_list[j].group);
            type = get_cfg_item_type_by_group_name(rule_cfg, dependency_chk_list[i].check_list[j].name, 
							cur_group_row->group_name); 

            /* check the item is in item_list or not */
            dep_check_str = item_list_head;
            while(dep_check_str != NULL)
            {
                if(!strcmp(dep_check_str->item_name, dependency_chk_list[i].check_list[j].name))
                    break;
                dep_check_str = dep_check_str->next;
            }
#ifdef CLI_DEBUG             
            print_list(item_list_head);
#endif
            if(dep_check_str != NULL)	/* If the item is in item_list, get the value from item_list */
            {
                cmp_str = dep_check_str->item_value;
            }
            else /* If the item is not in item_list, get value from configuration file */	
            {
                get_cfg_item_by_str(cipcfg, dependency_chk_list[i].check_list[j].group, 
					dependency_chk_list[i].check_list[j].name, code);
                cmp_str = code;
            }

	    if (cmp_str == NULL) {
		ret = CIPCFG_ERR_DEPENDENCY; 
		goto end;
	    }

            ret = check_input_validity(type, cmp_str, dependency_chk_list[i].check_list[j].rule);
	    rule_item[j] = malloc(sizeof(char) * MAX_VALUE_LEN);
            /* ret == 0 means the state is TRUE */
            if(ret == 0) 
            {
                if(dependency_chk_list[i].check_list[j].op[0] == '&')
                    correct = correct & 1;
                else
                    cipcfg_print("ERROR: Unkown operator.\n");

                strcpy(rule_item[j], cmp_str);
                cmp_str = NULL;
            }
            else
            {
                if(dependency_chk_list[i].check_list[j].op[0] == '&')
                    correct = correct & 0;
                else
                    cipcfg_print("ERROR: Unkown operator.\n");
                cmp_str = NULL;

                for(k=j;k>=0;k--)
                    free(rule_item[k]);
                break;
             }
        }

        if(correct == 1)
        {
            dep_check_str = item_list_head;
            while(dep_check_str != NULL)
            {
                if(!strcmp(dep_check_str->item_name, 
			dependency_chk_list[i].check_item.name))
                    break;
                
                dep_check_str = dep_check_str->next;
            }

            if(dep_check_str != NULL)
                cmp_str = dep_check_str->item_value;
            else
            {
                get_cfg_item_by_str(cipcfg, dependency_chk_list[i].check_item.group,
                			    dependency_chk_list[i].check_item.name, code);
                cmp_str = code;
            }

            cur_group_row = group_table_row_str(dependency_chk_list[i].check_item.group);
            type = get_cfg_item_type_by_group_name(rule_cfg, dependency_chk_list[i].check_item.name, 
                                                    cur_group_row->group_name); 

            ret = check_input_validity(type, cmp_str , dependency_chk_list[i].check_item.rule);
            if(ret != 0)
            {
                char *errmsg_ptr = err_msg;
                char tmp_err_msg[CIPCFG_ERR_MSG_MAXLEN];
                sprintf(tmp_err_msg, "There are something "
			"conflicting in your configuration:\nWhen\n");
                strcpy(errmsg_ptr, tmp_err_msg);
                errmsg_ptr+=strlen(tmp_err_msg);

                for(k=0;dependency_chk_list[i].check_list[k].group;k++)
                {
		    if (dependency_chk_list[i].check_list[k].name == NULL)
			break;
                    sprintf(tmp_err_msg, "%s is \"%s\"\n", 
			dependency_chk_list[i].check_list[k].name, rule_item[k]);
                    strcpy(errmsg_ptr, tmp_err_msg);
                    errmsg_ptr+=strlen(tmp_err_msg);
                    free(rule_item[k]);
                }
                sprintf(tmp_err_msg, "%s cannot be \"%s\"\n", 
			dependency_chk_list[i].check_item.name, cmp_str);
                strcpy(errmsg_ptr, tmp_err_msg);
                errmsg_ptr+=strlen(tmp_err_msg);
                    
                ret = CIPCFG_ERR_DEPENDENCY;
		goto end;
	    } else {
		for (k = 0; dependency_chk_list[i].check_list[k].group; k++) {
		    if (dependency_chk_list[i].check_list[k].name == NULL)
			break;
		    free(rule_item[k]);
		}
	    }
	    cmp_str = NULL;
        }   /* End of inner for loop */	
    }
end:
    if (rule_item != NULL)
	free(rule_item);
    if (attr != NULL)
	free_attr_list(attr);
    if (cipcfg != NULL)
	close_cipcfg(cipcfg);
    if (rule_cfg != NULL)
	close_cipcfg(rule_cfg);

    return ret;
}
#endif

int verifySystemDevice(ITEM_LIST *items)
{
    ITEM_LIST *item;

    item=items;
    while (item) {
        if (strcmp(item->item_name, "name")==0) {
            if (strlen(item->item_value) > 32) {
                return CIPCFG_ERR_INVALID_LENGTH;
            }
        } else if (strcmp(item->item_name, "desc")==0) {
            if (strlen(item->item_value) > 256) {
                return CIPCFG_ERR_INVALID_LENGTH;
            }
        } else if (strcmp(item->item_name, "location")==0) {
            if (strlen(item->item_value) > 256) {
                return CIPCFG_ERR_INVALID_LENGTH;
            }
        } else {
	    cipcfg_print("<error>Invalid item %s\n", item->item_name);
	    return CIPCFG_ERR_UNKNOWN_ITEM;
	}
	item=item->next;
    }

    return CIPCFG_SUCCESS;
}

int verifySystemSystime(ITEM_LIST *items)
{
    
    return CIPCFG_SUCCESS;
}
int verifyNetworkWan_ip(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkL2(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkL2Stp(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkL2Proxy_arp(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkL2Igmp_snooping(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkL2Port_trunk(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyNetworkVlan(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtAccounts(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtSnmp(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtSyslog(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtRmt_srv(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtRmt_srvHttp(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyMgmtRmt_acc(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanMacacl(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcard(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcardWds(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcardVap(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcardVapMacAcl(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcardVapSecurity(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanRfcardVapIapp(ITEM_LIST *items){return CIPCFG_SUCCESS;}
int verifyWlanScan(ITEM_LIST *items){return CIPCFG_SUCCESS;}

int validate_input(ITEM_LIST *items)
{
    int ret=CIPCFG_SUCCESS;
    
    switch (items->gid) {
	case GID_DEVICE: 	// cipconfig.system.device
	    if ((ret=verifySystemDevice(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_SYSTIME:		// cipconfig.system.systime
	    if ((ret=verifySystemSystime(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_WAN_IP:		// cipconfig.network.wan_ip
	    if ((ret=verifyNetworkWan_ip(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_NET_L2:		// cipconfig.network.l2
	    if ((ret=verifyNetworkL2(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_STP:	// cipconfig.network.l2.stp
	    if ((ret=verifyNetworkL2Stp(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_PROXY_ARP:	// cipconfig.network.l2.proxy_arp
	    if ((ret=verifyNetworkL2Proxy_arp(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_IGMP:	// cipconfig.network.l2.igmp_snooping
	    if ((ret=verifyNetworkL2Igmp_snooping(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_PORT_TRUNK:	// cipconfig.network.l2.port_trunk
	    if ((ret=verifyNetworkL2Port_trunk(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_NET_VLAN:	// cipconfig.network.vlan
	    if ((ret=verifyNetworkVlan(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_ACCOUNT:	// cipconfig.mgmt.accounts
	    if ((ret=verifyMgmtAccounts(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_SNMP:	        // cipconfig.mgmt.snmp
	    if ((ret=verifyMgmtSnmp(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_SYSLOG:	// cipconfig.mgmt.syslog
	    if ((ret=verifyMgmtSyslog(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_REMOTE_PROTOCOL:// cipconfig.mgmt.rmt_srv
	    if ((ret=verifyMgmtRmt_srv(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_HTTP:// cipconfig.mgmt.rmt_srv.http
	    if ((ret=verifyMgmtRmt_srvHttp(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_REMOTE_ACCESS:	// cipconfig.mgmt.rmt_acc
	    if ((ret=verifyMgmtRmt_acc(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_RF1:	// cipconfig.wlan.rfcard
        case GID_RF2:
        case GID_RF3:
        case GID_RF4:
	    if ((ret=verifyWlanRfcard(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_RF1_WDS1:	// cipconfig.wlan.rfcard.wds:0
    	case GID_RF1_WDS2:	// cipconfig.wlan.rfcard.wds:1
    	case GID_RF1_WDS3:	// cipconfig.wlan.rfcard.wds:2
    	case GID_RF1_WDS4:	// cipconfig.wlan.rfcard.wds:3
    	case GID_RF2_WDS1:	// cipconfig.wlan.rfcard.wds:0
    	case GID_RF2_WDS2:	// cipconfig.wlan.rfcard.wds:1
    	case GID_RF2_WDS3:	// cipconfig.wlan.rfcard.wds:2
    	case GID_RF2_WDS4:	// cipconfig.wlan.rfcard.wds:3
    	case GID_RF3_WDS1:	// cipconfig.wlan.rfcard.wds:0
    	case GID_RF3_WDS2:	// cipconfig.wlan.rfcard.wds:1
    	case GID_RF3_WDS3:	// cipconfig.wlan.rfcard.wds:2
    	case GID_RF3_WDS4:	// cipconfig.wlan.rfcard.wds:3
    	case GID_RF4_WDS1:	// cipconfig.wlan.rfcard.wds:0
    	case GID_RF4_WDS2:	// cipconfig.wlan.rfcard.wds:1
    	case GID_RF4_WDS3:	// cipconfig.wlan.rfcard.wds:2
    	case GID_RF4_WDS4:	// cipconfig.wlan.rfcard.wds:3
	    if ((ret=verifyWlanRfcardWds(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_RF1_VAP1:	// cipconfig.wlan.rfcard.vap
    	case GID_RF1_VAP2:
    	case GID_RF1_VAP3:
    	case GID_RF1_VAP4:
    	case GID_RF2_VAP1:
    	case GID_RF2_VAP2:
    	case GID_RF2_VAP3:
    	case GID_RF2_VAP4:
    	case GID_RF3_VAP1:
    	case GID_RF3_VAP2:
    	case GID_RF3_VAP3:
    	case GID_RF3_VAP4:
    	case GID_RF4_VAP1:
    	case GID_RF4_VAP2:
    	case GID_RF4_VAP3:
    	case GID_RF4_VAP4:
	    if ((ret=verifyWlanRfcardVap(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
     	case GID_RF1_VAP1_ACL:	// cipconfig.wlan.rfcard.vap.macacl
    	case GID_RF1_VAP2_ACL:
    	case GID_RF1_VAP3_ACL:
    	case GID_RF1_VAP4_ACL:
    	case GID_RF2_VAP1_ACL:
    	case GID_RF2_VAP2_ACL:
    	case GID_RF2_VAP3_ACL:
    	case GID_RF2_VAP4_ACL:
    	case GID_RF3_VAP1_ACL:
    	case GID_RF3_VAP2_ACL:
    	case GID_RF3_VAP3_ACL:
    	case GID_RF3_VAP4_ACL:
    	case GID_RF4_VAP1_ACL:
    	case GID_RF4_VAP2_ACL:
    	case GID_RF4_VAP3_ACL:
    	case GID_RF4_VAP4_ACL:
	    if ((ret=verifyWlanRfcardVapMacAcl(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
      	case GID_RF1_VAP1_SEC:	// cipconfig.wlan.rfcard.vap.security
    	case GID_RF1_VAP2_SEC:
    	case GID_RF1_VAP3_SEC:
    	case GID_RF1_VAP4_SEC:
    	case GID_RF2_VAP1_SEC:
    	case GID_RF2_VAP2_SEC:
    	case GID_RF2_VAP3_SEC:
    	case GID_RF2_VAP4_SEC:
    	case GID_RF3_VAP1_SEC:
    	case GID_RF3_VAP2_SEC:
    	case GID_RF3_VAP3_SEC:
    	case GID_RF3_VAP4_SEC:
    	case GID_RF4_VAP1_SEC:
    	case GID_RF4_VAP2_SEC:
    	case GID_RF4_VAP3_SEC:
    	case GID_RF4_VAP4_SEC:
	    if ((ret=verifyWlanRfcardVapSecurity(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
       	case GID_RF1_VAP1_IAPP:	// cipconfig.wlan.rfcard.vap.iapp
    	case GID_RF1_VAP2_IAPP:
    	case GID_RF1_VAP3_IAPP:
    	case GID_RF1_VAP4_IAPP:
    	case GID_RF2_VAP1_IAPP:
    	case GID_RF2_VAP2_IAPP:
    	case GID_RF2_VAP3_IAPP:
    	case GID_RF2_VAP4_IAPP:
    	case GID_RF3_VAP1_IAPP:
    	case GID_RF3_VAP2_IAPP:
    	case GID_RF3_VAP3_IAPP:
    	case GID_RF3_VAP4_IAPP:
    	case GID_RF4_VAP1_IAPP:
    	case GID_RF4_VAP2_IAPP:
    	case GID_RF4_VAP3_IAPP:
        case GID_RF4_VAP4_IAPP:
	    if ((ret=verifyWlanRfcardVapIapp(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
    	case GID_RF1_SCAN:	        // cipconfig.rf0.scan
    	case GID_RF2_SCAN:
    	case GID_RF3_SCAN:
    	case GID_RF4_SCAN:
	    if ((ret=verifyWlanScan(items))!=CIPCFG_SUCCESS) {
		return ret;
	    }
	    break;
	default:
	    return CIPCFG_ERR_UNKNOWN_GROUP;
    }
    return CIPCFG_SUCCESS;
}
