#ifndef _CIPCFG_H
#define _CIPCFG_H

#include <sys/file.h>
#include "ezxml.h"
#include "version.h"
#include "permission.h"

#define MAX_ATTRLEN     64
#define MAX_VALUELEN    512
#define CIPCFG_PRINT_ERR "CIPCFG_PRINT_ERR"

#define DEFAULT_CONFIG          "/etc/default-config.xml"
#define CUSTOMIZE_CONFIG 	"/conf/customize/config-backup.conf"
#define SYSLOG_MSG_FILE         "/tmp/syslog_messages"
#define COUNTRY_CODE	        "/etc/rc.d/country_code"
#define ALLOWED_CHANNEL	        "/etc/allowed_channel"
#define ALLOWED_POWER	        "/etc/allowed_power"
#define CLI_CFG_FILE 			"/etc/cfg-rule.xml"
#define SNMPD_PID_FILE		"/var/run/snmpd.pid"
#define FW_VERSION_FILE         "/tmp/NEW_FW_VERSION"

#define CLI_CFG_GET_NAME 1
#define CLI_CFG_GET_TYPE 2
#define CLI_CFG_GET_RULE 3
#define CLI_CFG_GET_DESC 4
#define CLI_CFG_RULE_MAXLEN 1000
#define STACK_MAX_NUM 100

#define MAX_GRPLIST_STRING_LEN    2048	      
#define MAX_GRPLIST_NUM	           256	      

#define MAX_BUF_LEN            256 /* 1024 */
#define SMALL_BUF_LEN           32
#define MAX_VALUE_LEN		MAX_BUF_LEN
#define MAGIC_LEN               8

#ifndef MAX_RF
#define MAX_RF			1
#endif
#ifndef MAX_VAP
#define MAX_VAP            	4
#endif
#ifndef MAX_WDS_LINK
#define MAX_WDS_LINK            4
#endif
#ifndef MAX_STA
#define MAX_STA			1
#endif
#ifndef MAX_REP
#define MAX_REP			1
#endif

#define MAX_ATTRS               9

// define Wireless mode
#define AUTO                    0
#define _802_11a                1
#define _802_11b                2
#define _802_11g                3


typedef struct ezxml CIPCFG;

struct item_list {
	int gid; //group id
	char *item_name; //the name of the input item
	char *item_value; //the value of the input item
	struct item_list *next;
};

enum cli_cfg_type {
    CFG_TYPE_INT = 0,
    CFG_TYPE_STRING,
    CFG_TYPE_MAC,
    CFG_TYPE_IP,
    CFG_TYPE_DOMAIN,
    CFG_TYPE_BAND,
    CFG_TYPE_CHANNEL,
    CFG_TYPE_TIMEZONE,
    CFG_TYPE_RESTRICT
};

typedef struct cli_cfg{
    enum cli_cfg_type cfg_type;
    char *type_string;
}CFG_RULE; 

typedef struct item_list ITEM_LIST;

struct attribute{
    char *attr_name;
    char *attr_value;
    int idx;
    struct attribute *next;
};

#ifndef FLASE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

// error code
#define CIPCFG_SUCCESS                   0
#define CIPCFG_ERR_NODE_NOT_FOUND       -1
#define CIPCFG_ERR_MEM                  -2
#define CIPCFG_ERR_UNKNOWN_GROUP        -3
#define CIPCFG_ERR_UNKNOWN_ITEM         -4
#define CIPCFG_ERR_INVALID_LENGTH       -5
#define CIPCFG_ERR_ILLEGAL_CFGFILE      -6
#define CIPCFG_ERR_OPENFILE             -7
#define CIPCFG_ERR_INVALID_ITEM         -8
#define CIPCFG_ERR_UNKNOWN_PLATFORM     -9
#define CIPCFG_ERR_LOCK                 -10
#define CIPCFG_ERR_GETSTAT	  	-11
#define CIPCFG_ERR_VALIDATION_FAIL	-12
#define CIPCFG_ERR_DEPENDENCY		-13
#define CIPCFG_ERR_CFG_SYNC             -14	
#define CIPCFG_ERR_EXCEED_MAX_CHILD     -15	

#define CIPCFG_ERR_DEP_SNMP_READ_WRITE  -101
#define CIPCFG_ERR_DEP_NETMASK		-102
#define CIPCFG_ERR_MSG_MAXLEN		1000	

// global error message
extern char err_msg[CIPCFG_ERR_MSG_MAXLEN];

// lock signal
#define EX_LOCK                 LOCK_EX
#define SHARE_LOCK              LOCK_SH
#define LOCK_DEFAULT_FILE       "/var/lock/default_lock"
#define LOCK_RAM_FILE           "/var/lock/cipcfg_ram"
#define LOCK_ROM_FILE           "/var/lock/cipcfg_rom"
#define LOCK_BACKUP_FILE        "/var/lock/cipcfg_backup"
#define LOCK_RESTORED_FILE      "/var/lock/cipcfg_restored"
#define LOCK_CURRENT_FILE       "/var/lock/cipcfg_current"
#define LOCK_DEFAULT            0
#define LOCK_CIPCFG_RAM         1
#define LOCK_CIPCFG_ROM         2
#define LOCK_CIPCFG_BACKUP      3
#define LOCK_CIPCFG_RESTORED    4
#define LOCK_CIPCFG_CURRENT     5

typedef struct {
    unsigned int gid;
    char *group_name ;
    int dirty;
    char *prefix;
} group_row_t;

typedef struct {
    char *key ;
    char *value ;
} STRING_PAIR ;

typedef struct {
    unsigned int cc;
    int ch_11a[32];
    int ch_11b[32];
    int ch_11g[32];
    int ch_11ng[32];
    int ch_11ngu[32];
    int ch_11ngl[32];
    int ch_11na[32];
    int ch_11nau[32];
    int ch_11nal[32];
    int ch_11n[32];
} COUNTRY_CHANNEL ;

/* Firmware image file header, total len = 128bytes. */
typedef struct fw_header {
    unsigned char magic[8];         /* Magic number string */
    unsigned char hw_type[16];      /* Hardware Type, ie. A300 */
    unsigned char hw_ver[16];       /* Hardware version */
    unsigned char fw_ver[16];       /* Firmware version */
    unsigned int fw_len;            /* fimrware image length w/o header */
    unsigned int fw_sum;            /* The checksum of Config XML file */
    unsigned int fis_len;           /* The length of FIS table */
    unsigned int kernel_len;        /* The length of Linux Kernel image */
    unsigned int rootfs_len;            /* The length of Root FS image */
    unsigned int conf_len;          /* The length of conf partition */
    unsigned int redboot_conf_len;  /* The length of RedBoot config */
    unsigned char brand_id[2];	    /* The brand ID of firmware */
    unsigned char reserved[42];
} FW_HEADER;

/* Configuration file header, total len = 128bytes. */
typedef struct cfg_header {
    unsigned char hw_type[16];      /* Hardware Type, ie. A800 */
    unsigned char hw_ver[16];       /* Hardware version */
    unsigned char cfg_ver[16];      /* Configuration version */
    unsigned int cfg_len;           /* Config XML File length */
    unsigned int cfg_sum;           /* The checksum of Config XML file */
    unsigned char magic[8];         /* Magic number string */
    unsigned char reserved[64];
} CFG_HEADER;

/* for id_tble.c */
typedef struct id_tbl_elm {
    unsigned int    id;
    unsigned char   magic[8];
    unsigned char   hw_type[8];
} id_tbl_e;

// exported function
CIPCFG* init_cipcfg();
CIPCFG* init_cipcfg_f(char *xml_file);
void print_list(ITEM_LIST *list);
int close_cipcfg(CIPCFG *cipcfg);
ITEM_LIST *append_cfg_item(ITEM_LIST *item_list, int gid, 
                        char *item_name, char *item_value);

int save_cipcfg(CIPCFG *cipcfg, char *file);
int set_cfg_items_f(CIPCFG *cipcfg, ITEM_LIST *set_list, char *file);
int set_cfg_items_lite(CIPCFG *cipcfg, ITEM_LIST *set_list);
int set_cfg_items(CIPCFG *cipcfg, ITEM_LIST *set_list);
int apply_cfg(CIPCFG *cipcfg);
int get_cfg_item_by_prefix(CIPCFG *cipcfg, char *prefix, char *item_name, char *item_value);
int get_cfg_item(CIPCFG *cipcfg, int gid, char *item_name, char *item_value);
int get_cfg_item_by_str(CIPCFG *cipcfg, char *group_name, char *item_name, char *item_value);

//group_row_t *group_table_prefix(char *prefix);
struct attribute *get_new_attr(char *attr_name, char *attr_value);
struct attribute * get_all_attr(CIPCFG *node, char *item);
int free_attr_list(struct attribute *head);
CIPCFG *cipcfg_search(CIPCFG *root, char name[]);
struct attribute *cipcfg_list_all_child(CIPCFG *root);
ITEM_LIST *get_new_item(int gid, char *item_name, char *item_value);

// Utility functions
int reset_cipcfg();
int backup_cipcfg(char *file_name);
int decrypt_uploaded_cipcfg(unsigned char *buffer, int len);
int restore_cipcfg(unsigned char *buffer, int len);
int prd_tbl_lookup(unsigned int prod, unsigned char *magic, unsigned char *hw_type);
void upperCase( char *str);
int getUserGroup(const char *name);
unsigned int getConfigPermissionByName(const char *g_name);
unsigned int getOperationPermissionByName(const char *o_name);
unsigned int getConfigPermissionByGID(const unsigned int g_id);
unsigned int getOperationPermissionByOID(const unsigned int o_id);

group_row_t *group_table_prefix(char *prefix);
group_row_t *group_table_row_str(char *group_name);
group_row_t *group_table_row(int gid);
int free_item_list(ITEM_LIST *list);

struct dep_list{
    char *group;
    char *name;
    char *rule;
    char *op;
};

struct dep_item{
    char *group;
    char *name;
    char *rule;
};

struct dep_arr{
    struct dep_list check_list[6];
    struct dep_item check_item;
};

struct dep_fun_map_list
{
    char *name;
    int (*fp)(ITEM_LIST *, char *group_name);
};

enum set_cfg_flag { 
    CIPCFG_SET_SYS = 0,		// system wise setting,  
    CIPCFG_SET_DATA, 		// save = apply
    CIPCFG_SET_FIREWALL, 	// firewall related setting
};

#ifdef CIPCFG_DEPENDENCY_CHECK
extern const struct dep_fun_map_list dr_function_list[]; 
extern const struct dep_arr dependency_chk_list[]; 
#endif

#define DEF_PROMPT_ROOT "cipconfig"
#define CLI_MAX_STATUS_NUM 30
#define CLI_STATUS_MAX_COL 100
#define CLI_STATUS_MAX_ROW 30
#define CLI_STAT_MAXLEN 100 
#define CLI_MAX_PATH_LEN 100

char *get_cfg_item_name(CIPCFG *cipcfg, char *item_name);
char *get_cfg_item_rule(CIPCFG *cipcfg, char *item_name);
char *get_cfg_item_desc(CIPCFG *cipcfg, char *item_name);
int get_cfg_item_type(CIPCFG *cipcfg, char *item_name);
int get_cfg_item_type_by_group_name(CIPCFG *cipcfg, char *item_name, char *group_name);
char *parse_cfg_rule(char *string, int flag);
int cfg_type_map(char *type);
ITEM_LIST *set_related_items(int grpgid, char* itemname, char* itemvalue);
struct attribute *get_item_by_path(char *item, char *path);
struct attribute *list_child_by_path(CIPCFG *cipcfg, char *path);
char *get_cfg_item_rule_by_group_name(CIPCFG *cipcfg, char *item_name, char *group_name);

#ifdef CIPCFG_DEPENDENCY_CHECK
extern int dependency_check(ITEM_LIST *item_list_head);
#endif
ITEM_LIST *check_item_exist(ITEM_LIST *head, char *item_name);
extern int rep_agent_json_string_handler(char *json_string, char ***cli_stat_tbl); 
extern void free_cli_stat_tbl(char ***cli_stat_tbl);
extern void cli_stat_tbl_print(char ***cli_stat_tbl);
extern int input_validation_by_item(char *item_name, char *item_value, char *group_name);
extern int cipcfg_date_input_validation(char *input);
int cipcfg_print(char *fmt, ...);

struct attribute *get_attr(CIPCFG *node, char *prefix, int count, int next_flag);
int check_sync_file(int *gid, int *match, int *count);
int can_apply(int *gid);
int chkGIDdup(int *applylist, int num_applist, int gid);
#if defined(CIPCFG_CFGARCV) && CIPCFG_CFGARCV == 1
// External functions (defined in extension)
extern int do_cfgarcv();
#endif

#endif

