#ifndef _WLAN_H_
#define _WLAN_H_

#include "id.h"
#include "cipcfg.h"
#include "syscall.h"

typedef struct assoc_sta_list {
    char mac[18];
    int rssi;
    double tx_rate;
    int tx_okcnt;
    int tx_errcnt;
    int rx_errcnt;
    int rx_okcnt;
    int tx_packets;
    int rx_packets;
    int authmode;
    long long int tx_byte;
    long long int rx_byte;
    long long int assoc_time;
    double last_rx;
    struct assoc_sta_list *next;
} ASSOC_STA_LIST, ASSOC_WDS_LIST;

#define foreach_sta_get(node, list) \
    for (node = list; node; node = node->next)

AP_LIST *wdsd_scan(int rfno, char *secret, int ch, int mode);
int wdsd_connect(int rfno, char *secret, char *mac, int ch, int mode);
AP_LIST *wdsd_connected(int rfno);
AP_LIST *rf_scan(int rfno, int is_scan, char t[]);
AP_LIST *ap_scan(void);     // for ap management
int set_txpwr(char *txpwr); // for ap management
int set_channel_switch(int channel, int count); // for ap management
int getWDSLinkAttr(int, int, int, char *);
/* Station list */
ASSOC_STA_LIST * get_sta_list(char *);
void free_sta_list(ASSOC_STA_LIST *);
int sta_list_count(ASSOC_STA_LIST *);
/* WDS list */
ASSOC_WDS_LIST * get_wds_list(char *);
void free_wds_list(ASSOC_WDS_LIST *);
int wds_list_count(ASSOC_WDS_LIST *);

int get_allowed_ch(COUNTRY_CHANNEL *);

typedef struct {
    char *desc;
    unsigned int level;
} TXPWR_T;
TXPWR_T *get_txpwr_level(void);

enum wds_link_item_id {
    WDSLINK_STATE, WDSLINK_RATE, WDSLINK_QUALITY, WDSLINK_HEALTH,
    WDSLINK_TXTOTAL, WDSLINK_TXFAIL
};

enum band_type {
    IEEE80211_UNKNOWN, IEEE80211_A, IEEE80211_B, IEEE80211_G, IEEE80211_BG,
    IEEE80211_N, IEEE80211_AN, IEEE80211_GN, IEEE80211_BGN, IEEE80211_ABGN, 
    IEEE80211_TA, IEEE80211_DISABLED
};

typedef enum {
    IF_ROLE_MASTER = 0, IF_ROLE_VAP, IF_ROLE_WDS, IF_ROLE_STA, IF_ROLE_REP
} IF_ROLE_T;

extern char *get_ifname_by_idx(IF_ROLE_T, int, int, char *);
#define get_master_ifname_by_idx(a, b, c) \
    get_ifname_by_idx(IF_ROLE_MASTER, a, b, c)
#define get_vap_ifname_by_idx(a, b, c)	get_ifname_by_idx(IF_ROLE_VAP, a, b, c)
#define get_wds_ifname_by_idx(a, b, c)	get_ifname_by_idx(IF_ROLE_WDS, a, b, c)
#define get_sta_ifname_by_idx(a, b, c)	get_ifname_by_idx(IF_ROLE_STA, a, b, c)
#define get_rep_ifname_by_idx(a, b, c)	get_ifname_by_idx(IF_ROLE_REP, a, b, c)
extern const unsigned int max_rf;
extern const char *rf1_if, *rf2_if, *rf3_if, *rf4_if;
extern int isWDSLinkAlive(char *);
extern int getWDSLinkTxRate(char *, char *);
extern int getWDSLinkQuality(char *, char *);
extern int getWDSLinkHealth(char *, char *);
extern int getWDSLinkTxTotal(char *, char *);
extern int getWDSLinkTxFail(char *, char *);
extern int rfcards(char *, char *);
#endif


