/* This function parse status report-agent output to a three-dimensional array */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "json.h"
#include "cipcfg.h"

extern struct json_object* json_object_array_get_idx(struct json_object *obj, int idx);

void free_cli_stat_tbl(char ***cli_stat_tbl)
{
    int i,j;
    int cli_stat_col_idx = 1;
    int cli_stat_row_idx = 1;

    for(i=1;i<CLI_STATUS_MAX_COL;i++){
        if(cli_stat_tbl[i] == NULL){
            cli_stat_col_idx = i;
            break;
        }
    }
    for(i=1;i<CLI_STATUS_MAX_ROW;i++){
        if(cli_stat_tbl[1][i] == NULL){
            cli_stat_row_idx = i;
            break;
        }
    }
    for(i=cli_stat_col_idx-1;i>0;i--){
        for(j=cli_stat_row_idx-1;j>0;j--){
            free(cli_stat_tbl[i][j]);
        }
        free(cli_stat_tbl[i]);
    }
    free(cli_stat_tbl);
}

void cli_stat_tbl_add(char *value, char ***cli_stat_tbl, int cli_stat_col_idx, int cli_stat_row_idx)
{
#ifdef CLI_DEBUG
    cipcfg_print("table add col=%d row=%d\n", cli_stat_col_idx, cli_stat_row_idx);
#endif
    cli_stat_tbl[cli_stat_col_idx][cli_stat_row_idx] = malloc(sizeof(char)* CLI_STAT_MAXLEN);
    /* To build the table boundary */
    cli_stat_tbl[cli_stat_col_idx][cli_stat_row_idx+1] = NULL;

    memset(cli_stat_tbl[cli_stat_col_idx][cli_stat_row_idx], '\0', CLI_STAT_MAXLEN);
    strcpy(cli_stat_tbl[cli_stat_col_idx][cli_stat_row_idx], value);
}

/* Add table title */
void stat_tbl_add_title(struct json_object *data_obj, char ***cli_stat_tbl)
{
    struct json_object_iter
    {
        char *key;
        struct json_object *val;
        struct lh_entry *entry;
    };

    struct json_object_iter iter;
    char *title=malloc(sizeof(char)* CLI_STAT_MAXLEN);

    memset(title, '\0', CLI_STAT_MAXLEN);

    int cli_stat_row_idx = 1;
    json_object_object_foreachC(data_obj, iter){
        strcpy(title, iter.key);
        cli_stat_tbl_add(title, cli_stat_tbl, 1, cli_stat_row_idx);
        cli_stat_row_idx++;
    }
    free(title);
}

/* Add data to table */
void json_data_add_to_tbl(struct json_object *data_obj, char ***cli_stat_tbl, int cli_stat_col_idx)
{
    struct json_object_iter
    {
        char *key;
        struct json_object *val;
        struct lh_entry *entry;
    };

    struct json_object_iter iter;
    char *value=malloc(sizeof(char)* CLI_STAT_MAXLEN);

    memset(value, '\0', CLI_STAT_MAXLEN);

    int cli_stat_row_idx=1;
    json_object_object_foreachC(data_obj, iter){
        if(!json_object_object_get(data_obj, iter.key)){
        }
        else{
            switch( json_object_get_type(json_object_object_get(data_obj, iter.key))){
                case json_type_int:
                    sprintf(value, "%d", json_object_get_int(iter.val));
                    break;
                case json_type_string:
                    strcpy(value, json_object_get_string(iter.val));
                    break;
                case json_type_array:
                    strcpy(value, json_object_to_json_string(iter.val));
                    break;
                default:
                    break;
            }
        }
        cli_stat_tbl_add(value, cli_stat_tbl, cli_stat_col_idx, cli_stat_row_idx);
        cli_stat_row_idx++;
    }
    free(value);
}

/* Convert json string to status table */
int rep_agent_json_string_handler(char *json_string, char ***cli_stat_tbl)
{
    char *node_data;
    struct json_object *node_data_obj;
    struct json_object *new_obj;
    struct json_object *data_obj;
    int i=0, j;
    int cli_stat_col_idx=1;

    /* json string to json object */
    new_obj = json_tokener_parse(json_string);
    node_data = json_object_object_get(new_obj, "node-data");
    if(node_data == NULL)
        return -1;

    node_data_obj = json_object_array_get_idx( node_data, i);
    if(node_data_obj == NULL)
        return -1;

    while(node_data_obj != NULL){
        /* report agent data object */
        data_obj = json_object_object_get( node_data_obj, "data" );

        /* The data object may have two kinds of json type - array or object */
        if(json_object_get_type(data_obj) == json_type_object){
            cli_stat_tbl[cli_stat_col_idx] = malloc(sizeof(char)* CLI_STAT_MAXLEN);
            cli_stat_tbl[cli_stat_col_idx+1] = NULL;
            /* Add table title */
            stat_tbl_add_title(data_obj, cli_stat_tbl);
            cli_stat_col_idx++;
            cli_stat_tbl[cli_stat_col_idx] = malloc(sizeof(char)* CLI_STAT_MAXLEN);
            cli_stat_tbl[cli_stat_col_idx+1] = NULL;
            /* Add data to table */
            json_data_add_to_tbl(data_obj, cli_stat_tbl, cli_stat_col_idx);
            cli_stat_col_idx++;
        }
        else if(json_object_get_type(data_obj) == json_type_array){
            j=0;
            cli_stat_tbl[cli_stat_col_idx] = malloc(sizeof(char)* CLI_STAT_MAXLEN);
            cli_stat_tbl[cli_stat_col_idx+1] = NULL;
            /* Add title */
            stat_tbl_add_title(json_object_array_get_idx( data_obj, j), cli_stat_tbl);
            cli_stat_col_idx++;
            /* If data object is array type, add all elements to table, 
               (each element is a column of table)
             */
            while(json_object_array_get_idx( data_obj, j)){
                cli_stat_tbl[cli_stat_col_idx] = malloc(sizeof(char)* CLI_STAT_MAXLEN);
                cli_stat_tbl[cli_stat_col_idx+1] = NULL;
                json_data_add_to_tbl(json_object_array_get_idx( data_obj, j), cli_stat_tbl, cli_stat_col_idx);
                cli_stat_col_idx++;
                j++;
            }
        }
        i++;
        node_data_obj = json_object_array_get_idx( node_data, i);
    }
    return 0;
}
