/* syscall.c */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <dirent.h>
#include <sys/time.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/syslog.h>
#include "id.h"
#include "syscall.h"
#include "wlan.h"
#include "cipcfg.h"

// extern char **environ;
extern struct SYS_ITEM_LIST sys_item_list[] ;

int get_pid(char *process)
{
	int pid = 0;
	char buffer[256];
	FILE *fp;
	char *ptr;

	if ((fp = popen("/bin/ps", "r")) != NULL) {
		while (fgets(buffer, sizeof(buffer), fp) != NULL) {
			if (strstr(buffer, process) != NULL) {
				char *tok;
				tok = strtok_r(buffer, " ", &ptr);
				tok = strtok_r(buffer, " ", &ptr);
				pid = atoi(tok);
#ifdef DEBUG
				cipcfg_print("<debug>The pid of %s is %d\n", process,
					pid);
#endif
			}
		}
		pclose(fp);
	}
	else {
#ifdef DEBUG
		cipcfg_print("<error>Can't execute /bin/ps\n");
#endif
		return -1;
	}

	return pid;
}


int cip_system(char *cmd)
{
    if (cmd == 0) return 1;
    return system(cmd);
}

unsigned int checksum(char *addr, unsigned int count)
{
  unsigned int sum = 0;

  // Main summing loop
  while(count > 0)
  {
    sum = sum + (unsigned char)*((char *) addr)++;
    count--;
  }

  return(~sum);
}


int apply_sys(int sys_id)
{
    int ret = -1;
    const char *reboot_cmd="(sleep 5;reboot)&";
    pid_t pid;
    char buf[256];
    switch(sys_id)
    {
        case CMD_RENEW_IP:
            sprintf(buf, "%s check_dhcpc", PATH_RC_NET);
            ret = cip_system(buf);
            break;
        case CMD_REBOOT:
/*
            pid = fork();
            if (pid == 0) {
            	sleep(20); //delay 5 sec
            	printf("Child: Start rebooting system...\n");
		        syslog(LOG_USER | LOG_NOTICE, "Reboot system");
            	ret = system("reboot");
            }else if (pid < 0) {
            	printf("Fork reboot process fail!\n");
            } else {
                printf("Parent: Continue..\n");
            }
*/
            cipcfg_print("Reboot by system() in the background..\n");
            system(reboot_cmd);
            break;
        case CMD_CLEAR_SYSLOG:
            sprintf(buf, "%s restart_syslogd", PATH_RC_MGNT );
            ret = cip_system(buf);
            break;
	case CMD_GEN_SYSLOG:
            sprintf(buf, "%s > %s", PATH_LOGREAD, SYSLOG_MSG_FILE );
            ret = cip_system(buf);
	    break;
        default:
            // unknown operation
            ret = -1;
    }
    return ret;
}

int get_sys_item(char *item_name, char *item_value)
{
    int i;
    char *item_key;
    for (i=0; (sys_item_list[i].item != NULL); i++){
        if ( !strcmp(sys_item_list[i].item, item_name) ){
            item_key = sys_item_list[i].key ;
            sys_item_list[i].fp( item_key, item_value, 
		    sys_item_list[i].get_stat );
            break;
        }
    }

    if (sys_item_list[i].item != NULL){
        return CIPCFG_SUCCESS;
    }else{
        return CIPCFG_ERR_UNKNOWN_ITEM;
    }
}

/*
  for WDS Discovery

*/
void print_aps(AP_LIST *list)
{
    int count =0;

    while (list != NULL)
    {
        count++;
        printf("%d. SSID %s, MAC %s Channel %d, Singal %d, Security %s, Max Rate %d\n", 
                count, list->ssid, list->mac, list->ch, list->signal, list->security, list->rate);
        list = list->next;
    }
}

int free_ap_list(AP_LIST *list)
{
    AP_LIST *tmp;
    while (list != NULL) {
        tmp = list;
        list = list->next;
        free(tmp);
    }
    return 0;
}

/* 
 *  set system time
 */
void set_systime(char *systime, char *zone){
    char buf[128];

    unsetenv("TZ");
    if (zone != NULL) {
        sprintf(buf, "ln -sf /usr/share/zoneinfo/%s /tmp/localtime", zone);
        cip_system(buf); //adjust to local time ( GMT + ?? )
    }
    if (systime != NULL) {
        sprintf(buf, "date %s; hwclock -w -u", systime);
        cip_system(buf); // set date and write to RTC
        sprintf(buf, "/etc/rc.d/rc.mgnt restart_crond");
        cip_system(buf);
    }
}

int pkg_info(int f_size, char *value)
{
    FILE* fp = NULL;
    char *f = PATH_PKG_INFO;   
    if ((fp = fopen( f, "r")) != NULL) {
        fread(value, 1, f_size, fp);
        // value[f_size]= '\0';
        fclose(fp);
        return CIPCFG_SUCCESS;
    }else{
        return CIPCFG_ERR_UNKNOWN_ITEM;
    }
}


int chk_cfg_sync(char *chk_sync_file, void *retval)
{
    int *sync = (int *)retval;
    FILE* fp = NULL;
    if ((fp = fopen( chk_sync_file, "r")) != NULL) {
        fclose (fp);
	*sync = 1;
    }else{
	*sync = 0;
    }
    return CIPCFG_SUCCESS;
}


void print_arg(char *arg, char *value, int (*get_stat)(char *, void *))
{
    sprintf(value, arg);
}

void print_int(char *arg, char *value, int (*get_stat)(char *, void *))
{
    int num;

    if ((*get_stat)(arg, &num) == CIPCFG_SUCCESS)
	sprintf(value, "%d", num);
    else
	sprintf(value, "N/A");
}

void print_double(char *arg, char *value, int (*get_stat)(char *, void *))
{
    double num;

    if ((*get_stat)(arg, &num) == CIPCFG_SUCCESS)
	sprintf(value, "%f", num);
    else
	sprintf(value, "N/A");
}

void print_str(char *arg, char *value, int (*get_stat)(char *, void *))
{
    if ((*get_stat)(arg, value) == CIPCFG_ERR_GETSTAT)
	sprintf(value, "N/A");
}

void print_uptime(char *arg, char *value, int (*get_stat)(char *, void *))
{
    struct timeval tv;
    unsigned long min = 60;
    unsigned long hour = min*60;
    unsigned long day = hour*24;

    if ((*get_stat)(arg, &tv) == CIPCFG_SUCCESS) {
	/* Translate fotmat to days, hh:mm:ss. */
	sprintf(value, "%lu days, %lu:%02lu:%02lu", tv.tv_sec/day,
		(tv.tv_sec%day)/hour, (tv.tv_sec%hour)/min, tv.tv_sec%min);
    } else {
	sprintf(value, "N/A");
    }
}

int get_dev_ver(char *build_tag, void *retval)
{
    char *value = (char *)retval;
    FILE* fp = NULL;
    char ver[6];
    if ((fp = fopen( build_tag, "r")) != NULL) {
        fclose(fp);
        strncpy( ver, "(Rel)", 6 );
    }else{
        strncpy( ver, "(Dev)", 6 );
    }
    sprintf(value, "%s %s", SOFTWARE_VERSION, ver);
    return CIPCFG_SUCCESS;
}

int get_arg(char *arg, void *retval)
{
    char *value = (char *)retval;
    sprintf(value, "%s", arg);
    return CIPCFG_SUCCESS;
}

int get_date(char *date_file, void *retval)
{
    struct tm *now = (struct tm *)retval;
    time_t t;

    setenv("TZ", "", 1);
    tzset();
    unsetenv("TZ");
    tzset();

    time(&t);
    if (localtime_r(&t, now) == NULL)
	return CIPCFG_ERR_GETSTAT;
    else
	return CIPCFG_SUCCESS;
}

void print_date(char *arg, char *value, int (*get_stat)(char *, void *))
{
    struct tm now;
     
    if ((*get_stat)(arg, &now) == CIPCFG_SUCCESS) {
	strftime(value, MAX_VALUE_LEN, "%Y/%m/%d %H:%M:%S", &now);
    } else {
	sprintf(value, "N/A");
    }
}

void print_ip(char *arg, char *value, int (*get_stat)(char *, void *))
{
    struct sockaddr_in addr;

    if ((*get_stat)(arg, &addr) == CIPCFG_SUCCESS)
	/* Assumes address is in network byte order */
	sprintf(value, "%s", inet_ntoa(addr.sin_addr));
    else
	sprintf(value, "N/A");
}

void print_mac(char *arg, char *value, int (*get_stat)(char *, void *))
{
    unsigned char mac[6];

    if ((*get_stat)(arg, mac) == CIPCFG_SUCCESS) {
        sprintf(value, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
		mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
    } else {
	sprintf(value, "N/A");
    }
}

void print_band(char *arg, char *value, int (*get_stat)(char *, void *))
{
    enum band_type band;

    if ((*get_stat)(arg, &band) == CIPCFG_SUCCESS) {
	switch (band) {
	    case IEEE80211_A:
		sprintf(value, "802.11a");
		break;
	    case IEEE80211_TA: // Super Mode (turbo)
		sprintf(value, "802.11a(S)");
		break;
	    case IEEE80211_B:
		sprintf(value, "802.11b");
		break;
	    case IEEE80211_G:
		sprintf(value, "802.11g");
		break;
	    case IEEE80211_BG:
		sprintf(value, "802.11b+g");
		break;
	    case IEEE80211_N:
		sprintf(value, "802.11n");
		break;
	    case IEEE80211_AN:
		sprintf(value, "802.11a+n");
		break;
	    case IEEE80211_GN:
		sprintf(value, "802.11g+n");
		break;
	    case IEEE80211_BGN:
		sprintf(value, "802.11b+g+n");
		break;
	    case IEEE80211_ABGN:
		sprintf(value, "802.11a+b+g+n");
		break;
	    case IEEE80211_DISABLED: 
		sprintf(value, "Disabled");
		break;
	    case IEEE80211_UNKNOWN: 
	    default:
		sprintf(value, "Unknown Band");
	} 
    } else {
	sprintf(value, "N/A");
    }
}

int get_file_content(char *file_name, void *retval)
{
    char *value = (char *)retval;
    FILE* fp = NULL;
    char *ptr;

    if ((fp = fopen( file_name, "r")) != NULL) {
        fgets(value, MAX_VALUE_LEN, fp);
        fclose(fp);
        if( !strcmp(PATH_KERNEL, file_name) ){
            strtok_r( value, "(", &ptr );
        }
	return CIPCFG_SUCCESS;
    }else{
        cipcfg_print("can not open file : %s\n", file_name);
	return CIPCFG_ERR_GETSTAT;
    }
}

int get_product_info(char *key, void *retval)
{
    char *value = (char *)retval;
    int flag = 0;
    FILE* fp = NULL;
    char buf[MAX_VALUE_LEN];

    if ((fp = fopen( PATH_PRODUCT_INFO, "r")) != NULL ){
        while( fgets(buf, MAX_VALUE_LEN, fp) != NULL ){
            if( !strncmp(buf, key, strlen(key)) ){
                if( !sscanf(buf, "%*[^\"]\"%[^\"]", value) ){
                    cipcfg_print("the buf is :%s\n", buf);
                    cipcfg_print("the value is :%s\n", value);
                    cipcfg_print("ERROR!! Didn't match pattern in %s\n", PATH_PRODUCT_INFO);
                }else{
                    flag = 1;
                    break;
                }
            }
        }
        if(!flag){ // Didn't get value
            cipcfg_print("didn't find item \"%s\" in file: %s\n", key, PATH_PRODUCT_INFO);
            fclose(fp);
            return CIPCFG_ERR_GETSTAT;
        }
        fclose(fp);
    }else{
        cipcfg_print("can not open file : %s\n", PATH_PRODUCT_INFO);
        return CIPCFG_ERR_GETSTAT;
    }
    return CIPCFG_SUCCESS;
}

int chkFeature(char *key, char *value)
{
    int flag = 0;
    FILE* fp = NULL;
    char buf[MAX_VALUE_LEN];

    if ((fp = fopen( PATH_FEATURE_CTRL, "r")) != NULL ){
        while( fgets(buf, MAX_VALUE_LEN, fp) != NULL ){
            if( !strncmp(buf, key, strlen(key)) ){
                if( !sscanf(buf, "%*[^\"]\"%[^\"]", value) ){
                    cipcfg_print("the buf is :%s\n", buf);
                    cipcfg_print("the value is :%s\n", value);
                    cipcfg_print("ERROR!! Didn't match pattern in %s\n", PATH_FEATURE_CTRL);
                }else{
                    flag = 1;
                    break;
                }
            }
        }
        fclose(fp);
        if(!flag){ // Didn't get value
            cipcfg_print("didn't find item \"%s\" in file: %s\n", key, PATH_FEATURE_CTRL);
            sprintf(value, "N");
            return CIPCFG_ERR_UNKNOWN_ITEM;
        }else{
            return CIPCFG_SUCCESS;
        }
    }else{
        cipcfg_print("can not open file : %s\n", PATH_FEATURE_CTRL);
        sprintf(value, "N");
        return CIPCFG_ERR_OPENFILE;
    }
}

/*
 * Use to find out mtd number by name 
 * @param: name: partition name
 * @param: mtd_no: mtd no.
 * return: 0: success
           < 0: fault
 */
#define BUF_SIZE    80
int find_partition(const char *name, char *partition)
{
    FILE *fp = NULL;
    char buf[BUF_SIZE], full_name[BUF_SIZE];
    int ret = -1;
    char *ptr;
    char *tmp;

    memset(full_name, 0, BUF_SIZE);
    fp = fopen("/proc/mtd", "r");
    if (fp != NULL) {
        while (!feof(fp)) {
            fgets(buf, sizeof(buf), fp);
            sprintf(full_name, "\"%s\"", name);
            if (strstr(buf, full_name)) {
                if((tmp = strtok_r(buf, ":", &ptr)) != NULL) {
                    sprintf(partition, "%s", tmp);
                    ret=0;
                    cipcfg_print("find_mtd: %s : %s\n", name, partition);
                    break;
                }
            }
        }
        fclose(fp);
    }
    else {
        cipcfg_print("Error, Can't open /proc/mtd\n");
        ret=-2;
    }
    return ret;
}

int find_mtd(const char *name, char *mtd)
{
    char    buf[BUF_SIZE];
    int     ret;
    
    memset(buf, 0, BUF_SIZE);
    ret = find_partition(name, buf);
    if(ret >= 0)
        sprintf(mtd, "/dev/%s", buf);
    return ret;
}

int find_mtdblock(const char *name, char *mtdblock)
{
    char    buf[BUF_SIZE];
    int     ret;
    
    memset(buf, 0, BUF_SIZE);
    ret = find_partition(name, buf);
    if(ret >= 0)
        sprintf(mtdblock, "/dev/mtdblock%c", buf[strlen(buf)-1]);
    return ret;
}

int cip_ping(char *url)
{
    char *p, cmd[304];
    p = strpbrk(url, ";<>\"'@!?&$*"); // illegal characters
    if ( !p ){
        sprintf( cmd, "%s -c 5 %s > %s 2>&1 &", PATH_PING, url, PATH_PING_RESULT);
        cip_system(cmd);
        return CIPCFG_SUCCESS;
    }else{
        return CIPCFG_ERR_INVALID_ITEM;
    }
}

int cip_ping_p(char *url, int *pid_p)
{
    char *p, cmd[304], buf[256];
    FILE *fp = NULL;
    p = strpbrk(url, ";<>\"'@!?&$*"); // illegal characters
    if ( !p ){
	int pid;

	unlink(PATH_PING_RESULT);
        sprintf( cmd, "%s -c 5 %s > %s 2>&1 & echo $!", PATH_PING, url, PATH_PING_RESULT);
        //cip_system(cmd);
	fp = popen(cmd, "r");
	if (fp == NULL)
	    return CIPCFG_ERR_INVALID_ITEM;
	*buf = '\0';
	fgets(buf, 256, fp);
	pclose(fp);
	pid = atoi(buf);
	//printf("pid=%d, buf=%s\n", pid, buf);
	if (pid <= 0)
	    return CIPCFG_ERR_INVALID_ITEM;
	if (pid_p != NULL)
	    *pid_p = pid;
        return CIPCFG_SUCCESS;
    }else{
        return CIPCFG_ERR_INVALID_ITEM;
    }
}

int cip_tracert_p(char *url, int *pid_p)
{
    char *p, cmd[304], buf[256];
    FILE *fp = NULL;
    p = strpbrk(url, ";<>\"'@!?&$*"); // illegal characters
    if ( !p ){
	int pid;

	unlink(PATH_TRACERT_RESULT);
        sprintf( cmd, "%s -q 2 -w 2 -m 20 %s > %s 2>&1 & echo $!", PATH_TRACERT, url, PATH_TRACERT_RESULT);
	fp = popen(cmd, "r");
	if (fp == NULL)
	    return CIPCFG_ERR_INVALID_ITEM;
	*buf = '\0';
	fgets(buf, 256, fp);
	pclose(fp);
	pid = atoi(buf);
	if (pid <= 0)
	    return CIPCFG_ERR_INVALID_ITEM;
	if (pid_p != NULL)
	    *pid_p = pid;
        return CIPCFG_SUCCESS;
    }else{
        return CIPCFG_ERR_INVALID_ITEM;
    }
}

int getpwd(int len, char *passwd)
{

int res;
char pwd[40];
int j=0;

struct timeval t;

gettimeofday(&t,NULL);

srand(t.tv_usec);
for(j=0;j<len;j++){
   	int ptr = rand()%2+1;

	if(ptr ==1){
		res = rand()%8+50;
	}else{ 
   		res = rand()%26+97;

		if(res==105){
			res = 101;
		}
		
		if(res==106){
			res = 109;		
		}
	
		if(res==108){
			res = 107;		
		}

		if(res==111){
			res = 115;		
		}
	}

	pwd[j]=(char)res;
}

pwd[j]='\0';

strcpy(passwd, pwd);
}

int enc(char *src, char *dst,int shift)
{
        const char s_code[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-/";

        int remain = 0,  p = 0, i;
        int tmp;
        char in_buff[4096];
        char out_buff[8192];

        strcpy(in_buff,src);
        out_buff[0] = s_code[shift];


        for (i = 0; i< strlen(in_buff); i++)
        {
                tmp = in_buff[i] & 0xff;
                switch (i % 3)
                {
                        case 0:
                        out_buff[++p] =  s_code[ ((tmp >> 2) - shift) & 0x3f];
                        remain = (tmp << 4) & 0x3f;
                        break;

                        case 1:
                        out_buff[++p] =  s_code[((remain | (tmp >> 4)) - shift) & 0x3f];
                        remain = (tmp << 2) & 0x3f;
                        break;

                        case 2:
                        out_buff[++p] = s_code[((remain | (tmp >> 6)) - shift) & 0x3f];
                        out_buff[++p] = s_code[((tmp & 0x03f) - shift) & 0x3f];
                        break;
                }
        }
        if (i % 3 != 0)
                out_buff[++p] = s_code[(remain - shift) & 0x3f];
        out_buff[++p] = '\0';

        dst[0]='\0';
        sprintf(dst,"%s%03d",out_buff,p);

        return p;
}


int dec(char *src, char *dst)
{
        char s_code[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-/";

        int remain = 0,  p = 0, i;
        char *in_buff=src;
        char *out_buff=dst;
        int sub_l,str_l;
        int tmp, shift = 0;
        char *tmp_str;

        if(strlen(in_buff)<3)
        {
                cipcfg_print("No data!\n");
                out_buff[0]='\0';
                return -1;
        }
        out_buff[0] = s_code[shift];

        tmp_str=&in_buff[strlen(in_buff)-3];
        //  printf("TMP:%s\n",tmp_str);
        sub_l=atoi(tmp_str);
        in_buff[strlen(in_buff)-3]='\0';
        str_l=strlen(in_buff);
        if (str_l != sub_l)
        {
                cipcfg_print("checksum error! %d != %d\n", str_l,sub_l);
                out_buff[0]='\0';
                return -1;
        }

        for (i = 0; i<str_l; i++)
        {
                tmp_str=strchr(s_code,in_buff[i]);
                if(NULL==tmp_str)
                {
                        break;
                }
                tmp=strlen(s_code)-strlen(tmp_str);
                if (i == 0)
                {
                        shift = tmp;
                //      printf("Shift=%d\n",shift);
                }
                else
                {
                        tmp = (tmp + shift) & 0x3f;
                        switch (i % 4)
                        {
                                case 1:
                                        remain = (tmp << 2) & 0xff;
                                        break;
                                case 2:
                                        out_buff[p++]= (remain | (tmp >> 4));
                                        remain  = ((tmp << 4) & 0xff);
                                        break;
                                case 3:
                                        out_buff[p++]=(remain | (tmp >> 2));
                                        remain  = ((tmp << 6) & 0xff);
                                        break;
                                case 0:
                                        out_buff[p++]=(remain | tmp);
                                        break;
                        }
                }
        }

        out_buff[p]='\0';
        // printf("Source:%s\n",in_buff);
        // printf("%s\n",out_buff);
        return 1;
}

void checkSession(char *sess_str,  char *ipAdd, char *macAdd)
{
   char *p1, *p2;
   char ipAddr[12], macAddr[24], sysTime[24];
   int ipLen, macLen;
   char session[1024];

   if( !strlen( sess_str ))
          exit(printf( "Error\n"));

   /* Decode the encoded session attributes */
   dec(sess_str, session);
 
   /* parse IP */
   p1 = strstr(session, "ip");
   p2 = strstr(session, "&mac");

   ipLen = (p2 - p1) - 3; //3 = len("ip");
   strncpy(ipAddr, session + 3, ipLen); ipAddr[ipLen] = '\0';

   strcpy(ipAdd, ipAddr);

   /* parse MAC */
   p1 = strstr(session, "&mac");
   p2 = strstr(session, "&time");
   macLen = (p2 - p1) - 5;
   strncpy(macAddr, session + (ipLen+8), macLen); macAddr[macLen] = '\0';
   strcpy(macAdd, macAddr);

   return;
}

int get_group_index( int vlanNo, int mgmt_ind)
{
    CIPCFG *cipcfg = NULL;
    char group_name[MAX_VALUE_LEN], auth[MAX_VALUE_LEN], policyname[MAX_VALUE_LEN];
    int ret, policyInd;

    cipcfg = init_cipcfg();
    if( !cipcfg ) {
	cipcfg_print("Init cipcfg error.\n");
	return 0;
    }
    sprintf(group_name, "vlan%d%s", vlanNo, "_policy");
    sprintf(auth, "auth%d", mgmt_ind);

    memset(policyname, '\0', MAX_VALUE_LEN);
    ret = get_cfg_item_by_str( cipcfg, group_name, auth, policyname);
    if (ret != CIPCFG_SUCCESS) {
        cipcfg_print("Either the VLAN NO=%d. or the Auth Server No=%d. is wrong.\n",vlanNo, mgmt_ind);
        close_cipcfg(cipcfg);
        return -1;
    }
   
    if( strcmp(policyname, "disable") == 0 )
  	policyInd = -1;
    else
	policyInd = atoi( policyname );

    close_cipcfg(cipcfg);
    return policyInd ;
}
 
      
int get_mgmt_index(char *username, int vlanId)
{
    char postfix[POSTFIX_LEN], *tmp, group_name[20], vlanName[20];
    CIPCFG *cipcfg=NULL;
    char mgmt_ind[5], cpostfix[POSTFIX_LEN];
    int pflag = 0, ret, i, mgmtInd = -1;
    char *ptr;

    if(strstr(username, "@"))
        pflag = 1;

    tmp = strtok_r(username, "@", &ptr);
    tmp = strtok_r(NULL, "@", &ptr);
    strcpy(postfix, tmp);

    cipcfg=init_cipcfg();
    if(!cipcfg){
        cipcfg_print("initial cipcfg error.\n");
        return 0;
    }

    sprintf(vlanName, "vlan%d%s", vlanId, "_policy");
    for(i=0 ; i < MAX_AUTH_SERVERS; i++){
        sprintf(group_name, "auth%d", i);
        if(pflag)
                ret = get_cfg_item_by_str(cipcfg, group_name, "postfix", cpostfix);
        else
                ret = get_cfg_item_by_str(cipcfg, vlanName, "defauth", mgmt_ind);
		

        if (ret != CIPCFG_SUCCESS) {
                cipcfg_print("Wrong Postfix\n");
                break;
        }
        if(pflag) {
                if(!strcmp(cpostfix, postfix) ){
                        ret = get_cfg_item_by_str(cipcfg, group_name, "idx", mgmt_ind);
			mgmtInd = atoi(mgmt_ind)+1 ;
                }
        }
        else
        {
                mgmtInd = atoi( mgmt_ind) + 1;
        }
     }

     close_cipcfg(cipcfg);
     return (mgmtInd);
}



/* Get the VLAN No. using the client IP */
int getVlanbyIP(char *clientip)
{
   FILE *fd;
   char temp[50];
   int vlanNo;

   sprintf(temp, "/ramfs/bin/getVlanbyIP.sh %s", clientip);
   system(temp);

   sprintf(temp, "/tmp/status/%s", clientip); 
   if( (fd=fopen(temp, "r")) == NULL)
       return -1;

   fscanf(fd, "%d", &vlanNo);
   fclose(fd);
   unlink(temp);

   return vlanNo;
}

int getAuthStatus(char *mac)
{
   struct stat buf;
   char fname[50], ErrFile[50];
   int i;

   sprintf(fname, "/tmp/authStatus/%s", mac);
   sprintf(ErrFile, "/tmp/authStatus/ERR-%s", mac);
 
   cipcfg_print("checking MAC : %s\n", mac);
   for( i=0 ; i < 100; i++)
   {

        if( !stat(fname, &buf) )
	{
		unlink(fname);
		return 1;
	}
        if( !stat(ErrFile, &buf) )
	{
		unlink(ErrFile);
		return 0;
	} 
  }
  return 0;
}	   

/* TODO: It is from account management package, so these codes
 * 	 (user account related functions) have to move out from 
 *	 cipcfg. 
 */       
#define LOGIN_USER_NAME_LINE            1
#define LOGIN_USER_FNAME_LINE           2
/* Find Whether the user is online or not */
int userStatus(char *uname, int fullname)
{
    char userdir[]="/tmp/online/.";
    DIR *pdir = opendir(userdir);
    struct dirent* pent;
    struct stat dir_stat;
    FILE *fp;
    char buf[512];

    if ( pdir ) {
        cipcfg_print("Scanning Dirs..\n");
        chdir(userdir);
        while ((pent = readdir(pdir))) {
            if (stat(pent->d_name, &dir_stat) == -1)
                continue;
            if ( S_ISDIR(dir_stat.st_mode) )
                continue;

            if ((fp = fopen( pent->d_name, "r")) != NULL ) {
		char *p;
    		int found = 0, i = 0;
                while (fgets( buf, sizeof(buf), fp ) != NULL) {
 		    if (p = strchr(buf, '\n'))
			*p = '\0';
		    switch (i) {
		    case LOGIN_USER_NAME_LINE:
		        if (fullname == 0 && !strcmp(buf, uname))
			    found = 1;
			break;
		    case LOGIN_USER_FNAME_LINE:
			if (fullname != 0 && !strcmp(buf, uname))
			    found = 1;
			break;
		    default:
			if (i > LOGIN_USER_NAME_LINE && i > LOGIN_USER_FNAME_LINE)
			    goto NEXTUSER;
			break;
		    }
		    if (found != 0)
                        goto USERFOUND;
		    i++;
	        }
NEXTUSER:
         	fclose(fp);
            }
        }
        closedir(pdir);
    }
    cipcfg_print("User / Dir not found..\n");
    return 0;

USERFOUND:
    fclose(fp);
    closedir(pdir);
    return 1;
}

unsigned long getexptime(int nDays)
{
    unsigned long l , f;
    time_t tnow;

    time(&tnow);
    l = tnow;
    f = (86400*(unsigned long)nDays) + l;
    return( f );
}

/*
 * Use to get value in rc.env 
 * @param: name
 * @param: value
 * return: value: success
 *         NULL: fail
 */
char *get_rc_env(char *name, char *value, int len)
{
    FILE *fp;
    char buf[MAX_VALUE_LEN];
    if (!name || !*name || !value || len <= 0)
        return NULL;
    *value = '\0';
    sprintf(buf, ". /etc/rc.d/rc.env; echo -n \"$%s\"", name);
    if (!(fp = popen(buf, "r")))
        return NULL;
    *buf = '\0';
    fgets(buf, sizeof(buf), fp);
    pclose(fp);

    if (len < sizeof(buf))
        buf[len-1] = '\0';
    strcpy(value, buf);

    return value;
}

int ip_strtoaddr(char *ipstr, unsigned long *ipaddr)
{
    int ret;
    struct in_addr inval;

    if (!ipstr)
	return -1;
    if (!ipaddr)
	return 0;

    ret = inet_aton(ipstr, &inval);
    // ret = inet_pton(AF_INET, ipstr, &inval);
    if (ret <= 0)
	return ret;

    *ipaddr = ntohl(inval.s_addr);
    return ret;
}

char *ip_addrtostr(unsigned long ipaddr, char *ipstr)
{
    struct in_addr inval;
    char str[INET_ADDRSTRLEN];

    if (!ipstr)
	return NULL;
    inval.s_addr = htonl(ipaddr);
    if (!inet_ntop(AF_INET, &inval, str, INET_ADDRSTRLEN))
	return NULL;
    strcpy(ipstr, str);
    return ipstr;
}

int get_ipmask(char *ipstr, char *maskstr, unsigned long *broadcast, unsigned long *netaddr)
{
    int ret;
    unsigned long ipaddr, netmask;

    if (!ipstr || !maskstr)
	return -1;
    if (!broadcast || !netaddr)
	return 0;
    ret = ip_strtoaddr(ipstr, &ipaddr);
    if (ret <= 0)
	return ret;
    ret = ip_strtoaddr(maskstr, &netmask);
    if (ret <= 0)
	return ret;
    *broadcast = ipaddr | (~netmask);
    *netaddr = ipaddr & netmask;
    return ret;
}

// direct = 2 mean hex-to-bin; direct = 1 mean bin-to-hex
char* hexbinconvert(char* attr, int direct)
{
/*
	char *istr1 = "Class01", *ostr1 = NULL;
	char *istr2 = "0x436c6173733031", *ostr2 = NULL;
*/
	char *output = NULL;
	char buf[3];
	int len, i, c;

	if (!attr)
		return NULL;
	len = strlen(attr);
	if (direct == 1) {
	    if (len > 0) {
		output = (char *) calloc(len*2+3, sizeof(char));
		if (output != NULL) {
			strcpy(output, "0x");
			for (i=0; i<len; i++) {
				sprintf(output+i*2+2, "%02x", (int) ((unsigned char) attr[i]));
			}
		}
		//printf("istr1 = %s, ostr1 = %s\n", istr1, ostr1);
	    }
	}
	else if (direct == 2) {
	    if (!(len%2) && !strncmp(attr, "0x", 2)) {
		output = (char *) calloc((len-2)/2+1, sizeof(char));
		if (output != NULL) {
			for (i=2; i<len; i+=2) {
				strncpy(buf, attr+i, 2);
				buf[2] = 0;
				c = strtol(buf, NULL, 16);
				// sscanf(buf, "%02X", &c);
				output[(i-2)/2] = c;
			}
		}
		//printf("istr2 = %s, ostr2 = %s\n", istr2, ostr2);
	    }
	}
	return output;
}
