#include <stdio.h>
#include <string.h>
#include "cipcfg.h"

#define MAX_ATTRLEN     64
#define MAX_DELIMLEN    8
#define MAX_VALUELEN    512

extern unsigned int magic;
extern char* config_rom;
void migrate(CIPCFG *target_cfg, CIPCFG *data_cfg);
int pre_integrity(CIPCFG *target_cfg, CIPCFG *data_cfg);
int post_integrity(CIPCFG *target_cfg);
int write_cipcfg(char *target_file, CIPCFG *target_cipcfg);


int main(int argc, char *argv[])
{
    if( argc < 2 ){
        printf("Usage: %s DATA_SOURCE_FILE [SYSTEM_COMPATIBLE_FILE] [TARGET_FILE]\n\n", argv[0]);
        printf("if not input [SYSTEM_COMPATIBLE_FILE] [TARGET_FILE],\n");
        printf("the default will be %s & %s\n", DEFAULT_CONFIG, config_rom );
        return -1;
    }

    int ret;
    char *target_file, *sys_file;
    CIPCFG  *target_cfg, *data_cfg;

    // initialization
    sys_file = DEFAULT_CONFIG;
    target_file = config_rom;
    data_cfg = init_cipcfg_f(argv[1]);
    if(argc >= 3){    
        sys_file = argv[2]; 
    }
    if(argc >= 4){ 
        target_file = argv[3]; 
    }
    target_cfg = init_cipcfg_f(sys_file);

    printf("\nThe data source file is %s,\nsystem compatible file is %s, and\ntarget file is %s\n\n",argv[1],sys_file, target_file);
    if ( !data_cfg || !target_cfg ) {
        printf("Initial config error!! code : %d\n", CIPCFG_ERR_NODE_NOT_FOUND);
        close_cipcfg(target_cfg);
        close_cipcfg(data_cfg);
        return CIPCFG_ERR_NODE_NOT_FOUND;
    }

    // Step 1: pre-integrity
    ret = pre_integrity(target_cfg, data_cfg);
    if (ret != CIPCFG_SUCCESS ){
        printf("Pre-migration error!! code : %d\n", ret);
        close_cipcfg(target_cfg);
        close_cipcfg(data_cfg);
        return ret;
    }


    // Step 2: migration

    // check <cipconfig> first
    ret = strcmp(target_cfg->name, data_cfg->name);
    if ( 0 != ret ){
        printf("migration error!!\n" );
        close_cipcfg(target_cfg);
        close_cipcfg(data_cfg);
        return CIPCFG_ERR_INVALID_ITEM;
    }
    ret = strcmp(ezxml_attr(target_cfg, "devtype"), ezxml_attr(data_cfg, "devtype"));
    if ( 0 != ret ){
        printf("Config migration error!! Incompatible device type.\n" );
        close_cipcfg(target_cfg);
        close_cipcfg(data_cfg);
        return CIPCFG_ERR_ILLEGAL_CFGFILE;
    }
    int i=0;
    // set verion from system to data first
    while (target_cfg->attr[i] != NULL && target_cfg->attr[i+1] != NULL) {
        // check if attr is in target_node
        if (ezxml_attr(data_cfg, target_cfg->attr[i]) != NULL) {
            ezxml_set_attr_d(data_cfg, target_cfg->attr[i], target_cfg->attr[i+1]);
        }
        i = i+2;
    }
    migrate(target_cfg, data_cfg);
    close_cipcfg(data_cfg);

    // Step 3: post-integrity
    ret = post_integrity(target_cfg);
    if (ret != CIPCFG_SUCCESS ){
        printf("Post-migration error!! code : %d\n", ret);
        close_cipcfg(target_cfg);
        return ret;
    }

    // Step 4: write to target file
    ret = write_cipcfg(target_file, target_cfg);
    if (ret != CIPCFG_SUCCESS ){
        printf("write back to file \"%s\" error!!\n", target_file );
    }

    close_cipcfg(target_cfg);
    
    return ret;
}   // END of main


/*
 *  There are 2 hypothesis of the migration function about the two CIPCFG:
 *      1. the XML structures are the same.
 *      2. In these two CIPCFG, the same item name carry with the same meaning.
 *
 *  And the function do overwrite item values from data_node to 
 *  target_node ONLY when the item exist in target_node.
 */
void migrate(CIPCFG *target_node, CIPCFG *data_node)
{
    int i = 0 ;
    CIPCFG *t = target_node, *d = data_node, *tmp;

    if ( !d || !t ) {
        return ;
    }

    while (d->attr[i] != NULL && d->attr[i+1] != NULL) {
        // check if attr is in target_node
        if (ezxml_attr(t, d->attr[i]) != NULL) {
	    ezxml_set_attr_d(t, d->attr[i], d->attr[i+1]);
        }
        i = i+2;
    }

    // recursive : Child
    if ( t->child && d->child ){
        if ( !strcmp(t->child->name, d->child->name) ) {
            // the same node, just do migration.
            migrate(t->child, d->child);
        }else{ // diffrernt node
            tmp = t->child;
            while( tmp->sibling ){
                if ( !strcmp(tmp->sibling->name, d->child->name) ) {
                    migrate(tmp->sibling, d->child);
                    break;
                }else{
                    tmp = tmp->sibling ;
                }
            }
        }
    }

    // recursive : Sibling
    if ( t->sibling && d->sibling ){
        if ( !strcmp(t->sibling->name, d->sibling->name) ) {
            migrate(t->sibling, d->sibling);
        }else{
            tmp = t->sibling;
            while( tmp->sibling ){
                if ( !strcmp(tmp->sibling->name, d->sibling->name) ) {
                    migrate(tmp->sibling, d->sibling);
                    break;
                }else{
                    tmp = tmp->sibling ;
                }
            }
        }
    }

    // recursive : Next
    if ( t->next && d->next ) {
        migrate(t->next, d->next);
    }
}


/*
 *  Check if XML hierarchy is NOT the same at data_cfg & target_cfg.
 */
int pre_integrity(CIPCFG *target_cfg, CIPCFG *data_cfg)
{
    int ret = CIPCFG_SUCCESS;
    // add check function if necessary when system revision
    // check_function_1();
    // check_function_2();
    // check_function_3();
    //     .
    //     .
    //     .

    return ret;
}


/*
 *  Check logical dependency between attributes.
 */
int post_integrity(CIPCFG *target_cfg)
{
    int ret = CIPCFG_SUCCESS;
    // add check function if necessary when system revision
    // check_function_1();
    // check_function_2();
    // check_function_3();
    //     .
    //     .
    //     .

    return ret;
}

/*
 *  write target_cfg to target_file
 */
int write_cipcfg(char *target_file, CIPCFG *target_cipcfg)
{
    FILE *fp;
    char *xml_str;

    xml_str=ezxml_toxml(target_cipcfg);
    fp=fopen(target_file, "w");
    if (!fp) {
        free(xml_str);
        return CIPCFG_ERR_OPENFILE;
    }

    fputs(xml_str, fp);
    fflush(fp); //force write data to flash
    fclose(fp);

    free(xml_str);
 
    return CIPCFG_SUCCESS;
}


