#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <ctype.h>
#include "cipcfg.h"


extern group_row_t *group_table_row(int gid);
extern group_row_t *group_table_row_str(char *group_name);
extern group_row_t *group_table_prefix(char *prefix);
extern CIPCFG *cipcfg_search(CIPCFG *root, char name[]);
extern group_row_t group_table[];
extern int get_cfg_item(CIPCFG *cipcfg, int gid, char *item_name, char *item_value);
extern int get_cfg_item_by_str(CIPCFG *cipcfg, char *group_name, char *item_name, char *item_value);

#define MAX_GNAMELEN    128
#define MAX_TNAMELEN    256
#define MAX_TAILPATH    512
#define MAX_FULLPATH    1024
#define MAX_VARPREFIX   16
#define MAX_ATTRLEN     64
#define MAX_DELIMLEN    8
#define MAX_VALUELEN    512
#define MAX_FILENAMELEN 512

#define CMDSTR_RESET    "reset"
#define CMDSTR_ZERO     "0"

enum { CMD_NONE=0, CMD_ZERO, CMD_GROUPID, CMD_GROUPNAME, 
       CMD_RESET, CMD_GROUPID_PATH, CMD_GROUPNAME_PATH } ;
enum { LVAR_NONE=0, LVAR_LOCAL, LVAR_EXPORT} ;

static char aname[MAX_ATTRLEN + 1] = "" ;
static char gname[MAX_GNAMELEN + 1] = "" ;
static char tname[MAX_TNAMELEN + 1] = "" ;
static int gid = -1 ;

static struct my_options {
    char cfg_file[MAX_FILENAMELEN + 1] ;
    char delim[MAX_DELIMLEN + 1] ;
    int lvar ;
    char varpfx[MAX_VARPREFIX + 1] ;
/* TODO:    int fullp ; */
} my_options = {
    "",
    "_",
    0,
    "",
} ;

/*  for Shell Script only !!
 *  Special Characters like $ " ` \ in attributes for Shell Script
 *  Encodes backslash , \ , appending the results to *dst.
 */
char *bash_str_encode(char *src, char *dst)
{
    const char *e;
    size_t dlen = 0;
    // size_t max = MAX_VALUELEN ; 
    for (e = src-1 ; src != e; src++) {
        // while (*dlen + 10 > max) *dst = realloc(*dst, max += MAX_VALUELEN );
        switch (*src) {
        case '\0': return dst;
        case '$': dlen += sprintf(dst + dlen, "\\$"); break;
        case '"': dlen += sprintf(dst + dlen, "\\\""); break;
        case '`': dlen += sprintf(dst + dlen, "\\`"); break;
        case '\\': dlen += sprintf(dst + dlen, "\\\\"); break;
        default: dst[dlen++] = *src;
        }
    }
    return dst;
}


/* rely on global variable 'my_options' to determine its output */
void print_attr(char *attrname, int idx, char *attrval)
{
    char *str_local = "" ;
    char *str_prefix = my_options.varpfx ;
    /* TODO */
#if 0
    char *str_delim = my_options.delim ;
#endif
    char str_attr[MAX_ATTRLEN+1] ;
    char value[MAX_VALUELEN];
    memset(value, 0, MAX_VALUELEN);

    if(idx >= 0)
        sprintf(str_attr, "%s%s%d", str_prefix, attrname, idx) ;
    else sprintf(str_attr, "%s%s", str_prefix, attrname) ;

    if(my_options.lvar)
        str_local = "local " ; /* "local" followed by a blank " " */
    bash_str_encode( attrval, value );
    printf("%s%s=\"%s\"\n", str_local, str_attr, value);
}

int dump_cfg_by_gid(CIPCFG *cipcfg, int gid, char *path)
{
    CIPCFG *node;
    group_row_t *group;
    struct attribute *head = NULL;

    group=group_table_row(gid);
    if (!group) {       // no matched group !
        return CIPCFG_ERR_UNKNOWN_GROUP;
    }

    if (path == NULL) {
        node=cipcfg_search(cipcfg, group->prefix);
    } else {
        char full_path[MAX_FULLPATH];
        sprintf(full_path, "%s.%s", group->prefix, path);
        node=cipcfg_search(cipcfg, full_path);
    }
    if (!node) {
        return CIPCFG_ERR_NODE_NOT_FOUND;
    }
    
    head = get_attr(node, group->prefix, -1, -1);
    while(head != NULL)
    {
        print_attr(head->attr_name, head->idx, head->attr_value);
        head = head->next;
    }
    free_attr_list(head);
    return CIPCFG_SUCCESS;
}

int dump_cfg_by_group_name(CIPCFG *cipcfg, char *group_name, char *path)
{
    CIPCFG *node;
    group_row_t *group;
    struct attribute *head = NULL;
    char full_path[MAX_FULLPATH];

    /* if gid=0, get the cipconfig version and devtype */
    if ( !strcmp(group_name,"0") || !strcmp(group_name,"0x0") ) {
        print_attr(cipcfg->attr[0], -1, cipcfg->attr[1]);
        print_attr(cipcfg->attr[2], -1, cipcfg->attr[3]);
        return CIPCFG_SUCCESS;
    }
    
    group=group_table_row_str(group_name);
    if (!group) {       // no matched group !
        return CIPCFG_ERR_UNKNOWN_GROUP;
    }

    if (path == NULL) {
        node=cipcfg_search(cipcfg, group->prefix);
    } else {
        sprintf(full_path, "%s.%s", group->prefix, path);
        node=cipcfg_search(cipcfg, full_path);
    }

    if (!node) {
        return CIPCFG_ERR_NODE_NOT_FOUND;
    }
    
    head = get_attr(node, group->prefix, -1, -1);
    while(head != NULL)
    {
        print_attr(head->attr_name, head->idx, head->attr_value);
        head = head->next;
    }
    free_attr_list(head);
    return CIPCFG_SUCCESS;
}

void usage(char *cmdline)
{
        printf("Usage: %s [ options ] { gid | groupname | reset | 0 } [attr]\n", cmdline);
        printf("Options:\n"
               "  -F filename : specify the config file name\n"
               "  -d delim    : delimiter, default is '_' \n"
               "  -l          : add 'local' before every line of output \n"
               "  -p pfx      : put pfx as the variable prefix\n");
/* TODO:       "  -f          : output full path of attributes\n" */
        printf("gid           : can be a decimal/hexidecimal number\n"
               "groupname     : name of a specific group\n"
               "reset         : reset to factory default\n"
               "0             : print version info and device type\n") ;
/*             "  listall     : list all groups" */

}


/* returns: cmdval */
int parse_opts(int argc, char *argv[], struct my_options *opts)
{

    while(1) {
        int c = getopt(argc, argv, "d:lp:fF:") ;
        if( c == -1 )
            break ;

        switch(c) {
            case 'd':
                opts->delim[MAX_DELIMLEN] = '\0' ;
                if(optarg)
                    strncpy(opts->delim, optarg, MAX_DELIMLEN) ;
                else 
                    return -1 ;
                break ;

            case 'l':
                opts->lvar = 1 ;
                break ;
/* TODO
            case 'f':
                opts->fullp= 1 ;
                break ;
*/
            case 'p':
                opts->varpfx[MAX_VARPREFIX] = '\0' ;
                if(optarg)
                    strncpy(opts->varpfx, optarg, MAX_VARPREFIX) ;
                else return -1 ;
                break ;

            case 'F':
                opts->cfg_file[MAX_FILENAMELEN] = '\0' ;
                if(optarg)
                    strncpy(opts->cfg_file, optarg, MAX_FILENAMELEN) ;
                else 
                    return -1 ;
                break ;

            default:
                fprintf(stderr, "Unknown option: '%c'\n", c) ;
                return -1 ;
        }
    }

    if (optind < argc) {
        /* string comparison must be done before interpreting gid */
        if(strcmp(argv[optind], CMDSTR_ZERO) == 0) {
            /* Print version info and device type */
            return CMD_ZERO ;
        }
        else if(strcmp(argv[optind], CMDSTR_RESET) == 0) {
            return CMD_RESET ;
        }
        else {
            int ret ;
            char *cmdstr = argv[optind ++] ;
            char *p = strchr(cmdstr, '.');

            if (p != NULL) {
                tname[MAX_TNAMELEN] = '\0';
                strncpy(tname, p+1, MAX_TNAMELEN);
            }
            if( isdigit(cmdstr[0] )) {
                /* could be a decimal number or hex-decimal number ('0xNNNN') */
                if(cmdstr[0] == '0' && toupper(cmdstr[1]) == 'X')
                    sscanf(cmdstr, "%x", &gid) ;
                else gid = atol(cmdstr) ; /* decimal */

                ret = CMD_GROUPID ;
                if (p != NULL)
                    ret = CMD_GROUPID_PATH;
            }
            else {
                gname[MAX_GNAMELEN] = '\0' ;
                strncpy(gname, cmdstr, MAX_GNAMELEN) ;

                ret = CMD_GROUPNAME ;
                if (p != NULL) {
                    p = strchr(gname, '.');
                    if (p != NULL)
                        *p = '\0';
                    ret = CMD_GROUPNAME_PATH;
                }
            }

            /* get attribute name (if available) */
            aname[MAX_ATTRLEN] = '\0' ;
            if( (optind < argc) && argv[optind] )
                strncpy(aname, argv[optind], MAX_ATTRLEN) ;

            return ret ;
        }
    }
    else return -1 ;

    /* shouldn't reach here */
    return CMD_NONE ;
}


int main(int argc, char *argv[])
{
    CIPCFG *cipcfg=NULL;
    int ret = 0; /*res: result of called function, ret: value to be returned */
    int cmdval ;
    char value[MAX_VALUELEN + 1] ;
    CIPCFG *node = NULL;
    group_row_t *group;
    struct attribute *head = NULL;
    char path[MAX_TAILPATH];

    if(getenv(CIPCFG_PRINT_ERR) == NULL)
        setenv(CIPCFG_PRINT_ERR, "0", 1);

    cmdval = parse_opts(argc, argv, &my_options) ;
    if ( cmdval < 0 ) {
        usage(argv[0]) ;
        return 1;
    }
    
    if(CMD_RESET == cmdval){
        return reset_cipcfg();
    }

    if( !strcmp(my_options.cfg_file, "") ){
        cipcfg = init_cipcfg();
    }else{
        cipcfg = init_cipcfg_f(my_options.cfg_file);
    }
    if(!cipcfg){
        printf("Config File \"%s\" Not Found.\nPlease Confirm Again.\n\n", my_options.cfg_file);
        return CIPCFG_ERR_OPENFILE;
    }

    switch( cmdval ) {
        case CMD_GROUPNAME_PATH:
            if(aname[0]) {
                sprintf(path, "%s.%s", tname, aname);
                ret = get_cfg_item_by_str(cipcfg, gname, path, value);
                if(ret == CIPCFG_SUCCESS) 
                    printf("%s", value);
            }
            else ret = dump_cfg_by_group_name(cipcfg, gname, tname);
            break;
        case CMD_GROUPNAME:
            if(aname[0]) {/* attribute name is specified */
                ret = get_cfg_item_by_str(cipcfg, gname, aname, value);
                if(ret == 0)
                    printf("%s", value) ;
            }
            else ret = dump_cfg_by_group_name(cipcfg, gname, NULL);
            break ;

        case CMD_GROUPID_PATH:
            if(aname[0]) { /* attribute name is specified */
                sprintf(path, "%s.%s", tname, aname);
                ret = get_cfg_item(cipcfg, gid, path, value) ;
                if(ret == CIPCFG_SUCCESS)
                    printf("%s", value) ;
            }
            else ret = dump_cfg_by_gid(cipcfg, gid, tname);
            break;
        case CMD_GROUPID:
            if(aname[0]) { /* attribute name is specified */
                ret = get_cfg_item(cipcfg, gid, aname, value) ;
                if(ret == 0)
                    printf("%s", value) ;
            }
            else ret = dump_cfg_by_gid(cipcfg, gid, NULL);
            break ;

        case CMD_ZERO: /* print version info */
            print_attr(cipcfg->attr[0], -1, cipcfg->attr[1]); /* cipcfg version */
            print_attr(cipcfg->attr[2], -1, cipcfg->attr[3]); /* device type*/
            break ;

        default:
            fprintf(stderr, "Unrecogonized command\n") ;
    }

    close_cipcfg(cipcfg);

    return ret;
}

