#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <arpa/inet.h>
#include "cipcfg.h"
#include "wlan.h"
#include "syscall.h"

extern char* current_config;

static int lock_rf_channels(const int rf_idx, const char *ifname)
{
    CIPCFG *cipcfg = NULL;
    char group_name[MAX_VALUE_LEN], channel[MAX_VALUE_LEN], buf[MAX_VALUE_LEN];
    int ch = 0;

    sprintf(group_name, "rf%d", rf_idx + 1);

    cipcfg = init_cipcfg_f(current_config);

    /* Get channel */
    if (get_cfg_item_by_str(cipcfg, group_name, "channel", channel) 
	    != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return -1;
    }

    ch = atoi(channel);
    if (ch) {
	sprintf(buf, "iwconfig %s channel %d", ifname, ch);
	system(buf);
	sprintf(buf, "athchans -i %s %d", ifname, ch);
	system(buf);
    }

    close_cipcfg(cipcfg);
    return 0;
}

static int unlock_rf_channels(const int rf_idx, const char *ifname)
{
    CIPCFG *cipcfg = NULL;
    char group_name[MAX_VALUE_LEN], band[MAX_VALUE_LEN], buf[MAX_VALUE_LEN];
    int is_5GHz = 0;

    sprintf(group_name, "rf%d", rf_idx + 1);

    cipcfg = init_cipcfg_f(current_config);

    /* Get band */
    if (get_cfg_item_by_str(cipcfg, group_name, "mode", band) 
	    != CIPCFG_SUCCESS) {
	close_cipcfg(cipcfg);
	return -1;
    }

    is_5GHz = (strcmp(band, "11a") == 0 || strcmp(band, "11na") == 0) ? 1 : 0;
    sprintf(buf, "athchans -i %s %s", ifname, (is_5GHz ? "15-256" : "1-14"));
    system(buf);

    close_cipcfg(cipcfg);
    return 0;
}

/**
 * Call wdsds to discover vaild WDS peers.
 * It will put info of the vaild peer in AP_LIST
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery
 * @param ch        The current channel for this RF card
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_scan(int rfno, char *secret, int ch, int mode)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i, ret;
    char *q, *ptr;

    /* Notice!! When discovery WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
#if 0
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 
#endif

    sprintf(buf, "%s scan -s %s -o /tmp/wdsds%d -i %d -I ath%dsta0 -c %d -m %d 2> /dev/null",
        PATH_WDS_SLAVE, secret, rfno, rfno, rfno, ch, mode);
    ret = system(buf);

#if 0
    sprintf(buf, "%s start %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf);

    sprintf(buf, "%s restart 2> /dev/null", PATH_RC_BRIDGE );
    system(buf);
    sprintf(buf, "%s restart_wan 2> /dev/null", PATH_RC_NET );
    system(buf);    
#endif

    sprintf(buf, "/tmp/wdsds%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        strtok_r(buf, "\r\n", &ptr);
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        strncpy(ap->ssid, strtok_r(buf, " ", &ptr), sizeof(ap->ssid));
        if ( !strncmp(ap->ssid, buf, sizeof(ap->ssid) ) ){
            strncpy(ap->mac, (q = strtok_r(NULL, " ", &ptr)) ? q : "", sizeof(ap->mac));
        }else{
            strncpy(ap->mac, ap->ssid, sizeof(ap->mac));
            strcpy( ap->ssid, "");
        }

        ap->mac[17] = '\0';
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }
        ap->ch=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->rate=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->signal=atoi((q = strtok_r(NULL, ":", &ptr)) ? q : "");
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}

/**
 * Call wdsds to connect a vaild WDS peers.
 * It will return the connect result
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery,
 * @param mac       The MAC address of peer.
 * @param ch        The current channel for this RF card
 * @return	    0: success, -1: failure (please reference wdsds.c)
 */
int wdsd_connect(int rfno, char *secret, char *mac, int ch, int mode)
{
    /* TODO: This function still be unstable. Don't use this function. */

    char buf[MAX_VALUE_LEN];
    int ret;

    /* Notice!! When connect WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s add -a %s -s %s -i %d -c %d -m %d 2> /dev/null" ,
        PATH_WDS_SLAVE, mac, secret, rfno, ch, mode);
    ret=system(buf); //0: success, -1: failure

    sprintf(buf, "%s start %d 2> /dev/null", PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s restart  2> /dev/null", PATH_RC_BRIDGE );
    system(buf); 
    
    sprintf(buf, "%s restart_wan  2> /dev/null", PATH_RC_NET );
    system(buf); 
    return ret;
}

/**
 * Get the peer that connected to WDS discovery master
 * It will return AP_LIST
 *
 * @param rfno      The RF card number
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_connected(int rfno)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;

    sprintf(buf, "/tmp/wdsdm%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        /* The file format is "MAC: xx:xx....." and "RSSI: xx" */
        if (strstr(buf, "MAC")) { 
            strncpy(ap->mac, &buf[5], sizeof(ap->mac));
            ap->mac[17] = '\0';
        
            fgets(buf, sizeof(buf), fp);

            if (strstr(buf, "RSSI")) {
                ap->signal=atoi(&buf[6]);
            }
            else {
                cipcfg_print("Incorrectly file format, RSSI don't fellow MAC\n");
                free(ap);
                fclose(fp);
                return ap_list;
            }
            ap->next=NULL;
        }

        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}

static AP_LIST *get_scan_result(const char *ifname)
{
    AP_LIST *ap = NULL, *ap_list = NULL;
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    int i;
    int max_rate = 0, is_privacy = 0, is_wpa1 = 0, is_wpa2 = 0, is_psk = 0;

    sprintf(buf, "iwlist %s scanning", ifname);
    if ((fp = popen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't get scan result\n");
        return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
	char *pos = NULL;
	if ((pos = strstr(buf, "Address:")) != NULL) {
	    // Calculate the security type and rate for previous entry
	    if (ap) {
		ap->rate = max_rate;
		if (is_privacy) {
		    if (is_wpa1 && is_wpa2)
			strcpy(ap->security, 
				(is_psk ? "WPA(1/2)-PSK": "WPA/WPA2"));
		    else if (is_wpa1)
			strcpy(ap->security, 
				(is_psk ? "WPA-PSK": "WPA"));
		    else if (is_wpa2)
			strcpy(ap->security, 
				(is_psk ? "WPA2-PSK": "WPA2"));
		    else
			strcpy(ap->security, "WEP");
		} else {
		    strcpy(ap->security, "None");
		}
	    }
	    max_rate = is_privacy = is_wpa1 = is_wpa2 = is_psk = 0;
	    // Allocate a new entry
	    if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
		cipcfg_print("Error: memory allocate failure, "
			"can't allocate size = %d\n", sizeof(AP_LIST));
		pclose(fp);
		return NULL;
	    }
	    memset(ap, 0, sizeof(AP_LIST));
	    ap->next = ap_list;
	    ap_list = ap;
	    // Copy the MAC and skip the backspace
	    strncpy(ap->mac, pos + strlen("Address:") + 1, 17);
	} else if ((pos = strstr(buf, "ESSID:")) != NULL) {
	    // Copy the SSID and skip the beginning double quote
	    strncpy(ap->ssid, pos + strlen("ESSID:") + 1, 33);
	    // Remove tail double quote
	    for (i = 32; i >= 0; i--) {
		if (ap->ssid[i] == '"') {
		    ap->ssid[i] = '\0';
		    break;
		}
	    }
	} else if ((pos = strstr(buf, "Mode")) != NULL) {
	    // Check the type
	    if (strncmp(pos + strlen("Mode") + 1, "Master", 6) == 0)
		strcpy(ap->type, "AP");
	    else if (strncmp(pos + strlen("Mode") + 1, "Ad-Hoc", 6) == 0)
		strcpy(ap->type, "Ad-Hoc");
	} else if ((pos = strstr(buf, "Channel")) != NULL) {
	    // Copy the channel number
	    ap->ch = atoi(pos + strlen("Channel") + 1);
	} else if ((pos = strstr(buf, "Quality")) != NULL) {
	    // Copy the signal
	    ap->signal = atoi(pos + strlen("Quality") + 1);
	} else if ((pos = strstr(buf, "Encryption key")) != NULL) {
	    // mark the privacy flag
	    is_privacy = 
		(strncmp(pos + strlen("Encryption key") + 1, "on", 2) == 0);
	} else if ((pos = strstr(buf, "Mb/s")) != NULL) {
	    // find the max rate
	    char *tok = NULL, *tok_pos = NULL;
	    int rate;
	    for (tok = strtok_r(buf, ":;", &tok_pos); tok; 
		    tok = strtok_r(NULL, ":;", &tok_pos)) {
		if (sscanf(tok, "%d Mb/s", &rate) && rate > max_rate)
		    max_rate = rate;
	    }
	} else if ((pos = strstr(buf, "WPA Version 1")) != NULL) {
	    // mark the WPA1
	    is_wpa1 = 1;
	} else if ((pos = strstr(buf, "IEEE 802.11i")) != NULL) {
	    // mark the WPA2
	    is_wpa2 = 1;
	} else if ((pos = strstr(buf, "Authentication Suites")) != NULL) {
	    // mark the PSK
	    is_psk = (strstr(pos, "PSK") ? 1 : 0);
	}
    }
    // for last entry
    if (ap) {
	ap->rate = max_rate;
	if (is_privacy) {
	    if (is_wpa1 && is_wpa2)
		strcpy(ap->security, 
			(is_psk ? "WPA(1/2)-PSK": "WPA/WPA2"));
	    else if (is_wpa1)
		strcpy(ap->security, 
			(is_psk ? "WPA-PSK": "WPA"));
	    else if (is_wpa2)
		strcpy(ap->security, 
			(is_psk ? "WPA2-PSK": "WPA2"));
	    else
		strcpy(ap->security, "WEP");
	} else {
	    strcpy(ap->security, "None");
	}
    }
    pclose(fp);
    return ap_list;
}

/**
 * Call rfscan to scanning all APs.
 * It will put info of the APs in AP_LIST
 *
 * @param rfno      The RF card number
 * @param is_scan   1: restart scan now, 0: only retrive the file
 * @return	    pointer to ap list, NULL if not found
 */   
AP_LIST *rf_scan(int rfno, int is_scan, char t[MAX_VALUE_LEN])
{
    char buf[MAX_VALUE_LEN], *sec;
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    struct stat file_stat ;
    int i;
    char *q, *ptr;
    char ifname[]="ath0sta0";

    unlock_rf_channels(0, ifname);
#ifdef LSDK_9_1_0_214
    ap_list = get_scan_result(ifname);
#else
    sprintf(buf, "iwlist %s scanning >/dev/null 2>&1",ifname);
    system(buf);
    sprintf(buf, "wlanconfig %s list ap",ifname);
    if ((fp=popen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    stat(buf, &file_stat);
    struct tm *p;
    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        if (strlen(buf) < 33) continue;
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            pclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

	strncpy(ap->ssid, buf, 33);
	for (i = 32; i >= 0 && isspace(ap->ssid[i]); i--)
	    ap->ssid[i] = '\0';

	strncpy(ap->mac, (q = strtok_r(buf+33, " ", &ptr)) ? q : "", sizeof(ap->mac));
	/*
        strncpy(ap->ssid, strtok_r(buf, " ", &ptr), sizeof(ap->ssid));
        if ( !strncmp(ap->ssid, buf, sizeof(ap->ssid) ) ){
            strncpy(ap->mac, strtok_r(NULL, " ", &ptr), sizeof(ap->mac));
        }else{
            strncpy(ap->mac, ap->ssid, sizeof(ap->mac));
            strcpy( ap->ssid, "");
        }

        ap->mac[17] = '\0';
	*/
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }
        ap->ch=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->rate=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->signal=atoi((q = strtok_r(NULL, ":", &ptr)) ? q : "");
        
        sec=(q = strtok_r(NULL, "\n", &ptr)) ? q : "";
        if (strstr(sec, "EP")) {
	    sprintf(ap->type, "AP");
	    if (strstr(sec, "WPA") && strstr(sec, "RSN")) {
		if (strstr(sec, "PSK"))
		    sprintf(ap->security, "WPA(1/2)-PSK"); //WPA
		else
		    sprintf(ap->security, "WPA/WPA2"); //WPA
	    }
            else if (strstr(sec, "WPA")) {
		if (strstr(sec, "PSK"))
		    sprintf(ap->security, "WPA-PSK"); //WPA
		else
		    sprintf(ap->security, "WPA"); //WPA
            }
            else if (strstr(sec, "RSN")) {
		if (strstr(sec, "PSK"))
		    sprintf(ap->security, "WPA2-PSK"); //WPA2
		else
		    sprintf(ap->security, "WPA2"); //WPA2
            }
            else {
                sprintf(ap->security, "WEP"); //WEP
            }
        }
	else if (strstr(sec, "I")) {
            sprintf(ap->type, "Ad-hoc");
            sprintf(ap->security, "None");
	} else {
            sprintf(ap->type, "NA");
            sprintf(ap->security, "None");
        }
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    pclose(fp);
#endif
    lock_rf_channels(0, ifname);
    return ap_list;
}

/** 
 * @brief Get WDS Link current attributes (not configuration attributes).
 * 
 * @param rf_idx 	RF index.
 * @param link_idx 	Link index.
 * @param attr		Attribute type. 
 * @param value 	Attribute value.
 * 
 * @return 		1 for success otherwise 0. 
 */
int getWDSLinkAttr(int rf_idx, int link_idx, int attr, char *value)
{
    char group_name[MAX_VALUE_LEN], ret[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    char buf[128], iface[16];
    FILE *fp;
    int retval;

    sprintf(group_name, "rf%d_wds%d", rf_idx + 1, link_idx + 1);

    /* Get interface name by index */
    if (get_wds_ifname_by_idx(rf_idx, link_idx, iface) == NULL)
	return 0;

    switch (attr) {
	case WDSLINK_STATE:
	    cipcfg = init_cipcfg_f(current_config);
	    retval = get_cfg_item_by_str(cipcfg, group_name, "state", ret);
	    close_cipcfg(cipcfg);
	    if (retval != CIPCFG_SUCCESS) {
		return 0;
	    } 

	    if (strcmp(ret, "disable") == 0) {
		sprintf(value, "Disabled");
		return 1;
	    }

	    /* Check if WDS link is disabled */
	    if (!isWDSLinkAlive(iface)) {
	        sprintf(value, "Disconnected");
	        return 1;
	    }

	    /* Check the STP status of WDS link via sysfs */
	    sprintf(buf, "/sys/class/net/%s/brport/state", iface);
	    if ((fp = fopen(buf, "r")) != NULL) {
		int state;
		fgets(ret, MAX_VALUE_LEN, fp);
		fclose(fp);
		state = atoi(ret);
		switch (state) {
		    case 0: sprintf(value, "Disabled"); break;	// disabled
		    case 1:					// listening
		    case 2:					// learning
		    case 3: sprintf(value, "Active"); break;	// forwarding
		    case 4: sprintf(value, "Backup"); break;	// blocking
		}
		return 1;
	    }

	    /* Check the STP status of WDS link via brctl */
	    sprintf(buf, "brctl showstat br0 %s > /tmp/wdslink", iface);
	    cip_system(buf);
	    if ((fp = fopen("/tmp/wdslink", "r")) != NULL) {
		fgets(ret, MAX_VALUE_LEN, fp);
		fclose(fp);
	    }

	    if (strncmp(ret, "listening", 9) == 0 || strncmp(ret, "learning", 8) == 0
		    || strncmp(ret, "forwarding", 10) == 0) {
		sprintf(value, "Active");
		return 1;
	    } else if (strncmp(ret, "disabled", 8) == 0) {
		sprintf(value, "Disabled");
		return 1;
	    } else { // STP status is "blocking".
		sprintf(value, "Backup");
		return 1;
	    }
	    break;
	case WDSLINK_RATE:
	    /* Get WDS Link TX Rate */
	    if (getWDSLinkTxRate(iface, ret)) {
		sprintf(value, "%s M", ret);
		return 1;
	    }
	    break;
	case WDSLINK_QUALITY:
	    /* Get WDS Link Quality */
	    if (getWDSLinkQuality(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_HEALTH:
	    /* Get WDS Link Health */
	    if (getWDSLinkHealth(iface, ret)) {
		sprintf(value, "%s &#37;", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXTOTAL:
	    /* Get WDS Link Tx total frames */
	    if (getWDSLinkTxTotal(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXFAIL:
	    /* Get WDS Link Tx fail frames */
	    if (getWDSLinkTxFail(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
    }
    return 0;
}
#ifdef FUSION_GET_STA_LIST 
/** 
 * @brief Call ath_get_sta_info at ath_wlan_sta_info.c to get station info 
 * 
 * @param ifname        Interface name.
 * 
 * @return              Return the list of station.
 */
ASSOC_STA_LIST * get_sta_list(char * ifname)
{
    return ath_get_sta_info(ifname);
}

#else
/** 
 * @brief Get associated station list from proc file of specific interface.
 * 
 * @param ifname	Interface name.
 * 
 * @return 		Return the list of station.
 */
ASSOC_STA_LIST * get_sta_list(char * ifname)
{
    ASSOC_STA_LIST *head = NULL;
    ASSOC_STA_LIST *node = NULL;
    char buf[MAX_VALUE_LEN];
    FILE *fp;
    char *pos;

    sprintf(buf, "/proc/net/madwifi/%s/associated_sta", ifname);

    fp = fopen(buf, "r");
    if (fp == NULL)
	return NULL;

    while (!feof(fp)) {
	fgets(buf, sizeof(buf), fp);
	if ((pos = strstr(buf, "macaddr"))) {
	    /* Create a new node */
	    node = (ASSOC_STA_LIST *)malloc(sizeof(ASSOC_STA_LIST));
	    if (!node) {
		cipcfg_print("Can't allocate memory for ASSOC_STA_LIST\n");
		goto fail;
	    } else {
		bzero(node, sizeof(ASSOC_STA_LIST));
		node->next = head;
		head = node;
	    }
	    strncpy(node->mac, &pos[10], 17);
	} else if ((pos = strstr(buf, "rssi"))) {
	    if (!node) goto fail;
	    node->rssi = atoi(&pos[5]);
	} else if ((pos = strstr(buf, "tx_rate"))) {
	    if (!node) goto fail;
	    node->tx_rate = atof(&pos[8]);
	} else if ((pos = strstr(buf, "tx_okcnt"))) {
	    if (!node) goto fail;
	    node->tx_okcnt = atoi(&pos[9]);
	} else if ((pos = strstr(buf, "tx_errcnt"))) {
	    if (!node) goto fail;
	    node->tx_errcnt = atoi(&pos[10]);
	} else if ((pos = strstr(buf, "last_rx"))) {
	    if (!node) goto fail;
	    node->last_rx = atof(&pos[8]);
	}
    }
    fclose(fp);
    return head;

fail:
    fclose(fp);
    return NULL;
}

#endif

/** 
 * @brief Free the associated station list.
 * 
 * @param head 	The head of the station list.
 */
void free_sta_list(ASSOC_STA_LIST *head)
{
    ASSOC_STA_LIST *cur;

    while (head) {
	cur = head;
	head = head->next;
	free(cur);
    }
}

/** 
 * @brief Count the sta list.
 * 
 * @param head 	The head of station list.
 * 
 * @return 	The count of stations.
 */
int sta_list_count(ASSOC_STA_LIST *head)
{
    int count = 0;
    ASSOC_STA_LIST *node = NULL;

    foreach_sta_get(node, head) {
	count++;
    }

    return count;
}


ASSOC_WDS_LIST * get_wds_list(char * ifname)
{
    return (ASSOC_WDS_LIST *)get_sta_list(ifname);
}

void free_wds_list(ASSOC_WDS_LIST *head)
{
    free_sta_list((ASSOC_STA_LIST *)head);
}

int wds_list_count(ASSOC_WDS_LIST *head)
{
    return sta_list_count((ASSOC_STA_LIST *)head);
}

/*
 * convert string to int array for channel use
 */
void convert_ch(char *str, int *ch)
{
    int i;
    char *delim = " ";
    char *p;
    char *q, *ptr;

    p = str;
    ch[0] = atoi(strtok_r(p, delim, &ptr));
    for( i=1; (q=strtok_r(NULL, delim, &ptr)); i++){
        ch[i] = atoi(q);
    }
    ch[i] = -1; // terminator
}


/*
 *  get allowed channels under a country code.
 */
int get_allowed_ch(COUNTRY_CHANNEL *ccObj)
{
    int ret, ct;
    char band[MAX_VALUE_LEN], code[MAX_VALUE_LEN], *tmp, buf[256];
    FILE *fp = NULL;
    CIPCFG *cipcfg = NULL;
    char *ptr;

    memset(buf, 0, 256);
    cipcfg=init_cipcfg();
    if(!cipcfg){
        cipcfg_print("initial cipcfg error.\n");
        return -1;
    }else{
        ret = get_cfg_item(cipcfg, GID_RF1, "country", code);
        close_cipcfg(cipcfg);
        if (ret != CIPCFG_SUCCESS) {
            cipcfg_print("Can NOT get country code, return value is %d.\n", ret);
            sprintf(code, "NA" ); // if can not get it, give it a default value.
            return -1;
        }
    }

    sprintf(buf, "eval \"`cat %s`\"; echo $%s", COUNTRY_CODE, code);
    if ((fp = popen( buf, "r")) == NULL) {
        cipcfg_print("Can't open the country code file\n");
        return -1;
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        ret = atoi(buf); // get the country code
    }
    pclose(fp);

    if ((fp = fopen( ALLOWED_CHANNEL, "r")) == NULL) {
        cipcfg_print("Can't open the channel list!!\n");
        return -1;
    }else{
        while (fgets(buf, sizeof(buf), fp) != NULL) {
	    char *ptr2;
            sscanf(buf, "c%u_11%s='", &ct, band);
	    strtok_r(band, "=", &ptr2);
            strtok_r(buf, "'", &ptr);
            tmp = strtok_r( NULL, "'", &ptr);
            if( ct == (unsigned int)ret ){ // if match coubtry code
                break;
            }
        }
        ccObj->cc = ret;
        ccObj->ch_11b[0] = -1;
        ccObj->ch_11g[0] = -1;
        ccObj->ch_11a[0] = -1;
        ccObj->ch_11ng[0] = -1;
        ccObj->ch_11ngu[0] = -1;
        ccObj->ch_11ngl[0] = -1;
        ccObj->ch_11na[0] = -1;
        ccObj->ch_11nau[0] = -1;
        ccObj->ch_11nal[0] = -1;

        while((ct==(unsigned int)ret)&&(NULL!=tmp)){
	    if (!strcmp(band, "b"))
                convert_ch(tmp, ccObj->ch_11b);
	    if (!strcmp(band, "g")) {
                convert_ch(tmp, ccObj->ch_11g);
            }else if (!strcmp(band, "a")){
                convert_ch(tmp, ccObj->ch_11a);
	    }else if (!strcmp(band, "ng")){
                convert_ch(tmp, ccObj->ch_11ng);
            }else if (!strcmp(band, "ngu")){
                convert_ch(tmp, ccObj->ch_11ngu);
            }else if (!strcmp(band, "ngl")){
                convert_ch(tmp, ccObj->ch_11ngl);
            }else if (!strcmp(band, "na")){
                convert_ch(tmp, ccObj->ch_11na);
            }else if (!strcmp(band, "nau")){
                convert_ch(tmp, ccObj->ch_11nau);
            }else if (!strcmp(band, "nal")){
                convert_ch(tmp, ccObj->ch_11nal);
            }else{
                cipcfg_print("unknown channel group : %c\n", band);
                // return -1; 
            }
            if (fgets(buf, sizeof(buf), fp) != NULL) {
		char *ptr2;
                sscanf(buf, "c%u_11%s='", &ct, band);
		strtok_r(band, "=", &ptr2);
                strtok_r(buf, "'", &ptr);
                tmp = strtok_r( NULL, "'", &ptr);
            }else{
                break;
            }
        }
        fclose(fp);
    }
    return 0;
}

TXPWR_T *get_txpwr_level(void)
{
    static TXPWR_T txpwr[] = {
	{"Auto",	0},
	{"Highest", 	17},
	{"High", 	13},
	{"Medium", 	9},
	{"Low", 	5},
	{"Lowest",	1},
	{NULL,		0}
    };

    return txpwr;
}

/* for AP management start */
int set_txpwr(char *txpwr)
{
    char cmd[BUFSIZ];

    snprintf(cmd,BUFSIZ,"/etc/rc.d/rc.wlan setup_txpow 0 %s",txpwr);
    cip_system(cmd);
    return 0;
}

AP_LIST *ap_scan(void)
{
#if defined(AP83FUS) || defined(AP93FUS)
#define GET_HOSTROAMING	"get_roaming"
#define HOSTROAMING	"roaming"
#else
#define GET_HOSTROAMING	"get_hostroaming"
#define HOSTROAMING	"hostroaming"
#endif
    char ifname[]="ath0sta0";
    char wlandev[]="wifi0";
    char cmd[BUFSIZ],value[BUFSIZ],value2[BUFSIZ],*buf, val[MAX_VALUE_LEN];
    int i, retval;
    AP_LIST *ap_list=NULL, *ap=NULL;
    FILE *fd=NULL;
    char del[]=" \t";
    int hostroaming;
    CIPCFG *cipcfg = NULL;
    value[0] = 0;
    /* Check if universal repeater interface exist */
    memset(val, 0, MAX_VALUE_LEN);
    cipcfg = init_cipcfg_f(current_config);
    retval = get_cfg_item_by_str(cipcfg, "rf1", "roles", val);
    close_cipcfg(cipcfg);
    if (strlen(val) && strstr(val, "rep")) {
	AP_LIST *node;
	char t[MAX_VALUE_LEN];
	ap_list = rf_scan(0, 1, t);

	foreach_ap_get(node, ap_list) {
	    if (strstr(node->security, "WPA"))
		sprintf(node->security, "WPA");
	    else if (strstr(node->security, "WEP"))
		sprintf(node->security, "WEP");
	    else
		sprintf(node->security, "None");
	}
	return ap_list;
    }

#ifdef LSDK_9_1_0_214
    snprintf(cmd,BUFSIZ,"ifconfig | grep ath.ap | awk '{print $1}' ");
    fd = popen(cmd, "r");
    fgets(value, BUFSIZ, fd);
    if(value[strlen(value) - 1] == '\n') 
	value[strlen(value) - 1] = '\0';
    
    pclose(fd);
    if(strlen(value) < 4){
	sprintf(value, "ath0ap0");
	snprintf(cmd,BUFSIZ,
	"wlanconfig %s create wlandev %s wlanmode ap > /dev/null 2>&1",value,wlandev);
	cip_system(cmd);

	// enable station
        snprintf(cmd,BUFSIZ,"ifconfig %s up > /dev/null 2>&1",value);
	cip_system(cmd);
	ap_list = get_scan_result(value);

        snprintf(cmd,BUFSIZ,"ifconfig %s down > /dev/null 2>&1",value);
	cip_system(cmd);

	// destroy interface
        snprintf(cmd,BUFSIZ,"wlanconfig %s destroy > /dev/null 2>&1",value);
	cip_system(cmd);
    }else
	ap_list = get_scan_result(value);
#else
    // destroy interface - to make sure interface does not exist
    snprintf(cmd,BUFSIZ,"wlanconfig %s destroy > /dev/null 2>&1",ifname);
    cip_system(cmd);

    // create station
    snprintf(cmd,BUFSIZ,
      "wlanconfig %s create wlandev %s wlanmode sta nosbeacon > /dev/null 2>&1",ifname,wlandev);
    cip_system(cmd);

    // get hostroaming value -- in order to set back
    snprintf(cmd,BUFSIZ,"iwpriv %s " GET_HOSTROAMING, ifname);
    fd = popen(cmd, "r");
    fread(value,sizeof(char),BUFSIZ, fd);
    pclose(fd);
    buf = strstr(value, GET_HOSTROAMING ":");
    if(!buf) goto exit;
    buf = buf + strlen(GET_HOSTROAMING ":");
    hostroaming = buf[0] - '0';
    if( hostroaming != 0 && hostroaming != 1 && hostroaming != 2) goto exit;

    /* close hostroaming 
    Argument  Roaming Mode  Description
    ---------------------------------------------------
    0         Device        Driver/hardware control  
    1         Auto          802.11 layer control  
    2         Manual        ioctl/application control

    0 and 1 will lead to channel jumping
    */
    snprintf(cmd,BUFSIZ,"iwpriv %s " HOSTROAMING " 2 > /dev/null 2>&1",ifname);
    cip_system(cmd);

    // enable station
    snprintf(cmd,BUFSIZ,"ifconfig %s up > /dev/null 2>&1",ifname);
    cip_system(cmd);

    unlock_rf_channels(0, ifname);
    // do scan -- After doing iwlist scanning, the wlanconfig list scan result will be refresh.
    snprintf(cmd,BUFSIZ,"iwlist %s scanning > /dev/null 2>&1",ifname);
    cip_system(cmd);
    usleep(1500*1000); // wait 1.5sec for scanning stop

    // list scan
    snprintf(cmd,BUFSIZ,"wlanconfig %s list scan",ifname);
    fd = popen(cmd, "r");
    fgets(value, BUFSIZ, fd);// delete the title line
    
    // retrieve information
    while(fgets(value, BUFSIZ, fd))
    {
        if(ap_list){
            ap->next = (AP_LIST *)malloc(sizeof(AP_LIST));
            ap = ap->next;
        }else{
            ap = ap_list = (AP_LIST *)malloc(sizeof(AP_LIST));
        }
        memset(ap, 0, sizeof(AP_LIST));
        // essid
        strncpy(ap->ssid,value,sizeof(ap->ssid)-1);
        for(i=sizeof(ap->ssid)-2; i>=0 && isspace(ap->ssid[i]); i--)
            ap->ssid[i]='\0';
        // bssid
        strncpy(ap->mac,strtok(value+34,del),sizeof(ap->mac)-1);
        for(i=0; i<strlen(ap->mac); i++)
            ap->mac[i]=toupper(ap->mac[i]);
        // channel
        ap->ch = atoi(strtok(NULL,del));
        // rate
        strcpy(value2,strtok(NULL,del));
        if(strlen(value2)>0 && value2[strlen(value2)-1]=='M')
            value2[strlen(value2)-1]='\0';
        ap->rate = atoi(value2);
        // signal
        strcpy(value2,strtok(NULL,del));
        if(strlen(value2)>0){
            for(i=strlen(value2)-1; i>=0 && value2[i]!=':';i--)
                value2[i]='\0';
            if(i>=0) value2[i]='\0';
        }
        ap->signal=atoi(value2);
        
        // skip INT (interval)
        strtok(NULL,del);
        // type
        strcpy(ap->type,"NA");
        strcpy(value2,strtok(NULL,del));
        if(strlen(value2)>0){
            for(i=strlen(value2)-1; i>=0 && isspace(value2[i]); i--)
                value2[i]='\0';
            if(value2[0]=='I') strcpy(ap->type,"Ad-hoc");
            else if(value2[0]=='E') strcpy(ap->type,"AP");
        }
        // enc
        if(strstr(value+79,"WPA") || strstr(value+79,"RSN"))
            strcpy(ap->security,"WPA");
        else if(strstr(value2,"P"))
            strcpy(ap->security,"WEP");
        else
            strcpy(ap->security,"None");
    }
    pclose(fd);
    lock_rf_channels(0, ifname);

    // disable interface
    snprintf(cmd,BUFSIZ,"ifconfig %s down > /dev/null 2>&1",ifname);
    cip_system(cmd);

    // set hostroaming to original value
    snprintf(cmd,BUFSIZ,"iwpriv %s " HOSTROAMING " %d > /dev/null 2>&1",ifname,hostroaming);
    cip_system(cmd);
exit:
    // destroy interface
    snprintf(cmd,BUFSIZ,"wlanconfig %s destroy > /dev/null 2>&1",ifname);
    cip_system(cmd);
#endif

    return ap_list;
#undef GET_HOSTROAMING
#undef HOSTROAMING
}

int set_channel_switch(int channel, int count)
{
#ifdef LSDK_9_1_0_214
    char cmd[BUFSIZ],value[BUFSIZ];
    FILE *fd=NULL;
    value[0]=0;
    snprintf(cmd,BUFSIZ,"ifconfig | grep ath.ap | awk '{print $1}' ");
    fd = popen(cmd, "r");
    fgets(value, BUFSIZ, fd);
    if(value[strlen(value) - 1] == '\n') 
	value[strlen(value) - 1] = '\0';
    pclose(fd);
    
    if(strlen(value) > 4){
	snprintf(cmd,BUFSIZ,"iwpriv %s doth_chanswitch %d %d",value,channel,count);
	cip_system(cmd);
	return channel;
    }
    return 0;
#else
    return 0;
#endif
}

/* for AP management end */
