#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/ioctl.h>
//#include <net/if.h>
#include <arpa/inet.h>
#include "cipcfg.h"
#include "wlan.h"
#include "syscall.h"
#include <linux/wireless.h>
#define SCAN_LOCK   "/var/lock/cipcfg_scan_lock"

extern char* current_config;

/**
 * Call wdsds to discover vaild WDS peers.
 * It will put info of the vaild peer in AP_LIST
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery
 * @param ch        The current channel for this RF card
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_scan(int rfno, char *secret, int ch, int mode)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i, ret;
    char *q, *ptr;

    /* Notice!! When discovery WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
#if 0
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 
#endif

    sprintf(buf, "%s scan -s %s -o /tmp/wdsds%d -i %d -I ath%dsta0 -c %d -m %d 2> /dev/null",
        PATH_WDS_SLAVE, secret, rfno, rfno, rfno, ch, mode);
    ret = system(buf);

#if 0
    sprintf(buf, "%s start %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf);

    sprintf(buf, "%s restart 2> /dev/null", PATH_RC_BRIDGE );
    system(buf);
    sprintf(buf, "%s restart_wan 2> /dev/null", PATH_RC_NET );
    system(buf);    
#endif

    sprintf(buf, "/tmp/wdsds%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        strtok_r(buf, "\r\n", &ptr);
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        strncpy(ap->ssid, strtok_r(buf, " ", &ptr), sizeof(ap->ssid));
        if ( !strncmp(ap->ssid, buf, sizeof(ap->ssid) ) ){
            strncpy(ap->mac, (q = strtok_r(NULL, " ", &ptr)) ? q : "", sizeof(ap->mac));
        }else{
            strncpy(ap->mac, ap->ssid, sizeof(ap->mac));
            strcpy( ap->ssid, "");
        }

        ap->mac[17] = '\0';
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }
        ap->ch=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->rate=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        ap->signal=atoi((q = strtok_r(NULL, ":", &ptr)) ? q : "");
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}

/**
 * Call wdsds to connect a vaild WDS peers.
 * It will return the connect result
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery,
 * @param mac       The MAC address of peer.
 * @param ch        The current channel for this RF card
 * @return	    0: success, -1: failure (please reference wdsds.c)
 */
int wdsd_connect(int rfno, char *secret, char *mac, int ch, int mode)
{
    /* TODO: This function still be unstable. Don't use this function. */

    char buf[MAX_VALUE_LEN];
    int ret;

    /* Notice!! When connect WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s add -a %s -s %s -i %d -c %d -m %d 2> /dev/null" ,
        PATH_WDS_SLAVE, mac, secret, rfno, ch, mode);
    ret=system(buf); //0: success, -1: failure

    sprintf(buf, "%s start %d 2> /dev/null", PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s restart  2> /dev/null", PATH_RC_BRIDGE );
    system(buf); 
    
    sprintf(buf, "%s restart_wan  2> /dev/null", PATH_RC_NET );
    system(buf); 
    return ret;
}

/**
 * Get the peer that connected to WDS discovery master
 * It will return AP_LIST
 *
 * @param rfno      The RF card number
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_connected(int rfno)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;

    sprintf(buf, "/tmp/wdsdm%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        /* The file format is "MAC: xx:xx....." and "RSSI: xx" */
        if (strstr(buf, "MAC")) { 
            strncpy(ap->mac, &buf[5], sizeof(ap->mac));
            ap->mac[17] = '\0';
        
            fgets(buf, sizeof(buf), fp);

            if (strstr(buf, "RSSI")) {
                ap->signal=atoi(&buf[6]);
            }
            else {
                cipcfg_print("Incorrectly file format, RSSI don't fellow MAC\n");
                free(ap);
                fclose(fp);
                return ap_list;
            }
            ap->next=NULL;
        }

        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}


/**
 * Call rfscan to scanning all APs.
 * It will put info of the APs in AP_LIST
 *
 * @param rfno      The RF card number
 * @param is_scan   1: restart scan now, 0: only retrive the file
 * @return	    pointer to ap list, NULL if not found
 */   
AP_LIST *rf_scan(int rfno, int is_scan, char t[MAX_VALUE_LEN])
{
    char buf[MAX_VALUE_LEN], *sec;
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    struct stat file_stat ;
    int i, fd;
    char *q, *ptr, tmp[4];
    
    fd = open(SCAN_LOCK, O_RDWR | O_CREAT);
    if (fd < 0) {
        cipcfg_print("Error: can't open LOCK file\n");
	return NULL;
    }
    // Lock the file
    if(flock(fd, LOCK_EX) != 0) {
	close(fd);
        cipcfg_print("Error: can't set LOCK\n");
	return NULL;
    }
    if ((fp = popen("iwpriv ra0 set SiteSurvey=1", "r")) == NULL) {
        cipcfg_print("Error: can't set SiteSurvey\n");
	goto Fail;
    }
    pclose(fp);
    sleep(4);
    if ((fp = popen("iwpriv ra0 get_site_survey", "r")) == NULL) {
        cipcfg_print("Error: can't get SiteSurvey result\n");
	goto Fail;
    }
    // skip the first line
    fgets(buf, sizeof(buf), fp);
    // skip the second line
    fgets(buf, sizeof(buf), fp);
    // site survey format:
    // channel	ssid	bssid	security    signal  mode    NT
    while (fgets(buf, sizeof(buf), fp)) {
	AP_LIST ent;

	ap = &ent;
        memset(ap, 0, sizeof(AP_LIST));

	q = strtok_r(buf, " ", &ptr);
	ap->ch = atoi(buf);
	if ((q = strtok_r(NULL, " ", &ptr)) == NULL)
	    continue;
	strncpy(ap->ssid, q, sizeof(ap->ssid));
	for (i = 32; i >= 0 && isspace(ap->ssid[i]); i--)
	    ap->ssid[i] = '\0';

	strncpy(ap->mac, (q = strtok_r(NULL, " ", &ptr)) ? q : "", sizeof(ap->mac));
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i] = toupper(ap->mac[i]);
        }
        //ap->rate=atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        strncpy(ap->security, (q = strtok_r(NULL, " ", &ptr)) ? q : "", sizeof(ap->security));
        ap->signal = atoi((q = strtok_r(NULL, " ", &ptr)) ? q : "");
        
	if ((q = strtok_r(NULL, "\n", &ptr)) != NULL) {
	    if (strstr(q, " In") != NULL)
		strcpy(ap->type,"AP");
	    else if (strstr(q, " Ad") != NULL)
		strcpy(ap->type,"Ad-hoc");
	}
        ap->next=NULL;
        
	if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
	    cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
		sizeof(AP_LIST));
	    pclose(fp);
	    goto Fail;
	}
	memcpy(ap, &ent, sizeof(AP_LIST));

        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    pclose(fp);
Fail:
    flock(fd, LOCK_UN);
    close(fd);
    return ap_list;
}

/** 
 * @brief Get WDS Link current attributes (not configuration attributes).
 * 
 * @param rf_idx 	RF index.
 * @param link_idx 	Link index.
 * @param attr		Attribute type. 
 * @param value 	Attribute value.
 * 
 * @return 		1 for success otherwise 0. 
 */
int getWDSLinkAttr(int rf_idx, int link_idx, int attr, char *value)
{
    char group_name[MAX_VALUE_LEN], ret[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    char buf[128], iface[16];
    FILE *fp;
    int retval;

    sprintf(group_name, "rf%d_wds%d", rf_idx + 1, link_idx + 1);

    /* Get interface name by index */
    if (get_wds_ifname_by_idx(rf_idx, link_idx, iface) == NULL)
	return 0;

    switch (attr) {
	case WDSLINK_STATE:
	    cipcfg = init_cipcfg_f(current_config);
	    retval = get_cfg_item_by_str(cipcfg, group_name, "state", ret);
	    close_cipcfg(cipcfg);
	    if (retval != CIPCFG_SUCCESS) {
		return 0;
	    } 

	    if (strcmp(ret, "disable") == 0) {
		sprintf(value, "Disabled");
		return 1;
	    }

	    /* Check if WDS link is disabled */
	    if (!isWDSLinkAlive(iface)) {
	        sprintf(value, "Disconnected");
	        return 1;
	    }

	    /* Check the STP status of WDS link via sysfs */
	    sprintf(buf, "/sys/class/net/%s/brport/state", iface);
	    if ((fp = fopen(buf, "r")) != NULL) {
		int state;
		fgets(ret, MAX_VALUE_LEN, fp);
		fclose(fp);
		state = atoi(ret);
		switch (state) {
		    case 0: sprintf(value, "Disabled"); break;	// disabled
		    case 1:					// listening
		    case 2:					// learning
		    case 3: sprintf(value, "Active"); break;	// forwarding
		    case 4: sprintf(value, "Backup"); break;	// blocking
		}
		return 1;
	    }

	    /* Check the STP status of WDS link via brctl */
	    sprintf(buf, "brctl showstat br0 %s > /tmp/wdslink", iface);
	    cip_system(buf);
	    if ((fp = fopen("/tmp/wdslink", "r")) != NULL) {
		fgets(ret, MAX_VALUE_LEN, fp);
		fclose(fp);
	    }

	    if (strncmp(ret, "listening", 9) == 0 || strncmp(ret, "learning", 8) == 0
		    || strncmp(ret, "forwarding", 10) == 0) {
		sprintf(value, "Active");
		return 1;
	    } else if (strncmp(ret, "disabled", 8) == 0) {
		sprintf(value, "Disabled");
		return 1;
	    } else { // STP status is "blocking".
		sprintf(value, "Backup");
		return 1;
	    }
	    break;
	case WDSLINK_RATE:
	    /* Get WDS Link TX Rate */
	    if (getWDSLinkTxRate(iface, ret)) {
		sprintf(value, "%s M", ret);
		return 1;
	    }
	    break;
	case WDSLINK_QUALITY:
	    /* Get WDS Link Quality */
	    if (getWDSLinkQuality(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_HEALTH:
	    /* Get WDS Link Health */
	    if (getWDSLinkHealth(iface, ret)) {
		sprintf(value, "%s &#37;", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXTOTAL:
	    /* Get WDS Link Tx total frames */
	    if (getWDSLinkTxTotal(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXFAIL:
	    /* Get WDS Link Tx fail frames */
	    if (getWDSLinkTxFail(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
    }
    return 0;
}

/** 
 * @brief Get associated station list from proc file of specific interface.
 * 
 * @param ifname	Interface name.
 * 
 * @return 		Return the list of station.
 */
ASSOC_STA_LIST * get_sta_list(char * ifname)
{
    ASSOC_STA_LIST *head = NULL;
    ASSOC_STA_LIST *node = NULL;
    char buf[MAX_VALUE_LEN], cmd[MAX_VALUE_LEN];
    FILE *fp = NULL;
    char *ptr, *pos;
    char tmp[MAX_VALUE_LEN];
    int skip, cnt, iter;
    int match_if_flag=0,idx=-1;

    buf[0]=tmp[0]=0;
    if (!strncmp(ifname, "wds", 3)) {
	    sprintf(cmd, "iwpriv %s showwds", ifname);
	    idx=atoi(&ifname[3]);
    }
    else if(!strcmp(ifname,"apcli0")){
	    sprintf(cmd, "iwpriv %s showrep", ifname);
	    idx=atoi(&ifname[5]);
    }
    else {
	    sprintf(cmd, "iwpriv %s showsta", ifname);
	    idx=atoi(&ifname[2]);
    }

    skip = iter = 0;
    while (1) {
	sprintf(buf, "%s idx=%d,skip=%d", cmd, idx, skip);
	fp = popen(buf, "r");
	if (fp == NULL)
	    goto fail;
	cnt = 0;
	while (!feof(fp)) {
	    fgets(buf, sizeof(buf), fp);
	    if (!iter && strstr(buf, "skip=")) // iteration support or not
		iter = 1;
	    if ((ptr = strstr(buf, "macaddr")) && (pos = strstr(ptr, "idx"))) {
		strncpy(tmp, &ptr[10], 17);
		tmp[17]='\0';
		if( (atoi(&pos[4])==idx ) && (strcmp(tmp,"AA:AA:AA:AA:AA:A0")!=0  )
					&&(strcmp(tmp,"AA:AA:AA:AA:AA:A1")!=0  )
					&&(strcmp(tmp,"AA:AA:AA:AA:AA:A2")!=0  )
					&&(strcmp(tmp,"AA:AA:AA:AA:AA:A3")!=0  )	){
		    match_if_flag=1;
		    // Create a new node
		    node = (ASSOC_STA_LIST *)malloc(sizeof(ASSOC_STA_LIST));
		    if (!node) {
			cipcfg_print("Can't allocate memory for ASSOC_STA_LIST\n");
			goto fail;
		    } else {
			bzero(node, sizeof(ASSOC_STA_LIST));
			node->next = head;
			head = node;
		    }
		    node->tx_packets = node->rx_packets = -1;
		    node->tx_byte = node->rx_byte = -1;
		    strncpy(node->mac, &buf[10], 17);
		    cnt++;
		}else
		    match_if_flag=0;
	    } else if ((pos = strstr(buf, "rssi0"))&& match_if_flag) {
		int rssi, rssi_max;
		if (!node) goto fail;
		rssi_max = 108;
		rssi = rssi_max + atoi(&pos[5]);
		if (rssi < 0 || rssi >= rssi_max)
		    rssi = 0;
		else if (rssi > 100)
		    rssi = 100;
		node->rssi = rssi;
	    } else if ((pos = strstr(buf, "MaxSupportedRate"))&& match_if_flag) {
		if (!node) goto fail;
		node->tx_rate = atof(&pos[17]);
	    } else if ((pos = strstr(buf, "tx_okcnt"))&& match_if_flag) {
		if (!node) goto fail;
		node->tx_okcnt = atoi(&pos[9]);
	    } else if ((pos = strstr(buf, "ContinueTxFailCnt"))&& match_if_flag) {
		if (!node) goto fail;
		node->tx_errcnt = atoi(&pos[17]);
	    } else if ((pos = strstr(buf, "TxPackets")) && match_if_flag) {
		if (!node) goto fail;
		node->tx_packets = atof(&pos[10]);
	    } else if ((pos = strstr(buf, "RxPackets")) && match_if_flag) {
		if (!node) goto fail;
		node->rx_packets = atof(&pos[10]);
	    } else if ((pos = strstr(buf, "TxBytes")) && match_if_flag) {
		if (!node) goto fail;
		node->tx_byte = atof(&pos[8]);
	    } else if ((pos = strstr(buf, "RxBytes")) && match_if_flag) {
		if (!node) goto fail;
		node->rx_byte = atof(&pos[8]);
	    } else if ((pos = strstr(buf, "last_rx"))&& match_if_flag) {
		if (!node) goto fail;
		node->last_rx = atof(&pos[8]);
	    }
	}
	pclose(fp);
	if (cnt <= 0)
	    break;
	skip += cnt;
	if (!iter)
	    break;
    }
    return head;

fail:
    if (fp != NULL)
	pclose(fp);
    if (head != NULL)
	free_sta_list(head);
    return NULL;
}

/** 
 * @brief Free the associated station list.
 * 
 * @param head 	The head of the station list.
 */
void free_sta_list(ASSOC_STA_LIST *head)
{
    ASSOC_STA_LIST *cur;

    while (head) {
	cur = head;
	head = head->next;
	free(cur);
    }
}

/** 
 * @brief Count the sta list.
 * 
 * @param head 	The head of station list.
 * 
 * @return 	The count of stations.
 */
int sta_list_count(ASSOC_STA_LIST *head)
{
    int count = 0;
    ASSOC_STA_LIST *node = NULL;

    foreach_sta_get(node, head) {
	count++;
    }

    return count;
}


ASSOC_WDS_LIST * get_wds_list(char * ifname)
{
    return (ASSOC_WDS_LIST *)get_sta_list(ifname);
}

void free_wds_list(ASSOC_WDS_LIST *head)
{
    free_sta_list((ASSOC_STA_LIST *)head);
}

int wds_list_count(ASSOC_WDS_LIST *head)
{
    return sta_list_count((ASSOC_STA_LIST *)head);
}

/*
 * convert string to int array for channel use
 */
void convert_ch(char *str, int *ch)
{
    int i;
    char *delim = " ";
    char *p;
    char *q, *ptr;

    p = str;
    ch[0] = atoi(strtok_r(p, delim, &ptr));
    for( i=1; (q=strtok_r(NULL, delim, &ptr)); i++){
        ch[i] = atoi(q);
    }
    ch[i] = -1; // terminator
}


/*
 *  get allowed channels under a country code.
 */
//
int get_allowed_ch(COUNTRY_CHANNEL *ccObj)
{
    int ret, ct;
    char band[MAX_VALUE_LEN], code[MAX_VALUE_LEN], *tmp, buf[256];
    FILE *fp = NULL;
    CIPCFG *cipcfg = NULL;
    char *ptr;

    memset(buf, 0, 256);
    cipcfg=init_cipcfg();
    if(!cipcfg){
        cipcfg_print("initial cipcfg error.\n");
        return -1;
    }else{
        ret = get_cfg_item(cipcfg, GID_RF1, "country", code);
        close_cipcfg(cipcfg);
        if (ret != CIPCFG_SUCCESS) {
            cipcfg_print("Can NOT get country code, return value is %d.\n", ret);
            sprintf(code, "NA" ); // if can not get it, give it a default value.
            return -1;
        }
    }

    sprintf(buf, "eval \"`cat %s`\"; echo $%s", COUNTRY_CODE, code);
    if ((fp = popen( buf, "r")) == NULL) {
        cipcfg_print("Can't open the country code file\n");
        return -1;
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        ret = atoi(buf); // get the country code
    }
    pclose(fp);

    if ((fp = fopen( ALLOWED_CHANNEL, "r")) == NULL) {
        cipcfg_print("Can't open the channel list!!\n");
        return -1;
    }else{
        while (fgets(buf, sizeof(buf), fp) != NULL) {
	    char *ptr2;
            sscanf(buf, "c%u_11%s='", &ct, band);
	    strtok_r(band, "=", &ptr2);
            strtok_r(buf, "'", &ptr);
            tmp = strtok_r( NULL, "'", &ptr);
            if( ct == (unsigned int)ret ){ // if match coubtry code
                break;
            }
        }
        ccObj->cc = ret;
        ccObj->ch_11b[0] = -1;
        ccObj->ch_11g[0] = -1;
        ccObj->ch_11a[0] = -1;
        ccObj->ch_11ng[0] = -1;
        ccObj->ch_11ngu[0] = -1;
        ccObj->ch_11ngl[0] = -1;
        ccObj->ch_11na[0] = -1;
        ccObj->ch_11nau[0] = -1;
        ccObj->ch_11nal[0] = -1;

        while((ct==(unsigned int)ret)&&(NULL!=tmp)){
	    if (!strcmp(band, "b"))
                convert_ch(tmp, ccObj->ch_11b);
	    if (!strcmp(band, "g")) {
                convert_ch(tmp, ccObj->ch_11g);
            }else if (!strcmp(band, "a")){
                convert_ch(tmp, ccObj->ch_11a);
	    }else if (!strcmp(band, "ng")){
                convert_ch(tmp, ccObj->ch_11ng);
            }else if (!strcmp(band, "ngu")){
                convert_ch(tmp, ccObj->ch_11ngu);
            }else if (!strcmp(band, "ngl")){
                convert_ch(tmp, ccObj->ch_11ngl);
            }else if (!strcmp(band, "na")){
                convert_ch(tmp, ccObj->ch_11na);
            }else if (!strcmp(band, "nau")){
                convert_ch(tmp, ccObj->ch_11nau);
            }else if (!strcmp(band, "nal")){
                convert_ch(tmp, ccObj->ch_11nal);
            }else{
                cipcfg_print("unknown channel group : %c\n", band);
                // return -1; 
            }
            if (fgets(buf, sizeof(buf), fp) != NULL) {
		char *ptr2;
                sscanf(buf, "c%u_11%s='", &ct, band);
		strtok_r(band, "=", &ptr2);
                strtok_r(buf, "'", &ptr);
                tmp = strtok_r( NULL, "'", &ptr);
            }else{
                break;
            }
        }
        fclose(fp);
    }
    return 0;
}


TXPWR_T *get_txpwr_level(void)
{
    static TXPWR_T txpwr[] = {
	{"Auto",	0},
	{"Highest", 	17},
	{"High", 	13},
	{"Medium", 	9},
	{"Low", 	5},
	{"Lowest",	1},
	{NULL,		0}
    };

    return txpwr;
}

/* for AP management start */
int set_txpwr(char *txpwr)
{
    char cmd[BUFSIZ];
    int txp = atoi(txpwr), txpower = 100;
   
    if (txp <= 0 || txp >= 17) {
	txpower = 100;
    } else if (txp >= 13) {
	txpower = 50;
    } else if (txp >= 9) {
	txpower = 25;
    } else if (txp >= 5) {
	txpower = 13;
    } else if (txp >= 1) {
	txpower = 6;
    }

    snprintf(cmd, BUFSIZ, "iwpriv ra0 set TxPower=%d", txpower);
    cip_system(cmd);
    return 0;
}

AP_LIST *ap_scan(void)
{
    return rf_scan(0, 1, NULL);
}

int set_channel_switch(int channel, int count)
{
    return 0;
}
/* for AP management end */
