#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <arpa/inet.h>
#include "cipcfg.h"
#include "wlan.h"
#include "syscall.h"


/**
 * Call wdsds to discover vaild WDS peers.
 * It will put info of the vaild peer in AP_LIST
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery
 * @param ch        The current channel for this RF card
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_scan(int rfno, char *secret, int ch, int mode)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i, ret;

    /* Notice!! When discovery WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
#if 0
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 
#endif

    sprintf(buf, "%s scan -s %s -o /tmp/wdsds%d -i %d -c %d -m %d 2> /dev/null",
        PATH_WDS_SLAVE, secret, rfno, rfno, ch, mode);
    ret = system(buf);

#if 0
    sprintf(buf, "%s start %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf);

    sprintf(buf, "%s restart 2> /dev/null", PATH_RC_BRIDGE );
    system(buf);
    sprintf(buf, "%s restart_wan 2> /dev/null", PATH_RC_NET );
    system(buf);    
#endif

    sprintf(buf, "/tmp/wdsds%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        strncpy(ap->ssid, strtok(buf, " "), sizeof(ap->ssid));
        if ( !strncmp(ap->ssid, buf, sizeof(ap->ssid) ) ){
            strncpy(ap->mac, strtok(NULL, " "), sizeof(ap->mac));
        }else{
            strncpy(ap->mac, ap->ssid, sizeof(ap->mac));
            strcpy( ap->ssid, "");
        }

        ap->mac[17] = '\0';
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }
        ap->ch=atoi(strtok(NULL, " "));
        ap->rate=atoi(strtok(NULL, " "));
        ap->signal=atoi(strtok(NULL, ":"));
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}

/**
 * Call wdsds to connect a vaild WDS peers.
 * It will return the connect result
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery,
 * @param mac       The MAC address of peer.
 * @param ch        The current channel for this RF card
 * @return	    0: success, -1: failure (please reference wdsds.c)
 */
int wdsd_connect(int rfno, char *secret, char *mac, int ch, int mode)
{
    /* TODO: This function still be unstable. Don't use this function. */

    char buf[MAX_VALUE_LEN];
    int ret;

    /* Notice!! When connect WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
    sprintf(buf, "%s stop %d 2> /dev/null" ,
        PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s add -a %s -s %s -i %d -c %d -m %d 2> /dev/null" ,
        PATH_WDS_SLAVE, mac, secret, rfno, ch, mode);
    ret=system(buf); //0: success, -1: failure

    sprintf(buf, "%s start %d 2> /dev/null", PATH_RC_WLAN, rfno);
    system(buf); 

    sprintf(buf, "%s restart  2> /dev/null", PATH_RC_BRIDGE );
    system(buf); 
    
    sprintf(buf, "%s restart_wan  2> /dev/null", PATH_RC_NET );
    system(buf); 
    return ret;
}

/**
 * Get the peer that connected to WDS discovery master
 * It will return AP_LIST
 *
 * @param rfno      The RF card number
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_connected(int rfno)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;

    sprintf(buf, "/tmp/wdsdm%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        cipcfg_print("Error: can't open %s\n", buf);
        return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            cipcfg_print("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        /* The file format is "MAC: xx:xx....." and "RSSI: xx" */
        if (strstr(buf, "MAC")) { 
            strncpy(ap->mac, &buf[5], sizeof(ap->mac));
            ap->mac[17] = '\0';
        
            fgets(buf, sizeof(buf), fp);

            if (strstr(buf, "RSSI")) {
                ap->signal=atoi(&buf[6]);
            }
            else {
                cipcfg_print("Incorrectly file format, RSSI don't fellow MAC\n");
                free(ap);
                fclose(fp);
                return ap_list;
            }
            ap->next=NULL;
        }

        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}


/**
 * Call rfscan to scanning all APs.
 * It will put info of the APs in AP_LIST
 *
 * @param rfno      The RF card number
 * @param is_scan   1: restart scan now, 0: only retrive the file
 * @return	    pointer to ap list, NULL if not found
 */   
AP_LIST *rf_scan(int rfno, int is_scan, char t[MAX_VALUE_LEN])
{
    char cmd[BUFSIZ], buf[BUFSIZ], *sec, del[] = " \t";
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i;
    
    sprintf(cmd, "%s wlan%d", PATH_SCAN, rfno);
    if ((fp = popen(cmd, "r")) == NULL) {
	cipcfg_print("Error, can't run scan program\n");
	return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
	    cipcfg_print("Error: memory allocate failure, "
		    "can't allocate size = %d\n", sizeof(AP_LIST));
            pclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

	strncpy(ap->ssid, buf, 32);
	for (i = 31; i >= 0 && isspace(ap->ssid[i]); i--)
		ap->ssid[i] = '\0';

	strncpy(ap->mac, strtok(buf+32, del), sizeof(ap->mac));
        ap->mac[17] = '\0';
        for(i = 0; i < strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }

        ap->ch=atoi(strtok(NULL, del));
        ap->rate=atoi(strtok(NULL, del));

        strncpy(ap->type,strtok(NULL, del),15);
        for(i = 14; i >=0 && isspace(ap->type[i]); i--)
            ap->type[i] = '\0';

        sec = strtok(NULL, del);
	if (strcmp(sec, "no") == 0) {
	    sprintf(ap->security, "None");
	} else if (strcmp(sec, "WPA-PSK/WPA2-PSK") == 0) {
                sprintf(ap->security, "WPA(1/2)-PSK");
	} else {
                sprintf(ap->security, sec);
	}

        ap->signal=atoi(strtok(NULL, del));

        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        } else {
            ap_list=ap;
        }
    }
    pclose(fp);
    return ap_list;
}

/** 
 * @brief Get WDS Link current attributes (not configuration attributes).
 * 
 * @param rf_idx 	RF index.
 * @param link_idx 	Link index.
 * @param attr		Attribute type. 
 * @param value 	Attribute value.
 * 
 * @return 		1 for success otherwise 0. 
 */
int getWDSLinkAttr(int rf_idx, int link_idx, int attr, char *value)
{
    char group_name[MAX_VALUE_LEN], ret[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    char buf[128], iface[16];
    FILE *fp;
    int retval;

    sprintf(group_name, "rf%d_wds%d", rf_idx + 1, link_idx + 1);

    /* Get interface name by index */
    if (get_wds_ifname_by_idx(rf_idx, link_idx, iface) == NULL)
	return 0;

    switch (attr) {
	case WDSLINK_STATE:
	    cipcfg = init_cipcfg();
	    retval = get_cfg_item_by_str(cipcfg, group_name, "state", ret);
	    close_cipcfg(cipcfg);
	    if (retval != CIPCFG_SUCCESS) {
		return 0;
	    } 

	    if (strcmp(ret, "disable") == 0) {
		sprintf(value, "Disabled");
		return 1;
	    }

	    /* Check if WDS link is disabled */
	    if (!isWDSLinkAlive(iface)) {
	        sprintf(value, "Disconnected");
	        return 1;
	    }

	    /* Check the STP status of WDS link */
	    sprintf(buf, "brctl showstat br0 %s > /tmp/wdslink", iface);
	    cip_system(buf);
	    if ((fp = fopen("/tmp/wdslink", "r")) != NULL) {
		fgets(ret, MAX_VALUE_LEN, fp);
		fclose(fp);
	    }

	    if (strncmp(ret, "listening", 9) == 0 || strncmp(ret, "learning", 8) == 0
		    || strncmp(ret, "forwarding", 10) == 0) {
		sprintf(value, "Active");
		return 1;
	    } else if (strncmp(ret, "disabled", 8) == 0) {
		sprintf(value, "Disabled");
		return 1;
	    } else { // STP status is "blocking".
		sprintf(value, "Backup");
		return 1;
	    }
	    break;
	case WDSLINK_RATE:
	    /* Get WDS Link TX Rate */
	    if (getWDSLinkTxRate(iface, ret)) {
		sprintf(value, "%s M", ret);
		return 1;
	    }
	    break;
	case WDSLINK_QUALITY:
	    /* Get WDS Link Quality */
	    if (getWDSLinkQuality(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_HEALTH:
	    /* Get WDS Link Health */
	    if (getWDSLinkHealth(iface, ret)) {
		sprintf(value, "%s &#37;", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXTOTAL:
	    /* Get WDS Link Tx total frames */
	    if (getWDSLinkTxTotal(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
	case WDSLINK_TXFAIL:
	    /* Get WDS Link Tx fail frames */
	    if (getWDSLinkTxFail(iface, ret)) {
		sprintf(value, "%s", ret);
		return 1;
	    }
	    break;
    }
    return 0;
}

/** 
 * @brief Get associated station list for specific interface.
 * 
 * @param ifname	Interface name.
 * 
 * @return 		Return the list of station.
 */
ASSOC_STA_LIST * get_sta_list(char * ifname)
{
    ASSOC_STA_LIST *head = NULL;
    ASSOC_STA_LIST *node = NULL;
    char buf[MAX_VALUE_LEN], cmd[BUFSIZ], del[] = " \t";
    FILE *fp;

    sprintf(cmd, "%s %s", PATH_STALIST, ifname);

    if ((fp = popen(cmd, "r")) == NULL)
	return NULL;

    /* 
     * The format of sta_list is 
     *     MAC address   RSSI TX_PKT RX_PKT ERR_PKT RATE  IDEL
     * XX:XX:XX:XX:XX:XX  87   1000    600     30    54M   3 
     */
    while (fgets(buf, sizeof(buf), fp)) {
	/* Create a new node */
	node = (ASSOC_STA_LIST *)malloc(sizeof(ASSOC_STA_LIST));
	if (!node) {
	    cipcfg_print("Can't allocate memory for ASSOC_STA_LIST\n");
	    goto fail;
	} else {
	    bzero(node, sizeof(ASSOC_STA_LIST));
	    node->next = head;
	    head = node;
	}

	strncpy(node->mac, strtok(buf, del), 17);
	node->rssi = atoi(strtok(NULL, del));
	node->tx_okcnt = atoi(strtok(NULL, del));
	strtok(NULL,del); /* skip RX_PTK */
	node->tx_errcnt = atoi(strtok(NULL, del));
	node->tx_rate = atof(strtok(NULL, del));
	node->last_rx = atof(strtok(NULL, del));
    }
    pclose(fp);
    return head;

fail:
    pclose(fp);
    return NULL;
}

/** 
 * @brief Get associated WDS list for specific interface.
 * 
 * @param ifname	Interface name.
 * 
 * @return 		Return the list of WDS.
 */
ASSOC_STA_LIST * get_wds_list(char * ifname)
{
    ASSOC_WDS_LIST *head = NULL;
    ASSOC_WDS_LIST *node = NULL;
    char buf[MAX_VALUE_LEN], cmd[BUFSIZ], del[] = " \t";
    char master_iface[16];
    int rfidx, wdsidx, i;
    FILE *fp;

    if (sscanf(ifname, "wlan%d-wds%d", &rfidx, &wdsidx) < 2) {
	cipcfg_print("Can't determine the master interface of %s\n", ifname);
	return NULL;
    }
    sprintf(master_iface, "wlan%d", rfidx);

    sprintf(cmd, "%s %s", PATH_WDSLIST, master_iface);

    if ((fp = popen(cmd, "r")) == NULL)
	return NULL;

    /* 
     * The format of wds_list is 
     *   Status       MAC address    TX_PKT RX_PKT ERR_PKT RATE RSSI 
     * Connected  XX:XX:XX:XX:XX:XX   1000   600     30     54M 60
     */
    i = 0;
    while (fgets(buf, sizeof(buf), fp)) {
	if (i++ != wdsidx)
	    continue;
	/* Create a new node */
	node = (ASSOC_WDS_LIST *)malloc(sizeof(ASSOC_WDS_LIST));
	if (!node) {
	    cipcfg_print("Can't allocate memory for ASSOC_WDS_LIST\n");
	    goto fail;
	} else {
	    bzero(node, sizeof(ASSOC_WDS_LIST));
	    node->next = head;
	    head = node;
	}

	strtok(buf, del); /* skip Status */
	strncpy(node->mac, strtok(NULL, del), 17);
	node->tx_okcnt = atoi(strtok(NULL, del));
	strtok(NULL,del); /* skip RX_PTK */
	node->tx_errcnt = atoi(strtok(NULL, del));
	node->tx_rate = atof(strtok(NULL, del));
	node->rssi = atoi(strtok(NULL,del));
	node->last_rx = 0;
    }
    pclose(fp);
    return head;

fail:
    pclose(fp);
    return NULL;
}

/** 
 * @brief Free the associated station list.
 * 
 * @param head 	The head of the station list.
 */
void free_sta_list(ASSOC_STA_LIST *head)
{
    ASSOC_STA_LIST *cur;

    while (head) {
	cur = head;
	head = head->next;
	free(cur);
    }
}

/** 
 * @brief Count the sta list.
 * 
 * @param head 	The head of station list.
 * 
 * @return 	The count of stations.
 */
int sta_list_count(ASSOC_STA_LIST *head)
{
    int count = 0;
    ASSOC_STA_LIST *node = NULL;

    foreach_sta_get(node, head) {
	count++;
    }

    return count;
}


/*
 * convert string to int array for channel use
 */
void convert_ch(char *str, int *ch)
{
    int i;
    char *delim = " ";
    char *p;
    p = str;
    ch[0] = atoi(strtok(p, delim));
    for( i=1; (p=strtok(NULL, delim)); i++){
        ch[i] = atoi(p);
    }
    ch[i] = -1; // terminator
}


/*
 *  get allowed channels under a country code.
 */
int get_allowed_ch(COUNTRY_CHANNEL *ccObj)
{
    int ret, ct;
    char band, code[MAX_VALUE_LEN], *tmp, buf[256];
    FILE *fp = NULL;
    CIPCFG *cipcfg = NULL;

    memset(buf, 0, 256);
    cipcfg=init_cipcfg();
    if(!cipcfg){
        cipcfg_print("initial cipcfg error.\n");
        return -1;
    }else{
        ret = get_cfg_item(cipcfg, GID_RF1, "country", code);
        close_cipcfg(cipcfg);
        if (ret != CIPCFG_SUCCESS) {
            cipcfg_print("Can NOT get country code, return value is %d.\n", ret);
            sprintf(code, "NA" ); // if can not get it, give it a default value.
            return -1;
        }
    }

    sprintf(buf, "eval \"`cat %s`\"; echo $%s", COUNTRY_CODE, code);
    if ((fp = popen( buf, "r")) == NULL) {
        cipcfg_print("Can't open the country code file\n");
        return -1;
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
        ret = atoi(buf); // get the country code
    }
    pclose(fp);

    if ((fp = fopen( ALLOWED_CHANNEL, "r")) == NULL) {
        cipcfg_print("Can't open the channel list!!\n");
        return -1;
    }else{
        while (fgets(buf, sizeof(buf), fp) != NULL) {
            sscanf(buf, "c%u_11%c='", &ct, &band);
            strtok(buf, "'");
            tmp = strtok( NULL, "'");
            if( ct == (unsigned int)ret ){ // if match coubtry code
                break;
            }
        }
        ccObj->cc = ret;
        ccObj->ch_11b[0] = -1;
        ccObj->ch_11g[0] = -1;
        ccObj->ch_11a[0] = -1;
        ccObj->ch_11n[0] = -1;
        while((ct==(unsigned int)ret)&&(NULL!=tmp)){
            if (band=='b'){
                convert_ch(tmp, ccObj->ch_11b);
            }else if (band=='g'){
                convert_ch(tmp, ccObj->ch_11g);
            }else if (band=='a'){
                convert_ch(tmp, ccObj->ch_11a);
            }else if (band=='n'){
                convert_ch(tmp, ccObj->ch_11n);
            }else{
                cipcfg_print("unknown channel group : %c\n", band);
                // return -1; 
            }
            if (fgets(buf, sizeof(buf), fp) != NULL) {
                sscanf(buf, "c%u_11%c='", &ct, &band);
                strtok(buf, "'");
                tmp = strtok( NULL, "'");
            }else{
                break;
            }
        }
        fclose(fp);
    }
    return 0;
}

TXPWR_T *get_txpwr_level(void)
{
    static TXPWR_T txpwr[] = {
	{"Highest", 	0},
	{"High", 	1},
	{"Medium", 	2},
	{"Low", 	3},
	{"Lowest",	4},
	{NULL,		0}
    };

    return txpwr;
}

/* for AP management start */
int set_txpwr(char *txpwr)
{
    char cmd[BUFSIZ],country[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    int atten_CCK, atten_OFDM, txpower;

    // Get Country
    cipcfg=init_cipcfg();
    if(!cipcfg){
        printf("initial cipcfg error.\n");
        return -1;
    }else{
        if (get_cfg_item(cipcfg, GID_RF1, "country", country) != CIPCFG_SUCCESS) {
            printf("Can NOT get country code.\n");
            sprintf(country, "NA" ); // if can not get it, give it a default value.
        }
        close_cipcfg(cipcfg);
    }
    // Setup new txpower level
    txpower = atoi(txpwr);
    atten_CCK=0;
    atten_OFDM=0;
    if(strcmp(country,"DE")==0){
        atten_CCK=3;
        atten_OFDM=0;
    }

    snprintf(cmd,BUFSIZ,"iwpriv wlan0 set_mib TxPowerCCK=$(/sbin/rtl8186_hw_config txPowerCCK %d)",(txpower*3+atten_CCK));
    cip_system(cmd);

    snprintf(cmd,BUFSIZ,"iwpriv wlan0 set_mib TxPowerOFDM=$(/sbin/rtl8186_hw_config txPowerOFDM %d)",(txpower*3+atten_OFDM));
    cip_system(cmd);
    return 0;
}

AP_LIST *ap_scan(void)
{
    return rf_scan(0,1,NULL);
}

int set_channel_switch(int channel, int count)
{
    return 0;
}
/* for AP management end */
