#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "debug.h"
#include "cipfwd.h"
#include "proxy_arp.h"

/* = = = = = = = = = = = = = = = The process flows: = = = = = = = = = = = = = = = 
 *
 * +---------+               (fork)    +-------------+ 
 * | INITIAL +-----------+============>| INFO CHANGE +-----+
 * +---------+           |             +-----+-------+     | (Receive info from other APS,
 *                 +-----+-----+             |             |  and save them)
 *         +------>| MAIN LOOP |             +-------------+
 *         |       +-----+-----+
 *         |             |           +--------------+
 *         |             +---------->| DHCP Handler +---+
 *         |             |           +--------------+   |
 *         |             |           +-------------+    |
 *         |             +---------->| ARP Handler +----+
 *	   |                         +-------------+    |
 *         |                         +-------------+    |
 *         +-------------------------+ Update Info |<---+
 *                                   +-------------+
 *
 * 							2009/04/17 10:04 by sam 
 * = = = = = = = = = = = = = = = DHCP handler = = = = = = = = = = = = = = = = =
 * 
 * 1. check the packet is DHCP ACK
 * 2. catch the yiaddr and chaddr of dhcp packet
 * 3. send update msg 
 *
 * = = = = = = = = = = = = = = = ARP  handler = = = = = = = = = = = = = = = = =
 * 
 * 1. check the packet is ARP
 * 2. catch the sender ip and sender mac address of ARP packet
 * 3. send update msg
 * 	a. if the MAC is not found but the IP is found, ignore this.
 * 	b. if the either MAC nor IP is found in the list, add the new pair 
 * 	   to the list. The TTL is 3600(secs).
 * 	c. if the roaming feature is on and the MAC/IP pair is found, add 
 * 	   the new pair to the list.
 * 	
 * = = = = = = = = = = = = = = = info change  = = = = = = = = =	= = = = = = = =
 *
 * 1. create socket and receiver packet from APs
 * 2. check the checksum 
 * 3. call DAI.sh to update msg
 */

void handle_packet(ebt_ulog_packet_msg_t *pkt,int pkt_length)
{
	int curr_len=0,dhcp_offset=0,dhcpopt_offset=0;
	int lease_time = 360;  // 6 minutes
	struct ethhdr *ehdr;
	char interface[32];
	time_t now;
	int seconds, lock_fd;
	
	struct vlan_hdr *vahdr; 
	
	unsigned char dhcpopt[6],mac[ETH_ALEN],yourip[ETH_IPLEN]; 
	struct iphdr *iph;
	struct udphdr *udph;
	struct apinfo *cip_info;
	cip_info = (struct apinfo*)malloc(sizeof(struct apinfo));		
	unsigned char opcode[1];
	unsigned char hwlen[1];	//hw addr length
	unsigned char sdmac[ETH_ALEN]; //hw addr
	unsigned char plen[1];	//porotocl addr length
	unsigned char sdip[ETH_IPLEN];
	
	sprintf(interface,"%s",pkt->physindev);
	ehdr = (struct ethhdr *)pkt->data;
	curr_len=ETH_HLEN;
	if (ehdr->h_proto == htons(ETH_P_8021Q)) {
		vahdr=(struct vlan_hdr *)(((char *)ehdr) + curr_len);
		dbg_printf("[802_1Q] \n encapsulated type/length: %x\n",vahdr->h_vlan_encapsulated_proto);
		ehdr = (struct ethhdr *)(((char *)ehdr) + 4);	// shift the 802_1q header
		curr_len+=4;
	}
	


	//		DHCP Handler
	if (ehdr->h_proto == htons(ETH_P_IP)) {
		iph = (struct iphdr *)(pkt->data + curr_len);	
		if(iph->protocol==17){  //UDP
			curr_len += iph->ihl * 4;
		udph = (struct udphdr *)(pkt->data + curr_len);	
			if (udph->source == htons(67) && udph->dest == htons(68)) {   //DHCP
			curr_len +=  2*4;  // bootp 
			dhcp_offset =curr_len  ;
			/* the offset from bootp header to dhcp option 
			 *  {Opcode + Hw type + Hw length + Hop count +
			 *   Transaction ID +
			 *   Number of seconds  + Flags +
			 *   Client IP +
			 *   Your IP +
			 *   Server IP +
			 *   Gateway IP }	= 7*4
			 *  {Client MAC}  	= 16
			 *  {Host name}   	= 64
			 *  {Boot filename}    	= 128
			 *  {Magic cookie} 	= 4
			 */
			curr_len +=  7*4 + 16 + 64 + 128 + 4;

			memcpy(&mac, (pkt->data + dhcp_offset+7*4),ETH_ALEN);

	 			memcpy(&dhcpopt ,(pkt->data +  curr_len),4);
				dhcpopt_offset=0; 
				while(*dhcpopt!=0xff && *dhcpopt!=53){	// find message type 
					curr_len+=2+*(dhcpopt+1);	
						if( curr_len + dhcpopt_offset >= pkt_length){
							dbg_printf("Can't find bootp type\n");
							return;	
						}
	 				memcpy(&dhcpopt ,(pkt->data + curr_len+ dhcpopt_offset),4);
				}
				if(*dhcpopt==53 && *(dhcpopt+1)==1 && *(dhcpopt+2)==5){   // dhcp ACK	
	 				memcpy(&yourip ,(pkt->data + dhcp_offset+ 4*4),ETH_IPLEN);
					dbg_printf("[DHCP ACK] client mac= %02x:%02x:%02x:%02x:%02x:%02x\n",*mac,*(mac+1),*(mac+2),*(mac+3),*(mac+4),*(mac+5));
					dbg_printf("your ip= %d.%d.%d.%d\n",*yourip,*(yourip+1),*(yourip+2),*(yourip+3));

					// get the release time from dhcp packet
					if( enable_fdhcp ){ 
						dhcpopt_offset=0; 
						while(*dhcpopt!=0xff && *dhcpopt!=51){	// find lease time
							dhcpopt_offset+=2+*(dhcpopt+1);
							if( curr_len + dhcpopt_offset >= pkt_length){
								dbg_printf("Can't find DHCP lease time\n");
								break;	
							}
	 						memcpy(&dhcpopt ,(pkt->data + curr_len + dhcpopt_offset),4);
						}
						if(*dhcpopt==51){
	 						memcpy(&dhcpopt ,(pkt->data + curr_len + dhcpopt_offset),6);
							lease_time=*(dhcpopt+3)*65536 + *(dhcpopt+4)*256 + *(dhcpopt+5);
							dbg_printf("DHCP lease time %d seconds \n",lease_time);
						}
					}

					seconds = time (&now)+ lease_time ;
				
					//memcpy(&cip_info->staip,(ETH_HLEN+pkt->data + iph->ihl*4+2*4+4*4),4);
					memcpy(&cip_info->staip,yourip,ETH_IPLEN);
					cip_info->timestamp=(time_t)seconds;
					memcpy(cip_info->stamac,mac,ETH_ALEN);
					lock_fd= lock_cipfwd();
					if(update_info(cip_info,"D",interface)==-1){
						dbg_printf("update error \n");
					} 
					unlock_cipfwd(lock_fd);
				}// dhcp ACK
				else if(*dhcpopt==53 && *(dhcpopt+1)==1 && *(dhcpopt+2)==1)   // dhcp discover	
					dbg_printf("[DHCP DISCOVER]\n");
				else if(*dhcpopt==53 && *(dhcpopt+1)==1 && *(dhcpopt+2)==2)   // dhcp offer	
					dbg_printf("[DHCP OFFER]\n");
				else if(*dhcpopt==53 && *(dhcpopt+1)==1 && *(dhcpopt+2)==3)   // dhcp request	
					dbg_printf("[DHCP REQUEST]\n");
				else if(*dhcpopt==53 && *(dhcpopt+1)==1 && *(dhcpopt+2)==7)   // dhcp lease	
					dbg_printf("[DHCP RELEASE]\n");
			        	
			}//DHCP
			
		}// UDP
		
	}//   	ARP Handler          
	else if (ehdr->h_proto == htons(ETH_P_ARP)) {
		memcpy(&opcode, (pkt->data + curr_len+7),1);
		if(*opcode==1 || *opcode==2){	
			memcpy(&hwlen,(pkt->data + curr_len+ 4),1);	//hw addr length
			memcpy(&sdmac, (pkt->data + curr_len+ 8),ETH_ALEN); //hw addr
			memcpy(&plen,(pkt->data + curr_len + 5),1);	//porotocl addr length
			dbg_printf("[ARP] \nsender mac= %02x:%02x:%02x:%02x:%02x:%02x\n",*sdmac,*(sdmac+1),*(sdmac+2),*(sdmac+3),*(sdmac+4),*(sdmac+5));
			memcpy(&sdip, (pkt->data + curr_len+ 8+ *hwlen),ETH_IPLEN);
			if(*plen==4){
				dbg_printf("sender ip= %d.%d.%d.%d\n",*sdip,*(sdip+1),*(sdip+2),*(sdip+3));
			}
			seconds = time (&now)+3600;
			
			memcpy(&cip_info->staip,sdip,ETH_IPLEN);
			cip_info->timestamp=(time_t)seconds;
			memcpy(cip_info->stamac,sdmac,ETH_ALEN);
			lock_fd= lock_cipfwd();
			if(update_info(cip_info,"A",interface)==-1){
				dbg_printf("update error \n");
			} 
			unlock_cipfwd(lock_fd);
			
		}
		
	}//ARP
	
	free(cip_info);
}

/*
 * 54  Timestamp  50     System Name    18    AP MAC    12  Entry IP   8    Entry MAC    2  Chksum   0
 *  |------4------|---------32-----------|-------6-------|------4------|--------6--------|-----2-----|
 *
 *  Chksum = [ip upper bytes] XOR [ip lower bytes] XOR [mac 1,2 bytes] XOR [mac 3,4 bytes] XOR [mac 5,6 bytes]
 *
 *  sender staip stamac ttl name apip apmac
 */

//	recv other APs update info
void info_change(){
	int sockfd, addrlen, nbytes, lock_fd;
	char buffer[MSGBUFSIZE];
        struct sockaddr_in client_addr;
	char cmd[256];
	sockfd=info_change_create_handle();
	struct apinfo *info;
	char *tmp;
	if(!sockfd){
		printf("socket error \n");	
		exit(1);
	}
	while(1){
                addrlen = sizeof(client_addr);
		bzero(buffer, MSGBUFSIZE);
		if (( nbytes=recvfrom(sockfd,buffer,MYBUFSIZ,0,(struct sockaddr *) &client_addr,&addrlen)) < 0) {
		    printf("recvfrom error %d\n",nbytes);
		    continue; 
		}

	        info=(struct apinfo *)buffer;
	        dbg_printf("[info change] recevier \n");
	        dbg_printf("timestamp=%d\n",info->timestamp);
		dbg_printf("name=%s\n",info->name);
		dbg_printf("ip=%d\n",info->staip);
		
		dbg_printf("stamac=%02x:%02x:%02x:%02x:%02x:%02x\n",info->stamac[0],info->stamac[1],info->stamac[2],
							   info->stamac[3],info->stamac[4],info->stamac[5]);
		dbg_printf("apmac=%02x:%02x:%02x:%02x:%02x:%02x\n",info->apmac[0],info->apmac[1],info->apmac[2],
							   info->apmac[3],info->apmac[4],info->apmac[5]);
		
		unsigned char pkgip[4];
		memcpy(pkgip,&info->staip,4);
		char sum[2],pkgsum[2];
		sprintf(pkgsum,"%x%x",info->chksum[0],info->chksum[1]);
		sprintf(sum,"%x%x",pkgip[0]^pkgip[2]^info->stamac[0]^info->stamac[2]^info->stamac[4]^info->apmac[0]^info->apmac[2]^info->apmac[4],
				   pkgip[1]^pkgip[3]^info->stamac[1]^info->stamac[3]^info->stamac[5]^info->apmac[1]^info->apmac[3]^info->apmac[5]);
				   
		if(strcmp(pkgsum,sum)!=0){
			dbg_printf("checksum conflict\n");
			continue;
		}
		
		struct in_addr staddr;
	        staddr.s_addr=info->staip;	
		tmp=(char *)inet_ntoa(staddr);
		
		lock_fd= lock_cipfwd();
		if(update_info(info,"R","")==-1){
			printf("update error \n");
		} 
		unlock_cipfwd(lock_fd);
			
	}	
}

int update_info(struct apinfo *input, char *packet_type, char *interface)
{
    FILE *sfp, *afp, *tfp; 
    char buf[MAX_BUFSIZE], tmp_buf[MAX_BUFSIZE];
    char *entry,*save_tok;
    char *ip, *type, *mac_or_if, *mac;
    char *sta_ip, sta_mac[32], sta_mac_or_if[32], *sta_type;
    char among[256], cmd[256];
    time_t now;
    int sta_ttl,index_flag, update_flag=0, match_flag=0, keep_flag=0, modify_among_flag=0;
    int ttl, now_time = time (&now), output_count=0, index=0;

	struct apinfo *info;
	info=input;
	dbg_printf("\n<update_info> receiver: \n");

	struct in_addr staddr;
	staddr.s_addr=info->staip;
	sta_ip=(char *)inet_ntoa(staddr);
	dbg_printf("staip=%s ",sta_ip);

	sprintf(sta_mac,"%02X:%02X:%02X:%02X:%02X:%02X",
			info->stamac[0],info->stamac[1],info->stamac[2],info->stamac[3],info->stamac[4],info->stamac[5]);
	dbg_printf("stamac=%s ",sta_mac);

	sta_type=packet_type;
	dbg_printf("statype=%s ",sta_type);
	sta_ttl=info->timestamp;
	dbg_printf("stattl=%d ",sta_ttl);

	if(strcmp(sta_type,"R")==0){	// mac_or_if is apmac
		sprintf(sta_mac_or_if,"%02X:%02X:%02X:%02X:%02X:%02X",
			info->apmac[0],info->apmac[1],info->apmac[2],info->apmac[3],info->apmac[4],info->apmac[5]);
	
	}else{				//mac_or_if is interface
		if(strcmp("0.0.0.0",sta_ip)==0 ){
			return 1;
		}
		sprintf(sta_mac_or_if,"%s",interface);
	}
	dbg_printf("mac/if=%s \n",sta_mac_or_if);

	if(sta_ttl<=now_time){
		dbg_printf("This data is exprie\n");
		return 1;
	}	

					
    // step 1.   open status file and read entry
	
	if ((sfp = fopen( STATUS_FILE, "r")) == NULL) {
	        printf("Can't open the status file\n");
	        return -1;
    	}
	
	if ((tfp = fopen( TMP_FILE, "w")) == NULL) {
	        printf("Can't open the tmp file\n");
	        return -1;
    	}

	if ((afp = fopen( AMONG_FILE, "w")) == NULL) {
	        printf("Can't open the status file\n");
	        return -1;
    	}

	index_flag=0;
	match_flag=0;
 	update_flag=0;
        while( fgets(buf, MAX_BUFSIZE, sfp) ){
		//printf("\nbuf=%s \n",buf);
		entry=(char *)strtok_r(buf," \n",&save_tok);	
		index_flag=0;
    		while(entry!=NULL){
			switch(index_flag){
				case 0:	// ip
					ip=entry;
				break;	
				case 1:	// mac 
					mac=entry;
				break;	
				case 2:	// type 
					type=entry;
				break;
				case 3:	// ttl 
					ttl=atoi(entry);
				break;
				case 4:	// mac_of_if 
					mac_or_if=entry;
				break;
			}
			index_flag++;
	        	entry=(char *)strtok_r(NULL," \n",&save_tok);
		}	
    // step 2.   cmp status entry
		keep_flag=0;
    // step 2-1  remove expire entry
		if(index_flag==5){
			sprintf(tmp_buf,"%s %s %s %d %s\n",ip,mac,type,ttl,mac_or_if);
			if(ttl<now_time){
				modify_among_flag=1;
				remove_proxy_arp(ip, mac);
				dbg_printf("%d < %d remove %s %s \n",ttl,now_time,ip,mac);
				continue;	
			}
		}
		else{
			sprintf(tmp_buf,"%s %s %s \n",ip,mac,type);
		}

    // step 2-2  DHCP, ROAMING
		if(strcmp(sta_type,"R")==0 || strcmp(sta_type,"D")==0 ){
			if(strcmp(mac,sta_mac)!=0 && strcmp(ip,sta_ip)!=0){
				//printf("neither mac nor ip match \n");
				keep_flag=1;
			}
			else if( strcmp(mac,sta_mac)==0 || strcmp(ip,sta_ip)==0 ){
				//printf(" mac or ip match \n");
				match_flag=1;
				if(sta_ttl > ttl && strcmp(type,"S")!=0){
					//printf(" ttl > entry update \n");
					update_flag=1;
				}
				else{
					//printf(" ttl < entry no update \n");
					keep_flag=1;
				}
			}


	
		}else{// type : A 
    // step 2-3  ARP
    			//printf("mac=%s sta_mac=%s cmp=%d \n",mac,sta_mac,strcmp(mac,sta_mac));
			if(strcmp(mac,sta_mac)==0 && strcmp(ip,sta_ip)!=0){
				match_flag=1;
				if( sta_ttl > ttl && strcmp(type,"S")!=0 ){
					dbg_printf("sta ttl %d > ttl %d update \n",sta_ttl,ttl);
					if(update_flag!=-1){
						update_flag=1;
					}
				}
				else{
					keep_flag=1;
				}
			}
			else if(strcmp(ip,sta_ip)==0){ // hacker ?
				dbg_printf("ip conflict \n");
				match_flag=1;
				update_flag=-1;
				keep_flag=1;
			}
			else{
				keep_flag=1;
			} 
		
		}
		if(keep_flag==1){
			//sprintf(output[output_count],"%s",tmp_buf);
			fputs(tmp_buf,tfp);
			output_count++;
			if( strcmp(ip,"0.0.0.0")==0 ){
				sprintf(among,"%s,",mac);
			}
			else{
				sprintf(among,"%s=%s,",mac,ip);
			}
			if( strcmp(type,"R")!=0){
				fputs(among,afp);
			}
		} else {
			remove_proxy_arp(ip, mac);
		}

	}// while
    // step 3.	 update among file and send update msg
	if(update_flag==1 || match_flag==0 ){
		dbg_printf("update :");
		sprintf(tmp_buf,"%s %s %s %d %s\n",sta_ip,sta_mac,sta_type,sta_ttl,sta_mac_or_if);
		dbg_printf("%s ",tmp_buf);
		//sprintf(output[output_count],"%s",tmp_buf);
		fputs(tmp_buf,tfp);
		add_proxy_arp(sta_ip, sta_mac);
		output_count++;
		if( strcmp(sta_ip,"0.0.0.0")==0 ){
			sprintf(among,"%s,",sta_mac);
		}
		else{
			sprintf(among,"%s=%s,",sta_mac,sta_ip);
		}
		if( strcmp(sta_type,"R")!=0){
			fputs(among,afp);
		}
		modify_among_flag=1;
		dbg_printf("flag:%d %d %s\n",update_flag, match_flag,sta_type);
		if(strcmp(sta_type,"R")==0){
			sprintf(cmd,"%s %s %s %d \"%s\" %s\n",SENDER,sta_ip, sta_mac, sta_ttl, dev_name, sta_mac_or_if);
		}
		else{
			sprintf(cmd,"%s %s %s %d \"%s\" %s\n",SENDER, sta_ip, sta_mac, sta_ttl, dev_name, dev_mac);
		}
		if(roaming_flag==1){
			system(cmd);
		}
		//printf("cmd = %s \n",cmd);
	}else{
		dbg_printf("no update\n");
	}
	fputs("\n",afp);


    fclose(sfp);
    fclose(afp);
    fclose(tfp);
    fflush(sfp);
    fflush(afp);
    fflush(tfp);
   // step 4. update status_file,  and ebtables rule 
    if(modify_among_flag==1){
	if (rename(TMP_FILE,STATUS_FILE)<0){
	    printf("rename %s fail !\n",STATUS_FILE);
    	    return 1;
    	}

    	if(strcmp(sta_type,"R")!=0){
	    if(enable_fdhcp){	
    	    	sprintf(cmd,"ebtables -t nat -D DAI_CHECK -1");
    	    	system(cmd);
    
	    	sprintf(cmd,"ebtables -t nat -A DAI_CHECK --mamong-ether arp --mamong-vlan 1 --mamong-src-file ! %s -j DROP",AMONG_FILE);
    	    	system(cmd);

	    }
	    else{
    	    	sprintf(cmd,"ebtables -t nat -D DAI_CHECK -1");
    	    	system(cmd);
	    	sprintf(cmd,"ebtables -t nat -A DAI_CHECK --mamong-ether arp --mamong-vlan 1 --mamong-src-file ! %s --ulog --ulog-nlgroup 3 --ulog-cprange 0 -j DROP",AMONG_FILE);
    	    	system(cmd);

    	    	sprintf(cmd,"ebtables -t nat -D DAI_LEARN -1");
    	    	system(cmd);
	    	sprintf(cmd,"ebtables -t nat -A DAI_LEARN --mamong-ether arp --mamong-vlan 1 --mamong-src-file ! %s --ulog --ulog-nlgroup 3 --ulog-cprange 0 -j CONTINUE",AMONG_FILE);
    	    	system(cmd);
	    }
	}
    }
    return 1;
}

int lock_cipfwd(){
	int fd = -1;
	fd = open(LOCK_CIPFWD, O_RDWR|O_CREAT|O_TRUNC);
	if( -1 == fd ){
		return OPENFILE_ERROR;
	}
	if( 0 != flock(fd, LOCK_EX) ){
	        close(fd);
	        return LOCK_ERROR;
	}
	else{
		return fd;
	}
}

int unlock_cipfwd(int fd){
	flock(fd, LOCK_UN);
	close(fd);
}

int main(int argc, char *argv[])
{
	unsigned char* buf;
	int len;
	int info_change_pid=1,check_pid;
	struct ebt_ulog_packet_msg *upkt;
	int sockfd, addrlen, nbytes, lock_fd;
	char buffer[MSGBUFSIZE];
        struct sockaddr_in client_addr;
	char cmd[256];
	struct apinfo *info;
	char *tmp;

	if (argc < 6) {
		fprintf(stderr, "Usage: %s port roaming_flag dev_name dev_mac force_dhcp [debug]\n", argv[0]);
		exit(1);
	}

	strncpy(dev_name,argv[3],strlen(argv[3]));
	dev_name[strlen(dev_name)]='\0';
	if(argc == 7){ 
		enable_debug=atoi(argv[6]);
		dbg_printf("cipfwd: debug mode %d\n",enable_debug);
	}
	dbg_printf("cipfwd: port: %d\n",atoi(argv[1]));
	enable_fdhcp=atoi(argv[5]);
	dbg_printf("cipfwd: force dhcp mode %d\n",enable_fdhcp);
	strncpy(dev_mac,argv[4],strlen(argv[4]));
	dev_mac[strlen(dev_mac)]='\0';
	dbg_printf("device name: %s device mac: %s \n",dev_name,dev_mac);
	if(atoi(argv[2])==1){
		roaming_flag=1;
		if(!fork()){
			dbg_printf("info_change....\n");
			while(info_change_pid!=0){
				info_change_pid=fork();
				if(info_change_pid==0)break;
				check_pid=wait(NULL);
				sleep(10);
				dbg_printf("wait %d\n",check_pid);
			}
			info_change();
		}
	}
	sockfd=ulog_socket_create_handle(atoi(argv[1]));

	if(!sockfd){
		printf("socket error\n");
		exit(1);
	}
	while(1){
                addrlen = sizeof(client_addr);
		bzero(buffer, MSGBUFSIZE);
		if (( nbytes=recvfrom(sockfd,buffer,MYBUFSIZ,0,(struct sockaddr *) &client_addr,&addrlen)) < 0) {
		    printf("recvfrom error %d\n",nbytes);
		    continue; 
		}
		dbg_printf("\ncipfwd %d bytes received\n", nbytes);
		if(enable_debug==2){
			int i=0;
			while(i<nbytes){
				dbg_printf("%02x",buffer[i]);
				i++;
			}
			dbg_printf("\n");
		}
		handle_packet((ebt_ulog_packet_msg_t *)buffer,nbytes);
	}

	return 0;
}
