#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h> 
#define HELLO_PORT 12345
#define HELLO_GROUP "225.0.0.37"

/*
 * 54  Timestamp  50     System Name    18    AP MAC    12  Entry IP   8    Entry MAC    2  Chksum   0
 *  |------4------|---------32-----------|-------6-------|------4------|--------6--------|-----2-----| 
 *
 *  Chksum = [ip upper bytes] XOR [ip lower bytes] XOR [mac 1,2 bytes] XOR [mac 3,4 bytes] XOR [mac 5,6 bytes] 
 *
 *  sender staip stamac ttl name apmac
 */

// debug flag
int enable_debug=1;

#define dbg_printf(format, args...)                             \
    do {                                                        \
        if (enable_debug) {                                     \
	    fprintf(stderr, format,                             \
	             ##args);                                   \
	}                                                       \
    } while(0)


struct apinfo{
    time_t timestamp;
    char name[32];
    unsigned char apmac[6];
    int staip;
    unsigned char stamac[6];
    unsigned char chksum[2];
    
};
main(int argc, char *argv[])
{
    struct sockaddr_in addr;
    
    int fd, cnt;
    int second;
    struct ip_mreq mreq;
    char message[256]="Hello, World!";
    unsigned char *tmp;
    struct in_addr iaddr;
    unsigned int ip;
    unsigned char octet[6] ;
    int x1,x2,x3,x4,x5,x6 ;
    struct apinfo info; 

    if(argc<6){
	fprintf(stderr, "Usage: %s IP MAC ttl system_name AP_MAC \n", argv[0]);
	exit(1);
    }


    //  Timestamp
    second=atoi(argv[3]);
    info.timestamp=atoi(argv[3]);
    //  System Name
    sprintf(info.name,"%s",argv[4]);   
    
    //  Station IP
    ip=inet_aton(argv[1],&iaddr);
    info.staip=iaddr.s_addr;
    //tmp = inet_ntoa(iaddr); 
    
    //  Station MAC  
    sscanf( argv[2], "%X:%X:%X:%X:%X:%X",&x1,&x2,&x3,&x4,&x5,&x6 ) ;
    info.stamac[0] = (char)x1;
    info.stamac[1] = (char)x2;
    info.stamac[2] = (char)x3;
    info.stamac[3] = (char)x4;
    info.stamac[4] = (char)x5;
    info.stamac[5] = (char)x6;
    
    //  AP IP
    //ip=inet_aton(argv[5],&iaddr);
    //info.apip=iaddr.s_addr;
    //tmp = inet_ntoa(iaddr); 
    
    //  AP MAC  
    sscanf( argv[5], "%X:%X:%X:%X:%X:%X",&x1,&x2,&x3,&x4,&x5,&x6 ) ;
    info.apmac[0] = (char)x1;
    info.apmac[1] = (char)x2;
    info.apmac[2] = (char)x3;
    info.apmac[3] = (char)x4;
    info.apmac[4] = (char)x5;
    info.apmac[5] = (char)x6;
    
    
    sprintf(message,"%s %s %s %s %s",argv[1],argv[2],argv[3],argv[4],argv[5]);
    
    // check sum
    tmp=&info.staip;
    //apip=&info.apip; 
    info.chksum[0]=tmp[0]^tmp[2]^info.stamac[0]^info.stamac[2]^info.stamac[4]^info.apmac[0]^info.apmac[2]^info.apmac[4];
    info.chksum[1]=tmp[1]^tmp[3]^info.stamac[1]^info.stamac[3]^info.stamac[5]^info.apmac[1]^info.apmac[3]^info.apmac[5];
    /* create what looks like an ordinary UDP socket */
    if ((fd=socket(AF_INET,SOCK_DGRAM,0)) < 0) {
        perror("socket");
 	exit(1);
    }
    /* set up destination address */
    memset(&addr,0,sizeof(addr));
    addr.sin_family=AF_INET;
    addr.sin_addr.s_addr=inet_addr(HELLO_GROUP);
    addr.sin_port=htons(HELLO_PORT);
    tmp=&info;				     
 
    if(argc==7){
	dbg_printf("<send> set %s as destination interface\n",argv[6]);
	struct in_addr localInterface;
	localInterface.s_addr = inet_addr(argv[6]);

	if(setsockopt(fd, IPPROTO_IP, IP_MULTICAST_IF, (char *)&localInterface, sizeof(localInterface)) < 0){
	    perror("Setting local interface error");
	    exit(1);
	}
    }

    /* now just sendto() our destination! */
    if (sendto(fd,tmp,sizeof(info),0,(struct sockaddr *) &addr,
     	sizeof(addr)) < 0) {
     	perror("sendto");
     	exit(1);
    }
	dbg_printf("<send> %s\n",message);
}
