
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <netdb.h>
#include "../include/ebtables_u.h"
#include <linux/netfilter_bridge/ebt_cipfw.h>
#define CIPFW  '1'

static struct option opts[] =
{
	{ "cipfw"      , required_argument, 0, CIPFW   },
	{ 0 }
};

static void print_help()
{
	printf(
"cipfw options:\n"
"--cipfw    [!] --cipfw--dhcp : source MAC match the CHWADDR MAC in dhcp packet\n"
"	    ie : ebtables -A chain -p ip --ip-proto udp --ip-sport 68 --ip-dport 67 --cipfw ! --cipfw-dhcp -j DROP\n"
"--cipfw    [!] --cipfw--arp  : source MAC match the sender MAC in ARP packet\n"
"	    ie : ebtables -A chain -p arp --cipfw ! --cipfw-arp -j DROP\n");
}

static void init(struct ebt_entry_match *match)
{
	struct ebt_cipfw_info *ipinfo = (struct ebt_cipfw_info *)match->data;

	ipinfo->invflags = 0;
	ipinfo->bitmask = 0;
}

#define OPT_CIPFW  0x01
static int parse(int c, char **argv, int argc, const struct ebt_u_entry *entry,
   unsigned int *flags, struct ebt_entry_match **match)
{
	struct ebt_cipfw_info *ipinfo = (struct ebt_cipfw_info *)(*match)->data;
	char *end;
	long int i;
	switch (c) {
	case CIPFW:
		ebt_check_option2(flags, OPT_CIPFW);
		ipinfo->bitmask |= EBT_CIPFW;
		if (ebt_check_inverse2(optarg)){
			ipinfo->invflags |= EBT_CIPFW;
		}
	        if(strcmp("--cipfw-dhcp",optarg)==0){
			ipinfo->tos = CIPFW_DHCP;
			
		}
		else if(strcmp("--cipfw-arp",optarg)==0){
			ipinfo->tos = CIPFW_ARP;
		}
		else
			ebt_print_error2("Only accept --cipfw-dhcp or --cipfw-arp");
			
		break;	
	default:
		return 0;
	}
	return 1;
}

static void final_check(const struct ebt_u_entry *entry,
   const struct ebt_entry_match *match, const char *name,
   unsigned int hookmask, unsigned int time)
{
        
}

static void print(const struct ebt_u_entry *entry,
   const struct ebt_entry_match *match)
{
	struct ebt_cipfw_info *ipinfo = (struct ebt_cipfw_info *)match->data;
	int j;
	
	if (ipinfo->bitmask & EBT_CIPFW){ 
		printf("--cipfw ");
		if (ipinfo->invflags & EBT_CIPFW)
			printf("! ");
		if(ipinfo->tos ==CIPFW_DHCP)
		printf("--cipfw-dhcp ");
		else if(ipinfo->tos ==CIPFW_ARP)
		printf("--cipfw-arp ");
	}
}
/*
static int compare(const struct ebt_entry_match *m1,
   const struct ebt_entry_match *m2)
{
	return 1;
}
*/
static struct ebt_u_match cipfw_match =
{
	.name		= EBT_CIPFW_MATCH,
	.size		= sizeof(struct ebt_cipfw_info),
	.help		= print_help,
	.init		= init,
	.parse		= parse,
	.final_check	= final_check,
	.print		= print,
	//.compare	= compare,
	.extra_ops	= opts,
};

void _init(void)
{
	ebt_register_match(&cipfw_match);
}
