/*
 * hostapd cipcfg extension -- used to save new AP settings
 * Copyright (c) 2002-2009, Dex Chen <dex@cipherium.com.tw>
 *
 * $Id: hostapd_cipcfg.c,v 1.1 2009/06/04 09:22:07 dex Exp $
 */

#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include "wps_defs.h"
#include "debug.h"
#include "cipcfg.h"

extern const int gid_vap[][MAX_VAP];
extern const int gid_vap_sec[][MAX_VAP];
extern const char *current_config;

#define N(arr)	sizeof((arr))/sizeof((arr)[0])
#define WPA_GET_BE16(a) ((unsigned short) (((a)[0] << 8) | (a)[1]))

#define MAX_SSID_LEN	32
#define MAX_KEY_LEN	64
struct wps_cred {
    char ssid[MAX_SSID_LEN + 1];
    uint16_t auth_type;
    uint16_t encr_type;
    uint8_t key_idx;
    uint8_t key[MAX_KEY_LEN + 1];
    uint16_t key_len;
};

static struct {
	int len;
	char *method;
	char *keylen;
} wep_key_map[] = {
	{5, 	"ascii",	"64"},
	{10, 	"hex",		"64"},
	{13, 	"ascii",	"128"},
	{26, 	"hex",		"128"},
	{16, 	"ascii",	"152"},
	{32, 	"hex",		"152"},
};

static int ctoi(char c)
{
    if (c >= '0' && c <= '9')
	return (c - '0');
    c = toupper(c);
    if (c >= 'A' && c <= 'F')
	return (c - 'A' + 10);
    return 0;
}

static int str2hex(uint8_t *str, uint8_t *buf, int len)
{
    int i, size, hex_size;

    size = strlen(str);
    if ((size % 2) != 0) {
	dbg_printf("The size of hex string is invalid.\n");
	return -1;
    }
    hex_size = (size / 2);
    if (hex_size > len) {
	dbg_printf("The buffer size is too small, it need %d.\n", hex_size);
	return -1;
    }

    for (i = 0; i < hex_size; i++)
	buf[i] = (ctoi(str[i*2]) << 4) + (ctoi(str[i*2+1]));

    return hex_size;
}

static int parse_wps_msg(unsigned char *buf, int size, struct wps_cred *cred)
{
    const unsigned char *pos, *end;
    unsigned short type, len;
    
    pos = buf;
    end = buf + size;
    while (pos < end) {
	if (end - pos < 4) {
	    dbg_printf("Invalid WPS message, %lu bytes remaining.\n",
		    (unsigned long)(end - pos));
	    return -1;
	}
	type = WPA_GET_BE16(pos);
	pos += 2;
	len = WPA_GET_BE16(pos);
	pos += 2;
	dbg_printf("WPS attr type=0x%x len=%u\n", type, len);
	if (len > (end - pos)) {
	    dbg_printf("WPS attribute overflow.\n");
	    return -1;
	}
	switch (type) {
	    case ATTR_SSID:
		if (len > MAX_SSID_LEN) {
		    dbg_printf("WPS ATTR_SSID lengh=%d is too long.\n", len);
		    return -1;
		}
		strncpy(cred->ssid, pos, len);
		cred->ssid[len] = '\0';
		break;
	    case ATTR_AUTH_TYPE:
		if (len != 2) {
		    dbg_printf("WPS invalid Authentication Type length=%d.\n", 
			    len);
		    return -1;
		}
		cred->auth_type = WPA_GET_BE16(pos);
		break;
	    case ATTR_ENCR_TYPE:
		if (len != 2) {
		    dbg_printf("WPS invalid Encryption Type length=%d.\n", 
			    len);
		    return -1;
		}
		cred->encr_type = WPA_GET_BE16(pos);
		break;
	    case ATTR_NETWORK_KEY_INDEX:
		if (len != 1) {
		    dbg_printf("WPS invalid Key Index length=%d.\n", 
			    len);
		    return -1;
		}
		cred->key_idx = *pos;
		break;
	    case 0x1027:	// ATTR_NETWORK_KEY
		if (len > MAX_KEY_LEN) {
		    dbg_printf("WPS invalid Key length=%d.\n", len);
		    return -1;
		}
		memcpy(cred->key, pos, len);
		cred->key_len = len;
		break;
	    default:
		break;
	}
	pos += len;
    }
    return 0;
}

static ITEM_LIST *append_wep_cfg_items(ITEM_LIST *list, int gid, 
	struct wps_cred *cred)
{
    char *method = wep_key_map[0].method, *keylen = wep_key_map[0].keylen;
    char txidx[8], key_item[16], key[MAX_VALUE_LEN];
    uint8_t key_idx = 0;
    int i;

    // method & keylen
    for (i = 0; i < N(wep_key_map); i++) {
	if (cred->key_len == wep_key_map[i].len) {
	    method = wep_key_map[i].method;
	    keylen = wep_key_map[i].keylen;
	    break;
	}
    }

    // txidx & keyN
    if (cred->key_idx >= 1 && cred->key_idx <= 4)
	key_idx = cred->key_idx;
    sprintf(txidx, "%d", key_idx);
    sprintf(key_item, "wep.key%d", key_idx-1);
    memcpy(key, cred->key, cred->key_len);
    key[cred->key_len] = '\0';

    list = append_cfg_item(list, gid, "wep.enabled", "enable");
    list = append_cfg_item(list, gid, "wep.method", method);
    list = append_cfg_item(list, gid, "wep.keylen", keylen);
    list = append_cfg_item(list, gid, "wep.txidx", txidx);
    list = append_cfg_item(list, gid, key_item, key);

    return list;
}

static int save_setting_with_cipcfg(int rfidx, int vapidx, 
				    struct wps_cred *cred)
{
#define AUTH_TYPE(c)	((c)->auth_type & WPS_AUTH_TYPES)
#define ENCR_TYPE(c)	((c)->encr_type & WPS_ENCR_TYPES)
    CIPCFG *cipcfg = NULL;
    ITEM_LIST *list = NULL;
    char group_name[MAX_VALUE_LEN];
    char enabled[MAX_VALUE_LEN];
    char configurable[MAX_VALUE_LEN];
    char *mode[6] = {"open", "shared", "8021x", "wpa", "auto", "mac"};
    char *eap[2] = {"psk", "eap"};
    char *version[3] = {"1", "2", "1:2"};
    char *enc[3] = {"rc4-tkip", "aes-ccmp", "mixed"};
    char value[MAX_VALUE_LEN];
    unsigned vapgid, secgid;
    uint16_t auth_type, encr_type;
    char *wpa_encr = enc[2];

    vapgid = gid_vap[rfidx][vapidx];
    secgid = gid_vap_sec[rfidx][vapidx];


    /* Check if vap support WPS AP configure */
    cipcfg = init_cipcfg_f(current_config);
    sprintf(group_name, "rf%d_vap%d_sec", rfidx+1, vapidx+1);
    get_cfg_item_by_str(cipcfg, group_name, "wps.enabled", enabled);
    get_cfg_item_by_str(cipcfg, group_name, "wps.configure", configurable);
    if (strcmp(enabled, "enable") != 0 || strcmp(configurable, "enable") != 0) {
	dbg_printf("rf%d_vap%d doesn't support WPS AP configre, skip it.\n",
		rfidx+1, vapidx+1);
	close_cipcfg(cipcfg);
	return -1;
    }
    close_cipcfg(cipcfg);

    cipcfg = init_cipcfg();

    /* Check Auth-type and Encr-type */
    if (!(auth_type = AUTH_TYPE(cred))) {
	dbg_printf("WPS-CIPCFG: auth_type is invalid\n");
	goto fail;
    }
    if (!(encr_type = ENCR_TYPE(cred))) {
	dbg_printf("WPS-CIPCFG: auth_type is invalid\n");
	goto fail;
    }

    /* Set SSID */
    list = append_cfg_item(list, vapgid, "ssid", cred->ssid);

    /* Set Security */
    switch (auth_type) {
	case (WPS_AUTH_OPEN | WPS_AUTH_SHARED):
	    list = append_cfg_item(list, secgid, "mode", mode[4]);
	    list = append_wep_cfg_items(list, secgid, cred);
	    break;
	case WPS_AUTH_OPEN:
	    list = append_cfg_item(list, secgid, "mode", mode[0]);
	    if (encr_type & WPS_ENCR_WEP)
		list = append_wep_cfg_items(list, secgid, cred);
	    else
		list = append_cfg_item(list, secgid, "wep.enabled", "disable");
	    break;
	case WPS_AUTH_SHARED:
	    list = append_cfg_item(list, secgid, "mode", mode[1]);
	    list = append_wep_cfg_items(list, secgid, cred);
	    break;
	case WPS_AUTH_WPAPSK:
	case WPS_AUTH_WPA2PSK:
	case (WPS_AUTH_WPAPSK | WPS_AUTH_WPA2PSK) :
	    // mode & eap
	    list = append_cfg_item(list, secgid, "mode", mode[3]);
	    list = append_cfg_item(list, secgid, "wpa.eap", eap[0]);
	    // WPA version
	    list = append_cfg_item(list, secgid, "wpa.version", 
		    (auth_type == (WPS_AUTH_WPAPSK|WPS_AUTH_WPA2PSK) ? 
		     version[2] : (auth_type == WPS_AUTH_WPAPSK ? 
			 version[0] : version[1])));
	    // Encrption
	    if (encr_type == (WPS_ENCR_TKIP | WPS_ENCR_AES))
		wpa_encr = enc[2];
	    else if (encr_type == WPS_ENCR_TKIP)
		wpa_encr = enc[0];
	    else if (encr_type == WPS_ENCR_AES)
		wpa_encr = enc[1];
	    list = append_cfg_item(list, secgid, "wpa.enc", wpa_encr);
	    // Key type and content
	    list = append_cfg_item(list, secgid, "wpa.type", 
		    (cred->key_len == 64 ?  "psk": "passphrase"));
	    memcpy(value, cred->key, cred->key_len);
	    value[cred->key_len] = '\0';
	    list = append_cfg_item(list, secgid, "wpa.passphrase", value);
	default:
	    dbg_printf("Unknwon authentication type: 0x%x.\n", auth_type);
    }
    /* Set AP to non-configrable */
    list = append_cfg_item(list, secgid, "wps.configure", "disable");

    /* Set cipcfg */
    if (set_cfg_items_lite(cipcfg, list) != CIPCFG_SUCCESS) {
	dbg_printf("Can't set cipcfg.\n");
	goto fail;
    }

    close_cipcfg(cipcfg);
    return 0;
fail:
    close_cipcfg(cipcfg);
    return -1;
}

int hostapd_cipcfg_ap_setting(int idx, char *str)
{
    unsigned char buf[BUFSIZ];
    int size = 0;
    struct wps_cred cred;

    dbg_printf("Cred buffer=>%s\n", str);
    if ((size = str2hex(str, buf, BUFSIZ)) <= 0) {
	dbg_printf("Can't convert cred to binray format from string.\n");
	return -1;
    }
    if (parse_wps_msg(buf, size, &cred) < 0) {
	dbg_printf("Process WPS message error.\n");
	return -1;
    }
    if (save_setting_with_cipcfg(0, idx, &cred) < 0) {
	dbg_printf("Can't save new AP setting in cipcfg.\n");
	return -1;
    }
    return 0;
}
