/*
 * hostapd wrapper program
 * Copyright (c) 2002-2009, Dex Chen <dex@cipherium.com.tw>
 *
 * $Id$
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "debug.h"

extern int hostapd_wps_monitor(void);

#define HOSTAPD_CMD	"/sbin/hostapd"
#define MAXRETRY	10

int enable_debug = 0;

char **hostapd_argv = NULL;
int count = 0;

/*
 * Run hostapd in the child process and record it's pid
 */
static void run_hostapd(char *argv[])
{
    pid_t pid;

    pid = fork();
    if (pid == 0)
	execv(HOSTAPD_CMD, argv);
    dbg_printf("Child procces ID: %d\n", pid);
}

/*
 * We try to run hostapd when it terminated abnormally 
 */
static void chd_handler(int signum, siginfo_t *info, void *context)
{
    pid_t pid;
    int status;

    pid = wait(&status);
    dbg_printf("Child process %d terminated\n", pid);
    if (info->si_code == CLD_DUMPED) {
	/* Segmentation fault */
	dbg_printf("Child process segmentation fault\n");
	run_hostapd(hostapd_argv);
    } else if (info->si_code == CLD_EXITED && WEXITSTATUS(status) != 0) {
	/* Exit failure */
	if (count++ < MAXRETRY) {
	    dbg_printf("Retry to run hostapd retry=%d\n", count);
	    run_hostapd(hostapd_argv);
	} else {
	    dbg_printf("Exceed retry count %d, exit program\n", MAXRETRY);
	    exit(1);
	}
    } else {
	dbg_printf("Exit normally\n");
	exit(0);
    }
}

int main(int argc, char *argv[])
{
    int i;
    struct sigaction sig;
    char *debug = NULL;

    /* Check debug environment variable */
    debug = getenv("DEBUG");
    if (debug) {
	if (strcmp(debug, "yes") == 0 || strcmp(debug, "1") == 0)
	    enable_debug = 1;
    }

    /* Duplicated argument list */
    hostapd_argv = (char **)calloc(argc + 1, sizeof(char *));
    hostapd_argv[0] = strdup(HOSTAPD_CMD);
    for (i = 1; i < argc; i++)
	hostapd_argv[i] = strdup(argv[i]);
    hostapd_argv[argc] = NULL;

    /* Add SIGCHILD signal handler */
    sig.sa_sigaction = chd_handler;
    sig.sa_flags |= SA_SIGINFO;
    sigaction(SIGCHLD, &sig, NULL);

    /* Run hostapd */
    run_hostapd(hostapd_argv);

    /* Wait for hostapd to setup */
    sleep(3);

    /* Run event monitor */
    if (hostapd_event_monitor() == 1) {
	dbg_printf("Restarting WLAN for new AP setting...\n");
	system("/etc/rc.d/rc.wlan restart 0");
	system("/etc/rc.d/rc.wlan start_hostapd");
    }

    /* Free argument list */
    for (i = 0; i < argc; i++)
	free(hostapd_argv[i]);
    free(hostapd_argv);

    return 0;
}
