/*
 *  Atheros AP91 board support
 *
 *  Copyright (C) 2010 Dex Chen <dex.chen@cipherium.com.tw>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 */

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>

#include <asm/mach-ar71xx/ar71xx.h>

#include "machtype.h"
#include "devices.h"
#include "dev-m25p80.h"
#include "dev-ap91-eth.h"
#include "dev-ap91-pci.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"

#define AP91_MAC_ADDR_OFFSET		0x0
#define AP91_CALDATA_OFFSET		0x1000

#define AP91_GPIO_BTN_RESTART		0
#define AP91_GPIO_LED_SYS		11

#define AP91_BUTTONS_POLL_INTERVAL	20

static struct flash_platform_data ap91_flash_data = {
        .name		= "ar7240-nor0",
#ifdef CONFIG_MTD_PARTITIONS
        //.parts          = ap91_partitions,
        //.nr_parts       = ARRAY_SIZE(ap91_partitions),
#endif
};

static struct gpio_button ap91_gpio_buttons[] __initdata = {
	{
		.desc		= "restart",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.threshold	= 3,
		.gpio		= AP91_GPIO_BTN_RESTART,
		.active_low	= 1,
	}
};

static struct gpio_led ap91_leds_gpio[] __initdata = {
	{
		.name		= "ap91:red:sys",
		.gpio		= AP91_GPIO_LED_SYS,
		.active_low	= 1,
	}
};

static void __init ap91_setup(void)
{
	u8 *art = (u8 *) KSEG1ADDR(0x1fff0000);
	u8 *ee = art + AP91_CALDATA_OFFSET;
	u8 mac[6];

	ar71xx_add_device_m25p80(&ap91_flash_data);

	memcpy(mac, art + AP91_MAC_ADDR_OFFSET, 6);

	ar71xx_eth1_data.has_ar7240_switch = 1; 
	ar71xx_set_mac_base(mac); 

	/* WAN port */ 
	ar71xx_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_RMII; 
	ar71xx_eth0_data.speed = SPEED_1000; 
	ar71xx_eth0_data.duplex = DUPLEX_FULL; 

	/* LAN ports */ 
	ar71xx_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_RMII; 
	ar71xx_eth1_data.speed = SPEED_1000; 
	ar71xx_eth1_data.duplex = DUPLEX_FULL; 

	ar71xx_add_device_mdio(0x0); 
	ar71xx_add_device_eth(0); 
	ar71xx_add_device_eth(1); 

	ap91_pci_init(ee, NULL);

	ar71xx_add_device_gpio_buttons(-1, AP91_BUTTONS_POLL_INTERVAL,
					ARRAY_SIZE(ap91_gpio_buttons),
					ap91_gpio_buttons);

	ar71xx_add_device_leds_gpio(-1, ARRAY_SIZE(ap91_leds_gpio),
					ap91_leds_gpio);

}

MIPS_MACHINE(AR71XX_MACH_AP91, "AP91", "Atheros AP91", ap91_setup);
