/*
 *  LanReady GN526 board support
 *
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 */

#include <linux/platform_device.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/delay.h>
#include <linux/rtl8367r.h>

#include <asm/mach-ar71xx/ar71xx.h>

#include "machtype.h"
#include "devices.h"
#include "dev-leds-gpio.h"

#define GN526_GPIO_LED_STATUS	2

#define GN526_GPIO_RTL8367_WAN_SDA	3
#define GN526_GPIO_RTL8367_WAN_SCK	1

#define GN526_GPIO_RTL8367_LAN_SDA	11
#define GN526_GPIO_RTL8367_LAN_SCK	6

static struct gpio_led gn526_leds_gpio[] __initdata = {
	{
		.name		= "gn526:status",
		.gpio		= GN526_GPIO_LED_STATUS,
		.active_low	= 0,
	}
};

static struct rtl8367r_platform_data gn526_rtl8367r_wan_data = {
	.gpio_sda	= GN526_GPIO_RTL8367_WAN_SDA,
	.gpio_sck	= GN526_GPIO_RTL8367_WAN_SCK,
};

static struct rtl8367r_platform_data gn526_rtl8367r_lan_data = {
	.gpio_sda	= GN526_GPIO_RTL8367_LAN_SDA,
	.gpio_sck	= GN526_GPIO_RTL8367_LAN_SCK,
};

static struct platform_device gn526_rtl8367r_wan_device = {
	.name		= RTL8367R_DRIVER_NAME,
	.id		= 0,
	.dev = {
		.platform_data = &gn526_rtl8367r_wan_data,
	}
};

static struct platform_device gn526_rtl8367r_lan_device = {
	.name		= RTL8367R_DRIVER_NAME,
	.id		= 1,
	.dev = {
		.platform_data = &gn526_rtl8367r_lan_data,
	}
};


static void __init gn526_setup(void)
{
	ar71xx_add_device_mdio(0x0);

	ar71xx_eth0_pll_data.pll_1000 = 0x11110000;
	//ar71xx_eth0_data.mii_bus_dev = &gn526_rtl8367r_wan_device.dev;
	ar71xx_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_RGMII;
	ar71xx_eth0_data.speed = SPEED_1000;
	ar71xx_eth0_data.duplex = DUPLEX_FULL;

	ar71xx_eth1_pll_data.pll_1000 = 0x11110000;
	//ar71xx_eth1_data.mii_bus_dev = &gn526_rtl8367r_lan_device.dev;
	ar71xx_eth1_data.phy_if_mode = PHY_INTERFACE_MODE_RGMII;
	ar71xx_eth1_data.speed = SPEED_1000;
	ar71xx_eth1_data.duplex = DUPLEX_FULL;

	ar71xx_add_device_eth(0);
	ar71xx_add_device_eth(1);

	ar71xx_add_device_leds_gpio(-1, ARRAY_SIZE(gn526_leds_gpio),
				    gn526_leds_gpio);

	platform_device_register(&gn526_rtl8367r_wan_device);
	platform_device_register(&gn526_rtl8367r_lan_device);

}

MIPS_MACHINE(AR71XX_MACH_GN526, "GN526", "LANREADY GN526",
	     gn526_setup);
