#ifndef _AR71XX_H
#define _AR71XX_H

#include <asm/addrspace.h>

typedef unsigned int ar71xx_reg_t;

#define ar71xx_reg_rd(_phys)    (*(volatile ar71xx_reg_t *)KSEG1ADDR(_phys))
#define ar71xx_reg_wr_nf(_phys, _val) \
                    ((*(volatile ar71xx_reg_t *)KSEG1ADDR(_phys)) = (_val))

#define ar71xx_reg_wr(_phys, _val) do {     \
         ar71xx_reg_wr_nf(_phys, _val);     \
         ar71xx_reg_rd(_phys);       \
}while(0);

#define ar71xx_reg_rmw_set(_reg, _mask)  do {                        \
    ar71xx_reg_wr((_reg), (ar71xx_reg_rd((_reg)) | (_mask)));      \
    ar71xx_reg_rd((_reg));                                           \
}while(0);

#define ar71xx_reg_rmw_clear(_reg, _mask)  do {                        \
    ar71xx_reg_wr((_reg), (ar71xx_reg_rd((_reg)) & ~(_mask)));      \
    ar71xx_reg_rd((_reg));                                           \
}while(0);

/*
 * Address map
 */
#ifndef CONFIG_AR9100
#define AR71XX_PCI_MEM_BASE             0x10000000  /* 128M */
#endif

#define AR71XX_APB_BASE                 0x18000000  /* 384M */
#define AR71XX_GE0_BASE                 0x19000000  /* 16M */
#define AR71XX_GE1_BASE                 0x1a000000  /* 16M */
#define AR71XX_USB_EHCI_BASE            0x1b000000  
#define AR71XX_USB_OHCI_BASE            0x1c000000
#define AR71XX_SPI_BASE                 0x1f000000

/*
 * APB block
 */
#define AR71XX_DDR_CTL_BASE             AR71XX_APB_BASE+0x00000000
//#define AR71XX_CPU_BASE                 AR71XX_APB_BASE+0x00010000
//#define AR71XX_UART_BASE                AR71XX_APB_BASE+0x00020000
#define AR71XX_USB_CONFIG_BASE          AR71XX_APB_BASE+0x00030000
//#define AR71XX_GPIO_BASE                AR71XX_APB_BASE+0x00040000
//#define AR71XX_PLL_BASE                 AR71XX_APB_BASE+0x00050000
//#define AR71XX_RESET_BASE               AR71XX_APB_BASE+0x00060000
//#define AR71XX_SLIC_BASE                AR71XX_APB_BASE+0x00090000
//#define AR71XX_DMA_BASE                 AR71XX_APB_BASE+0x000A0000
//#define AR71XX_STEREO_BASE              AR71XX_APB_BASE+0x000B0000


#ifdef CONFIG_AR9100
#define AR9100_WMAC_BASE                AR71XX_APB_BASE+0x000c0000
#define AR9100_WMAC_LEN                 0x4000
#define AR9100_FLASH_BASE               AR71XX_APB_BASE+0x000F0000
#define AR9100_FLASH_CONFIG             AR9100_FLASH_BASE+0x4
#endif



/*
 * DDR block
 */
#define AR71XX_DDR_CONFIG               AR71XX_DDR_CTL_BASE+0
#define AR71XX_DDR_CONFIG2              AR71XX_DDR_CTL_BASE+4
#define AR71XX_DDR_MODE                 AR71XX_DDR_CTL_BASE+0x08
#define AR71XX_DDR_EXT_MODE             AR71XX_DDR_CTL_BASE+0x0c
#define AR71XX_DDR_CONTROL              AR71XX_DDR_CTL_BASE+0x10
#define AR71XX_DDR_REFRESH              AR71XX_DDR_CTL_BASE+0x14
#define AR71XX_DDR_RD_DATA_THIS_CYCLE   AR71XX_DDR_CTL_BASE+0x18
#define AR71XX_DDR_TAP_CONTROL0         AR71XX_DDR_CTL_BASE+0x1c
#define AR71XX_DDR_TAP_CONTROL1         AR71XX_DDR_CTL_BASE+0x20
#define AR71XX_DDR_TAP_CONTROL2         AR71XX_DDR_CTL_BASE+0x24
#define AR71XX_DDR_TAP_CONTROL3         AR71XX_DDR_CTL_BASE+0x28

/*
 * DDR Config values
 */
#define AR71XX_DDR_CONFIG_16BIT             (1 << 31)
#define AR71XX_DDR_CONFIG_PAGE_OPEN         (1 << 30)
#define AR71XX_DDR_CONFIG_CAS_LAT_SHIFT      27
#define AR71XX_DDR_CONFIG_TMRD_SHIFT         23
#define AR71XX_DDR_CONFIG_TRFC_SHIFT         17
#define AR71XX_DDR_CONFIG_TRRD_SHIFT         13
#define AR71XX_DDR_CONFIG_TRP_SHIFT          9
#define AR71XX_DDR_CONFIG_TRCD_SHIFT         5
#define AR71XX_DDR_CONFIG_TRAS_SHIFT         0

#define AR71XX_DDR_CONFIG2_BL2          (2 << 0)
#define AR71XX_DDR_CONFIG2_BL4          (4 << 0)
#define AR71XX_DDR_CONFIG2_BL8          (8 << 0)

#define AR71XX_DDR_CONFIG2_BT_IL        (1 << 4)       
#define AR71XX_DDR_CONFIG2_CNTL_OE_EN   (1 << 5)
#define AR71XX_DDR_CONFIG2_PHASE_SEL    (1 << 6)
#define AR71XX_DDR_CONFIG2_DRAM_CKE     (1 << 7)
#define AR71XX_DDR_CONFIG2_TWR_SHIFT    8
#define AR71XX_DDR_CONFIG2_TRTW_SHIFT   12
#define AR71XX_DDR_CONFIG2_TRTP_SHIFT   17
#define AR71XX_DDR_CONFIG2_TWTR_SHIFT   21
#define AR71XX_DDR_CONFIG2_HALF_WIDTH_L (1 << 31)   

#define AR71XX_DDR_TAP_DEFAULT          0x18

/*
 * DDR block, gmac flushing 
 */
#ifndef CONFIG_AR9100

#define AR71XX_DDR_GE0_FLUSH            AR71XX_DDR_CTL_BASE+0x9c
#define AR71XX_DDR_GE1_FLUSH            AR71XX_DDR_CTL_BASE+0xa0
#define AR71XX_DDR_USB_FLUSH            AR71XX_DDR_CTL_BASE+0xa4
#define AR71XX_DDR_PCI_FLUSH            AR71XX_DDR_CTL_BASE+0xa8

#else

#define AR71XX_DDR_GE0_FLUSH            AR71XX_DDR_CTL_BASE+0x7c
#define AR71XX_DDR_GE1_FLUSH            AR71XX_DDR_CTL_BASE+0x80
#define AR71XX_DDR_USB_FLUSH            AR71XX_DDR_CTL_BASE+0x84
#define AR71XX_DDR_WMAC_FLUSH           AR71XX_DDR_CTL_BASE+0x88
#endif


#define AR71XX_EEPROM_GE0_MAC_ADDR      0xbfff1000
#define AR71XX_EEPROM_GE1_MAC_ADDR      0xbfff1006

/*
 * PLL block/CPU
 */

#define AR71XX_PLL_CONFIG               AR71XX_PLL_BASE+0x0

#ifndef CONFIG_AR9100

#define AR71XX_USB_PLL_CONFIG           AR71XX_PLL_BASE+0x4

/* Legacy */
#define AR71XX_USB_PLL_CONFIG           AR71XX_PLL_BASE+0x4
#define AR71XX_USB_PLL_GE0_OFFSET       AR71XX_PLL_BASE+0x10
#define AR71XX_USB_PLL_GE1_OFFSET       AR71XX_PLL_BASE+0x14

/* From Data Sheet */
#define AR71XX_CONFIG_SEC_PLL           AR71XX_PLL_BASE+0x4
#define AR71XX_CONFIG_ETH_INT0_CLOCK    AR71XX_PLL_BASE+0x10
#define AR71XX_CONFIG_ETH_INT1_CLOCK    AR71XX_PLL_BASE+0x14
#define AR71XX_CONFIG_ETH_EXT_CLOCK     AR71XX_PLL_BASE+0x18
#define AR71XX_CONFIG_PCI_CLOCK         AR71XX_PLL_BASE+0x1C

/* #define AR71XX_USB */

#define PLL_CONFIG_PLL_POWER_DOWN_MASK  (1 << 0)
#define PLL_CONFIG_PLL_BYPASS_MASK      (1 << 1)
#define PLL_CONFIG_PLL_FB_SHIFT         3
#define PLL_CONFIG_PLL_FB_MASK          (0x1f << PLL_CONFIG_PLL_FB_SHIFT)
#define PLL_CONFIG_PLL_LOOP_BW_SHIFT    12
#define PLL_CONFIG_PLL_LOOP_BW_MASK     (0xf << PLL_CONFIG_PLL_LOOP_BW_SHIFT)
#define PLL_CONFIG_CPU_DIV_SHIFT        16
#define PLL_CONFIG_CPU_DIV_MASK         (3 << PLL_CONFIG_CPU_DIV_SHIFT)
#define PLL_CONFIG_DDR_DIV_SHIFT        18
#define PLL_CONFIG_DDR_DIV_MASK         (3 << PLL_CONFIG_DDR_DIV_SHIFT)
#define PLL_CONFIG_AHB_DIV_SHIFT        20
#define PLL_CONFIG_AHB_DIV_MASK         (7 << PLL_CONFIG_AHB_DIV_SHIFT)
#define PLL_CONFIG_LOCKED_SHIFT         30
#define PLL_CONFIG_LOCKED_MASK          (1 << PLL_CONFIG_LOCKED_SHIFT)
#define PLL_CONFIG_SW_UPDATE_SHIFT      31
#define PLL_CONFIG_SW_UPDATE_MASK       (1 << 31)

/*
 * CPU Clock
 */
#define AR71XX_CPU_CLOCK_CONTROL        AR71XX_PLL_BASE+8
#define CLOCK_CONTROL_CLOCK_SWITCH_SHIFT  0
#define CLOCK_CONTROL_CLOCK_SWITCH_MASK  (1 << CLOCK_CONTROL_CLOCK_SWITCH_SHIFT)
#define CLOCK_CONTROL_RST_SWITCH_SHIFT    1
#define CLOCK_CONTROL_RST_SWITCH_MASK    (1 << CLOCK_CONTROL_RST_SWITCH_SHIFT)

#define PLL_DIV_SHIFT   3
#define PLL_DIV_MASK    0x1f
#define CPU_DIV_SHIFT   16
#define CPU_DIV_MASK    0x3
#define DDR_DIV_SHIFT   18
#define DDR_DIV_MASK    0x3
#define AHB_DIV_SHIFT   20
#define AHB_DIV_MASK    0x7

#else

#define PLL_DIV_SHIFT   0
#define PLL_DIV_MASK    0x3ff
#define DDR_DIV_SHIFT   22
#define DDR_DIV_MASK    0x3
#define AHB_DIV_SHIFT   19
#define AHB_DIV_MASK    0x1
#define AR9100_ETH_PLL_CONFIG           AR71XX_PLL_BASE+0x4

#define AR9100_ETH_INT0_CLK             AR71XX_PLL_BASE+0x14
#define AR9100_ETH_INT1_CLK             AR71XX_PLL_BASE+0x18

#endif

/*
 * USB block
 */
#define AR71XX_USB_FLADJ_VAL            AR71XX_USB_CONFIG_BASE
#define AR71XX_USB_CONFIG               AR71XX_USB_CONFIG_BASE+0x4
#define AR71XX_USB_WINDOW               0x1000000

#ifndef CONFIG_AR9100
/*
 * PCI block
 */
#define AR71XX_PCI_WINDOW           0x8000000       /* 128MB */
#define AR71XX_PCI_WINDOW0_OFFSET   AR71XX_DDR_CTL_BASE+0x7c
#define AR71XX_PCI_WINDOW1_OFFSET   AR71XX_DDR_CTL_BASE+0x80
#define AR71XX_PCI_WINDOW2_OFFSET   AR71XX_DDR_CTL_BASE+0x84
#define AR71XX_PCI_WINDOW3_OFFSET   AR71XX_DDR_CTL_BASE+0x88
#define AR71XX_PCI_WINDOW4_OFFSET   AR71XX_DDR_CTL_BASE+0x8c
#define AR71XX_PCI_WINDOW5_OFFSET   AR71XX_DDR_CTL_BASE+0x90
#define AR71XX_PCI_WINDOW6_OFFSET   AR71XX_DDR_CTL_BASE+0x94
#define AR71XX_PCI_WINDOW7_OFFSET   AR71XX_DDR_CTL_BASE+0x98

#define AR71XX_PCI_WINDOW0_VAL      0x10000000
#define AR71XX_PCI_WINDOW1_VAL      0x11000000
#define AR71XX_PCI_WINDOW2_VAL      0x12000000
#define AR71XX_PCI_WINDOW3_VAL      0x13000000
#define AR71XX_PCI_WINDOW4_VAL      0x14000000
#define AR71XX_PCI_WINDOW5_VAL      0x15000000
#define AR71XX_PCI_WINDOW6_VAL      0x16000000
#define AR71XX_PCI_WINDOW7_VAL      0x07000000

#define ar71xx_write_pci_window(_no)             \
  ar71xx_reg_wr(AR71XX_PCI_WINDOW##_no##_OFFSET, AR71XX_PCI_WINDOW##_no##_VAL);
  
/*
 * CRP. To access the host controller config and status registers
 */
#define AR71XX_PCI_CRP   (AR71XX_PCI_MEM_BASE|(AR71XX_PCI_WINDOW7_VAL+0x10000))

#define AR71XX_PCI_CRP_AD_CBE            AR71XX_PCI_CRP
#define AR71XX_PCI_CRP_WRDATA            AR71XX_PCI_CRP+0x4
#define AR71XX_PCI_CRP_RDDATA            AR71XX_PCI_CRP+0x8
#define AR71XX_PCI_ERROR                 AR71XX_PCI_CRP+0x1c
#define AR71XX_PCI_ERROR_ADDRESS         AR71XX_PCI_CRP+0x20
#define AR71XX_PCI_AHB_ERROR             AR71XX_PCI_CRP+0x24
#define AR71XX_PCI_AHB_ERROR_ADDRESS     AR71XX_PCI_CRP+0x28

#define AR71XX_CRP_CMD_WRITE             0x00010000
#define AR71XX_CRP_CMD_READ              0x00000000

/*
 * PCI CFG. To generate config cycles
 */
#define AR71XX_PCI_CFG_AD           AR71XX_PCI_CRP+0xc
#define AR71XX_PCI_CFG_CBE          AR71XX_PCI_CRP+0x10
#define AR71XX_PCI_CFG_WRDATA       AR71XX_PCI_CRP+0x14
#define AR71XX_PCI_CFG_RDDATA       AR71XX_PCI_CRP+0x18
#define AR71XX_CFG_CMD_READ         0x0000000a
#define AR71XX_CFG_CMD_WRITE        0x0000000b

#define AR71XX_PCI_IDSEL_ADLINE_START           17

#endif /* ifndef ar9100 */

/*
 * gpio configs
 */
#define AR71XX_GPIO_OE                  AR71XX_GPIO_BASE+0x0
#define AR71XX_GPIO_IN                  AR71XX_GPIO_BASE+0x4
#define AR71XX_GPIO_OUT                 AR71XX_GPIO_BASE+0x8
#define AR71XX_GPIO_SET                 AR71XX_GPIO_BASE+0xc
#define AR71XX_GPIO_CLEAR               AR71XX_GPIO_BASE+0x10
#define AR71XX_GPIO_INT_ENABLE          AR71XX_GPIO_BASE+0x14
#define AR71XX_GPIO_INT_TYPE            AR71XX_GPIO_BASE+0x18
#define AR71XX_GPIO_INT_POLARITY        AR71XX_GPIO_BASE+0x1c
#define AR71XX_GPIO_INT_PENDING         AR71XX_GPIO_BASE+0x20
#define AR71XX_GPIO_INT_MASK            AR71XX_GPIO_BASE+0x24
#define AR71XX_GPIO_FUNCTIONS           AR71XX_GPIO_BASE+0x28

/*
 * IRQ Map.
 * There are 4 conceptual ICs in the system. We generally give a block of 16
 * irqs to each IC.
 * CPU:                     0    - 0xf
 *      MISC:               0x10 - 0x1f
 *          GPIO:           0x20 - 0x2f
 *      PCI :               0x30 - 0x40
 * 
 */
//#define AR71XX_CPU_IRQ_BASE         0x00
//#define AR71XX_MISC_IRQ_BASE        0x10
//#define AR71XX_GPIO_IRQ_BASE        0x20

//#ifndef CONFIG_AR9100
//#define AR71XX_PCI_IRQ_BASE         0x30
//#endif

/*
 * The IPs. Connected to CPU (hardware IP's; the first two are software)
 */
#ifndef CONFIG_AR9100
#define AR71XX_CPU_IRQ_PCI                  AR71XX_CPU_IRQ_BASE+2
#else
#define AR71XX_CPU_IRQ_WMAC                 AR71XX_CPU_IRQ_BASE+2
#endif

//#define AR71XX_CPU_IRQ_USB                  AR71XX_CPU_IRQ_BASE+3
//#define AR71XX_CPU_IRQ_GE0                  AR71XX_CPU_IRQ_BASE+4
//#define AR71XX_CPU_IRQ_GE1                  AR71XX_CPU_IRQ_BASE+5
//#define AR71XX_CPU_IRQ_MISC                 AR71XX_CPU_IRQ_BASE+6
//#define AR71XX_CPU_IRQ_TIMER                AR71XX_CPU_IRQ_BASE+7

/*
 * Interrupts connected to the CPU->Misc line.
 */
//#define AR71XX_MISC_IRQ_TIMER               AR71XX_MISC_IRQ_BASE+0
//#define AR71XX_MISC_IRQ_ERROR               AR71XX_MISC_IRQ_BASE+1
//#define AR71XX_MISC_IRQ_GPIO                AR71XX_MISC_IRQ_BASE+2
//#define AR71XX_MISC_IRQ_UART                AR71XX_MISC_IRQ_BASE+3
#define AR71XX_MISC_IRQ_WATCHDOG            AR71XX_MISC_IRQ_BASE+4
#define AR71XX_MISC_IRQ_PERF_COUNTER        AR71XX_MISC_IRQ_BASE+5
#define AR71XX_MISC_IRQ_USB_OHCI            AR71XX_MISC_IRQ_BASE+6
//#define AR71XX_MISC_IRQ_DMA                 AR71XX_MISC_IRQ_BASE+7

#define AR71XX_MISC_IRQ_COUNT                 13

#define MIMR_TIMER                          0x01
#define MIMR_ERROR                          0x02
#define MIMR_GPIO                           0x04
#define MIMR_UART                           0x08
#define MIMR_WATCHDOG                       0x10
#define MIMR_PERF_COUNTER                   0x20
#define MIMR_OHCI_USB                       0x40
#define MIMR_DMA                            0x80

#define MISR_TIMER                          MIMR_TIMER
#define MISR_ERROR                          MIMR_ERROR
#define MISR_GPIO                           MIMR_GPIO
#define MISR_UART                           MIMR_UART
#define MISR_WATCHDOG                       MIMR_WATCHDOG
#define MISR_PERF_COUNTER                   MIMR_PERF_COUNTER
#define MISR_OHCI_USB                       MIMR_OHCI_USB
#define MISR_DMA                            MIMR_DMA

/*
 * Interrupts connected to the Misc->GPIO line
 */
#define AR71XX_GPIO_IRQn(_gpio)             AR71XX_GPIO_IRQ_BASE+(_gpio)
//#define AR71XX_GPIO_IRQ_COUNT                 22
void ar71xx_gpio_irq_init(int irq_base);

void ar71xx_misc_enable_irq (unsigned int mask);
void ar71xx_misc_disable_irq (unsigned int mask);

unsigned int ar71xx_misc_get_irq_mask (void);
unsigned int ar71xx_misc_get_irq_status (void);

#ifndef CONFIG_AR9100

/*
 * Interrupts connected to CPU->PCI
 */
//#define AR71XX_PCI_IRQ_DEV0                  AR71XX_PCI_IRQ_BASE+0
//#define AR71XX_PCI_IRQ_DEV1                  AR71XX_PCI_IRQ_BASE+1
//#define AR71XX_PCI_IRQ_DEV2                  AR71XX_PCI_IRQ_BASE+2
//#define AR71XX_PCI_IRQ_CORE                  AR71XX_PCI_IRQ_BASE+3
//#define AR71XX_PCI_IRQ_COUNT                 4

/*
 * PCI interrupt mask and status
 */
#define PIMR_DEV0                           0x01
#define PIMR_DEV1                           0x02
#define PIMR_DEV2                           0x04
#define PIMR_CORE                           0x10

#define PISR_DEV0                           PIMR_DEV0
#define PISR_DEV1                           PIMR_DEV1
#define PISR_DEV2                           PIMR_DEV2
#define PISR_CORE                           PIMR_CORE

void ar71xx_pci_irq_init(int irq_base); /* ??? */

#endif /* ifndef ar9100 */

//#define AR71XX_GPIO_COUNT                   22

/*
 * GPIO Function Enables
 */
#define AR71XX_GPIO_FUNCTION_STEREO_EN       (1<<17)
#define AR71XX_GPIO_FUNCTION_SLIC_EN         (1<<16)
#define AR71XX_GPIO_FUNCTION_SPI_CS_1_EN     (1<<15)
#define AR71XX_GPIO_FUNCTION_SPI_CS_0_EN     (1<<14)
#define AR71XX_GPIO_FUNCTION_UART_EN         (1<< 8)
#define AR71XX_GPIO_FUNCTION_OVERCURRENT_EN  (1<< 4)
#define AR71XX_GPIO_FUNCTION_USB_CLK_CORE_EN (1<< 0)
#define AR71XX_GPIO_FUNCTION_WMAC_LED        (1<<22)

/*
 * GPIO Access & Control
 */
void ar71xx_gpio_init(void);
void ar71xx_gpio_down(void);
void ar71xx_gpio_up(void);

/*
 * GPIO Helper Functions
 */
void ar71xx_gpio_enable_slic(void);

/* enable UART block, takes away GPIO 10 and 9 */
void ar71xx_gpio_enable_uart(void);

/* enable STEREO block, takes away GPIO 11,8,7, and 6 */
void ar71xx_gpio_enable_stereo(void);

/* allow CS0/CS1 to be controlled via SPI register, takes away GPIO0/GPIO1 */
void ar71xx_gpio_enable_spi_cs1_cs0(void);

/* allow GPIO0/GPIO1 to be used as SCL/SDA for software based i2c */
void ar71xx_gpio_enable_i2c_on_gpio_0_1(void);

/*
 * GPIO General Functions
 */
void ar71xx_gpio_drive_low(unsigned int mask);
void ar71xx_gpio_drive_high(unsigned int mask);

unsigned int ar71xx_gpio_float_high_test(unsigned int mask);

/*
 * Software support of i2c on gpio 0/1
 */
int ar71xx_i2c_raw_write_bytes_to_addr(int addr, unsigned char *buffer, int count);
int ar71xx_i2c_raw_read_bytes_from_addr(int addr, unsigned char *buffer, int count);

/* SPI, SLIC and GPIO are all multiplexed on gpio pins */

#define AR71XX_SPI_FS           AR71XX_SPI_BASE
#define AR71XX_SPI_READ         AR71XX_SPI_BASE
#define AR71XX_SPI_CLOCK        AR71XX_SPI_BASE+4
#define AR71XX_SPI_WRITE        AR71XX_SPI_BASE+8
#define AR71XX_SPI_RD_STATUS    AR71XX_SPI_BASE+12

#define AR71XX_SPI_D0_HIGH      (1<<0)             /* Pin spi_do   */
#define AR71XX_SPI_CLK_HIGH     (1<<8)             /* Pin spi_clk  */

#define AR71XX_SPI_CS_ENABLE_0  (6<<16)            /* Pin gpio/cs0 (active low) */
#define AR71XX_SPI_CS_ENABLE_1  (5<<16)            /* Pin gpio/cs1 (active low) */
#define AR71XX_SPI_CS_ENABLE_2  (3<<16)            /* Pin gpio/cs2 (active low) */
#define AR71XX_SPI_CS_DIS       (AR71XX_SPI_CS_ENABLE_0|AR71XX_SPI_CS_ENABLE_1|AR71XX_SPI_CS_ENABLE_2)

#define AR71XX_SPI_RD_STATUS     AR71XX_SPI_BASE+12 /* spi_di is clocked into register pos 0 every clock */

/* 
 * SOC 
 */
#define AR71XX_SPI_CMD_WREN         0x06
#define AR71XX_SPI_CMD_RD_STATUS    0x05
#define AR71XX_SPI_CMD_FAST_READ    0x0b
#define AR71XX_SPI_CMD_PAGE_PROG    0x02
#define AR71XX_SPI_CMD_SECTOR_ERASE 0xd8

/* Functions to access SPI through software. Example:
 *
 * ar71xx_spi_down(); ---------------------- disable others from accessing SPI bus taking semaphore
 * ar71xx_spi_enable_soft_access(); -------- disable HW control of SPI
 *
 * <board specific chip select routine>
 *
 * <read/write SPI using using custom routine or general purposeflash routines
 * Custom routine may use:
 *
 *  ar71xx_spi_raw_output_u8(unsigned char)
 *  ar71xx_spi_raw_output_u32(unsigned int)
 *  ar71xx_spi_raw_input_u32()
 *
 * General purpose flash routines:
 *  ar71xx_spi_flash_read_page(unsigned int addr, unsigned char *data, int len);
 *  ar71xx_spi_flash_write_page(unsigned int addr, unsigned char *data, int len);
 *  ar71xx_spi_flash_sector_erase(unsigned int addr);
 * >
 *
 * <board specific chip deselect routine>
 *
 * ar71xx_spi_disable_soft_acess(); ------- enable HW control of SPI bus
 * ar71xx_spi_up(); ----------------------- enable others to access SPI bus releasing semaphore
 */
void ar71xx_spi_init(void);
void ar71xx_spi_down(void);
void ar71xx_spi_up(void);

static inline void 
ar71xx_spi_enable_soft_access(void)
{
  ar71xx_reg_wr_nf(AR71XX_SPI_FS, 1); 
}

static inline void 
ar71xx_spi_disable_soft_access(void)
{
  ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_DIS);
  ar71xx_reg_wr_nf(AR71XX_SPI_FS, 0);
}

void ar71xx_spi_raw_output_u8(unsigned char val);
void ar71xx_spi_raw_output_u32(unsigned int val);
unsigned int ar71xx_spi_raw_input_u32(void);

#define AR71XX_SPI_SECTOR_SIZE      (1024*64)

void ar71xx_spi_flash_read_page(unsigned int addr, unsigned char *data, int len);
void ar71xx_spi_flash_write_page(unsigned int addr, unsigned char *data, int len);
void ar71xx_spi_flash_sector_erase(unsigned int addr);

/* 
 * Allow access to cs0-2 when GPIO Function enables cs0-2 through SPI register.
 */
static inline void ar71xx_spi_enable_cs0(void)
{
  unsigned int cs;
  ar71xx_spi_down();
  ar71xx_spi_enable_soft_access();
  cs = ar71xx_reg_rd(AR71XX_SPI_WRITE) & ~AR71XX_SPI_CS_DIS;
  ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_ENABLE_0 | cs);
}

static inline void ar71xx_spi_enable_cs1(void)
{
  unsigned int cs;
  ar71xx_spi_down();
  ar71xx_spi_enable_soft_access();
  cs = ar71xx_reg_rd(AR71XX_SPI_WRITE) & ~AR71XX_SPI_CS_DIS;
  ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_ENABLE_1 | cs);
}

static inline void ar71xx_spi_disable_cs(void)
{
  unsigned int cs = ar71xx_reg_rd(AR71XX_SPI_WRITE) | AR71XX_SPI_CS_DIS;
  ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, cs);
  ar71xx_spi_disable_soft_access();
  ar71xx_spi_up();
}

/* 
 * Example usage to access BOOT flash
 */
static inline void ar71xx_spi_flash_cs0_sector_erase(unsigned int addr) 
{
  ar71xx_spi_enable_cs0();
  ar71xx_spi_flash_sector_erase(addr);
  ar71xx_spi_disable_cs();
}

static inline void ar71xx_spi_flash_cs0_write_page(unsigned int addr, unsigned char *data, int len)
{
  ar71xx_spi_enable_cs0();
  ar71xx_spi_flash_write_page(addr, data, len);
  ar71xx_spi_disable_cs();
}

/*
 * Reset block
 */
#define AR71XX_GENERAL_TMR            AR71XX_RESET_BASE+0
#define AR71XX_GENERAL_TMR_RELOAD     AR71XX_RESET_BASE+4
#define AR71XX_WATCHDOG_TMR_CONTROL   AR71XX_RESET_BASE+8
#define AR71XX_WATCHDOG_TMR           AR71XX_RESET_BASE+0xc
#define AR71XX_MISC_INT_STATUS        AR71XX_RESET_BASE+0x10
#define AR71XX_MISC_INT_MASK          AR71XX_RESET_BASE+0x14

#ifndef CONFIG_AR9100

#define AR71XX_PCI_INT_STATUS         AR71XX_RESET_BASE+0x18
#define AR71XX_PCI_INT_MASK           AR71XX_RESET_BASE+0x1c
#define AR71XX_GLOBAL_INT_STATUS      AR71XX_RESET_BASE+0x20
#define AR71XX_RESET                  AR71XX_RESET_BASE+0x24
#define AR71XX_OBSERVATION_ENABLE     AR71XX_RESET_BASE+0x28

#else /* ar9100 */

#define AR71XX_GLOBAL_INT_STATUS      AR71XX_RESET_BASE+0x18
#define AR71XX_RESET                  AR71XX_RESET_BASE+0x1c
#define AR71XX_RST_REVISION_ID        AR71XX_RESET_BASE+0x90
#endif

#ifdef CONFIG_AR9100

#define AR71XX_WD_ACT_MASK	3u
#define AR71XX_WD_ACT_NONE	0u /* No Action */
#define AR71XX_WD_ACT_GP_INTR	1u /* General purpose intr */
#define AR71XX_WD_ACT_NMI	2u /* NMI */
#define AR71XX_WD_ACT_RESET	3u /* Full Chip Reset */

#define AR71XX_WD_LAST_SHIFT	31
#define AR71XX_WD_LAST_MASK	((uint32_t)(1 << AR71XX_WD_LAST_SHIFT))

#endif /* CONFIG_AR9100 */

/*
 * Performace counters
 */
#ifndef CONFIG_AR9100
#define AR71XX_PERF_CTL               AR71XX_RESET_BASE+0x2c
#define AR71XX_PERF0_COUNTER          AR71XX_RESET_BASE+0x30
#define AR71XX_PERF1_COUNTER          AR71XX_RESET_BASE+0x34
#else
#define AR71XX_PERF_CTL               AR71XX_RESET_BASE+0x20
#define AR71XX_PERF0_COUNTER          AR71XX_RESET_BASE+0x24
#define AR71XX_PERF1_COUNTER          AR71XX_RESET_BASE+0x28
#endif

/*
 * SLIC/STEREO DMA Size Configurations 
 */
#define AR71XX_DMA_BUF_SIZE_4X2      0x00
#define AR71XX_DMA_BUF_SIZE_8X2      0x01
#define AR71XX_DMA_BUF_SIZE_16X2     0x02
#define AR71XX_DMA_BUF_SIZE_32X2     0x03
#define AR71XX_DMA_BUF_SIZE_64X2     0x04
#define AR71XX_DMA_BUF_SIZE_128X2    0x05
#define AR71XX_DMA_BUF_SIZE_256X2    0x06
#define AR71XX_DMA_BUF_SIZE_512X2    0x07

/*
 * SLIC/STEREO DMA Assignments
 */
#define AR71XX_DMA_CHAN_SLIC0_RX     0
#define AR71XX_DMA_CHAN_SLIC1_RX     1
#define AR71XX_DMA_CHAN_STEREO_RX    2
#define AR71XX_DMA_CHAN_SLIC0_TX     3
#define AR71XX_DMA_CHAN_SLIC1_TX     4
#define AR71XX_DMA_CHAN_STEREO_TX    5

/* Low-level routines */
void ar71xx_dma_addr_wr  (int chan, unsigned int val);
void ar71xx_dma_config_wr(int chan, unsigned int val);
void ar71xx_dma_update_wr(int chan, unsigned int val);

unsigned int ar71xx_dma_addr_rd  (int chan);
unsigned int ar71xx_dma_config_rd(int chan);

/* Use this routine to configure DMA access. Example:
 *
 * ar71xx_dma_config_buffer( AR71XX_DMA_CHAN_SLIC0_TX, 
 *                           < address of buffer >,
 *                           AR71XX_DMA_BUF_SIZE_512X2
 */
void ar71xx_dma_config_buffer(int chan, void *buffer, int sizeCfg);

/*
 * SLIC register definitions
 */
#define AR71XX_SLIC_STATUS                   (AR71XX_SLIC_BASE+0x00)
#define AR71XX_SLIC_CNTRL                    (AR71XX_SLIC_BASE+0x04)
#define AR71XX_SLIC_SLOT0_NUM                (AR71XX_SLIC_BASE+0x08)
#define AR71XX_SLIC_SLOT1_NUM                (AR71XX_SLIC_BASE+0x0c)
#define AR71XX_SLIC_SAM_POS                  (AR71XX_SLIC_BASE+0x2c)
#define AR71XX_SLIC_FREQ_DIV                 (AR71XX_SLIC_BASE+0x30)

/*
 * SLIC Control bits
 */
#define AR71XX_SLIC_CNTRL_ENABLE             (1<<0)
#define AR71XX_SLIC_CNTRL_SLOT0_ENABLE       (1<<1)
#define AR71XX_SLIC_CNTRL_SLOT1_ENABLE       (1<<2)
#define AR71XX_SLIC_CNTRL_IRQ_ENABLE         (1<<3)

/*
 * SLIC Helper Functions
 */
unsigned int ar71xx_slic_status_rd(void);
unsigned int ar71xx_slic_cntrl_rd(void);

void ar71xx_slic_cntrl_wr(unsigned int val); 
void ar71xx_slic_0_slot_pos_wr(unsigned int val);
void ar71xx_slic_1_slot_pos_wr(unsigned int val);
void ar71xx_slic_freq_div_wr(unsigned int val);
void ar71xx_slic_sample_pos_wr(unsigned int val);

void ar71xx_slic_setup(int _sam, int _s0n, int _s1n);

/*
 * STEREO register definitions
 */
#define AR71XX_STEREO_CONFIG                 (AR71XX_STEREO_BASE+0x00)
#define AR71XX_STEREO_VOLUME                 (AR71XX_STEREO_BASE+0x04)

/*
 * Stereo Configuration Bits
 */
#define AR71XX_STEREO_CONFIG_ENABLE          (1<<24)
#define AR71XX_STEREO_CONFIG_RESET           (1<<23)
#define AR71XX_STEREO_CONFIG_DELAY           (1<<22)
#define AR71XX_STEREO_CONFIG_MIC_WORD_SIZE   (1<<20)

#define AR71XX_STEREO_CONFIG_MODE(x)           ((3&x)<<18)
#define AR71XX_STEREO_MODE_STEREO             0
#define AR71XX_STEREO_MODE_LEFT               1
#define AR71XX_STEREO_MODE_RIGHT              2

#define AR71XX_STEREO_CONFIG_DATA_WORD_SIZE(x) ((3&x)<<16)

#define AR71XX_STEREO_CONFIG_I2S_32B_WORD    (1<<15)
#define AR71XX_STEREO_CONFIG_MASTER          (1<<8)
#define AR71XX_STEREO_CONFIG_PSEDGE(x)       (0xff&x)

/*
 * Word sizes to use with common configurations:
 */
#define AR71XX_STEREO_WS_8B                     0
#define AR71XX_STEREO_WS_16B                    1

/* 
 * Audio data is little endian so 16b values must be swapped in the DMA buffers.
 */
static inline int ar71xx_stereo_sample_16b_cvt(unsigned int _v) { return (((_v<<8)&0xff00)|((_v>>8)&0xff)) & 0xffff; }

/* Low level read/write of configuration */
void ar71xx_stereo_config_wr(unsigned int val);
void ar71xx_stereo_volume_wr(unsigned int val);

unsigned int ar71xx_stereo_config_rd(void);
unsigned int ar71xx_stereo_volume_rd(void);

/*
 * Common configurations for stereo block
 */
#define AR71XX_STEREO_CFG_MASTER_STEREO_FS32_48KHZ(ws) ( \
		       AR71XX_STEREO_CONFIG_DELAY   | \
		       AR71XX_STEREO_CONFIG_RESET   | \
		       AR71XX_STEREO_CONFIG_DATA_WORD_SIZE(ws) | \
                       AR71XX_STEREO_CONFIG_MODE(AR71XX_STEREO_MODE_LEFT) | \
		       AR71XX_STEREO_CONFIG_MASTER  | \
		       AR71XX_STEREO_CONFIG_PSEDGE(26))
     
#define AR71XX_STEREO_CFG_MASTER_STEREO_FS64_48KHZ(ws) ( \
		       AR71XX_STEREO_CONFIG_DELAY   | \
		       AR71XX_STEREO_CONFIG_RESET   | \
		       AR71XX_STEREO_CONFIG_DATA_WORD_SIZE(ws) | \
                       AR71XX_STEREO_CONFIG_MODE(AR71XX_STEREO_MODE_STEREO) | \
		       AR71XX_STEREO_CONFIG_I2S_32B_WORD | \
		       AR71XX_STEREO_CONFIG_MASTER  | \
		       AR71XX_STEREO_CONFIG_PSEDGE(13))

#define AR71XX_STEREO_CFG_SLAVE_STEREO_FS32_48KHZ(ws) ( \
		       AR71XX_STEREO_CONFIG_RESET   | \
		       AR71XX_STEREO_CONFIG_DATA_WORD_SIZE(ws) | \
                       AR71XX_STEREO_CONFIG_MODE(AR71XX_STEREO_MODE_STEREO) | \
		       AR71XX_STEREO_CONFIG_PSEDGE(26))

#define AR71XX_STEREO_CFG_SLAVE_STEREO_FS64_48KHZ(ws) ( \
		       AR71XX_STEREO_CONFIG_RESET   | \
		       AR71XX_STEREO_CONFIG_I2S_32B_WORD | \
		       AR71XX_STEREO_CONFIG_DATA_WORD_SIZE(ws) | \
                       AR71XX_STEREO_CONFIG_MODE(AR71XX_STEREO_MODE_STEREO) | \
		       AR71XX_STEREO_CONFIG_PSEDGE(13))

/* Routine sets up STEREO block for use. Use one of the predefined
 * configurations. Example:
 * 
 * ar71xx_stereo_config_setup(
 *   AR71XX_STEREO_CFG_MASTER_STEREO_FS32_48KHZ(AR71XX_STEREO_WS_16B))
 *
 */
void ar71xx_stereo_config_setup(unsigned int cfg);

/* 48 kHz, 16 bit data & i2s 32fs */
static inline void ar71xx_setup_for_stereo_master(int ws)
{ ar71xx_stereo_config_setup(AR71XX_STEREO_CFG_MASTER_STEREO_FS32_48KHZ(ws)); }

/* 48 kHz, 16 bit data & 32fs i2s */
static inline void ar71xx_setup_for_stereo_slave(int ws) 
{ ar71xx_stereo_config_setup(AR71XX_STEREO_CFG_SLAVE_STEREO_FS32_48KHZ(ws)); }

/*
 * PERF CTL bits
 */
#define PERF_CTL_PCI_AHB_0           ( 0)
#define PERF_CTL_PCI_AHB_1           ( 1)
#define PERF_CTL_USB_0               ( 2)
#define PERF_CTL_USB_1               ( 3)
#define PERF_CTL_GE0_PKT_CNT         ( 4)
#define PERF_CTL_GEO_AHB_1           ( 5)
#define PERF_CTL_GE1_PKT_CNT         ( 6)
#define PERF_CTL_GE1_AHB_1           ( 7)
#define PERF_CTL_PCI_DEV_0_BUSY      ( 8)
#define PERF_CTL_PCI_DEV_1_BUSY      ( 9)
#define PERF_CTL_PCI_DEV_2_BUSY      (10)
#define PERF_CTL_PCI_HOST_BUSY       (11)
#define PERF_CTL_PCI_DEV_0_ARB       (12)
#define PERF_CTL_PCI_DEV_1_ARB       (13)
#define PERF_CTL_PCI_DEV_2_ARB       (14)
#define PERF_CTL_PCI_HOST_ARB        (15)
#define PERF_CTL_PCI_DEV_0_ACTIVE    (16)
#define PERF_CTL_PCI_DEV_1_ACTIVE    (17)
#define PERF_CTL_PCI_DEV_2_ACTIVE    (18)
#define PERF_CTL_HOST_ACTIVE         (19)

#define ar71xx_perf0_ctl(_val) ar71xx_reg_wr(AR71XX_PERF_CTL, (_val))
#define ar71xx_perf1_ctl(_val) ar71xx_reg_rmw_set(AR71XX_PERF_CTL, ((_val) << 8))

#ifndef CONFIG_AR9100

/* These are values used in platform.inc to select PLL settings */

#define AR71XX_REV_ID           (AR71XX_RESET_BASE + 0x90)
#define AR71XX_REV_ID_MASK      0xff
#define AR71XX_REV_ID_AR7130    0xa0
#define AR71XX_REV_ID_AR7141    0xa1
#define AR71XX_REV_ID_AR7161    0xa2

#define AR71XX_PLL_USE_REV_ID    0
#define AR71XX_PLL_200_200_100   1
#define AR71XX_PLL_300_300_150   2
#define AR71XX_PLL_333_333_166   3
#define AR71XX_PLL_266_266_133   4
#define AR71XX_PLL_266_266_66    5
#define AR71XX_PLL_400_400_200   6
#define AR71XX_PLL_600_400_150   7

#else /* ar9100 */

/*
 * Chip revision Id
 */
#define AR71XX_CHIP_REV               AR71XX_RESET_BASE+0x90
#define AR71XX_CHIP_REV_MAJOR_M       0x000000f0
#define AR71XX_CHIP_REV_MAJOR_S       4
#define AR71XX_CHIP_REV_MINOR_M       0x0000000f
#define AR71XX_CHIP_REV_MINOR_S       0

#endif

/*
 * AR71XX_RESET bit defines
 */
#define AR71XX_RESET_EXTERNAL               (1 << 28)
#define AR71XX_RESET_FULL_CHIP              (1 << 24)
#define AR71XX_RESET_CPU_NMI                (1 << 21)
#define AR71XX_RESET_CPU_COLD_RESET_MASK    (1 << 20)
#define AR71XX_RESET_DMA                    (1 << 19)
#define AR71XX_RESET_SLIC                   (1 << 18)
#define AR71XX_RESET_STEREO                 (1 << 17)
#define AR71XX_RESET_DDR                    (1 << 16)
#define AR71XX_RESET_GE1_MAC                (1 << 13)
#define AR71XX_RESET_GE1_PHY                (1 << 12)
#define AR71XX_RESET_USBSUS_OVRIDE	    (1 << 10)
#define AR71XX_RESET_GE0_MAC                (1 << 9)
#define AR71XX_RESET_GE0_PHY                (1 << 8)
#define AR71XX_RESET_USB_OHCI_DLL           (1 << 6)
#define AR71XX_RESET_USB_HOST               (1 << 5)
#define AR71XX_RESET_USB_PHY                (1 << 4)

#ifndef CONFIG_AR9100

#define AR71XX_RESET_PCI_BUS                (1 << 1)
#define AR71XX_RESET_PCI_CORE               (1 << 0)

#endif /* ar9100 */

void ar71xx_reset(unsigned int mask);

/*
 * Mii block
 */
#define AR71XX_MII0_CTRL                    0x18070000
#define AR71XX_MII1_CTRL                    0x18070004

//#define BIT(_x) (1 << (_x))

#define ar71xx_get_bit(_reg, _bit)  (ar71xx_reg_rd((_reg)) & (1 << (_bit)))

#define ar71xx_flush_ge(_unit) do {                             \
    u32     reg = (_unit) ? AR71XX_DDR_GE1_FLUSH : AR71XX_DDR_GE0_FLUSH;   \
    ar71xx_reg_wr(reg, 1);                 \
    while((ar71xx_reg_rd(reg) & 0x1));   \
    ar71xx_reg_wr(reg, 1);                 \
    while((ar71xx_reg_rd(reg) & 0x1));   \
}while(0);

#ifndef CONFIG_AR9100

#define ar71xx_flush_pci() do {                             \
    ar71xx_reg_wr(AR71XX_DDR_PCI_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_PCI_FLUSH) & 0x1));   \
    ar71xx_reg_wr(AR71XX_DDR_PCI_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_PCI_FLUSH) & 0x1));   \
}while(0);

#define ar9100_ahb2wmac_reset()

#else

#define ar9100_flush_wmac() do {                             \
    ar71xx_reg_wr(AR71XX_DDR_WMAC_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_WMAC_FLUSH) & 0x1));   \
    ar71xx_reg_wr(AR71XX_DDR_WMAC_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_WMAC_FLUSH) & 0x1));   \
}while(0);

#define ar9100_ahb2wmac_reset() do {                    \
    u_int32_t reg;                                      \
    reg = ar71xx_reg_rd(AR71XX_RESET);                  \
    ar71xx_reg_wr(AR71XX_RESET, (reg | 0x00400000));    \
    udelay(100000);                                     \
                                                        \
    reg = ar71xx_reg_rd(AR71XX_RESET);                  \
    ar71xx_reg_wr(AR71XX_RESET, (reg & 0xffbfffff));    \
    udelay(100000);                                     \
} while(0)
 

#define ar9100_get_rst_revisionid(version) do {         \
    version = ar71xx_reg_rd(AR71XX_RST_REVISION_ID);    \
} while(0)

#endif /* ar9100 */

#define ar71xx_flush_USB() do {                             \
    ar71xx_reg_wr(AR71XX_DDR_USB_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_USB_FLUSH) & 0x1));   \
    ar71xx_reg_wr(AR71XX_DDR_USB_FLUSH, 1);                 \
    while((ar71xx_reg_rd(AR71XX_DDR_USB_FLUSH) & 0x1));   \
}while(0);

int ar71xx_local_read_config(int where, int size, u32 *value);
int ar71xx_local_write_config(int where, int size, u32 value);
int ar71xx_check_error(int verbose);
unsigned char __ar71xx_readb(const volatile void __iomem *p);
unsigned short __ar71xx_readw(const volatile void __iomem *p);
#ifdef CONFIG_AR9100
void enable_wmac_led(void);
void disable_wmac_led(void);
#endif

#endif
