#ifndef _AR71XX_SPI_H
#define _AR71XX_SPI_H

#include <asm/mach-ar71xx/ar71xx-spi.h>

//#define AR71XX_SPI_FS           0x1f000000
//#define AR71XX_SPI_CLOCK        0x1f000004
//#define AR71XX_SPI_WRITE        0x1f000008
//#define AR71XX_SPI_READ         0x1f000000
//#define AR71XX_SPI_RD_STATUS    0x1f00000c

//#define AR71XX_SPI_CS_DIS       0x70000
#define AR71XX_SPI_CE_LOW       0x60000
#define AR71XX_SPI_CE_HIGH      0x60100

#define AR71XX_SPI_CE_LOW1      0x60000
#define AR71XX_SPI_CE_LOW2      0x50000

#define AR71XX_SPI_CE_HIGH1	0x60100
#define AR71XX_SPI_CE_HIGH2	0x50100

#define AR71XX_SPI_CMD_WREN         0x06
#define AR71XX_SPI_CMD_READ         0x03
#define AR71XX_SPI_CMD_RD_STATUS    0x05
#define AR71XX_SPI_CMD_FAST_READ    0x0b
#define AR71XX_SPI_CMD_PAGE_PROG    0x02
#define AR71XX_SPI_CMD_SECTOR_ERASE 0xd8

#define AR71XX_SPI_SECTOR_SIZE      (1024*64)
#define AR71XX_SPI_PAGE_SIZE        256

#define AR71XX_FLASH_MAX_BANKS  1


#define display(_x)     ar71xx_reg_wr_nf(0x18040008, (_x))

/*
 * primitives
 */

#define ar71xx_be_msb(_val, __i) (((_val) & (1 << (7 - __i))) >> (7 - __i))

#define ar71xx_spi_bit_banger(_idx, _byte)  do {        \
    int _i;                                      \
    for(_i = 0; _i < 8; _i++) {                    \
        if(_idx) { \
        ar71xx_reg_wr_nf(AR71XX_SPI_WRITE,      \
                        AR71XX_SPI_CE_LOW2 | ar71xx_be_msb(_byte, _i));  \
        ar71xx_reg_wr_nf(AR71XX_SPI_WRITE,      \
                        AR71XX_SPI_CE_HIGH2 | ar71xx_be_msb(_byte, _i)); \
        } \
        else { \
        ar71xx_reg_wr_nf(AR71XX_SPI_WRITE,      \
                        AR71XX_SPI_CE_LOW1 | ar71xx_be_msb(_byte, _i));  \
        ar71xx_reg_wr_nf(AR71XX_SPI_WRITE,      \
                        AR71XX_SPI_CE_HIGH1 | ar71xx_be_msb(_byte, _i)); \
        } \
    }       \
}while(0);

#define ar71xx_spi_go(_idx) do {        \
  if(_idx) { \
    ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CE_LOW2); \
    ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_DIS); \
  } \
  else { \
    ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CE_LOW1); \
    ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_DIS); \
  } \
}while(0);


#define ar71xx_spi_send_addr(_idx, _addr) do {                    \
    ar71xx_spi_bit_banger(_idx, ((addr & 0xff0000) >> 16));                 \
    ar71xx_spi_bit_banger(_idx, ((addr & 0x00ff00) >> 8));                 \
    ar71xx_spi_bit_banger(_idx, addr & 0x0000ff);                 \
}while(0);

#define ar71xx_spi_delay_8(_idx)    ar71xx_spi_bit_banger(_idx, 0)

#define ar71xx_spi_done() do {       \
  ar71xx_reg_wr_nf(AR71XX_SPI_WRITE, AR71XX_SPI_CS_DIS); \
  ar71xx_reg_wr_nf(AR71XX_SPI_FS, 0);      \
}while(0);

#endif /*_AR71XX_SPI_H*/
