/*
 * Copyright (c) 2008, Atheros Communications Inc.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "athrs_hw_nat.h"

static void athr_lut_reg_clear_status(int portdir)
{
      /* Clear the req, status regs to 0 before the operation */
      athr_reg_write_lut(EG_CPU_REQ_REG, 0, portdir);
      athr_reg_write_lut(EG_CPU_REQ_STAT_REG, 0, portdir);
}
static int ops_status(int cmd,int dir)
{
   int status = 0, cnt = 1000, reg, res = 0;

   cmd &= CMD_MASK;

   reg = (dir == INGRESS ? IG_CPU_REQ_STAT_REG : EG_CPU_REQ_STAT_REG);

   while (cnt != 0) {
       res = athr_reg_read_lut(reg, dir);
       status = (res & STAT_REQ_DONE);
       if (status) {
           printk("%s: req done bit is set, lut op success cnt = %d\n", __func__, cnt);
           break;
       }
       cnt --;
   }

   if (status) {
       if(cmd == CMD_INS || cmd == CMD_DEL) {
          if (res & STAT_INSERT)
             return SUCCESS;
          else if (res & STAT_BKT_FULL)
             return -EBKT_FULL;
          else if (res & STAT_BIN_FULL)
             return -EBIN_FULL;
          else if (res & STAT_DUPLICATE)
             return -EDUP_KEY;
       }
       else {
          if (res & STAT_DATA_FND)
             return SUCCESS;
          else
             return -ENODATA;
       }
   }
   else
       printk("%s: lut op req bit not set cnt = %d\n", __func__, cnt);

   return -EINVAL;
}

int
athrs_hwtbl_nat_ops(void *args, int portdir,int cmd)
{
   ath_egrs_t *egrs;
   ath_ingrs_t *ingrs;

   if (portdir == EGRESS) {
       egrs =  (ath_egrs_t *)args;
       if (is_supp(egrs->prot_key)) {
          if (!is_icmp(egrs->prot_key)) { // For TCP/UDP
              athr_reg_write_lut(EG_INFO_INS_DW0_REG,
                                (EG_INFO_WAN_PORT(egrs->info.wan_port)|
                                 EG_INFO_WAN_IP_IDX(egrs->info.wan_ip_idx) |
                                 EG_INFO_CONN_STAT(egrs->info.l2_con_st)), portdir);

              athr_reg_write_lut(EG_KEY_DW0_REG, (EG_KEY_DW0_LAN_ADDR(egrs->lan_ip_addr) |
                                 EG_KEY_DW0_PROT(egrs->prot_key) |
                                 EG_KEY_DW0_LAN_PORT(egrs->lan_port)), portdir);
          }

          else { // For ICMP Protocol
              athr_reg_write_lut(EG_INFO_INS_DW0_REG,
                                (EG_INFO_WAN_PORT(egrs->info.wan_icmp_id) |
                                 EG_INFO_WAN_IP_IDX(egrs->info.wan_ip_idx) |
                                 EG_INFO_CONN_STAT(egrs->info.l2_con_st)), portdir);

              athr_reg_write_lut(EG_KEY_DW0_REG, (EG_KEY_DW0_LAN_ADDR(egrs->lan_ip_addr) |
                                 EG_KEY_DW0_PROT(egrs->prot_key) |
                                 EG_KEY_DW0_LAN_PORT(egrs->lan_icmp_id)), portdir);
          }

          athr_reg_write_lut(EG_KEY_DW1_REG, EG_KEY_DW1_LAN_ADDR(egrs->lan_ip_addr),
                             portdir);


          athr_lut_reg_clear_status(portdir);

          athr_reg_write_lut(EG_CPU_REQ_REG,
                              (EG_PKT_TYPE(egrs->prot_key) |
                               EG_REQ_ENABLE(1) |
                               EG_LUT_INIT(0) |
                               EG_LUT_CMD(cmd & CMD_MASK)), portdir);

          return(ops_status(cmd,portdir));

       }
   }
   else if (portdir == INGRESS) {
     uint32_t lan_mac_ms;
     uint16_t lan_mac_ls;

     ingrs = (ath_ingrs_t *)args;
     lan_mac_ls = (ingrs->info.lan_mac_addr[5] | (ingrs->info.lan_mac_addr[4] << 8));
     lan_mac_ms = (ingrs->info.lan_mac_addr[3] | (ingrs->info.lan_mac_addr[2] << 8) |
                  (ingrs->info.lan_mac_addr[1] << 16) | (ingrs->info.lan_mac_addr[0] << 24));

        if (is_supp(ingrs->prot_key)) {
            if (!is_icmp(ingrs->prot_key)) {
                athr_reg_write_lut(IG_INFO_DW0_REG,IG_INFO_DW0_LAN_IP(ingrs->info.lan_ip_addr), portdir);
                athr_reg_write_lut(IG_INFO_DW1_REG,
                                  (IG_INFO_DW1_LAN_PORT(ingrs->info.lan_port) |
                                   IG_INFO_DW1_LAN_MAC_LS(lan_mac_ls)), portdir);

                athr_reg_write_lut(IG_INFO_DW2_REG,IG_INFO_DW2_LAN_MAC_MS(lan_mac_ms), portdir);
                athr_reg_write_lut(IG_INFO_DW3_REG,IG_INFO_DW3_CONN_STAT(ingrs->info.l2_con_st), portdir);

                athr_reg_write_lut(IG_KEY_DW0_REG,
                                  (IG_KEY_DW0_WAN_PORT(ingrs->wan_port) |
                                   IG_KEY_DW0_PROT(ingrs->prot_key)     |
                                   IG_KEY_DW0_WAN_IP_IDX(ingrs->wan_ip_idx)), portdir);
           }
           else {
                athr_reg_write_lut(IG_INFO_DW0_REG,IG_INFO_DW0_LAN_IP(ingrs->info.lan_ip_addr), portdir);
                athr_reg_write_lut(IG_INFO_DW1_REG,
                                  (IG_INFO_DW1_LAN_PORT(ingrs->info.lan_icmp_id) |
                                   IG_INFO_DW1_LAN_MAC_LS(lan_mac_ls)), portdir);

                athr_reg_write_lut(IG_INFO_DW2_REG,IG_INFO_DW2_LAN_MAC_MS(lan_mac_ms), portdir);
                athr_reg_write_lut(IG_INFO_DW3_REG,IG_INFO_DW3_CONN_STAT(ingrs->info.l2_con_st), portdir);

                athr_reg_write_lut(IG_KEY_DW0_REG,
                                  (IG_KEY_DW0_WAN_PORT(ingrs->wan_icmp_id) |
                                   IG_KEY_DW0_PROT(ingrs->prot_key)        |
                                   IG_KEY_DW0_WAN_IP_IDX(ingrs->wan_ip_idx)), portdir);

          }

          athr_lut_reg_clear_status(portdir);

          athr_reg_write_lut(IG_CPU_REQ_REG,
                            (IG_PKT_TYPE(ingrs->prot_key) |
                             IG_REQ_ENABLE(1)            |
                             IG_LUT_INIT(0) |
                             IG_LUT_CMD(cmd & CMD_MASK)), portdir);

          return(ops_status(cmd,portdir));
      }
   }
   return -1;
}

static int
athrs_hwtbl_lkup(void *args, int portdir,int cmd)
{
   ath_egrs_t *egrs;
   ath_ingrs_t *ingrs;

   if (portdir == EGRESS) {
       egrs =  (ath_egrs_t *)args;
       if (is_supp(egrs->prot_key)) {
           if (!is_icmp(egrs->prot_key)) {

              athr_reg_write_lut(EG_KEY_DW0_REG,EG_KEY_DW0_LAN_ADDR(egrs->lan_ip_addr), portdir);
              athr_reg_write_lut(EG_KEY_DW1_REG,(EG_KEY_DW1_LAN_PORT(egrs->lan_port) |
                               EG_KEY_DW1_PROT(egrs->prot_key)), portdir);
          }
          else {

              athr_reg_write_lut(EG_KEY_DW0_REG,EG_KEY_DW0_LAN_ADDR(egrs->lan_ip_addr), portdir);
              athr_reg_write_lut(EG_KEY_DW1_REG,
                                (EG_KEY_DW1_LAN_PORT(egrs->lan_icmp_id) |
                                 EG_KEY_DW1_PROT(egrs->prot_key)), portdir);

        }
        athr_reg_write_lut(EG_CPU_REQ_REG,
                          (EG_PKT_TYPE(egrs->prot_key) |
                           EG_REQ_ENABLE(1) |
                           EG_LUT_CMD(cmd & CMD_MASK)), portdir);

	return(ops_status(cmd,portdir));

       }
   }
   else if (portdir == INGRESS) {
     uint32_t lan_mac_ms;
     uint16_t lan_mac_ls;

     ingrs = (ath_ingrs_t *)args;
     lan_mac_ls = (ingrs->info.lan_mac_addr[5] | (ingrs->info.lan_mac_addr[4] << 8));
     lan_mac_ms = (ingrs->info.lan_mac_addr[3] | (ingrs->info.lan_mac_addr[2] << 8) |
                  (ingrs->info.lan_mac_addr[1] << 16) | (ingrs->info.lan_mac_addr[0] << 24));

        if (is_supp(ingrs->prot_key)) {
            if (!is_icmp(ingrs->prot_key)) {
                athr_reg_write_lut(IG_KEY_DW0_REG,
                                  (IG_KEY_DW0_WAN_PORT(ingrs->wan_port) |
                                   IG_KEY_DW0_PROT(ingrs->prot_key)     |
                                   IG_KEY_DW0_WAN_IP_IDX(ingrs->wan_ip_idx)), portdir);
           }
           else {
                athr_reg_write_lut(IG_KEY_DW0_REG,
                                  (IG_KEY_DW0_WAN_PORT(ingrs->wan_icmp_id) |
                                   IG_KEY_DW0_PROT(ingrs->prot_key)        |
                                   IG_KEY_DW0_WAN_IP_IDX(ingrs->wan_ip_idx)), portdir);

          }
          athr_reg_write_lut(IG_CPU_REQ_REG,
                            (IG_PKT_TYPE(egrs->prot_key) |
                             IG_REQ_ENABLE(1)            |
                             IG_LUT_CMD(cmd & CMD_MASK)), portdir);

          return(ops_status(cmd,portdir));
      }
   }
   return -1;
}

void athrs_hwtbl_set_wan_ip_address(u32 ipaddr)
{
        athr_reg_write_global_ip_address(ipaddr);
}
