#include <linux/types.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/icmp.h>
#include <linux/skbuff.h>

#include "athrs_hw_nat.h"


#define ATHRS_SRC_NAT 0
#define ATHRS_DEST_NAT 1

#ifndef DEBUG
#define ATHR_DBG_PRINT()
#else
#define ATHRS_DBG_PRINT()
#endif

extern int athrs_hwtbl_nat_ops(void *args, int portdir,int cmd);

static int athrs_wan_ipaddr_to_idx(uint32_t newip)
{
    return 0;
}

static int athrs_nat_ops(uint32_t oldip, uint16_t oldport,
                      uint32_t newip, uint16_t newport,
                      u_int8_t protocol, u_int8_t maniptype,
                      int cmd)
{
    ath_egrs_t egrs;
    ath_ingrs_t ingrs;
    int ret;

    switch(maniptype) {
        case EGRESS:
            egrs.prot_key = protocol;
            egrs.lan_ip_addr = oldip;
            egrs.lan_port = oldport;
            egrs.info.wan_ip_idx = athrs_wan_ipaddr_to_idx(newip);
            egrs.info.wan_port = newport;

            ret = athrs_hwtbl_nat_ops((void *)&egrs, maniptype, cmd);
            break;

        case INGRESS:
            ingrs.prot_key = protocol;
            ingrs.wan_ip_idx = athrs_wan_ipaddr_to_idx(oldip);
            ingrs.wan_port = oldport;
            ingrs.info.lan_ip_addr = newip;
            ingrs.info.lan_port = newport;
#if 1
            ingrs.info.lan_mac_addr[0] =  0x00;
            ingrs.info.lan_mac_addr[1] =  0x03;
            ingrs.info.lan_mac_addr[2] =  0x7f;
            ingrs.info.lan_mac_addr[3] =  0xff;
            ingrs.info.lan_mac_addr[4] =  0xff;
            ingrs.info.lan_mac_addr[5] =  0xff;
#endif
            ret = athrs_hwtbl_nat_ops((void *)&ingrs, maniptype, cmd);
            break;

        default:
            printk("%s: invalid maniptype\n", __func__);
            return -1;
    }
    return ret;
}

void athrs_hw_nat_add_entry(uint32_t oldip, uint16_t oldport,
                            uint32_t newip, uint16_t newport,
                            u_int8_t protocol, int maniptype)
{
    int ret;
    ret = athrs_nat_ops(oldip, oldport, newip,
                           newport, protocol, maniptype,
                           CMD_INS);

    printk("%s: athrs hw nat add ret value : %d\n", __func__, ret);

}
EXPORT_SYMBOL(athrs_hw_nat_add_entry);

void athrs_hw_nat_del_entry(uint32_t oldip, uint16_t oldport,
                            uint32_t newip, uint16_t newport,
                            u_int8_t protocol, int maniptype)
{
    int ret;
    ret = athrs_nat_ops(oldip, oldport, newip,
                           newport, protocol, maniptype,
                           CMD_DEL);
    printk("%s: athrs hw nat del ret value success : %d\n", __func__, ret);

}
EXPORT_SYMBOL(athrs_hw_nat_del_entry);

static int athrs_hw_nat_lookup_entry(uint32_t ipaddr, uint16_t port,
                           u_int8_t protocol)
{
    int ret;
    ret = athrs_nat_ops(ipaddr, port, 0,
                           0, protocol, 0,
                           CMD_LKP);
    printk("%s: athrs hw nat lookup ret value : %d\n", __func__, ret);
    return ret;
}
//EXPORT_SYMBOL(athrs_hw_nat_lookup_entry);

void athrs_check_and_set_hw_nat_entry(struct sk_buff *skb)
{
        const struct iphdr *iph;
        struct tcphdr *tcp_hdr;
        struct udphdr *udp_hdr;
        struct icmphdr *icmp_hdr;
        unsigned int hdroff = 0, iphdroff = 0;
        __be32 ipaddr;
        __be16 port;
        int hdrsize = 8; /* TCP connection tracking guarantees this much */
        int nat_manip_type;
        int protocol = 0;

        if (htons(skb->protocol) != ETH_P_IP)
                return;

        iph = (struct iphdr *)(skb->data + iphdroff);
        hdroff = iphdroff + iph->ihl*4;

        ipaddr = iph->saddr;

        if (iph->protocol == IPPROTO_TCP) {
                tcp_hdr = (struct tcphdr *)(skb->data + hdroff);
                port = tcp_hdr->source;
                protocol = 1;
        }
        else if (iph->protocol == IPPROTO_UDP) {
                udp_hdr = (struct udphdr *)(skb->data + hdroff);
                port = udp_hdr->source;
                protocol = 2;
        }
        else if (iph->protocol == IPPROTO_ICMP) {
                icmp_hdr = (struct icmphdr *)(skb->data + hdroff);
                port = icmp_hdr->un.echo.id;
                protocol = 3;
        }

        if (athrs_hw_nat_lookup_entry(ipaddr, port, protocol) != -1)
        {
            return;
        }
#if 0
           skb->ath_hw_nat_fw_flags = 1;
#endif
}

void athrs_nat_init_and_enable(int portdir)
{
    char dir[30];
    strcpy(dir, (portdir == INGRESS ? "Ingress" : "Egress"));

    // Disable ageing timer
    athrs_lut_ageing_timer_disable(portdir);
    printk("%s: %s lut ageing timer disable done\n", __func__, dir);

    // Enable nat
    athrs_nat_enable(portdir);
    printk("%s: %s nat enable done\n", __func__, dir);

    // Init nat
    athrs_lut_init(portdir);
    printk("%s: %s lut init done\n", __func__, dir);
}

void athrs_nat_init_add_entries()
{
    unsigned char router_mac[6] = { 0x0, 0x3, 0x7f, 0xff, 0xff, 0xff };
    unsigned char next_hop_mac[6] = { 0x0, 0x4, 0x23, 0xbe, 0x79, 0xed };
    int prot = TCP_PROT;
    char *proto = "TCP";

    printk("%s: Setting h/w nat router mac addr \n", __func__);
    athrs_hw_nat_set_router_mac_addr(router_mac);

    printk("%s: Setting h/w nat next hop mac addr \n", __func__);
    athrs_hw_nat_set_next_hop_mac_addr(next_hop_mac);

#if 0
    // For egress UDP
    athrs_lut_init(EGRESS);
    athrs_nat_enable(EGRESS);
    athrs_lut_ageing_timer_disable(EGRESS);
    printk("%s: Adding egr lut entry from 192.168.1.2 port 6000\n", __func__);
    athrs_hw_nat_add_entry(0xc0a80102, 6000, 0xca010102, 7000, UDP_PROT, EGRESS);
    printk("%s: Writing 202.1.1.2 to global address register index 0\n", __func__);
    athr_reg_write_global_ip_address(0xca010102);
#endif

#if 0
    // For ingress UDP
    athrs_lut_init(INGRESS);
    printk("%s; Ingress lut init done\n", __func__);
    athrs_nat_enable(INGRESS);
    printk("%s; Ingress nat enable done\n", __func__);
    athrs_lut_ageing_timer_disable(INGRESS);
    printk("%s; Ingress lut ageing timer disable done\n", __func__);
    printk("%s: Adding ingr lut entry from 192.168.1.2 port 6000 to 127.0.0.1 7000\n", __func__);
    athrs_hw_nat_add_entry(0xc0a80202, 6000, 0xc0a80202, 7000, UDP_PROT, INGRESS);
    printk("%s: Writing 192.168.1.2 to global address register index 0\n", __func__);
    athr_reg_write_global_ip_address(0xc0a80102);
#endif

#if 1
    // For egress ICMP
    athrs_nat_init_and_enable(EGRESS);
    athrs_nat_init_and_enable(INGRESS);
    printk("%s: Adding egr lut entry from 192.168.1.2 port 1234 to\n" ,__func__);
    printk("%s: 202.1.1.2 port 5678 for %s\n", __func__, proto);
    athrs_hw_nat_add_entry(0xc0a80102, 1234, 0xca010102, 5678, prot, EGRESS);
    // just for testing ......
    //mdelay(3000);
#if 1
    // For Ingress ICMP
    printk("%s: Adding ingr lut entry from 202.1.1.2 port 5678 to\n" ,__func__);
    printk("%s: 192.168.1.2 port 1234 for %s\n", __func__, proto);

    athrs_hw_nat_add_entry(0xca010102, 5678, 0xc0a80102, 1234, prot, INGRESS);
    printk("%s: Writing 202.1.1.2 to global address register index 0\n", __func__);
#endif

    athr_reg_write_global_ip_address(0xca010102);

#endif


}


