#ifndef _ATHRS_QOPS_H
#define _ATHRS_QOPS_H

#include "athrs_ioctl.h"

#if defined(CONFIG_MACH_AR7240)

#define ENET_UNIT_LAN 1
#define ENET_UNIT_WAN 0

#include "athrs_mac.h"
extern athr_gmac_t *athr_gmacs[2];

#ifdef CONFIG_AR7240_S26_PHY
#include "ar7240_s26_phy.h"
#endif

#ifdef CONFIG_F1_PHY
#include "athrf1_phy.h"
#endif

#ifdef CONFIG_S16_PHY
#include "athrs16_phy.h"
#endif

static uint32_t
qos_gmac_reg_read(char *ad_name,uint32_t Reg) {

#ifdef CONFIG_S16_PHY
    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);

    if (mac->mac_unit == 1 && is_ar7242()) 
        return (athrs16_reg_read(Reg));
    else
#endif
        return (athrs26_reg_read(Reg));

}

static void
qos_gmac_reg_write(char *ad_name,uint32_t Reg, uint32_t Val) 
{

    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);
#ifdef CONFIG_S16_PHY

    if (mac->mac_unit == 1 && is_ar7242()) 
        athrs16_reg_write(Reg,Val);
    else
#endif
    printk(KERN_DEBUG "[%x] QoS Reg write %x=%x\n",mac->mac_unit,Reg,Val);
        athrs26_reg_write(Reg,Val);
    return;

}

static void
qos_gmac_reg_rmw_set(char *ad_name,uint32_t Reg, uint32_t Val) 
{
    int tval = qos_gmac_reg_read(ad_name,Reg);

    qos_gmac_reg_write(ad_name,Reg,(tval | Val));
}

static void
qos_gmac_reg_rmw_clear(char *ad_name,uint32_t Reg, uint32_t Val) 
{
    int tval = qos_gmac_reg_read(ad_name,Reg);

    qos_gmac_reg_write(ad_name,Reg,(tval & ~Val));
}

static int 
qos_gmac_enable_mac_qos(char *ad_name) {

    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);
    struct net_device *dev = mac->mac_dev;
    int val = 0;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
    dev->netdev_ops->ndo_stop(dev);
#else
    dev->stop(dev);
#endif
    if(mac_has_flag(mac,ETH_SWONLY_MODE))
        return -1;

    mac_set_flag(mac,WAN_QOS_SOFT_CLASS);

    if (mac->mac_unit == ENET_UNIT_LAN) {
        val = qos_gmac_reg_read(ad_name, ATHR_CTRL_PORT_0);
        if( (val & ATHR_HDR_EN) == 0){
            qos_gmac_reg_rmw_set(ad_name,ATHR_CTRL_PORT_0, ATHR_HDR_EN);
            mac->qos->qos_flag |= 0x1 << 0;
        } 

        val = qos_gmac_reg_read(ad_name, ATHR_CPU_PORT); 
        if( (val & ATHR_CPU_EN) == 0){
            qos_gmac_reg_rmw_set(ad_name,ATHR_CPU_PORT, ATHR_CPU_EN);
            mac->qos->qos_flag |= 0x1 << 1;
        } 
        qos_gmac_reg_rmw_set(ad_name,ATHR_QOS_MODE_REGISTER,ATHR_QOS_WEIGHTED);

#ifdef CONFIG_S16_PHY
        if (is_ar7242() && mac->mac_unit == 0)
            mac_set_flag(mac,ATHR_S16_HEADER);
        else
#endif
#ifndef CONFIG_ATHEROS_HEADER_EN
            mac_set_flag(mac,ATHR_S26_HEADER);
#endif
        for (val = 1; val <= 4; val++){
            qos_gmac_reg_rmw_clear(ad_name,ATHR_PRI_CTRL_PORT_0 +
              (val * 256) , (ATHR_DA_PRI_EN|ATHR_TOS_PRI_EN|ATHR_VLAN_PRI_EN|ATHR_PORT_PRI_EN));
        }
    }
    else {
     /* WAN MAC setting */
        qos_gmac_reg_rmw_clear(ad_name,ATHR_PRI_CTRL_PORT_0 +
               (5 * 256) , (ATHR_DA_PRI_EN|ATHR_TOS_PRI_EN|ATHR_VLAN_PRI_EN|ATHR_PORT_PRI_EN));
    }
    mac->qos->qos_cap = 1;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
    dev->netdev_ops->ndo_open(dev);
#else
    dev->open(dev);
#endif
    return 0;
}

static void 
qos_gmac_disable_mac_qos(char *ad_name) {

    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);
    struct net_device *dev = mac->mac_dev;
    int val = 0;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
    dev->netdev_ops->ndo_stop(dev);
#else
    dev->stop(dev);
#endif

    mac_clear_flag(mac,WAN_QOS_SOFT_CLASS);

    if (mac->mac_unit == ENET_UNIT_LAN) {
        if(mac->qos->qos_flag != 0){
            if( (0x1 & mac->qos->qos_flag) != 0)
                qos_gmac_reg_rmw_clear(ad_name,ATHR_CTRL_PORT_0 ,ATHR_HDR_EN);
            if( ( (0x1 << 1) & mac->qos->qos_flag) != 0)
                qos_gmac_reg_rmw_clear(ad_name,ATHR_CPU_PORT, ATHR_CPU_EN);
        }
        qos_gmac_reg_rmw_clear(ad_name,ATHR_QOS_MODE_REGISTER,ATHR_QOS_WEIGHTED);

#ifdef CONFIG_S16_PHY
        if (is_ar7242() && mac->mac_unit == 0)
            mac_clear_flag(mac,ATHR_S16_HEADER);
        else
#endif
#ifndef CONFIG_ATHEROS_HEADER_EN
            mac_clear_flag(mac,ATHR_S26_HEADER);
#endif
        for (val = 1; val <= 4; val++){
            qos_gmac_reg_rmw_clear(ad_name,ATHR_PRI_CTRL_PORT_0 +
                 (val * 256) , (ATHR_DA_PRI_EN|ATHR_TOS_PRI_EN|ATHR_VLAN_PRI_EN|ATHR_PORT_PRI_EN));
        }
    }
    else {
        /* WAN MAC setting */
        qos_gmac_reg_rmw_clear(ad_name,ATHR_PRI_CTRL_PORT_0 +
                 (5 * 256) , (ATHR_DA_PRI_EN|ATHR_TOS_PRI_EN|ATHR_VLAN_PRI_EN|ATHR_PORT_PRI_EN));

    }
    mac->qos->qos_cap = 0;
	mac->qos->qos_flag = 0;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31))
    dev->netdev_ops->ndo_open(dev);
#else
    dev->open(dev);
#endif
}

struct ath_qops ath_qos_ops = {
    qos_gmac_reg_read,
    qos_gmac_reg_write,
    qos_gmac_reg_rmw_set,
    qos_gmac_reg_rmw_clear,
    qos_gmac_enable_mac_qos,
    qos_gmac_disable_mac_qos,
    0,
};

static inline struct ath_qops
*get_qmac(char *ad_name)
{
    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);

    if (mac_has_flag(mac,ETH_SWONLY_MODE))
        mac = athr_gmac_unit2mac(1);

    return (mac->qos);
}

static inline  struct net_device 
*get_ndev(char *ad_name)
{
    athr_gmac_t *mac = athr_gmac_name2mac(ad_name);

    if (mac_has_flag(mac,ETH_SWONLY_MODE))
        mac = athr_gmac_unit2mac(1);

    return (mac->mac_dev);
}

extern int set_mac_qops(void *mac);

#endif
#endif  //_ATHRS_QOS_H
