#include <linux/netfilter/x_tables.h>
#include <linux/netfilter_bridge/ebtables.h>
#include <linux/netfilter_bridge/ebt_cipfw.h>
#include <linux/ip.h>
#include <linux/if_arp.h>
#include <linux/if_ether.h>
#include <linux/if_vlan.h>
#include <linux/udp.h>
#include <linux/in.h>
#include <linux/module.h>
#include <linux/skbuff.h>

static int get_vlan_encap(const struct sk_buff *skb,unsigned short *encap)
{
	struct vlan_hdr _frame, *fp;
	int offset;

	if (eth_hdr(skb)->h_proto == htons(ETH_P_8021Q)) {
		offset=VLAN_HLEN;
		fp = skb_header_pointer(skb, 0, sizeof(_frame), &_frame);
		*encap = fp->h_vlan_encapsulated_proto;
	}else{
		offset=0;
	}
	return offset;
}

static bool ebt_cipfw_mt(const struct sk_buff *skb, 
	const struct xt_match_param *par)
{
	const struct ebt_cipfw_info *info = par->matchinfo;
	struct iphdr _iph, *iph;
	struct udphdr _udph, *udph;
	unsigned char mp[ETH_ALEN],*cmac,*smac;
	unsigned short encap = 0;
	int offset=get_vlan_encap(skb,&encap);


	if (info->bitmask & EBT_CIPFW){
		smac = eth_hdr(skb)->h_source;
		//printk("source mac = %02x:%02x:%02x:%02x:%02x:%02x offset1=%d\n",*smac,*(smac+1),*(smac+2),*(smac+3),*(smac+4),*(smac+5),offset);	
		if(info->tos==CIPFW_DHCP){
			if(eth_hdr(skb)->h_proto == htons(ETH_P_IP) || encap == ETH_P_IP ){
				iph = skb_header_pointer(skb, offset, sizeof(_iph), &_iph);
				if(iph->protocol==IPPROTO_UDP){
					offset += iph->ihl * 4;  // IP header length
					udph = skb_header_pointer(skb, offset, sizeof(_udph), &_udph);
					if(udph->source==68 && udph->dest==67){   //DHCP
						offset +=  2*4;  // UDP header length
						offset +=  7*4;
						/* the offset from bootp header to client mac 
						 * {Opcode + Hw type + Hw length + Hop count + 
						 *  Transaction ID + 
						 *  Number of seconds  + Flags +
						 *  Client IP +
						 *  Your IP +
						 *  Server IP +
						 *  Gateway IP } = 7*4
						 * {Client MAC}  = 16
						 * {Host name}   = 64
						 * {Boot filename}    = 128
						 * {Magic cookie} =4
						 */
						cmac = skb_header_pointer(skb, offset,
                                        		     sizeof(mp), &mp);
						//printk("client mac= %02x:%02x:%02x:%02x:%02x:%02x offset=%d\n",*cmac,*(cmac+1),*(cmac+2),*(cmac+3),*(cmac+4),*(cmac+5),offset);	
						if(FWINV(memcmp(smac,cmac,ETH_ALEN)!=0, EBT_CIPFW)){
							return false;
						}
					}
					else	
					return false;
				}	
				else	
				return false;
			
			}
			else	
			return false;

		}
		else if (info->tos==CIPFW_ARP){
			if(eth_hdr(skb)->h_proto == htons(ETH_P_ARP) || encap == ETH_P_ARP ){
				cmac = skb_header_pointer(skb, offset+sizeof(struct arphdr),
                                        		     sizeof(mp), &mp);
//				printk("arp sender mac= %02x:%02x:%02x:%02x:%02x:%02x offset=%d\n",cmac[0],cmac[1],cmac[2],cmac[3],cmac[4],cmac[5],offset);	
				if(FWINV(memcmp(smac,cmac,ETH_ALEN)!=0, EBT_CIPFW)	){
					return false;
				}

			}
			else	
			return false;
		}
	
	}	
	return true;
}

static bool ebt_cipfw_mt_check(const struct xt_mtchk_param *par)
{
	const struct ebt_cipfw_info *info = par->matchinfo;

	if (info->bitmask & ~EBT_CIPFW_MASK || info->invflags & ~EBT_CIPFW_MASK)
		return false;
	return true;
}

static struct xt_match ebt_cipfw_mt_reg __read_mostly = {
	.name		= "cipfw",
	.revision	= 0,
	.family		= NFPROTO_BRIDGE,
	.match		= ebt_cipfw_mt,
	.checkentry	= ebt_cipfw_mt_check,
	.matchsize	= XT_ALIGN(sizeof(struct ebt_cipfw_info)),
	.me		= THIS_MODULE,
};

static int __init ebt_cipfw_init(void)
{
	return xt_register_match(&ebt_cipfw_mt_reg);
}

static void __exit ebt_cipfw_fini(void)
{
	xt_unregister_match(&ebt_cipfw_mt_reg);
}

module_init(ebt_cipfw_init);
module_exit(ebt_cipfw_fini);
MODULE_DESCRIPTION("Ebtables: CIPFW match");
MODULE_LICENSE("GPL");
