/* Kernel module to match MAC address parameters. */

/* (C) 1999-2001 Paul `Rusty' Russell
 * (C) 2002-2004 Netfilter Core Team <coreteam@netfilter.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/if_ether.h>
#include <linux/etherdevice.h>

#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv6.h>
#include <linux/netfilter/xt_loadavg.h>
#include <linux/netfilter/x_tables.h>
#include <linux/sched.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Cipherium <vic@cipherium.com.tw>");
MODULE_DESCRIPTION("Xtables: CPU load average match");
MODULE_ALIAS("ipt_loadavg");
MODULE_ALIAS("ip6t_loadavg");

#define LOAD_INT(x) ((x) >> FSHIFT)
static bool loadavg_mt(const struct sk_buff *skb, const struct xt_match_param *par)
{
    const struct xt_loadavg_info *info = par->matchinfo;
    unsigned long avnrun[3];

    get_avenrun(avnrun, FIXED_1/200, 0);
    if (avnrun[0] >= info->limit)
	return true;
    else
	return false;
}

static struct xt_match loadavg_mt_reg __read_mostly = {
	.name      = "loadavg",
	.revision  = 0,
	.family    = NFPROTO_UNSPEC,
	.match     = loadavg_mt,
	.matchsize = sizeof(struct xt_loadavg_info),
	.hooks     = (1 << NF_INET_PRE_ROUTING) | (1 << NF_INET_LOCAL_IN) |
	             (1 << NF_INET_FORWARD),
	.me        = THIS_MODULE,
};

static int __init loadavg_mt_init(void)
{
	return xt_register_match(&loadavg_mt_reg);
}

static void __exit loadavg_mt_exit(void)
{
	xt_unregister_match(&loadavg_mt_reg);
}

module_init(loadavg_mt_init);
module_exit(loadavg_mt_exit);
