/****************************************************************************
*                                                                           *
*  File Name:           rt_ieee802dot11.c                                   *
*  Used By:		Ralink						    *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                               Includes                                    *
****************************************************************************/
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "ieee802dot11.h"
#include "iwlib.h"
#include <unistd.h> //fork
#include <signal.h>
#include <sys/wait.h>

#define DISPLAYWIEXT                        // display wireless ext info
#define TABLE_SIZE   1
#define MINLOADFREQ 5                       // min reload frequency in seconds      // for testing
#define PROC_NET_DEV      "/proc/net/dev"
#define PROC_NET_WIRELESS "/proc/net/wireless"
#ifndef UCHAR
  typedef unsigned char UCHAR;
#endif

#ifdef WITH_CIPCFG
#include "cipcfg.h"
#include "id.h"
#endif
/****************************************************************************
*                            Private Functions                              *
****************************************************************************/
static void loadTables();
static void loadWiExt ( int, char *, struct wireless_info * );
static void load80211Structs ( int, char *, struct wireless_info * );
static void initStructs();
void child_handler(int signum, siginfo_t *info, void *context);

// Wireless Extensions Specific Functions
static void loadWiExtTo80211Structs ( int, char *, struct wireless_info * );
static void displayWiExt ( struct wireless_info );

// Linked List Functions
static void addList ( char *, char *, int );
static void initLists();                    // initialize all the linked lists
static void flushLists();                   // flush all the linked lists
static void flushList ( char * );           // flush a single linked list

// Utility Functions
static int  openSocket ( void );
static char *htob ( char * );
static int  hasChanged ( char *, int );

CIPCFG * rt_get_cipcfg();
/****************************************************************************
*                            Private Variables                              *
****************************************************************************/
static unsigned long lastLoad = 0;          // ET in secs at last table load
static unsigned long currentChannel_lastLoad = 0;          // ET in secs at last write setting 

static struct avNode *lastNode, *newNode, *np;

extern char* config_ram;
extern char* current_config;
static int  mapping_index[100];	     //this array record the index of interface
static char interface[100][IFNAMSIZ];//all interface name in /proc/net/dev
char retval[MAX_VALUE_LEN];
int reinit_rt_cipcfg=0;	//flag to decide doing init_cipcfg();
CIPCFG *rt_cipcfg = NULL;	//cipcfg to current_config
/****************************************************************************
*                            External Functions                             *
****************************************************************************/

/****************************************************************************
*   rt_ieee802dot11_variables_oid:                                             *
*       this is the top level oid that we want to register under.  This     *
*       is essentially a prefix, with the suffix appearing in the           *
*       variable below.                                                     *
****************************************************************************/
oid rt_ieee802dot11_variables_oid[] = { 1,2,840,10036 };

/****************************************************************************
*   variable7 rt_ieee802dot11_variables:                                       *
*     this variable defines function callbacks and type return information  *
*     for the ieee802dot11 mib section                                      *
****************************************************************************/
struct variable7 rt_ieee802dot11_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
#define   DOT11STATIONID        3
  { DOT11STATIONID      , ASN_OCTET_STR , RWRITE, var_dot11StationConfigTable, 4, { 1,1,1,1 } },
#define   DOT11PRIVACYOPTIONIMPLEMENTED  9
  { DOT11PRIVACYOPTIONIMPLEMENTED, ASN_INTEGER   , RONLY , var_dot11StationConfigTable, 4, { 1,1,1,7 } },
#define   DOT11DESIREDSSID      11
  { DOT11DESIREDSSID    , ASN_OCTET_STR , RWRITE, var_dot11StationConfigTable, 4, { 1,1,1,9 } },
#define   DOT11DESIREDBSSTYPE   12
  { DOT11DESIREDBSSTYPE , ASN_INTEGER   , RWRITE, var_dot11StationConfigTable, 4, { 1,1,1,10 } },
#define   DOT11BEACONPERIOD     14
  { DOT11BEACONPERIOD   , ASN_INTEGER   , RWRITE, var_dot11StationConfigTable, 4, { 1,1,1,12 } },
#define   DOT11AUTHENTICATIONALGORITHM  26
  { DOT11AUTHENTICATIONALGORITHM, ASN_INTEGER   , RONLY , var_dot11AuthenticationAlgorithmsTable, 4, { 1,2,1,2 } },
#define   DOT11AUTHENTICATIONALGORITHMSENABLE  27
  { DOT11AUTHENTICATIONALGORITHMSENABLE, ASN_INTEGER   , RWRITE, var_dot11AuthenticationAlgorithmsTable, 4, { 1,2,1,3 } },
#define   DOT11WEPDEFAULTKEYVALUE  31
  { DOT11WEPDEFAULTKEYVALUE, ASN_OCTET_STR , RWRITE, var_dot11WEPDefaultKeysTable, 4, { 1,3,1,2 } },
#define   DOT11PRIVACYINVOKED   41
  { DOT11PRIVACYINVOKED , ASN_INTEGER   , RWRITE, var_dot11PrivacyTable, 4, { 1,5,1,1 } },
#define   DOT11WEPDEFAULTKEYID  42
  { DOT11WEPDEFAULTKEYID, ASN_INTEGER   , RWRITE, var_dot11PrivacyTable, 4, { 1,5,1,2 } },
#define   DOT11EXCLUDEUNENCRYPTED  44
  { DOT11EXCLUDEUNENCRYPTED, ASN_INTEGER   , RWRITE, var_dot11PrivacyTable, 4, { 1,5,1,4 } },
#define   DOT11MACADDRESS       49
  { DOT11MACADDRESS     , ASN_OCTET_STR , RONLY , var_dot11OperationTable, 4, { 2,1,1,1 } },
#define   DOT11RTSTHRESHOLD     50
  { DOT11RTSTHRESHOLD   , ASN_INTEGER   , RWRITE, var_dot11OperationTable, 4, { 2,1,1,2 } },
#define   DOT11FRAGMENTATIONTHRESHOLD  53
  { DOT11FRAGMENTATIONTHRESHOLD, ASN_INTEGER   , RWRITE, var_dot11OperationTable, 4, { 2,1,1,5 } },
#define   DOT11MANUFACTURERID   56
  { DOT11MANUFACTURERID , ASN_OCTET_STR , RONLY , var_dot11OperationTable, 4, { 2,1,1,8 } },
#define   DOT11RESOURCETYPEIDNAME  79
  { DOT11RESOURCETYPEIDNAME, ASN_OCTET_STR , RONLY , var_ieee802dot11, 3, { 3,1,1 } },
#define   DOT11MANUFACTUREROUI  82
  { DOT11MANUFACTUREROUI, ASN_OCTET_STR , RONLY , var_dot11ResourceInfoTable, 5, { 3,1,2,1,1 } },
#define   DOT11PHYTYPE          88
  { DOT11PHYTYPE        , ASN_INTEGER   , RONLY , var_dot11PhyOperationTable, 4, { 4,1,1,1 } },
#define   DOT11CURRENTCHANNEL   119
  { DOT11CURRENTCHANNEL , ASN_INTEGER   , RWRITE, var_dot11PhyDSSSTable, 4, { 4,5,1,1 } },
};
// ( L = length of the oidsuffix )

/****************************************************************************
*                                                                           *
*         init_ieee802dot11() - perform any required initialization         *
*                                                                           *
****************************************************************************/
void init_rt_ieee802dot11 ( void ) {

  /* register ourselves with the agent to handle our mib tree */
  REGISTER_MIB("ieee802dot11", rt_ieee802dot11_variables, variable7,
               rt_ieee802dot11_variables_oid);

  initLists();
  rt_cipcfg = init_cipcfg_f(current_config);
}

/****************************************************************************
*                                                                           *
*    shutdown_ieee802dot11() - perform any required cleanup @ shutdown      *
*                                                                           *
****************************************************************************/
void shutdown_rt_ieee802dot11 ( void )
{
    flushLists();
    close_cipcfg(rt_cipcfg);
}
/****************************************************************************
*                                                                           *
*   var_ieee802dot11() -                                                    *
*                                                                           *
****************************************************************************/
unsigned char *
var_ieee802dot11 ( struct variable *vp, 
                    oid     *name, 
                    size_t  *length, 
                    int     exact, 
                    size_t  *var_len, 
                    WriteMethod **write_method)
{
  loadTables();                                               

  if ( header_generic ( vp, name, length, exact,var_len,write_method )
                                  == MATCH_FAILED )
    return NULL;

  switch ( vp->magic ) {

    case DOT11RESOURCETYPEIDNAME:
      if ( !haveResourceTypeIDName )
        return NULL;
      *var_len = strlen ( resourceTypeIDName );
      return ( UCHAR * ) resourceTypeIDName;

    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*  var_dot11StationConfigTable() - return a variable value from the table   *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11StationConfigTable ( struct variable *vp,
                              oid     *name,
                              size_t  *length,
                              int     exact,
                              size_t  *var_len,
                              WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned
  static char MACWork[17];

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &scList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    sc = ( struct scTbl_data * ) np->data;
    rName[vp->namelen] = sc->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) || 
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {

      switch ( vp->magic ) {      // found requested OID, now check for requested variable
        case DOT11STATIONID: 
          if ( sc->haveStationID                     ) found = TRUE; break;
        case DOT11PRIVACYOPTIONIMPLEMENTED:
          if ( sc->havePrivacyOptionImplemented      ) found = TRUE; break;
        case DOT11DESIREDSSID:
          if ( sc->haveDesiredSSID                   ) found = TRUE; break;
        case DOT11DESIREDBSSTYPE:
          if ( sc->haveDesiredBSSType                ) found = TRUE; break;
        case DOT11BEACONPERIOD: 
          if ( sc->haveBeaconPeriod                  ) found = TRUE; break;
      }
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11STATIONID: 
//    *write_method = write_dot11StationID;
      MACWork[ 0] = sc->stationID [ 0];
      MACWork[ 1] = sc->stationID [ 1];
      MACWork[ 2] = sc->stationID [ 3];
      MACWork[ 3] = sc->stationID [ 4];
      MACWork[ 4] = sc->stationID [ 6];
      MACWork[ 5] = sc->stationID [ 7];
      MACWork[ 6] = sc->stationID [ 9];
      MACWork[ 7] = sc->stationID [10];
      MACWork[ 8] = sc->stationID [12];
      MACWork[ 9] = sc->stationID [13];
      MACWork[10] = sc->stationID [15];
      MACWork[11] = sc->stationID [16];
      MACWork[12] = '\0';
      *var_len = 6;
      return ( UCHAR * ) htob ( MACWork );

    case DOT11PRIVACYOPTIONIMPLEMENTED:
      return ( UCHAR * ) &sc->privacyOptionImplemented;
    case DOT11DESIREDSSID:
//    *write_method = write_dot11DesiredSSID;
      *var_len = strlen ( sc->desiredSSID );
      return ( UCHAR * ) sc->desiredSSID;
    case DOT11DESIREDBSSTYPE:
//    *write_method = write_dot11DesiredBSSType;
      return ( UCHAR * ) &sc->desiredBSSType;
    case DOT11BEACONPERIOD: 
//    *write_method = write_dot11BeaconPeriod;
      return ( UCHAR * ) &sc->beaconPeriod;
    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*  var_dot11AuthenticationAlgorithmsTable() -                               *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11AuthenticationAlgorithmsTable (  struct variable *vp,
                                          oid     *name,
                                          size_t  *length,
                                          int     exact,
                                          size_t  *var_len,
                                          WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &aaList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    aa = ( struct aaTbl_data * ) np->data;
    rName[vp->namelen + 0] = aa->ifIndex;
    rName[vp->namelen + 1] = aa->authenticationAlgorithmsIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 2, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 2, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11AUTHENTICATIONALGORITHM:
          if ( aa->haveAuthenticationAlgorithm    ) found = TRUE; break;
        case DOT11AUTHENTICATIONALGORITHMSENABLE:
          if ( aa->authenticationAlgorithmsEnable ) found = TRUE; break;
      }
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 2 ) * sizeof ( oid ));
  *length = vp->namelen + 2;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11AUTHENTICATIONALGORITHM:
      return ( UCHAR * ) &aa->authenticationAlgorithm;
        
    case DOT11AUTHENTICATIONALGORITHMSENABLE:
//    *write_method = write_dot11AuthenticationAlgorithmsEnable;
      return ( UCHAR * ) &aa->authenticationAlgorithmsEnable;

    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*  var_dot11WEPDefaultKeysTable() -                                         *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11WEPDefaultKeysTable ( struct variable *vp,
                                oid     *name,
                                size_t  *length,
                                int     exact,
                                size_t  *var_len,
                                WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &dfList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    df = ( struct dfTbl_data * ) np->data;
    rName[vp->namelen + 0] = df->ifIndex;
    rName[vp->namelen + 1] = df->WEPDefaultKeyIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 2, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 2, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11WEPDEFAULTKEYVALUE:
          if ( df->haveWEPDefaultKeyValue ) found = TRUE; break;
      }          
    }          
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 2 ) * sizeof ( oid ));
  *length = vp->namelen + 2;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11WEPDEFAULTKEYVALUE:
//    *write_method = write_dot11WEPDefaultKeyValue;
      *var_len = strlen ( df->WEPDefaultKeyValue );
      return ( UCHAR * ) df->WEPDefaultKeyValue;

    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}


/****************************************************************************
*                                                                           *
*   var_dot11PrivacyTable() -                                               *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11PrivacyTable ( struct variable *vp,
                        oid     *name,
                        size_t  *length,
                        int     exact,
                        size_t  *var_len,
                        WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &prList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    pr = ( struct prTbl_data * ) np->data;
    rName[vp->namelen] = pr->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11PRIVACYINVOKED:
          if ( pr->havePrivacyInvoked      ) found = TRUE; break;
        case DOT11WEPDEFAULTKEYID:
          if ( pr->haveWEPDefaultKeyID     ) found = TRUE; break;
        case DOT11EXCLUDEUNENCRYPTED:
          if ( pr->haveExcludeUnencrypted  ) found = TRUE; break;
      }      
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11PRIVACYINVOKED:
//    *write_method = write_dot11PrivacyInvoked;
      return ( UCHAR * ) &pr->privacyInvoked;
    case DOT11WEPDEFAULTKEYID:
//    *write_method = write_dot11WEPDefaultKeyID;
      return ( UCHAR * ) &pr->WEPDefaultKeyID;
    case DOT11EXCLUDEUNENCRYPTED:
//    *write_method = write_dot11ExcludeUnencrypted;
      return ( UCHAR * ) &pr->excludeUnencrypted;
    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*   var_dot11OperationTable() -                                             *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11OperationTable ( struct variable *vp,
                          oid     *name,
                          size_t  *length,
                          int     exact,
                          size_t  *var_len,
                          WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned
  static char MACWork[17];

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &opList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    op = ( struct opTbl_data * ) np->data;
    rName[vp->namelen] = op->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {

      switch ( vp->magic ) {      // found requested OID, now check for requested variable
        case DOT11MACADDRESS:             
          if ( op->haveMACAddress              ) found = TRUE; break;
        case DOT11RTSTHRESHOLD:           
          if ( op->haveRTSThreshold            ) found = TRUE; break;
        case DOT11FRAGMENTATIONTHRESHOLD: 
          if ( op->haveFragmentationThreshold  ) found = TRUE; break;
        case DOT11MANUFACTURERID:
          if ( op->haveManufacturerID          ) found = TRUE; break;
      }
    }
    if ( found ) 
      break;
  }

  if ( !found )
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11MACADDRESS:
      MACWork[ 0] = op->MACAddress[ 0];
      MACWork[ 1] = op->MACAddress[ 1];
      MACWork[ 2] = op->MACAddress[ 3];
      MACWork[ 3] = op->MACAddress[ 4];
      MACWork[ 4] = op->MACAddress[ 6];
      MACWork[ 5] = op->MACAddress[ 7];
      MACWork[ 6] = op->MACAddress[ 9];
      MACWork[ 7] = op->MACAddress[10];
      MACWork[ 8] = op->MACAddress[12];
      MACWork[ 9] = op->MACAddress[13];
      MACWork[10] = op->MACAddress[15];
      MACWork[11] = op->MACAddress[16];
      MACWork[12] = '\0';
      *var_len = 6;
      return ( UCHAR * ) htob ( MACWork );
        
    case DOT11RTSTHRESHOLD:
      return ( UCHAR * ) &op->RTSThreshold;

    case DOT11FRAGMENTATIONTHRESHOLD:
      return ( UCHAR * ) &op->fragmentationThreshold;

    case DOT11MANUFACTURERID:
      *var_len = strlen ( op->manufacturerID );
      return ( UCHAR * ) op->manufacturerID;
        
    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*   var_dot11ResourceInfoTable() -                                          *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11ResourceInfoTable ( struct variable *vp,
                              oid     *name,
                              size_t  *length,
                              int     exact,
                              size_t  *var_len,
                              WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &riList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    ri = ( struct riTbl_data * ) np->data;
    rName[vp->namelen] = ri->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11MANUFACTUREROUI:
          if ( ri->haveManufacturerOUI            ) found = TRUE; break;
      }
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11MANUFACTUREROUI:
      *var_len = strlen ( ri->manufacturerOUI );
      return ( UCHAR * ) ri->manufacturerOUI;
        
        
    default: 
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*   var_dot11PhyOperationTable() -                                          *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11PhyOperationTable ( struct variable *vp,
                              oid     *name,
                              size_t  *length,
                              int     exact,
                              size_t  *var_len,
                              WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &poList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    po = ( struct poTbl_data * ) np->data;
    rName[vp->namelen] = po->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11PHYTYPE:
          if ( po->havePHYType          ) found = TRUE; break;
      }
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11PHYTYPE:
      return ( UCHAR * ) &po->PHYType;
        
    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
*     var_dot11PhyDSSSTable() -                                             *
*                                                                           *
****************************************************************************/
unsigned char *
var_dot11PhyDSSSTable ( struct variable *vp,
                        oid     *name,
                        size_t  *length,
                        int     exact,
                        size_t  *var_len,
                        WriteMethod **write_method )
{
  int found = FALSE;
  oid rName [ MAX_OID_LEN ];                            // OID to be returned

  loadTables();
  memcpy (( char * ) rName, ( char * ) vp->name, ( int ) vp->namelen * sizeof ( oid ));
  for ( np = LIST_FIRST ( &pdList ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    pd = ( struct pdTbl_data * ) np->data;
    rName[vp->namelen] = pd->ifIndex;
    if ((  exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) == 0 )) ||
        ( !exact && ( snmp_oid_compare ( rName, vp->namelen + 1, name, *length ) >  0 ))) {
      switch ( vp->magic ) {
        case DOT11CURRENTCHANNEL:
          if ( pd->haveCurrentChannel   ) found = TRUE; break;
      }
    }
    if ( found )
      break;
  }

  if ( !found ) 
    return NULL;

  memcpy (( char * ) name, ( char * ) rName, ( vp->namelen + 1 ) * sizeof ( oid ));
  *length = vp->namelen + 1;
  *var_len = sizeof ( long );
  *write_method = NULL;

  switch ( vp->magic ) {

    case DOT11CURRENTCHANNEL:
    //*write_method = write_dot11CurrentChannel;
      return ( UCHAR * ) &pd->currentChannel;

    default:
      ERROR_MSG ( "" );
  }

  return NULL;
}

/****************************************************************************
*                                                                           *
****************************************************************************/
int
write_dot11CurrentChannel(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static long *long_ret;
    int size;
    int i=0,chan_len=0,ret,value,pid,gid,rf,valid_value=0,index=0;
    ITEM_LIST       *list = NULL;
    CIPCFG          *cipcfg=NULL;
    char currentChannel[4],mode[MAX_VALUE_LEN],group_name[8];
    struct timeval et;   
    struct sigaction sig;
    COUNTRY_CHANNEL cc;
    extern const int gid_rf[MAX_RF];

    switch ( action ) {

      case RESERVE1:
	  if ( var_val_type != ASN_INTEGER ) {  //check type
	      fprintf ( stderr, "write to dot11CurrentChannel not ASN_INTEGER\n" );
	      return SNMP_ERR_WRONGTYPE;
	  }
	  if ( var_val_len > sizeof ( long_ret )){ //check length
	      fprintf ( stderr, "write to dot11CurrentChannel: bad length\n" );
	      return SNMP_ERR_WRONGLENGTH;
	  }
	  gettimeofday ( &et, ( struct timezone * ) 0 );  // get time-of-day
	  if ( et.tv_sec < currentChannel_lastLoad + 1)       // only reload so often
	      return SNMP_ERR_WRONGVALUE;
	  currentChannel_lastLoad = et.tv_sec;
	  break;
      case RESERVE2:
	  /* Add SIGCHILD signal handler */
	  sig.sa_sigaction = child_handler;
	  sig.sa_flags |= SA_SIGINFO;
	  sigaction(SIGCHLD, &sig, NULL);
	  //get channle length according country code , defined in cipcfg
	  memset((char *) &cc , -1 ,sizeof(COUNTRY_CHANNEL));
	  get_allowed_ch(&cc);
	  value = *((int *)var_val);
	  memset(currentChannel,0,4);
	  sprintf(currentChannel,"%d",value);
	  cipcfg  = init_cipcfg_f(current_config);
	  if(!cipcfg){
	      cipcfg_print("initial cipcfg error.\n");
	      close_cipcfg(cipcfg);
	      return SNMP_ERR_WRONGVALUE;
	  }
	  else {
	      i =(int)*(name+8) ;
	      for(index =0 ; index < 100 ; index++) {
		  if( mapping_index[index] == i ) 
		      break;
	      }
	      rf = atoi(interface[index]+3)+1; 
	      sprintf(group_name,"%s%d","rf",rf);
	      memset(mode,0,MAX_VALUE_LEN);
	      ret = get_cfg_item_by_str(cipcfg, group_name, "mode", mode);
	      close_cipcfg(cipcfg);
	      if (ret != CIPCFG_SUCCESS) {
		  cipcfg_print("Can NOT get wireless mode, return value is %d.\n", ret);
		  return SNMP_ERR_WRONGVALUE;
	      }
	  }
	  //get wireless band mode , setting chan_len according band mode
	  if(strncmp(mode,"auto",4) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11b[i])
		      valid_value = 1;
		  if(cc.ch_11b[i] == -1)
		      break;
	      }
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11g[i])
		      valid_value=1;
		  if(cc.ch_11g[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11a",3) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11a[i])
		      valid_value=1;
		  if(cc.ch_11a[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11b",3) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11b[i])
		      valid_value=1;
		  if(cc.ch_11b[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11g",3) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11g[i])
		      valid_value=1;
		  if(cc.ch_11g[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11ng",4) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11ng[i])
		      valid_value=1;
		  if(cc.ch_11ng[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11ngu",5) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11ngu[i])
		      valid_value=1;
		  if(cc.ch_11ngu[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11ngl",5) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11ngl[i])
		      valid_value=1;
		  if(cc.ch_11ngl[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11na",4) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11na[i])
		      valid_value=1;
		  if(cc.ch_11na[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11nau",5) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11nau[i])
		      valid_value=1;
		  if(cc.ch_11nau[i] == -1)
		      break;
	      }
	  }
	  else if(strncmp(mode,"11nal",5) == 0) {
	      for(i = 0 ; i < 32 ; i++) {
		  if(value == cc.ch_11nal[i])
		      valid_value=1;
		  if(cc.ch_11nal[i] == -1)
		      break;
	      }
	  }
	  if (valid_value == 0) {
	      fprintf ( stderr, "write to dot11CurrentChannel error value\n" );
	      return SNMP_ERR_WRONGVALUE;
	  }
	  else if(valid_value == 1){
	      pid = fork();
	      if ( pid == 0 ) {
		/* child process */
		gid = gid_rf[rf -1];
		  cipcfg  = init_cipcfg_f(config_ram);
		  if(cipcfg != NULL){
		      list    = append_cfg_item(NULL, gid, "channel", currentChannel);
		      print_list(list);
		      ret=set_cfg_items(cipcfg, list);
		      if (ret != CIPCFG_SUCCESS) {
			  printf("Set cipcfg fault..%d\n",ret);
			  close_cipcfg(cipcfg);
			  exit(1);
		      }
		      close_cipcfg(cipcfg);
		      exit(0);
		  }
		exit(0);
	      }
	      else if ( pid > 0 ) {
		  printf("parent process return\n");
	      }
	      else
		  printf("fork fail\n");
	  }

      break;

    case FREE:
    case ACTION:
    case UNDO:
      break;

    case COMMIT:
      break;
  }

  return SNMP_ERR_NOERROR;
}


/****************************************************************************
*                                                                           *
*                      loadTables() - Load the Tables                       *
*                                                                           *
****************************************************************************/
//hochange
static void loadTables()
{
  int skfd,i=0;                               // generic raw socket desc
  struct iwreq wrq;                       // ioctl request structure
  struct ifreq ifr;
  struct timeval et;                      // elapsed time
  struct wireless_info info;              // workarea for wireless ioctl information
  FILE *fp;
  char  bfr[1024], ifName[1024];
  char *s, *t;

  gettimeofday ( &et, ( struct timezone * ) 0 );  // get time-of-day
  if ( et.tv_sec < lastLoad + MINLOADFREQ )       // only reload so often
    return;
  lastLoad = et.tv_sec;

  skfd = openSocket();                            // open socket
  if ( skfd < 0 ) {
    syslog ( LOG_ERR, "SNMP ieee802dot11.loadTables() - %s\n", "socket open failure" );
    return;
  }

  flushLists();

  // find interfaces in /proc/net/dev and find the wireless interfaces
  fp = fopen ( PROC_NET_DEV, "r" );
  if ( fp ) {
    while ( fgets ( bfr, sizeof ( bfr ), fp )) {
      if ( strstr ( bfr, ":" )) {
        s = bfr; t = ifName;
        while ( isspace ( *s ))                     // discard white space
          *s++;
        while ( *s != ':' )                         // get interface name
          *t++ = *s++;
        *t = '\0';
	strcpy(interface[i], ifName);
	mapping_index[i] = netsnmp_arch_interface_index_find(ifName);
	i++;
	// verify as a wireless device
	memset (( char * ) &info, 0, sizeof ( struct wireless_info ));
	strncpy ( wrq.ifr_name, ifName, IFNAMSIZ );
	//if ( ioctl ( skfd, SIOCGIWNAME, &wrq ) >= 0 && int_filter(skfd, ifName)) {
	if ( ioctl ( skfd, SIOCGIWNAME, &wrq ) >= 0 && (strstr(wrq.ifr_name,"ra")!=NULL || strstr(wrq.ifr_name,"wds")!=NULL) ) {
	    printf ( "%s ifName: %s\n", "loadTables() -", ifName );
	    initStructs();
	    loadWiExt( skfd, ifName, &info );
	    //displayWiExt ( info );
	    load80211Structs ( skfd, ifName, &info );
	}
      }
    }
    fclose ( fp );
  }
  close ( skfd );
}

/****************************************************************************
*                                                                           *
*              load80211Structs() - load the 802.11 structures              *
*                                                                           *
****************************************************************************/
static void 
load80211Structs ( int skfd, char *ifName, struct wireless_info *wi )
{
  int rc, ifIndex = 0;
  struct ifreq ifr;
  char  MACAddress [ MACADDR_LEN + 1 ];

  strcpy ( ifr.ifr_name, ifName );
  rc = ioctl ( skfd, SIOCGIFHWADDR, &ifr );
  if ( rc >= 0 ) {

    sprintf ( MACAddress, "%02X:%02X:%02X:%02X:%02X:%02X\0", 
                 ( UCHAR ) ifr.ifr_hwaddr.sa_data[0], ( UCHAR ) ifr.ifr_hwaddr.sa_data[1], 
                 ( UCHAR ) ifr.ifr_hwaddr.sa_data[2], ( UCHAR ) ifr.ifr_hwaddr.sa_data[3], 
                 ( UCHAR ) ifr.ifr_hwaddr.sa_data[4], ( UCHAR ) ifr.ifr_hwaddr.sa_data[5] );

    nSc.haveStationID = TRUE;
    strcpy  ( nSc.stationID, MACAddress );
    nOp.haveMACAddress = TRUE;
    strcpy  ( nOp.MACAddress, MACAddress );
    nRi.haveManufacturerOUI = TRUE;
    strncpy ( nRi.manufacturerOUI, MACAddress, MAN_OUI_LEN ); 

    ifIndex = if_nametoindex ( ifName );
    if ( !ifIndex ) {
      syslog ( LOG_ERR, "SNMP %s - %s %s\n", 
        "ieee802dot11.load80211Structs()", ifName, "has no ifIndex" );
      return;
    }

    loadWiExtTo80211Structs ( ifIndex, ifName, wi );

    if ( hasChanged (( char * ) &nSc, sizeof ( nSc ))) {
      nSc.ifIndex = ifIndex;
      sprintf ( nSc.UID, "%04d\0", nSc.ifIndex );
      strcpy ( nSc.ifName, ifName );
      addList (( char * ) &scList, ( char * ) &nSc, sizeof ( nSc ));
    }

    if ( hasChanged (( char * ) &nPr, sizeof ( nPr ))) {
      nPr.ifIndex = ifIndex;
      sprintf ( nPr.UID, "%04d\0", nPr.ifIndex );
      strcpy ( nPr.ifName, ifName );
      addList (( char * ) &prList, ( char * ) &nPr, sizeof ( nPr ));
    }

    if ( hasChanged (( char * ) &nOp, sizeof ( nOp ))) {
      nOp.ifIndex = ifIndex;
      sprintf ( nOp.UID, "%04d\0", nOp.ifIndex );
      strcpy ( nOp.ifName, ifName );
      addList (( char * ) &opList, ( char * ) &nOp, sizeof ( nOp ));
    }

    if ( hasChanged (( char * ) &nRi, sizeof ( nRi ))) {
      nRi.ifIndex = ifIndex;
      sprintf ( nRi.UID, "%04d\0", nRi.ifIndex );
      strcpy ( nRi.ifName, ifName );
      addList (( char * ) &riList, ( char * ) &nRi, sizeof ( nRi ));
    }

    if ( hasChanged (( char * ) &nPo, sizeof ( nPo ))) {
      nPo.ifIndex = ifIndex;
      sprintf ( nPo.UID, "%04d\0", nPo.ifIndex );
      strcpy ( nPo.ifName, ifName );
      addList (( char * ) &poList, ( char * ) &nPo, sizeof ( nPo ));
    }

    if ( hasChanged (( char * ) &nPd, sizeof ( nPd ))) {
      nPd.ifIndex = ifIndex;
      sprintf ( nPd.UID, "%04d\0", nPd.ifIndex );
      strcpy ( nPd.ifName, ifName );
      addList (( char * ) &pdList, ( char * ) &nPd, sizeof ( nPd ));
    }
  }

//printf ( "%s - ifIndex: %d ifName: %s UID: %s\n", 
//         "load80211Structs() - HASCHANGED", ifIndex, ifName, nSc.UID );
}

/****************************************************************************
*                                                                           *
*                     initStructs() - initialize structures                 *
*                                                                           *
****************************************************************************/
static void initStructs()
{
  int i;

  // 802.11 MIB Stuctures
  memset (( char * ) &nSc, 0, sizeof ( nSc ));  memset (( char * ) &nAa, 0, sizeof ( nAa ));
  memset (( char * ) &nDf, 0, sizeof ( nDf ));  memset (( char * ) &nRr, 0, sizeof ( nRr ));
  memset (( char * ) &nPr, 0, sizeof ( nPr ));  memset (( char * ) &nOp, 0, sizeof ( nOp ));
  memset (( char * ) &nRi, 0, sizeof ( nRi ));  memset (( char * ) &nPo, 0, sizeof ( nPo ));
  memset (( char * ) &nPd, 0, sizeof ( nPd ));  memset (( char * ) &nRt, 0, sizeof ( nRt ));
  // Wireless Extensions
  wepCurrentKey = 0;
  haveWepCurrentKey = FALSE;
  for ( i = 0; i < MAX_WEP_KEYS; i++ ) {
    wep[i].len = 0;
    wep[i].key[0] = '\0';
    wep[i].haveKey = FALSE;
  }
}

/****************************************************************************
*                                                                           *
*                Wireless Extensions Specific Functions                     *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
* loadWiExtTo80211Structs() - load wireless extensions to 802.11 structures *
*                                                                           *
****************************************************************************/
static void 
loadWiExtTo80211Structs ( int ifIndex, char *ifName, struct wireless_info *wi )
{
  int i, j = 0;
  int ret;
  ITEM_LIST       *list = NULL;
  CIPCFG * lcipcfg=NULL;
  char group_name[64];
  char item_name[20];

  nSc.havePrivacyOptionImplemented = TRUE;
  nSc.privacyOptionImplemented = 1;           // assume we support WEP

  if ( wi->has_essid && strlen ( wi->essid )) {
    nSc.haveDesiredSSID = TRUE;
    strcpy ( nSc.desiredSSID, wi->essid ); 
  }
  lcipcfg=rt_get_cipcfg();
  if(lcipcfg != NULL) {
      memset(group_name ,0 ,sizeof(group_name));
      strcpy(group_name ,"opmode");
      sprintf(item_name,"%s","type");
      ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
      if(strstr(retval,"AP") != NULL) {
	  //group_name = rf1
	  memset(group_name ,0 ,sizeof(group_name));
	  strcpy(group_name ,"rf1");
	  sprintf(item_name ,"%s" ,"channel");
	  ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	  if (ret != CIPCFG_SUCCESS) {
	      printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	      return;
	  }
	  else {
	    nPd.haveCurrentChannel = TRUE;
	    nPd.currentChannel = atoi(retval); 
	  }  
	    
	  //group_name = rf1_vapx
	  memset(group_name ,0 ,sizeof(group_name));
	  if (strstr(ifName, "wds") == NULL) {
	    strcpy(group_name ,"rf1_vap");
	    //check vap
	    switch( atoi(ifName+2))
	    {
	      case 0:
		    strcat(group_name , "1");
		    break;
	      case 1:
		    strcat(group_name , "2");
		    break;
	      case 2:
		    strcat(group_name , "3");
		    break;
	      case 3:
		    strcat(group_name , "4");
		    break;
	      case 4:
		    strcat(group_name , "5");
		    break;
	      case 5:
		    strcat(group_name , "6");
		    break;
	      case 6:
		    strcat(group_name , "7");
		    break;
	      case 7:
		    strcat(group_name , "8");
		    break;
	    }
	    nSc.haveBeaconPeriod = TRUE;
	    sprintf(item_name ,"%s" ,"beacon");
	    ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	    if (ret != CIPCFG_SUCCESS) {
	        printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	        return;
	    }
	    //printf("ret:%d,group_name:%s,item_read:%s\n",ret,group_name,read);
	    nSc.beaconPeriod = atoi(retval);

	    sprintf(item_name ,"%s" ,"rts_threshold");
	    ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	    if (ret != CIPCFG_SUCCESS) {
	        printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	        return;
	    }
	    else {
	        nOp.haveRTSThreshold = TRUE;
	        nOp.RTSThreshold = atoi(retval);
	    }

	    sprintf(item_name ,"%s" ,"frag_threshold");
	    ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	    if (ret != CIPCFG_SUCCESS) {
	        printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	        return;
	    }
	    else {
	        nOp.haveFragmentationThreshold = TRUE;
	        nOp.fragmentationThreshold = atoi(retval);
	    }
	  }
	}
	else if(strstr(retval ,"CPE") != NULL){
	  memset(group_name ,0 ,sizeof(group_name));
	  strcpy(group_name ,"rf1_sta1");
	  sprintf(item_name ,"%s" ,"rts_threshold");
	  ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	  if (ret != CIPCFG_SUCCESS) {
	      printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	      return;
	  }
	  else {
	      nOp.haveRTSThreshold = TRUE;
	      nOp.RTSThreshold = atoi(retval);
	  }

	  sprintf(item_name ,"%s" ,"frag_threshold");
	  ret = get_cfg_item_by_str(lcipcfg, group_name, item_name, retval);
	  if (ret != CIPCFG_SUCCESS) {
	      printf("Wrong gid:'%s' or item_name:'%s'\n", group_name, item_name);
	      return;
	  }
	  else {
	      nOp.haveFragmentationThreshold = TRUE;
	      nOp.fragmentationThreshold = atoi(retval);
	  }
	}
  }
  if ( wi->has_mode ) {
    nSc.haveDesiredBSSType = TRUE;
    if ( wi->mode == IW_MODE_ADHOC ) 
      nSc.desiredBSSType = 2;         // independent
    else if ( wi->has_ap_addr )
      nSc.desiredBSSType = 1;         // infrastructure
    else
      nSc.desiredBSSType = 3;         // any
  }

  if ( wi->has_range ) {
    for ( i = 0; i < wi->range.num_bitrates && j < 126; i++ ) {
      nSc.haveOperationalRateSet = TRUE;
      nSc.operationalRateSet[j++] = ( char ) ( wi->range.bitrate[i] / 500000L );
    }
  }

  // dot11AuthenticationAlgorithmsTable
  nAa.haveAuthenticationAlgorithm = TRUE;           // it's a rule to always have 
  nAa.haveAuthenticationAlgorithmsEnable = TRUE;    //    'open' supported
  nAa.ifIndex = ifIndex;
  nAa.authenticationAlgorithmsIndex = 1;            // index number one
  nAa.authenticationAlgorithm = 1;                  // 1 => open key
  sprintf ( nAa.UID, "%04d%04d\0", nAa.ifIndex, nAa.authenticationAlgorithmsIndex );
  nAa.authenticationAlgorithmsEnable = 1;           // enabled by default
  if ( ( wi->has_key                        ) &&
       ( wi->key_size  != 0                 ) &&
      !( wi->key_flags & IW_ENCODE_DISABLED ))
    nAa.authenticationAlgorithmsEnable = 2;
  addList (( char * ) &aaList, ( char * ) &nAa, sizeof ( nAa ));

  nAa.haveAuthenticationAlgorithm = TRUE;           // I'm gonna assume we always support WEP
  nAa.haveAuthenticationAlgorithmsEnable = TRUE;
  nAa.ifIndex = ifIndex;
  nAa.authenticationAlgorithmsIndex = 2;            // index number 2
  nAa.authenticationAlgorithm = 2;                  // 2 => shared key
  sprintf ( nAa.UID, "%04d%04d\0", nAa.ifIndex, nAa.authenticationAlgorithmsIndex );
  nAa.authenticationAlgorithmsEnable = 2;
  if ( ( wi->has_key                        ) &&
       ( wi->key_size  != 0                 ) &&
      !( wi->key_flags & IW_ENCODE_DISABLED ))
    nAa.authenticationAlgorithmsEnable = 1;         // disabled by default
  addList (( char * ) &aaList, ( char * ) &nAa, sizeof ( nAa ));
  //dot11WEPDefaultKeysTable
  if ( wi->has_range ) {
    for ( i = 0; i < MAX_WEP_KEYS; i++ ) {
      nDf.haveWEPDefaultKeyValue = TRUE;
      nDf.ifIndex = ifIndex;
      nDf.WEPDefaultKeyIndex = i + 1;               // index number
      sprintf ( nDf.UID, "%04d%04d\0", nDf.ifIndex, nDf.WEPDefaultKeyIndex );
      if ( wep[i].haveKey )
        strcpy ( nDf.WEPDefaultKeyValue, "*****" );
      else
        nDf.WEPDefaultKeyValue[0] = '\0';
      addList (( char * ) &dfList, ( char * ) &nDf, sizeof ( nDf ));
    }
  }

  // dot11PrivacyTable
  nPr.havePrivacyInvoked = TRUE;
  nPr.privacyInvoked = 2;                   // 2 => FALSE
  nPr.haveWEPDefaultKeyID = TRUE;
  nPr.WEPDefaultKeyID = 0;
  nPr.haveExcludeUnencrypted = TRUE;
  nPr.excludeUnencrypted = 2;               // 2 => FALSE
  if ( wi->has_range ) {
    if ( ( wi->key_size != 0 ) &&
        !( wi->key_flags & IW_ENCODE_DISABLED )) {
       nPr.privacyInvoked = 1;
       if ( wi->key_flags & IW_ENCODE_RESTRICTED )
          nPr.excludeUnencrypted = 1;
       nPr.WEPDefaultKeyID = wepCurrentKey;
    }
  }


  // dot11Phy Group
  // dot11PhyOperationTable
  if ( strstr ( wi->name, "IEEE 802.11-FS"      )) nPo.PHYType = 1;   // So what if I
  if ( strstr ( wi->name, "IEEE 802.11-DS"      )) nPo.PHYType = 2;   // made up a couple?
  if ( strstr ( wi->name, "IEEE 802.11-IR"      )) nPo.PHYType = 3;   
  if ( strstr ( wi->name, "IEEE 802.11-OFDM"    )) nPo.PHYType = 4;   // 802.11a
  if ( strstr ( wi->name, "IEEE 802.11-OFDM/DS" )) nPo.PHYType = 5;   // 802.11g
  if ( strstr ( wi->name, "IEEE 802.11-TURBO"   )) nPo.PHYType = 6;   // Atheros TURBO mode
  if ( nPo.PHYType ) nPo.havePHYType = TRUE;

  // dot11SupportedDataRatesTxTable
  if ( wi->has_range ) {
    for ( i = 0; i < wi->range.num_bitrates; i++ ) {
      nRt.ifIndex = ifIndex;
      nRt.supportedDataRatesTxIndex = i + 1;
      nRt.supportedDataRatesTxValue = wi->range.bitrate[i] / 500000L;
      nRt.haveSupportedDataRatesTxValue = TRUE;
      sprintf ( nRt.UID, "%04d%04d\0", nRt.ifIndex, nRt.supportedDataRatesTxIndex );
      strcpy ( nRt.ifName, ifName );
      addList (( char * ) &rtList, ( char * ) &nRt, sizeof ( nRt ));
    }
  }

  // dot11SupportedDataRatesRxTable
  if ( wi->has_range ) {
    for ( i = 0; i < wi->range.num_bitrates; i++ ) {
      nRr.ifIndex = ifIndex;
      nRr.supportedDataRatesRxIndex = i + 1;
      nRr.supportedDataRatesRxValue = wi->range.bitrate[i] / 500000L;
      nRr.haveSupportedDataRatesRxValue = TRUE;
      sprintf ( nRr.UID, "%04d%04d\0", nRr.ifIndex, nRr.supportedDataRatesRxIndex );
      strcpy ( nRr.ifName, ifName );
      addList (( char * ) &rrList, ( char * ) &nRr, sizeof ( nRr ));
    }
  }

//printf ( "%s max_encoding_tokens: %d\n", 
//          "loadWiExtTo80211Structs() - ", wi->range.max_encoding_tokens );
}

/****************************************************************************
*                                                                           *
*      loadWiExt() - load wireless extensions structures;                   *
*                    use ioctl calls and read /proc/net/wireless            *
*                                                                           *
****************************************************************************/
static void loadWiExt ( int skfd, char *ifname, struct wireless_info *wi )
{
  struct iwreq wrq;                       // ioctl request structure
  FILE *fp;
  char  bfr[1024];
  char  buffer[sizeof ( iwrange ) * 2]; /* Large enough */ //iwrange:564 , sizeof buffter:1128
  char *s, *t;
  int i, j,ret=0;

  strncpy ( wrq.ifr_name, ifname, IFNAMSIZ );

  /* Get wireless name */
  if ( ret=ioctl( skfd, SIOCGIWNAME, &wrq ) >= 0 ) {
    strncpy ( wi->name, wrq.u.name, IFNAMSIZ );
    wi->name[IFNAMSIZ] = '\0';
  }

  /* Get ranges */    // NOTE: some version checking in iwlib.c
  memset ( buffer, 0, sizeof ( buffer ));
  wrq.u.data.pointer = ( caddr_t ) &buffer;
  wrq.u.data.length = sizeof ( buffer );
  wrq.u.data.flags = 0;
  if ( ioctl ( skfd, SIOCGIWRANGE, &wrq ) >= 0 ) {
    memcpy (( char * ) &wi->range, buffer, sizeof ( iwrange ));
    wi->has_range = 1;
  }

  /* Get frequency / channel */         // THIS NUMBER LOOKS FUNNY
  if ( ioctl ( skfd, SIOCGIWFREQ, &wrq ) >= 0 ) {
    wi->has_freq = 1;
    wi->freq = (( double ) wrq.u.freq.m ) * pow ( 10, wrq.u.freq.e );
  }
  if (strstr(ifname, "wds") == NULL) {
    /* Get ESSID */
    wrq.u.essid.pointer = ( caddr_t ) &wi->essid;
    wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
    wrq.u.essid.flags = 0;
    if ( ioctl ( skfd, SIOCGIWESSID, &wrq ) >= 0 ) {
      wi->has_essid = 1;
      wi->essid_on = wrq.u.data.flags;
    }
  }

  /* Get AP address */
  if ( ioctl ( skfd, SIOCGIWAP, &wrq ) >= 0 ) {
    wi->has_ap_addr = 1;
    memcpy ( &wi->ap_addr, &wrq.u.ap_addr, sizeof ( sockaddr ));
  }

  /* Get bit rate */
  if ( ioctl ( skfd, SIOCGIWRATE, &wrq ) >= 0 ) {
    wi->has_bitrate = 1;
    memcpy ( &wi->bitrate, &wrq.u.bitrate, sizeof ( iwparam ));
  }

  /* Get operation mode */
  if ( ioctl ( skfd, SIOCGIWMODE, &wrq ) >= 0 ) {
      wi->mode = wrq.u.mode;
      if ( wi->mode < IW_NUM_OPER_MODE && wi->mode >= 0 )
        wi->has_mode = 1;
  }

}


/****************************************************************************
*                                                                           *
*                        Linked List Functions                              *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
*                addList() - add an entry to a linked list                  *
*                                                                           *
****************************************************************************/
static void 
addList ( char *l, char *data, int len  )
{
  char uid[256];
  LIST_HEAD ( , avNode ) *list;       

  // NOTE: this assumes the UID is at the begining of the 
  //       data structure and that UIDs are strings
  
  list = ( LIST_HEAD ( , avNode ) * ) l;            // NOTE: don't know how to get 
  strcpy ( uid, data );                             //  rid of compiler warning on
                                                    //  LISTHEAD typecast
  // create a new node and the data that goes in it
  newNode = malloc ( sizeof ( struct avNode ));
  newNode->data = malloc ( len );
  memcpy ( newNode->data, data, len );

  // this deals with an empty list
  if ( LIST_EMPTY ( list )) {
    LIST_INSERT_HEAD ( list, newNode, nodes );
    return;
  }

  // this deals with UIDs that match
  for ( np = LIST_FIRST ( list ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    if ( strncmp ( uid, np->data, strlen ( uid )) == 0 ) {                      // found matching UID
      LIST_INSERT_AFTER ( np, newNode, nodes );
      if ( np->data )
        free ( np->data );
      LIST_REMOVE ( np, nodes );
      free ( np );
      return;
    }
  }

  // this deals with inserting a new UID in the list
  for ( np = LIST_FIRST ( list ); np != NULL; np = LIST_NEXT ( np, nodes )) {
    lastNode = np;
    if ( strncmp ( np->data, uid, strlen ( uid )) > 0 ) {                       // old ID > new ID AND
      LIST_INSERT_BEFORE ( np, newNode, nodes );
      return;
    }
  }

  // this deals with a UID that needs to go on the end of the list
  LIST_INSERT_AFTER ( lastNode, newNode, nodes );

  return;
}

/****************************************************************************
*                                                                           *
*              initLists() - initialize all the linked lists                *
*                                                                           *
****************************************************************************/
static void initLists()
{
  LIST_INIT ( &scList );  LIST_INIT ( &aaList );  LIST_INIT ( &dfList );
  LIST_INIT ( &kmList );  LIST_INIT ( &prList );
  LIST_INIT ( &opList );  LIST_INIT ( &coList );
  LIST_INIT ( &gaList );  LIST_INIT ( &riList );  LIST_INIT ( &poList );
  LIST_INIT ( &paList );  LIST_INIT ( &ptList );  LIST_INIT ( &pfList );
  LIST_INIT ( &pdList );  LIST_INIT ( &piList );  LIST_INIT ( &rdList );
  LIST_INIT ( &alList );  LIST_INIT ( &rtList );  LIST_INIT ( &rrList );
}
/****************************************************************************
*                                                                           *
*                 flushLists() - flush all linked lists                     *
*                                                                           *
****************************************************************************/
static void flushLists()
{
  flushList (( char * ) &scList );  flushList (( char * ) &aaList );
  flushList (( char * ) &dfList );  flushList (( char * ) &kmList );
  flushList (( char * ) &prList );
  flushList (( char * ) &opList );  flushList (( char * ) &coList );
  flushList (( char * ) &gaList );  flushList (( char * ) &riList );
  flushList (( char * ) &poList );  flushList (( char * ) &paList );
  flushList (( char * ) &ptList );  flushList (( char * ) &pfList );
  flushList (( char * ) &pdList );  flushList (( char * ) &piList );
  flushList (( char * ) &rdList );  flushList (( char * ) &alList );
  flushList (( char * ) &rtList );  flushList (( char * ) &rrList );
}

/****************************************************************************
*                                                                           *
*                   flushList() - flush a linked list                       *
*                                                                           *
****************************************************************************/
static void flushList ( char *l )
{
  LIST_HEAD ( , avNode ) *list;
  
  list = ( LIST_HEAD ( , avNode ) * ) l;    // NOTE: don't know how to get 
  while ( !LIST_EMPTY ( list )) {           //  rid of compiler warning on
    np = LIST_FIRST ( list );               //  LISTHEAD typecast
    if ( np->data )
      free ( np->data );
    LIST_REMOVE ( np, nodes );
    free ( np );
  }
}

/****************************************************************************
*                                                                           *
*                            Utility Functions                              *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
*        The following two routines were taken directly from iwlib.c        *
*                                                                           *
****************************************************************************/
 /*
 * Open a socket.
 * Depending on the protocol present, open the right socket. The socket
 * will allow us to talk to the driver.
 */
static int openSocket ( void )
{
  static const int families[] = {
    AF_INET, AF_IPX, AF_AX25, AF_APPLETALK
  };
  unsigned int  i;
  int   sock;

  /*
   * Now pick any (exisiting) useful socket family for generic queries
   * Note : don't open all the socket, only returns when one matches,
   * all protocols might not be valid.
   * Workaround by Jim Kaba <jkaba@sarnoff.com>
   * Note : in 99% of the case, we will just open the inet_sock.
   * The remaining 1% case are not fully correct...
   */

  /* Try all families we support */
  for(i = 0; i < sizeof(families)/sizeof(int); ++i) {
      /* Try to open the socket, if success returns it */
      sock = socket(families[i], SOCK_DGRAM, 0);
      if(sock >= 0)
  return sock;
  }

  return -1;
}


/****************************************************************************
*                                                                           *
*                 htob - converts hex string to binary                      *
*                                                                           *
****************************************************************************/
static char *htob ( char *s )
{
    char nibl, *byt;
    static char bin[20];

    byt = bin;

    while ((nibl = *s++) && nibl != ' ') {    /* While not end of string. */
      nibl -= ( nibl > '9') ?  ('A' - 10): '0';
      *byt = nibl << 4;                              /* place high nibble */
      if((nibl = *s++) && nibl != ' ') {
        nibl -= ( nibl > '9') ?  ('A' - 10): '0';
        *byt |= nibl;                                /*  place low nibble */
      }
      else break;
      ++byt;
    }
    *++byt = '\0';
    return ( bin );
}

/****************************************************************************
*                                                                           *
*           hasChanged() - see if area has been changed from NULLs          *
*                                                                           *
****************************************************************************/
static int hasChanged ( char *loc, int len )
{
  char *wrk;
  int changed = TRUE;

  wrk = malloc ( len );
  memset ( wrk, 0, len );
  if ( memcmp ( loc, wrk, len ) == 0 )
    changed = FALSE;
  free ( wrk );

  return ( changed );
}

void child_handler(int signum, siginfo_t *info, void *context)
{
    pid_t pid;
    int status;
    pid = wait(&status);
    printf("Child process %d terminated\n", pid);
    if (info->si_code == CLD_DUMPED) {
        /* Segmentation fault */
        printf("Child process segmentation fault\n");
    } else if (info->si_code == CLD_EXITED && WEXITSTATUS(status) != 0) {
        /* Exit failure */
    } else {
        printf("ieee802dot11 Exit normally\n");
    }

}

/****************************************************************************
 *									     *
 * rt_conf_init_handler() - while getting  SIGUSR2 signal , set a flag     *
 *			      to remind doing cipcfg initialization	     *
 *                                                                           *
 ****************************************************************************/
void rt_conf_init_handler(int signum, siginfo_t *info, void *context)
{
    /* set reinit_rt_cipcfg = 1 */
    reinit_rt_cipcfg=1;
    printf("Get SIGUSR2!! reinit_rt_cipcfg:%d\n",reinit_rt_cipcfg);
}

/****************************************************************************
 *									     *
 * rt_get_cipcfg() - check whether the flag is set , if the flag is set	     *
 *		  do init_cipcfg() for the new configuration		     *
 *                                                                           *
 ****************************************************************************/
CIPCFG * rt_get_cipcfg()
{
    if(reinit_rt_cipcfg) {
	printf("Start reinit cipcfg\n");
	close_cipcfg(rt_cipcfg);
	rt_cipcfg = init_cipcfg_f(current_config);
	reinit_rt_cipcfg = 0;
    }
    return rt_cipcfg;	//cipcfg is globe variable
}
