#include "common.h"
#include "sysinfo.h"
#include "rflist.h"
#include "vaplist.h"
#include "wdslist.h"
#include "scanlist.h"
#include "stalist.h"
#include "ifstat.h"
#include "agent.h"

#define VERSION "1.0"

extern struct agent_data report_agent;

typedef struct REPORT{
    enum report_type rep_type; 
    char *type_input;
}REPORT;

const REPORT report_list[] = {
    {ALL_TYPE,"all"}, {SYSINFO_TYPE,"sysinfo"}, {RFLIST_TYPE,"rflist"}, {VAPLIST_TYPE,"vaplist"},
    {WDSLIST_TYPE,"wdslist"}, {SCANLIST_TYPE,"scanlist"}, {STALIST_TYPE,"stalist"}, {IFSTAT_TYPE,"ifstat"}, { -1, NULL}
};

#ifdef JSON_STRING_TO_ARRAY 
char **parse_reptype(char *string)
{
    int i, j;
    char **type;
    int type_idx=0;
    type = malloc(sizeof(char *) * CLI_MAX_STATUS_NUM);
    i=0;

    if(string == NULL)
        return NULL;
    while(i < strlen(string))
    {
        if(string[i] == '"')
        {
            type[type_idx] = malloc(sizeof(char) * CLI_STAT_MAXLEN);
            memset(type[type_idx], '\0', CLI_STAT_MAXLEN);
            type[type_idx+1] = NULL;
            i++;
            j=0;
            /* characters between two '"' */
            while(i+j < strlen(string) && string[i+j] != '"')
            {
                j++;
            }
            strncpy(type[type_idx], string+i, j);
            type_idx++;
            i = i+j;
        }
        i++;
    }
    return type;
}

char **repent_get_support_type()
{
    struct repreq req;
    int i;
    char ***cli_stat_tbl;  // status table is a three-dimentional table 
    char **type_arr;

    req.rep_type =SYSINFO_TYPE;
    req.rep_format = REP_JSON_FORMAT;
    sprintf(req.rep_uid, "abcdefg");
    req.rep_method = REP_METHOD_PASSIVE;

    cli_stat_tbl = malloc(sizeof(char *) * CLI_STATUS_MAX_COL * CLI_STATUS_MAX_ROW);
    report_agent_get(&req);
    rep_agent_json_string_handler(req.rep_output, cli_stat_tbl);

    for(i=1;cli_stat_tbl[1][i];i++)
    {
        if(!strcmp(cli_stat_tbl[1][i], "reptypes"))
            type_arr = parse_reptype(cli_stat_tbl[2][i]);
    }
    free_cli_stat_tbl(cli_stat_tbl);
    return type_arr;
}
#endif

int getReportType(char *type)
{
    // match report type
    int i=0;
    
    while(report_list[i].type_input != NULL )
    {
        if (strcmp(type, report_list[i].type_input) == 0){
            break;
        }
        i++;
    }

    if(report_list[i].type_input == NULL)
	return -1;

    return report_list[i].rep_type;
}

int report_agent_get(struct repreq *req)
{
    char date[MAX_VALUE_LEN];
    struct tm now;
    SYSINFO *sysinfo_data = NULL;
    RFLIST rflist_h;
    VAPLIST vaplist_h;
    WDSLIST wdslist_h;
    SCANLIST scanlist_h;
    STALIST stalist_h;
    IFSTAT iflist_h;
    struct json_object *report_obj = NULL, *obj_arr = NULL;

    /*
     * Check UID. If don't sepcify the UID manually, use the MAC as its UID
     */
    if (req->rep_uid[0] == '\0')
	getUidFromMac(req->rep_uid);

    memset(&rflist_h, 0, sizeof(RFLIST));
    memset(&vaplist_h, 0, sizeof(VAPLIST));
    memset(&wdslist_h, 0, sizeof(WDSLIST));
    memset(&scanlist_h, 0, sizeof(SCANLIST));
    memset(&stalist_h, 0, sizeof(STALIST));
    memset(&iflist_h, 0, sizeof(IFSTAT));
    
    /* Get system time */
    get_stat_by_name("date", &now);
    strftime(date, MAX_VALUE_LEN, "%Y-%m-%d %H:%M:%S", &now);

    switch (req->rep_type) {
	case ALL_TYPE:
	    if (sysinfo_data == NULL)
		sysinfo_data = sysinfo_init();
	    get_sysinfo(&report_agent, sysinfo_data);
	    get_rflist(&report_agent, &rflist_h);
	    get_vaplist(&report_agent, &vaplist_h);
	    get_wdslist(&report_agent, &wdslist_h);
	    get_scanlist(&report_agent, &scanlist_h);
	    get_stalist(&report_agent, &stalist_h);
	    get_iflist(&report_agent, &iflist_h);
	    break;
	case SYSINFO_TYPE:
	    if (sysinfo_data == NULL)
		sysinfo_data = sysinfo_init();
	    get_sysinfo(&report_agent, sysinfo_data);
	    break;
	case RFLIST_TYPE:
	    get_rflist(&report_agent, &rflist_h);
	    break;
	case VAPLIST_TYPE:
	    get_vaplist(&report_agent, &vaplist_h);
	    break;
	case WDSLIST_TYPE:
	    get_wdslist(&report_agent, &wdslist_h);
	    break;
	case SCANLIST_TYPE:
	    get_scanlist(&report_agent, &scanlist_h);
	    break;
	case STALIST_TYPE:
	    get_stalist(&report_agent, &stalist_h);
	    break;
	case IFSTAT_TYPE:
	    get_iflist(&report_agent, &iflist_h);
	    break;
    }

    req->rep_output = NULL;
    switch (req->rep_format) {
	case REP_XML_FORMAT:
	    break;
	case REP_JSON_FORMAT:
	default:
	    report_obj = json_object_new_object();
	    obj_arr = json_object_new_array();
	    json_object_object_add(report_obj, "version",
		    json_object_new_string(VERSION));
	    json_object_object_add(report_obj, "UID",
		    json_object_new_string(req->rep_uid));
	    json_object_object_add(report_obj, "timestamp",
		    json_object_new_string(date));
	    json_object_object_add(report_obj, "report-method",
		    (req->rep_method == REP_METHOD_ACTIVE ? 
		     json_object_new_string("active") :
		     json_object_new_string("passive")
		     ));
	    json_object_object_add(report_obj, "aggregation",
		    json_object_new_boolean(0));
	    json_object_object_add(report_obj, "node-data", obj_arr);

	    if ((req->rep_type == ALL_TYPE || req->rep_type == SYSINFO_TYPE) 
		    && report_agent.has_sysinfo && sysinfo_data)
		json_object_array_add(obj_arr, 
			sysinfo_to_json(&report_agent, sysinfo_data));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == RFLIST_TYPE) 
		    && report_agent.has_rflist && rflist_h.next)
		json_object_array_add(obj_arr,
			rflist_to_json(&report_agent, &rflist_h));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == VAPLIST_TYPE) 
		    && report_agent.has_vaplist && vaplist_h.next)
		json_object_array_add(obj_arr,
			vaplist_to_json(&report_agent, &vaplist_h));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == WDSLIST_TYPE) 
		&& report_agent.has_wdslist && wdslist_h.next)
		json_object_array_add(obj_arr,
			wdslist_to_json(&report_agent, &wdslist_h));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == SCANLIST_TYPE) 
		    && report_agent.has_scanlist && scanlist_h.next)
		json_object_array_add(obj_arr,
			scanlist_to_json(&report_agent, &scanlist_h));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == STALIST_TYPE) 
		    && report_agent.has_stalist && stalist_h.next)
		json_object_array_add(obj_arr,
			stalist_to_json(&report_agent, &stalist_h));

	    if ((req->rep_type == ALL_TYPE || req->rep_type == IFSTAT_TYPE) 
		    && report_agent.has_ifstat && iflist_h.next)
		json_object_array_add(obj_arr,
			iflist_to_json(&report_agent, &iflist_h));

	    req->rep_output = strdup(json_object_to_json_string(report_obj));
	    json_object_put(report_obj);
    }

    sysinfo_free(sysinfo_data);
    rflist_free_list(&rflist_h.next);
    vaplist_free_list(&vaplist_h.next);
    wdslist_free_list(&wdslist_h.next);
    scanlist_free_list(&scanlist_h.next);
    stalist_free_list(&stalist_h.next);
    ifstat_free_list(&iflist_h.next);

    return 0;
}
