#include "common.h"

extern struct SYS_ITEM_LIST sys_item_list[];

int get_stat_by_name(char *item_name, void *item_value)
{
    int i;
    char *item_key;
    for (i=0; (sys_item_list[i].item != NULL); i++){
        if ( !strcmp(sys_item_list[i].item, item_name) ){
            item_key = sys_item_list[i].key ;
	    return sys_item_list[i].get_stat(item_key, item_value);
        }
    }

    return CIPCFG_ERR_UNKNOWN_ITEM;
}

int getIfInfo(char *ifname, struct ifreq *ifr)
{
    int s = socket(PF_INET, SOCK_STREAM, 0);
    
    strncpy(ifr->ifr_name, ifname, IFNAMSIZ);
    if (ioctl(s, SIOCGIFFLAGS, ifr) < 0) {
	fprintf(stderr, "No such interface: %s\n", ifr->ifr_name);
	close(s);
	return REPENT_ERR_COMMON;
    }
    close(s);
    return REPENT_SUCCESS;
}

int get_ifstat_by_ifname(char *ifname, struct ifstat *stat)
{
    FILE *fp;
    char buf[MAX_VALUE_LEN];
    char *interface;
    struct ifstat tmp;
    int i, j;

    if ((fp = fopen(PROCNET_DEV, "r")) == NULL)
	return REPENT_ERR_COMMON;

    /* Eat first two lines of /proc/net/dev */
    fgets(buf, MAX_VALUE_LEN, fp);
    fgets(buf, MAX_VALUE_LEN, fp);

    while(fgets(buf, MAX_VALUE_LEN, fp)) {
	interface = buf;
	for (i = 0; i < MAX_VALUE_LEN; i++) {
	    if (buf[i] != ' ') {
		interface = &buf[i];
		break;
	    }
	}
	for (j = i; j < (i + IFNAMSIZ); j++) {
	    if (buf[j] == ':') {
		buf[j] = '\0';
		break;
	    }
	}
	j++;
	if(sscanf(buf+j, "%llu %llu %llu %*llu %*llu %*llu %*llu %*llu %llu %llu %llu %*llu %*llu %*llu %*llu %*llu", &tmp.rxbyte, &tmp.rxcount, &tmp.rxerror, &tmp.txbyte, &tmp.txcount, &tmp.txerror) == 6) {
	    if (strcmp(interface, ifname) == 0) {
		stat->txbyte = tmp.txbyte;
		stat->txcount = tmp.txcount;
		stat->txerror = tmp.txerror;
		stat->rxbyte = tmp.rxbyte;
		stat->rxcount = tmp.rxcount;
		stat->rxerror = tmp.rxerror;
		fclose(fp);
		return REPENT_SUCCESS;
	    }
	}
    }

    fclose(fp);
    return REPENT_ERR_COMMON;
}

int getUidFromMac(char *uid)
{
    char mac[6];

    if (get_stat_by_name("wan_mac", mac) != CIPCFG_SUCCESS) {
	fprintf(stderr, "Can't get MAC address of WAN interface\n");
	return REPENT_ERR_COMMON;
    }
    sprintf(uid, "ID_%.2X%.2X%.2X%.2X%.2X%.2X",
	    (unsigned char)mac[0], 
	    (unsigned char)mac[1], 
	    (unsigned char)mac[2],
	    (unsigned char)mac[3], 
	    (unsigned char)mac[4], 
	    (unsigned char)mac[5]);

    return REPENT_SUCCESS;
}
