#include "rflist.h"

RFLIST head;

RFLIST *rflist_new(void)
{
    RFLIST *obj = malloc(sizeof(RFLIST));
    if (obj)
	memset(obj, 0, sizeof(RFLIST));
    return obj;
}

void rflist_free(RFLIST *info)
{
    if (info)
	free(info);
}

void rflist_free_list(RFLIST **list)
{
    RFLIST *cur = NULL, *next = NULL;

    if (list == NULL)
	return;

    for (cur = *list; cur; cur = next) {
	next = cur->next;
	rflist_free(cur);
    }
}

int get_rflist(struct agent_data *agent, RFLIST *head)
{
    int i, j, txpower, channel;
    RFLIST *node = NULL, *prev = head;
    char ifname[IFNAMSIZ + 1], buf[MAX_VALUE_LEN], value[MAX_VALUE_LEN];
    char vap_ifname[IFNAMSIZ + 1], wds_ifname[IFNAMSIZ + 1];
    struct ifreq ifr;
    CIPCFG *cipcfg = NULL;

    if (head == NULL)
	return CIPCFG_ERR_NODE_NOT_FOUND;

    cipcfg = init_cipcfg();

    for (i = 0; i < agent->max_rf; i++) {
	get_master_ifname_by_idx(i, 0, ifname);

	if (getIfInfo(ifname, &ifr) != REPENT_SUCCESS || 
		!(ifr.ifr_flags & IFF_UP))
	    continue;

	if ((node = rflist_new()) == NULL) 
	    continue;

	/* RF ID */
	node->rfid = i;

	/* hardware address */
	sprintf(buf, "rf%d_mac", i + 1);
	if (get_stat_by_name(buf, node->hwaddr) != CIPCFG_SUCCESS)
	    fprintf(stderr, "Can't get stat for %s\n", buf);

	/* ifname */
	strncpy(node->ifname, ifname, IFNAMSIZ + 1);

	/* txpower */
	txpower = 0;
	sprintf(buf, "rf%d_txpwr", i + 1);
	if (get_stat_by_name(buf, &txpower) != CIPCFG_SUCCESS)
	    fprintf(stderr, "Can't get stat for %s\n", buf);
	node->txpower = (short)txpower;

	/* channel */
	channel = 0;
	sprintf(buf, "rf%d_ch", i + 1);
	if (get_stat_by_name(buf, &channel) != CIPCFG_SUCCESS)
	    fprintf(stderr, "Can't get stat for %s\n", buf);
	node->channel = (unsigned short)channel;

	/* roles */
	sprintf(buf, "rf%d", i + 1);
	if (get_cfg_item_by_str(cipcfg, buf, "roles", value) 
		== CIPCFG_SUCCESS) {
	    if (strstr(value, "ap") != NULL)
		node->roles |= RFLIST_ROLES_AP;
	    if (strstr(value, "sta") != NULL)
		node->roles |= RFLIST_ROLES_STA;
	    if (strstr(value, "wds") != NULL)
		node->roles |= RFLIST_ROLES_WDS;
	    if (strstr(value, "rep") != NULL)
		node->roles |= RFLIST_ROLES_REP;
	}

	/* number of vap */
	for (j = 0; j < agent->max_vap; j++) {
	    struct ifreq ifr_vap;
	    get_vap_ifname_by_idx(i, j, vap_ifname);
	    if (getIfInfo(vap_ifname, &ifr_vap) != REPENT_SUCCESS || 
		    !(ifr_vap.ifr_flags & IFF_UP))
		continue;
	    node->n_vap++;
	}

	/* number of wds */
	for (j = 0; j < agent->max_wds; j++) {
	    struct ifreq ifr_wds;
	    get_wds_ifname_by_idx(i, j, wds_ifname);
	    if (getIfInfo(wds_ifname, &ifr_wds) != REPENT_SUCCESS || 
		    !(ifr_wds.ifr_flags & IFF_UP))
		continue;
	    node->n_wds++;
	}

	prev->next = node;
	prev = node;
    }

    close_cipcfg(cipcfg);
    return CIPCFG_SUCCESS;
}

struct json_object *rflist_to_json(struct agent_data *agent, RFLIST * head)
{
    RFLIST *node;
    struct json_object *obj = NULL, *rflist_obj_arr = NULL;
    char macaddr[18];

    rflist_obj_arr = json_object_new_array();
    for (node = head->next; node; node = node->next) {
	struct json_object *rflist_obj = NULL, *roles = NULL;

	roles = json_object_new_array();
	if (node->roles & RFLIST_ROLES_AP)
	    json_object_array_add(roles, json_object_new_string("AP"));
	if (node->roles & RFLIST_ROLES_WDS)
	    json_object_array_add(roles, json_object_new_string("WDS"));
	if (node->roles & RFLIST_ROLES_STA)
	    json_object_array_add(roles, json_object_new_string("STA"));
	if (node->roles & RFLIST_ROLES_REP)
	    json_object_array_add(roles, json_object_new_string("REP"));

	rflist_obj = json_object_new_object();
	json_object_object_add(rflist_obj, "rfid", 
		json_object_new_int(node->rfid));
	json_object_object_add(rflist_obj, "hwaddr", 
		json_object_new_string(mac2str(node->hwaddr, macaddr)));
	json_object_object_add(rflist_obj, "ifname", 
		json_object_new_string(node->ifname));
	json_object_object_add(rflist_obj, "txpower", 
		json_object_new_int(node->txpower));
	json_object_object_add(rflist_obj, "channel", 
		json_object_new_int(node->channel));
	json_object_object_add(rflist_obj, "roles", roles);
	json_object_object_add(rflist_obj, "n_vap", 
		json_object_new_int(node->n_vap));
	json_object_object_add(rflist_obj, "n_wds", 
		json_object_new_int(node->n_wds));

	json_object_array_add(rflist_obj_arr, rflist_obj);
    }

    obj = json_object_new_object();
    json_object_object_add(obj, "type", json_object_new_string("rflist"));
    json_object_object_add(obj, "data", rflist_obj_arr);

    return obj;
}
