#include "stalist.h"

STALIST *stalist_new(void)
{
    STALIST *obj = malloc(sizeof(STALIST));
    if (obj)
	memset(obj, 0, sizeof(STALIST));
    return obj;
}

void stalist_free(STALIST *info)
{
    if (info)
	free(info);
}

void stalist_free_list(STALIST **list)
{
    STALIST *cur = NULL, *next = NULL;

    if (list == NULL)
	return;

    for (cur = *list; cur; cur = next) {
	next = cur->next;
	stalist_free(cur);
    }
}

int get_stalist(struct agent_data *agent, STALIST *head)
{
    STALIST *node = NULL, *prev = head;
    char ifname[IFNAMSIZ + 1], buf[MAX_VALUE_LEN], value[MAX_VALUE_LEN];
    struct ifreq ifr;
    char *tok, del[] = ":";
    int i, rfid, vapid;

    if (head == NULL)
	return CIPCFG_ERR_NODE_NOT_FOUND;
    
    for (rfid = 0; rfid < agent->max_rf; rfid++) {
	for (vapid = 0; vapid < agent->max_vap; vapid++) {
	    ASSOC_STA_LIST *list = NULL, *cur = NULL;

	    get_vap_ifname_by_idx(rfid, vapid, ifname);
	    if (getIfInfo(ifname, &ifr) != REPENT_SUCCESS ||
		    !(ifr.ifr_flags & IFF_UP))
		continue;

	    if ((list = get_sta_list(ifname)) == NULL)
		continue;

	    foreach_sta_get(cur, list) {
		if ((node = stalist_new()) == NULL)
		    continue;

		/* RF ID */
		node->rfid = rfid;

		/* VAP ID */
		node->vapid = vapid;

		/* Station MAC */
		for (i = 0, tok = strtok(cur->mac, del); i < MACLEN && tok;
			i++, tok = strtok(NULL, del)) {
		    unsigned int tmp;
		    sscanf(tok, "%X", &tmp);
		    node->stamac[i] = tmp;
		}

		/* BSSID */
		sprintf(buf, "rf%d_vap%d_mac", rfid + 1, vapid + 1);
		if (get_stat_by_name(buf, node->bssid) != CIPCFG_SUCCESS)
		    fprintf(stderr, "Can't get stat for %s\n", buf);

		/* SNR */
		node->snr = cur->rssi;

		/* txrate */
		node->txrate = cur->tx_rate;

		/* TODO: rxrate */

		/* stat */
		node->stat.txbyte = cur->tx_byte;
		node->stat.txcount = (cur->tx_okcnt + cur->tx_errcnt);
		node->stat.txerror = cur->tx_errcnt;
		node->stat.rxbyte = cur->rx_byte;
		node->stat.rxcount = (cur->rx_okcnt + cur->rx_errcnt);
		node->stat.rxerror = cur->rx_errcnt;

		/* Idle time */
		node->idletime = cur->last_rx;

		prev->next = node;
		prev = node;
	    }
	}
    }

    return CIPCFG_SUCCESS;
}

struct json_object *stalist_to_json(struct agent_data *agent, STALIST *head)
{
    STALIST *node;
    struct json_object *obj = NULL, *stalist_obj_arr = NULL;
    char macaddr[18];

    stalist_obj_arr = json_object_new_array();
    for (node = head->next; node; node = node->next) {
	struct json_object *stalist_obj = NULL;

	stalist_obj = json_object_new_object();
	json_object_object_add(stalist_obj, "rfid",
		json_object_new_int(node->rfid));
	json_object_object_add(stalist_obj, "vapid",
		json_object_new_int(node->vapid));
	json_object_object_add(stalist_obj, "stamac",
		json_object_new_string(mac2str(node->stamac, macaddr)));
	json_object_object_add(stalist_obj, "bssid",
		json_object_new_string(mac2str(node->bssid, macaddr)));
	json_object_object_add(stalist_obj, "snr",
		json_object_new_int(node->snr));
	json_object_object_add(stalist_obj, "txrate",
		json_object_new_double(node->txrate));
	json_object_object_add(stalist_obj, "rxrate",
		json_object_new_double(node->rxrate));
	json_object_object_add(stalist_obj, "txbyte",
		json_object_new_int(node->stat.txbyte));
	json_object_object_add(stalist_obj, "txcount",
		json_object_new_int(node->stat.txcount));
	json_object_object_add(stalist_obj, "txerror",
		json_object_new_int(node->stat.txerror));
	json_object_object_add(stalist_obj, "rxbyte",
		json_object_new_int(node->stat.rxbyte));
	json_object_object_add(stalist_obj, "rxcount",
		json_object_new_int(node->stat.rxcount));
	json_object_object_add(stalist_obj, "rxerror",
		json_object_new_int(node->stat.rxerror));
	json_object_object_add(stalist_obj, "idletime",
		json_object_new_int(node->idletime));

	json_object_array_add(stalist_obj_arr, stalist_obj);
    }

    obj = json_object_new_object();
    json_object_object_add(obj, "type", json_object_new_string("stalist"));
    json_object_object_add(obj, "data", stalist_obj_arr);

    return obj;
}
