#define _GNU_SOURCE
#include "sysinfo.h"

SYSINFO sysinfo;

SYSINFO *sysinfo_init(void)
{
    SYSINFO *obj;

    obj = malloc(sizeof(SYSINFO));
    if (obj)
	memset(obj, 0, sizeof(SYSINFO));
    return obj;
}

void sysinfo_free(SYSINFO *info)
{
    if (!info)
	return;

#define free_member(p, m)	while (0) { if (p->m) free(p->m); }
    free_member(info, vendor);
    free_member(info, model);
    free_member(info, version);
    free_member(info, sysname);
    free_member(info, location);
    free_member(info, desc);

    free(info);
}

int get_sysinfo(struct agent_data *agent, SYSINFO *info)
{
    char value[MAX_VALUE_LEN];
    CIPCFG *cipcfg = NULL;
    int nphyif, nrfif, nethif;

    cipcfg = init_cipcfg();

    /* Get vendor */
    if (get_stat_by_name("company", value) == CIPCFG_SUCCESS)
	info->vendor = strndup(value, MAX_VALUE_LEN - 1);

    /* Get model */
    if (get_stat_by_name("model", value) == CIPCFG_SUCCESS)
	info->model = strndup(value, MAX_VALUE_LEN - 1);

    /* Get software version */
    if (get_stat_by_name("sw_ver", value) == CIPCFG_SUCCESS)
	info->version = strndup(value, MAX_VALUE_LEN - 1);

    /* Get system name */
    if (get_cfg_item_by_str(cipcfg, "device", "name", value) == CIPCFG_SUCCESS)
	info->sysname = strndup(value, MAX_VALUE_LEN - 1);

    /* Get location */
    if (get_cfg_item_by_str(cipcfg, "device", "location", value) == CIPCFG_SUCCESS)
	info->location = strndup(value, MAX_VALUE_LEN - 1);

    /* Get description */
    if (get_cfg_item_by_str(cipcfg, "device", "desc", value) == CIPCFG_SUCCESS)
	info->desc = strndup(value, MAX_VALUE_LEN - 1);

    /* Get number of all physical network interfaces */
    if (get_stat_by_name("phy_cnt", &nphyif) == CIPCFG_SUCCESS)
	info->nphyif = nphyif;
    
    /* Get number of physical RF interfaces */
    if (get_stat_by_name("rf_phy_cnt", &nrfif) == CIPCFG_SUCCESS)
	info->nrfif = nrfif;

    /* Get number of physical Ethernet interfaces */
    if (get_stat_by_name("ether_phy_cnt", &nethif) == CIPCFG_SUCCESS)
	info->nethif = nethif;

    /* Get the health of system */
    info->health = HEALTH_NORMAL;

    close_cipcfg(cipcfg);

    return CIPCFG_SUCCESS;
}

struct json_object *sysinfo_to_json(struct agent_data *agent, SYSINFO *info)
{
    struct json_object *report_type = NULL, *obj = NULL, *sysinfo_obj = NULL;

    report_type = json_object_new_array();

#define ADD_REPORT_TYPE_TO_ARR(array, type) \
    if (agent->has_##type) \
	json_object_array_add(array, json_object_new_string(#type))
    ADD_REPORT_TYPE_TO_ARR(report_type, sysinfo);
    ADD_REPORT_TYPE_TO_ARR(report_type, rflist);
    ADD_REPORT_TYPE_TO_ARR(report_type, vaplist);
    ADD_REPORT_TYPE_TO_ARR(report_type, wdslist);
    ADD_REPORT_TYPE_TO_ARR(report_type, scanlist);
    ADD_REPORT_TYPE_TO_ARR(report_type, stalist);
    ADD_REPORT_TYPE_TO_ARR(report_type, ifstat);

    sysinfo_obj = json_object_new_object();
    json_object_object_add(sysinfo_obj, "vendor", 
	    (info->vendor ? json_object_new_string(info->vendor) : NULL));
    json_object_object_add(sysinfo_obj, "model", 
	    (info->model ? json_object_new_string(info->model) : NULL));
    json_object_object_add(sysinfo_obj, "version", 
	    (info->version ? json_object_new_string(info->version) : NULL));
    json_object_object_add(sysinfo_obj, "sysname", 
	    (info->sysname ? json_object_new_string(info->sysname) : NULL));
    json_object_object_add(sysinfo_obj, "location", 
	    (info->location ? json_object_new_string(info->location) : NULL));
    json_object_object_add(sysinfo_obj, "description", 
	    (info->desc ? json_object_new_string(info->desc) : NULL));
    json_object_object_add(sysinfo_obj, "nphyif", 
	    json_object_new_int(info->nphyif));
    json_object_object_add(sysinfo_obj, "nrfif", 
	    json_object_new_int(info->nrfif));
    json_object_object_add(sysinfo_obj, "nethif", 
	    json_object_new_int(info->nethif));
    json_object_object_add(sysinfo_obj, "health", 
	    json_object_new_string("normal"));
    json_object_object_add(sysinfo_obj, "reptypes", report_type);

    obj = json_object_new_object();
    json_object_object_add(obj, "type", json_object_new_string("sysinfo"));
    json_object_object_add(obj, "data", sysinfo_obj);

    return obj;
}
