#include "vaplist.h"

VAPLIST *vaplist_new(void)
{
    VAPLIST *obj = malloc(sizeof(VAPLIST));
    if (obj)
	memset(obj, 0, sizeof(VAPLIST));
    return obj;
}

void vaplist_free(VAPLIST *info)
{
    if (!info)
	return;

    if (info->desc)
	free(info->desc);

    free(info);
}

void vaplist_free_list(VAPLIST **list)
{
    VAPLIST *cur = NULL, *next = NULL;

    if (list == NULL)
	return;

    for (cur = *list; cur; cur = next) {
	next = cur->next;
	vaplist_free(cur);
    }
}

int get_vaplist(struct agent_data *agent, VAPLIST *head)
{
    int i, rfid, sta_num;
    VAPLIST *node = NULL, *prev = head;
    char ifname[IFNAMSIZ + 1], buf[MAX_VALUE_LEN], value[MAX_VALUE_LEN];
    struct ifreq ifr;
    CIPCFG *cipcfg = NULL;
    struct ifstat stat;

    if (head == NULL)
	return CIPCFG_ERR_NODE_NOT_FOUND;

    cipcfg = init_cipcfg();

    for (rfid = 0; rfid < agent->max_rf; rfid++) {
	for (i = 0; i < agent->max_vap; i++) {
	    get_vap_ifname_by_idx(rfid, i, ifname);

	    if (getIfInfo(ifname, &ifr) != REPENT_SUCCESS ||
		    !(ifr.ifr_flags & IFF_UP))
		continue;

	    if ((node = vaplist_new()) == NULL)
		continue;

	    /* RF ID */
	    node->rfid = rfid;

	    /* VAP ID */
	    node->vapid = i;

	    /* ifname */
	    strncpy(node->ifname, ifname, IFNAMSIZ + 1);

	    /* BSSID */
	    sprintf(buf, "rf%d_vap%d_mac", rfid + 1, i + 1);
	    if (get_stat_by_name(buf, node->bssid) != CIPCFG_SUCCESS)
		fprintf(stderr, "Can't get stat for %s\n", buf);

	    /* ESSID */
	    sprintf(buf, "rf%d_vap%d", rfid + 1, i + 1);
	    if (get_cfg_item_by_str(cipcfg, buf, "ssid", value) == CIPCFG_SUCCESS) {
		strncpy(node->essid, value, SSIDLEN);
		node->essid[SSIDLEN] = '\0';
	    }

	    /* TODO: description */
	    node->desc = NULL;

	    /* Number of associated wireless stations */
	    sta_num = 0;
	    sprintf(buf, "rf%d_vap%d_sta", rfid + 1, i + 1);
	    if (get_stat_by_name(buf, &sta_num) != CIPCFG_SUCCESS)
		fprintf(stderr, "Can't get stat for %s\n", buf);
	    node->nsta = sta_num;

	    /* get stats */
	    if (get_ifstat_by_ifname(ifname, &stat) == REPENT_SUCCESS)
		node->stat = stat;

	    prev->next = node;
	    prev = node;
	}
    }

    close_cipcfg(cipcfg);
    return CIPCFG_SUCCESS;
};

struct json_object *vaplist_to_json(struct agent_data *agent, VAPLIST *head)
{
    VAPLIST *node;
    struct json_object *obj = NULL, *vaplist_obj_arr = NULL;
    char macaddr[18];

    vaplist_obj_arr = json_object_new_array();
    for (node = head->next; node; node = node->next) {
	struct json_object *vaplist_obj = NULL;

	vaplist_obj = json_object_new_object();
	json_object_object_add(vaplist_obj, "rfid",
		json_object_new_int(node->rfid));
	json_object_object_add(vaplist_obj, "vapid",
		json_object_new_int(node->vapid));
	json_object_object_add(vaplist_obj, "ifname",
		json_object_new_string(node->ifname));
	json_object_object_add(vaplist_obj, "bssid",
		json_object_new_string(mac2str(node->bssid, macaddr)));
	json_object_object_add(vaplist_obj, "essid",
		json_object_new_string(node->essid));
	json_object_object_add(vaplist_obj, "description",
		(node->desc ? json_object_new_string(node->desc) : NULL));
	json_object_object_add(vaplist_obj, "nsta",
		json_object_new_int(node->nsta));
	json_object_object_add(vaplist_obj, "txbyte",
		json_object_new_int(node->stat.txbyte));
	json_object_object_add(vaplist_obj, "txcount",
		json_object_new_int(node->stat.txcount));
	json_object_object_add(vaplist_obj, "txerror",
		json_object_new_int(node->stat.txerror));
	json_object_object_add(vaplist_obj, "rxbyte",
		json_object_new_int(node->stat.rxbyte));
	json_object_object_add(vaplist_obj, "rxcount",
		json_object_new_int(node->stat.rxcount));
	json_object_object_add(vaplist_obj, "rxerror",
		json_object_new_int(node->stat.rxerror));

	json_object_array_add(vaplist_obj_arr, vaplist_obj);
    }

    obj = json_object_new_object();
    json_object_object_add(obj, "type", json_object_new_string("vaplist"));
    json_object_object_add(obj, "data", vaplist_obj_arr);

    return obj;
}
