#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include "wdschk.h"

#define RF_PATH	"/db/wlan/"
#define WDS_PATH "wds/"

int get_num_dir(char *path)
{
    DIR *pdir = opendir(path);
    struct dirent *pent;
    struct stat dir_stat;
    int ret = 0;

    if (pdir) {
	chdir(path);
	while (pent = readdir(pdir)) {
            if (stat(pent->d_name, &dir_stat) == -1)
                continue;
            if ( S_ISDIR(dir_stat.st_mode) && 
		(strcmp(pent->d_name, ".") && strcmp(pent->d_name, "..")))
		ret++;
	}
    }

    return ret;
}

static void trim(char *val)
{
    int i;
    char nval[512];

    i=strlen(val);
    while(--i >= 0) {
	if((val[i] == ' ') || (val[i] == '\t') || (val[i] == '\n') || val[i] == '"')
	    val[i]=0;
	else
	    break;
    }
    for(i=0; i < strlen(val); i++) {
	if((val[i] == ' ') || (val[i] == '\t') || val[i] == '"')
	    continue;
	else
	    break;
    }
    strcpy(nval, &val[i]);
    strcpy(val, nval);
}

static int split_buf(char *buff, char *l_v, char *r_v, char F)
{
    char fmt_str[40];
    *l_v='\0';
    *r_v='\0';
    
    if(!strchr(buff, F))  return -1;

    sprintf(fmt_str,"%%[^=\n]%c%%[^=\n]", F);

    sscanf(buff, fmt_str, l_v, r_v);

    trim(l_v);
    trim(r_v);

    return 1;
}

struct wds_data *backend_get_wds_list(int *count)
{
    struct wds_data *wds_list = NULL, *wds = NULL;
    char LV[512], RV[512], buf[1024];
    char mac[128];
    int rf_no = 0, wds_no = 0;
    int RF_NUM = 0, VAP_NUM = 0;

    *count = 0;

    RF_NUM = get_num_dir(RF_PATH);
    dbg_printf("RF number = %d\n", RF_NUM);

    for (rf_no = 0; rf_no < RF_NUM; rf_no++) {
	char path[128];
	memset(path, 0, sizeof(path));

	sprintf(path, "%s/%d/%s", RF_PATH, rf_no, WDS_PATH);
	VAP_NUM = get_num_dir(path);
	dbg_printf("VAP number = %d\n", VAP_NUM);

	for (wds_no = 0; wds_no < VAP_NUM; wds_no++) {
	    FILE *fp;
	    char filename[128];
	    int enable = 0;

	    memset(mac, 0, sizeof(mac));
	    sprintf(filename, "/db/wlan/%d/wds/%d/wds_param", rf_no, wds_no);
	    fp = fopen(filename, "r");
	    while(fgets(buf, sizeof(buf), fp) != 0) {
		int ret;

		if ((ret = split_buf(buf, LV, RV, '=')) > 0) {
		    dbg_printf("LV = %s, RV = %s\n", LV, RV);
		    if(strcasecmp(LV, "state") == 0) {
			if (!strcmp(RV, "enable"))
			    enable = 1;
		    }
			
		    if(enable == 1 && strcasecmp(LV, "mac") == 0) {
			strcpy(mac, RV);
			if (strlen(mac) > 0) {
			    wds_list = realloc(wds_list, 
				sizeof(struct wds_data) * (++(*count)));
			    wds = wds_list + (*count - 1);
			    sscanf(mac, "%02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX",
				&wds->mac[0], &wds->mac[1], &wds->mac[2],
				&wds->mac[3], &wds->mac[4], &wds->mac[5]);

			    sprintf(wds->ifname, "ath%dwds%d", rf_no, wds_no);
			    wds->status = WDS_LINK_DOWN;
			    wds->l2 = NULL;
			    time(&wds->timestamp);
			    dbg_printf("Found enabled WDS peer %s (%s).\n", 
				wds->ifname, mac);
			} else {
			    dbg_printf("Found enabled WDS peer w/o MAC, ignore it.\n");
			}
		    }
		}
	    }
	    fclose(fp);
	}
    }
    return wds_list;
}
