#include <stdio.h>
#include <stdlib.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>

#define DEF_UNIX_PATH	"/tmp/wdschk.sock"

static void usage(void)
{
    fprintf(stderr, "Usage: check_peer [-h] [-p sock] if_name\n",
	    "\tThis program will return one of following values:\n",
	    "\t'ACTIVE', 'DOWN' or 'N/A'.\n");
}

static void check_peer_by_ifname(const char *path, const char *ifname)
{
    int sock_fd, len;
    struct sockaddr_un cliaddr, servaddr;
    char buf[BUFSIZ], tmpfile[BUFSIZ];
    fd_set rfds;
    struct timeval tv;
    int ret;

    sock_fd = socket(AF_UNIX, SOCK_DGRAM, 0);
    /* Client address */
    memset(&cliaddr, 0, sizeof(cliaddr));
    cliaddr.sun_family = AF_UNIX;
    strcpy(cliaddr.sun_path, tmpnam_r(tmpfile));
    bind(sock_fd, (struct sockaddr *)&cliaddr, sizeof(cliaddr));
    /* Server address */
    memset(&servaddr, 0, sizeof(servaddr));
    servaddr.sun_family = AF_UNIX;
    strcpy(servaddr.sun_path, path);
    /* Setup select parameters */
    FD_ZERO(&rfds);
    FD_SET(sock_fd, &rfds);
    tv.tv_sec = 1;   // timeout = 1 sec
    tv.tv_usec = 0;
    memset(buf, 0, sizeof(buf));

    sendto(sock_fd, ifname, strlen(ifname)+1, 0, 
	    (struct sockaddr *)&servaddr, sizeof(servaddr));

    ret = select(sock_fd + 1, &rfds, NULL, NULL, &tv);
    if (ret > 0 && FD_ISSET(sock_fd, &rfds)) 
	len = recvfrom(sock_fd, buf, sizeof(buf), 0, NULL, NULL);
    else 
	len = 0;

    if (len > 0)
	printf("%s\n", buf);
    else {
	fprintf(stderr, "Can't receive message from unix socket %s.\n", path);
	printf("N/A\n");
    }

    unlink(tmpfile);
}

int main(int argc, char *argv[])
{
    int opt;
    char *ifname = NULL;
    char *unix_path = DEF_UNIX_PATH;

    while ((opt = getopt(argc, argv, "p:h")) != -1) {
	switch(opt) {
	    case 'p':
		unix_path = optarg;
		break;
	    case 'h':
	    default:
		usage();
		exit(EXIT_SUCCESS);
	}
    }

    if (optind >= argc) {
	usage();
	exit(EXIT_FAILURE);
    }
    ifname = argv[optind];

    check_peer_by_ifname(unix_path, ifname);

    exit(EXIT_SUCCESS);
}
