/* rfscan.c
 *
 * RF Scan Deamon
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <stdint.h>
#include <iwlib.h>

#include <sys/types.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/wireless.h>

#ifndef WT_VERSION
/* support wireless-tools.26 */
#define iw_sockets_close(fd) close(fd)
#endif

#define NBBY 8
#define setbit(a,i) ((a)[(i)/NBBY] |= 1<<((i)%NBBY))
/*
 * Linux uses __BIG_ENDIAN and __LITTLE_ENDIAN while BSD uses _foo
 * and an explicit _BYTE_ORDER.  Sorry, BSD got there first--define
 * things in the BSD way...
**/
#define _LITTLE_ENDIAN  1234    /* LSB first: i386, vax */
#define _BIG_ENDIAN     4321    /* MSB first: 68000, ibm, net */
#include <asm/byteorder.h>
#if defined(__LITTLE_ENDIAN)
#define _BYTE_ORDER     _LITTLE_ENDIAN
#elif defined(__BIG_ENDIAN)
#define _BYTE_ORDER     _BIG_ENDIAN
#else
#error "Please fix asm/byteorder.h"
#endif

/* unalligned little endian access */
#define LE_READ_4(p)                                    \
        ((u_int32_t)                                    \
         ((((const u_int8_t *)(p))[0]      ) |          \
          (((const u_int8_t *)(p))[1] <<  8) |          \
          (((const u_int8_t *)(p))[2] << 16) |          \
          (((const u_int8_t *)(p))[3] << 24)))

#include "wdsd.h"
#include "include/compat.h"
#ifdef MADWIFI_IEEE80211_EXTERNAL
#include "os/linux/include/ieee80211_external.h"
#else
#include "net80211/ieee80211_ioctl.h"
#endif

int scan_ap();

char ifname[16];
char tmp_file[32] = "/tmp/rfscan_tmp";
char output_file[32] = "/tmp/rfscan";
int scan_intval = 60; //60 second
int flag_scanning=0;
int scan_all_channel_on_demand = 0;

void usage(void)
{
    printf("Usage: rfscan [-o output file] [-i interface name] [-t scan interval]\n");
}

void exit_rfscan(int sig)
{
    char buf[256];

#if 0
    sprintf(buf, "ifconfig %s down", ifname);
    system(buf);

    sprintf(buf, "wlanconfig %s destroy", ifname);
    system(buf);
#endif

    printf("Terminate rfscan\n");
    exit(-1);
}

void scan_now(int sig)
{
    if (!flag_scanning) {
        flag_scanning=1;
        scan_ap();
        flag_scanning=0;
    }
}

static int getsocket(void)
{
    static int s = -1;

    if (s < 0) {
	s = socket(AF_INET, SOCK_DGRAM, 0);
	if (s < 0)
	    fprintf(stderr, "socket(SOCK_DRAGM)");
    }
    return s;
}

static int
do80211priv(struct iwreq *iwr, const char *ifname, int op, void *data, size_t len)
{
#define	N(a)	(sizeof(a)/sizeof(a[0]))

    memset(iwr, 0, sizeof(iwr));
    strncpy(iwr->ifr_name, ifname, IFNAMSIZ);
    if (len < IFNAMSIZ) {
	/*
	 * Argument data fits inline; put it there.
	**/
	memcpy(iwr->u.name, data, len);
    } else {
	/*
	 * Argument data too big for inline transfer; setup a
	 * parameter block instead; the kernel will transfer
	 * the data for the driver.  
	**/
	iwr->u.data.pointer = data;
	iwr->u.data.length = len;
    }

    if (ioctl(getsocket(), op, iwr) < 0) {
	static const char *opnames[] = {
		"ioctl[IEEE80211_IOCTL_SETPARAM]",
		"ioctl[IEEE80211_IOCTL_GETPARAM]",
		"ioctl[IEEE80211_IOCTL_SETKEY]",
		"ioctl[SIOCIWFIRSTPRIV+3]",
		"ioctl[IEEE80211_IOCTL_DELKEY]",
		"ioctl[SIOCIWFIRSTPRIV+5]",
		"ioctl[IEEE80211_IOCTL_SETMLME]",
		"ioctl[SIOCIWFIRSTPRIV+7]",
		"ioctl[IEEE80211_IOCTL_SETOPTIE]",
		"ioctl[IEEE80211_IOCTL_GETOPTIE]",
		"ioctl[IEEE80211_IOCTL_ADDMAC]",
		"ioctl[SIOCIWFIRSTPRIV+11]",
		"ioctl[IEEE80211_IOCTL_DELMAC]",
		"ioctl[SIOCIWFIRSTPRIV+13]",
		"ioctl[IEEE80211_IOCTL_CHANLIST]",
		"ioctl[SIOCIWFIRSTPRIV+15]",
		"ioctl[IEEE80211_IOCTL_GETRSN]",
		"ioctl[SIOCIWFIRSTPRIV+17]",
		"ioctl[IEEE80211_IOCTL_GETKEY]",
	};
	op -= SIOCIWFIRSTPRIV;
	if (0 <= op && op < N(opnames))
	    perror(opnames[op]);
	else
	    perror("ioctl[unknown???]");
	return -1;
    }
    return 0;
#undef N
}

static int get80211priv(const char *ifname, int op, void *data, size_t len)
{
    struct iwreq iwr;

    if (do80211priv(&iwr, ifname, op, data, len) < 0)
	return -1;
    if (len < IFNAMSIZ)
	memcpy(data, iwr.u.name, len);
    return iwr.u.data.length;
}

static const char *getcaps(int capinfo)
{
    static char capstring[32];
    char *cp = capstring;

    if (capinfo & IEEE80211_CAPINFO_ESS)
	*cp++ = 'E';
    if (capinfo & IEEE80211_CAPINFO_IBSS)
	*cp++ = 'I';
    if (capinfo & IEEE80211_CAPINFO_CF_POLLABLE)
	*cp++ = 'c';
    if (capinfo & IEEE80211_CAPINFO_CF_POLLREQ)
	*cp++ = 'C';
    if (capinfo & IEEE80211_CAPINFO_PRIVACY)
	*cp++ = 'P';
    if (capinfo & IEEE80211_CAPINFO_SHORT_PREAMBLE)
	*cp++ = 'S';
    if (capinfo & IEEE80211_CAPINFO_PBCC)
	*cp++ = 'B';
    if (capinfo & IEEE80211_CAPINFO_CHNL_AGILITY)
	*cp++ = 'A';
    if (capinfo & IEEE80211_CAPINFO_SHORT_SLOTTIME)
	*cp++ = 's';
#ifndef MADWIFI_IEEE80211_EXTERNAL
    if (capinfo & IEEE80211_CAPINFO_RSN)
	*cp++ = 'R';
#endif
    if (capinfo & IEEE80211_CAPINFO_DSSSOFDM)
	*cp++ = 'D';
    *cp = '\0';
    return capstring;
}

static int getmaxrate(uint8_t rates[15], uint8_t nrates)
{
    int i, maxrate = -1;

    for (i = 0; i < nrates; i++) {
	int rate = rates[i] & IEEE80211_RATE_VAL;
	if (rate > maxrate)
	    maxrate = rate;
    }
    return maxrate / 2;
}

static int __inline iswpaoui(const u_int8_t *frm)
{
    return frm[1] > 3 && LE_READ_4(frm+2) == ((WPA_OUI_TYPE<<24)|WPA_OUI);
}

static int __inline iswmeoui(const u_int8_t *frm)
{
    return frm[1] > 3 && LE_READ_4(frm+2) == ((WME_OUI_TYPE<<24)|WME_OUI);
}

static int __inline isatherosoui(const u_int8_t *frm)
{
    return frm[1] > 3 && LE_READ_4(frm+2) == ((ATH_OUI_TYPE<<24)|ATH_OUI);
}


/*
 *  * Convert MHz frequency to IEEE channel number.
 *   */
static u_int ieee80211_mhz2ieee(u_int freq)
{
    if (freq == 2484)
	return 14;
    if (freq < 2484)
	return (freq - 2407) / 5;
    if (freq < 5000)
	return 15 + ((freq - 2512) / 20);
    return (freq - 5000) / 5;
}
/*
 * Copy the ssid string contents into buf, truncating to fit.  If the
 * ssid is entirely printable then just copy intact.  Otherwise convert
 * to hexadecimal.  If the result is truncated then replace the last
 * three characters with "...".
**/
static size_t copy_essid(char buf[], size_t bufsize, const u_int8_t *essid, size_t essid_len)
{
    const u_int8_t *p; 
    size_t maxlen;
    int i;

    if (essid_len > bufsize)
	maxlen = bufsize;
    else
	maxlen = essid_len;
    /* determine printable or not */
    for (i = 0, p = essid; i < maxlen; i++, p++) {
	if (*p < ' ' || *p > 0x7e)
	    break;
    }
    if (i != maxlen) {		/* not printable, print as hex */
	if (bufsize < 3)
	    return 0;
#if 0
	strlcpy(buf, "0x", bufsize);
#else
	strncpy(buf, "0x", bufsize);
#endif
	bufsize -= 2;
	p = essid;
	for (i = 0; i < maxlen && bufsize >= 2; i++) {
	    sprintf(&buf[2+2*i], "%02x", *p++);
	    bufsize -= 2;
	}
	maxlen = 2+2*i;
    } else {			/* printable, truncate as needed */
	memcpy(buf, essid, maxlen);
    }
    if (maxlen != essid_len)
	memcpy(buf+maxlen-3, "...", 3);
    return maxlen;
}

static char *parse_ies(const u_int8_t *vp, int ielen, int maxcols, char **ret)
{
    char *s;

    s = *ret;

    while (ielen > 0) {
	switch (vp[0]) {
	    case IEEE80211_ELEMID_VENDOR:
		if (iswpaoui(vp)) {
		    strncpy(s, " WPA", 4);
		    s += 4;
		} else if (iswmeoui(vp)) {
		    strncpy(s, " WME", 4);
		    s += 4;
		} else if (isatherosoui(vp)) {
		    strncpy(s, " ATH", 4);
		    s += 4;
		} else {
		    strncpy(s, " VEN", 4);
		    s +=4;
		}
		break;
	    case IEEE80211_ELEMID_RSN:
		strncpy(s, " RSN", 4);
		s += 4;
		break;
	    default:
		break;
	}
	ielen -= 2 + vp[1];
	vp += 2 + vp[1];
    }

    return *ret;
}

int get_current_channel(char *ifname)
{
#if 0 
    char cmd[BUFSIZ];
    char buf[BUFSIZ];
    int channel = 0;
    FILE *fp = NULL;

    sprintf(cmd, "iwlist %s channel | grep Freq", ifname);
    if ((fp = popen(cmd, "r")) == NULL) {
	fprintf(stderr, "Can't get channel\n");
	return 0;
    }
    while (fgets(buf, BUFSIZ-1, fp)) {
	if(sscanf(strstr(buf, "Current"), 
		    "Current Frequency:%*fGHz (channel %d)", &channel)) {
	    break;
	}
    }
    pclose(fp);
    return channel;
#else
    int iw_skfd;
    int channel = 0;
    struct iw_range range;
    struct iwreq wrq;

    if ((iw_skfd = iw_sockets_open()) < 0) {
	printf("Can't connect to iwlib.\n");
	goto fail;
    }

    if (iw_get_range_info(iw_skfd, ifname, &range) < 0) {
	printf("Can't get frequence range.\n");
	iw_sockets_close(iw_skfd);
	goto fail;
    }

    if (iw_get_ext(iw_skfd, ifname, SIOCGIWFREQ, &wrq) >= 0) {
	double freq = iw_freq2float(&(wrq.u.freq));
	channel = iw_freq_to_channel(freq, &range);
	if (channel < 0) {
	    channel = ieee80211_mhz2ieee(freq);
	}
    }
    iw_sockets_close(iw_skfd);

fail:
    return channel;
#endif
}

struct scan_result {
    char ssid[32+1];
    char bssid[17+1];
    int channel;
    int rate;
    int signal;
    int noise;
    char cap[32];
    char ies[100];
    struct scan_result *next;
};

int do_scan(char *ifname)
{
    int skfd;
    struct iwreq wrq;

    if ((skfd = iw_sockets_open()) < 0) {
	/* Fail */
	fprintf(stderr, "open iw socket fail.\n");
	return -1;
    }
    
    wrq.u.data.pointer = NULL;
    wrq.u.data.flags = 0;
    wrq.u.data.length = 0;

    if (iw_set_ext(skfd, ifname, SIOCSIWSCAN, &wrq) < 0) {
	/* Fail */
	fprintf(stderr, "ioctl scan error.\n");
	iw_sockets_close(skfd);
	return -1;
    }

    iw_sockets_close(skfd);
    return 0;
}
static struct scan_result *list_scan(const char *ifname)
{
    uint8_t buf[24*1024];
    struct iwreq iwr;
    char ssid[32+1];
    uint8_t *cp;
    int len;
    struct scan_result *head = NULL;

    len = get80211priv(ifname, IEEE80211_IOCTL_SCAN_RESULTS,
		    buf, sizeof(buf));
    if (len == -1)
	fprintf(stderr, "unable to get scan results");
    if (len < sizeof(struct ieee80211req_scan_result))
	return;

    cp = buf;
    do {
	struct ieee80211req_scan_result *sr;
	uint8_t *vp;
	char macstr[18];
	struct scan_result *tmp;

	sr = (struct ieee80211req_scan_result *) cp;
	vp = (u_int8_t *)(sr+1);

	tmp = calloc(1, sizeof(struct scan_result));
	tmp->next = head;
	head = tmp;
	/* ssid */
	memset(ssid, 0, sizeof(ssid));
	copy_essid(ssid, sizeof(ssid), vp, sr->isr_ssid_len);
	strcpy(tmp->ssid, ssid);

	/* bssid */
	memset(macstr, 0, sizeof(macstr));
	sprintf(macstr, "%02x:%02x:%02x:%02x:%02x:%02x", 
		sr->isr_bssid[0], sr->isr_bssid[1], sr->isr_bssid[2], 
		sr->isr_bssid[3], sr->isr_bssid[4], sr->isr_bssid[5]);
	strcpy(tmp->bssid, macstr);
	
	/* channel */
	tmp->channel = ieee80211_mhz2ieee(sr->isr_freq);

	/* Max rate */
	tmp->rate = getmaxrate(sr->isr_rates, sr->isr_nrates);

	/* Signal */
	tmp->signal = sr->isr_rssi;
	
	/* Noise */
	tmp->noise = sr->isr_noise;

	/* Cap */
	strcpy(tmp->cap, getcaps(sr->isr_capinfo));

	/* IE */
	{
	    char *ie = malloc(sizeof(char)*100);
	    memset(ie, 0, 100);
	    parse_ies(vp + sr->isr_ssid_len, sr->isr_ie_len, 24, &ie);
	    strcpy(tmp->ies, ie);
	    free(ie);
	}
	
	cp += sr->isr_len;
	len -= sr->isr_len;
    } while (len >= sizeof(struct ieee80211req_scan_result));

    return head;
}

void free_scan_list(struct scan_result *list)
{
    struct scan_result *tmp;
    while (list != NULL) {
	tmp = list->next;
	free(list);
	list = tmp;
    }
}

int scan_ap()
{
    char buf[256];
    int cur_chan = 0;
    struct scan_result *list;
    FILE *fp = NULL;

    if (scan_all_channel_on_demand) {
	struct ieee80211req_chanlist chanlist;
	struct iwreq iwr;
	int i;
	/* Enable scan for all channels */
	cur_chan = get_current_channel(ifname);
	for (i = 1; i <= 256; i++) {
	    setbit(chanlist.ic_channels, i);
	}
	do80211priv(&iwr, ifname, IEEE80211_IOCTL_SETCHANLIST, 
			&chanlist, sizeof(chanlist));
    }

    do_scan(ifname);
#if 0
    sprintf(buf, "iwlist %s scanning > /dev/null", ifname);
    system(buf);
#endif

    fp = fopen(output_file, "w");
    fprintf(fp, "%-32.32s  %-17.17s  %4s %4s  %-5s %3s %4s\n"
	, "SSID", "BSSID", "CHAN", "RATE", "S:N", "INT", "CAPS");

    list = list_scan(ifname);
    while (list != NULL) {
	fprintf(fp, "%-32.*s  %s  %3d  %3dM %2d:%-2d  %3d %-4.4s %s\n", 
		strlen(list->ssid), list->ssid, list->bssid, list->channel, 
		list->rate, list->signal, list->noise, 100, list->cap, list->ies);
	list = list->next;
    }
    fclose(fp);
    free_scan_list(list);

#if 0
    sprintf(buf, "wlanconfig %s list ap > %s", ifname, output_file);
    system(buf);
#endif

    if (scan_all_channel_on_demand) {
	struct ieee80211req_chanlist chanlist;
	struct iwreq iwr;

	setbit(chanlist.ic_channels, cur_chan);
	do80211priv(&iwr, ifname, IEEE80211_IOCTL_SETCHANLIST, 
			&chanlist, sizeof(chanlist));
    }

    return 0;
}

int main(int argc, char **argv)
{
    int i;
    int rfno=3;
    char buf[256];
    struct sigaction act; 
    int no_bg_scan = 0;

    if (argc < 3 || argc > 7) { 
        usage();
        return -1;
    }
    
    memset(ifname, 0, 16);
    /* parse the input parameter */
    for (i=1; i<argc; i++) {
        if (!strcmp(argv[i], "-t")) { //scan interval
            scan_intval=atoi(argv[i+1]);
            if (scan_intval < 5 || scan_intval > 86400) { //5sec to 1 day
                printf("Error, scan interval must be 5 - 86400\n");
                return -1;
            }
            i++;
            continue; 
        }
        else if (!strcmp(argv[i], "-o")) { //output file
            strncpy(output_file, argv[i+1], sizeof(output_file));
            i++;
            continue;  
        }
        else if (!strcmp(argv[i], "-i")) { //interface number
            if((argv[i+1] == '-') || (strlen(argv[i+1]) > 16)){
                printf("Error: Interface name is not correct.\n");
                return -2;
            }
            strncpy(ifname, argv[i+1], strlen(argv[i+1]));
            i++;
/*
            rfno=atoi(argv[i+1]);
            i++;
            if (rfno < 0 || rfno > 3) {
                printf("Error: rfno must be 0-3\n");
                return -2;
            }
*/
            continue;
        }
	else if (!strcmp(argv[i], "-T")) { //don't backgroud scan
	    no_bg_scan = 1;
	}
	else if (!strcmp(argv[i], "-S")) { //Scan all channel on demand
	    scan_all_channel_on_demand = 1;
	}
        else {
            usage();
            return -1;
        }
    }
    if(strlen(ifname) == 0){
        printf("Please specify interface name.\n");
        return -3;
    }
    sigemptyset (&act.sa_mask); 
    act.sa_flags = 0; 

    act.sa_handler = (void *) exit_rfscan; 
    sigaction(SIGINT, &act, NULL); 
    sigaction(SIGHUP, &act, NULL); 
    sigaction(SIGTERM, &act, NULL);

    act.sa_handler = (void *) scan_now; 
    sigaction(SIGUSR1, &act, NULL); 
 
//    sprintf(ifname, "scan%d", rfno);

    /* create rf scan interface */
    if (!isIfExist(ifname)) {
//	sprintf(buf, 
//		"wlanconfig %s create wlandev wifi%d wlanmode sta nosbeacon",
//		ifname, rfno);
//	system(buf);
        printf("The interface is not exist!!");
        return -4;
    }
    
//    sprintf(buf, "ifconfig %s up", ifname);
//    system(buf);
    if (no_bg_scan == 0) {
	sleep(10); //sleep 10 sec first
	while (1) {
	    if (!flag_scanning) {
		flag_scanning=1;
		scan_ap();
		flag_scanning=0;
	    }
	    sleep(scan_intval);
	}
    } else {
	while (1);
    }

#if 0
    sprintf(buf, "ifconfig %s down", ifname);
    system(buf);

    sprintf(buf, "wlanconfig %s destroy", ifname);
    system(buf);
#endif
    
    return 0;
}

