#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <signal.h>

//#include "cipcfg.h"
#define MAX_VALUE_LEN 256

typedef struct ap_list {
    char ssid[32];
    char mac[18];
    int ch;
    int rate;
    int signal;
    char security[10];
    struct ap_list *next;
} AP_LIST;

void print_aps(AP_LIST *list)
{
    int count =0;

    while (list != NULL)
    {
        count++;
        printf("%d. SSID %s, MAC %s Channel %d, Singal %d Security %s\n", 
                count, list->ssid, list->mac, list->ch, list->signal, list->security);
        list = list->next;
    }
}

int free_ap_list(AP_LIST *list)
{
    AP_LIST *tmp;
    while (list != NULL) {
        tmp = list;
        list = list->next;
        free(tmp);
    }
    return 0;
}

int get_pid(char *process)
{
        int pid = 0;
        char buffer[256];
        FILE *fp;

        system("ps > /tmp/ps");
        if ((fp=fopen("/tmp/ps", "r")) != NULL) {
                while (fgets(buffer, sizeof(buffer), fp) != NULL) {
                        if (strstr(buffer, process) != NULL) {
                                char *tok;
                                tok = strtok(buffer, " ");
                                tok = strtok(buffer, " ");
                                pid=atoi(tok);
                                printf("<debug>The pid of %s is %d\n",process, pid);
                        }
                }
                fclose(fp);
        }
        else {
                printf("<error>Can't access /tmp/ps\n");
                return -1;
        }

        return pid;
}


/**
 * Call rfscan to scanning all APs.
 * It will put info of the APs in AP_LIST
 *
 * @param rfno      The RF card number
 * @param is_scan   1: restart scan now, 0: only retrive the file
 * @return	    pointer to ap list, NULL if not found
 */   
AP_LIST *rf_scan(int rfno, int is_scan)
{
    char buf[MAX_VALUE_LEN], *sec;
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i;
    
    if (is_scan) { //if want to scan now, set is_scan = 1
        int pid=0;

        sprintf(buf, "/tmp/rfscan%d", rfno);
        pid = get_pid(buf);

        if (pid != 0) {
            kill(pid, SIGUSR1); //force the rfscan to scan APs
            sleep(3);
        }
        else {
            printf("Error, can't find the rfscan process\n");
            return NULL;
        }
    }

    sprintf(buf, "/tmp/rfscan%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        printf("Error: can't open %s\n", buf);
        return NULL;
    }

    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            printf("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        strncpy(ap->ssid, strtok(buf, " "), sizeof(ap->ssid));

        strncpy(ap->mac, strtok(NULL, " "), sizeof(ap->mac));
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }

        ap->ch=atoi(strtok(NULL, " "));
        ap->rate=atoi(strtok(NULL, " "));
        ap->signal=atoi(strtok(NULL, ":"));
        
        sec=strtok(NULL, "\n");
        if (strstr(sec, "EP")) {
            if (strstr(sec, "WPA")) {
                sprintf(ap->security, "WPA1"); //WPA
            }
            else if (strstr(sec, "RSN")) {
                sprintf(ap->security, "WPA2"); //WPA2
            }
            else {
                sprintf(ap->security, "WEP"); //WEP
            }
        }
        else {
            sprintf(ap->security, "None");
        }
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}


int main(void)
{
    AP_LIST *ap_list=NULL;

    printf("first time !!\n");
    ap_list=rf_scan(3, 1);
    print_aps(ap_list);
    free_ap_list(ap_list);

    printf("second time !!\n");
    ap_list=rf_scan(3, 0);
    print_aps(ap_list);
    free_ap_list(ap_list);
}
