#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/signal.h>

#define MAX_VALUE_LEN 256
typedef struct ap_list {
    char ssid[32];
    char mac[18];
    int ch;
    int rate;
    int signal;
    char security[10];
    struct ap_list *next;
} AP_LIST;

void print_aps(AP_LIST *list)
{
    int count =0;

    while (list != NULL)
    {
        count++;
        printf("%d. SSID %s, MAC %s Channel %d, Singal %d Security %s\n", 
                count, list->ssid, list->mac, list->ch, list->signal, list->security);
        list = list->next;
    }
}

int free_ap_list(AP_LIST *list)
{
    AP_LIST *tmp;
    while (list != NULL) {
        tmp = list;
        list = list->next;
        free(tmp);
    }
    return 0;
}

int rf2dev(int rfno)
{
    FILE *fp=NULL;
    int rf2irq[4] = {26, 27, 25, 28}, dev=-1;
    char buf[80], *tok;

    if ((fp=fopen("/proc/interrupts", "r")) == NULL) {
        printf("Error, can't open /proc/interrupts\n");
        return -1;
    }
    
    while (fgets(buf, sizeof(buf), fp) != NULL) {
        tok=strtok(buf, ":");
        if (rf2irq[rfno] == atoi(tok)) {
            tok=strtok(NULL, " ");
            tok=strtok(NULL, " ");
            dev=atoi(tok+3);
            break;
        }

    }
    fclose(fp);
    return dev;
}
/**
 * Call wdsds to discover vaild WDS peers.
 * It will put info of the vaild peer in AP_LIST
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery
 * @param ch        The current channel for this RF card
 * @return	    pointer to ap list, NULL if not found
 */   
AP_LIST *wdsd_scan(int rfno, char *secret, int ch, int mode)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i, ret;

    /* Notice!! When discovery WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
    sprintf(buf, "/etc/rc.d/rc.wlan stop %d 2> /dev/null" ,
        rfno);
    system(buf); 

    sprintf(buf, "wdsds scan -s %s -o /tmp/wdsds%d -i %d -c %d -m %d 2> /dev/null",
        secret, rfno, rf2dev(rfno), ch, mode);
    ret = system(buf);

    sprintf(buf, "/etc/rc.d/rc.wlan start %d 2> /dev/null" ,
        rfno);
    system(buf); 

    system("/etc/rc.d/rc.bridge restart  2> /dev/null");
    system("/etc/rc.d/rc.net restart_wan  2> /dev/null");

    sprintf(buf, "/tmp/wdsds%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        printf("Error: can't open %s\n", buf);
        return NULL;
    }

    // skip the first line
    fgets(buf, sizeof(buf), fp);
    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            printf("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        strncpy(ap->ssid, strtok(buf, " "), sizeof(ap->ssid));

        strncpy(ap->mac, strtok(NULL, " "), sizeof(ap->mac));
        ap->mac[17] = '\0';
        for(i=0; i<strlen(ap->mac); i++) {
            ap->mac[i]=toupper(ap->mac[i]);
        }

        ap->ch=atoi(strtok(NULL, " "));
        ap->rate=atoi(strtok(NULL, " "));
        ap->signal=atoi(strtok(NULL, ":"));
        ap->next=NULL;
        
        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);

    return ap_list;
}

/**
 * Call wdsds to connect a vaild WDS peers.
 * It will return the connect result
 *
 * @param rfno      The RF card number
 * @param secret    The shared secret of WDS discovery,
 * @param mac       The MAC address of peer.
 * @param ch        The current channel for this RF card
 * @return	    0: success, -1: failure (please reference wdsds.c)
 */
int wdsd_connect(int rfno, char *secret, char *mac, int ch, int mode)
{
    char buf[MAX_VALUE_LEN];
    int ret;
  
    /* Notice!! When connect WDS peer, it will stop WDS link and can't recover
     * Therefore, we need restart RF card again
     */
    sprintf(buf, "/etc/rc.d/rc.wlan stop %d 2> /dev/null" ,
        rfno);
    system(buf); 

    sprintf(buf, "wdsds add -a %s -s %s -i %d -c %d -m %d 2> /dev/null" ,
        mac, secret, rf2dev(rfno), ch, mode);
    ret=system(buf); //0: success, -1: failure

    sprintf(buf, "/etc/rc.d/rc.wlan start %d 2> /dev/null" ,
        rfno);
    system(buf); 

    system("/etc/rc.d/rc.bridge restart  2> /dev/null");
    system("/etc/rc.d/rc.net restart_wan  2> /dev/null");
    return ret;
}

/**
 * Get the peer that connected to WDS discovery master
 * It will return AP_LIST
 *
 * @param rfno      The RF card number
 * @return	    pointer to ap list, NULL if not found
 */
AP_LIST *wdsd_connected(int rfno)
{
    char buf[MAX_VALUE_LEN];
    FILE *fp = NULL;
    AP_LIST *ap = NULL, *ap_list = NULL;
    int i;

    sprintf(buf, "/tmp/wdsdm%d", rfno);
    if ((fp=fopen(buf, "r")) == NULL) {
        printf("Error: can't open %s\n", buf);
        return NULL;
    }

    while (fgets(buf, sizeof(buf), fp)) {
        if ((ap = malloc(sizeof(AP_LIST))) == NULL) {
            printf("Error: memory allocate failure, can't allocate size = %d\n", 
                sizeof(AP_LIST));
            fclose(fp);
	    return NULL;
        }
        memset(ap, 0, sizeof(AP_LIST));

        /* The file format is "MAC: xx:xx....." and "RSSI: xx" */
        if (strstr(buf, "MAC")) { 
            strncpy(ap->mac, &buf[5], sizeof(ap->mac));
            ap->mac[17] = '\0';
        
            fgets(buf, sizeof(buf), fp);

            if (strstr(buf, "RSSI")) {
                ap->signal=atoi(&buf[6]);
            }
            else {
                printf("Incorrectly file format, RSSI don't fellow MAC\n");
                free(ap);
                fclose(fp);
                return ap_list;
            }
            ap->next=NULL;
        }

        if (ap_list) {
            ap->next=ap_list;
            ap_list=ap;
        }
        else {
            ap_list=ap;
        }
    }
    fclose(fp);
    return ap_list;
}

int main(void)
{
    AP_LIST *ap_list=NULL;

    ap_list=wdsd_scan(2, "discovery", 8, 3); //channel 8, mode 11g
    print_aps(ap_list);
    free_ap_list(ap_list);
/*
    if ( wdsd_connect(1, "1234", "00:03:7f:09:63:1e", 5, 3) == 0) {
        printf("Connect WDS peer 00:03:7f:09:63:1e success!!\n");
    }
    else {
        printf("Connect WDS peer 00:03:7f:09:63:1e failure!!\n");
    }
    
    ap_list=wdsd_connected(2); //channel 5, mode 11g
    print_aps(ap_list);
    free_ap_list(ap_list);
    system("rm -f /tmp/wdsdm2");

    if ( wdsd_connect(1, "1234", "06:0E:2E:7C:AA:FD", 5, 3) == 0) {
        printf("Connect WDS peer 06:0E:2E:7C:AA:FD success!!\n");
    }
    else {
        printf("Connect WDS peer 06:0E:2E:7C:AA:FD failure!!\n");
    }
*/
    return 0;
}
