#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/ioctl.h>
#include <net/if.h>
#include <arpa/inet.h>

#include "wdsd.h"

int get_mac(char *ifname, char *mac)
{
    int s=socket(PF_INET, SOCK_STREAM, 0);
    struct ifreq ifr;

    strncpy(ifr.ifr_name, ifname, IFNAMSIZ-1);
    if (ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
        close(s);
        return -1;
    }
    else {
        sprintf(mac, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
            ifr.ifr_hwaddr.sa_data[0],
            ifr.ifr_hwaddr.sa_data[1],
            ifr.ifr_hwaddr.sa_data[2],
            ifr.ifr_hwaddr.sa_data[3],
            ifr.ifr_hwaddr.sa_data[4],
            ifr.ifr_hwaddr.sa_data[5]);
    }
    close(s);
    return 0;
}

void toup_mac(char *mac)
{
    int i;
    for(i=0; i<strlen(mac); i++) {
        mac[i]=toupper(mac[i]);
    }
}

int gen_wep_key(struct wdsd_data *wdsd, char *mac)
{
    char buf[256];
    FILE *fp = NULL;

    if (mac == NULL || (strlen(wdsd->secret) < 1)) {
        printf("Invaild MAC : %s, Secret Key : %s\n", mac, wdsd->secret);
        return -1;
    }
    else {
        /* Generate SSID */
        toup_mac(mac);
        sprintf(buf, "echo -n %s+%s | md5sum > /tmp/wdsd_wep_%d",
            mac, wdsd->secret, wdsd->rfno);
        system(buf);

        sprintf(buf, "/tmp/wdsd_wep_%d", wdsd->rfno);
        if ((fp=fopen(buf, "r")) == NULL) {
            printf("Can't open the file %s\n", buf);
        }
        else {
            fgets(buf, sizeof(buf), fp);
            memcpy(wdsd->wep, buf, sizeof(wdsd->wep));
            wdsd->wep[26]='\0';
            //printf("%s>wep =%s\n", __func__, wdsd->wep);
            close(fp);
        }
        sprintf(buf, "rm -f /tmp/wdsd_wep_%d >> /dev/null", wdsd->rfno);
        system(buf);
    }
    return 0;
}


int gen_ssid(struct wdsd_data *wdsd, char *mac)
{
    char buf[256];
    FILE *fp = NULL;
    int len=strlen(wdsd->secret);

    if (mac == NULL || len < 1) {
        printf("Invaild MAC : %s, Secret Key : %s\n", mac, wdsd->secret);
        return -1;
    }
    else {
        /* Generate SSID */
        toup_mac(mac);
        sprintf(buf, "echo -n %s+%s | md5sum > /tmp/wdsd_ssid_%d",
            wdsd->secret, mac, wdsd->rfno);
        system(buf);

        sprintf(buf, "/tmp/wdsd_ssid_%d", wdsd->rfno);
        if ((fp=fopen(buf, "r")) == NULL) {
            printf("Can't open the file %s\n", buf);
        }
        else {
            fgets(buf, sizeof(buf), fp);
            strncpy(wdsd->ssid, buf, len);
            wdsd->ssid[len] = '\0' ;
            //printf("%s>ssid =%s\n", __func__, wdsd->ssid);
            close(fp);
        }
        sprintf(buf, "rm -f /tmp/wdsd_ssid_%d >> /dev/null", wdsd->rfno);
        system(buf);
    }
    return 0;
}

int isIfExist(char *ifname)
{
    struct ifreq ifr;
    int s = socket(PF_INET, SOCK_STREAM, 0);

    if (!ifname) 
	return 0;

    strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
    return (ioctl(s, SIOCGIFINDEX, &ifr) < 0 ? 0 : 1);
}
